/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.backend;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * AbstractVascBackendControllerLocal
 * 
 * @author Willem Cazander
 * @version 1.0 Jun 5, 2012
 */
abstract public class AbstractVascBackendControllerLocal implements VascBackendControllerLocal {

	private Map<String,VascBackend> backends = null;
	
	public AbstractVascBackendControllerLocal() {
		backends = new HashMap<String,VascBackend>(1000);
	}
	
	@Override
	@SuppressWarnings("unchecked")
	public <BACKEND_TYPE extends VascBackend> BACKEND_TYPE getVascBackendById(Class<BACKEND_TYPE> clazz,String id) {
		return (BACKEND_TYPE) backends.get(id);
	}
	
	/**
	 * @see net.forwardfire.vasc.backend.VascBackendController#getVascBackendIds()
	 */
	@Override
	public List<String> getVascBackendIds() {
		List<String> result = new ArrayList<String>(50);
		for (String id:backends.keySet()) {
			result.add(id);
		}
		Collections.sort(result); // lets do abc for consistance behauvior.
		return result;
	}
	
	/**
	 * @see net.forwardfire.vasc.backend.VascBackendControllerLocal#addVascBackend(net.forwardfire.vasc.backend.crud.VascBackendCrud)
	 */
	@Override
	public void addVascBackend(VascBackend backend) {
		if (backend==null) {
			throw new NullPointerException("backend must not be null.");
		}
		if (backend.getId()==null) {
			throw new IllegalArgumentException("The backend must have an id.");
		}
		backends.put(backend.getId(), backend);
	}
	
	@Override
	public VascBackend removeVascBackend(VascBackend backend) {
		VascBackend result = getVascBackendById(VascBackend.class,backend.getId());
		if (result==null) {
			throw new NullPointerException("Could not find backend to remove with id: "+backend.getId());
		}
		return backends.remove(backend.getId());
	}
	
	@Override
	public void stopBackends() {
		for (String backendId:getVascBackendIds()) {
			VascBackend vb = getVascBackendById(VascBackend.class,backendId);
			if (vb instanceof VascBackendLocal) {
				VascBackendLocal vbl = (VascBackendLocal)vb;
				vbl.stopBackend();
			}
		}
	}
	
	@Override
	public void startBackends() {
		for (String backendId:getVascBackendIds()) {
			VascBackend vb = getVascBackendById(VascBackend.class,backendId);
			if (vb instanceof VascBackendLocal) {
				VascBackendLocal vbl = (VascBackendLocal)vb;
				vbl.startBackend();
			}
		}
	}
}
