package net.forwardfire.vasc.backend;

import java.io.Serializable;
import java.util.Collection;
import java.util.Objects;

abstract public class AbstractVascBackendLocal<DATA_PK extends Serializable> implements VascBackendLocal<DATA_PK> {
	
	private static final String CHECK_MSG_ID = " in ";
	private String id = null;
	private volatile boolean running = false;
	
	/**
	 * @see net.forwardfire.vasc.backend.crud.VascBackend#getId()
	 */
	@Override
	final public String getId() {
		return id;
	}
	
	/**
	 * @see net.forwardfire.vasc.backend.crud.VascBackendLocal#setId(java.lang.String)
	 */
	@Override
	final public void setId(String id) {
		if (id==null) {
			throw new IllegalArgumentException("id may not be null");
		}
		if (this.id!=null) {
			throw new IllegalStateException("Can't change id after setting.");
		}
		this.id=id;
	}
	
	abstract protected void startBackendLocal();
	protected void stopBackendLocal() {
	}
	
	/**
	 * @see net.forwardfire.vasc.backend.crud.VascBackendLocal#startBackend()
	 */
	@Override
	final public void startBackend() {
		requireNonEmpty(getId(),"id is empty.");
		startBackendLocal();
		running = true;
	}
	
	/**
	 * @see net.forwardfire.vasc.backend.crud.VascBackendLocal#stopBackend()
	 */
	@Override
	final public void stopBackend() {
		stopBackendLocal();
		running = false;
	}
	
	/**
	 * @see net.forwardfire.vasc.backend.VascBackendLifeCycle#isRunning()
	 */
	@Override
	final public boolean isRunning() {
		return running;
	}

	final private String createCheckMessage(String message) {
		return message + CHECK_MSG_ID + getId();
	}
	
	final protected void requireNonNull(Object value, String message) {
		Objects.requireNonNull(value, createCheckMessage(message));
	}
	
	final protected void requireNonEmpty(String value, String message) {
		requireNonNull(value, message);
		if (value.isEmpty()) {
			throw new IllegalArgumentException(createCheckMessage(message));
		}
	}
	
	final protected void requireNonEmpty(Collection<?> value, String message) {
		requireNonNull(value, message);
		if (value.isEmpty()) {
			throw new IllegalArgumentException(createCheckMessage(message));
		}
	}
}
