/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.backend.data;

import java.io.Serializable;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.HashMap;
import java.util.Map;

import net.forwardfire.vasc.backend.VascBackendException;

/**
 * HashMapVascBackendAccessDataKey provides get/set support on Map key object.
 * note: all types need to have a String constructor.
 * 
 * @author Willem Cazander
 * @version 1.0 Oct 26, 2014
 */
@SuppressWarnings("serial")
public class HashMapVascBackendAccessDataKey extends AbstractVascBackendAccessDataKey<HashMap<String,Serializable>> {
	
	public HashMapVascBackendAccessDataKey(Map<String,Class<?>> keyMapType) {
		super(keyMapType);
	}
	
	@Override
	public Map<String, Serializable> getKeyMapFromPrimaryKey(HashMap<String, Serializable> primaryKey) throws VascBackendException {
		return new HashMap<String, Serializable>(primaryKey); // wrap only as key is already object map
	}
	
	@Override
	public HashMap<String, Serializable> getPrimaryKeyFromKeyMap(Map<String, Serializable> keyMap) throws VascBackendException {
		return new HashMap<String, Serializable>(keyMap); // wrap only as key is already object map
	}
	
	@Override
	public Map<String, String> getStringMapFromPrimaryKey(HashMap<String, Serializable> primaryKey) throws VascBackendException {
		HashMap<String,String> result = new HashMap<String,String>(primaryKey.size());
		for (String key:primaryKey.keySet()) {
			Serializable value = primaryKey.get(key);
			if (value == null) {
				result.put(key, null);
			} else {
				result.put(key, value.toString());
			}
		}
		return result;
	}
	
	@Override
	public HashMap<String, Serializable> getPrimaryKeyFromStringMap(Map<String, String> keyMap) throws VascBackendException {
		try {
			HashMap<String,Serializable> result = new HashMap<String,Serializable>(keyMap.size());
			for (String key:keyMap.keySet()) {
				String valueStr = keyMap.get(key);
				Class<?> valueType = getKeyMapType().get(key);
				if (valueType == null) {
					throw new IllegalArgumentException("unknown key: "+key);
				}
				if (valueStr == null) {
					result.put(key, null);
					continue;
				}
				Constructor<?> valueObj = valueType.getConstructor(new Class<?>[]{String.class});
				Object value = valueObj.newInstance(new Object[]{valueStr});
				result.put(key, (Serializable)value);
			}
			return result;
		} catch (NoSuchMethodException e) {
			throw new VascBackendException(e);
		} catch (SecurityException e) {
			throw new VascBackendException(e);
		} catch (InstantiationException e) {
			throw new VascBackendException(e);
		} catch (IllegalAccessException e) {
			throw new VascBackendException(e);
		} catch (IllegalArgumentException e) {
			throw new VascBackendException(e);
		} catch (InvocationTargetException e) {
			throw new VascBackendException(e);
		}
	}
}
