/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.backend.data;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;

import net.forwardfire.vasc.backend.VascBackendException;
import net.forwardfire.vasc.backend.VascBackendAccessDataRecord;

/**
 * HashMapVascBackendAccessDataRecord provides get/set support on Map record object.
 * 
 * @author Willem Cazander
 * @version 1.0 Dec 05, 2009
 */
@SuppressWarnings("serial")
public class HashMapVascBackendAccessDataRecord implements VascBackendAccessDataRecord<HashMap<String,Serializable>,HashMap<String,Serializable>> {

	private final Map<String,Class<?>> keyMapType;
	
	public HashMapVascBackendAccessDataRecord(Map<String,Class<?>> keyMapType) {
		this.keyMapType = keyMapType;
	}
	
	/**
	 * @see net.forwardfire.vasc.backend.VascBackendAccessDataRecord#getPrimaryKey(java.io.Serializable)
	 */
	@Override
	public HashMap<String, Serializable> getPrimaryKey(HashMap<String, Serializable> record) throws VascBackendException {
		HashMap<String, Serializable> result = new HashMap<String, Serializable>(keyMapType.size());
		for (String key:keyMapType.keySet()) {
			result.put(key, record.get(key));
		}
		return result;
	}
	
	/**
	 * @see net.forwardfire.vasc.core.entry.VascBackendAccessDataRecord#getValue(java.lang.String, java.lang.Object)
	 */
	@Override
	public Serializable getValue(String backendName,HashMap<String,Serializable> map) throws VascBackendException {
		if (backendName==null) {
			throw new NullPointerException("Can't get value of null backendName field.");
		}
		if (map==null) {
			throw new NullPointerException("Can't get value of null object.");
		}
		Serializable fieldValue = map.get(backendName);
		return fieldValue;
	}
	
	/**
	 * @see net.forwardfire.vasc.core.entry.VascBackendAccessDataRecord#setValue(java.lang.String, java.lang.Object, java.lang.Object)
	 */
	@Override
	public void setValue(String backendName, HashMap<String,Serializable> map,Serializable value) throws VascBackendException {
		map.put(backendName, value);
	}
}
