/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.backend.list;

import java.io.Serializable;
import java.util.List;
import java.util.Map;

/**
 * DefaultVascBackendResult.
 * 
 * @author Willem Cazander
 * @version 1.0 Aug 23, 2014
 */
@SuppressWarnings("serial")
public final class DefaultVascBackendListResult<DATA_OBJECT extends Serializable> implements VascBackendListResult<DATA_OBJECT> {
	
	private final List<DATA_OBJECT> data;
	private final long totalRecordCount;
	private final Map<String,Map<String,Serializable>> dataExtra;
	
	public DefaultVascBackendListResult(List<DATA_OBJECT> data) {
		this(data,data.size());
	}
	
	public DefaultVascBackendListResult(List<DATA_OBJECT> data,long totalRecordCount) {
		this(data,totalRecordCount,null);
	}
	
	public DefaultVascBackendListResult(List<DATA_OBJECT> data,long totalRecordCount,Map<String,Map<String,Serializable>> dataExtra) {
		this.data = data;
		this.totalRecordCount = totalRecordCount;
		this.dataExtra = dataExtra;
	}
	
	/**
	 * @see net.forwardfire.vasc.backend.list.VascBackendListResult#getData()
	 */
	@Override
	public List<DATA_OBJECT> getData() {
		return data;
	}
	
	/**
	 * @see net.forwardfire.vasc.backend.list.VascBackendListResult#getTotalRecordCount()
	 */
	@Override
	public long getTotalRecordCount() {
		return totalRecordCount;
	}
	
	/**
	 * @see net.forwardfire.vasc.backend.list.VascBackendListResult#getDataExtra()
	 */
	@Override
	public Map<String,Map<String,Serializable>> getDataExtra() {
		return dataExtra;
	}
}
