/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.backend.list;

import java.io.Serializable;
import java.util.EnumSet;
import java.util.List;

import net.forwardfire.vasc.backend.AbstractVascBackendRecord;
import net.forwardfire.vasc.backend.VascBackendException;

/**
 * VascBackendList provides stateless data access to a data source which result one or multiple results.
 * 
 * @author Willem Cazander
 * @version 1.0 Aug 25, 2014
 */
public interface VascBackendList<DATA_PK extends Serializable,DATA_RECORD extends Serializable> extends AbstractVascBackendRecord<DATA_PK,DATA_RECORD> {
	
	final static String ROW_ACTION_MOVE_UP = "rowActionMoveUp";
	final static String ROW_ACTION_MOVE_DOWN = "rowActionMoveDown";
	final static String ROW_ACTION_DELETE = "rowActionDelete";
	
	/**
	 * Defines the features this list backend supports.
	 * @return	An enum set of feature flags.
	 */
	EnumSet<VascBackendListFeature> getSupportedFeatures();
	
	/**
	 * Executes the list request with parameters to return a list result.
	 * @param request
	 * @return
	 * @throws VascBackendException
	 */
	VascBackendListResult<DATA_RECORD> execute(VascBackendListRequest request) throws VascBackendException;
	
	/**
	 * @return	Returns the action names of the row actions we support.
	 */
	List<String> getSupportedRowActions();
	
	/**
	 * Executes a single row action on one or more rows of fresh data.
	 * @param request
	 * @param recordPKs
	 * @param actionName
	 * @throws VascBackendException
	 */
	void executeRowAction(VascBackendListRequest request,List<DATA_PK> recordPKs,String actionName) throws VascBackendException;
}
