/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.backend.jdbc;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;

import net.forwardfire.vasc.backend.AbstractVascBackendLocal;
import net.forwardfire.vasc.backend.VascBackendAccessDataKey;
import net.forwardfire.vasc.backend.data.HashMapVascBackendAccessDataKey;

/**
 * AbstractJdbcVascBackend.
 * 
 * @author Willem Cazander
 * @version 1.0 Sep 5, 2008
 */
abstract public class AbstractJdbcVascBackend extends AbstractVascBackendLocal<HashMap<String,Serializable>> {

	private JdbcConnectionProvider jdbcConnectionProvider = null;
	private final Map<String,Class<?>> keyMapType = new HashMap<String,Class<?>>(5);
	
	abstract protected void startBackendJdbc();
	
	/**
	 * @see net.forwardfire.vasc.backend.AbstractVascBackendLocal#startBackendLocal()
	 */
	@Override
	protected final void startBackendLocal() {
		requireNonNull(getJdbcConnectionProvider(), "jdbcConnectionProvider is null.");
		requireNonEmpty(keyMapType.keySet(), "No primary keys defined.");
		startBackendJdbc();
	}
	
	protected final void addKeyMapType(String field,Class<?> fieldType) {
		requireNonEmpty(field,"field has to have value");
		requireNonNull(fieldType,"field has to have value");
		keyMapType.put(field, fieldType);
	}
	
	protected final Map<String,Class<?>> getKeyMapType() {
		return keyMapType;
	}
	
	/**
	 * @see net.forwardfire.vasc.backend.VascBackend#createVascBackendAccessDataKey()
	 */
	@Override
	public final VascBackendAccessDataKey<HashMap<String, Serializable>> createVascBackendAccessDataKey() {
		return new HashMapVascBackendAccessDataKey(keyMapType);
	}
	
	/**
	 * @return the JdbcConnectionProvider
	 */
	public final JdbcConnectionProvider getJdbcConnectionProvider() {
		return jdbcConnectionProvider;
	}
	
	/**
	 * @param JdbcConnectionProvider the JdbcConnectionProvider to set
	 */
	public final void setJdbcConnectionProvider(JdbcConnectionProvider jdbcConnectionProvider) {
		this.jdbcConnectionProvider = jdbcConnectionProvider;
	}
}
