/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.backend.jdbc;

import java.sql.Connection;
import java.sql.SQLException;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;

/**
 * JdbcConnectionProviderJdniImpl provides jdbc connection from jndi data source.
 * 
 * @author Willem Cazander
 * @version 1.0 Mar 15, 2009
 */
public class JdbcConnectionProviderJdniImpl implements JdbcConnectionProvider {

	/** The context in which database data sources are found. */
	private String dataSourceContext = "java:/";
	
	private String dataSourceName = null;
	
	/**
	 * @see net.forwardfire.vasc.backend.jdbc.JdbcConnectionProvider#getJdbcConnection()
	 */
	@Override
	public Connection getJdbcConnection() throws SQLException {
		Connection connection = getDataSource(dataSourceName).getConnection();
		return connection;
	}
	

	/**
	 * Gets a DataSource out of the JNDI context.
	 * <p>
	 * @param name	The name of the data source in the JDNI context.
	 * @return	The data source.
	 */
	private DataSource getDataSource(String name) throws SQLException {

		try {
			Context initialContext = new InitialContext();
			DataSource datasource = (DataSource)initialContext.lookup(dataSourceContext+name);
			if ( datasource == null ) {
				throw new SQLException("Cannot lookup datasource: "+name);
			}
			return datasource;
		} catch ( NamingException e ) {
			throw new SQLException("Cannot get connection " + e,e);
		}
	}
	
	/**
	 * @return the dataSourceContext
	 */
	public String getDataSourceContext() {
		return dataSourceContext;
	}
	
	/**
	 * @param dataSourceContext the dataSourceContext to set
	 */
	public void setDataSourceContext(String dataSourceContext) {
		this.dataSourceContext = dataSourceContext;
	}
	
	/**
	 * @return the dataSourceName
	 */
	public String getDataSourceName() {
		return dataSourceName;
	}
	
	/**
	 * @param dataSourceName the dataSourceName to set
	 */
	public void setDataSourceName(String dataSourceName) {
		this.dataSourceName = dataSourceName;
	}
}
