/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.backend.jdbc;

import java.io.Serializable;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.EnumSet;
import java.util.HashMap;
import java.util.List;

import net.forwardfire.vasc.backend.VascBackendException;
import net.forwardfire.vasc.backend.VascBackendAccessDataRecord;
import net.forwardfire.vasc.backend.data.HashMapVascBackendAccessDataRecord;
import net.forwardfire.vasc.backend.list.DefaultVascBackendListResult;
import net.forwardfire.vasc.backend.list.VascBackendList;
import net.forwardfire.vasc.backend.list.VascBackendListFeature;
import net.forwardfire.vasc.backend.list.VascBackendListResult;
import net.forwardfire.vasc.backend.list.VascBackendListRequest;
import net.forwardfire.vasc.backend.select.VascBackendSelect;
import net.forwardfire.vasc.backend.select.VascBackendSelectItem;

/**
 * JdbcVascBackend Simple jdbc vasc backend.
 * 
 * @author Willem Cazander
 * @version 1.0 Sep 5, 2008
 */
public class JdbcVascBackendSelect extends AbstractJdbcVascBackend implements VascBackendSelect<HashMap<String,Serializable>> {

	private String query = null;
	
	@Override
	protected void startBackendJdbc() {
		requireNonEmpty(getQuery(), "query is empty.");
	}
	
	@Override
	public List<VascBackendSelectItem<HashMap<String, Serializable>>> execute()	throws VascBackendException {
		return null;
	}
	
	public VascBackendListResult<HashMap<String,Serializable>> execute(VascBackendListRequest state) throws VascBackendException {
		JdbcConnectionProvider prov = getJdbcConnectionProvider();
		List<HashMap<String,Serializable>> result = new ArrayList<HashMap<String,Serializable>>(50);
		Connection connection = null;
		try {
			connection = prov.getJdbcConnection();
			Statement s = connection.createStatement();
			s.execute(getQuery());
			ResultSet rs = s.getResultSet();
			int cols = rs.getMetaData().getColumnCount();
			while (rs.next()) {
				HashMap<String,Serializable> map = new HashMap<String,Serializable>(cols);
				for (int i=1;i<=cols;i++) {
					String columnName = rs.getMetaData().getColumnName(i);
					Serializable columnObject = (Serializable) rs.getObject(i);
					map.put(columnName, columnObject);
				}
				result.add(map);
			}
		} catch (SQLException e) {
			throw new VascBackendException(e);
		} finally {
			if (connection!=null) {
				try {
					connection.close();
				} catch (Exception e) {
				}
			}
		}
		return new DefaultVascBackendListResult<HashMap<String,Serializable>>(result);
	}
	
	/**
	 * @return the query
	 */
	public String getQuery() {
		return query;
	}
	
	/**
	 * @param query the query to set
	 */
	public void setQuery(String query) {
		this.query = query;
	}
}
