/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.backend.jdbc.xpql;

import java.io.Serializable;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.EnumSet;
import java.util.HashMap;
import java.util.List;

import net.forwardfire.vasc.backend.VascBackendException;
import net.forwardfire.vasc.backend.VascBackendAccessDataRecord;
import net.forwardfire.vasc.backend.jdbc.AbstractJdbcVascBackend;
import net.forwardfire.vasc.backend.list.DefaultVascBackendListResult;
import net.forwardfire.vasc.backend.list.VascBackendList;
import net.forwardfire.vasc.backend.list.VascBackendListFeature;
import net.forwardfire.vasc.backend.list.VascBackendListResult;
import net.forwardfire.vasc.backend.list.VascBackendListRequest;
import net.forwardfire.vasc.xpql.query.QueryParameterValue;

/**
 * JdbcVascBackendXpql jdbc vasc backend with xpql query support.
 * 
 * @author Willem Cazander
 * @version 1.0 Sep 5, 2008
 */
public class JdbcVascBackendXpqlList extends AbstractJdbcVascBackend implements VascBackendList<HashMap<String,Serializable>,HashMap<String,Serializable>> {

	private final EnumSet<VascBackendListFeature> features = EnumSet.noneOf(VascBackendListFeature.class);
	private final List<String> rowActions = new ArrayList<String>();
	private net.forwardfire.vasc.xpql.query.Query query = null;
	private net.forwardfire.vasc.xpql.query.Query queryExtra = null;
	
	@Override
	protected void startBackendJdbc() {
		requireNonNull(getQuery(), "query is null.");
		if (queryExtra!=null) {
			features.add(VascBackendListFeature.DATA_EXTRA);
		}
	}
	
	@Override
	public VascBackendAccessDataRecord<HashMap<String, Serializable>, HashMap<String, Serializable>> createVascBackendAccessDataRecord() {
		// TODO Auto-generated method stub
		return null;
	}
	
	@Override
	public EnumSet<VascBackendListFeature> getSupportedFeatures() {
		return features;
	}
	
	@Override
	public List<String> getSupportedRowActions() {
		return rowActions;
	}
	
	@Override
	public void executeRowAction(VascBackendListRequest request,List<HashMap<String, Serializable>> recordPKs, String actionName) throws VascBackendException {
	}
	
	/**
	 * @see net.forwardfire.vasc.backend.crud.VascBackendCrud#execute(VascBackendListRequest state)
	 */
	@Override
	public VascBackendListResult<HashMap<String,Serializable>> execute(VascBackendListRequest state) throws VascBackendException {
		// Copy parameters
		for (String key:state.getDataParameterKeys()) {
			Object value = state.getDataParameter(key);
			query.setQueryParameter(key, value);
		}
		Connection c = null;
		try {
			c = getJdbcConnectionProvider().getJdbcConnection();;
			PreparedStatement q = c.prepareStatement(query.toPreparedSQL(query));
			
			List<QueryParameterValue> values = query.getOrderQueryParameterValues();
			int ii = 1;
			for (QueryParameterValue value:values) {
				q.setObject(ii,value.getValue());
				ii++;
			}
			q.execute();
			ResultSet rs = q.getResultSet();
			int cols = rs.getMetaData().getColumnCount();
			List<HashMap<String,Serializable>> result = new ArrayList<HashMap<String,Serializable>>(50); 
			while (rs.next()) {
				HashMap<String,Serializable> map = new HashMap<String,Serializable>(cols);
				for (int i=1;i<=cols;i++) {
					String columnName = rs.getMetaData().getColumnName(i);
					Serializable columnObject = (Serializable) rs.getObject(i);
					map.put(columnName, columnObject);
				}
				result.add(map);
			}
			return new DefaultVascBackendListResult<HashMap<String,Serializable>>(result);
		} catch (SQLException e) {
			throw new VascBackendException(e);
		} finally {
			if (c!=null) {
				try {
					c.close();
				} catch (Exception e) {
				}
			}
		}
	}
	
	/**
	 * @return the query
	 */
	public net.forwardfire.vasc.xpql.query.Query getQuery() {
		return query;
	}
	
	/**
	 * @param query the query to set
	 */
	public void setQuery(net.forwardfire.vasc.xpql.query.Query query) {
		this.query = query;
	}
}
