/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.backends.jpa;

import java.io.Serializable;

import javax.persistence.EntityManager;

import net.forwardfire.vasc.backend.AbstractVascBackendLocal;

/**
 * AbstractPersistenceVascBackend.
 * 
 * @author Willem Cazander
 * @version 1.0 Aug 25, 2014
 */
abstract public class AbstractPersistenceVascBackend<DATA_PK extends Serializable> extends AbstractVascBackendLocal<DATA_PK> {

	private Boolean emTransaction;
	
	private EntityManagerProvider entityManagerProvider = null;
	
	abstract protected void startBackendPersistence();
	
	/**
	 * @see net.forwardfire.vasc.backend.AbstractVascBackendLocal#startBackendLocal()
	 */
	@Override
	protected final void startBackendLocal() {
		requireNonNull(getEntityManagerProvider(), "entityManagerProvider is null.");
		//requireNonEmpty(keyMapType.keySet(), "No primary keys defined.");
		startBackendPersistence();
	}
	
	protected final EntityManager getEntityManager() {
		if (emTransaction == null) {
			emTransaction=entityManagerProvider.hasEntityManagerTransaction();
		}
		return entityManagerProvider.getEntityManager();
	}
	
	protected final boolean hasTransaction() {
		return emTransaction;
	}
	
	/**
	 * @return the entityManagerProvider
	 */
	public final EntityManagerProvider getEntityManagerProvider() {
		return entityManagerProvider;
	}
	
	/**
	 * @param entityManagerProvider the entityManagerProvider to set
	 */
	public final void setEntityManagerProvider(EntityManagerProvider entityManagerProvider) {
		this.entityManagerProvider = entityManagerProvider;
	}
}