/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.backends.jpa;

import java.io.Serializable;

import javax.persistence.EntityManager;

import net.forwardfire.vasc.backend.VascBackendAccessDataKey;
import net.forwardfire.vasc.backend.VascBackendException;
import net.forwardfire.vasc.backend.VascBackendAccessDataRecord;
import net.forwardfire.vasc.backend.crud.VascBackendCrud;

/**
 * Manages persistance with xpql queries
 * 
 * @author Willem Cazander
 * @version 1.0 Mar 17, 2009
 */
public class PersistanceVascBackendCrud<DATA_PK extends Serializable,DATA_RECORD extends Serializable> extends AbstractPersistenceVascBackend<DATA_PK> implements VascBackendCrud<DATA_PK,DATA_RECORD> {
	
	private Class<DATA_RECORD> recordClass = null;
	
	public PersistanceVascBackendCrud() {
	}
	
	public PersistanceVascBackendCrud(Class<DATA_RECORD> recordClass) {
		setRecordClass(recordClass);
	}
	
	@Override
	protected void startBackendPersistence() {
		requireNonNull(getRecordClass(), "recordClass is null");
	}
	
	@Override
	public VascBackendAccessDataRecord<DATA_PK, DATA_RECORD> createVascBackendAccessDataRecord() {
		// TODO Auto-generated method stub
		return null;
	}
	
	@Override
	public VascBackendAccessDataKey<DATA_PK> createVascBackendAccessDataKey() {
		// TODO Auto-generated method stub
		return null;
	}
	
	
	@Override
	public DATA_RECORD newRecord() throws VascBackendException {
		try {
			return getRecordClass().newInstance();
		} catch (InstantiationException | IllegalAccessException e) {
			throw new VascBackendException(e);
		}
	}
	
	@Override
	public DATA_RECORD fetch(DATA_PK recordPK) throws VascBackendException {
		EntityManager s  = getEntityManager();
		try {
			if (hasTransaction()) {
				s.getTransaction().begin();
			}
			DATA_RECORD record = s.find(getRecordClass(), recordPK);
			if (hasTransaction()) {
				s.getTransaction().commit();
			}
			return record;
		} finally {
			if (s!=null) {
				//s.close();
			}
		}
	}
	
	@Override
	public DATA_RECORD persist(DATA_RECORD record) throws VascBackendException {
		EntityManager s  = getEntityManager();
		try {
			if (hasTransaction()) {
				s.getTransaction().begin();
			}
			s.persist(record);
			if (hasTransaction()) {
				s.getTransaction().commit();
			}
			return record;
		} finally {
			if (s!=null) {
				//s.close();
			}
		}
	}
	
	@Override
	public DATA_RECORD merge(DATA_RECORD record) throws VascBackendException {
		EntityManager s  = getEntityManager();
		try {
			if (hasTransaction()) {
				s.getTransaction().begin();
			}
			DATA_RECORD result = (DATA_RECORD)s.merge(record);
			if (hasTransaction()) {
				s.getTransaction().commit();
			}
			return result;
		} finally {
			if (s!=null) {
				//s.close();
			}
		}
	}
	
	@Override
	public void delete(DATA_PK recordPK) throws VascBackendException {
		EntityManager s  = getEntityManager();
		try {
			if (hasTransaction()) {
				s.getTransaction().begin();
			}
			Object newObject = s.find(Object.class,recordPK);
			s.remove(newObject);
			if (hasTransaction()) {
				s.getTransaction().commit();
			}
		} finally {
			if (s!=null) {
				//s.close();
			}
		}
	}
	
	public Class<DATA_RECORD> getRecordClass() {
		return recordClass;
	}
	
	public void setRecordClass(Class<DATA_RECORD> recordClass) {
		this.recordClass = recordClass;
	}
}
