/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.backends.jpa;

import java.io.Serializable;
import java.util.EnumSet;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.Query;

import net.forwardfire.vasc.backend.VascBackendAccessDataKey;
import net.forwardfire.vasc.backend.VascBackendException;
import net.forwardfire.vasc.backend.VascBackendAccessDataRecord;
import net.forwardfire.vasc.backend.data.BeanVascBackendAccessDataRecord;
import net.forwardfire.vasc.backend.list.DefaultVascBackendListResult;
import net.forwardfire.vasc.backend.list.VascBackendList;
import net.forwardfire.vasc.backend.list.VascBackendListFeature;
import net.forwardfire.vasc.backend.list.VascBackendListResult;
import net.forwardfire.vasc.backend.list.VascBackendListRequest;
import net.forwardfire.vasc.xpql.query.QueryParameterValue;

/**
 * Manages persistance with xpql queries
 * 
 * @author Willem Cazander
 * @version 1.0 Mar 17, 2009
 */
public class PersistanceVascBackendXpqlList<DATA_PK extends Serializable,DATA_RECORD extends Serializable> extends AbstractPersistenceVascBackend<DATA_PK> implements VascBackendList<DATA_PK,DATA_RECORD> {
	
	private net.forwardfire.vasc.xpql.query.Query query = null;
	
	private net.forwardfire.vasc.xpql.query.Query queryTotal = null;
	
	private net.forwardfire.vasc.xpql.query.Query queryMoveUp = null;
	
	private net.forwardfire.vasc.xpql.query.Query queryMoveUpDown = null;
	
	private net.forwardfire.vasc.xpql.query.Query queryMoveDown = null;
	
	private net.forwardfire.vasc.xpql.query.Query queryMoveDownUp = null;
	
	public PersistanceVascBackendXpqlList() {
	}
	
	@Override
	protected void startBackendPersistence() {
		// TODO Auto-generated method stub
		
	}
	
	@Override
	public VascBackendAccessDataRecord<DATA_PK, DATA_RECORD> createVascBackendAccessDataRecord() {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public VascBackendAccessDataKey<DATA_PK> createVascBackendAccessDataKey() {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public EnumSet<VascBackendListFeature> getSupportedFeatures() {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public List<String> getSupportedRowActions() {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public void executeRowAction(VascBackendListRequest request,List<DATA_PK> recordPKs, String actionName) throws VascBackendException {
		// TODO Auto-generated method stub
		
	}
	
	@Override
	public VascBackendListResult<DATA_RECORD> execute(VascBackendListRequest state) throws VascBackendException {
		return new DefaultVascBackendListResult<DATA_RECORD>(executeList(state),fetchTotalExecuteSize(state));
	}
	
	@SuppressWarnings("unchecked")
	private List<DATA_RECORD> executeList(VascBackendListRequest state) throws VascBackendException {
		
		// Copy parameters
		for (String key:state.getDataParameterKeys()) {
			Object value = state.getDataParameter(key);
			query.setQueryParameter(key, value);
			if (queryTotal!=null) {
				queryTotal.setQueryParameter(key, value);
			}
		}
		
		EntityManager em = getEntityManager();
		try {
			Query q = em.createQuery(query.toPreparedSQL(query));
			List<QueryParameterValue> values = query.getOrderQueryParameterValues();
			int i = 1;
			for (QueryParameterValue value:values) {
				Object valueObject = value.getValue();
				q.setParameter(i,valueObject);
				//System.out.println("Set para index: "+i+" value: "+valueObject+" valueClass: "+valueObject.getClass()+" valueType: "+value.getValueType());
				i++;
			}
			if (isPageable() && state.getPageSize()>0) {
				q.setFirstResult(state.getPageIndex()*state.getPageSize());
				q.setMaxResults(state.getPageSize());
			}
			if (hasTransaction()) {
				em.getTransaction().begin();
			}
			List<DATA_RECORD> data = q.getResultList();
			if (hasTransaction()) {
				em.getTransaction().commit();
			}
			return data;
		} finally {
			if (em!=null) {
				//em.close();
			}
		}
	}
	
	/**
	 * @return the query
	 */
	public net.forwardfire.vasc.xpql.query.Query getQuery() {
		return query;
	}
	
	/**
	 * @param query the query to set
	 */
	public void setQuery(net.forwardfire.vasc.xpql.query.Query query) {
		this.query = query;
	}
	
	/**
	 * @return the queryTotal
	 */
	public net.forwardfire.vasc.xpql.query.Query getQueryTotal() {
		return queryTotal;
	}
	
	/**
	 * @param queryTotal the queryTotal to set
	 */
	public void setQueryTotal(net.forwardfire.vasc.xpql.query.Query queryTotal) {
		this.queryTotal = queryTotal;
	}
	
	/**
	 * @see net.forwardfire.vasc.backend.crud.VascBackendList#isPageable()
	 */
	public boolean isPageable() {
		if (queryTotal==null) {
			return false;
		}
		return true;
	}
	
	private long fetchTotalExecuteSize(VascBackendListRequest state) {
		EntityManager em = getEntityManager();
		try {
			Query q = em.createQuery(queryTotal.toPreparedSQL(queryTotal));
			List<QueryParameterValue> values = queryTotal.getOrderQueryParameterValues();
			int i = 1;
			for (QueryParameterValue value:values) {
				q.setParameter(i,value.getValue());
				i++;
			}
			if (hasTransaction()) {
				em.getTransaction().begin();
			}
			Long resultTotal = (Long)q.getSingleResult();
			if (hasTransaction()) {
				em.getTransaction().commit();
			}
			return resultTotal;
		} finally {
			if (em!=null) {
				//em.close();
			}
		}
	}
	
	/**
	 * @see net.forwardfire.vasc.core.AbstractVascBackend#doRecordMoveDownById(VascBackendState state,java.lang.Object)
	 */
	public long doRecordMoveDownById(Object primaryId) throws VascBackendException {
		long result = 0l;
		if (queryMoveDown!=null) {
			EntityManager em = getEntityManager();
			try {
				if (hasTransaction()) {
					em.getTransaction().begin();
				}
				
		        // Copy parameters
//		        for (String key:state.getDataParameterKeys()) {
//		        	Object value = state.getDataParameter(key);
//		        	queryMoveDown.setQueryParameter(key, value);
//		        }
				
		        // set id
		        queryMoveDown.setQueryParameter("id", primaryId);
		        
		        // execute update
				Query q = em.createQuery(queryMoveDown.toPreparedSQL(queryMoveDown));
				List<QueryParameterValue> values = queryMoveDown.getOrderQueryParameterValues();
				int i = 1;
				for (QueryParameterValue value:values) {
					q.setParameter(i,value.getValue());          	
					i++;
				}
				result+=q.executeUpdate();
				
		        // Copy parameters
//		        for (String key:state.getDataParameterKeys()) {
//		        	Object value = state.getDataParameter(key);
//		        	queryMoveDownUp.setQueryParameter(key, value);
//		        }
				
		        // set id
		        queryMoveDownUp.setQueryParameter("id", primaryId);
		        
		        // execute update
				q = em.createQuery(queryMoveDownUp.toPreparedSQL(queryMoveDownUp));
				values = queryMoveDownUp.getOrderQueryParameterValues();
				i = 1;
				for (QueryParameterValue value:values) {
					q.setParameter(i,value.getValue());          	
					i++;
				}
				result+=q.executeUpdate();
				
				if (hasTransaction()) {
					em.getTransaction().commit();
				}
			} finally {
				if (em!=null) {
					//em.close();
				}
			}
		}
		return result;
	}

	/**
	 * @see net.forwardfire.vasc.core.AbstractVascBackend#doRecordMoveUpById(VascBackendState state,java.lang.Object)
	 */
	public long doRecordMoveUpById(Object primaryId) throws VascBackendException  {
		long result = 0l;
		if (queryMoveUp!=null) {
			EntityManager em = getEntityManager();
			try {
//				if (emTransaction) {
//					em.getTransaction().begin();
//				}
				
		        // Copy parameters
//		        for (String key:state.getDataParameterKeys()) {
//		        	Object value = state.getDataParameter(key);
//		        	queryMoveUp.setQueryParameter(key, value);
//		        }
				
		        // set id
		        queryMoveUp.setQueryParameter("id", primaryId);
		        
		        // execute update
				Query q = em.createQuery(queryMoveUp.toPreparedSQL(queryMoveUp));
				List<QueryParameterValue> values = queryMoveUp.getOrderQueryParameterValues();
				int i = 1;
				for (QueryParameterValue value:values) {
					q.setParameter(i,value.getValue());          	
					i++;
				}
				result+=q.executeUpdate();
				
		        // Copy parameters
//		        for (String key:state.getDataParameterKeys()) {
//		        	Object value = state.getDataParameter(key);
//		        	queryMoveUpDown.setQueryParameter(key, value);
//		        }
				
		        // set id
		        queryMoveUpDown.setQueryParameter("id", primaryId);
		        
		        // execute update
				q = em.createQuery(queryMoveUpDown.toPreparedSQL(queryMoveUpDown));
				values = queryMoveUpDown.getOrderQueryParameterValues();
				i = 1;
				for (QueryParameterValue value:values) {
					q.setParameter(i,value.getValue());          	
					i++;
				}
				result+=q.executeUpdate();
				
//				if (emTransaction) {
//					em.getTransaction().commit();
//				}
			} finally {
				if (em!=null) {
					//em.close();
				}
			}
		}
		return result;
	}
	
	
	/**
	 * @see net.forwardfire.vasc.backend.crud.VascBackendList#isSearchable()
	 */
	public boolean isSearchable() {
		if (query.getQueryParameterValue("text_search")!=null) {
			return true;
		}
		return false;
	}

	/**
	 * @see net.forwardfire.vasc.backend.crud.AbstractVascBackendCrud#isRecordMoveable()
	 */
	public boolean isRecordMoveable() {
		return queryMoveUp!=null & queryMoveDown!=null;
	}

	/**
	 * @return the queryMoveUp
	 */
	public net.forwardfire.vasc.xpql.query.Query getQueryMoveUp() {
		return queryMoveUp;
	}

	/**
	 * @param queryMoveUp the queryMoveUp to set
	 */
	public void setQueryMoveUp(net.forwardfire.vasc.xpql.query.Query queryMoveUp) {
		this.queryMoveUp = queryMoveUp;
	}

	/**
	 * @return the queryMoveDown
	 */
	public net.forwardfire.vasc.xpql.query.Query getQueryMoveDown() {
		return queryMoveDown;
	}

	/**
	 * @param queryMoveDown the queryMoveDown to set
	 */
	public void setQueryMoveDown(net.forwardfire.vasc.xpql.query.Query queryMoveDown) {
		this.queryMoveDown = queryMoveDown;
	}

	/**
	 * @return the queryMoveUpDown
	 */
	public net.forwardfire.vasc.xpql.query.Query getQueryMoveUpDown() {
		return queryMoveUpDown;
	}

	/**
	 * @param queryMoveUpDown the queryMoveUpDown to set
	 */
	public void setQueryMoveUpDown(net.forwardfire.vasc.xpql.query.Query queryMoveUpDown) {
		this.queryMoveUpDown = queryMoveUpDown;
	}

	/**
	 * @return the queryMoveDownUp
	 */
	public net.forwardfire.vasc.xpql.query.Query getQueryMoveDownUp() {
		return queryMoveDownUp;
	}

	/**
	 * @param queryMoveDownUp the queryMoveDownUp to set
	 */
	public void setQueryMoveDownUp(net.forwardfire.vasc.xpql.query.Query queryMoveDownUp) {
		this.queryMoveDownUp = queryMoveDownUp;
	}
}
