/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.backend.metamodel;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Logger;

import org.eobjects.metamodel.DataContext;

import net.forwardfire.vasc.backend.AbstractVascBackendLocal;
import net.forwardfire.vasc.backend.VascBackendAccessDataKey;
import net.forwardfire.vasc.backend.data.HashMapVascBackendAccessDataKey;

/**
 * AbstractMetaModelVascBackend provides shared metamodel options.
 * 
 * @author Willem Cazander
 * @version 1.0 Oct 27, 2014
 */
abstract public class AbstractMetaModelVascBackend extends AbstractVascBackendLocal<HashMap<String,Serializable>> {

	private Logger logger = Logger.getLogger(MetaModelVascBackendCrud.class.getName());
	private MetaModelDataContextProvider dataContextProvider = null;
	private DataContext dataContext = null;
	
	private String table = null;
	private String tableId = null;
	private final Map<String,Class<?>> keyMapType = new HashMap<String,Class<?>>(5);
	
	abstract protected void startBackendMetaModel();
	
	@Override
	protected void stopBackendLocal() {
		dataContext = null;
	}
	
	/**
	 * @see net.forwardfire.vasc.backend.AbstractVascBackendLocal#startBackendLocal()
	 */
	@Override
	protected final void startBackendLocal() {
		requireNonNull(getDataContextProvider(), "dataContextProvider is null.");
		requireNonEmpty(keyMapType.keySet(), "No primary keys defined.");
		requireNonNull(table,"table needs value");
		//requireNonNull(tableId,"table needs value");
		
		long startTime = System.currentTimeMillis();
		dataContext = dataContextProvider.getDataContext();
		requireNonNull(dataContext,"dataContextProvider created  null dataContext.");

		long stopTime = System.currentTimeMillis();
		logger.fine(dataContext.getClass().getSimpleName()+" created for: "+table+" in: "+(stopTime-startTime)+" ms.");
		
		startBackendMetaModel();
	}
	
	protected final void addKeyMapType(String field,Class<?> fieldType) {
		requireNonEmpty(field,"field has to have value");
		requireNonNull(fieldType,"field has to have value");
		keyMapType.put(field, fieldType);
	}
	
	protected final Map<String,Class<?>> getKeyMapType() {
		return keyMapType;
	}
	
	protected DataContext getDataContext() {
		return dataContext;
	}
	
	/**
	 * @see net.forwardfire.vasc.backend.VascBackend#createVascBackendAccessDataKey()
	 */
	@Override
	public final VascBackendAccessDataKey<HashMap<String, Serializable>> createVascBackendAccessDataKey() {
		return new HashMapVascBackendAccessDataKey(keyMapType);
	}
	
	/**
	 * @return the dataContextProvider
	 */
	public final MetaModelDataContextProvider getDataContextProvider() {
		return dataContextProvider;
	}
	
	/**
	 * @param dataContextProvider the dataContextProvider to set
	 */
	public final void setDataContextProvider(MetaModelDataContextProvider dataContextProvider) {
		this.dataContextProvider = dataContextProvider;
	}
	
	/**
	 * @return the table
	 */
	public final String getTable() {
		return table;
	}
	
	/**
	 * @param table the table to set
	 */
	public final void setTable(String table) {
		this.table = table;
	}
	
	/**
	 * @return the tableId
	 */
	public final String getTableId() {
		return tableId;
	}
	
	/**
	 * @param tableId the tableId to set
	 */
	public final void setTableId(String tableId) {
		this.tableId = tableId;
	}
}
