/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.backend.metamodel;

import java.io.File;
import java.net.URL;

import org.eobjects.metamodel.DataContext;
import org.eobjects.metamodel.csv.CsvConfiguration;
import org.eobjects.metamodel.csv.CsvDataContext;

/**
 * MetaModelDataContextCsv provides csv file/url backend.
 * 
 * @author Willem Cazander
 * @version 1.0 May 3, 2012
 */
public class MetaModelDataContextCsv implements MetaModelDataContextProvider {

	private String file = null;
	private String resource = null;
	private String url = null;
	private CsvConfiguration csvConfiguration = null;
	
	@Override
	public DataContext getDataContext() {
		if (csvConfiguration==null) {
			csvConfiguration = new CsvConfiguration();
		}
		CsvDataContext dataContext;
		if (file != null) {
			dataContext = new CsvDataContext(new File(file),csvConfiguration);	
		} else if (url != null) {
			try {
				dataContext = new CsvDataContext(new URL(url),csvConfiguration);
			} catch (Exception e) {
				throw new IllegalStateException("Could not parse url: "+e.getMessage(),e);
			}
		} else {
			throw new IllegalStateException("No file or url defined.");
		}
		return dataContext;
	}

	/**
	 * @return the file
	 */
	public String getFile() {
		return file;
	}

	/**
	 * @param file the file to set
	 */
	public void setFile(String file) {
		this.file = file;
	}

	/**
	 * @return the resource
	 */
	public String getResource() {
		return resource;
	}

	/**
	 * @param resource the resource to set
	 */
	public void setResource(String resource) {
		this.resource = resource;
	}

	/**
	 * @return the url
	 */
	public String getUrl() {
		return url;
	}

	/**
	 * @param url the url to set
	 */
	public void setUrl(String url) {
		this.url = url;
	}

	/**
	 * @return the csvConfiguration
	 */
	public CsvConfiguration getCsvConfiguration() {
		return csvConfiguration;
	}

	/**
	 * @param csvConfiguration the csvConfiguration to set
	 */
	public void setCsvConfiguration(CsvConfiguration csvConfiguration) {
		this.csvConfiguration = csvConfiguration;
	}
}
