package net.forwardfire.vasc.backend.metamodel;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;

import org.eobjects.metamodel.data.Row;
import org.eobjects.metamodel.query.SelectItem;

import net.forwardfire.vasc.backend.VascBackendException;
import net.forwardfire.vasc.backend.VascBackendAccessDataRecord;
import net.forwardfire.vasc.backend.metamodel.crud.UpdateableRow;

@SuppressWarnings("serial")
public class MetaModelVascBackendAccessDataRecord implements VascBackendAccessDataRecord<HashMap<String,Serializable>,Row> {
	
	private final Map<String,Class<?>> keyMapType;
	
	public MetaModelVascBackendAccessDataRecord(Map<String,Class<?>> keyMapType) {
		this.keyMapType = keyMapType;
	}
	
	@Override
	public HashMap<String, Serializable> getPrimaryKey(Row record) throws VascBackendException {
		HashMap<String, Serializable> result = new HashMap<String, Serializable>();
		for (String key:keyMapType.keySet()) {
			result.put(key, getValue(key,record));
		}
		return result;
	}
	
	/**
	 * @see net.forwardfire.vasc.backend.VascBackendAccessDataRecord#getValue(java.lang.String, java.lang.Object)
	 */
	@Override
	public Serializable getValue(String backendName, Row record) throws VascBackendException {
		if (backendName==null) {
			throw new NullPointerException("Can't get value of null backendName.");
		}
		if (record==null) {
			throw new NullPointerException("Can't get value of null object.");
		}
		if (record instanceof UpdateableRow) {
			UpdateableRow row = (UpdateableRow)record;
			return (Serializable) row.getValue(backendName);
		}
		Serializable fieldValue = (Serializable) record.getValue(indexOf(backendName,record));
		return fieldValue;
	}
	
	private int indexOf(String columnName,Row row) { // RM after MM Row update
		if (columnName==null) {
			return UpdateableRow.INDEX_NOT_FOUND;
		}
		int index = 0;
		for (SelectItem si:row.getSelectItems()) {
			if (si.getColumn().getName().equals(columnName)) {
				return index;
			}
			index++;
		}
		return UpdateableRow.INDEX_NOT_FOUND;
	}
	
	/**
	 * @see net.forwardfire.vasc.backend.VascBackendAccessDataRecord#setValue(java.lang.String, java.lang.Object, java.lang.Object)
	 */
	@Override
	public void setValue(String backendName, Row record,Serializable value) throws VascBackendException {
		if (record instanceof UpdateableRow) {
			UpdateableRow row = (UpdateableRow)record;
			row.setValue(backendName, value);
		}
	}
}