/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.backend.metamodel;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.EnumSet;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Logger;

import org.eobjects.metamodel.DataContext;
import org.eobjects.metamodel.UpdateableDataContext;
import org.eobjects.metamodel.data.DataSet;
import org.eobjects.metamodel.data.Row;
import org.eobjects.metamodel.jdbc.JdbcDataContext;
import org.eobjects.metamodel.query.OrderByItem.Direction;
import org.eobjects.metamodel.query.Query;
import org.eobjects.metamodel.query.SelectItem;
import org.eobjects.metamodel.query.builder.SatisfiedQueryBuilder;
import org.eobjects.metamodel.query.builder.SatisfiedWhereBuilder;
import org.eobjects.metamodel.schema.Column;
import org.eobjects.metamodel.schema.Schema;
import org.eobjects.metamodel.schema.Table;

import net.forwardfire.vasc.backend.VascBackendAccessDataRecord;
import net.forwardfire.vasc.backend.VascBackendException;
import net.forwardfire.vasc.backend.crud.VascBackendCrud;
import net.forwardfire.vasc.backend.data.HashMapVascBackendAccessDataRecord;
import net.forwardfire.vasc.backend.list.DefaultVascBackendListResult;
import net.forwardfire.vasc.backend.list.VascBackendList;
import net.forwardfire.vasc.backend.list.VascBackendListFeature;
import net.forwardfire.vasc.backend.list.VascBackendListResult;
import net.forwardfire.vasc.backend.list.VascBackendListRequest;
import net.forwardfire.vasc.backend.metamodel.crud.CrudDataContext;
import net.forwardfire.vasc.backend.metamodel.crud.UpdateableRow;
import net.forwardfire.vasc.backend.metamodel.crud.CrudDataContextImpl;
import net.forwardfire.vasc.backend.metamodel.crud.UpdateableRowMapImpl;

/**
 * MetaModelVascBackend provides vasc backend for metamodel.
 * 
 * @author Willem Cazander
 * @version 1.0 Dec 31, 2011
 */
public class MetaModelVascBackendList extends AbstractMetaModelVascBackend implements VascBackendList<HashMap<String,Serializable>,Row> {

	private Logger logger = Logger.getLogger(MetaModelVascBackendList.class.getName());
	private final EnumSet<VascBackendListFeature> features = EnumSet.noneOf(VascBackendListFeature.class);
	private final List<String> rowActions = new ArrayList<String>();
	
	@Override
	protected void startBackendMetaModel() {
	}
	
	@Override
	public VascBackendAccessDataRecord<HashMap<String,Serializable>,Row> createVascBackendAccessDataRecord() {
		return new MetaModelVascBackendAccessDataRecord(getKeyMapType());
	}
	
	@Override
	public EnumSet<VascBackendListFeature> getSupportedFeatures() {
		return features;
	}
	
	@Override
	public List<String> getSupportedRowActions() {
		return rowActions;
	}
	
	@Override
	public void executeRowAction(VascBackendListRequest request,List<HashMap<String, Serializable>> recordPKs, String actionName) throws VascBackendException {
	}
	
	private long fetchTotalExecuteSize(VascBackendListRequest state) {
		DataContext dataContext = getDataContext();
		Schema schema = dataContext.getDefaultSchema();
		Table t = schema.getTableByName(getTable());
		Query q = createFilterQuery(state,t,true);
		DataSet ds = dataContext.executeQuery(q);
		if (ds.next()==false) {
			ds.close();
			return -1;
		}
		Row row = ds.getRow();
		Number result = (Number)row.getValue(0);
		ds.close();
		return result.longValue();
	}
	
	private Query createFilterQuery(VascBackendListRequest state,Table t,boolean count) {
		DataContext dataContext = getDataContext();
		
		Object qWhere = null;
		if (count==false) {
			qWhere = dataContext.query().from(t).select(t.getColumns());
		} else {
			qWhere = dataContext.query().from(t).selectCount();	
		}
		Iterator<String> keys = state.getDataParameterKeys().iterator();
		boolean first = true;
		while (keys.hasNext()) {
			String key = keys.next();
			Object value = state.getDataParameter(key); /// TODO: redo this nasty code
			if (value instanceof Number) {
				if (first) {					
					qWhere = ((SatisfiedQueryBuilder<?>)qWhere).where(key).isEquals((Number)value);
				} else {
					qWhere = ((SatisfiedWhereBuilder<?>)qWhere).and(key).isEquals((Number)value);
				}
			} else if (value instanceof Date) {
				if (first) {
					qWhere = ((SatisfiedQueryBuilder<?>)qWhere).where(key).isEquals((Date)value);
				} else {
					qWhere = ((SatisfiedWhereBuilder<?>)qWhere).and(key).isEquals((Date)value);
				}
			} else if (value instanceof Boolean) {
				if (first) {
					qWhere = ((SatisfiedQueryBuilder<?>)qWhere).where(key).isEquals((Boolean)value);
				} else {
					qWhere = ((SatisfiedWhereBuilder<?>)qWhere).and(key).isEquals((Boolean)value);
				}
			} else {
				if (value==null) {
					if (first) {
						qWhere = ((SatisfiedQueryBuilder<?>)qWhere).where(key).isNull();
					} else {
						qWhere = ((SatisfiedWhereBuilder<?>)qWhere).and(key).isNull();
					}
				} else {
					if (first) {
						qWhere = ((SatisfiedQueryBuilder<?>)qWhere).where(key).isEquals(value.toString());
					} else {
						qWhere = ((SatisfiedWhereBuilder<?>)qWhere).and(key).isEquals(value.toString());
					}
				}
			}
			first = false;
		}
		Query q = ((SatisfiedQueryBuilder<?>)qWhere).toQuery();
		return q;
	}
	
	@Override
	public VascBackendListResult<Row> execute(VascBackendListRequest state) throws VascBackendException {
		return new DefaultVascBackendListResult<Row>(executeList(state),fetchTotalExecuteSize(state));
	}
	
	private List<Row> executeList(VascBackendListRequest state) throws VascBackendException {
		DataContext dataContext = getDataContext();
		Schema schema = dataContext.getDefaultSchema();
		Table t = schema.getTableByName(getTable());
		if (t==null) {
			throw new VascBackendException("Could not get meta table for: '"+getTable()+"'.");
		}
		Query q = createFilterQuery(state,t,false);
		if (state.getSortField() != null) {
			Column orderColumn = t.getColumnByName(state.getSortField());
			if (state.isSortAscending()) {
				q.orderBy(orderColumn,Direction.ASC);
			} else {
				q.orderBy(orderColumn,Direction.DESC);
			}
		}
		if (state.getPageSize()>0) {
			if (state.getPageIndex()>0) {
				q.setFirstRow((state.getPageIndex()*state.getPageSize())+1); // MM is 1 based ??
				
				// workaround for bug in MM in jdbc/pg mode which is zero based.
				if (dataContext instanceof JdbcDataContext) {
					q.setFirstRow((state.getPageIndex()*state.getPageSize())+0); // MM is 1 based but sql is 0 based
				}
			}
			q.setMaxRows(state.getPageSize());
		}
		List<Row> result = new ArrayList<Row>(50); 
//		if (crudDataContext!=null) {
//			DataSet ds = crudDataContext.executeQuery(q);
//			result.addAll(ds.toRows());
//			ds.close();
//			return result;
//		}
		
		DataSet ds = dataContext.executeQuery(q);
		while (ds.next()) {
			Row row = ds.getRow();
			SelectItem[] cols = row.getSelectItems();
			List<String> keys = new ArrayList<String>(1);
			keys.add(cols[0].getColumn().getName());
			UpdateableRowMapImpl rowMM = new UpdateableRowMapImpl(dataContext.getDefaultSchema().getTableByName(getTable()),keys,cols);
			for (SelectItem col:cols) {
				Object value = row.getValue(col);
				rowMM.setValue(col, value);
			}
			result.add(rowMM);
		}
		ds.close();

		return result;
	}
}
