package net.forwardfire.vasc.backend.metamodel;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.SortedMap;
import java.util.TreeMap;
import java.util.Map.Entry;

import org.bson.types.ObjectId;
import org.eobjects.metamodel.MetaModelException;
import org.eobjects.metamodel.QueryPostprocessDataContext;
import org.eobjects.metamodel.data.AbstractDataSet;
import org.eobjects.metamodel.data.DataSet;
import org.eobjects.metamodel.data.DefaultRow;
import org.eobjects.metamodel.data.FirstRowDataSet;
import org.eobjects.metamodel.data.MaxRowsDataSet;
import org.eobjects.metamodel.data.Row;
import org.eobjects.metamodel.query.FilterItem;
import org.eobjects.metamodel.query.FromItem;
import org.eobjects.metamodel.query.Query;
import org.eobjects.metamodel.query.SelectItem;
import org.eobjects.metamodel.schema.Column;
import org.eobjects.metamodel.schema.ColumnType;
import org.eobjects.metamodel.schema.MutableColumn;
import org.eobjects.metamodel.schema.MutableRelationship;
import org.eobjects.metamodel.schema.MutableSchema;
import org.eobjects.metamodel.schema.MutableTable;
import org.eobjects.metamodel.schema.Relationship;
import org.eobjects.metamodel.schema.Schema;
import org.eobjects.metamodel.schema.Table;
import org.eobjects.metamodel.schema.TableType;

import com.mongodb.BasicDBList;
import com.mongodb.BasicDBObject;
import com.mongodb.DB;
import com.mongodb.DBCollection;
import com.mongodb.DBCursor;
import com.mongodb.DBObject;

public class MongoDbDataContextSchemaDetector /* implements MetaModelSchemaDetector */ {

	private int dataCheckSize = 100;
	private boolean searchReference = true;
	
	private MongoDbDataContextSchemaDetector() {
	}
	
	public class MongoDbDataContextExtended extends QueryPostprocessDataContext /* MongoDbDataContext */ {
		protected DB mongoDb;
		protected Schema schema;
		protected MongoDbDataContextSchemaDetector walker;
		public MongoDbDataContextExtended(DB mongoDb,MongoDbDataContextSchemaDetector walker) {
	        this.mongoDb=mongoDb;
	        this.walker=walker;
	    }
	    @Override
	    protected Schema getMainSchema() throws MetaModelException {
	    	if (schema!=null) {
	    		return schema;
	    	}
	    	schema = walker.detectSchema(mongoDb, getMainSchemaName());
	        return schema;
	    }
	    
	    @Override
	    protected String getMainSchemaName() throws MetaModelException {
	        return mongoDb.getName();
	    }
	    
	    @Override
	    protected DataSet materializeMainSchemaTable(Table table, Column[] columns, int maxRows) {
	        return materializeMainSchemaTableInternal(table, columns, null, maxRows, true);
	    }

		// Overidded these method because they do not use the getMainSchema() method
	    @Override
	    public DataSet executeQuery(Query query) {
	        // Check for queries containing only simple selects and where clauses,
	        // or if it is a COUNT(*) query.

	        // if from clause only contains a main schema table
	        List<FromItem> fromItems = query.getFromClause().getItems();
	        if (fromItems.size() == 1 && fromItems.get(0).getTable() != null
	                && fromItems.get(0).getTable().getSchema() == getMainSchema()) {
	            final Table table = fromItems.get(0).getTable();

	            // if GROUP BY, HAVING and ORDER BY clauses are not specified
	            if (query.getGroupByClause().isEmpty() && query.getHavingClause().isEmpty()
	                    && query.getOrderByClause().isEmpty()) {

	                final List<FilterItem> whereItems = query.getWhereClause().getItems();

	                // if all of the select items are "pure" column selection
	                boolean allSelectItemsAreColumns = true;
	                List<SelectItem> selectItems = query.getSelectClause().getItems();

	                // if it is a
	                // "SELECT [columns] FROM [table] WHERE [conditions]"
	                // query.
	                for (SelectItem selectItem : selectItems) {
	                    if (selectItem.getFunction() != null || selectItem.getColumn() == null) {
	                        allSelectItemsAreColumns = false;
	                        break;
	                    }
	                }

	                if (allSelectItemsAreColumns) {
	                    //logger.debug("Query can be expressed in full MongoDB, no post processing needed.");

	                    // prepare for a non-post-processed query
	                    Column[] columns = new Column[selectItems.size()];
	                    for (int i = 0; i < columns.length; i++) {
	                        columns[i] = selectItems.get(i).getColumn();
	                    }

	                    DataSet dataSet = materializeMainSchemaTableInternal(table, columns, whereItems, -1, false);
	                    if (query.getFirstRow() != null) {
	                        dataSet = new FirstRowDataSet(dataSet, query.getFirstRow());
	                    }
	                    if (query.getMaxRows() != null) {
	                        dataSet = new MaxRowsDataSet(dataSet, query.getMaxRows());
	                    }
	                    return dataSet;
	                }
	            }
	        }
	        //logger.debug("Query will be simplified for MongoDB and post processed.");
	        return super.executeQuery(query);
	    }
	    private DataSet materializeMainSchemaTableInternal(Table table, Column[] columns, List<FilterItem> whereItems,
	            int maxRows, boolean queryPostProcessed) {
	        final DBCollection collection = mongoDb.getCollection(table.getName());

	        final DBObject query = createMongoDbQuery(table, whereItems);

	        //logger.info("Executing MongoDB 'find' query: {}", query);
	        DBCursor cursor = collection.find(query);

	        if (maxRows > 0) {
	            cursor = cursor.limit(maxRows);
	        }

	        return new MongoDbDataSet(cursor, columns, queryPostProcessed);
	    }
	    final class MongoDbDataSet extends AbstractDataSet {

	    	//private static final Logger logger = LoggerFactory
	    	//		.getLogger(MongoDbDataSet.class);

	    	private final DBCursor _cursor;
	    	private final SelectItem[] _selectItems;
	    	private final boolean _queryPostProcessed;

	    	private boolean _closed;
	    	private volatile DBObject _dbObject;

	    	public MongoDbDataSet(DBCursor cursor, Column[] columns,boolean queryPostProcessed) {
	    		
	    		super(columns); // create header
	    		
	    		_cursor = cursor;
	    		_selectItems = new SelectItem[columns.length];
	    		for (int i = 0; i < columns.length; i++) {
	    			_selectItems[i] = new SelectItem(columns[i]);
	    		}
	    		_queryPostProcessed = queryPostProcessed;
	    		_closed = false;
	    	}

	    	public boolean isQueryPostProcessed() {
	    		return _queryPostProcessed;
	    	}

	    	@Override
	    	public SelectItem[] getSelectItems() {
	    		return _selectItems;
	    	}

	    	@Override
	    	public void close() {
	    		super.close();
	    		_cursor.close();
	    		_closed = true;
	    	}

	    	@Override
	    	protected void finalize() throws Throwable {
	    		super.finalize();
	    		if (!_closed) {
	    		//	logger.warn(
	    		//			"finalize() invoked, but DataSet is not closed. Invoking close() on {}",
	    		//			this);
	    			close();
	    		}
	    	}

	    	@Override
	    	public boolean next() {
	    		if (_cursor.hasNext()) {
	    			_dbObject = _cursor.next();
	    			return true;
	    		} else {
	    			_dbObject = null;
	    			return false;
	    		}
	    	}

	    	@Override
	    	public Row getRow() {
	    		if (_dbObject == null) {
	    			return null;
	    		}

	    		final Object[] values = new Object[_selectItems.length];
	    		for (int i = 0; i < values.length; i++) {
	    			String key = _selectItems[i].getColumn().getName();
	    			Object value = _dbObject.get(key);
	    			values[i] = toValue(_selectItems[i].getColumn(), value);
	    		}
	    		return new DefaultRow(getHeader(), values);
	    	}

	    	private Object toValue(Column column, Object value) {
	    		if (value instanceof List) {
	    			return value;
	    		}
	    		if (value instanceof DBObject) {
	    			DBObject basicDBObject = (DBObject) value;
	    			return basicDBObject.toMap();
	    		}
	    		return value;
	    	}

	    }

	    protected BasicDBObject createMongoDbQuery(Table table, List<FilterItem> whereItems) {
	        assert getMainSchema() == table.getSchema();

	        final BasicDBObject query = new BasicDBObject();
	        if (whereItems != null && !whereItems.isEmpty()) {
	            for (FilterItem item : whereItems) {
	                convertToCursorObject(query, item);
	            }
	        }

	        return query;
	    }
	    private void convertToCursorObject(BasicDBObject query, FilterItem item) {
	        if (item.isCompoundFilter()) {

	            BasicDBList orList = new BasicDBList();

	            final FilterItem[] childItems = item.getChildItems();
	            for (FilterItem childItem : childItems) {
	                BasicDBObject childObject = new BasicDBObject();
	                convertToCursorObject(childObject, childItem);
	                orList.add(childObject);
	            }

	            query.put("$or", orList);

	        } else {

	            final Column column = item.getSelectItem().getColumn();
	            final String columnName = column.getName();
	            final Object operand = item.getOperand();
	            final String operatorName = getOperatorName(item);

	            final BasicDBObject existingFilterObject = (BasicDBObject) query.get(columnName);
	            if (existingFilterObject == null) {
	                if (operatorName == null) {
	                    query.put(columnName, operand);
	                } else {
	                    query.put(columnName, new BasicDBObject(operatorName, operand));
	                }
	            } else {
	                if (operatorName == null) {
	                    throw new IllegalStateException("Cannot retrieve records for a column with two EQUALS_TO operators");
	                } else {
	                    existingFilterObject.append(operatorName, operand);
	                }
	            }
	        }
	    }
	    @SuppressWarnings("deprecation")
	    private String getOperatorName(FilterItem item) {
	        final String operatorName;
	        switch (item.getOperator()) {
	        case EQUALS_TO:
	            operatorName = null;
	            break;
	        case LESS_THAN:
	        case LOWER_THAN:
	            operatorName = "$lt";
	            break;
	        case GREATER_THAN:
	        case HIGHER_THAN:
	            operatorName = "$gt";
	            break;
	        case DIFFERENT_FROM:
	            operatorName = "$ne";
	            break;
	        case IN:
	            operatorName = "$in";
	            break;
	        default:
	            throw new IllegalStateException("Unsupported operator type: " + item.getOperator());
	        }
	        return operatorName;
	    }
	    
	    protected void addTable(MutableTable table) {
	        if (getMainSchema() instanceof MutableSchema) {
	            MutableSchema mutableSchema = (MutableSchema) getMainSchema();
	            mutableSchema.addTable(table);
	        } else {
	            throw new UnsupportedOperationException("Schema is not mutable");
	        }
	    }
	}

	public Schema detectSchema(Object nativeBackendConnection,String schemaName) {
		if ((nativeBackendConnection instanceof DB)==false) {
			throw new RuntimeException("Can only work with DB instance not with: "+nativeBackendConnection);
		}
		DB db = (DB)nativeBackendConnection;
		MutableSchema schema = new MutableSchema(schemaName);
    	for (MutableTable tableDef : detectSchemaTables(db)) {
    		tableDef.setSchema(schema);
    		schema.addTable(tableDef);
    	}
		return schema;
	}
	
	protected MutableTable[] detectSchemaTables(DB db) {
        Set<String> collectionNames = db.getCollectionNames();
        Map<String,PublicTable> tables = new HashMap<String,PublicTable>(collectionNames.size());
        for (String collectionName : collectionNames) {
        	PublicTable table = detectTable(db, collectionName);
        	tables.put(collectionName, table);
        }
        
        // Check for object id
        for (String collectionName:objectIdRef.keySet()) {
        	Map<String,ObjectId> refMap = objectIdRef.get(collectionName);
        	if (refMap==null) {
        		continue;
        	}
        	for (String columnName:refMap.keySet()) {
        		ObjectId objectId = refMap.get(columnName); 
        		
        		for (String colName : collectionNames) {
        			 Column primaryColumn = findColumnRef(db,colName,tables,objectId,columnName);
        			 if (primaryColumn==null) {
        				 continue;
        			 }
        			 PublicTable t = tables.get(collectionName);
        			 Column foreignColumn = t.getColumnByName(columnName);
        			 t.addRelationship(MutableRelationship.createRelationship(primaryColumn, foreignColumn));	 
                }
        	}
        }
        
        MutableTable[] result = new MutableTable[collectionNames.size()];
        return new ArrayList<MutableTable>(tables.values()).toArray(result);
    }

	Map<String,Map<String,ObjectId>> objectIdRef = new HashMap<String,Map<String,ObjectId>>(20);
	
	protected Column findColumnRef(DB db, String collectionName,Map<String,PublicTable> tables,ObjectId objectId,String colName) {
        DBCollection collection = db.getCollection(collectionName);
        DBObject query = new BasicDBObject();
        query.put("_id", objectId);
        DBCursor cursor = collection.find(query).limit(dataCheckSize);
        try {
	        if (cursor.hasNext()) {
	        	PublicTable t = tables.get(collectionName);
	            return t.getColumnByName("_id");
	        }
        } finally {
        	if (cursor!=null) {
        		cursor.close();
        	}
        }
        return null;
	}
	
	protected PublicTable detectTable(DB db, String collectionName) {
        final DBCollection collection = db.getCollection(collectionName);
        final DBCursor cursor = collection.find().limit(dataCheckSize);
        final SortedMap<String, Set<Class<?>>> columnsAndTypes = new TreeMap<String, Set<Class<?>>>();
        while (cursor.hasNext()) {
            DBObject row = cursor.next();
            Set<String> keysInObject = row.keySet();
            for (String key : keysInObject) {
                Set<Class<?>> types = columnsAndTypes.get(key);
                if (types == null) {
                    types = new HashSet<Class<?>>();
                    columnsAndTypes.put(key, types);
                }
                Object value = row.get(key);
                if (value != null) {
                    types.add(value.getClass());
                }
                if ("_id".equals(key)==false && value instanceof ObjectId) {
                	Map<String,ObjectId> map = objectIdRef.get(collectionName);
                	if (map==null) {
                		map = new HashMap<String,ObjectId>(10);
                		objectIdRef.put(collectionName,map);
                	}
                	map.put(key, (ObjectId)value);
                }
            }
        }
        cursor.close();

        final String[] columnNames = new String[columnsAndTypes.size()];
        final ColumnType[] columnTypes = new ColumnType[columnsAndTypes.size()];

        int i = 0;
        for (Entry<String, Set<Class<?>>> columnAndTypes : columnsAndTypes.entrySet()) {
            final String columnName = columnAndTypes.getKey();
            final Set<Class<?>> columnTypeSet = columnAndTypes.getValue();
            final Class<?> columnType;
            if (columnTypeSet.size() == 1) {
                columnType = columnTypeSet.iterator().next();
            } else {
                columnType = Object.class;
            }
            columnNames[i] = columnName;
            if (columnType == ObjectId.class) {
                columnTypes[i] = ColumnType.ROWID;
            } else {
                columnTypes[i] = ColumnType.convertColumnType(columnType);
            }
            i++;
        }
        
        PublicTable tableSchema = new PublicTable(collectionName,TableType.TABLE);
        for (int y = 0; y < columnNames.length; y++) {
        	MutableColumn col = new MutableColumn(columnNames[y], columnTypes[y], tableSchema, i, true);
			tableSchema.addColumn(col);
		}
        return tableSchema; // new SimpleTableDef(collectionName, columnNames, columnTypes);
    }

	// Override to make public
	class PublicTable extends MutableTable {
		private static final long serialVersionUID = 1L;
		public PublicTable(String name,TableType type) {
			super(name,type);
		}
		@Override
		public void addRelationship(Relationship relation) {
			super.addRelationship(relation);
		}
	}
	
	/**
	 * @return the dataCheckSize
	 */
	public int getDataCheckSize() {
		return dataCheckSize;
	}

	/**
	 * @param dataCheckSize the dataCheckSize to set
	 */
	public void setDataCheckSize(int dataCheckSize) {
		this.dataCheckSize = dataCheckSize;
	}

	/**
	 * @return the searchReference
	 */
	public boolean isSearchReference() {
		return searchReference;
	}

	/**
	 * @param searchReference the searchReference to set
	 */
	public void setSearchReference(boolean searchReference) {
		this.searchReference = searchReference;
	}
}
