package net.forwardfire.vasc.backend.metamodel.bundle;

import java.io.File;
import java.util.Properties;
import java.util.ResourceBundle;

import org.eobjects.metamodel.AbstractUpdateCallback;
import org.eobjects.metamodel.MetaModelException;
import org.eobjects.metamodel.UpdateCallback;
import org.eobjects.metamodel.create.ColumnCreationBuilder;
import org.eobjects.metamodel.create.TableCreationBuilder;
import org.eobjects.metamodel.data.DataSet;
import org.eobjects.metamodel.data.Row;
import org.eobjects.metamodel.delete.AbstractRowDeletionBuilder;
import org.eobjects.metamodel.delete.RowDeletionBuilder;
import org.eobjects.metamodel.drop.AbstractTableDropBuilder;
import org.eobjects.metamodel.drop.TableDropBuilder;
import org.eobjects.metamodel.insert.AbstractRowInsertionBuilder;
import org.eobjects.metamodel.insert.RowInsertionBuilder;
import org.eobjects.metamodel.schema.Column;
import org.eobjects.metamodel.schema.ColumnType;
import org.eobjects.metamodel.schema.Schema;
import org.eobjects.metamodel.schema.Table;

/**
 * ResourceBundleUpdateExecutor handles update to ResourceBundleDataContext
 * 
 * @author Willem Cazander
 * @version 1.0 May 26, 2012
 */
public class ResourceBundleUpdateExecutor extends AbstractUpdateCallback {

	protected ResourceBundleDataContext resourceBundleDataContext = null;
	protected ResourceBundleDataControl resourceBundleDataControl = null;
	
	public ResourceBundleUpdateExecutor(ResourceBundleDataContext resourceBundleDataContext,ResourceBundleDataControl resourceBundleDataControl) {
		super(resourceBundleDataContext);
		this.resourceBundleDataControl=resourceBundleDataControl;
	}
	
	public boolean isDataContextWritable() {
		return resourceBundleDataControl.isWritable();
	}
	
	@Override
	public boolean isCreateTableSupported() {
		return isDataContextWritable();
	}

	@Override
	public boolean isInsertSupported() {
		return isDataContextWritable();
	}
	
	@Override
	public boolean isDeleteSupported() {
		return isDataContextWritable();
	}

	@Override
	public boolean isDropTableSupported() {
		return isDataContextWritable();
	}
	
	@Override
	public TableCreationBuilder createTable(final Schema schema, final String name)	throws IllegalArgumentException, IllegalStateException {
		TableCreationBuilder result = new TableCreationBuilder() {
			@Override
			public Table toTable() {												return execute();	}
			@Override
			public String toSql() {													return "nosql";		}
			@Override
			public TableCreationBuilder like(Table table) {							return this;		}
			@Override
			public ColumnCreationBuilder withColumn(String name) {
				//return new ColumnCreationBuilderImpl(this,new MutableColumn(name));
				final TableCreationBuilder parent = this;
				return new ColumnCreationBuilder() {
					@Override
					public ColumnCreationBuilder withColumn(String name) {			return this;		}
					@Override
					public ColumnCreationBuilder ofType(ColumnType type) {			return this;		}
					@Override
					public ColumnCreationBuilder ofSize(int size) {					return this;		}
					@Override
					public ColumnCreationBuilder ofNativeType(String nativeType) {	return this;		}
					@Override
					public ColumnCreationBuilder nullable(boolean nullable) {		return this;		}
					@Override
					public ColumnCreationBuilder asPrimaryKey() {					return this;		}
					@Override
					public ColumnCreationBuilder like(Column column) {				return this;		}
					@Override
					public TableCreationBuilder  like(Table table) {					return this;		}
					@Override
					public String toSql() {											return "nosql";		}
					@Override
					public Table toTable() {										return execute();	}
					@Override
					public Table execute() throws MetaModelException {				return parent.execute();}
				};
			}
			
			@Override
			public Table execute() throws MetaModelException {
				File file = new File(name);
				Properties p = new Properties();
				try {
					resourceBundleDataControl.saveBundleFile(p, file);
					ResourceBundle bundle = resourceBundleDataControl.loadBundleFile(file);
					resourceBundleDataControl.putBundleData(bundle, name, file);
				} catch (Exception e) {
					throw new MetaModelException(e);
				}
				return resourceBundleDataContext.createTable(schema, name);
			}
		};
		return result;
	}
	
	@Override
	public TableDropBuilder dropTable(Table table) throws IllegalArgumentException, IllegalStateException,UnsupportedOperationException {
		TableDropBuilder result = new AbstractTableDropBuilder(table) {
			@Override
			public void execute() throws MetaModelException {
				resourceBundleDataControl.removeBundle(getTable().getName()); 
			}
		};
		return result;
	}
	
	@Override
	public RowInsertionBuilder insertInto(final Table table) throws IllegalArgumentException, IllegalStateException,UnsupportedOperationException {
		RowInsertionBuilder result = new AbstractRowInsertionBuilder<UpdateCallback>(this, table) {
			@Override
			public void execute() throws MetaModelException {
				Properties data = resourceBundleDataControl.getBundleData(table.getName());
				Object[] values = getValues();
				data.put(values[1], values[2]);
			}
		};
		return result;
	}
	
	@Override
	public RowDeletionBuilder deleteFrom(final Table table) throws IllegalArgumentException, IllegalStateException,UnsupportedOperationException {
		RowDeletionBuilder result = new AbstractRowDeletionBuilder(table) {
			@Override
			public void execute() throws MetaModelException {
				DataSet ds = resourceBundleDataContext.materializeMainSchemaTable(getTable(),getTable().getColumns(),0);
				Properties data = resourceBundleDataControl.getBundleData(table.getName());
				while (ds.next()) {
					Row row = ds.getRow();
					if (deleteRow(row)) {
						data.remove(row.getValue(1)); // delete by key
					}
				}
			}
		};
		return result;
	}
}
