package net.forwardfire.vasc.backend.metamodel.crud;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.eobjects.metamodel.DataContext;
import org.eobjects.metamodel.MetaModelException;
import org.eobjects.metamodel.UpdateCallback;
import org.eobjects.metamodel.UpdateScript;
import org.eobjects.metamodel.UpdateableDataContext;
import org.eobjects.metamodel.data.DataSet;
import org.eobjects.metamodel.data.EmptyDataSet;
import org.eobjects.metamodel.data.InMemoryDataSet;
import org.eobjects.metamodel.data.Row;
import org.eobjects.metamodel.delete.RowDeletionBuilder;
import org.eobjects.metamodel.insert.RowInsertionBuilder;
import org.eobjects.metamodel.query.FromItem;
import org.eobjects.metamodel.query.Query;
import org.eobjects.metamodel.query.SelectItem;
import org.eobjects.metamodel.query.builder.SatisfiedQueryBuilder;
import org.eobjects.metamodel.query.builder.SatisfiedWhereBuilder;
import org.eobjects.metamodel.schema.Table;
import org.eobjects.metamodel.update.RowUpdationBuilder;

/**
 * AbstractCrudDataContext implements the generic abstract crud actions.
 * 
 * @author Willem Cazander
 * @version 1.0 May 23, 2012
 */
abstract public class AbstractCrudDataContext  implements CrudDataContext {

	abstract public UpdateableRow wrapOrCreateToUpdateableRow(Row row,Table table);
	
	protected UpdateableDataContext abstractProviderUpdateableDataContext() {
		return this;
	}
	
	@Override
	public UpdateableRow createRow(Table table) {
		return wrapOrCreateToUpdateableRow(null,table);
	}
	
	@Override
	public void persist(final UpdateableRow row) {
		UpdateableDataContext dataContext = abstractProviderUpdateableDataContext();
		dataContext.executeUpdate(new UpdateScript() {
			@Override
			public void run(UpdateCallback backendImpl) {
				RowInsertionBuilder query = backendImpl.insertInto(row.getTable());
				for (int i=0;i<row.size();i++) {
					SelectItem si = row.getSelectItem(i);
					if (si.getColumn().isPrimaryKey()) {
						continue; // insert null will only work with correct default sequence attacted to column.
					}
					Object value = row.getValue(i);
					query.value(si.getColumn(), value);
				}
				query.execute();
			}
		});
	}
	
	@Override
	public UpdateableRow merge(final UpdateableRow row) {
		for (String column:row.getPrimaryKeys()) {
			if (row.getValue(column)==null) {
				throw new IllegalStateException("Can't update row where primary key: "+column+" value is null.");	
			}
		}
		UpdateableDataContext dataContext = abstractProviderUpdateableDataContext();
		dataContext.executeUpdate(new UpdateScript() {
			@Override
			public void run(UpdateCallback backendImpl) {
				Object qWhere = backendImpl.update(row.getTable());
				boolean first = true;
				List<String> primaryKeyColumns = new ArrayList<String>(30);
				Collections.addAll(primaryKeyColumns, row.getPrimaryKeys());
				for (String column:primaryKeyColumns) {
					Object value = row.getValue(column);
					queryWhereBuilderUpdate(qWhere,column,value,first);
					first = false;
				}
				List<String> columns = new ArrayList<String>(30);
				Collections.addAll(columns, row.getTable().getColumnNames());
				columns.removeAll(primaryKeyColumns);
				for (String column:columns) {
					Object value = row.getValue(column);
					((RowUpdationBuilder)qWhere).value(column, value);
				}
				if (first==false) {
					((RowUpdationBuilder)qWhere).execute();
				} else {
					// loggger.warning no where clause
				}
			}
		});
		
		Object qWhere = dataContext.query().from(row.getTable()).select(row.getTable().getColumns());
		boolean first = true;
		for (String column:row.getPrimaryKeys()) {
			Object value = row.getValue(column);
			queryWhereBuilder(qWhere,column,value,first);
			first = false;
		}
		Query mergeSelectQuery = ((SatisfiedQueryBuilder<?>)qWhere).toQuery();
		DataSet ds = dataContext.executeQuery(mergeSelectQuery);
		if (ds.next()==false) {
			ds.close();
			throw new IllegalStateException("Could not fetch row after merging.");
		}
		UpdateableRow rowResult = wrapOrCreateToUpdateableRow(ds.getRow(),row.getTable());
		ds.close();
		return rowResult;
	}
	
	@Override
	public void delete(final UpdateableRow row) {
		UpdateableDataContext dataContext = abstractProviderUpdateableDataContext();
		dataContext.executeUpdate(new UpdateScript() {
			@Override
			public void run(UpdateCallback backendImpl) {
				Object qWhere = backendImpl.deleteFrom(row.getTable());
				boolean first = true;
				for (String column:row.getPrimaryKeys()) {
					Object value = row.getValue(column);
					queryWhereBuilderDelete(qWhere,column,value,first);
					first = false;
				}
				if (first==false) {
					((RowDeletionBuilder)qWhere).execute();
				} else {
					///logger.warning no where clause
				}
			}
		});
	}

	// TODO: redo these queryWhereBuilder methods to cleaner code
	
	protected Object queryWhereBuilder(Object qWhere,String key,Object value,boolean first) {
		if (value==null) {
			if (first) {
				qWhere = ((SatisfiedQueryBuilder<?>)qWhere).where(key).isNull();
			} else {
				qWhere = ((SatisfiedWhereBuilder<?>)qWhere).and(key).isNull();
			}
		} else {
			if (first) {
				qWhere = ((SatisfiedQueryBuilder<?>)qWhere).where(key).isEquals(value);
			} else {
				qWhere = ((SatisfiedWhereBuilder<?>)qWhere).and(key).isEquals(value);
			}
		}
		return qWhere;
	}
	
	protected Object queryWhereBuilderUpdate(Object qWhere,String key,Object value,boolean first) {
		if (value==null) {
			if (first) {
				qWhere = ((RowUpdationBuilder)qWhere).where(key).isNull();
			} else {
				qWhere = ((RowUpdationBuilder)qWhere).where(key).isNull();
			}
		} else {
			if (first) {
				qWhere = ((RowUpdationBuilder)qWhere).where(key).isEquals(value);
			} else {
				qWhere = ((RowUpdationBuilder)qWhere).where(key).isEquals(value);
			}
		}
		return qWhere;
	}
	
	protected Object queryWhereBuilderDelete(Object qWhere,String key,Object value,boolean first) {
		if (value==null) {
			if (first) {
				qWhere = ((RowDeletionBuilder)qWhere).where(key).isNull();
			} else {
				qWhere = ((RowDeletionBuilder)qWhere).where(key).isNull();
			}
		} else {
			if (first) {
				qWhere = ((RowDeletionBuilder)qWhere).where(key).isEquals(value);
			} else {
				qWhere = ((RowDeletionBuilder)qWhere).where(key).isEquals(value);
			}
		}
		return qWhere;
	}
	
	/**
	 * executeQueryWrapDataSet interface so when implemented on backend then executeQueryWrapDataSet is not used.
	 * There for no need to expose this primary key handler method in this abstract class.
	 */
	protected interface QueryWrapDataSetPrimaryKeyHandler {
		public List<String> getPrimaryKeysForTable(Table table);
	}
	
	protected DataSet executeQueryWrapDataSet(DataContext dataContext,Query query,boolean checkPrimaryKeysAllRows,QueryWrapDataSetPrimaryKeyHandler keyHandler) throws MetaModelException {
		DataSet data = dataContext.executeQuery(query);
		if (query.getFromClause().getItems().size() > 1) {
			return data; // Only supports crud on single table data.
		}
		FromItem fromItem = query.getFromClause().getItems().get(0); 
		Table table = fromItem.getTable();
		List<String> keys = keyHandler.getPrimaryKeysForTable(table);
		if (keys.isEmpty()) {
			return data;
		}
		if (query.getSelectClause().getItemCount()==1 && query.getSelectClause().getItem(0).getColumn()==null) {
			return data; // TODO: use correct api to detect selectCount query
		}
		List<Row> dataWrap = new ArrayList<Row>(2000);
		boolean keysOke = false;  // NOTE: could add option checkPrimaryKeys to override if user/impl is really sure keys are there.
		boolean keysCheck = true; // always check first row
		List<String> keysCheckList = new ArrayList<String>(keys); // copy keys
		while (data.next()) {
			Row row = data.getRow();
			if (keysCheck) {
				for (SelectItem si:row.getSelectItems()) {
					keysCheckList.remove(si.getColumn().getName());
				}
				keysCheck = checkPrimaryKeysAllRows; // default is check first row only.
				keysOke = keysCheckList.isEmpty();   // Found all primary keys in result so we can wrap data.
				if (keysCheck) {
					keysCheckList.addAll(keys);      // Reload keys to check all data.
				}
			}
			if (keysOke) {
				dataWrap.add(wrapOrCreateToUpdateableRow(data.getRow(),table));
			} else {
				dataWrap.add(row);
			}
		}
		data.close();
		if (dataWrap.isEmpty()) {
			return new EmptyDataSet(table.getColumns());
		}
		return new InMemoryDataSet(dataWrap);
	}
}
