package net.forwardfire.vasc.backend.metamodel.crud;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eobjects.metamodel.data.Row;
import org.eobjects.metamodel.data.Style;
import org.eobjects.metamodel.query.SelectItem;
import org.eobjects.metamodel.schema.Column;

/**
 * AbstractRow implements most Row features onto of itself.
 * 
 * @author Willem Cazander
 * @version 1.0 May 23, 2012
 */
@SuppressWarnings("serial")
abstract public class AbstractRow implements Row,RowLocal {
	
	protected Map<Integer,MetaRowData> metaData = null;
    
    public AbstractRow() {
    	metaData = new HashMap<Integer,MetaRowData>(30);
    }
    
    public SelectItem getSelectItem(int index) {
    	MetaRowData metaRow = getMetaRowData(index);
    	return metaRow.selectItem;
    }
    
    public void addSelectItems(Column[] columns) {
    	for (Column column:columns) {
    		addSelectItem(new SelectItem(column));
    	}
    }
    
    public void addSelectItems(SelectItem[] selectItems) {
    	for (SelectItem selectItem:selectItems) {
    		addSelectItem(selectItem);
    	}
    }
    
    public void addSelectItems(List<SelectItem> selectItems) {
    	for (SelectItem selectItem:selectItems) {
    		addSelectItem(selectItem);
    	}
    }
    
    public void addSelectItem(SelectItem selectItem) {
    	MetaRowData metaRow = new MetaRowData();
    	metaRow.selectItem=selectItem;
    	metaData.put(metaData.size(), metaRow);
    }
    
    public MetaRowData getMetaRowData(int index) {
    	MetaRowData metaRow = metaData.get(index);
    	return metaRow;
    }
	
    public List<MetaRowData> getIndexedMetaRowDataList() {
		List<MetaRowData> result = new ArrayList<MetaRowData>(size());
		for (int i=0;i<metaData.size();i++) {
			MetaRowData metaRow = metaData.get(i);
			result.add(metaRow);
		}
		return result;
    }
    
    public List<SelectItem> getIndexedSelectItemList() {
		List<SelectItem> result = new ArrayList<SelectItem>(size());
		for (MetaRowData metaRow:getIndexedMetaRowDataList()) { 
			result.add(metaRow.selectItem);
		}
		return result;
    }
    
    @Override
    public void setStyle(int index,Style style) {
    	MetaRowData metaRow = metaData.get(index);
    	metaRow.style=style;
    }
    
    class MetaRowData implements Serializable /* ,DataSetRowMetaData */ {
    	SelectItem selectItem;
    	Style style;
    	// name and backendIndex,etc so DataSetApi does not need SchemaApi, when using schema aware code like an DataContext 
    	// the DC api does not change because it fill/config/wires the DS api with meta data from schema.
    	// When using DataSet api in non-schema aware code then caller has to provided meta data for api to work.
    	// ObjectConverter objectConverter
    	// List<ObjectValidator>
    	// etc/etc
    }
    
    // ==== Start interface
    
    
    @Override
	public Object[] getValues() {
		Object[] result = new Object[size()];
		for (int i=0;i<size();i++) {
			result[i] = getValue(i);
		}
		return result;
	}
    
    @Override
    public Object getValue(SelectItem item) {
    	return getValue(indexOf(item));
    }
    
    @Override
	public Object getValue(Column column) {
		return getValue(indexOf(column));
	}	
	
    @Override
	public int indexOf(SelectItem item) {
        if (item == null) {
            return UpdateableRow.INDEX_NOT_FOUND;
        }
        int i = 0;
        for (SelectItem selectItem : getIndexedSelectItemList()) {
            if (item.equalsIgnoreAlias(selectItem)) {
                return i;
            }
            i++;
        }
        return UpdateableRow.INDEX_NOT_FOUND;
    }
	
    @Override
    public int indexOf(Column column) {
        if (column == null) {
            return UpdateableRow.INDEX_NOT_FOUND;
        }
        SelectItem selectItem = new SelectItem(column);
        return indexOf(selectItem);
    }
	
    @Override
	public SelectItem[] getSelectItems() {
		List<SelectItem> list = getIndexedSelectItemList();
		SelectItem[] result = new SelectItem[list.size()];
		return list.toArray(result);
	}

    @Override
    public Style getStyle(SelectItem item) {
    	return getStyle(indexOf(item));
    }

    @Override
    public Style getStyle(Column column) {
    	return getStyle(indexOf(column));
    }

    @Override
    public Style getStyle(int index) throws IndexOutOfBoundsException {
    	MetaRowData metaRow = getMetaRowData(index);
    	if (metaRow==null) {
    		return Style.NO_STYLE;
    	}
    	if (metaRow.style==null) {
            return Style.NO_STYLE;
        }
        return metaRow.style;
    }
    
    @Override
    public int size() {
    	return metaData.size();
    }
}
