package net.forwardfire.vasc.backend.metamodel.crud;

import java.util.List;

import org.eobjects.metamodel.data.Row;
import org.eobjects.metamodel.query.SelectItem;
import org.eobjects.metamodel.schema.Column;
import org.eobjects.metamodel.schema.Table;

/**
 * UpdateableRow is the setValue(...) version of Row interface.
 * 
 * @author Willem Cazander
 * @version 1.0 May 23, 2012
 */
public interface UpdateableRow extends Row {

	// TODO: move these 3 to Row interface
	SelectItem getSelectItem(int index);
	Object getValue(String columnName);
	static final int INDEX_NOT_FOUND = -1;
	
	/**
	 * Returns the table 
	 */
	Table getTable();
	
	/**
	 * Returns primary keys of table.
	 */
	List<String> getPrimaryKeysList();
	
	/**
	 * Returns primary keys of table.
	 */
	String[] getPrimaryKeys();
	
	/**
	 * Sets the value by the column name.
	 * 
	 * @param columnName
	 * @param object
	 */
	void setValue(String columnName,Object object);
	
	/**
	 * Sets the value of the provided SelectItem.
	 * 
	 * @param item
	 * @return the value that corresponds to the provided SelectItem. Can be
	 *         null if either the value <i>is</i> null or if no value exists
	 *         that matches the SelectItem.
	 */
	void setValue(SelectItem item,Object object);
	
	/**
	 * Shorthand method for setting the value of a SelectItem based on the
	 * provided column. Invoking this method is equivalent to invoking
	 * setValue(new SelectItem(column)).
	 * 
	 * @param column
	 * @return the value of the specified column
	 */
	void setValue(Column column,Object object);
	
	/**
	 * Sets the value of the row at a given index
	 * 
	 * @param index
	 * @return the value at the specified index
	 * @throws IndexOutOfBoundsException
	 *             if the provided index is out of range
	 */
	void setValue(int index,Object object) throws IndexOutOfBoundsException;
}
