package net.forwardfire.vasc.backend.metamodel.crud;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eobjects.metamodel.data.DataSetHeader;
import org.eobjects.metamodel.data.Row;
import org.eobjects.metamodel.query.SelectItem;
import org.eobjects.metamodel.schema.Table;

/**
 * UpdateableRowMapImpl is basic implementation of UpdateableRow interface.
 * 
 * @author Willem Cazander
 * @version 1.0 May 23, 2012
 */
public class UpdateableRowMapImpl extends AbstractUpdateableRow {

	private static final long serialVersionUID = 7660759661023194759L;
	private Map<Integer,Object> dataMap = null;
	private Table table = null;
	private List<String> primaryKeys = null;
	
	public UpdateableRowMapImpl(Table table,List<String> primaryKeys) {
		this(table,primaryKeys,null);
	}
	
	public UpdateableRowMapImpl(Table table,List<String> primaryKeys,SelectItem[] selectItems) {
		if (table==null) {			throw new NullPointerException("Can't work with null table.");			}
		if (primaryKeys==null) {	throw new NullPointerException("Can't work with null primaryKeys.");	}
		this.dataMap = new HashMap<Integer,Object>(30);
		this.table=table;
		this.primaryKeys = primaryKeys;
		if (selectItems!=null) {
			addSelectItems(selectItems);
		}
	}
		
	// interface
	
	@Override
	public Table getTable() {
		return table;
	}
	
	@Override
	public List<String> getPrimaryKeysList() {
		return primaryKeys;
	}
	
	@Override
	public void setValue(int index, Object value) throws IndexOutOfBoundsException {
		if (index == UpdateableRow.INDEX_NOT_FOUND) {
			throw new IndexOutOfBoundsException("Can't save value without key index: "+UpdateableRow.INDEX_NOT_FOUND+" value:"+value);
		}
		dataMap.put(index, value);
	}
	
	@Override
	public Object getValue(int index) throws IndexOutOfBoundsException {
		return dataMap.get(index);
	}
	
	@Override
	public Row getSubSelection(SelectItem[] selectItems) {
		return getSubSelectionFromImpl(new UpdateableRowMapImpl(table,primaryKeys,selectItems),selectItems);
	}
	
	@Override
	public Row getSubSelection(DataSetHeader header) {
		return getSubSelection(header.getSelectItems());
	}
}
