package net.forwardfire.vasc.backend.metamodel.jndi.loader;

import java.util.Map;

import org.eobjects.metamodel.DataContext;
import org.eobjects.metamodel.csv.CsvConfiguration;
import org.eobjects.metamodel.csv.CsvDataContext;

/**
 * JndiCsvDataContextLoader Loads and confis csv DataContext from jndi loader config.
 * 
 * @author Willem Cazander
 * @version 1.0 May 22, 2012
 */
public class JndiCsvDataContextLoader implements JndiDataContextLoader {

	private boolean useUrlSource = false;
	
	public JndiCsvDataContextLoader() {
	}
	public JndiCsvDataContextLoader(boolean useUrlSource) {
		setUseUrlSource(useUrlSource);
	}
	
	@Override
	public DataContext loadDataContext(JndiDataContextLoaderConfig config) {
		if (useUrlSource) {
			return new CsvDataContext(config.checkBackendUrl(),createCsvConfig(config.getBackendParameters()));	
		} else {
			return new CsvDataContext(config.checkFile(),createCsvConfig(config.getBackendParameters()));	
		}
	}

	protected CsvConfiguration createCsvConfig(Map<String,String> param) {
		int columnNameLineNumber = 0;
		String encoding = "UTF-8";
		char separatorChar = CsvConfiguration.DEFAULT_SEPARATOR_CHAR;
		char quoteChar = CsvConfiguration.DEFAULT_QUOTE_CHAR;
		char escapeChar = CsvConfiguration.DEFAULT_ESCAPE_CHAR;
		boolean failOnInconsistentRowLength = false;
		if (param.containsKey("fileEnconding")) {
			encoding = param.get("fileEnconding");
		}
		if (param.containsKey("separatorChar")) {
			separatorChar = param.get("separatorChar").charAt(0);
		}
		if (param.containsKey("quoteChar")) {
			quoteChar = param.get("quoteChar").charAt(0);
		}
		if (param.containsKey("escapeChar")) {
			escapeChar = param.get("escapeChar").charAt(0);
		}
		if (param.containsKey("failOnInconsistentRowLength")) {
			failOnInconsistentRowLength = true;
		}
		return new CsvConfiguration(columnNameLineNumber,encoding,separatorChar,quoteChar,escapeChar,failOnInconsistentRowLength);
	}
	
	/**
	 * @return the useUrlSource
	 */
	public boolean isUseUrlSource() {
		return useUrlSource;
	}
	
	/**
	 * @param useUrlSource the useUrlSource to set
	 */
	public void setUseUrlSource(boolean useUrlSource) {
		this.useUrlSource = useUrlSource;
	}
}
