package net.forwardfire.vasc.backend.metamodel.jndi.loader;

import java.sql.Connection;
import java.sql.DriverManager;

import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;

import org.eobjects.metamodel.DataContext;
import org.eobjects.metamodel.jdbc.JdbcDataContext;

/**
 * JndiCsvDataContextLoader Loads and config jdbc DataContext from jndi loader config.
 * 
 * @author Willem Cazander
 * @version 1.0 May 22, 2012
 */
public class JndiJdbcDataContextLoader implements JndiDataContextLoader {

	private boolean useDataSource = true;
	
	public JndiJdbcDataContextLoader() {
	}
	public JndiJdbcDataContextLoader(boolean useDataSource) {
		setUseDataSource(useDataSource);
	}
	
	@Override
	public DataContext loadDataContext(JndiDataContextLoaderConfig config) {
		if (useDataSource) {
			if (config.getBackendUrl()==null) {
				throw new IllegalArgumentException("Can't work with null "+JndiDataContextLoaderConfig.BACKEND_URL+" attribute for: "+config.getJndiObjectName());
			}
			if (config.getBackendUrl().isEmpty()) {
				throw new IllegalArgumentException("Can't work with empty "+JndiDataContextLoaderConfig.BACKEND_URL+" attribute for: "+config.getJndiObjectName());
			}
			
			return new JdbcDataContext(getDataSource(config.getJndiObjectName(),config.getBackendUrl()));	
		} else {
			config.checkBackendUrl();
			return new JdbcDataContext(getConnection(
							config.getJndiObjectName(),
							config.getBackendClass(),
							config.getBackendUrl(),
							config.getBackendUsername(),
							config.getBackendPassword()
							));	
		}
	}

	protected Connection getConnection(String objectName,String driverClass,String url,String user,String pass) {
		try {
			if (driverClass!=null && driverClass.isEmpty()==false) {
				ClassLoader cl = Thread.currentThread().getContextClassLoader();
				if (cl==null) {
					cl = this.getClass().getClassLoader();
				}
				cl.loadClass(driverClass);
			}
			Connection connection = DriverManager.getConnection(url,user,pass);
			return connection;
		} catch (Exception e) {
			throw new RuntimeException("Could not get jdbc connection to: "+url+" for: "+objectName,e);
		}
	}
	
	protected DataSource getDataSource(String objectName,String jndiName) {
		try {			
			InitialContext context = new InitialContext();
			DataSource datasource = (DataSource)context.lookup(jndiName);
			return datasource;
		} catch (NamingException e) {
			throw new IllegalStateException("Jndi naming error on: "+jndiName+" for: "+objectName,e);
		}
	}

	/**
	 * @return the useDataSource
	 */
	public boolean isUseDataSource() {
		return useDataSource;
	}

	/**
	 * @param useDataSource the useDataSource to set
	 */
	public void setUseDataSource(boolean useDataSource) {
		this.useDataSource = useDataSource;
	}
}
