package net.forwardfire.vasc.backend.metamodel.jndi.loader;

import java.net.UnknownHostException;

import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.eobjects.metamodel.DataContext;
import org.eobjects.metamodel.mongodb.MongoDbDataContext;

import com.mongodb.DB;
import com.mongodb.Mongo;
import com.mongodb.MongoOptions;
import com.mongodb.ServerAddress;

/**
 * JndiCsvDataContextLoader Loads and config mongodb DataContext from jndi loader config.
 * 
 * @author Willem Cazander
 * @version 1.0 May 22, 2012
 */
public class JndiMongodbDataContextLoader implements JndiDataContextLoader {

	private boolean useJndiResource = false;

	public JndiMongodbDataContextLoader() {
	}
	public JndiMongodbDataContextLoader(boolean useJndiResource) {
		setUseJndiResource(useJndiResource);
	}
	
	@Override
	public DataContext loadDataContext(JndiDataContextLoaderConfig config) {
		//MongoDbDataContextBean result = null;
		DB mongoDb = null;
		if (useJndiResource) {
			config.checkBackendUrl();
			//result = new MongoDbDataContextBean(,));
			mongoDb = getMongodbJdni(config.getJndiObjectName(),config.getBackendUrl());
			
		} else {
			int mongoPort = 27017;
			if (config.getBackendPort()!=null && config.getBackendPort().isEmpty()==false) {
				mongoPort = new Integer(config.getBackendPort());
			}
			mongoDb = getMongodbConnection(
					config.getJndiObjectName(),
					config.getBackendHost(),
					mongoPort,
					config.getBackendDatabase(),
					config.getBackendUsername(),
					config.getBackendPassword()
					);
		}
		//MongoDbDataContextSchemaDetector detector = new MongoDbDataContextSchemaDetector();
		//detector.setDataCheckSize(10);
		//detector.setSearchReference(true);
		//DataContext dataContext = detector.new MongoDbDataContextExtended(mongoDb,detector);
		
		DataContext dataContext = new MongoDbDataContext(mongoDb);
		return dataContext;
		
		//result.setRegisterMBean(true); // TODO: make flag
		//result.start();
		//return result;
	}

	protected DB getMongodbJdni(String objectName,String jndiName) {
		try {			
			InitialContext context = new InitialContext();
			DB db = (DB)context.lookup(jndiName);
			return db;
		} catch (NamingException e) {
			throw new IllegalStateException("Jndi naming error on: "+jndiName+" for: "+objectName,e);
		}
	}
	
	protected DB getMongodbConnection(String objectName,String hostname,int port,String database,String username,String password) {
		if (database==null) {
			throw new IllegalArgumentException("Can't work with null "+JndiDataContextLoaderConfig.BACKEND_DB+" attribute for: "+objectName);
		}
		if (database.isEmpty()) {
			throw new IllegalArgumentException("Can't work with empty "+JndiDataContextLoaderConfig.BACKEND_DB+" attribute for: "+objectName);
		}
		ServerAddress server;
		try {
			server = new ServerAddress(hostname,port);
		} catch (UnknownHostException e) {
			throw new IllegalStateException("Could not create hostname from: "+hostname+" for: "+objectName,e);
		}
		MongoOptions options = new MongoOptions();
		Mongo mongo = new Mongo(server,options);
		DB db = mongo.getDB(database);
		if (username!=null && password!=null) {
			boolean auth = db.authenticate(username, password.toCharArray());
			if (auth==false) {
				throw new IllegalStateException("Could not auth to db: "+database+" with username: "+username+" for: "+objectName);
			}
		}
		return db;
	}

	/**
	 * @return the useJndiResource
	 */
	public boolean isUseJndiResource() {
		return useJndiResource;
	}


	/**
	 * @param useJndiResource the useJndiResource to set
	 */
	public void setUseJndiResource(boolean useJndiResource) {
		this.useJndiResource = useJndiResource;
	}
}
