package net.forwardfire.vasc.backend.metamodel.jndi.loader;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.eobjects.metamodel.DataContext;
import org.eobjects.metamodel.util.Ref;
import org.eobjects.metamodel.xml.XmlSaxDataContext;
import org.eobjects.metamodel.xml.XmlSaxTableDef;
import org.xml.sax.InputSource;

/**
 * JndiXmlSaxDataContextLoader Loads and config xml sax DataContext from jndi loader config.
 * 
 * @author Willem Cazander
 * @version 1.0 May 22, 2012
 */
public class JndiXmlSaxDataContextLoader implements JndiDataContextLoader {

	private boolean useUrlSource = false;
	
	public JndiXmlSaxDataContextLoader() {
	}
	public JndiXmlSaxDataContextLoader(boolean useUrlSource) {
		setUseUrlSource(useUrlSource);
	}
	
	@Override
	public DataContext loadDataContext(JndiDataContextLoaderConfig config) {
		List<XmlSaxTableDef> tableSchemas = createXmlSaxConfig(config.getJndiObjectName(),config.getBackendParameters());
		XmlSaxTableDef[] args = new XmlSaxTableDef[tableSchemas.size()];
		if (useUrlSource) {
			final URL url = config.checkBackendUrl();
			return new XmlSaxDataContext(new Ref<InputSource>() {  // TODO: missing constructor in XmlSaxDataContext
				@Override
				public InputSource get() {
					InputStream in = null;
					try {
						in = url.openStream();
						return new InputSource(in);
					} catch (IOException e) {
						throw new IllegalStateException("Could not read url; "+url,e);
					} finally {
						if (in!=null) {
							try {
								in.close();
							} catch (IOException e) {
							}
						}
					}
				}
			},tableSchemas.toArray(args));
		} else {
			return new XmlSaxDataContext(config.checkFile(),tableSchemas.toArray(args)); // copy List to array	
		}
	}
	
	protected List<XmlSaxTableDef> createXmlSaxConfig(String objectName,Map<String,String> param) {
		String rowXPath = param.get("rowXPath");
		String dataXPaths = param.get("dataXPaths");
		if (rowXPath==null) {
			throw new IllegalArgumentException("Can't work with null rowXPath attribute for: "+objectName);
		}
		if (rowXPath.isEmpty()) {
			throw new IllegalArgumentException("Can't work with empty rowXPath attribute for: "+objectName);
		}
		if (dataXPaths==null) {
			throw new IllegalArgumentException("Can't work with null dataXPaths attribute for: "+objectName);
		}
		if (dataXPaths.isEmpty()) {
			throw new IllegalArgumentException("Can't work with empty dataXPaths attribute for: "+objectName);
		}
		String[] args = dataXPaths.split(",");
		
		List<XmlSaxTableDef> tableSchemas = new ArrayList<XmlSaxTableDef>(5);
		XmlSaxTableDef tableSchema = new XmlSaxTableDef(rowXPath,args);
		tableSchemas.add(tableSchema);
		return tableSchemas;
	}

	/**
	 * @return the useUrlSource
	 */
	public boolean isUseUrlSource() {
		return useUrlSource;
	}
	
	/**
	 * @param useUrlSource the useUrlSource to set
	 */
	public void setUseUrlSource(boolean useUrlSource) {
		this.useUrlSource = useUrlSource;
	}
}
