package net.forwardfire.vasc.backend.metamodel.crud;

import java.io.File;

import org.eobjects.metamodel.DataContext;
import org.eobjects.metamodel.UpdateCallback;
import org.eobjects.metamodel.UpdateScript;
import org.eobjects.metamodel.UpdateableDataContext;
import org.eobjects.metamodel.csv.CsvDataContext;
import org.eobjects.metamodel.data.DataSet;
import org.eobjects.metamodel.data.Row;
import org.eobjects.metamodel.schema.Schema;
import org.eobjects.metamodel.schema.Table;
import org.eobjects.metamodel.util.MutableRef;

import junit.framework.TestCase;

public class CrudContextTest extends TestCase {

	public void testCrud() {
		final String tableName = "test-crud";  // Setup some small test data in tmp
		File testFile = new File(System.getProperty("java.io.tmpdir")+File.separatorChar+tableName+".csv");
		final boolean createTable = testFile.exists()==false;
		CsvDataContext dataContext = new CsvDataContext(testFile);
		assertTrue(dataContext instanceof UpdateableDataContext);
		final Schema schema = dataContext.getDefaultSchema();
		final MutableRef<Table> tableRef = new MutableRef<Table>();
		dataContext.executeUpdate(new UpdateScript() {
			@Override
            public void run(UpdateCallback cb) {
            	Table table = null;
            	int idx = 0;
            	if (createTable) { 
            		table = cb.createTable(schema, tableName).withColumn("id").withColumn("name").withColumn("active").execute();
            		tableRef.set(table);
            	} else {
            		table = cb.getDataContext().getDefaultSchema().getTableByName(tableName);
            		idx = generateNextId(cb.getDataContext(),table);
            	}
                cb.insertInto(table).value(0,idx+1).value(1, "foo").value(2, true).execute();
                cb.insertInto(table).value(0,idx+2).value(1, "foo").value(2, false).execute();
                cb.insertInto(table).value(0,idx+3).value(1, "bar").value(2, false).execute();
                cb.insertInto(table).value(0,idx+4).value(1, "bar").value(2, true).execute();
                cb.insertInto(table).value(0,idx+5).value(1, "object").value(2, true).execute();
            }
        });
		
		// Wrap to CrudDataContext and get table to test with.
		CrudDataContext crudDataContext = new CrudDataContextImpl((UpdateableDataContext)dataContext);
		Table table = crudDataContext.getDefaultSchema().getTableByName(tableName);
		assertNotNull(table);
		
		// Fetch and update data
		DataSet ds = crudDataContext.query().from(table).select(table.getColumns()).execute();
		while (ds.next()) {
			Row row = ds.getRow();
			if (row instanceof UpdateableRow) {
				UpdateableRow rowCrud = (UpdateableRow)row;
				if (rowCrud.getValue("name")!=null && rowCrud.getValue("name").toString().startsWith("test")==false) {
					rowCrud.setValue("active", false);
					crudDataContext.merge(rowCrud);
				}
			}
		}
		ds.close();
		
		// Lets add an record.
		UpdateableRow newRow = crudDataContext.createRow(table);
		newRow.setValue("id", generateNextId(crudDataContext,table)); // Leave key out on jdbc/mongo backends for sequence/auto id.
		newRow.setValue("name", "test date "+System.currentTimeMillis());
		newRow.setValue("active", true);
		crudDataContext.persist(newRow);
		
		// Delete some data.
		DataSet dsDel = crudDataContext.query().from(table).select(table.getColumns()).where("name").equals("foo").or("name").equals("bar").execute();
		while (dsDel.next()) {
			crudDataContext.delete((UpdateableRow)dsDel.getRow());
		}
		dsDel.close();
		//testFile.deleteOnExit();
	}
	
	private int generateNextId(DataContext dataContext,Table table) {
		//DataSet dsMax = dataContext.query().from(table).select(FunctionType.MAX, table.getColumn(0)).execute();
		//assertTrue(dsMax.next());
		//Row rowMax = dsMax.getRow();
		//int recordId = new Integer(rowMax.getValue(0).toString()); // MAX stops on first row  ?
		//dsMax.close();
		int recordId = 0;
		DataSet ds = dataContext.query().from(table).select(table.getColumns()).execute();
		while (ds.next()) {
			Row row = ds.getRow();
			int id = new Integer(row.getValue(0).toString());
			if (id > recordId) {
				recordId = id;
			}
		}
		return recordId + 1;  // Fetch new ID because CSV has no auto ID
	}
}
