/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.backend.mongodb;

import org.bson.types.ObjectId;

import net.forwardfire.vasc.backend.AbstractVascBackendLocal;
import net.forwardfire.vasc.backend.VascBackendAccessDataKey;
import net.forwardfire.vasc.backend.VascBackendException;

import com.mongodb.BasicDBObject;
import com.mongodb.DB;
import com.mongodb.DBCollection;

/**
 * AbstractMongodbVascBackend
 * 
 * @author Willem Cazander
 * @version 1.0 Oct 26, 2014
 */
abstract public class AbstractMongodbVascBackend extends AbstractVascBackendLocal<ObjectId> {

	public static final String PK_FIELD_NAME = "_id";
	private MongodbConnectionProvider connectionProvider = null;
	private String collection = null;
	private DB database = null; 
	
	@Override
	public final void startBackendLocal() {
		requireNonNull(connectionProvider, "Can't start with null connectionProvider.");
		requireNonEmpty(collection, "Can't start without collection.");
		database = connectionProvider.getMongodbConnection();
		requireNonNull(database, "connectionProvider provided us a null database.");
	}
	
	@Override
	public final void stopBackendLocal() {
		database = null;
	}
	
	protected final DBCollection getDBCollection() {
		return database.getCollection(collection);
	}
	
	protected final void deleteById(ObjectId objectId) throws VascBackendException {
		getDBCollection().remove(new BasicDBObject(PK_FIELD_NAME,objectId)); // remove by _id
	}
	
	/**
	 * @see net.forwardfire.vasc.backend.VascBackend#createVascBackendAccessDataKey()
	 */
	@Override
	public final VascBackendAccessDataKey<ObjectId> createVascBackendAccessDataKey() {
		return new MongodbVascBackendAccessDataKey();
	}
	
	/**
	 * @return the connectionProvider
	 */
	public final MongodbConnectionProvider getConnectionProvider() {
		return connectionProvider;
	}
	
	/**
	 * @param connectionProvider the connectionProvider to set
	 */
	public final void setConnectionProvider(MongodbConnectionProvider connectionProvider) {
		this.connectionProvider = connectionProvider;
	}
	
	/**
	 * @return the collection
	 */
	public final String getCollection() {
		return collection;
	}
	
	/**
	 * @param collection the collection to set
	 */
	public final void setCollection(String collection) {
		this.collection = collection;
	}
}
