/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.backend.mongodb;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;

import org.bson.types.ObjectId;

import net.forwardfire.vasc.backend.VascBackendAccessDataKey;
import net.forwardfire.vasc.backend.VascBackendException;

/**
 * MongodbVascBackendAccessDataKey convert key type to string and back.
 * 
 * @author Willem Cazander
 * @version 1.0 Oct 26, 2014
 */
@SuppressWarnings("serial")
public class MongodbVascBackendAccessDataKey implements VascBackendAccessDataKey<ObjectId> {
	
	private final Map<String,Class<?>> keyMapType;
	
	public MongodbVascBackendAccessDataKey() {
		keyMapType = new HashMap<String,Class<?>>(1);
		keyMapType.put(AbstractMongodbVascBackend.PK_FIELD_NAME, ObjectId.class);
	}
	
	@Override
	public Map<String, Class<?>> getKeyMapType() {
		return keyMapType;
	}
	
	@Override
	public Map<String, Serializable> getKeyMapFromPrimaryKey(ObjectId primaryKey) throws VascBackendException {
		Map<String, Serializable> result = new HashMap<String,Serializable>(1);
		result.put(MongodbVascBackendCrud.PK_FIELD_NAME, primaryKey);
		return result;
	}
	
	@Override
	public ObjectId getPrimaryKeyFromKeyMap(Map<String, Serializable> keyMap) throws VascBackendException {
		return (ObjectId)keyMap.get(MongodbVascBackendCrud.PK_FIELD_NAME);
	}
	
	@Override
	public Map<String, String> getStringMapFromPrimaryKey(ObjectId primaryKey) throws VascBackendException {
		Map<String, String> result = new HashMap<String,String>(1);
		result.put(MongodbVascBackendCrud.PK_FIELD_NAME, primaryKey.toString());
		return result;
	}
	
	@Override
	public ObjectId getPrimaryKeyFromStringMap(Map<String, String> keyMap) throws VascBackendException {
		return new ObjectId(keyMap.get(MongodbVascBackendCrud.PK_FIELD_NAME));
	}
}
