/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.backend.mongodb;

import java.util.logging.Logger;

import org.bson.types.ObjectId;

import net.forwardfire.vasc.backend.VascBackendAccessDataRecord;
import net.forwardfire.vasc.backend.VascBackendException;
import net.forwardfire.vasc.backend.crud.VascBackendCrud;

import com.mongodb.DBCollection;
import com.mongodb.DBObject;
import com.mongodb.BasicDBObject;
import com.mongodb.WriteConcern;
import com.mongodb.WriteResult;

/**
 * MongodbVascBackend Provides backend for mongodb.
 * 
 * @author Willem Cazander
 * @version 1.0 Dec 30, 2011
 */
public class MongodbVascBackendCrud extends AbstractMongodbVascBackend implements VascBackendCrud<ObjectId,BasicDBObject> {

	private Logger logger = null;
	
	public MongodbVascBackendCrud() {
		logger = Logger.getLogger(MongodbVascBackendCrud.class.getName());
	}
	
	@Override
	public BasicDBObject newRecord() throws VascBackendException {
		return new BasicDBObject();
	}
	
	@Override
	public BasicDBObject fetch(ObjectId recordPK) throws VascBackendException {
		return (BasicDBObject)getDBCollection().findOne(new BasicDBObject(PK_FIELD_NAME,recordPK));
	}
	
	@Override
	public BasicDBObject persist(BasicDBObject object) throws VascBackendException {
		getDBCollection().insert(object);
		return fetch(object.getObjectId(AbstractMongodbVascBackend.PK_FIELD_NAME));
	}
	
	@Override
	public BasicDBObject merge(BasicDBObject row) throws VascBackendException {
		DBCollection coll = getDBCollection();
		DBObject query = new BasicDBObject();
		query.put(AbstractMongodbVascBackend.PK_FIELD_NAME,row.getObjectId(AbstractMongodbVascBackend.PK_FIELD_NAME));
		WriteResult wr = coll.update(query,row,false,false,WriteConcern.SAFE);
		logger.finer("WriteResult: "+wr);
		return fetch(row.getObjectId(AbstractMongodbVascBackend.PK_FIELD_NAME));
	}
	
	@Override
	public void delete(ObjectId objectId) throws VascBackendException {
		deleteById(objectId);
	}
	
	/**
	 * @see net.forwardfire.vasc.backend.VascBackend#createVascBackendAccessDataRecord()
	 */
	@Override
	public VascBackendAccessDataRecord<ObjectId, BasicDBObject> createVascBackendAccessDataRecord() {
		return new MongodbVascBackendAccessDataRecord();
	}
}
