/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.backend.object;

import java.io.Serializable;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.List;

import net.forwardfire.vasc.backend.AbstractVascBackendLocal;
import net.forwardfire.vasc.backend.VascBackendAccessDataKey;
import net.forwardfire.vasc.backend.VascBackendException;
import net.forwardfire.vasc.backend.list.VascBackendListFeature;
import net.forwardfire.vasc.backend.select.DefaultVascBackendSelectItem;
import net.forwardfire.vasc.backend.select.VascBackendSelect;
import net.forwardfire.vasc.backend.select.VascBackendSelectItem;

/**
* ObjectEnumVascBackendSelect uses an Enum as select backend. 
* 
* @author Willem Cazander
* @version 1.0 Oct 27, 2007
*/
public class ObjectEnumVascBackendSelect<ENUM_RETURN_TYPE extends Serializable> extends AbstractVascBackendLocal<ENUM_RETURN_TYPE> implements VascBackendSelect<ENUM_RETURN_TYPE> {
	
	private Class<? extends Enum<?>> enumClass = null;
	private ObjectEnumVascBackendSelectType enumReturnType = null;
	private String enumReturnMethod = null;
	
	@Override
	protected void startBackendLocal() {
		requireNonNull(enumClass, "enumClass is null");
		requireNonNull(enumReturnType, "enumClass is null");
	}
	
	@Override
	public VascBackendAccessDataKey<ENUM_RETURN_TYPE> createVascBackendAccessDataKey() {
		return null;
	}
	
	@Override
	public List<VascBackendSelectItem<ENUM_RETURN_TYPE>> execute() throws VascBackendException {
		
		List<VascBackendSelectItem<ENUM_RETURN_TYPE>> result = new ArrayList<VascBackendSelectItem<ENUM_RETURN_TYPE>>();
		
		for (Enum<?> enumObject:enumClass.getEnumConstants()) {
			
			
			boolean header = false;
			boolean enabled = true;
			String name = enumObject.name();
			boolean nameI18nKey = false;
			String description = enumObject.name();
			boolean descriptionI18nKey = false;
			ENUM_RETURN_TYPE primaryKey = (ENUM_RETURN_TYPE)getEnumValue(enumObject);
			
			result.add(new DefaultVascBackendSelectItem<ENUM_RETURN_TYPE>(header,enabled,name,nameI18nKey,description,descriptionI18nKey,primaryKey));
		}
		
		return result;
	}
	
	private Object getEnumValue(Enum<?> enumObject) {
		switch (enumReturnType) {
		case VALUE:
			return enumObject;
		case METHOD:
			try {
				return enumObject.getClass().getMethod(enumReturnMethod, new Class<?>[]{}).invoke(null, new Object[]{});
			} catch (Exception e) {
				throw new IllegalStateException("Could not get static return method value: "+e.getMessage(),e);
			}
		case NAME:
			return enumObject.name();
		case ORDINAL:
			return enumObject.ordinal();
		default:
			throw new IllegalStateException("Unknown enum return type: "+enumReturnType.name());
		}
	}
	
/*
	public List<VascBackendSelectItem> getVascSelectItems(VascEntry currentEntry) throws VascException {
		List<VascBackendSelectItem> result = new ArrayList<VascBackendSelectItem>(100);

		if (nullLabel!=null) {
			if (nullKeyValue==null) {
				nullKeyValue = "null";
			}
			nullLabel = currentEntry.getVascFrontendController().getVascEntryResourceResolver().getTextValue(nullLabel);
			VascBackendSelectItem item = new VascBackendSelectItem(nullLabel,null,nullKeyValue);
			result.add(item);
		}
		
		if (enumClass!=null) {
			for (Object o:enumClass.getEnumConstants()) {
				String value = o.toString();
				result.add(new VascBackendSelectItem(value,value,value));
			}
		}
		if (data==null) {
			return result;
		}
		if (data instanceof String) {
			String[] values = ((String)data).split(",");
			for (String value:values) {
				result.add(new VascBackendSelectItem(value,value,value));
			}
		} else if (data instanceof List) {
			List<?> values = (List<?>)data;
			for (Object o:values) {
				String value = ""+o; 
				result.add(new VascBackendSelectItem(value,value,value));
			}
		}
		
		return result;
	}
*/
	
}