/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.ejb3;

import java.io.Serializable;
import java.util.List;
import java.util.Map;

import net.forwardfire.vasc.backend.VascBackendException;
import net.forwardfire.vasc.backend.VascBackendAccessDataRecord;
import net.forwardfire.vasc.backend.VascEntryRecordCreator;
import net.forwardfire.vasc.backend.crud.VascBackendCrud;
import net.forwardfire.vasc.backend.list.VascBackendListResult;
import net.forwardfire.vasc.backend.list.VascBackendListRequest;

/**
 * VascServiceRemoteBackend wraps VascBasckend to the ejb3 context.
 * 
 * @author Willem Cazander
 * @version 1.0 18 Nov 2012
 */
public class VascServiceRemoteBackend<DATA_OBJECT extends Serializable,DATA_PK extends Serializable> implements VascBackendCrud<DATA_OBJECT,DATA_PK> {

	private String backendId = null;
	private VascServiceManager vascManager = null;
	
	public VascServiceRemoteBackend(VascServiceManager vascManager,String backendId) {
		if (vascManager==null) {
			throw new NullPointerException("Can't be remote backend on null VascServiceManager.");
		}
		if (backendId==null) {
			throw new NullPointerException("Can't be remote backend on null backendId.");
		}
		this.vascManager=vascManager;
		this.backendId=backendId;
	}
	
	public void startBackend() {
	}
	public void stopBackend() {
	}
	
	public String getId() {
		return backendId;
	}
	
	public void setId(String id) {
		// we cant change id
	}
	
	public void delete(DATA_PK object) throws VascBackendException {
		Object[] args = new Object[1];
		args[0]=object;
		vascManager.invokeBackendMethod(backendId, "delete", args);
	}
	
	public long doRecordMoveDownById(VascBackendListRequest state,Object primaryId) throws VascBackendException {
		Object[] args = new Object[2];
		args[0]=state;
		args[1]=primaryId;
		Object result = vascManager.invokeBackendMethod(backendId, "doRecordMoveDownById", args);
		return (Long)result;
	}
	
	public long doRecordMoveUpById(VascBackendListRequest state, Object primaryId) throws VascBackendException {
		Object[] args = new Object[2];
		args[0]=state;
		args[1]=primaryId;
		Object result = vascManager.invokeBackendMethod(backendId, "doRecordMoveUpById", args);
		return (Long)result;
	}
	
	@SuppressWarnings("unchecked")
	public VascBackendListResult<DATA_OBJECT> execute(VascBackendListRequest state) throws VascBackendException {
		Object[] args = new Object[1];
		args[0]=state;
		Object result = vascManager.invokeBackendMethod(backendId, "execute", args);
		return (VascBackendListResult<DATA_OBJECT>)result;
	}
	
	public long fetchTotalExecuteSize(VascBackendListRequest state) {
		Object[] args = new Object[1];
		args[0]=state;
		Object result = vascManager.invokeBackendMethod(backendId, "fetchTotalExecuteSize", args);
		return (Long)result;
	}
	
	public boolean isPageable() {
		Object[] args = new Object[0];
		Object result = vascManager.invokeBackendMethod(backendId, "isPageable", args);
		return (Boolean)result;
	}
	
	public boolean isRecordMoveable() {
		Object[] args = new Object[0];
		Object result = vascManager.invokeBackendMethod(backendId, "isRecordMoveable", args);
		return (Boolean)result;
	}
	
	public boolean isSearchable() {
		Object[] args = new Object[0];
		Object result = vascManager.invokeBackendMethod(backendId, "isSearchable", args);
		return (Boolean)result;
	}
	
	public boolean isSortable() {
		Object[] args = new Object[0];
		Object result = vascManager.invokeBackendMethod(backendId, "isSortable", args);
		return (Boolean)result;
	}
	
	@SuppressWarnings("unchecked")
	public DATA_OBJECT merge(DATA_OBJECT object) throws VascBackendException {
		Object[] args = new Object[1];
		args[0]=object;
		Object result = vascManager.invokeBackendMethod(backendId, "merge", args);
		return (DATA_OBJECT)result;
	}
	
	public void persist(DATA_OBJECT object) throws VascBackendException {
		Object[] args = new Object[1];
		args[0]=object;
		vascManager.invokeBackendMethod(backendId, "delete", args);
	}
	
	@Override
	public VascBackendAccessDataRecord provideVascEntryFieldValue() {
		Object[] args = new Object[0];
		Object result = vascManager.invokeBackendMethod(backendId, "provideVascEntryFieldValue", args);
		return (VascBackendAccessDataRecord)result;
	}
	
	@Override
	public VascEntryRecordCreator provideVascEntryRecordCreator() {
		Object[] args = new Object[0];
		Object result = vascManager.invokeBackendMethod(backendId, "provideVascEntryRecordCreator", args);
		return (VascEntryRecordCreator)result;
	}
	
	@SuppressWarnings("unchecked")
	public Map<String, Object> executePageSummary() {
		Object[] args = new Object[0];
		Object result = vascManager.invokeBackendMethod(backendId, "executePageSummary", args);
		return (Map<String, Object>)result;
	}
	
	@SuppressWarnings("unchecked")
	public Map<String, Object> executeTotalSummary() {
		Object[] args = new Object[0];
		Object result = vascManager.invokeBackendMethod(backendId, "executeTotalSummary", args);
		return (Map<String, Object>)result;
	}
	
	public boolean hasPageSummary() {
		Object[] args = new Object[0];
		Object result = vascManager.invokeBackendMethod(backendId, "isPageSummary", args);
		return (Boolean)result;
	}
	
	public boolean hasTotalSummary() {
		Object[] args = new Object[0];
		Object result = vascManager.invokeBackendMethod(backendId, "isTotalSummary", args);
		return (Boolean)result;
	}
	
	public boolean isReadOnly() {
		Object[] args = new Object[0];
		Object result = vascManager.invokeBackendMethod(backendId, "isReadOnly", args);
		return (Boolean)result;
	}
}
