/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.annotations;

import java.lang.annotation.Annotation;
import java.lang.reflect.Method;

/**
 * Parses the Vasc annotations 
 * 
 * @author Willem Cazander
 * @version 1.0 May 27, 2006
 */
public class VascAnnotationParser {
    
    /**
     * Gets the key of the VascToolTip key for the property of the class
     * @param beanClass The class to search for the property
     * @param property The property for the ToolTip
     * @return  The i18n key for an ToolTip 
     */
    public String getVascI18nDescription(Class<?> beanClass,String property) {
        return (String)getValue(beanClass,VascI18n.class,property,"description");
    }
    
    public String getVascI18nDescription(Class<?> beanClass) {
        return (String)getValue(beanClass,VascI18n.class,null,"description");
    }
    
    public String getVascI18nName(Class<?> beanClass,String property) {
        return (String)getValue(beanClass,VascI18n.class,property,"name");
    }
    
    public String getVascI18nName(Class<?> beanClass) {
        return (String)getValue(beanClass,VascI18n.class,null,"name");
    }
    
    public String getVascI18nHelpId(Class<?> beanClass,String property) {
        return (String)getValue(beanClass,VascI18n.class,property,"helpId");
    }
    
    public String getVascI18nHelpId(Class<?> beanClass) {
        return (String)getValue(beanClass,VascI18n.class,null,"helpId");
    }

    public String getVascI18nImage(Class<?> beanClass,String property) {
        return (String)getValue(beanClass,VascI18n.class,property,"image");
    }
    
    public String getVascI18nImage(Class<?> beanClass) {
        return (String)getValue(beanClass,VascI18n.class,null,"image");
    }
    
    public Integer getVascOrder(Class<?> beanClass,String property) {
        return (Integer)getValue(beanClass,VascFieldOrder.class,property,"order");
    }
    public Integer getVascOrderCreate(Class<?> beanClass,String property) {
        return (Integer)getValue(beanClass,VascFieldOrder.class,property,"orderCreate");
    }
    public Integer getVascOrderEdit(Class<?> beanClass,String property) {
        return (Integer)getValue(beanClass,VascFieldOrder.class,property,"orderEdit");
    }
    public Integer getVascOrderList(Class<?> beanClass,String property) {
        return (Integer)getValue(beanClass,VascFieldOrder.class,property,"orderList");
    }
    

    public Integer getVascStyleSizeList(Class<?> beanClass,String property) {
        return (Integer)getValue(beanClass,VascStyle.class,property,"sizeList");
    }
    public Integer getVascStyleSizeEdit(Class<?> beanClass,String property) {
        return (Integer)getValue(beanClass,VascStyle.class,property,"sizeEdit");
    }
    public String getVascStyleStyleList(Class<?> beanClass,String property) {
        return (String)getValue(beanClass,VascStyle.class,property,"styleList");
    }
    public String getVascStyleStyleEdit(Class<?> beanClass,String property) {
        return (String)getValue(beanClass,VascStyle.class,property,"styleEdit");
    }
    
    
    public String getVascFieldBackendName(Class<?> beanClass,String property) {
        return (String)getValue(beanClass,VascField.class,property,"backendName");
    }
    public Boolean getVascFieldCreate(Class<?> beanClass,String property) {
    	return (Boolean)getValue(beanClass,VascField.class,property,"create");
    }
    public Boolean getVascFieldEdit(Class<?> beanClass,String property) {
        return (Boolean)getValue(beanClass,VascField.class,property,"edit");
    }
    public Boolean getVascFieldEditReadOnly(Class<?> beanClass,String property) {
        return (Boolean)getValue(beanClass,VascField.class,property,"editReadOnly");
    }
    public Boolean getVascFieldEditBlank(Class<?> beanClass,String property) {
        return (Boolean)getValue(beanClass,VascField.class,property,"editBlank");
    }
    public Boolean getVascFieldList(Class<?> beanClass,String property) {
        return (Boolean)getValue(beanClass,VascField.class,property,"list");
    }
    public Boolean getVascFieldView(Class<?> beanClass,String property) {
        return (Boolean)getValue(beanClass,VascField.class,property,"view");
    }
    public Boolean getVascFieldOptional(Class<?> beanClass,String property) {
        return (Boolean)getValue(beanClass,VascField.class,property,"optional");
    }
    public Boolean getVascFieldSortable(Class<?> beanClass,String property) {
        return (Boolean)getValue(beanClass,VascField.class,property,"sortable");
    }
    public Boolean getVascFieldSumable(Class<?> beanClass,String property) {
        return (Boolean)getValue(beanClass,VascField.class,property,"sumable");
    }
    public Boolean getVascFieldGraphable(Class<?> beanClass,String property) {
        return (Boolean)getValue(beanClass,VascField.class,property,"graphable");
    }
    
    
    public String getVascFieldType(Class<?> beanClass,String property) {
        return (String)getValue(beanClass,VascFieldType.class,property,"type");
    }
    public String[] getVascFieldTypeProperties(Class<?> beanClass,String property) {
        return (String[])getValue(beanClass,VascFieldType.class,property,"properties");
    }
    
    
    public Class<?> getVascFieldTemplateClass(Class<?> beanClass,String property) {
        return (Class<?>)getValue(beanClass,VascFieldTemplate.class,property,"templateClass");
    }
    public String getVascFieldTemplate(Class<?> beanClass,String property) {
        return (String)getValue(beanClass,VascFieldTemplate.class,property,"template");
    }
    
    public String getVascRolesCreate(Class<?> beanClass,String property) {
        return (String)getValue(beanClass,VascRoles.class,property,"rolesCreate");
    }
    public String getVascRolesEdit(Class<?> beanClass,String property) {
        return (String)getValue(beanClass,VascRoles.class,property,"rolesEdit");
    }
    public String getVascRolesEditReadOnly(Class<?> beanClass,String property) {
        return (String)getValue(beanClass,VascRoles.class,property,"rolesEditReadOnly");
    }
    public String getVascRolesList(Class<?> beanClass,String property) {
        return (String)getValue(beanClass,VascRoles.class,property,"rolesList");
    }

    
    public VascChoices getVascChoices(Class<?> beanClass,String property) {
        return (VascChoices)getValue(beanClass,VascChoices.class,property,null);
    }
    public VascEventListener getVascEventListener(Class<?> beanClass) {
        return (VascEventListener)getValue(beanClass,VascEventListener.class,null,null);
    }   
    
    
    /**
     * TODO: change this, because we need differnce between key and value instance
     * @param beanClass
     * @param property
     * @return
     */
    public String getVascDefaultValue(Class<?> beanClass,String property) {
        return (String)getValue(beanClass,VascDefaultValue.class,property,null);
    }
    

    public String getVascDisplayName(Class<?> beanClass) {
    	//System.out.println("========== GetDisPlayName: "+beanClass);
    	
    	// first search in class for display name
        for (Method method:beanClass.getMethods()) {
            if (method.getName().startsWith("get")==false) {  //a bit durty  
                continue;
            }
            Annotation anno = method.getAnnotation(VascDisplayName.class);
    		if (anno==null) {
    			continue;
    		}
    		return method.getName().substring(3,4).toLowerCase()+method.getName().substring(4); // field name without get
        }
        
    	// then search in class for display name on templated fields.
        for (Method method:beanClass.getMethods()) {
            if (method.getName().startsWith("get")==false) {  //a bit durty  
                continue;
            }
            VascFieldTemplate template = method.getAnnotation(VascFieldTemplate.class);
            if (template==null) {
            	continue; // no template anno
            }
        	//System.out.println("Search template for: "+method.getName());
        	String tempProp = method.getName().substring(3);
        	if ("".equals(template.template())==false) {
        		tempProp = template.template();
        	}
            for (Method method2:template.templateClass().getMethods()) {
                if (method2.getName().equalsIgnoreCase("get"+tempProp)==false) {  //a bit durty  
                    continue;
                }
                Annotation anno = method2.getAnnotation(VascDisplayName.class);
                //System.out.println("Template annot: "+anno+" prop: "+tempProp);
        		if (anno==null) {
        			break;
        		}
        		//System.out.println("Found template");
        		return method.getName().substring(3,4).toLowerCase()+method.getName().substring(4); // field name without get
            }
        }
        
        //System.out.println("Defraulting to key");
        return getVascPrimaryKey(beanClass); // fall back on primary key
    }
    
    public String getVascDisplayName(Class<?> beanClass,String property) {
        for (Method method:beanClass.getMethods()) {
            if (method.getName().equalsIgnoreCase("get"+property)==false) {  //a bit durty  
                continue;
            }
            VascFieldTemplate template = method.getAnnotation(VascFieldTemplate.class);
            if (template!=null) {
            	String tempProp = property;
            	if ("".equals(template.template())==false) {
            		tempProp = template.template();
            	}
            	String value = getVascDisplayName(template.templateClass(),tempProp);
            	if (value!=null) {
            		return value;
            	}
            }
            VascModelReference mt = method.getAnnotation(VascModelReference.class);
            if (mt==null) {
            	return null;  // no display name 
            }
            Class<?> typeClass = mt.type();
            if (Object.class==mt.type()) {
                typeClass = method.getReturnType();
            }
            String name = getVascDisplayName(typeClass);
            return name;
        }
        return null;
    }
    
    public String getVascPrimaryKey(Class<?> beanClass) {
    	
    	// first search Key anno
        for (Method method:beanClass.getMethods()) {
            if(method.getName().startsWith("get")==false) {  //a bit durty  
                continue;
            }
            Annotation anno = method.getAnnotation(VascPrimaryKey.class);
    		if (anno==null) {
    			continue;
    		}
    		return method.getName().substring(3,4).toLowerCase()+method.getName().substring(4); // field name without get
        }
    	
        for (Method method:beanClass.getMethods()) {
            if(method.getName().startsWith("get")==false) {  //a bit durty  
                continue;
            }
            VascFieldTemplate template = method.getAnnotation(VascFieldTemplate.class);
            if (template==null) {
            	continue; // no template
            }
        	String value = getVascPrimaryKey(template.templateClass()); // note templateing the templated class is possible
        	if (value!=null) {
        		return value;
        	}
        }
        return null; // maybe fallback on getId() ?
    }
   
    
    /**
     * No oop code here...refactor at some point in time
     * 
     * 
     * @param beanClass
     * @param property
     * @param annotationType
     * @return
     */
    @SuppressWarnings("unchecked")
    private Object getValue(Class<?> beanClass,Class annotationType,String property,String field) {
        if(beanClass==null)     { throw new NullPointerException("beanClass may not be null");      }
        if(annotationType==null){ throw new NullPointerException("annotationType may not be null"); }

        Object result = null;
        if (property==null) {
        	String def = beanClass.getName()+"."+annotationType.getSimpleName();
        	if (field!=null) {
        		def = def+"."+field;
        	}
        	
        	Annotation anno = beanClass.getAnnotation(annotationType);
        	if (anno==null) {
        		// no annotation == no default
        		if (annotationType.equals(VascDefaultValue.class)) {
        			return null;
        		}
        		if (annotationType.equals(VascEventListener.class)) {
        			return null;
        		}
        	} else {
                if (annotationType.equals(VascEventListener.class)) {
                	return anno;
                }
        		result = doAnnotation(anno,def,field);
        		if(result!=null) {
        			return result;
        		}
        	}
            return def;
        }
        
        String propRest = null;
        int index = property.indexOf(".");
        if (index>0) {
        	propRest = property.substring(index+1);
        	property = property.substring(0,index);
        }
        
        String def = beanClass.getName()+"."+property+"."+annotationType.getSimpleName();
    	if (field!=null) {
    		def = def+"."+field;
    	}
        for (Method method:beanClass.getMethods()) {
            if (method.getName().equalsIgnoreCase("get"+property)==false) {  //a bit durty  
                continue;
            }
            VascModelReference mt = method.getAnnotation(VascModelReference.class);
            if (mt!=null & (annotationType.equals(VascI18n.class) | annotationType.equals(VascField.class) | annotationType.equals(VascStyle.class) )) {
            	Class typeClass = mt.type();
                if (Object.class==mt.type()) {
                    typeClass = method.getReturnType();
                }
                
                if ( annotationType.equals(VascStyle.class) ) {
                	String disName = getVascDisplayName(typeClass);
                	return getValue(typeClass,annotationType,disName,field);
                }
                
                
                if ( annotationType.equals(VascField.class) ) {
                	if ("backendName".equals(field)) {
                		//return property+"."+getValue(typeClass,annotationType,propRest,field);
                	}
                } else {
                	// recursif function:
                	return getValue(typeClass,annotationType,propRest,field);
                }
            }
            
            VascFieldTemplate template = method.getAnnotation(VascFieldTemplate.class);
            if (template!=null) {
            	
            	if (annotationType.equals(VascFieldTemplate.class)) {
            		if ("templateClass".equals(field)) {
            			return template.templateClass();
            		}
            		if ("template".equals(field)) {
            			return template.template();
            		}
            		return null; 
            	}
            	
            	String tempProp = property;
            	if ("".equals(template.template())==false) {
            		tempProp = template.template();
            	}
            	Object value = getValue(template.templateClass(),annotationType,tempProp,field);
            	if (value!=null) {
            		return value;
            	}
            }
            
            Annotation anno = method.getAnnotation(annotationType);
            // no annotation == no default
    		if (anno==null && annotationType.equals(VascDefaultValue.class)) {
    			return null;
    		}
            result = doAnnotation(anno,def,field);
            if(result!=null) {
        		return result;
        	}
            break; // return default
        }
        
        if (annotationType.equals(VascField.class)) {
        	try {
	    		if ("backendName".equals(field)) {
	    			return VascField.class.getMethod("backendName").getDefaultValue();
	    		}
	    		if ("displayName".equals(field)) {
	    			return VascField.class.getMethod("displayName").getDefaultValue();
	    		}
	    		if ("create".equals(field)) {
	    			return VascField.class.getMethod("create").getDefaultValue();
	    		}
	    		if ("edit".equals(field)) {
	    			return VascField.class.getMethod("edit").getDefaultValue();
	    		}
	    		if ("editReadOnly".equals(field)) {
	    			return VascField.class.getMethod("editReadOnly").getDefaultValue();
	    		}
	    		if ("editBlank".equals(field)) {
	    			return VascField.class.getMethod("editBlank").getDefaultValue();
	    		}
	    		if ("list".equals(field)) {
	    			return VascField.class.getMethod("list").getDefaultValue();
	    		}
	    		if ("view".equals(field)) {
	    			return VascField.class.getMethod("view").getDefaultValue();
	    		}
	    		if ("optional".equals(field)) {
	    			return VascField.class.getMethod("optional").getDefaultValue();
	    		}
	    		if ("sortable".equals(field)) {
	    			return VascField.class.getMethod("sortable").getDefaultValue();
	    		}
	    		// This are the default value then....
	            return null;
			} catch (Exception e) {
				e.printStackTrace();
				return false;
			}
		}
        if (annotationType.equals(VascFieldTemplate.class)) {
        	return null;
        }
        if (annotationType.equals(VascFieldType.class)) {
        	return null;
        }
        if (annotationType.equals(VascStyle.class)) {
        	return null;
        }
        if (annotationType.equals(VascRoles.class)) {
        	return null;
        }
        if (annotationType.equals(VascChoices.class)) {
        	return null;
        }
        if (annotationType.equals(VascEventListener.class)) {
        	return null;
        }
        if (annotationType.equals(VascFieldOrder.class)) {
        	int indexOrder = 0;
            for (Method method:beanClass.getMethods()) {
                if (method.getName().equalsIgnoreCase("get"+property)==false) {  //a bit durty
                	indexOrder = indexOrder + 100;
                    continue;
                }
                return indexOrder;
            }
        	return null;
        }
        return def;
    }

    private Object doAnnotation(Annotation b,String def,String field) {
    	if (b==null) {
    		return null;
    	}
    	
    	Class<?> a = b.annotationType();
    	if (a.equals(VascI18n.class)) {
    		VascI18n l = (VascI18n)b;
    		String val = null;
    		
    		if ("name".equals(field)) {
    			val = l.name();
    		}
    		if ("description".equals(field)) {
    			val = l.description();
    		}
    		if ("image".equals(field)) {
    			val = l.image();
    		}
    		if ("helpId".equals(field)) {
    			val = l.helpId();
    		}
    		if (val==null) {
    			val = "";
    		}
    		if ("".equals(val)) {
    			return def;
    		}
            return val;
    	}
    	if (a.equals(VascFieldOrder.class)) {
    		VascFieldOrder v = (VascFieldOrder)b;
    		if ("orderCreate".equals(field)) {
    			return v.orderCreate();
    		}
    		if ("orderEdit".equals(field)) {
    			return v.orderEdit();
    		}
    		if ("orderList".equals(field)) {
    			return v.orderList();
    		}
    		return v.order();
    	}
    	if (a.equals(VascChoices.class)) {
    		VascChoices v = (VascChoices)b;
    		return v;
    	}
    	if (a.equals(VascDefaultValue.class)) {
    		VascDefaultValue v = (VascDefaultValue)b;
    		if ("null".equals(v.value())) {
    			return null;
    		}
    		return v.value();
    	}
    	if (a.equals(VascStyle.class)) {
    		VascStyle c = (VascStyle)b;
    		if ("sizeList".equals(field)) {
    		 	return c.sizeList();
    		}
    		if ("sizeEdit".equals(field)) {
    			return c.sizeEdit();
    		}
    		if ("styleList".equals(field)) {
    			return c.styleList();
    		}
    		if ("styleEdit".equals(field)) {
    			return c.styleEdit();
    		}
            return null;
    	}

    	if (a.equals(VascField.class)) {
    		VascField c = (VascField)b;
    		if ("backendName".equals(field)) {
    			if ("".equals(c.backendName())) {
    				return null;
    			}
    			return c.backendName();
    		}
    		if ("displayName".equals(field)) {
    			if ("".equals(c.displayName())) {
    				return null;
    			}
    			return c.displayName();
    		}
    		if ("create".equals(field)) {
    			return c.create();
    		}
    		if ("edit".equals(field)) {
    			return c.edit();
    		}
    		if ("editReadOnly".equals(field)) {
    			return c.editReadOnly();
    		}
    		if ("editBlank".equals(field)) {
    			return c.editBlank();
    		}
    		if ("list".equals(field)) {
    			return c.list();
    		}
    		if ("view".equals(field)) {
    			return c.view();
    		}
    		if ("optional".equals(field)) {
    			return c.optional();
    		}
    		if ("sortable".equals(field)) {
    			return c.sortable();
    		}
    		if ("sumable".equals(field)) {
    			return c.sumable();
    		}
    		if ("graphable".equals(field)) {
    			return c.graphable();
    		}
            return null;
    	}
    	if (a.equals(VascFieldType.class)) {
    		VascFieldType c = (VascFieldType)b;
    		if ("type".equals(field)) {
    			if ("".equals(c.type())) {
    				return null;
    			}
    			return c.type();
    		}
    		if ("properties".equals(field)) {
    			return c.properties();
    		}
    	}
    	if (a.equals(VascRoles.class)) {
    		VascRoles c = (VascRoles)b;
    		if ("rolesCreate".equals(field)) {
    			if ("".equals(c.rolesCreate())) {
    				return null;
    			}
    			return c.rolesCreate();
    		}
    		if ("rolesEdit".equals(field)) {
    			if ("".equals(c.rolesEdit())) {
    				return null;
    			}
    			return c.rolesEdit();
    		}
    		if ("rolesEditReadOnly".equals(field)) {
    			if ("".equals(c.rolesEditReadOnly())) {
    				return null;
    			}
    			return c.rolesEditReadOnly();
    		}
    		if ("rolesList".equals(field)) {
    			if ("".equals(c.rolesList())) {
    				return null;
    			}
    			return c.rolesList();
    		}
            return null;
    	}
    	
    	return null;
    }
}