/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.annotations;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import net.forwardfire.vasc.backend.select.VascBackendSelectItem;
import net.forwardfire.vasc.core.ui.VascSelectItemModel;
import net.forwardfire.vasc.core.VascException;



/**
 * Selected the vasc choices
 * 
 * 
 * @author Willem Cazander
 * @version 1.0 Apr 23, 2009
 */
@SuppressWarnings("serial")
public class VascChoicesSelectItemModel implements VascSelectItemModel,Serializable {

	private VascChoices vascChoices = null;
	private String nullLabel = null;
	private String nullKeyValue = null;
	
	public VascChoicesSelectItemModel(VascChoices vascChoices) {
		this.vascChoices=vascChoices;
	}
	
	/**
	 * @see net.forwardfire.vasc.core.ui.VascSelectItemModel#getVascSelectItems(net.forwardfire.vasc.core.VascEntry)
	 */
	@Override
	public List<VascBackendSelectItem> getVascSelectItems(net.forwardfire.vasc.core.VascEntry entry) throws VascException {
		List<VascBackendSelectItem> result = new ArrayList<VascBackendSelectItem>(10);
		if ("null".equals(vascChoices.nullLabel())==false) {
			nullLabel = vascChoices.nullLabel();
			nullKeyValue = vascChoices.nullKeyValue();
		}
		if (nullLabel!=null) {
			if (nullKeyValue==null) {
				nullKeyValue = "null";
			}
			String nullLabelText = entry.getVascFrontendController().getVascEntryResourceResolver().getTextValue(nullLabel);
			VascBackendSelectItem item = new VascBackendSelectItem(nullLabelText,null,nullKeyValue);
			result.add(item);
		}
		int index = 0;
		boolean val = false;
		String[] a = vascChoices.choisesKeys();
		if (a.length==0) {
			a=vascChoices.choisesValues();
			val = true;
		} else if (vascChoices.choisesKeys().length!=vascChoices.choisesValues().length) {
			throw new IllegalStateException("Can't have unequal sizes of array of keys and values of vascChoices annotation.");
		}
		for (String key:a) {
			VascBackendSelectItem item = new VascBackendSelectItem();
			String name = null;
			if (val) {
				name = key;
			} else {
				name = entry.getVascFrontendController().getVascEntryResourceResolver().getTextValue(key);
			}
			item.setLabel(name);
			Serializable value = vascChoices.choisesValues()[index];
			if (vascChoices.choisesType().equals(String.class)==false) {
				try {
					value = (Serializable) vascChoices.choisesType().getConstructor(String.class).newInstance(key);
				} catch (Exception e) {
				}
			}
			item.setValue(value);
			item.setKeyValue(key);
			result.add(item);
			
			index++;
		}
		return result;
	}
	
	/**
	 * @return the nullLabel
	 */
	@Override
	public String getNullLabel() {
		return nullLabel;
	}

	/**
	 * @param nullLabel the nullLabel to set
	 */
	@Override
	public void setNullLabel(String nullLabel) {
		this.nullLabel = nullLabel;
	}

	/**
	 * @return the nullKeyValue
	 */
	@Override
	public String getNullKeyValue() {
		return nullKeyValue;
	}

	/**
	 * @param nullKeyValue the nullKeyValue to set
	 */
	@Override
	public void setNullKeyValue(String nullKeyValue) {
		this.nullKeyValue = nullKeyValue;
	}
}
