/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.backend.proxy;

import java.io.Serializable;
import java.util.EnumSet;
import java.util.List;

import net.forwardfire.vasc.backend.VascBackendException;
import net.forwardfire.vasc.backend.list.VascBackendList;
import net.forwardfire.vasc.backend.list.VascBackendListFeature;
import net.forwardfire.vasc.backend.list.VascBackendListResult;
import net.forwardfire.vasc.backend.list.VascBackendListRequest;
import net.forwardfire.vasc.core.VascEntry;

/**
 * VascBackendProxy to implement missing features of the backend.
 * 
 * @author Willem Cazander
 * @version 1.0 Apr 1, 2009
 */
abstract public class AbstractVascBackendProxy<DATA_PK extends Serializable,DATA_RECORD extends Serializable> implements VascBackendProxy<DATA_PK,DATA_RECORD> {
	
	protected VascBackendList<DATA_PK,DATA_RECORD> backend = null;
	protected VascEntry entry = null;
	private volatile boolean running = false;
	
	@Override
	abstract public VascBackendProxy<DATA_PK,DATA_RECORD> clone() throws CloneNotSupportedException;
	
	protected void startBackendProxy() {
	}
	protected void stopBackendProxy() {	
	}
	
	/**
	 * @see net.forwardfire.vasc.backend.proxy.VascBackendProxy#initProxy(net.forwardfire.vasc.backend.crud.VascBackendList, net.forwardfire.vasc.core.VascEntry)
	 */
	@Override
	final public void initProxy(VascBackendList<DATA_PK,DATA_RECORD> backend, VascEntry entry) {
		if (backend==null) {
			throw new NullPointerException("backend object may not be null.");
		}
		if (entry==null) {
			throw new NullPointerException("entry object may not be null.");
		}
		this.backend=backend;
		this.entry=entry;
		startBackend();
	}
	
	/**
	 * @see net.forwardfire.vasc.backend.VascBackendLifeCycle#startBackend()
	 */
	@Override
	final public void startBackend() {
		startBackendProxy();
		running = true;
	}
	
	/**
	 * @see net.forwardfire.vasc.backend.VascBackendLifeCycle#stopBackend()
	 */
	@Override
	final public void stopBackend() {
		running = false;
		stopBackendProxy();
	}
	
	/**
	 * @see net.forwardfire.vasc.backend.VascBackendLifeCycle#isRunning()
	 */
	@Override
	public boolean isRunning() {
		return running;
	}

	/**
	 * @see net.forwardfire.vasc.backend.proxy.VascBackendProxy#isProxyNeeded()
	 */
	@Override
	public boolean isProxyNeeded() {
		return true;
	}
	
	/**
	 * @see net.forwardfire.vasc.backend.crud.VascBackendCrud#getId()
	 */
	@Override
	public String getId() {
		return backend.getId();
	}
	
	/**
	 * @see net.forwardfire.vasc.backend.list.VascBackendList#getSupportedFeatures()
	 */
	@Override
	public EnumSet<VascBackendListFeature> getSupportedFeatures() {
		return backend.getSupportedFeatures(); // backend will always provide cloned copy
	}
	
	@Override
	public VascBackendListResult<DATA_RECORD> execute(VascBackendListRequest request) throws VascBackendException {
		return backend.execute(request);
	}
	
	@Override
	public List<String> getSupportedRowActions() {
		return backend.getSupportedRowActions();
	}
	
	@Override
	public void executeRowAction(VascBackendListRequest request,List<DATA_PK> recordPKs,String actionName) throws VascBackendException {
		executeRowAction(request, recordPKs, actionName);
	}
}
