/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.backend.proxy;

import java.io.Serializable;
import java.util.EnumSet;

import net.forwardfire.vasc.backend.list.VascBackendListFeature;

/**
 * AbstractVascBackendProxyFeature to implement single features of the backend.
 * 
 * @author Willem Cazander
 * @version 1.0 Aug 27, 2014
 */
abstract public class AbstractVascBackendProxyFeature<DATA_PK extends Serializable,DATA_RECORD extends Serializable> extends AbstractVascBackendProxy<DATA_PK,DATA_RECORD> {
	
	private final VascBackendListFeature proxyFeature;
	private final VascBackendListFeature proxyFeatureExtraFlag;
	
	protected AbstractVascBackendProxyFeature(VascBackendListFeature proxyFeature,VascBackendListFeature proxyFeatureExtraFlag) {
		this.proxyFeature=proxyFeature;
		this.proxyFeatureExtraFlag=proxyFeatureExtraFlag;
	}
	
	/**
	 * @see net.forwardfire.vasc.backend.proxy.AbstractVascBackendProxy#isProxyNeeded()
	 */
	@Override
	final public boolean isProxyNeeded() {
		return !backend.getSupportedFeatures().contains(proxyFeature);
	}
	
	/**
	 * @see net.forwardfire.vasc.backend.proxy.AbstractVascBackendProxy#getSupportedFeatures()
	 */
	@Override
	final public EnumSet<VascBackendListFeature> getSupportedFeatures() {
		EnumSet<VascBackendListFeature> result = super.getSupportedFeatures();
		result.add(proxyFeature);
		result.add(proxyFeatureExtraFlag);
		return result;
	}
}
