/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.backend.proxy;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import net.forwardfire.vasc.backend.VascBackendException;
import net.forwardfire.vasc.backend.list.DefaultVascBackendListResult;
import net.forwardfire.vasc.backend.list.VascBackendListResult;
import net.forwardfire.vasc.backend.list.VascBackendListRequest;

/**
* Simple filter support
* 
* @author Willem Cazander
* @version 1.0 Oct 27, 2007
*/
public class VascBackendProxyFilter<DATA_PK extends Serializable,DATA_RECORD extends Serializable> extends AbstractVascBackendProxy<DATA_PK,DATA_RECORD> {
	
	private List<VascProxyFilter> filters = null;
	
	/**
	 * @see java.lang.Object#clone()
	 */
	@Override
	public VascBackendProxy<DATA_PK,DATA_RECORD> clone() throws CloneNotSupportedException {
		VascBackendProxyFilter<DATA_PK,DATA_RECORD> result = new VascBackendProxyFilter<>();
		return result;
	}
	
	/**
	 * @see net.forwardfire.vasc.backend.proxy.AbstractVascBackendProxy#startBackendProxy()
	 */
	@Override
	protected void startBackendProxy() {
		filters = entry.getVascBackendFilters();
		for (VascProxyFilter filter:filters) {
			filter.initFilter(entry);
		}
	}
	
	/**
	 * @see net.forwardfire.vasc.backend.proxy.AbstractVascBackendProxy#isProxyNeeded()
	 */
	@Override
	public boolean isProxyNeeded() {
		if (entry.getVascBackendFilters().isEmpty()) {
			return false;
		}
		return true;
	}
	
	/**
	 * @see net.forwardfire.vasc.backend.crud.VascBackendCrud#execute(VascBackendListRequest state)
	 */
	@Override
	public VascBackendListResult<Serializable> execute(VascBackendListRequest state) throws VascBackendException {
		VascBackendListResult<Serializable> result =  backend.execute(state);
		if (filters==null) {
			return result;
		}
		if (filters.isEmpty()) {
			return result;
		}
		List<Serializable> search = new ArrayList<Serializable>(result.getData().size()/2);
		for (Serializable o:result.getData()) {
			for (VascProxyFilter filter:filters) {
				Serializable r = filter.filterObject(o);
				if (r!=null) {
					search.add(r);
				}
			}
		}
		return new DefaultVascBackendListResult<Serializable>(search,search.size());
	}
}