/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.backend.proxy;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import net.forwardfire.vasc.backend.VascBackendException;
import net.forwardfire.vasc.backend.VascBackendAccessDataRecord;
import net.forwardfire.vasc.backend.list.DefaultVascBackendListResult;
import net.forwardfire.vasc.backend.list.VascBackendListFeature;
import net.forwardfire.vasc.backend.list.VascBackendListResult;
import net.forwardfire.vasc.backend.list.VascBackendListRequest;
import net.forwardfire.vasc.core.VascEntryField;

/**
* Simple text search 
* 
* @author Willem Cazander
* @version 1.0 Oct 27, 2007
*/
public class VascBackendProxySearch<DATA_PK extends Serializable,DATA_RECORD extends Serializable> extends AbstractVascBackendProxyFeature<DATA_PK,DATA_RECORD> {
	
	public VascBackendProxySearch() {
		super(VascBackendListFeature.TEXT_SEARCH,VascBackendListFeature.TEXT_SEARCH_PROXY);
	}
	
	/**
	 * @see java.lang.Object#clone()
	 */
	@Override
	public VascBackendProxy<DATA_PK,DATA_RECORD> clone() throws CloneNotSupportedException {
		VascBackendProxySearch<DATA_PK,DATA_RECORD> result = new VascBackendProxySearch<>();
		return result;
	}
	
	/**
	 * @see net.forwardfire.vasc.backend.crud.VascBackendCrud#execute(VascBackendListRequest state)
	 */
	@Override
	public VascBackendListResult<Serializable> execute(VascBackendListRequest state) throws VascBackendException {
		VascBackendListResult<Serializable> result =  backend.execute(state);
		if (state.getSearchString()==null) {
			return result;
		}
		if (state.getSearchString().isEmpty()) {
			return result;
		}
		String searchString = state.getSearchString().toLowerCase();
		List<Serializable> search = new ArrayList<Serializable>(result.getData().size()/4);
		for (Serializable record:result.getData()) {
			for (VascEntryField field:entry.getVascEntryFields()) {
				VascBackendAccessDataRecord<Serializable,Serializable> fieldValue = backend.provideVascEntryFieldValue();
				Serializable value = fieldValue.getValue(field.getBackendName(), record);
				if (value==null) {
					continue; // can't search null values.
				}
				String r = value.toString().toLowerCase();
				if (r.contains(searchString)) {
					search.add(record);
					break;
				}
			}
		}
		return new DefaultVascBackendListResult<Serializable>(search);
	}
}