/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.backend.proxy;

import java.io.Serializable;
import java.util.Collections;
import java.util.Comparator;

import net.forwardfire.vasc.backend.VascBackendException;
import net.forwardfire.vasc.backend.VascBackendAccessDataRecord;
import net.forwardfire.vasc.backend.list.VascBackendListFeature;
import net.forwardfire.vasc.backend.list.VascBackendListResult;
import net.forwardfire.vasc.backend.list.VascBackendListRequest;
import net.forwardfire.vasc.core.VascEntryField;

/**
* Add an sortware sort an a backend 
* 
* @author Willem Cazander
* @version 1.0 Oct 27, 2007
*/
public class VascBackendProxySort<DATA_PK extends Serializable,DATA_RECORD extends Serializable> extends AbstractVascBackendProxyFeature<DATA_PK,DATA_RECORD> {
	
	public VascBackendProxySort() {
		super(VascBackendListFeature.SORT_DATA,VascBackendListFeature.SORT_DATA_PROXY);
	}
	
	/**
	 * @see java.lang.Object#clone()
	 */
	@Override
	public VascBackendProxy<DATA_PK,DATA_RECORD> clone() throws CloneNotSupportedException {
		VascBackendProxySort<DATA_PK,DATA_RECORD> result = new VascBackendProxySort<>();
		return result;
	}
	
	public VascEntryField getVascEntryFieldByBackendName(String linkId) {
		for (VascEntryField a:entry.getVascEntryFields()) {
			if (a.getBackendName().equals(linkId)) {
				return a;
			}
		}
		return null;
	}
	
	/**
	 * @see net.forwardfire.vasc.backend.crud.VascBackendCrud#execute(VascBackendListRequest state)
	 */
	@SuppressWarnings("unchecked")
	@Override
	public VascBackendListResult<DATA_RECORD> execute(final VascBackendListRequest state) throws VascBackendException {
		VascBackendListResult<DATA_RECORD> result =  backend.execute(state);
		if (state.getSortField()==null) {
			return result;
		}
		final VascEntryField field = getVascEntryFieldByBackendName(state.getSortField());
		final VascBackendAccessDataRecord<Serializable,Serializable> fieldValue = backend.provideVascEntryFieldValue();
		Collections.sort(result.getData(), new Comparator<Serializable>() {
			@SuppressWarnings("rawtypes")
			@Override
			public int compare(Serializable o1, Serializable o2) {
				try {
					Comparable c1 = null;
					Comparable c2 = null;
//					if (field.getDisplayName()!=null) {
//						c1 = fieldValue.getValue(field.getBackendName(), o1);
//						c2 = fieldValue.getValue(field.getBackendName(), o2);	
//					} else {
					// TODO: fix none Comparable goes toString 
						c1 = (Comparable)fieldValue.getValue(field.getBackendName(), o1);
						c2 = (Comparable)fieldValue.getValue(field.getBackendName(), o2);
//					}
					if (c1==null & c2==null) {
						return 0;
					}
					if (c1==null) {
						if (state.isSortAscending()) {
							return 1;
						} else {
							return -1;
						}
					}
					if (c2==null) {
						if (state.isSortAscending()) {
							return -1;
						} else {
							return 1;
						}
					}
					if (c1 instanceof String & c2 instanceof String) {
						c1 = ((String)c1).toLowerCase();
						c2 = ((String)c2).toLowerCase();
					}
					
					if (state.isSortAscending()) {
						return c1.compareTo(c2);
					} else {
						return c2.compareTo(c1);
					}
				} catch (VascBackendException e) {
					e.printStackTrace();
					return 0;
				}
			}
		});
		return result;
	}
}