/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.backend.proxy;

import java.io.Serializable;
import java.util.logging.Level;
import java.util.logging.Logger;

import net.forwardfire.vasc.backend.VascBackendException;
import net.forwardfire.vasc.backend.list.VascBackendListResult;
import net.forwardfire.vasc.backend.list.VascBackendListRequest;

/**
* Does simple timer logging for the backend actions. 
* 
* @author Willem Cazander
* @version 1.0 Nov 19, 2009
*/
public class VascBackendProxyTimerLogger<DATA_PK extends Serializable,DATA_RECORD extends Serializable> extends AbstractVascBackendProxy<DATA_PK,DATA_RECORD> {
	
	protected Logger logger = null;
	protected Level logLevel = Level.INFO;
	
	public VascBackendProxyTimerLogger() {
		logger = Logger.getLogger(VascBackendProxyTimerLogger.class.getName());
	}
	
	/**
	 * @see java.lang.Object#clone()
	 */
	@Override
	public VascBackendProxy<DATA_PK,DATA_RECORD> clone() throws CloneNotSupportedException {
		VascBackendProxyTimerLogger<DATA_PK,DATA_RECORD> result = new VascBackendProxyTimerLogger<>();
		result.logLevel=logLevel;
		return result;
	}
	
	/**
	 * @see net.forwardfire.vasc.backend.crud.VascBackendCrud#execute(VascBackendListRequest state)
	 */
	@Override
	public VascBackendListResult<Serializable> execute(VascBackendListRequest state) throws VascBackendException {
		long t1 = System.currentTimeMillis();
		try {
			return backend.execute(state);
		} finally {
			long t2 = System.currentTimeMillis();
			logger.log(logLevel,"vasc-execute backend: "+getId()+" in "+(t2-t1)+" ms");
		}
	}
	
	/**
	 * @see net.forwardfire.vasc.backend.proxy.AbstractVascBackendProxy#doRecordMoveDownById(net.forwardfire.vasc.backend.list.VascBackendListRequest, java.lang.Object)
	 */
	@Override
	public long doRecordMoveDownById(VascBackendListRequest request,Serializable recordPK) throws VascBackendException {
		long t1 = System.currentTimeMillis();
		try {
			return backend.doRecordMoveDownById(request, recordPK);
		} finally {
			long t2 = System.currentTimeMillis();
			logger.log(logLevel,"vasc-doRecordMoveDownById backend: "+getId()+" in "+(t2-t1)+" ms");
		}
	}
	
	/**
	 * @see net.forwardfire.vasc.backend.proxy.AbstractVascBackendProxy#doRecordMoveUpById(net.forwardfire.vasc.backend.list.VascBackendListRequest, java.lang.Object)
	 */
	@Override
	public long doRecordMoveUpById(VascBackendListRequest request,Serializable recordPK) throws VascBackendException {
		long t1 = System.currentTimeMillis();
		try {
			return backend.doRecordMoveUpById(request, recordPK);
		} finally {
			long t2 = System.currentTimeMillis();
			logger.log(logLevel,"vasc-doRecordMoveUpById backend: "+getId()+" in "+(t2-t1)+" ms");
		}
	}
	
	/**
	 * @return the logLevel
	 */
	public Level getLogLevel() {
		return logLevel;
	}
	
	/**
	 * @param logLevel the logLevel to set
	 */
	public void setLogLevel(Level logLevel) {
		this.logLevel = logLevel;
	}
}