/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.core;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import net.forwardfire.vasc.backend.VascBackendAccessDataRecord;
import net.forwardfire.vasc.core.base.AbstractVascBaseIdRoleCrudOrderMetaLocal;
import net.forwardfire.vasc.validators.VascValidator;

/**
 * AbstractVascEntryFieldLocal
 * 
 * @author Willem Cazander
 * @version 1.0 Jun 2, 2012
 */
@SuppressWarnings("serial")
abstract public class AbstractVascEntryFieldLocal extends AbstractVascBaseIdRoleCrudOrderMetaLocal implements VascEntryFieldLocal {

	private VascEntry vascEntry = null;
	private String backendName = null;
	private String displayName = null;
	private VascEntryFieldType vascEntryFieldType = null;
	private VascBackendAccessDataRecord<Serializable,Serializable> vascEntryFieldValue = null;
	private List<VascValidator> vascValidators = null;
	private Serializable defaultValue = null;
	private Integer sizeList = null;
	private Integer sizeEdit = null;
	private String styleList = null;
	private String styleEdit = null;
	private String choices = null;
	private Boolean choicesAsRadio = null;
	private Boolean optional = null;
	private Boolean editBlank = null;
	private Boolean sortable = null;
	private Boolean sumable = null;
	private Boolean graphable = null;
	
	public AbstractVascEntryFieldLocal() {
		vascValidators = new ArrayList<VascValidator>(5);
	}

	@Override
	public VascEntryFieldLocal clone() throws CloneNotSupportedException {
		Object clone = cloneCreate();
		cloneFields(clone);
		return (VascEntryFieldLocal)clone;
	}
	

	@Override
	public void cloneFields(Object cloneObject) throws CloneNotSupportedException {
		super.cloneFields(cloneObject);
		VascEntryFieldLocal clone = (VascEntryFieldLocal)cloneObject;
		
		// TODO: rm
		// vascEntry and 
		// this polls full backend..
		//result.vascEntryFieldValue=vascEntryFieldValue;
		
		
		clone.setBackendName(getBackendName());
		clone.setDisplayName(getDisplayName());
		clone.setVascEntryFieldType(getVascEntryFieldType());
		for (VascValidator v:getVascValidators()) {
			clone.addVascValidator(v.clone());
		}
		clone.setDefaultValue(getDefaultValue());
		clone.setSizeList(getSizeList());
		clone.setSizeEdit(getSizeEdit());
		clone.setStyleList(getStyleList());
		clone.setStyleEdit(getStyleEdit());
		clone.setChoices(getChoices());
		clone.setChoicesAsRadio(getChoicesAsRadio());
		clone.setOptional(getOptional());
		clone.setEditBlank(getEditBlank());
		clone.setSortable(getSortable());
		clone.setSumable(getSumable());
		clone.setGraphable(getGraphable());
	}
	
	@Override
	public VascEntry getVascEntry() {
		return vascEntry;
	}
	
	@Override
	public void setVascEntry(VascEntry vascEntry) {
		this.vascEntry=vascEntry;
	}
	
	/**
	 * @return the vascEntryFieldType
	 */
	@Override
	public VascEntryFieldType getVascEntryFieldType() {
		return vascEntryFieldType;
	}
	
	/**
	 * @param vascEntryFieldType the vascEntryFieldType to set
	 */
	@Override
	public void setVascEntryFieldType(VascEntryFieldType vascEntryFieldType) {
		this.vascEntryFieldType = vascEntryFieldType;
	}
	
	/**
	 * @return the backendName
	 */
	@Override
	public String getBackendName() {
		return backendName;
	}
	
	/**
	 * @param backendName the backendName to set
	 */
	@Override
	public void setBackendName(String backendName) {
		this.backendName = backendName;
	}
	
	/**
	 * @return the vascEntryFieldValue
	 */
	@Override
	public VascBackendAccessDataRecord<Serializable,Serializable> getVascEntryFieldValue() {
		return vascEntryFieldValue;
	}
	
	/**
	 * @param vascEntryFieldValue the vascEntryFieldValue to set
	 */
	@Override
	public void setVascEntryFieldValue(VascBackendAccessDataRecord<Serializable,Serializable> vascEntryFieldValue) {
		this.vascEntryFieldValue = vascEntryFieldValue;
	}
		
	/**
	 * @return the vascValidators
	 */
	@Override
	public List<VascValidator> getVascValidators() {
		return vascValidators;
	}
	
	/**
	 * @param vascValidators the vascValidators to add
	 */
	@Override
	public void addVascValidator(VascValidator vascValidator) {
		this.vascValidators.add(vascValidator);
	}

	/**
	 * @param vascValidators the vascValidators to remove
	 */
	@Override
	public void removeVascValidator(VascValidator vascValidator) {
		this.vascValidators.remove(vascValidator);
	}

	
	/**
	 * @return the defaultValue
	 */
	@Override
	public Serializable getDefaultValue() {
		return defaultValue;
	}
	
	/**
	 * @param defaultValue the defaultValue to set
	 */
	@Override
	public void setDefaultValue(Serializable defaultValue) {
		this.defaultValue = defaultValue;
	}
	
	/**
	 * @return the sizeList
	 */
	@Override
	public Integer getSizeList() {
		return sizeList;
	}
	
	/**
	 * @param sizeList the sizeList to set
	 */
	@Override
	public void setSizeList(Integer sizeList) {
		this.sizeList = sizeList;
	}
	
	/**
	 * @return the sizeEdit
	 */
	@Override
	public Integer getSizeEdit() {
		return sizeEdit;
	}
	
	/**
	 * @param sizeEdit the sizeEdit to set
	 */
	@Override
	public void setSizeEdit(Integer sizeEdit) {
		this.sizeEdit = sizeEdit;
	}
	
	/**
	 * @return the styleList
	 */
	@Override
	public String getStyleList() {
		return styleList;
	}
	
	/**
	 * @param styleList the styleList to set
	 */
	@Override
	public void setStyleList(String styleList) {
		this.styleList = styleList;
	}
	
	/**
	 * @return the styleEdit
	 */
	@Override
	public String getStyleEdit() {
		return styleEdit;
	}
	
	/**
	 * @param styleEdit the styleEdit to set
	 */
	@Override
	public void setStyleEdit(String styleEdit) {
		this.styleEdit = styleEdit;
	}
	
	/**
	 * @return the choices
	 */
	@Override
	public String getChoices() {
		return choices;
	}
	
	/**
	 * @param choices the choices to set
	 */
	@Override
	public void setChoices(String choices) {
		this.choices = choices;
	}

	
	/**
	 * @return the optional
	 */
	@Override
	public Boolean getOptional() {
		return optional;
	}
	
	/**
	 * @param optional the optional to set
	 */
	@Override
	public void setOptional(Boolean optional) {
		this.optional = optional;
	}
	
	/**
	 * @return the choicesAsRadio
	 */
	@Override
	public Boolean getChoicesAsRadio() {
		return choicesAsRadio;
	}

	/**
	 * @param choicesAsRadio the choicesAsRadio to set
	 */
	@Override
	public void setChoicesAsRadio(Boolean choicesAsRadio) {
		this.choicesAsRadio = choicesAsRadio;
	}

	/**
	 * @return the editBlank
	 */
	@Override
	public Boolean getEditBlank() {
		return editBlank;
	}

	/**
	 * @param editBlank the editBlank to set
	 */
	@Override
	public void setEditBlank(Boolean editBlank) {
		this.editBlank = editBlank;
	}

	/**
	 * @return the displayName
	 */
	@Override
	public String getDisplayName() {
		return displayName;
	}

	/**
	 * @param displayName the displayName to set
	 */
	@Override
	public void setDisplayName(String displayName) {
		this.displayName = displayName;
	}

	/**
	 * @return the sortable
	 */
	@Override
	public Boolean getSortable() {
		return sortable;
	}

	/**
	 * @param sortable the sortable to set
	 */
	@Override
	public void setSortable(Boolean sortable) {
		this.sortable = sortable;
	}

	/**
	 * @return the sumable
	 */
	@Override
	public Boolean getSumable() {
		return sumable;
	}

	/**
	 * @param sumable the sumable to set
	 */
	@Override
	public void setSumable(Boolean sumable) {
		this.sumable = sumable;
	}

	/**
	 * @return the graphable
	 */
	@Override
	public Boolean getGraphable() {
		return graphable;
	}

	/**
	 * @param graphable the graphable to set
	 */
	@Override
	public void setGraphable(Boolean graphable) {
		this.graphable = graphable;
	}
}
