/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.core;

import java.util.ArrayList;
import java.util.List;

import net.forwardfire.vasc.core.base.AbstractVascBaseIdRoleCrudOrderMetaLocal;

/**
 * AbstractVascEntryFieldSetLocal
 * 
 * @author Willem Cazander
 * @version 1.0 Jun 2, 2012
 */
@SuppressWarnings("serial")
abstract public class AbstractVascEntryFieldSetLocal extends AbstractVascBaseIdRoleCrudOrderMetaLocal implements VascEntryFieldSetLocal {

	private String styleList = null;
	private String styleEdit = null;
	private Boolean collapsed = false;	
	private List<String> vascEntryFieldIds = null;

	public AbstractVascEntryFieldSetLocal() {
		vascEntryFieldIds = new ArrayList<String>(10);
	}
	
	@Override
	public VascEntryFieldSetLocal clone() throws CloneNotSupportedException {
		Object clone = cloneCreate();
		cloneFields(clone);
		return (VascEntryFieldSetLocal)clone;
	}

	@Override
	public void cloneFields(Object cloneObject) throws CloneNotSupportedException {
		super.cloneFields(cloneObject);
		VascEntryFieldSetLocal clone = (VascEntryFieldSetLocal)cloneObject;
		clone.setStyleList(getStyleList());
		clone.setStyleEdit(getStyleEdit());
		clone.setCollapsed(getCollapsed());
		for (String id:getVascEntryFieldIds()) {
			clone.addVascEntryFieldId(id);
		}
	}
	
	/**
	 * @return the styleList
	 */
	@Override
	public String getStyleList() {
		return styleList;
	}

	/**
	 * @param styleList the styleList to set
	 */
	@Override
	public void setStyleList(String styleList) {
		this.styleList = styleList;
	}

	/**
	 * @return the styleEdit
	 */
	@Override
	public String getStyleEdit() {
		return styleEdit;
	}

	/**
	 * @param styleEdit the styleEdit to set
	 */
	@Override
	public void setStyleEdit(String styleEdit) {
		this.styleEdit = styleEdit;
	}

	/**
	 * @return the collapsed
	 */
	@Override
	public Boolean getCollapsed() {
		return collapsed;
	}

	/**
	 * @param collapsed the collapsed to set
	 */
	@Override
	public void setCollapsed(Boolean collapsed) {
		this.collapsed = collapsed;
	}
	
	/**
	 * @return the vascEntryFieldIds
	 */
	@Override
	public List<String> getVascEntryFieldIds() {
		return vascEntryFieldIds;
	}

	/**
	 * @param vascEntryFieldIds the vascEntryFieldIds to set
	 */
	@Override
	public void addVascEntryFieldId(String vascEntryFieldId) {
		vascEntryFieldIds.add(vascEntryFieldId);
	}
	
	/**
	 * @param vascEntryFieldIds the vascEntryFieldIds to set
	 */
	@Override
	public void removeVascEntryFieldId(String vascEntryFieldId) {
		vascEntryFieldIds.remove(vascEntryFieldId);
	}
}
