/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.core;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.x4o.xml.conv.ObjectConverter;

import net.forwardfire.vasc.core.base.AbstractVascBaseIdLocal;
import net.forwardfire.vasc.core.ui.VascUIComponent;
import net.forwardfire.vasc.core.ui.VascValueModel;
import net.forwardfire.vasc.core.VascException;
import net.forwardfire.vasc.validators.VascValidator;

/**
 * AbstractVascEntryFieldTypeLocal
 * 
 * @author Willem Cazander
 * @version 1.0 Aug 2, 2007
 */
@SuppressWarnings("serial")
abstract public class AbstractVascEntryFieldTypeLocal extends AbstractVascBaseIdLocal implements VascEntryFieldTypeLocal {

	private Class<?> autoDetectClass = null;
	private List<VascValidator> vascValidators = null;
	private Map<String,String> properties = null;
	private ObjectConverter objectConverter = null;
	
	private Object dataObject = null;
	private String uiComponentId = null;
	private String inputMask = null;
	
	public AbstractVascEntryFieldTypeLocal() {
		vascValidators = new ArrayList<VascValidator>(4);
		properties = new HashMap<String,String>();
	}
	
	@Override
	public VascEntryFieldTypeLocal clone() throws CloneNotSupportedException {
		Object clone = super.cloneCreate();
		cloneFields(clone);
		return (VascEntryFieldTypeLocal)clone;
	}

	@Override
	public void cloneFields(Object cloneObject) throws CloneNotSupportedException {
		super.cloneFields(cloneObject);
		VascEntryFieldTypeLocal clone = (VascEntryFieldTypeLocal)cloneObject;
		clone.setAutoDetectClass(getAutoDetectClass());
		for (VascValidator v:getVascValidators()) {
			clone.addVascValidator(v.clone());
		}
		for (String key:getPropertyNames()) {
			clone.setProperty(key, getProperty(key));
		}
		if (getObjectConverter()!=null) {
			clone.setObjectConverter(getObjectConverter().clone()); // use clone
		}
		clone.setDataObject(getDataObject());
		clone.setUIComponentId(getUIComponentId());
		clone.setInputMask(getInputMask());
	}
	
	/**
	 * @see net.forwardfire.vasc.core.VascEntryFieldType#getProperty(java.lang.String)
	 */
	@Override
	public String getProperty(String name) {
		return properties.get(name);
	}

	/**
	 * @see net.forwardfire.vasc.core.VascEntryFieldType#setProperty(java.lang.String, java.lang.String)
	 */
	@Override
	public void setProperty(String name, String value) {
		properties.put(name, value);
	}
	
	/**
	 * @see net.forwardfire.vasc.core.VascEntryFieldType#getPropertyNames()
	 */
	@Override
	public List<String> getPropertyNames() {
		return new ArrayList<String>(properties.keySet());
	}

	/**
	 * @return the dataObject
	 */
	@Override
	public Object getDataObject() {
		return dataObject;
	}

	/**
	 * @param dataObject the dataObject to set
	 */
	@Override
	public void setDataObject(Object dataObject) {
		this.dataObject = dataObject;
	}

	/**
	 * @see net.forwardfire.vasc.core.VascEntryFieldType#getVascValidators()
	 */
	@Override
	public List<VascValidator> getVascValidators() {
		return vascValidators;
	}

	/**
	 * @see net.forwardfire.vasc.core.VascEntryFieldType#addVascValidator(net.forwardfire.vasc.validators.VascValidator)
	 */
	@Override
	public void addVascValidator(VascValidator vascValidator) {
		vascValidators.add(vascValidator);
	}
	
	/**
	 * @see net.forwardfire.vasc.core.VascEntryFieldType#removeVascValidator(net.forwardfire.vasc.validators.VascValidator)
	 */
	@Override
	public void removeVascValidator(VascValidator vascValidator) {
		vascValidators.remove(vascValidator);
	}

	/**
	 * @see net.forwardfire.vasc.core.VascEntryFieldType#getAutoDetectClass()
	 */
	@Override
	public Class<?> getAutoDetectClass() {
		return autoDetectClass;
	}

	/**
	 * @see net.forwardfire.vasc.core.VascEntryFieldType#setAutoDetectClass(java.lang.Class)
	 */
	@Override
	public void setAutoDetectClass(Class<?> classObject) {
		autoDetectClass=classObject;
	}

	/**
	 * @see net.forwardfire.vasc.core.VascEntryFieldType#getInputMask()
	 */
	@Override
	public String getInputMask() {
		return inputMask;
	}

	/**
	 * @see net.forwardfire.vasc.core.VascEntryFieldType#setInputMask(java.lang.String)
	 */
	@Override
	public void setInputMask(String inputMask) {
		this.inputMask=inputMask;
	}

	/**
	 * @see net.forwardfire.vasc.core.VascEntryFieldType#getUIComponentId()
	 */
	@Override
	public String getUIComponentId() {
		return uiComponentId;
	}

	/**
	 * @see net.forwardfire.vasc.core.VascEntryFieldType#setUIComponentId(java.lang.String)
	 */
	@Override
	public void setUIComponentId(String uiComponentId) {
		this.uiComponentId=uiComponentId;
	}
	
	/**
	 * @see net.forwardfire.vasc.core.VascEntryFieldType#getUIComponentCount()
	 */
	@Override
	public int getUIComponentCount(VascEntryField entryField) throws VascException {
		return 1;
	}

	/**
	 * @see net.forwardfire.vasc.core.VascEntryFieldType#provideEditorUIComponent(int)
	 */
	@Override
	public VascUIComponent provideEditorUIComponent(int index,VascEntryField entryField) throws VascException {
    	ClassLoader cl = Thread.currentThread().getContextClassLoader();
    	if (cl == null) {
    		cl = entryField.getClass().getClassLoader(); // fallback
    	}
    	String compId = getUIComponentId();
    	if (compId==null) {
    		compId = VascUIComponent.VASC_TEXT;
    	}
    	return entryField.getVascEntry().getVascFrontendController().getVascUIComponent(compId);
	}
	
	/**
	 * @see net.forwardfire.vasc.core.VascEntryFieldType#provideLabelUIComponent(int)
	 */
	@Override
	public VascUIComponent provideLabelUIComponent(int index,VascEntryField entryField) throws VascException {
    	return entryField.getVascEntry().getVascFrontendController().getVascUIComponent(VascUIComponent.VASC_LABEL);
	}

	/**
	 * @see net.forwardfire.vasc.core.VascEntryFieldType#provideEditorVascValueModel()
	 */
	@Override
	public VascValueModel provideEditorVascValueModel(int index,VascEntryField entryField) throws VascException {
		if (index>0) {
			throw new IllegalArgumentException("You have to override provideEditorVascValueModel if multi editor support is needed");
		}
		VascValueModel model = new VascValueModel();
		return model;
	}

	/**
	 * @return the objectConverter
	 */
	@Override
	public ObjectConverter getObjectConverter() {
		return objectConverter;
	}

	/**
	 * @param objectConverter the objectConverter to set
	 */
	@Override
	public void setObjectConverter(ObjectConverter objectConverter) {
		this.objectConverter = objectConverter;
	}
}