/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.core;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import net.forwardfire.vasc.core.base.AbstractVascBaseIdRoleViewOrderMetaLocal;

/**
 * AbstractVascEntryLinkLocal
 * 
 * @author Willem Cazander
 * @version 1.0 Jun 2, 2012
 */
@SuppressWarnings("serial")
abstract public class AbstractVascEntryLinkLocal extends AbstractVascBaseIdRoleViewOrderMetaLocal implements VascEntryLinkLocal {

	private String vascEntryId = null;
	private Map<String,String> entryParameterFieldIds = new HashMap<String,String>(3);
	private Map<String,String> entryCreateFieldValues = new HashMap<String,String>(3);
	private VascEntryLinkType vascEntryLinkType = null;
	private String doActionId = null;
	
	
	@Override
	public VascEntryLinkLocal clone() throws CloneNotSupportedException {
		Object clone = cloneCreate();
		cloneFields(clone);
		return (VascEntryLinkLocal)clone;
	}

	@Override
	public void cloneFields(Object cloneObject) throws CloneNotSupportedException {
		super.cloneFields(cloneObject);
		VascEntryLinkLocal clone = (VascEntryLinkLocal)cloneObject;
		clone.setVascEntryId(getVascEntryId());
		clone.setDoActionId(getDoActionId());
		clone.setVascEntryLinkType(getVascEntryLinkType());
		for (String key:getEntryParameterFieldIdKeys()) {
			clone.addEntryParameterFieldId(key, getEntryParameterFieldId(key));
		}
		for (String key:getEntryCreateFieldValueKeys()) {
			clone.addEntryCreateFieldValue(key, getEntryCreateFieldValue(key));
		}
	}
	
	@Override
	public String getEntryParameterFieldId(String parameterName) {
		return entryParameterFieldIds.get(parameterName);
	}
	
	@Override
	public void addEntryParameterFieldId(String parameterName,String valueFieldId) {
		entryParameterFieldIds.put(parameterName, valueFieldId);
	}
	
	@Override
	public List<String> getEntryParameterFieldIdKeys() {
		return new ArrayList<String>(entryParameterFieldIds.keySet());
	}
	
	@Override
	public String getEntryCreateFieldValue(String valueFieldId) {
		return entryCreateFieldValues.get(valueFieldId);
	}
	
	@Override
	public void addEntryCreateFieldValue(String valueFieldId,String selectedFieldId) {
		entryCreateFieldValues.put(valueFieldId, selectedFieldId);
	}
	
	@Override
	public List<String> getEntryCreateFieldValueKeys() {
		return new ArrayList<String>(entryCreateFieldValues.keySet());
	}
	
	/**
	 * @return the vascEntryId
	 */
	@Override
	public String getVascEntryId() {
		return vascEntryId;
	}

	/**
	 * @param vascEntryId the vascEntryId to set
	 */
	@Override
	public void setVascEntryId(String vascEntryId) {
		this.vascEntryId = vascEntryId;
	}

	/**
	 * @return the vascLinkEntryType
	 */
	@Override
	public VascEntryLinkType getVascEntryLinkType() {
		return vascEntryLinkType;
	}

	/**
	 * @param vascLinkEntryType the vascLinkEntryType to set
	 */
	@Override
	public void setVascEntryLinkType(VascEntryLinkType vascEntryLinkType) {
		this.vascEntryLinkType = vascEntryLinkType;
	}

	/**
	 * @return the doActionId
	 */
	@Override
	public String getDoActionId() {
		return doActionId;
	}

	/**
	 * @param doActionId the doActionId to set
	 */
	@Override
	public void setDoActionId(String doActionId) {
		this.doActionId = doActionId;
	}
}
