/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.core;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import net.forwardfire.vasc.backend.proxy.VascProxyFilter;
import net.forwardfire.vasc.core.actions.ColumnVascAction;
import net.forwardfire.vasc.core.actions.ColumnVascActionLocal;
import net.forwardfire.vasc.core.actions.GlobalVascAction;
import net.forwardfire.vasc.core.actions.GlobalVascActionLocal;
import net.forwardfire.vasc.core.actions.RowVascAction;
import net.forwardfire.vasc.core.actions.RowVascActionLocal;
import net.forwardfire.vasc.core.base.AbstractVascBaseIdRoleCrudLocal;
import net.forwardfire.vasc.core.entry.VascEntryFieldEventChannel;
import net.forwardfire.vasc.frontend.VascFrontendController;

/**
 * AbstractVascEntryLocal
 * 
 * @author Willem Cazander
 * @version 1.0 Jun 2, 2012
 */
@SuppressWarnings("serial")
abstract public class AbstractVascEntryLocal extends AbstractVascBaseIdRoleCrudLocal implements VascEntryLocal {

	private String name = null;
	private String helpId = null;
	private String image = null;
	
	private String listDescription = null;
	private String listImage = null;
	private String editDescription = null;
	private String editImage = null;
	private String deleteDescription = null;
	private String deleteImage = null;
	private String createDescription = null;
	private String createImage = null;
	
	private String primaryKeyFieldId = null;
	private String displayNameFieldId = null;
	
	private Boolean vascDisplayOnly = null; // todo rename
	private Boolean delete = null;
	private String rolesDelete = null;
	private VascEntryAccessType accessType = null;
	
	private List<VascEntryFieldLocal> vascFields = null;
	private List<RowVascActionLocal> rowActions = null;
	private List<ColumnVascActionLocal> columnActions = null;
	private List<GlobalVascActionLocal> globalActions = null;
	private List<GlobalVascActionLocal> exportActions = null;
	private List<VascEntryFieldSetLocal> vascEntryFieldSets = null;
	private List<VascEntryLinkLocal> vascEntryLinks = null;
	
	private Map<String,Object> entryParameters = null;
	private VascEntryFieldEventChannel vascEntryFieldEventChannel = null;
	private Map<String,List<String>>  eventEntryFrontendEventListeners = null;
	private List<String>  eventEntryBackendEventListeners = null;
	private List<VascEntryListOptionLocal> vascEntryListOptions = null;
	private List<VascProxyFilter> backendFilters = null;
	private Map<String,List<String>>  eventEntryFrontendActions = null;
	
	private String backendId = null;
	private String vascGroupId = null;
	private VascFrontendController vascFrontendData = null;
	
	public AbstractVascEntryLocal() {
		vascFields = new ArrayList<VascEntryFieldLocal>(20);
		
		rowActions = new ArrayList<RowVascActionLocal>(10);
		columnActions = new ArrayList<ColumnVascActionLocal>(5);
		globalActions = new ArrayList<GlobalVascActionLocal>(5);
		exportActions = new ArrayList<GlobalVascActionLocal>(10);
		
		vascEntryFieldSets = new ArrayList<VascEntryFieldSetLocal>(10);
		vascEntryLinks = new ArrayList<VascEntryLinkLocal>(10);
		entryParameters = new HashMap<String,Object>(10);
		
		eventEntryFrontendActions = new HashMap<String,List<String>>(10);
		eventEntryFrontendEventListeners = new HashMap<String,List<String>>(10);
		eventEntryBackendEventListeners = new ArrayList<String>(10);
		vascEntryListOptions = new ArrayList<VascEntryListOptionLocal>(10);
		backendFilters = new ArrayList<VascProxyFilter>(3);
	}
	
	@Override
	public VascEntryLocal clone() throws CloneNotSupportedException {
		Object clone = cloneCreate();
		cloneFields(clone);
		return (VascEntryLocal)clone;
	}

	@Override
	public void cloneFields(Object cloneObject) throws CloneNotSupportedException {
		super.cloneFields(cloneObject);
		VascEntryLocal clone = (VascEntryLocal)cloneObject;
		clone.setName(getName());
		clone.setHelpId(getHelpId());
		clone.setImage(getImage());
		clone.setListDescription(getListDescription());
		clone.setListImage(getListImage());
		clone.setEditDescription(getEditDescription());
		clone.setEditImage(getEditImage());
		clone.setCreateDescription(getCreateDescription());
		clone.setCreateImage(getCreateImage());
		clone.setDeleteDescription(getDeleteDescription());
		clone.setDeleteImage(getDeleteImage());
		
		clone.setPrimaryKeyFieldId(getPrimaryKeyFieldId());
		clone.setDisplayNameFieldId(getDisplayNameFieldId());
		
		clone.setDelete(getDelete());
		clone.setRolesDelete(getRolesDelete());
		clone.setAccessType(getAccessType());
		
		for (VascEntryFieldLocal field:getVascEntryFieldsLocal()) {
			VascEntryFieldLocal fieldClone = field.clone();
			fieldClone.setVascEntry(clone); // TODO rm me
			clone.addVascEntryField(fieldClone);
		}
		for (RowVascActionLocal a:getRowActionsLocal()) {			clone.addRowAction(a.clone());				}
		for (ColumnVascActionLocal a:getColumnActionsLocal()) {		clone.addColumnAction(a.clone());			}
		for (GlobalVascActionLocal a:getGlobalActionsLocal()) {		clone.addGlobalAction(a.clone());			}
		for (GlobalVascActionLocal a:getExportActionsLocal()) {		clone.addExportAction(a.clone());			}
		for (VascEntryFieldSetLocal set:getVascEntryFieldSetsLocal()) {
			clone.addVascEntryFieldSet(set.clone());
		}
		for (VascEntryLinkLocal link:getVascEntryLinksLocal()) {
			clone.addVascEntryLink(link.clone());
		}
		for (String key:getEntryParameterKeys()) {
			clone.setEntryParameter(key, getEntryParameter(key));
		}
		// TODO: vascEntryFieldEventChannel
		
		for (String key:eventEntryFrontendEventListeners.keySet()) {
			List<String> list = eventEntryFrontendEventListeners.get(key);
			if (list==null) {
				continue;
			}
			for (String listener:list) {
				clone.addVascEntryFrontendEventListener(listener, key);
			}
		}
		for (String key:eventEntryBackendEventListeners) {
			clone.addVascEntryBackendEventListener(key);
		}
		for (VascEntryListOptionLocal option:vascEntryListOptions) {
			VascEntryListOptionLocal optionClone = option.clone();
			optionClone.setVascEntry(clone);
			clone.addVascEntryListOption(optionClone);
		}
		for (VascProxyFilter filter:backendFilters) {
			clone.addVascBackendFilter(filter.clone());
		}
		for (String key:eventEntryFrontendActions.keySet()) {
			List<String> list = eventEntryFrontendActions.get(key);
			if (list==null) {
				continue;
			}
			for (String listener:list) {
				clone.addVascEntryFrontendAction(listener, key);
			}
		}
		clone.setBackendId(getBackendId());
		clone.setVascGroupId(getVascGroupId());
	}
	
	/**
	 * @return the name
	 */
	@Override
	public String getName() {
		return name;
	}

	/**
	 * @param name the name to set
	 */
	@Override
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * @return the helpId
	 */
	@Override
	public String getHelpId() {
		return helpId;
	}
	
	/**
	 * @param helpId the helpId to set
	 */
	@Override
	public void setHelpId(String helpId) {
		this.helpId = helpId;
	}
	
	/**
	 * @return the image
	 */
	@Override
	public String getImage() {
		return image;
	}
	
	/**
	 * @param image the image to set
	 */
	@Override
	public void setImage(String image) {
		this.image = image;
	}
		
	/**
	 * @return the listDescription
	 */
	@Override
	public String getListDescription() {
		return listDescription;
	}

	/**
	 * @param listDescription the listDescription to set
	 */
	@Override
	public void setListDescription(String listDescription) {
		this.listDescription = listDescription;
	}

	/**
	 * @return the listImage
	 */
	@Override
	public String getListImage() {
		return listImage;
	}

	/**
	 * @param listImage the listImage to set
	 */
	@Override
	public void setListImage(String listImage) {
		this.listImage = listImage;
	}

	/**
	 * @return the editDescription
	 */
	@Override
	public String getEditDescription() {
		return editDescription;
	}

	/**
	 * @param editDescription the editDescription to set
	 */
	@Override
	public void setEditDescription(String editDescription) {
		this.editDescription = editDescription;
	}

	/**
	 * @return the editImage
	 */
	@Override
	public String getEditImage() {
		return editImage;
	}

	/**
	 * @param editImage the editImage to set
	 */
	@Override
	public void setEditImage(String editImage) {
		this.editImage = editImage;
	}

	/**
	 * @return the deleteDescription
	 */
	@Override
	public String getDeleteDescription() {
		return deleteDescription;
	}

	/**
	 * @param deleteDescription the deleteDescription to set
	 */
	@Override
	public void setDeleteDescription(String deleteDescription) {
		this.deleteDescription = deleteDescription;
	}

	/**
	 * @return the deleteImage
	 */
	@Override
	public String getDeleteImage() {
		return deleteImage;
	}

	/**
	 * @param deleteImage the deleteImage to set
	 */
	@Override
	public void setDeleteImage(String deleteImage) {
		this.deleteImage = deleteImage;
	}

	/**
	 * @return the createDescription
	 */
	@Override
	public String getCreateDescription() {
		return createDescription;
	}

	/**
	 * @param createDescription the createDescription to set
	 */
	@Override
	public void setCreateDescription(String createDescription) {
		this.createDescription = createDescription;
	}

	/**
	 * @return the createImage
	 */
	@Override
	public String getCreateImage() {
		return createImage;
	}

	/**
	 * @param createImage the createImage to set
	 */
	@Override
	public void setCreateImage(String createImage) {
		this.createImage = createImage;
	}

	/**
	 * @return the primaryKeyFieldId
	 */
	@Override
	public String getPrimaryKeyFieldId() {
		return primaryKeyFieldId;
	}
	
	/**
	 * @param primaryKeyFieldId the primaryKeyFieldId to set
	 */
	@Override
	public void setPrimaryKeyFieldId(String primaryKeyFieldId) {
		this.primaryKeyFieldId = primaryKeyFieldId;
	}
	
	/**
	 * @return the displayNameFieldId
	 */
	@Override
	public String getDisplayNameFieldId() {
		return displayNameFieldId;
	}
	
	/**
	 * @param displayNameFieldId the displayNameFieldId to set
	 */
	@Override
	public void setDisplayNameFieldId(String displayNameFieldId) {
		this.displayNameFieldId = displayNameFieldId;
	}

	/**
	 * @return the delete
	 */
	@Override
	public Boolean getDelete() {
		return delete;
	}
	
	/**
	 * @param delete the delete to set
	 */
	@Override
	public void setDelete(Boolean delete) {
		this.delete = delete;
	}
	
	/**
	 * @return the rolesDelete
	 */
	@Override
	public String getRolesDelete() {
		return rolesDelete;
	}
	
	/**
	 * @param rolesDelete the rolesDelete to set
	 */
	@Override
	public void setRolesDelete(String rolesDelete) {
		this.rolesDelete = rolesDelete;
	}
	
	/**
	 * @return the vascFields
	 */
	@Override
	public Collection<VascEntryField> getVascEntryFields() {
		return new ArrayList<VascEntryField>(vascFields);
	}
	
	/**
	 * @return the vascFields
	 */
	@Override
	public Collection<VascEntryFieldLocal> getVascEntryFieldsLocal() {
		return vascFields;
	}
	
	/**
	 * @param vascField the vascFields to add
	 */
	@Override
	public void addVascEntryField(VascEntryFieldLocal vascField) {
		vascFields.add(vascField);
	}

	/**
	 * @param vascField the vascFields to remove
	 */
	@Override
	public void removeVascEntryField(VascEntryFieldLocal vascField) {
		vascFields.remove(vascField);
	}
	
	/**
	 * @see net.forwardfire.vasc.core.VascEntry#getVascEntryFieldById(java.lang.String)
	 */
	@Override
	public VascEntryField getVascEntryFieldById(String id) {
		for (VascEntryField field:vascFields) {
			if (id.equals(field.getId())) {
				return field;
			}
		}
		return null;
	}
		
	/**
	 * @return the rowActions
	 */
	@Override
	public Collection<RowVascAction> getRowActions() {
		return new ArrayList<RowVascAction>(rowActions);
	}
	
	/**
	 * @return the RowVascAction
	 */
	@Override
	public RowVascAction getRowActionById(String actionId) {
		for (RowVascAction a:rowActions) {
			if (a.getId().equals(actionId)) {
				return a;
			}
		}
		return null;
	}
	
	/**
	 * @param rowAction the rowAction to add
	 */
	@Override
	public void addRowAction(RowVascActionLocal rowAction) {
		rowActions.add(rowAction);
	}

	/**
	 * @param rowAction the rowAction to remove
	 */
	@Override
	public void removeRowAction(RowVascActionLocal rowAction) {
		rowActions.remove(rowAction);
	}
	
	/**
	 * @return the rowActions
	 */
	@Override
	public Collection<RowVascActionLocal> getRowActionsLocal() {
		return rowActions;
	}
	
	/**
	 * @return the columnActions
	 */
	@Override
	public Collection<ColumnVascAction> getColumnActions() {
		return new ArrayList<ColumnVascAction>(columnActions);
	}
	
	/**
	 * @return the ColumnVascAction
	 */
	@Override
	public ColumnVascAction getColumnActionById(String actionId) {
		for (ColumnVascAction a:columnActions) {
			if (a.getId().equals(actionId)) {
				return a;
			}
		}
		return null;
	}
	
	/**
	 * @param columnAction the columnActions to add
	 */
	@Override
	public void addColumnAction(ColumnVascActionLocal columnAction) {
		columnActions.add(columnAction);
	}

	/**
	 * @param columnAction the columnActions to remove
	 */
	@Override
	public void removeColumnAction(ColumnVascActionLocal columnAction) {
		columnActions.remove(columnAction);
	}
	
	/**
	 * @return the columnActions
	 */
	@Override
	public Collection<ColumnVascActionLocal> getColumnActionsLocal() {
		return columnActions;
	}
	
	/**
	 * @return the globalActions
	 */
	@Override
	public Collection<GlobalVascAction> getGlobalActions() {
		return new ArrayList<GlobalVascAction>(globalActions);
	}
	
	/**
	 * @return the GlobalVascAction
	 */
	@Override
	public GlobalVascAction getGlobalActionById(String actionId) {
		for (GlobalVascAction a:globalActions) {
			if (a.getId().equals(actionId)) {
				return a;
			}
		}
		return null;
	}
	
	/**
	 * @param globalAction the globalAction to add
	 */
	@Override
	public void addGlobalAction(GlobalVascActionLocal globalAction) {
		globalActions.add(globalAction);
	}
	
	/**
	 * @param globalAction the globalAction to remove
	 */
	@Override
	public void removeGlobalAction(GlobalVascActionLocal globalAction) {
		globalActions.remove(globalAction);
	}
	
	/**
	 * @return the globalActions
	 */
	@Override
	public Collection<GlobalVascActionLocal> getGlobalActionsLocal() {
		return globalActions;
	}
	
	/**
	 * @return the exportActions
	 */
	@Override
	public Collection<GlobalVascAction> getExportActions() {
		return new ArrayList<GlobalVascAction>(exportActions);
	}
	
	/**
	 * @return the GlobalVascAction exportAction
	 */
	@Override
	public GlobalVascAction getExportActionById(String actionId) {
		for (GlobalVascAction a:exportActions) {
			if (a.getId().equals(actionId)) {
				return a;
			}
		}
		return null;
	}
	
	/**
	 * @param exportAction the exportAction to add
	 */
	@Override
	public void addExportAction(GlobalVascActionLocal exportAction) {
		exportActions.add(exportAction);
	}
	
	/**
	 * @param exportAction the exportAction to remove
	 */
	@Override
	public void removeExportAction(GlobalVascActionLocal exportAction) {
		exportActions.remove(exportAction);
	}
	
	/**
	 * @return the exportActions
	 */
	@Override
	public Collection<GlobalVascActionLocal> getExportActionsLocal() {
		return exportActions;
	}
	
	/**
	 * @return the vascEntryFieldSets
	 */
	@Override
	public Collection<VascEntryFieldSet> getVascEntryFieldSets() {
		return new ArrayList<VascEntryFieldSet>(vascEntryFieldSets);
	}
	
	/**
	 * @return the vascEntryFieldSets
	 */
	@Override
	public Collection<VascEntryFieldSetLocal> getVascEntryFieldSetsLocal() {
		return vascEntryFieldSets;
	}
	
	/**
	 * @return the VascEntryFieldSet
	 */
	@Override
	public VascEntryFieldSet getVascEntryFieldSetById(String fieldSetId) {
		for (VascEntryFieldSet a:vascEntryFieldSets) {
			if (a.getId().equals(fieldSetId)) {
				return a;
			}
		}
		return null;
	}
	
	/**
	 * @param vascEntryFieldSet the vascEntryFieldSet to add
	 */
	@Override
	public void addVascEntryFieldSet(VascEntryFieldSetLocal vascEntryFieldSet) {
		vascEntryFieldSets.add(vascEntryFieldSet);
	}

	/**
	 * @param vascEntryFieldSet the vascEntryFieldSet to add
	 */
	@Override
	public void removeVascEntryFieldSet(VascEntryFieldSetLocal vascEntryFieldSet) {
		vascEntryFieldSets.remove(vascEntryFieldSet);
	}
	
	/**
	 * @return the vascEntryLinks
	 */
	@Override
	public Collection<VascEntryLink> getVascEntryLinks() {
		return new ArrayList<VascEntryLink>(vascEntryLinks);
	}
	
	/**
	 * @return the vascEntryLinks
	 */
	@Override
	public Collection<VascEntryLinkLocal> getVascEntryLinksLocal() {
		return vascEntryLinks;
	}
	
	/**
	 * @return the vascEntryLinks
	 */
	@Override
	public VascEntryLink getVascEntryLinkById(String linkId) {
		for (VascEntryLink a:vascEntryLinks) {
			if (a.getId().equals(linkId)) {
				return a;
			}
		}
		return null;
	}
	
	/**
	 * @param vascEntryLink the vascEntryLink to set
	 */
	@Override
	public void addVascEntryLink(VascEntryLinkLocal vascEntryLink) {
		vascEntryLinks.add(vascEntryLink);
	}

	/**
	 * @param vascEntryLink the vascEntryLink to remove
	 */
	@Override
	public void removeVascEntryLink(VascEntryLinkLocal vascEntryLink) {
		vascEntryLinks.remove(vascEntryLink);
	}

	/**
	 * @see net.forwardfire.vasc.core.VascEntry#getEntryParameter(java.lang.String)
	 */
	@Override
	public Object getEntryParameter(String key) {
		return entryParameters.get(key);
	}

	/**
	 * @see net.forwardfire.vasc.core.VascEntry#getEntryParameterKeys()
	 */
	@Override
	public List<String> getEntryParameterKeys() {
		return new ArrayList<String>(entryParameters.keySet());
	}

	/**
	 * @see net.forwardfire.vasc.core.VascEntry#setEntryParameter(java.lang.String, java.lang.Object)
	 */
	@Override
	public void setEntryParameter(String key, Object value) {
		entryParameters.put(key, value);
	}

	/**
	 * @see net.forwardfire.vasc.core.VascEntry#removeEntryParameter(java.lang.String)
	 */
	@Override
	public void removeEntryParameter(String key) {
		entryParameters.remove(key);
	}

	/**
	 * @return the vascFrontendData
	 */
	@Override
	public VascFrontendController getVascFrontendController() {
		return vascFrontendData;
	}

	/**
	 * @param vascFrontendData the vascFrontendData to set
	 */
	@Override
	public void setVascFrontendController(VascFrontendController vascFrontendData) {
		this.vascFrontendData = vascFrontendData;
	}

	/**
	 * @return the backendId
	 */
	@Override
	public String getBackendId() {
		return backendId;
	}

	/**
	 * @param backendId the backendId to set
	 */
	@Override
	public void setBackendId(String backendId) {
		this.backendId = backendId;
	}
	
	/**
	 * @return the vascEntryFieldEventChannel
	 */
	@Override
	public VascEntryFieldEventChannel getVascEntryFieldEventChannel() {
		return vascEntryFieldEventChannel;
	}
	
	/**
	 * @param vascEntryFieldEventChannel the vascEntryFieldEventChannel to set
	 */
	@Override
	public void setVascEntryFieldEventChannel(VascEntryFieldEventChannel vascEntryFieldEventChannel) {
		this.vascEntryFieldEventChannel = vascEntryFieldEventChannel;
	}

	/**
	 * Added an VascEntryBackendEventListener
	 * @param listener The class of the event listener.
	 */
	@Override
	public void addVascEntryBackendEventListener(String listener) {
		eventEntryBackendEventListeners.add(listener);
	}
	
	/**
	 * Returns the list of VascEntryBackendEventListener
	 * @return
	 */
	@Override
	public List<String> getVascEntryBackendEventListeners() {
		return eventEntryBackendEventListeners;
	}
	
	/**
	 * Added an VascEntryFrontendEventListener
	 * @param listener The class of the event listener.
	 */
	@Override
	public void addVascEntryFrontendEventListener(String listener,String frontendType) {
		if (frontendType==null) {
			frontendType = "__ALL__";
		}
		if (frontendType.isEmpty() | "all".equalsIgnoreCase(frontendType)) {
			frontendType = "__ALL__";
		}
		List<String> typeList = eventEntryFrontendEventListeners.get(frontendType);
		if (typeList==null) {
			typeList = new ArrayList<String>(10);
			eventEntryFrontendEventListeners.put(frontendType,typeList);
		}
		typeList.add(listener);
	}
	
	/**
	 * Returns the list of VascEntryFrontendEventListener
	 * @return
	 */
	@Override
	public List<String> getVascEntryFrontendEventListenersByType(String frontendType) {
		if (frontendType==null) {
			frontendType = "__ALL__";
		}
		if (frontendType.isEmpty() | "all".equalsIgnoreCase(frontendType)) {
			frontendType = "__ALL__";
		}
		List<String> typeList = eventEntryFrontendEventListeners.get(frontendType);
		if (typeList==null) {
			typeList = new ArrayList<String>(0);
		}
		if (frontendType.equals("__ALL__")==false) {
			// also add all 
			List<String> typeListAll = eventEntryFrontendEventListeners.get("__ALL__");
			if (typeListAll!=null) {
				typeList.addAll(typeListAll);
			}
		}
		return typeList;
	}
	
	@Override
	public void addVascEntryFrontendAction(String actionClass,String frontendType) {
		if (frontendType==null) {
			frontendType = "__ALL__";
		}
		if (frontendType.isEmpty() | "all".equalsIgnoreCase(frontendType)) {
			frontendType = "__ALL__";
		}
		List<String> typeList = eventEntryFrontendActions.get(frontendType);
		if (typeList==null) {
			typeList = new ArrayList<String>(10);
			eventEntryFrontendActions.put(frontendType,typeList);
		}
		typeList.add(actionClass);
	}
	
	@Override
	public List<String> getVascEntryFrontendActionsByType(String frontendType) {
		if (frontendType==null) {
			frontendType = "__ALL__";
		}
		if (frontendType.isEmpty() | "all".equalsIgnoreCase(frontendType)) {
			frontendType = "__ALL__";
		}
		List<String> typeList = eventEntryFrontendActions.get(frontendType);
		if (typeList==null) {
			typeList = new ArrayList<String>(0);
		}
		if (frontendType.equals("__ALL__")==false) {
			// also add all 
			List<String> typeListAll = eventEntryFrontendActions.get("__ALL__");
			if (typeListAll!=null) {
				typeList.addAll(typeListAll);
			}
		}
		return typeList;
	}
	
	@Override
	public List<VascEntryListOption> getVascEntryListOptions() {
		return new ArrayList<VascEntryListOption>(vascEntryListOptions);
	}
	
	@Override
	public Collection<VascEntryListOptionLocal> getVascEntryListOptionsLocal() {
		return vascEntryListOptions;
	}
	
	@Override
	public VascEntryListOption getVascEntryListOptionById(String listOptionId) {
		for (VascEntryListOption vef:vascEntryListOptions) {
			if (vef.getId().equals(listOptionId)) {
				return vef;
			}
		}
		return null;
	}
	
	@Override
	public void addVascEntryListOption(VascEntryListOptionLocal listOption) {
		if (listOption==null) {
			throw new NullPointerException("can not add null listOption.");
		}
		vascEntryListOptions.add(listOption);
	}
	
	@Override
	public void removeVascEntryListOption(VascEntryListOptionLocal listOption) {
		vascEntryListOptions.remove(listOption);
	}
	
	@Override
	public void addVascBackendFilter(VascProxyFilter filter) {
		backendFilters.add(filter);
	}
	
	@Override
	public List<VascProxyFilter> getVascBackendFilters() {
		return backendFilters;
	}

	/**
	 * @return the vascDisplayOnly
	 */
	@Override
	public Boolean getVascDisplayOnly() {
		return vascDisplayOnly;
	}

	/**
	 * @param vascDisplayOnly the vascDisplayOnly to set
	 */
	@Override
	public void setVascDisplayOnly(Boolean vascDisplayOnly) {
		this.vascDisplayOnly = vascDisplayOnly;
	}

	/**
	 * @return the vascGroupId
	 */
	@Override
	public String getVascGroupId() {
		return vascGroupId;
	}

	/**
	 * @param vascGroupId the vascGroupId to set
	 */
	@Override
	public void setVascGroupId(String vascGroupId) {
		this.vascGroupId = vascGroupId;
	}

	/**
	 * @return the accessType
	 */
	@Override
	public VascEntryAccessType getAccessType() {
		return accessType;
	}

	/**
	 * @param accessType the accessType to set
	 */
	@Override
	public void setAccessType(VascEntryAccessType accessType) {
		this.accessType = accessType;
	}
}
