/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.core;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import net.forwardfire.vasc.backend.crud.VascBackendCrud;
import net.forwardfire.vasc.backend.list.VascBackendListRequest;

/**
 * Holds all default state values
 * 
 * @author Willem Cazander
 * @version 1.0 Dec 15, 2009
 */
@SuppressWarnings("serial")
abstract public class AbstractVascEntryState implements VascEntryState {

	private List<Serializable> entryDataList = null;
	private Serializable entryDataObject = null;
	private Long totalBackendRecords = null;
	private VascBackendListRequest vascBackendState = null;
	private VascEntryState state = null;
	private Map<Integer,Boolean> multiActionSelection = null;
	private boolean isEditCreate = false;
	private VascBackendCrud<Serializable,Serializable> vascBackend = null;
	private VascEntry vascEntry = null;
	
	public AbstractVascEntryState() {
		entryDataList = new ArrayList<Serializable>(0);
		multiActionSelection = new HashMap<Integer,Boolean>(200);
	}
	
	/**
	 * @return the entryDataList
	 */
	@Override
	public List<Serializable> getEntryDataList() {
		return entryDataList;
	}
	
	/**
	 * @param entryDataList the entryDataList to set
	 */
	@Override
	public void setEntryDataList(List<Serializable> entryDataList) {
		this.entryDataList = entryDataList;
	}
	
	/**
	 * @return the entryDataObject
	 */
	@Override
	public Serializable getEntryDataObject() {
		return entryDataObject;
	}
	
	/**
	 * @param entryDataObject the entryDataObject to set
	 */
	@Override
	public void setEntryDataObject(Serializable entryDataObject) {
		this.entryDataObject = entryDataObject;
	}
	
	/**
	 * @return the vascBackendState
	 */
	@Override
	public VascBackendListRequest getVascBackendState() {
		return vascBackendState;
	}

	/**
	 * @param vascBackendState the vascBackendState to set
	 */
	@Override
	public void setVascBackendState(VascBackendListRequest vascBackendState) {
		this.vascBackendState = vascBackendState;
	}

	/**
	 * @return the totalBackendRecords
	 */
	@Override
	public Long getTotalBackendRecords() {
		return totalBackendRecords;
	}

	/**
	 * @param totalBackendRecords the totalBackendRecords to set
	 */
	@Override
	public void setTotalBackendRecords(Long totalBackendRecords) {
		this.totalBackendRecords = totalBackendRecords;
	}
	
	@Override
	public void setParent(VascEntryState state) {
		this.state=state;
	}
	
	@Override
	public VascEntryState getParent() {
		return state;
	}
	
	@Override
	public void setMultiActionSelection(Map<Integer,Boolean> multiActionSelection) {
		this.multiActionSelection=multiActionSelection;
	}
	
	@Override
	public Map<Integer,Boolean> getMultiActionSelection() {
		return multiActionSelection;
	}
	
	/**
	 * @return the vascBackend
	 */
	@Override
	public VascBackendCrud<Serializable,Serializable> getVascBackend() {
		return vascBackend;
	}

	/**
	 * @param vascBackend the vascBackend to set
	 */
	@Override
	public void setVascBackend(VascBackendCrud<Serializable,Serializable> vascBackend) {
		this.vascBackend = vascBackend;
	}
	

	/**
	 * @return the isEditCreate
	 */
	@Override
	public boolean isEditCreate() {
		return isEditCreate;
	}

	/**
	 * @param isEditCreate the isEditCreate to set
	 */
	@Override
	public void setEditCreate(boolean isEditCreate) {
		this.isEditCreate = isEditCreate;
	}
	
	@Override
	public void setVascEntry(VascEntry vascEntry) {
		this.vascEntry=vascEntry;
	}
	
	@Override
	public VascEntry getVascEntry() {
		return vascEntry;
	}
	
}
