/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.core;

import java.util.Collection;
import java.util.List;

import net.forwardfire.vasc.backend.proxy.VascProxyFilter;
import net.forwardfire.vasc.core.actions.ColumnVascAction;
import net.forwardfire.vasc.core.actions.GlobalVascAction;
import net.forwardfire.vasc.core.actions.RowVascAction;
import net.forwardfire.vasc.core.base.VascBaseIdRoleCrud;
import net.forwardfire.vasc.core.entry.VascEntryFieldEventChannel;
import net.forwardfire.vasc.frontend.VascFrontendController;

/**
 * VascEntry is the main vasc entry interface which defines an editable crud entry.  
 * 
 * 
 * @author Willem Cazander
 * @version 1.0 Mar 21, 2007
 */
public interface VascEntry extends VascBaseIdRoleCrud {
	
	/**
	 * @return the name
	 */
	String getName();
	
	/**
	 * @return the helpId
	 */
	String getHelpId();
	
	/**
	 * @return the image
	 */
	String getImage();
	
	/**
	 * @return the listDescription
	 */
	String getListDescription();
	
	/**
	 * @return the listImage
	 */
	String getListImage();
	
	/**
	 * @return the editDescription
	 */
	String getEditDescription();
	
	/**
	 * @return the editImage
	 */
	String getEditImage();
	
	/**
	 * @return the deleteDescription
	 */
	String getDeleteDescription();
	
	/**
	 * @return the deleteImage
	 */
	String getDeleteImage();
	
	/**
	 * @return the createDescription
	 */
	String getCreateDescription();
	
	/**
	 * @return the createImage
	 */
	String getCreateImage();
	
	/**
	 * @return the primaryKeyField
	 */
	String getPrimaryKeyFieldId();
	
	/**
	 * @return the displayNameField
	 */
	String getDisplayNameFieldId();
	
	/**
	 * @return the delete
	 */
	Boolean getDelete();
	
	/**
	 * @return the rolesDelete
	 */
	String getRolesDelete();
	
	/**
	 * @return the accessType
	 */
	VascEntryAccessType getAccessType();
	
	/**
	 * @return the vascFields
	 */
	Collection<VascEntryField> getVascEntryFields();
	
	/**
	 * @return the vascField
	 */
	VascEntryField getVascEntryFieldById(String id);
	
	/**
	 * @return the rowActions
	 */
	Collection<RowVascAction> getRowActions();
	
	/**
	 * @return the RowVascAction
	 */
	RowVascAction getRowActionById(String actionId);
	
	/**
	 * @return the columnActions
	 */
	Collection<ColumnVascAction> getColumnActions();
	
	/**
	 * @return the ColumnVascAction
	 */
	ColumnVascAction getColumnActionById(String actionId);
	
	/**
	 * @return the globalActions
	 */
	Collection<GlobalVascAction> getGlobalActions();
	
	/**
	 * @return the GlobalVascAction
	 */
	GlobalVascAction getGlobalActionById(String actionId);
	
	/**
	 * @return the exportActions
	 */
	Collection<GlobalVascAction> getExportActions();
	
	/**
	 * @return the GlobalVascAction exportAction
	 */
	GlobalVascAction getExportActionById(String actionId);
	
	/**
	 * @return the vascEntryFieldSets
	 */
	Collection<VascEntryFieldSet> getVascEntryFieldSets();
	
	/**
	 * @return the VascEntryFieldSet
	 */
	VascEntryFieldSet getVascEntryFieldSetById(String actionId);
	
	/**
	 * @return the vascLinkEntries
	 */
	Collection<VascEntryLink> getVascEntryLinks();
	
	/**
	 * @return the VascLinkEntry
	 */
	VascEntryLink getVascEntryLinkById(String linkId);
	
	Object getEntryParameter(String key);
	void setEntryParameter(String key,Object value);
	void removeEntryParameter(String key);
	List<String> getEntryParameterKeys();
	
	VascFrontendController getVascFrontendController();
	
	String getBackendId();
	
	String getVascGroupId();
	
	/**
	 * @return the vascDisplayOnly
	 */
	Boolean getVascDisplayOnly();
	
	/**
	 * @return the vascEntryFieldEventChannel
	 */
	VascEntryFieldEventChannel getVascEntryFieldEventChannel();
	
	/**
	 * Returns the list of VascEntryBackendEventListener
	 * @return
	 */
	List<String> getVascEntryBackendEventListeners();
	
	/**
	 * Returns the list of VascEntryFrontendEventListener
	 * @return
	 */
	List<String> getVascEntryFrontendEventListenersByType(String frontendType);
	
	Collection<VascEntryListOption> getVascEntryListOptions();
	
	VascEntryListOption getVascEntryListOptionById(String listOptionId);
	
	List<VascProxyFilter> getVascBackendFilters();
	
	List<String> getVascEntryFrontendActionsByType(String frontendType);
}
