/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.core;

import java.io.Serializable;
import java.util.List;

import net.forwardfire.vasc.backend.VascBackendAccessDataRecord;
import net.forwardfire.vasc.core.base.VascBaseIdRoleCrudOrderMeta;
import net.forwardfire.vasc.validators.VascValidator;

/**
 * VascEntryField defines values per field.
 * 
 *  
 * @author Willem Cazander
 * @version 1.0 Mar 21, 2007
 */
public interface VascEntryField extends VascBaseIdRoleCrudOrderMeta {
	
	/**
	 * @return the VascEntry
	 */
	VascEntry getVascEntry();
	
	/**
	 * @return the vascEntryFieldType
	 */
	VascEntryFieldType getVascEntryFieldType();
	
	/**
	 * @return the backendName
	 */
	String getBackendName();
	
	/**
	 * @return the vascEntryFieldValue
	 */
	VascBackendAccessDataRecord<Serializable,Serializable> getVascEntryFieldValue();
	
	/**
	 * @return the vascValidators
	 */
	List<VascValidator> getVascValidators();
	
	/**
	 * @return the defaultValue
	 */
	Serializable getDefaultValue();
	
	/**
	 * @return the sizeList
	 */
	Integer getSizeList();
	
	/**
	 * @return the sizeEdit
	 */
	Integer getSizeEdit();
	
	/**
	 * @return the styleList
	 */
	String getStyleList();
	
	/**
	 * @return the styleEdit
	 */
	String getStyleEdit();
	
	/**
	 * @return the choices
	 */
	String getChoices();
	
	/**
	 * @return the optional
	 */
	Boolean getOptional();
	
	/**
	 * @return the choicesAsRadio
	 */
	Boolean getChoicesAsRadio();
	
	/**
	 * @return the editBlank
	 */
	Boolean getEditBlank();
	
	/**
	 * @return the displayName
	 */
	String getDisplayName();
	
	/**
	 * @return the sortable
	 */
	Boolean getSortable();
	
	/**
	 * @return the sumable
	 */
	Boolean getSumable();
	
	/**
	 * @return the graphable
	 */
	Boolean getGraphable();
}