/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.core;

import java.util.Collection;

import net.forwardfire.vasc.backend.proxy.VascProxyFilter;
import net.forwardfire.vasc.core.actions.ColumnVascActionLocal;
import net.forwardfire.vasc.core.actions.GlobalVascActionLocal;
import net.forwardfire.vasc.core.actions.RowVascActionLocal;
import net.forwardfire.vasc.core.base.VascBaseIdRoleCrudLocal;
import net.forwardfire.vasc.core.entry.VascEntryFieldEventChannel;
import net.forwardfire.vasc.frontend.VascFrontendController;

/**
 * VascEntryLocal   
 * 
 * 
 * @author Willem Cazander
 * @version 1.0 Jun 2, 2012
 */
public interface VascEntryLocal extends VascEntry,VascBaseIdRoleCrudLocal {
	
	/**
	 * @param name the name to set
	 */
	void setName(String name);
	
	/**
	 * @param helpId the helpId to set
	 */
	void setHelpId(String helpId);
	
	/**
	 * @param image the image to set
	 */
	void setImage(String image);
	
	/**
	 * @param listDescription the listDescription to set
	 */
	void setListDescription(String listDescription);

	/**
	 * @param listImage the listImage to set
	 */
	void setListImage(String listImage);

	/**
	 * @param editDescription the editDescription to set
	 */
	void setEditDescription(String editDescription);

	/**
	 * @param editImage the editImage to set
	 */
	void setEditImage(String editImage);

	/**
	 * @param deleteDescription the deleteDescription to set
	 */
	void setDeleteDescription(String deleteDescription);

	/**
	 * @param deleteImage the deleteImage to set
	 */
	void setDeleteImage(String deleteImage);

	/**
	 * @param createDescription the createDescription to set
	 */
	void setCreateDescription(String createDescription);

	/**
	 * @param createImage the createImage to set
	 */
	void setCreateImage(String createImage);
	
	/**
	 * @param primaryKeyField the primaryKeyField to set
	 */
	void setPrimaryKeyFieldId(String primaryKeyField);
	
	/**
	 * @param displayNameField the displayNameField to set
	 */
	void setDisplayNameFieldId(String displayNameField);
	
	/**
	 * @param delete the delete to set
	 */
	void setDelete(Boolean delete);
	
	/**
	 * @param rolesDelete the rolesDelete to set
	 */
	void setRolesDelete(String rolesDelete);
	
	/**
	 * @param accessType the accessType to set
	 */
	void setAccessType(VascEntryAccessType accessType);
	
	/**
	 * @param vascField the vascField to add
	 */
	void addVascEntryField(VascEntryFieldLocal vascField);
	
	/**
	 * @param vascField the vascField to remove
	 */
	void removeVascEntryField(VascEntryFieldLocal vascField);

	/**
	 * @return the vascFields
	 */
	Collection<VascEntryFieldLocal> getVascEntryFieldsLocal();
	
	/**
	 * @param rowAction the rowAction to add
	 */
	void addRowAction(RowVascActionLocal rowAction);

	/**
	 * @param rowAction the rowAction to remove
	 */
	void removeRowAction(RowVascActionLocal rowAction);
	
	/**
	 * @return the rowActions
	 */
	Collection<RowVascActionLocal> getRowActionsLocal();
	
	/**
	 * @param columnAction the columnAction to add
	 */
	void addColumnAction(ColumnVascActionLocal columnAction);
	
	/**
	 * @param columnAction the columnAction to remove
	 */
	void removeColumnAction(ColumnVascActionLocal columnAction);

	/**
	 * @return the columnActions
	 */
	Collection<ColumnVascActionLocal> getColumnActionsLocal();
	
	/**
	 * @param globalAction the globalAction to add
	 */
	void addGlobalAction(GlobalVascActionLocal globalAction);

	/**
	 * @param globalAction the globalAction to remove
	 */
	void removeGlobalAction(GlobalVascActionLocal globalAction);
	
	/**
	 * @return the globalActions
	 */
	Collection<GlobalVascActionLocal> getGlobalActionsLocal();
	
	/**
	 * @param exportAction the exportAction to add
	 */
	void addExportAction(GlobalVascActionLocal exportAction);
	
	/**
	 * @param exportAction the exportAction to remove
	 */
	void removeExportAction(GlobalVascActionLocal exportAction);

	/**
	 * @return the exportActions
	 */
	Collection<GlobalVascActionLocal> getExportActionsLocal();
	
	/**
	 * @param vascEntryFieldSet the vascEntryFieldSet to add
	 */
	void addVascEntryFieldSet(VascEntryFieldSetLocal vascEntryFieldSet);
	
	/**
	 * @param vascEntryFieldSet the vascEntryFieldSet to remove
	 */
	void removeVascEntryFieldSet(VascEntryFieldSetLocal vascEntryFieldSet);
	
	/**
	 * @return the vascEntryFieldSets
	 */
	Collection<VascEntryFieldSetLocal> getVascEntryFieldSetsLocal();
	
	/**
	 * @param vascLinkEntry the vascLinkEntry to add
	 */
	void addVascEntryLink(VascEntryLinkLocal vascEntryLink);

	/**
	 * @param vascLinkEntry the vascLinkEntry to remover
	 */
	void removeVascEntryLink(VascEntryLinkLocal vascEntryLink);
	
	/**
	 * @return the vascLinkEntries
	 */
	Collection<VascEntryLinkLocal> getVascEntryLinksLocal();

	void setVascFrontendController(VascFrontendController vascFrontendData);

	void setBackendId(String backendId);

	void setVascGroupId(String groupId);
	
	/**
	 * @param vascDisplayOnly the vascDisplayOnly to set
	 */
	void setVascDisplayOnly(Boolean vascDisplayOnly);
	
	/**
	 * @param vascEntryFieldEventChannel the vascEntryFieldEventChannel to set
	 */
	void setVascEntryFieldEventChannel(VascEntryFieldEventChannel vascEntryFieldEventChannel);
	
	/**
	 * Added an VascEntryBackendEventListener
	 * @param listener The class of the event listener.
	 */
	void addVascEntryBackendEventListener(String listener);

	/**
	 * Added an VascEntryFrontendEventListener
	 * @param listener The class of the event listener.
	 */
	void addVascEntryFrontendEventListener(String listener,String frontendType);
	
	void addVascEntryListOption(VascEntryListOptionLocal listOption);
	
	void removeVascEntryListOption(VascEntryListOptionLocal listOption);
	
	Collection<VascEntryListOptionLocal> getVascEntryListOptionsLocal();
	
	void addVascBackendFilter(VascProxyFilter filter);
	
	void addVascEntryFrontendAction(String actionClass,String frontendType);
	
	/**
	 * Force impl to have public clone methode
	 * @return
	 * @throws CloneNotSupportedException
	 */
	VascEntryLocal clone() throws CloneNotSupportedException;
}
