/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.core;

import java.io.Serializable;
import java.util.List;
import java.util.Map;

import net.forwardfire.vasc.backend.crud.VascBackendCrud;
import net.forwardfire.vasc.backend.list.VascBackendListRequest;

/**
 * Holds all the data for state.
 * So we can jump forward or back to a state.
 * 
 * @author Willem Cazander
 * @version 1.0 Dec 15, 2009
 */
public interface VascEntryState extends Serializable {
	
	/**
	 * @return the entryDataList
	 */
	List<Serializable> getEntryDataList();
	
	/**
	 * @param entryDataList the entryDataList to set
	 */
	void setEntryDataList(List<Serializable> entryDataList);
	
	/**
	 * @return the entryDataObject
	 */
	Serializable getEntryDataObject();
	
	/**
	 * @param entryDataObject the entryDataObject to set
	 */
	void setEntryDataObject(Serializable entryDataObject);
	
	/**
	 * @return the vascBackendState
	 */
	VascBackendListRequest getVascBackendState();
	
	/**
	 * @param vascBackendState the vascBackendState to set
	 */
	void setVascBackendState(VascBackendListRequest vascBackendState);
	
	/**
	 * @return the totalBackendRecords
	 */
	Long getTotalBackendRecords();
	
	/**
	 * @param totalBackendRecords the totalBackendRecords to set
	 */
	void setTotalBackendRecords(Long totalBackendRecords);
	
	/**
	 * @param state The previous state we come from.
	 */
	void setParent(VascEntryState state);
	
	/**
	 * @return The previous state we come from.
	 */
	VascEntryState getParent();
	
	void setMultiActionSelection(Map<Integer,Boolean> multiActionSelection);
	Map<Integer,Boolean> getMultiActionSelection();
	
	void setVascEntry(VascEntry vascEntry);
	VascEntry getVascEntry();
	
	/**
	 * @return the vascBackend
	 */
	public VascBackendCrud<Serializable,Serializable> getVascBackend();
	
	/**
	 * @param vascBackend the vascBackend to set
	 */
	void setVascBackend(VascBackendCrud<Serializable,Serializable> vascBackend);
	
	/**
	 * @return the isEditCreate
	 */
	boolean isEditCreate();
	
	/**
	 * @param isEditCreate the isEditCreate to set
	 */
	void setEditCreate(boolean isEditCreate);
}
