/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.core.ui;

import net.forwardfire.vasc.core.VascEntryListOption;
import net.forwardfire.vasc.core.VascException;

/**
 * VascOptionValueModelListener to handle editors which sets entry parameter values.
 * 
 * @author Willem Cazander
 * @version 1.0 May 21, 2009
 */
public class VascOptionValueModelListener implements VascValueModelListener {
	
	private VascEntryListOption vascEntryListOption = null;
	
	public VascOptionValueModelListener() {
	}
	
	public VascOptionValueModelListener(VascEntryListOption vascEntryListOption) {
		setVascEntryListOption(vascEntryListOption);
	}
	
	@Override
	public void valueUpdate(VascValueModel model) throws VascException {
		String key = vascEntryListOption.getBackendName();
		Object value = model.getValue();
		if (value==null && vascEntryListOption.getOptional()!=null && vascEntryListOption.getOptional()) {
			vascEntryListOption.getVascEntry().removeEntryParameter(key); // null is a value except if optional
			return;
		}
		vascEntryListOption.getVascEntry().setEntryParameter(key, value);	
	}
	
	/**
	 * @return the vascEntryListOption
	 */
	public VascEntryListOption getVascEntryListOption() {
		return vascEntryListOption;
	}
	
	/**
	 * @param vascEntryListOption the vascEntryListOption to set
	 */
	public void setVascEntryListOption(VascEntryListOption vascEntryListOption) {
		this.vascEntryListOption = vascEntryListOption;
	}
}