/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.core.ui;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import net.forwardfire.vasc.core.VascException;



/**
 * VascValueModel
 * 
 * @author Willem Cazander
 * @version 1.0 Aug 12, 2007
 */
public class VascValueModel {
	
	private Serializable value = null;
	private List<VascValueModelListener> listeners = null;
	private VascValueModel parentModel = null;
	
	public VascValueModel() {
		listeners = new ArrayList<VascValueModelListener>(2);
	}
	public VascValueModel(VascValueModel parentModel) {
		this();
		this.parentModel=parentModel;
	}
	
	public Serializable getValue() throws VascException {
		if (parentModel!=null) {
			return parentModel.getValue();
		}
		return value;
	}

	public void setValue(Serializable value) throws VascException {
		if (parentModel!=null) {
			parentModel.setValue(value);
		} else {
			this.value = value;
		}
		fireListeners();
	}
	
	public void addListener(VascValueModelListener l) {
		listeners.add(l);
	}
	public void removeListener(VascValueModelListener l) {
		listeners.remove(l);
	}
	private void fireListeners() throws VascException {
		for (VascValueModelListener l:listeners) {
			l.valueUpdate(this);
		}
	}
}