/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.frontend;

import java.util.logging.Logger;

import net.forwardfire.vasc.core.VascEntry;

/**
 * AbstractVascFrontend
 * 
 * @author Willem Cazander
 * @version 1.0 Aug 2, 2007
 */
abstract public class AbstractVascFrontend implements VascFrontend {
	
	private String id = null;
	protected VascEntry entry = null;

	public String i18n(String key,Object...params) {
		return entry.getVascFrontendController().getVascEntryResourceResolver().getTextValue(key,params);
	}
	
	public Object i18nImage(String key) {
		return entry.getVascFrontendController().getVascEntryResourceImageResolver().getImageValue(entry,key);
	}
	
	
	public VascEntry getVascEntry() {
		return entry;
	}
	
	abstract protected void addUiComponents();
	
	/**
	 * @see net.forwardfire.vasc.frontend.VascFrontend#initEntry(net.forwardfire.vasc.core.VascEntry)
	 */
	@Override
	public void initEntry(VascEntry entry) {
		if (entry.getVascFrontendController().getVascFrontend()==null) {
			Logger.getLogger(AbstractVascFrontend.class.getName()).fine("Bind frontend: "+this+" to: "+entry.getVascFrontendController()+" entry: "+entry.getId());
			entry.getVascFrontendController().setVascFrontend(this);
		} else {
			if (entry.getVascFrontendController().getVascFrontend()!=this) {
				throw new IllegalArgumentException("VascEntry '"+entry.getId()+"' data: "+entry.getVascFrontendController()+" frontend: "+this+" has already a differtent VascFrontend attected");
			}
		}
		this.entry=entry;
		addUiComponents();
	}
	
	/**
	 * @see net.forwardfire.vasc.frontend.VascFrontend#getId()
	 */
	@Override
	public String getId() {
		return id;
	}
	
	/**
	 * @see net.forwardfire.vasc.frontend.VascFrontend#setId(java.lang.String)
	 */
	@Override
	public void setId(String id) {
		this.id=id;	
	}
}