/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.frontend;

import java.io.Serializable;
import java.util.List;

import net.forwardfire.vasc.core.VascController;
import net.forwardfire.vasc.core.VascEntry;
import net.forwardfire.vasc.core.VascEntryField;
import net.forwardfire.vasc.core.VascEntryLocal;
import net.forwardfire.vasc.core.VascEntryState;
import net.forwardfire.vasc.core.VascException;
import net.forwardfire.vasc.core.entry.VascEntryFieldValidatorService;
import net.forwardfire.vasc.core.entry.VascEntryFrontendEventListener;
import net.forwardfire.vasc.core.entry.VascEntryResourceImageResolver;
import net.forwardfire.vasc.core.entry.VascEntryResourceResolver;
import net.forwardfire.vasc.core.entry.VascEntryFrontendEventListener.VascFrontendEventType;
import net.forwardfire.vasc.core.ui.VascUIComponent;




/**
 * VascFrontendController holds all dynamic frontend code in one place.
 * 
 * 
 * @author Willem Cazander
 * @version 1.0 Mar 21, 2007
 */
public interface VascFrontendController {
	
	/**
	 * @return the vascFrontend
	 */
	VascFrontend getVascFrontend();
	
	/**
	 * @param vascFrontend the vascFrontend to set
	 */
	void setVascFrontend(VascFrontend vascFrontend);
	
	/**
	 * Gets the VascFrontendActions to make frontend actions simple.
	 */
	VascFrontendActions getVascFrontendActions();
	
	/**
	 * @return the vascFrontendPageInfo
	 */
	VascFrontendPageInfo getVascFrontendPageInfo();
	
	/**
	 * @return	the VascFrontendHelper
	 */
	VascFrontendHelper getVascFrontendHelper();
	
	/**
	 * @param vascFrontendHelper	The VascFrontendHelper to set.
	 */
	void setVascFrontendHelper(VascFrontendHelper vascFrontendHelper);
	
	/**
	 * @return the vascEntryResourceResolver
	 */
	VascEntryResourceResolver getVascEntryResourceResolver();
	
	
	
	void putVascUIComponent(String rendererId,String uiComponentClass);
	
	VascUIComponent getVascUIComponent(String rendererId) throws VascException;
	String getVascUIComponentClass(String rendererId);
	
	void setVascController(VascController vascController);
	
	VascController getVascController();
	
	void addFieldVascUIComponents(VascEntryField field,VascUIComponent uiComponent,Object editor);
	VascUIComponent getFieldVascUIComponent(VascEntryField field);
	Object getFieldRealRenderer(VascEntryField field);
	void clearFieldRenderObjects();

	/**
	 * @return the vascEntryResourceImageResolver
	 */
	VascEntryResourceImageResolver getVascEntryResourceImageResolver();

	
	List<VascEntryFieldValidatorService> getVascValidatorServices();
	
	VascEntryState getVascEntryState();
	
	void initFrontendListeners(VascEntryLocal entry,String frontendType) throws InstantiationException, IllegalAccessException;
	void addVascEntryFrontendEventListener(VascEntryFrontendEventListener listener);
	List<VascEntryFrontendEventListener> getVascEntryFrontendEventListener(VascEntryFrontendEventListener.VascFrontendEventType type);
	void fireVascFrontendEvent(VascEntry entry,VascFrontendEventType type,Serializable data);
	
	
	/**
	 * @return the vascFrontendDataSelector
	 */
	VascFrontendDataSelector getVascFrontendDataSelector();
	
	/**
	 * @return the vascFrontendUserController
	 */
	VascFrontendUserController getVascFrontendUserController();
	
	/**
	 * @return the vascFrontendUserSettingController
	 */
	VascFrontendUserSettingController getVascFrontendUserSettingController();
}
