/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.impl;

import java.util.ArrayList;
import java.util.List;

import net.forwardfire.vasc.core.VascEventChannelControllerLocal;
import net.forwardfire.vasc.core.VascEventControllerListener;
import net.forwardfire.vasc.core.VascEventControllerType;

/**
 * DefaultVascEventChannelController handles all vasc events.
 * 
 * @author Willem Cazander
 * @version 1.0 Oct 6, 2012
 */
public class DefaultVascEventChannelController implements VascEventChannelControllerLocal {

	private List<VascEventControllerListener> listeners = null;
	
	public DefaultVascEventChannelController() {
		listeners = new ArrayList<VascEventControllerListener>(10);
	}

	/**
	 * @see net.forwardfire.vasc.core.VascEventChannelController#fireEvent(net.forwardfire.vasc.core.VascEventControllerType, java.lang.Object)
	 */
	@Override
	public void fireEvent(VascEventControllerType eventType, Object eventObject) {
		if (eventType==null) {
			throw new NullPointerException("Can't fire event when null type.");
		}
		for (int i=0;i<listeners.size();i++) {
			VascEventControllerListener listener = listeners.get(i);
			for (VascEventControllerType type:listener.getVascEventControllerTypes()) {
				if (eventType.equals(type)) {
					listener.controllerEvent(eventType, eventObject);
				}
			}
		}
	}

	/**
	 * @see net.forwardfire.vasc.core.VascEventChannelControllerLocal#getVascEventControllerListeners()
	 */
	@Override
	public List<VascEventControllerListener> getVascEventControllerListeners() {
		return listeners;
	}

	/**
	 * @see net.forwardfire.vasc.core.VascEventChannelControllerLocal#addVascEventControllerListener(net.forwardfire.vasc.core.VascEventControllerListener)
	 */
	@Override
	public void addVascEventControllerListener(VascEventControllerListener listener) {
		listeners.add(listener);
	}

	/**
	 * @see net.forwardfire.vasc.core.VascEventChannelControllerLocal#removeVascEventControllerListener(net.forwardfire.vasc.core.VascEventControllerListener)
	 */
	@Override
	public void removeVascEventControllerListener(VascEventControllerListener listener) {
		listeners.remove(listener);
	}
}