/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.impl;

import net.forwardfire.vasc.backend.DefaultVascBackendController;
import net.forwardfire.vasc.backend.list.DefaultVascBackendListRequest;
import net.forwardfire.vasc.backend.proxy.VascBackendProxyCache;
import net.forwardfire.vasc.backend.proxy.VascBackendProxyEventExecutor;
import net.forwardfire.vasc.backend.proxy.VascBackendProxyFilter;
import net.forwardfire.vasc.backend.proxy.VascBackendProxyPaged;
import net.forwardfire.vasc.backend.proxy.VascBackendProxySearch;
import net.forwardfire.vasc.backend.proxy.VascBackendProxySort;
import net.forwardfire.vasc.backend.proxy.VascBackendProxyTimerLogger;
import net.forwardfire.vasc.core.VascController;
import net.forwardfire.vasc.core.VascEntryControllerLocal;
import net.forwardfire.vasc.core.VascEntryLocal;
import net.forwardfire.vasc.core.VascInterfaceKey;
import net.forwardfire.vasc.core.VascInterfaceKeyFrontend;
import net.forwardfire.vasc.core.VascException;
import net.forwardfire.vasc.impl.entry.VascValidatorsValidatorService;
import net.forwardfire.vasc.impl.entry.config.VascActionsFinalizer;
import net.forwardfire.vasc.impl.entry.config.VascDefaultsFinalizer;
import net.forwardfire.vasc.impl.entry.config.VascHelpIdFinalizer;
import net.forwardfire.vasc.impl.entry.config.VascI18nFinalizer;
import net.forwardfire.vasc.impl.entry.config.VascIdAutoFinalizer;
import net.forwardfire.vasc.impl.entry.config.VascIdCheckFinalizer;
import net.forwardfire.vasc.impl.frontend.DefaultVascFrontendActions;
import net.forwardfire.vasc.impl.frontend.DefaultVascFrontendController;
import net.forwardfire.vasc.impl.frontend.DefaultVascFrontendDataSelector;
import net.forwardfire.vasc.impl.frontend.DefaultVascFrontendPager;
import net.forwardfire.vasc.impl.frontend.DummyVascFrontendUserController;
import net.forwardfire.vasc.impl.frontend.DummyVascFrontendUserSettingController;
import net.forwardfire.vasc.impl.type.DefaultVascEntryFieldTypeController;

/**
 * Default Vasc Factory for creating some base object plumming for vasc useage.
 * 
 * @author Willem Cazander
 * @version 1.0 Dec 15, 2009
 */
public class DefaultVascFactory {

	
	static public VascController getDefaultVascController() throws VascException {
		
		// Create 
		DefaultVascEntryConfigController vascConfig = new DefaultVascEntryConfigController();
		
		vascConfig.putVascInterfaceImplemention(VascInterfaceKey.VASC_ENTRY,						DefaultVascEntry.class);
		vascConfig.putVascInterfaceImplemention(VascInterfaceKey.VASC_ENTRY_FIELD,					DefaultVascEntryField.class);
		vascConfig.putVascInterfaceImplemention(VascInterfaceKey.VASC_ENTRY_FIELD_SET,				DefaultVascEntryFieldSet.class);
		vascConfig.putVascInterfaceImplemention(VascInterfaceKey.VASC_ENTRY_STATE,					DefaultVascEntryState.class);
		vascConfig.putVascInterfaceImplemention(VascInterfaceKey.VASC_ENTRY_LINK,					DefaultVascEntryLink.class);
		vascConfig.putVascInterfaceImplemention(VascInterfaceKey.VASC_ENTRY_LIST_OPTION,			DefaultVascEntryListOption.class);
		
		vascConfig.putVascInterfaceImplemention(VascInterfaceKey.VASC_BACKEND_STATE,				DefaultVascBackendListRequest.class);
		vascConfig.putVascInterfaceImplemention(VascInterfaceKey.VASC_FRONTEND_CONTROLLER_LOCAL,	DefaultVascFrontendController.class);
		
		vascConfig.putVascInterfaceImplementionFrontend(VascInterfaceKeyFrontend.VASC_FRONTEND_ACTIONS,					DefaultVascFrontendActions.class);
		vascConfig.putVascInterfaceImplementionFrontend(VascInterfaceKeyFrontend.VASC_FRONTEND_DATA_SELECTOR,			DefaultVascFrontendDataSelector.class);
		vascConfig.putVascInterfaceImplementionFrontend(VascInterfaceKeyFrontend.VASC_FRONTEND_PAGER,					DefaultVascFrontendPager.class);
		vascConfig.putVascInterfaceImplementionFrontend(VascInterfaceKeyFrontend.VASC_FRONTEND_USER_CONTROLLER,			DummyVascFrontendUserController.class);
		vascConfig.putVascInterfaceImplementionFrontend(VascInterfaceKeyFrontend.VASC_FRONTEND_USER_SETTING_CONTROLLER,	DummyVascFrontendUserSettingController.class);
		
		
		// Add all backend proxy in ORDER 
		vascConfig.addVascBackendProxy(new VascBackendProxyTimerLogger());
		vascConfig.addVascBackendProxy(new VascBackendProxyEventExecutor());
		vascConfig.addVascBackendProxy(new VascBackendProxyCache());
		vascConfig.addVascBackendProxy(new VascBackendProxyFilter());
		vascConfig.addVascBackendProxy(new VascBackendProxySearch());
		vascConfig.addVascBackendProxy(new VascBackendProxySort());
		vascConfig.addVascBackendProxy(new VascBackendProxyPaged());
		
		// Add all finalizers in ORDER
		vascConfig.addVascEntryConfigFinalizer(new VascActionsFinalizer());
		vascConfig.addVascEntryConfigFinalizer(new VascIdAutoFinalizer());
		vascConfig.addVascEntryConfigFinalizer(new VascIdCheckFinalizer());
		vascConfig.addVascEntryConfigFinalizer(new VascI18nFinalizer());
		vascConfig.addVascEntryConfigFinalizer(new VascHelpIdFinalizer());
		vascConfig.addVascEntryConfigFinalizer(new VascDefaultsFinalizer());
		
		// Add only the default internal vasc validator.
		vascConfig.addVascEntryFieldValidatorService(new VascValidatorsValidatorService());
		
		// Create default controller and fill resolvers.
		DefaultVascController c = new DefaultVascController();
		c.setVascEntryConfigController(vascConfig);
		c.setVascBackendController(new DefaultVascBackendController());
		c.setVascEntryController(new DefaultVascEntryController());
		c.setVascEntryFieldTypeController(new DefaultVascEntryFieldTypeController());
		c.setVascEventChannelController(new DefaultVascEventChannelController());
		
		// Ready to go
		return c;
	}

	static public void fillVascControllerLocalEntries(VascEntryControllerLocal c,VascController vascController) throws VascException  {
		for (String id:c.getVascEntryIds()) {
    		VascEntryLocal entry = (VascEntryLocal)c.getMasterVascEntryById(id);
    		vascController.getVascEntryConfigController().configVascEntry(vascController, entry);
		}
	}
}