/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.impl;

import net.forwardfire.vasc.backend.list.VascBackendListRequest;
import net.forwardfire.vasc.core.VascEntry;
import net.forwardfire.vasc.core.VascEntryConfigControllerLocal;
import net.forwardfire.vasc.core.VascEntryField;
import net.forwardfire.vasc.core.VascEntryFieldSet;
import net.forwardfire.vasc.core.VascEntryLink;
import net.forwardfire.vasc.core.VascEntryListOption;
import net.forwardfire.vasc.core.VascEntryState;
import net.forwardfire.vasc.core.VascInterfaceKey;
import net.forwardfire.vasc.core.VascInterfaceKeyFrontend;
import net.forwardfire.vasc.core.VascInterfaceLoader;
import net.forwardfire.vasc.frontend.VascFrontendActions;
import net.forwardfire.vasc.frontend.VascFrontendControllerLocal;
import net.forwardfire.vasc.frontend.VascFrontendDataSelector;
import net.forwardfire.vasc.frontend.VascFrontendPageInfo;
import net.forwardfire.vasc.frontend.VascFrontendUserController;
import net.forwardfire.vasc.frontend.VascFrontendUserSettingController;

/**
 * DefaultVascInterfaceLoader is auto caster.
 * 
 * @author Willem Cazander
 * @version 1.0 May 9, 2012
 */
public class DefaultVascInterfaceLoader implements VascInterfaceLoader {

	private VascEntryConfigControllerLocal localConfig = null;
	
	public DefaultVascInterfaceLoader(VascEntryConfigControllerLocal localConfig) {
		this.localConfig=localConfig;
	}
	
	private Object createImpl(VascInterfaceKey key) {
		return localConfig.createVascInterfaceImplemention(key);
	}
	private Object createImpl(VascInterfaceKeyFrontend key,VascEntry entry) {
		return localConfig.createVascInterfaceImplementionFrontend(key,entry);
	}
	
	
	/**
	 * @see net.forwardfire.vasc.core.VascInterfaceLoader#createVascEntryImpl()
	 */
	@Override
	public VascEntry createVascEntryImpl() {
		return (VascEntry)createImpl(VascInterfaceKey.VASC_ENTRY);
	}

	/**
	 * @see net.forwardfire.vasc.core.VascInterfaceLoader#createVascEntryFieldImpl()
	 */
	@Override
	public VascEntryField createVascEntryFieldImpl() {
		return (VascEntryField)createImpl(VascInterfaceKey.VASC_ENTRY_FIELD);
	}

	/**
	 * @see net.forwardfire.vasc.core.VascInterfaceLoader#createVascEntryStateImpl()
	 */
	@Override
	public VascEntryState createVascEntryStateImpl() {
		return (VascEntryState)createImpl(VascInterfaceKey.VASC_ENTRY_STATE);
	}

	/**
	 * @see net.forwardfire.vasc.core.VascInterfaceLoader#createVascEntryFieldSetImpl()
	 */
	@Override
	public VascEntryFieldSet createVascEntryFieldSetImpl() {
		return (VascEntryFieldSet)createImpl(VascInterfaceKey.VASC_ENTRY_FIELD_SET);
	}

	/**
	 * @see net.forwardfire.vasc.core.VascInterfaceLoader#createVascEntryLinkImpl()
	 */
	@Override
	public VascEntryLink createVascEntryLinkImpl() {
		return (VascEntryLink)createImpl(VascInterfaceKey.VASC_ENTRY_LINK);
	}

	/**
	 * @see net.forwardfire.vasc.core.VascInterfaceLoader#createVascEntryListOptionImpl()
	 */
	@Override
	public VascEntryListOption createVascEntryListOptionImpl() {
		return (VascEntryListOption)createImpl(VascInterfaceKey.VASC_ENTRY_LIST_OPTION);
	}

	/**
	 * @see net.forwardfire.vasc.core.VascInterfaceLoader#createVascBackendStateImpl()
	 */
	@Override
	public VascBackendListRequest createVascBackendStateImpl() {
		return (VascBackendListRequest)createImpl(VascInterfaceKey.VASC_BACKEND_STATE);
	}

	/**
	 * @see net.forwardfire.vasc.core.VascInterfaceLoader#createVascFrontendControllerLocalImpl()
	 */
	@Override
	public VascFrontendControllerLocal createVascFrontendControllerLocalImpl() {
		return (VascFrontendControllerLocal)createImpl(VascInterfaceKey.VASC_FRONTEND_CONTROLLER_LOCAL);
	}

	/**
	 * @see net.forwardfire.vasc.core.VascInterfaceLoader#createVascFrontendPagerImpl()
	 */
	@Override
	public VascFrontendPageInfo createVascFrontendPagerImpl(VascEntry entry) {
		return (VascFrontendPageInfo)createImpl(VascInterfaceKeyFrontend.VASC_FRONTEND_PAGER,entry);
	}

	/**
	 * @see net.forwardfire.vasc.core.VascInterfaceLoader#createVascFrontendActionsImpl()
	 */
	@Override
	public VascFrontendActions createVascFrontendActionsImpl(VascEntry entry) {
		return (VascFrontendActions)createImpl(VascInterfaceKeyFrontend.VASC_FRONTEND_ACTIONS,entry);
	}

	/**
	 * @see net.forwardfire.vasc.core.VascInterfaceLoader#createVascFrontendDataSelectorImpl()
	 */
	@Override
	public VascFrontendDataSelector createVascFrontendDataSelectorImpl(VascEntry entry) {
		return (VascFrontendDataSelector)createImpl(VascInterfaceKeyFrontend.VASC_FRONTEND_DATA_SELECTOR,entry);
	}

	/**
	 * @see net.forwardfire.vasc.core.VascInterfaceLoader#createVascFrontendUserControllerImpl()
	 */
	@Override
	public VascFrontendUserController createVascFrontendUserControllerImpl(VascEntry entry) {
		return (VascFrontendUserController)createImpl(VascInterfaceKeyFrontend.VASC_FRONTEND_USER_CONTROLLER,entry);
	}

	/**
	 * @see net.forwardfire.vasc.core.VascInterfaceLoader#createVascFrontendUserSettingControllerImpl()
	 */
	@Override
	public VascFrontendUserSettingController createVascFrontendUserSettingControllerImpl(VascEntry entry) {
		return (VascFrontendUserSettingController)createImpl(VascInterfaceKeyFrontend.VASC_FRONTEND_USER_SETTING_CONTROLLER,entry);
	}
}
