/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.impl.actions;

import net.forwardfire.vasc.core.VascEntry;
import net.forwardfire.vasc.core.VascException;
import net.forwardfire.vasc.core.actions.AbstractVascGlobalActionLocal;
import net.forwardfire.vasc.core.entry.VascEntryExport;

/**
 * ExportDataGlobalAction is action per exporter.
 * 
 * @author Willem Cazander
 * @version 1.0 May 10, 2012
 */
public class ExportDataGlobalAction extends AbstractVascGlobalActionLocal {
	
	private static final long serialVersionUID = 3474627214820048829L;
	private String exporterId = null;
	static public final String ACTION_POSTFIX = "ExportAction";
	
	public ExportDataGlobalAction() {
		
	}
	
	public ExportDataGlobalAction(String exporterId) {
		setExporterId(exporterId);
		setId(exporterId+ACTION_POSTFIX);
	}
	
	@Override
	protected String getActionId() {
		return exporterId;
	}
	
	@Override
	public void doGlobalAction(VascEntry entry) throws VascException {
		VascEntryExport exporter = entry.getVascFrontendController().getVascController().getVascEntryConfigController().getVascEntryExporterById(exporterId);
		entry.getVascFrontendController().getVascFrontend().renderExport(exporter);
	}

	@Override
	public void cloneFields(Object cloneObject)	throws CloneNotSupportedException {
		super.cloneFields(cloneObject);
		ExportDataGlobalAction clone = (ExportDataGlobalAction)cloneObject;
		clone.setExporterId(getExporterId());
	}

	/**
	 * @return the exporterId
	 */
	public String getExporterId() {
		return exporterId;
	}

	/**
	 * @param exporterId the exporterId to set
	 */
	public void setExporterId(String exporterId) {
		this.exporterId = exporterId;
	}
}