/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.impl.entry;


import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import net.forwardfire.vasc.core.VascException;
import net.forwardfire.vasc.core.entry.VascEntryExport;
import net.forwardfire.vasc.core.entry.VascEntryExportWriter;

/**
 * DefaultVascEntryExport holds the id and roles for the export writer.
 * 
 * @author Willem Cazander
 * @version 1.0 Nov 23, 2012
 */
public class DefaultVascEntryExport implements VascEntryExport {

	private static final long serialVersionUID = 4949926724589747264L;
	private String id = null;
	private String rolesExport = null;
	private String exportWriterClass = null;
	private Map<String,String> writerParameters = null;
	
	public DefaultVascEntryExport() {
		writerParameters = new HashMap<String,String>();
	}
	
	@Override
	public VascEntryExportWriter createExportWriter() throws VascException {
		ClassLoader cl = Thread.currentThread().getContextClassLoader();
		if (cl == null) {
			cl = DefaultVascEntryExport.class.getClassLoader();
		}
		try {
			Class<?> clazz = cl.loadClass(getExportWriterClass());
			Object instance = clazz.newInstance();
			VascEntryExportWriter writer = (VascEntryExportWriter)instance;
			return writer;
		} catch (Exception e) {
			throw new VascException(e);
		}
	}
	
	@Override
	public void addWriterParameter(String key,String value) {
		writerParameters.put(key,value);
	}
	
	@Override
	public String getWriterParameter(String key) {
		return writerParameters.get(key);
	}
	
	@Override
	public Set<String> getWriterParameterKeys() {
		return writerParameters.keySet();
	}
	
	/**
	 * @return the id
	 */
	@Override
	public String getId() {
		return id;
	}
	
	/**
	 * @param id the id to set
	 */
	public void setId(String id) {
		this.id = id;
	}
	
	/**
	 * @return the rolesExport
	 */
	@Override
	public String getRolesExport() {
		return rolesExport;
	}
	
	/**
	 * @param rolesExport the rolesExport to set
	 */
	public void setRolesExport(String rolesExport) {
		this.rolesExport = rolesExport;
	}
	
	/**
	 * @return the exportWriterClass
	 */
	@Override
	public String getExportWriterClass() {
		return exportWriterClass;
	}
	
	/**
	 * @param exportWriterClass the exportWriterClass to set
	 */
	public void setExportWriterClass(String exportWriterClass) {
		this.exportWriterClass = exportWriterClass;
	}
}
