/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.impl.entry;

import java.text.MessageFormat;
import java.util.Locale;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import net.forwardfire.vasc.core.entry.VascEntryResourceResolver;


/**
 * 
 * @author Willem Cazander
 * @version 1.0 Mar 21, 2007
 */
public class DefaultVascEntryResourceResolver implements VascEntryResourceResolver  {

	protected ResourceBundle resourceBundle = null;
	
	
	public DefaultVascEntryResourceResolver() {
	}
	
	public DefaultVascEntryResourceResolver(ResourceBundle resourceBundle) {
		this.resourceBundle = resourceBundle;
	}
	
	public DefaultVascEntryResourceResolver(String baseName,Locale locale) {
		this(ResourceBundle.getBundle(baseName, locale));
	}
	
	@Override
	public String getTextValue(String text,Object...params) {
		if (resourceBundle==null) {
			return text;
		}
		if (text==null) {
			throw new NullPointerException("Can't get null text key value.");
		}
			
		String textValue = null;
		try {
			textValue = resourceBundle.getString(text);
		} catch (MissingResourceException mre) {
			return text;
		}
		if (params.length>0) {
			textValue = MessageFormat.format(textValue, params);
		}
		return textValue;
	}

	/**
	 * @return the resourceBundle
	 */
	public ResourceBundle getResourceBundle() {
		return resourceBundle;
	}

	/**
	 * @param resourceBundle the resourceBundle to set
	 */
	public void setResourceBundle(ResourceBundle resourceBundle) {
		if (resourceBundle==null) {
			throw new NullPointerException("may not set resourceBundle to null.");
		}
		this.resourceBundle = resourceBundle;
	}
}