/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.impl.entry;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;

import javax.persistence.Column;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;

import net.forwardfire.vasc.core.VascEntryField;
import net.forwardfire.vasc.core.entry.VascEntryFieldValidatorService;
import net.forwardfire.vasc.core.VascException;
import net.forwardfire.vasc.validators.VascObjectNotNullValidator;
import net.forwardfire.vasc.validators.VascStringLengthValidator;
import net.forwardfire.vasc.validators.VascValidator;
import net.forwardfire.vasc.validators.VascValidatorMessages;



/**
 * Scans for JPA annotations and wrap some items aroud in a vasc validator.
 * 
 * @author Willem Cazander
 * @version 1.0 May 13, 2009
 */
public class PersistanceValidatorService implements VascEntryFieldValidatorService {

	private Logger logger = Logger.getLogger(PersistanceValidatorService.class.getName());
	
	@Override
	public List<String> validateObjectField(VascEntryField field, Object selectedRecord,Object objectValue) throws VascException {
		List<String> error = new ArrayList<String>(3);
    	try {
    		List<VascValidator> vals = new ArrayList<VascValidator>(2);
    		String property = field.getBackendName();
            for (Method method:selectedRecord.getClass().getMethods()) {
                if (method.getName().equalsIgnoreCase("get"+property)==false) {  //a bit durty  
                    continue;
                }
                Column col = method.getAnnotation(Column.class);
                if (col!=null) {
                	if (col.nullable()==false) {
                		Id idAnno = method.getAnnotation(Id.class);
                		GeneratedValue genAnno = method.getAnnotation(GeneratedValue.class);
                		if (idAnno!=null || genAnno!=null) {
                			continue; // don't add validator in JPA ID.
                		}
                		VascObjectNotNullValidator val = new VascObjectNotNullValidator();
                		vals.add(val);
                	}
                	// weird default 255 value
                	if (col.length()!=255) {
                		VascStringLengthValidator val = new VascStringLengthValidator();
                		val.setMaxLenght(col.length());
                		val.setNullable(col.nullable());
                		vals.add(val);
                	}
                }
            }
            
            VascValidatorMessages m = new VascValidatorMessages();
        	for (VascValidator val:vals) {
        		logger.finer("Validating: "+val);
        		if (val.isObjectValid(objectValue)==false) {
        			error.add(m.getErrorMessage(field.getVascEntry(), val));
        		}
        	}
    	} catch (Exception e) {
    		throw new VascException(e);
    	}
    	return error;
	}
}
