/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.impl.entry;

import net.forwardfire.vasc.backend.list.VascBackendListRequest;
import net.forwardfire.vasc.core.VascEntry;
import net.forwardfire.vasc.core.entry.VascEntryBackendEventListener;


/**
 * Sets parameter on the backend.
 * 
 * @author Willem Cazander
 *
 */
public class SetParameterBackendListener implements VascEntryBackendEventListener  {

	private static final long serialVersionUID = 1L;
	private String name = null;
	private String value = null;
	private String type = null;
	
	private boolean isUserValue = false;
	private Long userId = null;
	private String userName = null;
	
	public SetParameterBackendListener() {
		
	}
	
	public SetParameterBackendListener(Long userId,String userName) {
		this.userId=userId;
		this.userName=userName;
		isUserValue = true;
	}
	
	/**
	 * @see net.forwardfire.vasc.core.entry.VascEntryBackendEventListener#getEventType()
	 */
	@Override
	public VascBackendEventType[] getEventTypes() {
		return new VascBackendEventType[] {
				VascBackendEventType.PRE_DELETE,
				VascBackendEventType.PRE_EXECUTE,
				VascBackendEventType.PRE_MERGE,
				VascBackendEventType.PRE_MOVE_DOWN,
				VascBackendEventType.PRE_MOVE_UP,
				VascBackendEventType.PRE_PERSIST
		};
	}
	
	/**
	 * @see net.forwardfire.vasc.core.entry.VascEntryBackendEventListener#vascEvent(net.forwardfire.vasc.core.VascEntry, java.lang.Object)
	 */
	@Override
	public void vascEvent(VascEntry entry, Object data) {
		if (data instanceof VascBackendListRequest) {
			VascBackendListRequest state = (VascBackendListRequest)data;
			if (isUserValue) {
				if (value==null) {
					value="id";
				}
				if (value.isEmpty()) {
					value="id";
				}
				if ("id".equals(value)) {
					if ("int".equals(type)) {
						state.setDataParameter(name, userId.intValue());
					} else {
						state.setDataParameter(name, userId);
					}
				} else {
					state.setDataParameter(name, userName);
				}
			} else {
				state.setDataParameter(name, value);
			}
		}
	}

	/**
	 * @return the name
	 */
	public String getName() {
		return name;
	}

	/**
	 * @param name the name to set
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * @return the value
	 */
	public String getValue() {
		return value;
	}

	/**
	 * @param value the value to set
	 */
	public void setValue(String value) {
		this.value = value;
	}

	/**
	 * @return the type
	 */
	public String getType() {
		return type;
	}

	/**
	 * @param type the type to set
	 */
	public void setType(String type) {
		this.type = type;
	}

	/**
	 * @return the userId
	 */
	public Long getUserId() {
		return userId;
	}

	/**
	 * @param userId the userId to set
	 */
	public void setUserId(Long userId) {
		this.userId = userId;
	}

	/**
	 * @return the userName
	 */
	public String getUserName() {
		return userName;
	}

	/**
	 * @param userName the userName to set
	 */
	public void setUserName(String userName) {
		this.userName = userName;
	}
}
