/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.impl.entry;

import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;

import net.forwardfire.vasc.core.VascEntryField;
import net.forwardfire.vasc.core.entry.VascEntryFieldValidatorService;
import net.forwardfire.vasc.core.VascException;
import net.forwardfire.vasc.validators.VascValidator;
import net.forwardfire.vasc.validators.VascValidatorMessages;


/**
 * Executes the normal vasc validators for fields.
 * 
 * @author Willem Cazander
 * @version 1.0 May 13, 2009
 */
public class VascValidatorsValidatorService implements VascEntryFieldValidatorService {

	private Logger logger = Logger.getLogger(VascValidatorsValidatorService.class.getName());
	
	@Override
	public List<String> validateObjectField(VascEntryField field, Object selectedRecord,Object objectValue) throws VascException {
		List<String> error = new ArrayList<String>(3);
    	try {
    		VascValidatorMessages m = new VascValidatorMessages();
        	for (VascValidator val:field.getVascEntryFieldType().getVascValidators()) {
        		logger.finer("Validating: "+val);
        		if (val.isObjectValid(objectValue)==false) {
        			error.add(m.getErrorMessage(field.getVascEntry(), val));
        		}
        	}
        	for (VascValidator val:field.getVascValidators()) {
        		logger.finer("Validating: "+val);
        		if (val.isObjectValid(objectValue)==false) {
        			error.add(m.getErrorMessage(field.getVascEntry(), val));
        		}
        	}
    	} catch (Exception e) {
    		throw new VascException(e);
    	}
    	return error;
	}
}
