/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.impl.entry.config;

import net.forwardfire.vasc.core.VascController;
import net.forwardfire.vasc.core.VascEntryConfigFinalizer;
import net.forwardfire.vasc.core.VascEntryLocal;
import net.forwardfire.vasc.core.VascException;
import net.forwardfire.vasc.impl.actions.AddRowAction;
import net.forwardfire.vasc.impl.actions.DeleteRowAction;
import net.forwardfire.vasc.impl.actions.EditRowAction;
import net.forwardfire.vasc.impl.actions.ExportDataGlobalAction;
import net.forwardfire.vasc.impl.actions.RefreshDataGlobalAction;

/**
 * VascActionsFinalizer Adds all the default actions.
 * 
 * @author Willem Cazander
 * @version 1.0 May 9, 2012
 */
public class VascActionsFinalizer implements VascEntryConfigFinalizer  {

	/**
	 * @see net.forwardfire.vasc.core.VascEntryConfigFinalizer#configVascEntry(net.forwardfire.vasc.core.VascController, net.forwardfire.vasc.core.VascEntry)
	 */
	@Override
	public void configVascEntry(VascController vascController, VascEntryLocal entry)	throws VascException {


		if (entry.getExportActions().isEmpty()==false) {
			return;  // hackje for calling multiple times on same entries.
		}
	
		entry.addRowAction(new AddRowAction());
		entry.addRowAction(new EditRowAction());
		entry.addRowAction(new DeleteRowAction());
		
		/*
		if (entry.isVascDisplayOnly()==false) {  // TODO: redo to flags
			if (entry.isVascAdminCreate()) {
			}
			if (entry.isVascAdminEdit()) {
			}
			if (entry.isVascAdminDelete()) {
			}
		}
		*/
		entry.addGlobalAction(new RefreshDataGlobalAction());
		
		// Add all exporter actions
		for (String exportId:vascController.getVascEntryConfigController().getVascEntryExporterIds()) {
			entry.addExportAction(new ExportDataGlobalAction(exportId));
		}
	}}
