/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.impl.entry.config;

import java.io.Serializable;
import java.util.Locale;

import org.x4o.xml.conv.ObjectConverterException;

import net.forwardfire.vasc.backend.crud.VascBackendCrud;
import net.forwardfire.vasc.core.VascController;
import net.forwardfire.vasc.core.VascEntryField;
import net.forwardfire.vasc.core.VascEntryConfigFinalizer;
import net.forwardfire.vasc.core.VascEntryFieldLocal;
import net.forwardfire.vasc.core.VascEntryLinkLocal;
import net.forwardfire.vasc.core.VascEntryListOptionLocal;
import net.forwardfire.vasc.core.VascEntryLocal;
import net.forwardfire.vasc.core.VascEntryLinkType;
import net.forwardfire.vasc.core.VascException;

/**
 * VascDefaultsFinalizer does set some default in objects of entry.
 * 
 * @author Willem Cazander
 * @version 1.0 May 9, 2012
 */
public class VascDefaultsFinalizer implements VascEntryConfigFinalizer  {

	/**
	 * @see net.forwardfire.vasc.core.VascEntryConfigFinalizer#configVascEntry(net.forwardfire.vasc.core.VascController, net.forwardfire.vasc.core.VascEntry)
	 */
	@Override
	public void configVascEntry(VascController vascController, VascEntryLocal entry)	throws VascException {

		// First Check if we all have ids
		String id = entry.getId();
		if (entry.getVascEntryFields().size()==0) {
			throw new IllegalArgumentException("We need at least one VascEntryField in entryId: "+id);
		}
		
		// Check if backendId is valid
		VascBackendCrud<Serializable,Serializable> back = vascController.getVascBackendController().getVascBackendById( VascBackendCrud.class, entry.getBackendId() );
		if (back==null) {
			throw new IllegalArgumentException("The VascEntry backend is not found in backends: '"+entry.getBackendId()+"' in entryId: "+id); 
		}
		
		// TODO boolean view helper
		//if (entry.isVascDisplayOnly()) {
		//	entry.setVascAdminCreate(false);
		//	entry.setVascAdminDelete(false);
		//	entry.setVascAdminEdit(false);
		//}
				
		// Set defaults field Id for key ad display
		if (entry.getPrimaryKeyFieldId()==null) {
			for (VascEntryField field:entry.getVascEntryFields()) {
				if ("id".equalsIgnoreCase(field.getId())) {
					entry.setPrimaryKeyFieldId(field.getId());		
					break;
				}
			}
			if (entry.getPrimaryKeyFieldId()==null) {
				for (VascEntryField field:entry.getVascEntryFields()) {
					if ("id".equalsIgnoreCase(field.getId())) {
						entry.setPrimaryKeyFieldId(field.getId());		
						break;
					}
				}
			}
		}
		
		if (entry.getDisplayNameFieldId()==null) {
			entry.setDisplayNameFieldId(entry.getPrimaryKeyFieldId());
		}
		
		// Check fields
		int orderIndex = 0;
		for (VascEntryFieldLocal vef:entry.getVascEntryFieldsLocal()) {
			String vid = vef.getId();
			
			
			
			// set manual stuff
			if (vef.getBackendName()==null) {
				vef.setBackendName(vid);
			}
			if (vef.getVascEntry()==null) {
				vef.setVascEntry(entry);
			}
			
			if (vef.getOrder()==null) {
				vef.setOrder(orderIndex);
			}
			orderIndex = orderIndex+100;
			if (vef.getOrderCreate()==null) {
				vef.setOrderCreate(vef.getOrder());
			}
			if (vef.getOrderEdit()==null) {
				vef.setOrderEdit(vef.getOrder());
			}
			if (vef.getOrderList()==null) {
				vef.setOrderList(vef.getOrder());
			}
			
			
			//if (vef.getDefaultValue()==null) {
			//	vef.setDefaultValue("vasc.entry."+id+"."+vid+".defaultValue");
			//}

			if (vef.getView()==null) {
				vef.setView(true);
			}
			if (vef.getList()==null) {
				vef.setList(true);
			}
			if (vef.getCreate()==null) {
				vef.setCreate(true);
			}
			if (vef.getEdit()==null) {
				vef.setEdit(true);
			}
			if (vef.getEditReadOnly()==null) {
				vef.setEditReadOnly(false);
			}
			
			/* moved to fillVascEntryFrontend
			if (vef.getVascEntryFieldValue()==null) {
				VascBackend back2 = vascController.getVascBackendController().getVascBackendById( entry.getBackendId() );
				vef.setVascEntryFieldValue(back2.provideVascEntryFieldValue(vef));
			}
			*/
			
			if (vef.getVascEntryFieldType()==null) {	
				Object defValue = vef.getDefaultValue();
				if (defValue != null) {
					vef.setVascEntryFieldType(vascController.getVascEntryFieldTypeController().getVascEntryFieldTypeByClass(defValue.getClass()));
				}
				if (vef.getVascEntryFieldType()==null) {	
					vef.setVascEntryFieldType(vascController.getVascEntryFieldTypeController().getVascEntryFieldTypeById("TextField"));
				}
			}
			
			
			if (vef.getDefaultValue()!=null && vef.getDefaultValue() instanceof String) {
				if (vef.getVascEntryFieldType().getObjectConverter() != null) {
					Serializable value;
					try {
						value = (Serializable) vef.getVascEntryFieldType().getObjectConverter().convertTo(vef.getDefaultValue(), Locale.getDefault());
					} catch (ObjectConverterException e) {
						throw new VascException(e);
					}
					vef.setDefaultValue(value);
				}
			}
			
			//for (VascValidator vv:vef.getVascValidators()) {
			//}
		}
		

		
		// Check if link entries excists
		for (VascEntryLinkLocal vle:entry.getVascEntryLinksLocal()) {
			if (vle.getVascEntryId()==null) {
				throw new IllegalArgumentException("All VascLinkEntry need an vascEntryId: "+id);
			}
			if (vle.getVascEntryLinkType()==null) {
				vle.setVascEntryLinkType(VascEntryLinkType.DEFAULT_TYPE);
			}
		}
		
		for (VascEntryListOptionLocal vef:entry.getVascEntryListOptionsLocal()) {
			String vid = vef.getId();
			// set manual stuff
			if (vef.getBackendName()==null) {
				vef.setBackendName(vid);
			}
			if (vef.getVascEntry()==null) {
				vef.setVascEntry(entry);
			}
			if (vef.getOrder()==null) {
				vef.setOrder(orderIndex);
			}
			orderIndex = orderIndex+100;
			if (vef.getOrderCreate()==null) {
				vef.setOrderCreate(vef.getOrder());
			}
			if (vef.getOrderEdit()==null) {
				vef.setOrderEdit(vef.getOrder());
			}
			if (vef.getOrderList()==null) {
				vef.setOrderList(vef.getOrder());
			}
						
			//if (vef.getDefaultValue()==null) {
			//	vef.setDefaultValue("vasc.entry."+id+"."+vid+".defaultValue");
			//}

			if (vef.getView()==null) {
				vef.setView(true);
			}
			if (vef.getList()==null) {
				vef.setList(true);
			}
			if (vef.getCreate()==null) {
				vef.setCreate(true);
			}
			if (vef.getEdit()==null) {
				vef.setEdit(true);
			}
			if (vef.getEditReadOnly()==null) {
				vef.setEditReadOnly(false);
			}
			
			if (vef.getVascEntryFieldValue()==null) {
				//VascBackend back2 = vascController.getVascBackendControllerResolver().getVascBackendController().getVascBackendById( entry.getBackendId() );
				//vef.setVascEntryFieldValue(back2.provideVascEntryFieldValue(vef));
			}
						
			if (vef.getDefaultValue()!=null && vef.getDefaultValue() instanceof String) {
				if (vef.getVascEntryFieldType()!=null && vef.getVascEntryFieldType().getObjectConverter() != null) {
					Serializable value;
					try {
						value = (Serializable) vef.getVascEntryFieldType().getObjectConverter().convertTo(vef.getDefaultValue(), Locale.getDefault());
					} catch (ObjectConverterException e) {
						throw new VascException(e);
					}
					vef.setDefaultValue(value);
				}
			}	
		}		
	}
}
