/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.impl.entry.config;

import net.forwardfire.vasc.core.VascController;
import net.forwardfire.vasc.core.VascEntryFieldLocal;
import net.forwardfire.vasc.core.VascEntryConfigFinalizer;
import net.forwardfire.vasc.core.VascEntryFieldSetLocal;
import net.forwardfire.vasc.core.VascEntryLinkLocal;
import net.forwardfire.vasc.core.VascEntryLocal;
import net.forwardfire.vasc.core.actions.VascActionLocal;
import net.forwardfire.vasc.core.VascException;

/**
 * VascI18NFinalizer fills all missing i18n fields.
 * 
 * @author Willem Cazander
 * @version 1.0 May 9, 2012
 */
public class VascI18nFinalizer implements VascEntryConfigFinalizer  {

	/**
	 * @see net.forwardfire.vasc.core.VascEntryConfigFinalizer#configVascEntry(net.forwardfire.vasc.core.VascController, net.forwardfire.vasc.core.VascEntry)
	 */
	@Override
	public void configVascEntry(VascController vascController, VascEntryLocal entry)	throws VascException {

		// Fill up all not field i18n keys
		String id = entry.getId();
		
		// entry fields
		if (entry.getName()==null) {
			entry.setName("vasc.entry."+id+".name");
		}
		if (entry.getImage()==null) {
			entry.setImage("vasc.entry."+id+".image");
		}
		if (entry.getListDescription()==null) {
			entry.setListDescription("vasc.entry."+id+".listDescription");
		}
		if (entry.getListImage()==null) {
			entry.setListImage("vasc.entry."+id+".listImage");
		}
		if (entry.getEditDescription()==null) {
			entry.setEditDescription("vasc.entry."+id+".editDescription");
		}
		if (entry.getEditImage()==null) {
			entry.setEditImage("vasc.entry."+id+".editImage");
		}
		if (entry.getDeleteDescription()==null) {
			entry.setDeleteDescription("vasc.entry."+id+".deleteDescription");
		}
		if (entry.getDeleteImage()==null) {
			entry.setDeleteImage("vasc.entry."+id+".deleteImage");
		}
		if (entry.getCreateDescription()==null) {
			entry.setCreateDescription("vasc.entry."+id+".createDescription");
		}
		if (entry.getCreateImage()==null) {
			entry.setCreateImage("vasc.entry."+id+".createImage");
		}
			
		// Field sets
		for (VascEntryFieldSetLocal s:entry.getVascEntryFieldSetsLocal()) {
			String sid = s.getId();
			if (s.getName()==null) {
				s.setName("vasc.entry."+id+"."+sid+".name");
			}
			if (s.getDescription()==null) {
				s.setDescription("vasc.entry."+id+"."+sid+".description");
			}
			if (s.getImage()==null) {
				s.setImage("vasc.entry."+id+"."+sid+".image");
			}
		}
		
		// Check fields
		for (VascEntryFieldLocal vef:entry.getVascEntryFieldsLocal()) {
			String vid = vef.getId();
			if (vef.getName()==null) {
				vef.setName("vasc.entry."+id+"."+vid+".name");
			}
			if (vef.getDescription()==null) {
				vef.setDescription("vasc.entry."+id+"."+vid+".description");
			}
			if (vef.getImage()==null) {
				vef.setImage("vasc.entry."+id+"."+vid+".image");
			}
		}
		
		for (VascEntryLinkLocal vle:entry.getVascEntryLinksLocal()) {
			if (vle.getName()==null) {
				vle.setName("vasc.entry."+vle.getVascEntryId()+".name");
			}
		}
		
		for (VascEntryFieldLocal vef:entry.getVascEntryListOptionsLocal()) {
			String vid = vef.getId();
			if (vef.getName()==null) {
				vef.setName("vasc.entry."+id+"."+vid+".name");
			}
			if (vef.getDescription()==null) {
				vef.setDescription("vasc.entry."+id+"."+vid+".description");
			}
			if (vef.getImage()==null) {
				vef.setImage("vasc.entry."+id+"."+vid+".image");
			}
		}
		
		for (VascActionLocal action:entry.getGlobalActionsLocal()) {
			String aid = action.getId();
			if (aid==null) {
				throw new IllegalArgumentException("Action has no id: "+action+" in entryId: "+id);
			}
			if (action.getName()==null) {
				action.setName("vasc.action."+aid+".name");
			}
			if (action.getDescription()==null) {
				action.setDescription("vasc.action."+aid+".description");
			}
			if (action.getImage()==null) {
				action.setImage("vasc.action."+aid+".image");
			}
		}
		for (VascActionLocal action:entry.getExportActionsLocal()) {
			String aid = action.getId();
			if (aid==null) {
				throw new IllegalArgumentException("Action has no id: "+action+" in entryId: "+id);
			}
			if (action.getName()==null) {
				action.setName("vasc.action."+aid+".name");
			}
			if (action.getDescription()==null) {
				action.setDescription("vasc.action."+aid+".description");
			}
			if (action.getImage()==null) {
				action.setImage("vasc.action."+aid+".image");
			}
		}
		for (VascActionLocal action:entry.getRowActionsLocal()) {
			String aid = action.getId();
			if (aid==null) {
				throw new IllegalArgumentException("Action has no id: "+action+" in entryId: "+id);
			}
			if (action.getName()==null) {
				action.setName("vasc.action."+aid+".name");
			}
			if (action.getDescription()==null) {
				action.setDescription("vasc.action."+aid+".description");
			}
			if (action.getImage()==null) {
				action.setImage("vasc.action."+aid+".image");
			}
		}
		for (VascActionLocal action:entry.getColumnActionsLocal()) {
			String aid = action.getId();
			if (aid==null) {
				throw new IllegalArgumentException("Action has no id: "+action+" in entryId: "+id);
			}
			if (action.getName()==null) {
				action.setName("vasc.action."+aid+".name");
			}
			if (action.getDescription()==null) {
				action.setDescription("vasc.action."+aid+".description");
			}
			if (action.getImage()==null) {
				action.setImage("vasc.action."+aid+".image");
			}
		}
	}
}
