/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.impl.entry.config;

import net.forwardfire.vasc.core.VascController;
import net.forwardfire.vasc.core.VascEntryField;
import net.forwardfire.vasc.core.VascEntryFieldSet;
import net.forwardfire.vasc.core.VascEntryConfigFinalizer;
import net.forwardfire.vasc.core.VascEntryLinkLocal;
import net.forwardfire.vasc.core.VascEntryLocal;
import net.forwardfire.vasc.core.actions.VascActionLocal;
import net.forwardfire.vasc.core.VascException;

/**
 * VascAutoIdFinalizer fills the missing id automaticly. 
 * 
 * @author Willem Cazander
 * @version 1.0 May 9, 2012
 */
public class VascIdAutoFinalizer implements VascEntryConfigFinalizer  {

	private char seperatorChar = '_';
	
	/**
	 * @see net.forwardfire.vasc.core.VascEntryConfigFinalizer#configVascEntry(net.forwardfire.vasc.core.VascController, net.forwardfire.vasc.core.VascEntry)
	 */
	@Override
	public void configVascEntry(VascController vascController, VascEntryLocal entry)	throws VascException {

		// count all auto objects as one index 
		int index = 0;
		String id = entry.getId();
		if (id==null) {
			throw new IllegalArgumentException("The VascEntry need an id.");
		}
		for (VascEntryField vef:entry.getVascEntryListOptions()) {
			if (vef.getId()==null) {
				vef.setId(format(id,"list",""+index++));
			}
		}
		for (VascEntryFieldSet s:entry.getVascEntryFieldSets()) {
			String sid = s.getId();
			if (sid==null) {
				s.setId(format(id,"fieldset",""+index++));	
			}
		}
		for (VascEntryLinkLocal vle:entry.getVascEntryLinksLocal()) {
			String vid = vle.getId();
			if (vid==null) {
				if (vle.getVascEntryId()==null) {
					vle.setId(format(id,"link",""+index++));
				} else {
					vle.setId(format(id,"link",vle.getVascEntryId()));
				}
			}
		}
		for (VascActionLocal action:entry.getGlobalActionsLocal()) {
			String aid = action.getId();
			if (aid==null) {
				action.setId(format(id,"action",""+index++));
			}
		}
		for (VascActionLocal action:entry.getExportActionsLocal()) {
			String aid = action.getId();
			if (aid==null) {
				action.setId(format(id,"action",""+index++));
			}
		}
		for (VascActionLocal action:entry.getRowActionsLocal()) {
			String aid = action.getId();
			if (aid==null) {
				action.setId(format(id,"action",""+index++));
			}
		}
		for (VascActionLocal action:entry.getColumnActionsLocal()) {
			String aid = action.getId();
			if (aid==null) {
				action.setId(format(id,"action",""+index++));
			}
		}
	}
	
	private String format(String ... ids) {
		StringBuilder idBuff = new StringBuilder();
		for (int i=0;i<ids.length;i++) {
			idBuff.append(ids[i]);
			if (i<ids.length-1) {
				idBuff.append(getSeperatorChar());
			}
		}
		return idBuff.toString();
	}
	
	/**
	 * @return the seperatorChar
	 */
	public char getSeperatorChar() {
		return seperatorChar;
	}

	/**
	 * @param seperatorChar the seperatorChar to set
	 */
	public void setSeperatorChar(char seperatorChar) {
		this.seperatorChar = seperatorChar;
	}
}
