/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.impl.frontend;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import net.forwardfire.vasc.core.VascController;
import net.forwardfire.vasc.core.VascEntry;
import net.forwardfire.vasc.core.VascEntryField;
import net.forwardfire.vasc.core.VascEntryConfigFinalizer;
import net.forwardfire.vasc.core.VascEntryLocal;
import net.forwardfire.vasc.core.VascEntryState;
import net.forwardfire.vasc.core.actions.ColumnVascActionLocal;
import net.forwardfire.vasc.core.actions.GlobalVascActionLocal;
import net.forwardfire.vasc.core.actions.RowVascActionLocal;
import net.forwardfire.vasc.core.actions.VascActionLocal;
import net.forwardfire.vasc.core.entry.VascEntryFieldValidatorService;
import net.forwardfire.vasc.core.entry.VascEntryFrontendEventListener;
import net.forwardfire.vasc.core.entry.VascEntryResourceImageResolver;
import net.forwardfire.vasc.core.entry.VascEntryResourceResolver;
import net.forwardfire.vasc.core.entry.VascEntryFrontendEventListener.VascFrontendEventType;
import net.forwardfire.vasc.core.ui.VascUIComponent;
import net.forwardfire.vasc.core.VascException;
import net.forwardfire.vasc.frontend.VascFrontend;
import net.forwardfire.vasc.frontend.VascFrontendActions;
import net.forwardfire.vasc.frontend.VascFrontendControllerLocal;
import net.forwardfire.vasc.frontend.VascFrontendDataSelector;
import net.forwardfire.vasc.frontend.VascFrontendHelper;
import net.forwardfire.vasc.frontend.VascFrontendPageInfo;
import net.forwardfire.vasc.frontend.VascFrontendUserController;
import net.forwardfire.vasc.frontend.VascFrontendUserSettingController;


/**
 * Stores state data for the frontend
 * 
 * @author Willem Cazander
 * @version 1.0 Mar 21, 2007
 */
public class DefaultVascFrontendController implements VascFrontendControllerLocal {
	
	private VascFrontend vascFrontend = null;
	private VascFrontendActions vascFrontendActions = null;
	private VascFrontendPageInfo vascFrontendPageInfo = null;
	private VascEntryConfigFinalizer vascEntryFinalizer = null;
	private VascFrontendHelper vascFrontendHelper = null;
	private VascEntryResourceResolver vascEntryResourceResolver = null;
	private VascEntryResourceImageResolver vascEntryResourceImageResolver = null;
	private Map<String,String> uiComponents = null;
	private VascController vascController = null;
	private Map<VascEntryFrontendEventListener.VascFrontendEventType,List<VascEntryFrontendEventListener>> vascEntryFrontendEventListeners = null;
	private VascEntryState state = null;
	private VascFrontendDataSelector vascFrontendDataSelector = null;
	private VascFrontendUserController vascFrontendUserController = null;
	private VascFrontendUserSettingController vascFrontendUserSettingController = null;
	
	
	private Map<VascEntryField,VascUIComponent> fieldComps = null;
	private Map<VascEntryField,Object> fieldEditors = null;
	private List<VascEntryFieldValidatorService> validatorServices = null;
	
	public DefaultVascFrontendController() {
		uiComponents = new HashMap<String,String>(8);
		fieldComps = new HashMap<VascEntryField,VascUIComponent>(8);
		fieldEditors = new HashMap<VascEntryField,Object>(8);
		validatorServices = new ArrayList<VascEntryFieldValidatorService>(4);
		vascEntryFrontendEventListeners = new HashMap<VascEntryFrontendEventListener.VascFrontendEventType,List<VascEntryFrontendEventListener>>(10);
	}
			
	/**
	 * @return the vascFrontend
	 */
	@Override
	public VascFrontend getVascFrontend() {
		return vascFrontend;
	}
	
	/**
	 * @param vascFrontend the vascFrontend to set
	 */
	@Override
	public void setVascFrontend(VascFrontend vascFrontend) {
		this.vascFrontend = vascFrontend;
	}

	/**
	 * @return the vascFrontendActions
	 */
	@Override
	public VascFrontendActions getVascFrontendActions() {
		return vascFrontendActions;
	}

	/**
	 * @param vascFrontendActions the vascFrontendActions to set
	 */
	@Override
	public void setVascFrontendActions(VascFrontendActions vascFrontendActions) {
		this.vascFrontendActions = vascFrontendActions;
	}

	/**
	 * @return the vascFrontendPager
	 */
	@Override
	public VascFrontendPageInfo getVascFrontendPageInfo() {
		return vascFrontendPageInfo;
	}

	/**
	 * @param vascFrontendPager the vascFrontendPager to set
	 */
	@Override
	public void setVascFrontendPageInfo(VascFrontendPageInfo vascFrontendPageInfo) {
		this.vascFrontendPageInfo = vascFrontendPageInfo;
	}

	/**
	 * @see net.forwardfire.vasc.core.VascBackendData#getVascEntryFinalizer()
	 */
	public VascEntryConfigFinalizer getVascEntryFinalizer() {
		return vascEntryFinalizer;
	}

	/**
	 * @see net.forwardfire.vasc.core.VascBackendData#setVascEntryFinalizer(net.forwardfire.vasc.core.VascEntryConfigFinalizer)
	 */
	public void setVascEntryFinalizer(VascEntryConfigFinalizer vascEntryFinalizer) {
		this.vascEntryFinalizer=vascEntryFinalizer;
	}

	/**
	 * @return the vascFrontendHelper
	 */
	@Override
	public VascFrontendHelper getVascFrontendHelper() {
		return vascFrontendHelper;
	}

	/**
	 * @param vascFrontendHelper the vascFrontendHelper to set
	 */
	@Override
	public void setVascFrontendHelper(VascFrontendHelper vascFrontendHelper) {
		this.vascFrontendHelper = vascFrontendHelper;
	}

	/**
	 * @return the vascEntryResourceResolver
	 */
	@Override
	public VascEntryResourceResolver getVascEntryResourceResolver() {
		return vascEntryResourceResolver;
	}

	/**
	 * @param vascEntryResourceResolver the vascEntryResourceResolver to set
	 */
	@Override
	public void setVascEntryResourceResolver(VascEntryResourceResolver vascEntryResourceResolver) {
		this.vascEntryResourceResolver = vascEntryResourceResolver;
	}

	@Override
	public VascUIComponent getVascUIComponent(String rendererId) throws VascException {
		ClassLoader cl = Thread.currentThread().getContextClassLoader();
    	if (cl == null) {
    		cl = rendererId.getClass().getClassLoader(); // fallback
    	}
		String componentClass = getVascUIComponentClass(rendererId);
		if (componentClass==null) {
			
			// TODO: auto wire text <-> object converts
			
			componentClass = getVascUIComponentClass(VascUIComponent.VASC_TEXT);
		}
		if (componentClass==null) {
			throw new VascException("No component Class found for frontend UIComponent: "+rendererId);	
		}
		try {
			return (VascUIComponent)cl.loadClass(componentClass).newInstance();
		} catch (Exception e) {
			throw new VascException(e);
		}
	}
	
	/**
	 * @see net.forwardfire.vasc.frontend.VascFrontendController#getVascUIComponent(java.lang.String)
	 */
	@Override
	public String getVascUIComponentClass(String rendererId) {
		return uiComponents.get(rendererId);
	}

	/**
	 * @see net.forwardfire.vasc.frontend.VascFrontendController#putVascUIComponent(java.lang.String, java.lang.String)
	 */
	@Override
	public void putVascUIComponent(String rendererId, String uiComponentClass) {
		uiComponents.put(rendererId, uiComponentClass);
	}

	/**
	 * @return the vascController
	 */
	@Override
	public VascController getVascController() {
		return vascController;
	}

	/**
	 * @param vascController the vascController to set
	 */
	@Override
	public void setVascController(VascController vascController) {
		this.vascController = vascController;
	}

	/**
	 * @see net.forwardfire.vasc.frontend.VascFrontendController#addFieldVascUIComponents(net.forwardfire.vasc.core.VascEntryField, net.forwardfire.vasc.core.ui.VascUIComponent, java.lang.Object)
	 */
	@Override
	public void addFieldVascUIComponents(VascEntryField field,VascUIComponent uiComponent, Object editor) {
		fieldComps.put(field, uiComponent);
		fieldEditors.put(field, editor);
	}
	
	@Override
	public void clearFieldRenderObjects() {
		fieldComps.clear();
		fieldEditors.clear();
	}
	
	/**
	 * @see net.forwardfire.vasc.frontend.VascFrontendController#getFieldRealRenderer(net.forwardfire.vasc.core.VascEntryField)
	 */
	@Override
	public Object getFieldRealRenderer(VascEntryField field) {
		return fieldEditors.get(field);
	}
	
	/**
	 * @see net.forwardfire.vasc.frontend.VascFrontendController#getFieldVascUIComponent(net.forwardfire.vasc.core.VascEntryField)
	 */
	@Override
	public VascUIComponent getFieldVascUIComponent(VascEntryField field) {
		return fieldComps.get(field);
	}
	
	/**
	 * @return the vascEntryResourceImageResolver
	 */
	@Override
	public VascEntryResourceImageResolver getVascEntryResourceImageResolver() {
		return vascEntryResourceImageResolver;
	}
	
	/**
	 * @param vascEntryResourceImageResolver the vascEntryResourceImageResolver to set
	 */
	@Override
	public void setVascEntryResourceImageResolver(VascEntryResourceImageResolver vascEntryResourceImageResolver) {
		this.vascEntryResourceImageResolver = vascEntryResourceImageResolver;
	}
	
	/**
	 * @see net.forwardfire.vasc.frontend.VascFrontendController#addVascValidatorService(net.forwardfire.vasc.core.entry.VascEntryFieldValidatorService)
	 */
	@Override
	public void addVascValidatorService(VascEntryFieldValidatorService validatorService) {
		validatorServices.add(validatorService);
	}
	
	/**
	 * @see net.forwardfire.vasc.frontend.VascFrontendController#getVascValidatorServices()
	 */
	@Override
	public List<VascEntryFieldValidatorService> getVascValidatorServices() {
		return validatorServices;
	}
	
	@Override
	public VascEntryState getVascEntryState() {
		return state;
	}
	
	@Override
	public void setVascEntryState(VascEntryState state) {
		this.state=state;
	}
	
	// TODO move to config
	@Override
	public void initFrontendListeners(VascEntryLocal entry,String frontendType) throws InstantiationException, IllegalAccessException {
		ClassLoader cl = Thread.currentThread().getContextClassLoader();
		if (cl==null) {
			cl = entry.getClass().getClassLoader();
		}
		for (String clazz:entry.getVascEntryFrontendEventListenersByType(frontendType)) {
			VascEntryFrontendEventListener listener;
			try {
				listener = (VascEntryFrontendEventListener)cl.loadClass(clazz).newInstance();
			} catch (ClassNotFoundException e) {
				throw new RuntimeException("Could not load VascEntryFrontendEventListener of: "+clazz);
			}
			addVascEntryFrontendEventListener(listener);
		}
		
		for (String clazz:entry.getVascEntryFrontendActionsByType(frontendType)) {
			Object obj = null;
			try {
				obj = cl.loadClass(clazz).newInstance();
			} catch (ClassNotFoundException e) {
				throw new RuntimeException("Could not load frontend action of: "+clazz);
			}
			if (obj instanceof VascActionLocal) {
				VascActionLocal action = (VascActionLocal)obj;
				String aid = action.getId();
				if (aid==null) {
					throw new IllegalArgumentException("Action has no id: "+action+" in entryId: "+entry.getId());
				}
				if (action.getName()==null) {
					action.setName("vasc.action."+aid+".name");
				}
				if (action.getDescription()==null) {
					action.setDescription("vasc.action."+aid+".description");
				}
				if (action.getImage()==null) {
					action.setImage("vasc.action."+aid+".image");
				}
				if (action.getHelpId()==null) {
					action.setHelpId("vasc.action."+aid+".helpId");
				}
			}
			
			if (obj instanceof RowVascActionLocal) {
				entry.addRowAction((RowVascActionLocal)obj);
			}
			if (obj instanceof ColumnVascActionLocal) {
				entry.addColumnAction((ColumnVascActionLocal)obj);
			}
			if (obj instanceof GlobalVascActionLocal) {
				entry.addGlobalAction((GlobalVascActionLocal)obj);
			}
		}
	}
	
	@Override
	public void addVascEntryFrontendEventListener(VascEntryFrontendEventListener listener) {
		for (VascEntryFrontendEventListener.VascFrontendEventType type:listener.getEventTypes()) {
			List<VascEntryFrontendEventListener> list = vascEntryFrontendEventListeners.get(type);
			if (list==null) {
				list = new ArrayList<VascEntryFrontendEventListener>(10);
				vascEntryFrontendEventListeners.put(type, list);
			}
			list.add(listener);
		}
	}
	
	@Override
	public List<VascEntryFrontendEventListener> getVascEntryFrontendEventListener(VascEntryFrontendEventListener.VascFrontendEventType type) {
		List<VascEntryFrontendEventListener> list = vascEntryFrontendEventListeners.get(type);
		if (list==null) {
			return new ArrayList<VascEntryFrontendEventListener>(0);
		}
		return list;
	}
	
	@Override
	public void fireVascFrontendEvent(VascEntry entry,VascFrontendEventType type, Serializable data) {
		List<VascEntryFrontendEventListener> list = getVascEntryFrontendEventListener(type);
		for (VascEntryFrontendEventListener l:list) {
			l.vascEvent(entry, data);
		}
	}

	/**
	 * @return the vascFrontendDataSelector
	 */
	@Override
	public VascFrontendDataSelector getVascFrontendDataSelector() {
		return vascFrontendDataSelector;
	}

	/**
	 * @param vascFrontendDataSelector the vascFrontendDataSelector to set
	 */
	@Override
	public void setVascFrontendDataSelector(VascFrontendDataSelector vascFrontendDataSelector) {
		this.vascFrontendDataSelector = vascFrontendDataSelector;
	}

	/**
	 * @return the vascFrontendUserController
	 */
	@Override
	public VascFrontendUserController getVascFrontendUserController() {
		return vascFrontendUserController;
	}

	/**
	 * @param vascFrontendUserController the vascFrontendUserController to set
	 */
	@Override
	public void setVascFrontendUserController(VascFrontendUserController vascFrontendUserController) {
		this.vascFrontendUserController = vascFrontendUserController;
	}

	/**
	 * @return the vascFrontendUserSettingController
	 */
	@Override
	public VascFrontendUserSettingController getVascFrontendUserSettingController() {
		return vascFrontendUserSettingController;
	}

	/**
	 * @param vascFrontendUserSettingController the vascFrontendUserSettingController to set
	 */
	@Override
	public void setVascFrontendUserSettingController(VascFrontendUserSettingController vascFrontendUserSettingController) {
		this.vascFrontendUserSettingController = vascFrontendUserSettingController;
	}
}