/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.impl.frontend;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import net.forwardfire.vasc.core.VascEntry;
import net.forwardfire.vasc.core.VascEntryField;
import net.forwardfire.vasc.core.actions.VascAction;
import net.forwardfire.vasc.frontend.VascFrontendDataSelector;

/**
 * DefaultVascFrontendDataSelector orders and filters out vasc data.
 * 
 * @author Willem Cazander
 * @version 1.0 Jun 1, 2012
 */
public class DefaultVascFrontendDataSelector implements VascFrontendDataSelector {

	private VascEntry entry = null;
	
	@Override
	public void init(VascEntry entry) {
		this.entry=entry;
	}
	
	@Override
	public boolean isActionAllowed(VascAction action) {
		return true;
	}
	
	@Override
	public boolean isFieldView(VascEntryField field) {
		if (field.getView()==false) {
			return false;
		}
		return true;
	}
	
	@Override
	public boolean isFieldCreate(VascEntryField field) {
		if (isFieldView(field)==false) {
			return false;
		}
		//VascUserRoleController u = field.getVascEntry().getVascFrontendData().getVascController().getVascUserRoleController();
		//if (field.getRolesCreate()!=null && u.hasRole(field.getRolesCreate())) {
		//	return true;
		//}
		if (field.getCreate()) {
			return true;
		}
		return false;
	}
	
	@Override
	public boolean isFieldEdit(VascEntryField field) {
		if (isFieldView(field)==false) {
			return false;
		}
		if (field.getEdit()) {
			return true;
		}
		return false;
	}
	
	@Override
	public boolean isFieldEditReadOnly(VascEntryField field) {
		if (isFieldView(field)==false) {
			return false;
		}
		if (field.getEditReadOnly()) {
			return true;
		}
		return false;
	}
	
	@Override
	public boolean isFieldList(VascEntryField field) { 
		if (isFieldView(field)==false) {
			return false;
		}
		if (field.getList()) {
			return true;
		}
		return false;
	}
	
	/*
	
			class OrderIndexComparator implements Comparator<VascEntryField> {
			public int compare(VascEntryField v1, VascEntryField v2) {
				return v1.getOrderIndex().compareTo(v2.getOrderIndex());
			}
		}
		Collections.sort(entry.getVascEntryFields(),new OrderIndexComparator());
		
		
				// place primary key in front 
		int index = 0;
		for (VascEntryField vef:entry.getVascEntryFields()) {
			if (entry.getPrimaryKeyFieldId().equals(vef.getId())) {
				break;
			}
			index++;
		}
		if (index==entry.getVascEntryFields().size()) {
			// no primarry key found selecting the first field.
			index=0;
		}
		VascEntryField idField = entry.getVascEntryFields().remove(index);
		entry.getVascEntryFields().add(0, idField);
	
	*/
	
	@Override
	public List<VascEntryField> getFields(final EntryFieldSelectType type) {
		List<VascEntryField> result = new ArrayList<VascEntryField>(50);
		for (VascEntryField field:entry.getVascEntryFields()) {
			
			
			if (EntryFieldSelectType.CREATE.equals(type)) {
				
				if (field.getCreate()!=null && field.getCreate()==false) {
					continue;
				}
				
			} else if (EntryFieldSelectType.EDIT.equals(type)) {
				
				if (field.getEdit()!=null && field.getEdit()==false) {
					continue;
				}
				
			} else if (EntryFieldSelectType.LIST.equals(type)) {
				
				if (field.getList()!=null && field.getList()==false) {
					continue;
				}
			}
			result.add(field);
		}
		Collections.sort(result,new Comparator<VascEntryField>() {
			@Override
			public int compare(VascEntryField o1, VascEntryField o2) {
				
				if (EntryFieldSelectType.CREATE.equals(type)) {
					return o1.getOrderCreate().compareTo(o2.getOrderCreate());
				} else if (EntryFieldSelectType.EDIT.equals(type)) {
					return o1.getOrderEdit().compareTo(o2.getOrderEdit());
				} else {
					// List is default
					return o1.getOrderList().compareTo(o2.getOrderList());
				}
			}
		});
		return result;
	}
	

}