/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.impl.frontend;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import net.forwardfire.vasc.backend.list.VascBackendListRequest;
import net.forwardfire.vasc.core.VascEntry;
import net.forwardfire.vasc.core.entry.VascEntryFrontendEventListener;
import net.forwardfire.vasc.frontend.VascFrontendPageInfoNumber;
import net.forwardfire.vasc.frontend.VascFrontendPageInfo;


/**
 * Default impl of default frontend pagers
 * 
 * @author Willem Cazander
 * @version 1.0 Jan 22, 2012
 */
public class DefaultVascFrontendPager implements VascFrontendPageInfo  {

	//private Logger logger = Logger.getLogger(DefaultVascFrontendPager.class.getName());
	private VascEntry entry = null;
	private List<VascFrontendPageInfoNumber> pagesAll = null;

	@Override
	public void init(VascEntry entry) {
		this.entry=entry;
		pagesAll = new ArrayList<VascFrontendPageInfoNumber>(0);
		entry.getVascFrontendController().addVascEntryFrontendEventListener(new DefaultVascFrontendPagerEventListener());
	}
	
	class DefaultVascFrontendPagerEventListener implements VascEntryFrontendEventListener {
		private static final long serialVersionUID = -6667099892801941650L;
		@Override
		public VascFrontendEventType[] getEventTypes() {
			VascFrontendEventType[] result = {VascEntryFrontendEventListener.VascFrontendEventType.POST_READ};
			return result;
		}
		@Override
		public void vascEvent(VascEntry entry, Serializable data) {
			pagesAll = getTablePagesFromBackend();
		}
	}
	
	@Override
	public long getPageTotalRecordCount() {
		long result = entry.getVascFrontendController().getVascEntryState().getTotalBackendRecords();
		return result;
	}
	
	@Override
	public long getPageSize() {
		long result = entry.getVascFrontendController().getVascEntryState().getVascBackendState().getPageSize();
		return result;
	}
	
	@Override
	public long getPageStartCount() {
		int index = entry.getVascFrontendController().getVascEntryState().getVascBackendState().getPageIndex();
		int pageSize = entry.getVascFrontendController().getVascEntryState().getVascBackendState().getPageSize();
		long result = index*pageSize;
		return result;
	}
	
	@Override
	public long getPageStopCount() {
		int index = entry.getVascFrontendController().getVascEntryState().getVascBackendState().getPageIndex();
		int pageSize = entry.getVascFrontendController().getVascEntryState().getVascBackendState().getPageSize();
		long result = (index*pageSize)+pageSize;
		
		// limit for small result sets.
		if (result>entry.getVascFrontendController().getVascEntryState().getTotalBackendRecords()) {
			result = entry.getVascFrontendController().getVascEntryState().getTotalBackendRecords();
		}
		return result;
	}
	
	@Override
	public boolean getHasPageNextAction() {
		int pageIndex = entry.getVascFrontendController().getVascEntryState().getVascBackendState().getPageIndex();
		pageIndex++;
		// copyed from helper
		Long total = entry.getVascFrontendController().getVascEntryState().getTotalBackendRecords();
		if (total!=null &&  pageIndex>(total/entry.getVascFrontendController().getVascEntryState().getVascBackendState().getPageSize())) {
			return false;
		}
		return true;
	}
	
	@Override
	public boolean getHasPagePreviousAction() {
		int pageIndex = entry.getVascFrontendController().getVascEntryState().getVascBackendState().getPageIndex();
		if (pageIndex==0) {
			return false;
		}
		return true;
	}
	
	@Override
	public boolean getHasOnlySinglePage() {
		int pages = pagesAll.size();
		if (pages==1) {
			return true;
		}
		return false;
	}
	
	@Override
	public boolean getHasExtendedPageMode() {
		int pages = pagesAll.size();
		if (pages>13) {
			return true;
		}
		return false;
	}
	
	@Override
	public boolean getHasExtendedPageModeCenter() {
		if (getHasExtendedPageMode()==false) {
			return false;
		}
		int page = entry.getVascFrontendController().getVascEntryState().getVascBackendState().getPageIndex();
		if (page<5) {
			return false;
		}
		int pages = pagesAll.size();
		if (page>pages-6) {
			return false;
		}
		return true;
	}
	
	@Override
	public List<VascFrontendPageInfoNumber> getTablePagesFromBackend() {
		List<VascFrontendPageInfoNumber> result = new ArrayList<VascFrontendPageInfoNumber>(30);
		VascBackendListRequest state = entry.getVascFrontendController().getVascEntryState().getVascBackendState();
		if (state.getPageSize()==0) {
			return result; // paging disabled
		}
		Long total = entry.getVascFrontendController().getVascEntryState().getTotalBackendRecords();
		if (total==null) {
			return result; // no pages
		}
		int pages = new Long(total/state.getPageSize()).intValue();		
		for (int i=0;i<=pages;i++) {
			VascFrontendPageInfoNumber pn = new VascFrontendPageInfoNumber(i); 
			if (state.getPageIndex()==i) {
				pn.setSelected(true);
			}
			result.add(pn);
		}
		return result;
	}
	
	@Override
	public List<VascFrontendPageInfoNumber> getTablePagesNormal() {
		if (getHasExtendedPageMode()) {
			return new ArrayList<VascFrontendPageInfoNumber>(0);
		} else {
			return pagesAll;
		}
	}
	
	@Override
	public List<VascFrontendPageInfoNumber> getTablePagesExtendedBegin() {
		List<VascFrontendPageInfoNumber> result = new ArrayList<VascFrontendPageInfoNumber>(6);
		if (pagesAll.size()>0) {
			result.add(pagesAll.get(0));
		}
		if (pagesAll.size()>1) {
			result.add(pagesAll.get(1));
		}
		if (pagesAll.size()>2) {
			result.add(pagesAll.get(2));
		}
		if (pagesAll.size()<=3) {
			return result;
		}
		int page = entry.getVascFrontendController().getVascEntryState().getVascBackendState().getPageIndex();
		if (page==2 | page==3 | page==4) {
			if (pagesAll.size()>3) {
				result.add(pagesAll.get(3));
			}
		}
		if (page==3 | page==4) {
			if (pagesAll.size()>4) {
				result.add(pagesAll.get(4));
			}
		}
		if (page==4) {
			if (pagesAll.size()>5) {
				result.add(pagesAll.get(5));
			}
		}
		return result;
	}
	
	@Override
	public List<VascFrontendPageInfoNumber> getTablePagesExtendedEnd() {
		List<VascFrontendPageInfoNumber> result = new ArrayList<VascFrontendPageInfoNumber>(6);
		int pages = pagesAll.size();
		int page = entry.getVascFrontendController().getVascEntryState().getVascBackendState().getPageIndex();
		int off = pages-page;
		
		if (off==5 && pagesAll.size() > pages-6 && pages-6 > 0) {
			result.add(pagesAll.get(pages-6));
		}
		if ((off==4  | off==5) && pagesAll.size() > pages-5 && pages-5 > 0) {
			result.add(pagesAll.get(pages-5));
		}
		if ((off==3 | off==4 | off==5) && pagesAll.size() > pages-4 && pages-6 > 4) {			
			result.add(pagesAll.get(pages-4));
		}
		
		if (pages>4 && pagesAll.size() >= pages-1 && pages-1 > 0) {
			result.add(pagesAll.get(pages-3));
			result.add(pagesAll.get(pages-2));
			result.add(pagesAll.get(pages-1));
		}
		return result;
	}
	
	@Override
	public List<VascFrontendPageInfoNumber> getTablePagesExtendedCenter() {
		List<VascFrontendPageInfoNumber> result = new ArrayList<VascFrontendPageInfoNumber>(3);
		int page = entry.getVascFrontendController().getVascEntryState().getVascBackendState().getPageIndex();
		if (page>0 && pagesAll.size() > page-1) {
			result.add(pagesAll.get(page-1));
		}
		if (pagesAll.size() > page) {
			result.add(pagesAll.get(page));
		}
		if (pagesAll.size() > page+1) {
			result.add(pagesAll.get(page+1));
		}
		return result;
	}
}