/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.impl.jndi;

import javax.naming.InitialContext;
import javax.naming.NamingException;

import net.forwardfire.vasc.core.VascController;
import net.forwardfire.vasc.core.VascControllerProvider;

/**
 * JndiVascControllerProvider gets the VascController from jndi.
 * 
 * @author Willem Cazander
 * @version 1.0 May 16, 2012
 */
public class JndiVascControllerProvider implements VascControllerProvider {

	private String jndiName = null;	
	
	/**
	 * Default constructor.
	 */
	public JndiVascControllerProvider() {
	}
	
	/**
	 * Constructor for in servlets/etc with "someJndiName@this.class.getName()" notation.
	 * @param jndiName
	 */
	public JndiVascControllerProvider(String jndiName) {
		setJndiName(jndiName);
	}
	
	/**
	 * Returns a VascController from jndi context.
	 * @see net.forwardfire.vasc.core.VascControllerProvider#getVascController()
	 */
	@Override
	public VascController getVascController() {
		try {			
			InitialContext context = new InitialContext();
			VascController vascController = (VascController)context.lookup(jndiName);
			if (vascController == null) {
				throw new NullPointerException("Cannot lookup vascController: "+jndiName);
			}
			return vascController;
		} catch (NamingException e) {
			throw new IllegalStateException("Naming error:"+e.getMessage()+" from name: "+jndiName,e);
		}
	}

	/**
	 * @return the jndiName
	 */
	public String getJndiName() {
		return jndiName;
	}

	/**
	 * @param jndiName the jndiName to set
	 */
	public void setJndiName(String jndiName) {
		this.jndiName = jndiName;
	}
}
