/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.impl.type;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import net.forwardfire.vasc.core.VascEntryFieldType;
import net.forwardfire.vasc.core.VascEntryFieldTypeControllerLocal;
import net.forwardfire.vasc.core.VascEntryFieldTypeLocal;
import net.forwardfire.vasc.core.VascException;


/**
 * 
 * @author Willem Cazander
 * @version 1.0 Sep 14, 2008
 */
public class DefaultVascEntryFieldTypeController implements VascEntryFieldTypeControllerLocal {
	
	private Map<String,VascEntryFieldTypeLocal> vascEntryFieldTypes = null;
	
	public DefaultVascEntryFieldTypeController() throws VascException {
		try {
			FieldTypeDriver driver = new FieldTypeDriver();
			vascEntryFieldTypes = new HashMap<String,VascEntryFieldTypeLocal>(35);
			for(VascEntryFieldTypeLocal v:driver.getVascFieldTypes()) {
				vascEntryFieldTypes.put(v.getId(),v);
			}
		} catch (Exception e) {
			throw new VascException(e);
		}
	}
	
	/**
	 * 
	 */
	@Override
	public void addVascEntryFieldType(VascEntryFieldTypeLocal vascEntryFieldType) {
		if (vascEntryFieldType==null) {
			throw new NullPointerException("Can't add null vascEntryFieldType.");
		}
		vascEntryFieldTypes.put(vascEntryFieldType.getId(),vascEntryFieldType);
	}

	/**
	 * @see net.forwardfire.vasc.core.VascEntryFieldTypeController#getVascEntryFieldTypeById(java.lang.String)
	 */
	@Override
	public VascEntryFieldType getVascEntryFieldTypeById(String id) {
		VascEntryFieldTypeLocal result = vascEntryFieldTypes.get(id);
		if (result==null) {
			throw new IllegalArgumentException("Field not found: "+id);
		}
		try {
			return result.clone();
		} catch (CloneNotSupportedException e) {
			throw new IllegalArgumentException("FieldType not clonable: "+e.getMessage());
		}
	}

	
	public VascEntryFieldType getRealVascEntryFieldTypeById(String id) {
		VascEntryFieldType result = vascEntryFieldTypes.get(id);
		if (result==null) {
			throw new IllegalArgumentException("Field not found: "+id);
		}
		return result;
	}
	
	
	/**
	 * @see net.forwardfire.vasc.core.VascEntryFieldTypeController#getVascEntryFieldTypeIds()
	 */
	@Override
	public List<String> getVascEntryFieldTypeIds() {
		List<String> result = new ArrayList<String>(5);
		result.addAll(vascEntryFieldTypes.keySet());
		Collections.sort(result); // lets do abc for consistance behauvior.
		return result;
	}
	
	@Override
	public VascEntryFieldType getVascEntryFieldTypeByClass(Class<?> clazz) {
		for (String typeId: vascEntryFieldTypes.keySet()) {
			VascEntryFieldType type = getVascEntryFieldTypeById(typeId);
			Class<?> autoClass = type.getAutoDetectClass();
			if (autoClass!=null && autoClass.isAssignableFrom(clazz)) {
				return type;
			}
		}
		return null;
	}
}
