/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.impl.type;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.xml.parsers.ParserConfigurationException;

import net.forwardfire.vasc.core.VascEntryFieldTypeLocal;

import org.xml.sax.SAXException;
import org.x4o.xml.X4ODriver;
import org.x4o.xml.X4ODriverManager;
import org.x4o.xml.conv.DefaultObjectConverterProvider;
import org.x4o.xml.conv.ObjectConverter;
import org.x4o.xml.element.Element;
import org.x4o.xml.io.X4OConnectionException;
import org.x4o.xml.io.X4OReaderSession;
import org.x4o.xml.lang.X4OLanguageSession;
import org.x4o.xml.lang.phase.X4OPhaseException;

/**
 * 
 * 
 * @author Willem Cazander
 * @version 1.0 Sep 11, 2008
 */
public class FieldTypeDriver extends X4ODriver<Object> {

	static public final String LANGUAGE_NAME = "fieldtype";
	static public final String[] LANGUAGE_VERSIONS = new String[]{X4ODriver.DEFAULT_LANGUAGE_VERSION};
	
	static public FieldTypeDriver getInstance() {
		return (FieldTypeDriver)X4ODriverManager.getX4ODriver(LANGUAGE_NAME);
	}
	
	@Override
	public String getLanguageName() {
		return LANGUAGE_NAME;
	}

	@Override
	public String[] getLanguageVersions() {
		return LANGUAGE_VERSIONS;
	}
	
	public List<VascEntryFieldTypeLocal> getVascFieldTypes() throws FileNotFoundException, SecurityException, NullPointerException, ParserConfigurationException, SAXException, IOException, X4OPhaseException, X4OConnectionException {
		X4OReaderSession<Object> reader = createReaderSession();
		//reader.setProperty(X4OLanguageProperty.PHASE_SKIP_RELEASE.toUri(), true);
		reader.addPhaseSkip("READ_RELEASE");
		X4OLanguageSession session = reader.readResourceSession("META-INF/fieldtypes.xml");
		//((X4OLanguageSessionLocal)session).addPhaseSkip("READ_RELEASE");// TODO: fixme
		
		List<VascEntryFieldTypeLocal> result = new ArrayList<VascEntryFieldTypeLocal>(40);
		DefaultObjectConverterProvider convProvider = new DefaultObjectConverterProvider(true);
		for (Element e:session.getRootElement().getChilderen()) {
			VascEntryFieldTypeLocal a = (VascEntryFieldTypeLocal)e.getElementObject();
			if (a.getObjectConverter()==null && a.getAutoDetectClass()!=null) {
				ObjectConverter conv = convProvider.getObjectConverterForClass(a.getAutoDetectClass());
				a.setObjectConverter(conv); // move to x4o or finalizer
			}
			result.add(a);
		}
		reader.releaseSession(session);
		return result;
	}
}