/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.impl.type;

import java.io.Serializable;
import java.util.List;

import net.forwardfire.vasc.core.VascEntryField;
import net.forwardfire.vasc.core.ui.VascValueModel;
import net.forwardfire.vasc.core.VascException;


/**
 * Custom FieldType for multi text values.
 * 
 * @author Willem Cazander
 * @version 1.0 Nov 17, 2008
 */
public class MultiTextVascEntryFieldType extends DefaultVascEntryFieldType {

	private static final long serialVersionUID = -5990597417468690400L;

	
	/*
	private Object getIndexValue(VascEntryField entryField,int index) throws VascException {
		Object record = entryField.getVascEntry().getVascFrontendData().getVascEntryState().getEntryDataObject();
		Object value = entryField.getVascEntryFieldValue().getValue(entryField, record);
		
		if (value instanceof List<?>) {
			return ((List<?>)value).get(index);
		}
		if (value instanceof String[]) {
			if (index >= ((String[])value).length ) {
				return "";
			}
			return ((String[])value)[index];
		}
		if (value instanceof String) {
			return ((String)value);
		}
		throw new VascException("Unknown object type");
	}
	
	@SuppressWarnings("unchecked")
	private void setIndexValue(VascEntryField entryField,int index,Object newValue) throws VascException {
		Object record = entryField.getVascEntry().getVascFrontendData().getVascEntryState().getEntryDataObject();
		Object value = entryField.getVascEntryFieldValue().getValue(entryField, record);
		
		if (value instanceof List<?>) {
			((List<Object>)value).set(index, newValue); // TODO: fix @SuppressWarnings here
			return;
		}
		if (value instanceof String[]) {
			
			if (index+1 > ((String[])value).length ) {
				String[] n = new String[index+1];
				for (int i=0;i<((String[])value).length;i++) {
					n[i]= ((String[])value)[i];
				}
				value = n;
			}
			((String[])value)[index] = newValue.toString();
			return;
		}
		if (value instanceof String) {
			value = new String[] { (String)value };
			return;
		}
		throw new VascException("Unknown object type: "+value);
	}
	*/


	/**
	 * @see net.forwardfire.vasc.core.AbstractVascEntryFieldType#getUIComponentCount(net.forwardfire.vasc.core.VascEntryField)
	 */
	@Override
	public int getUIComponentCount(VascEntryField entryField) throws VascException {
		return 1;
		/*
		Object record = entryField.getVascEntry().getVascFrontendData().getVascEntryState().getEntryDataObject();
		Object value = entryField.getVascEntryFieldValue().getValue(entryField, record);
		
		if (value instanceof List<?>) {
			return ((List<?>)value).size()+1;
		}
		if (value instanceof String[]) {
			return ((String[])value).length+1;
		}
		if (value instanceof String) {
			return 1+1;
		}
		throw new VascException("Unknown object type: "+value);
		*/
	}

	/**
	 * @see net.forwardfire.vasc.core.AbstractVascEntryFieldType#provideEditorVascValueModel(int, net.forwardfire.vasc.core.VascEntryField)
	 */
	@Override
	public VascValueModel provideEditorVascValueModel(final int index,final VascEntryField entryField) {
		MultiVascValueModel model = new MultiVascValueModel();
		return model;
		
		/* note: not allowed to use inner classes in backend , that breaks remoting !!
		VascValueModel model = new VascValueModel() {			
			public Object getValue() throws VascException {
				Object value = getIndexValue(entryField,index);
				return value;
			}

			public void setValue(Object value) throws VascException {
				setIndexValue(entryField,index,value);
				super.setValue(value);
			}
		};
		
		return model;
		*/
	}
}
class MultiVascValueModel extends  VascValueModel {
	@Override
	public Serializable getValue() throws VascException {
		Serializable value = super.getValue();
		//System.out.println("value: "+value+" type: "+value.getClass());
		StringBuffer buf = new StringBuffer(100);
		if (value instanceof List<?>) {
			for (Object o:((List<?>)value)) {
				buf.append(o);
				buf.append("\n");
			}
			return buf.toString();
		}
		if (value instanceof String[]) {
			for (Object o:((String[])value)) {
				buf.append(o);
				buf.append("\n");
			}
			return buf.toString();
		}
		if (value instanceof String) {
			return (String)value;
		}
		return value;
	}

	@SuppressWarnings("unchecked")
	@Override
	public void setValue(Serializable editValueObj) throws VascException {
		
		if (super.getValue()==null) {
			super.setValue(editValueObj);
			return;
		}
		
		if ((editValueObj instanceof String)==false) {
			throw new VascException("Can only handle string stuff.");
		}
		String editValue = (String)editValueObj;
		
		String[] editList = editValue.split(",|\t|\n");
		
		Serializable value = super.getValue();
		if (value instanceof List<?>) {
			((List<?>)value).clear();
			for (String o:editList) {
				((List<String>)value).add(o);
			}
			super.setValue(value);
			return;
		}
		if (value instanceof String[]) {
			super.setValue(value);
			return;
		}
		// solo String type
		super.setValue(value);
	}
}