/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.impl.x4o;

import java.io.IOException;
import java.io.Serializable;
import java.io.StringReader;
import java.lang.reflect.Method;
import java.util.Calendar;
import java.util.Collection;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.Set;

import net.forwardfire.vasc.annotations.VascAnnotationParser;
import net.forwardfire.vasc.annotations.VascChoices;
import net.forwardfire.vasc.annotations.VascChoicesSelectItemModel;
import net.forwardfire.vasc.annotations.VascEventListener;
import net.forwardfire.vasc.core.VascController;
import net.forwardfire.vasc.core.VascEntryFieldLocal;
import net.forwardfire.vasc.core.VascEntryFieldType;
import net.forwardfire.vasc.core.VascEntryFieldTypeLocal;
import net.forwardfire.vasc.core.VascEntryLocal;
import net.forwardfire.vasc.impl.DefaultVascEntryField;
import net.forwardfire.vasc.validators.VascValidator;
import net.forwardfire.vasc.validators.VascValidatorClassParser;

import org.x4o.xml.element.AbstractElement;
import org.x4o.xml.element.ElementException;
import org.x4o.xml.lang.X4OLanguageClassLoader;


/**
 * Parses the xml element for auto adding field or defaults of fields.
 * 
 * @author Willem Cazander
 * @version 1.0 Mar 24, 2009
 */
public class AnnotationParserElement extends AbstractElement {

	/**
	 * @see org.x4o.xml.element.AbstractElement#doElementRun()
	 */
	@Override
	public void doElementRun() throws ElementException {
		String className = getAttributes().get("className");
		String addFields = getAttributes().get("addFields");
		Object parentObject = getParent().getElementObject();
		
		Class<?> modelClass;
		try {
			modelClass = X4OLanguageClassLoader.loadClass(className);
		} catch (ClassNotFoundException e) {
			throw new ElementException(e);
		}
		VascAnnotationParser parser = new VascAnnotationParser(); 
		
		if (parentObject instanceof VascEntryLocal) {
			VascEntryLocal entry = (VascEntryLocal)parentObject;
			fillEntry(entry,modelClass,parser);
			
			if (addFields!=null && "false".equals(addFields)) {
				// when false we don't add the fields.
			} else {
				addFields(entry,modelClass);
			}
			for (VascEntryFieldLocal field:entry.getVascEntryFieldsLocal()) {
				fillField(field,modelClass,parser);
			}
			entry.setDisplayNameFieldId(parser.getVascDisplayName(modelClass));
			entry.setPrimaryKeyFieldId(parser.getVascPrimaryKey(modelClass));
			
		} else if (parentObject instanceof VascEntryFieldLocal) {
			VascEntryFieldLocal field = (VascEntryFieldLocal)parentObject;
			fillField(field,modelClass,parser);
		} else {
			throw new ElementException("Unknow parent object type: "+parentObject);
		}
	}
	
	private void fillEntry(VascEntryLocal entry,Class<?> modelClass,VascAnnotationParser parser) throws ElementException {
		
		VascEventListener vc = parser.getVascEventListener(modelClass);
		if (vc!=null) {
			for (String listener:vc.frontendEventListeners()) {
				entry.addVascEntryFrontendEventListener(listener,null);
			}
			for (String listener:vc.backendEventListeners()) {
				entry.addVascEntryBackendEventListener(listener);
			}
		}
		
	}
	
	private void fillField(VascEntryFieldLocal field,Class<?> modelClass,VascAnnotationParser parser) {
		String value = null;
		value = parser.getVascFieldBackendName(modelClass, field.getId());
		if (value!=null && "".equals(value)==false) {
			field.setBackendName(value);
		}

		
		if (field.getDisplayName()==null) {
			field.setDisplayName(	parser.getVascDisplayName			(modelClass, field.getId()));
		}
		if (field.getName()==null) {
			field.setName(			parser.getVascI18nName				(modelClass, field.getId()));
		}
		if (field.getDescription()==null) {
			field.setDescription(	parser.getVascI18nDescription		(modelClass, field.getId()));
		}
		if (field.getImage()==null) {
			field.setImage(			parser.getVascI18nImage				(modelClass, field.getId()));
		}
		if (field.getHelpId()==null) {
			field.setHelpId(		parser.getVascI18nHelpId			(modelClass, field.getId()));
		}
		if (field.getOrder()==null) {
			field.setOrder(			parser.getVascOrder					(modelClass, field.getId()));
		}
		if (field.getOrderCreate()==null) {
			field.setOrderCreate(	parser.getVascOrderCreate			(modelClass, field.getId()));
		}
		if (field.getOrderEdit()==null) {
			field.setOrderEdit(		parser.getVascOrderEdit				(modelClass, field.getId()));
		}
		if (field.getOrderList()==null) {
			field.setOrderList(		parser.getVascOrderList				(modelClass, field.getId()));
		}
		if (field.getCreate()==null) {
			field.setCreate(		parser.getVascFieldCreate			(modelClass, field.getId()));
		}
		if (field.getEdit()==null) {
			field.setEdit(			parser.getVascFieldEdit				(modelClass, field.getId()));
		}
		if (field.getEditReadOnly()==null) {
			field.setEditReadOnly(	parser.getVascFieldEditReadOnly		(modelClass, field.getId()));
		}
		if (field.getEditBlank()==null) {
			field.setEditBlank(		parser.getVascFieldEditBlank		(modelClass, field.getId()));
		}
		if (field.getList()==null) {
			field.setList(			parser.getVascFieldList				(modelClass, field.getId()));
		}
		if (field.getView()==null) {
			field.setView(			parser.getVascFieldView				(modelClass, field.getId()));
		}
		if (field.getOptional()==null) {
			field.setOptional(		parser.getVascFieldOptional			(modelClass, field.getId()));
		}
		if (field.getSortable()==null) {
			field.setSortable(		parser.getVascFieldSortable			(modelClass, field.getId()));
		}
		if (field.getSumable()==null) {
			field.setSumable(		parser.getVascFieldSumable			(modelClass, field.getId()));
			if (field.getSumable()==null) {
				Method methodCall = null;
				// note: model references properties are not resolved yet.
				
				// search for method on bean.
		        for (Method method:modelClass.getMethods()) {
		            if (method.getName().startsWith("get")==false) {  //a bit durty  
		                continue;
		            }
		            if (method.getName().equals("getClass")) {  
		                continue;
		            }
		            if (field.getBackendName()==null) {
		            	if (method.getName().equalsIgnoreCase("get"+field.getId())) {
		            		methodCall = method;
		            		break;
		            	}
		            } else {
		            	if (method.getName().equalsIgnoreCase("get"+field.getBackendName())) {
		            		methodCall = method;
		            		break;
		            	}
		            }
		        }
		       // System.out.println("Found method: "+methodCall);
		        
		        // search for type
		        if (methodCall!=null) {
					Class<?> retType = methodCall.getReturnType();
		            if (retType.isAssignableFrom(Number.class)) {
		            	field.setSumable(true);
		            }
		        }
			}
			if (field.getSumable()==null) {
				field.setSumable(false);
			}
		}
		if (field.getGraphable()==null) {
			field.setGraphable(		parser.getVascFieldGraphable		(modelClass, field.getId()));
			if (field.getGraphable()==null) {
				field.setGraphable(field.getSumable());
			}
		}
		
		if (field.getRolesCreate()==null) {
			field.setRolesCreate(	parser.getVascRolesCreate			(modelClass, field.getId()));
		}
		if (field.getRolesEdit()==null) {
			field.setRolesEdit(		parser.getVascRolesEdit				(modelClass, field.getId()));
		}
		if (field.getRolesEditReadOnly()==null) {
			field.setRolesEditReadOnly(parser.getVascRolesEditReadOnly	(modelClass, field.getId()));
		}
		if (field.getRolesList()==null) {
			field.setRolesList(		parser.getVascRolesList				(modelClass, field.getId()));
		}
		
		
		if (field.getSizeEdit()==null) {
			field.setSizeEdit(		parser.getVascStyleSizeEdit			(modelClass, field.getId()));
		}
		if (field.getSizeList()==null) {
			field.setSizeList(		parser.getVascStyleSizeList			(modelClass, field.getId()));
		}
		if (field.getStyleEdit()==null) {
			field.setStyleEdit(		parser.getVascStyleStyleEdit		(modelClass, field.getId()));
		}
		if (field.getStyleList()==null) {
			field.setStyleList(		parser.getVascStyleStyleList		(modelClass, field.getId()));
		}
		
		VascValidatorClassParser validatorParser = new VascValidatorClassParser();
		
		Class<?> temp = 		parser.getVascFieldTemplateClass		(modelClass, field.getId());
		String tempProp = 		parser.getVascFieldTemplate				(modelClass, field.getId());
		if (temp!=null) {
			if (tempProp==null) {
				tempProp=field.getId();
			}
			if (tempProp.isEmpty()) {
				tempProp=field.getId();
			}
			List<VascValidator> val = validatorParser.getValidatorsByPropertyName(temp, tempProp);
			for (VascValidator v:val) {
				field.addVascValidator(v);
			}	
		}
		
		List<VascValidator> val = validatorParser.getValidatorsByPropertyName(modelClass, field.getId());
		for (VascValidator v:val) {
			field.addVascValidator(v); // todo: merg with already added list of template so we can override.
		}
		
		VascController vascController = VascDriver.getVascController(this.getLanguageSession());
		VascChoices vc = parser.getVascChoices							(modelClass, field.getId());
		if (vc!=null) {
			VascEntryFieldType type = vascController.getVascEntryFieldTypeController().getVascEntryFieldTypeById("ListField");
			type.setDataObject(new VascChoicesSelectItemModel(vc));
			field.setVascEntryFieldType(type);
		}
		if (field.getVascEntryFieldType()==null) {
			String fieldType = parser.getVascFieldType					(modelClass, field.getId());
			if (fieldType!=null) {
				VascEntryFieldTypeLocal type = (VascEntryFieldTypeLocal)vascController.getVascEntryFieldTypeController().getVascEntryFieldTypeById(fieldType);
				field.setVascEntryFieldType(type);
				
				// copy properties
				String[] prop = parser.getVascFieldTypeProperties		(modelClass, field.getId());
				if (prop!=null) {
					Properties properties = new Properties();
					for (String p:prop) {
						try {
							// let properties class parse the key=value pairs.
							properties.load(new StringReader(p));
						} catch (IOException e) {
							e.printStackTrace();
						}	
					}
					for (Object key:properties.keySet()) {
						type.setProperty((String)key, (String)properties.get(key));
					}
				}
			}
		}
		
		if (field.getVascEntryFieldType()==null) {
			
			Method methodCall = null;
			// note: model references properties are not resolved yet.
			
			// search for method on bean.
	        for (Method method:modelClass.getMethods()) {
	            if (method.getName().startsWith("get")==false) {  //a bit durty  
	                continue;
	            }
	            if (method.getName().equals("getClass")) {  
	                continue;
	            }
	            if (field.getBackendName()==null) {
	            	if (method.getName().equalsIgnoreCase("get"+field.getId())) {
	            		methodCall = method;
	            		break;
	            	}
	            } else {
	            	if (method.getName().equalsIgnoreCase("get"+field.getBackendName())) {
	            		methodCall = method;
	            		break;
	            	}
	            }
	        }
	       // System.out.println("Found method: "+methodCall);
	        
	        // search for type
	        if (methodCall!=null) {
				Class<?> returnType = methodCall.getReturnType();
				for (String typeId: vascController.getVascEntryFieldTypeController().getVascEntryFieldTypeIds()) {
					VascEntryFieldType type = vascController.getVascEntryFieldTypeController().getVascEntryFieldTypeById(typeId);
					Class<?> autoClass = type.getAutoDetectClass();
					if (autoClass!=null && autoClass.isAssignableFrom(returnType)) {
						field.setVascEntryFieldType(type);
						break;
					}
				}
	        }
		}
		
		
		if (field.getDefaultValue()==null) {
			String defValue = parser.getVascDefaultValue			(modelClass, field.getId());
			if (defValue!=null) {
				Method methodCall = null;

				// note: model references properties are not resolved yet.
				
				// search for method on bean.
		        for (Method method:modelClass.getMethods()) {
		            if (method.getName().startsWith("get")==false) {  //a bit durty  
		                continue;
		            }
		            if (method.getName().equals("getClass")) {  
		                continue;
		            }
		            if (field.getBackendName()==null) {
		            	if (method.getName().equalsIgnoreCase("get"+field.getId())) {
		            		methodCall = method;
		            		break;
		            	}
		            } else {
		            	if (method.getName().equalsIgnoreCase("get"+field.getBackendName())) {
		            		methodCall = method;
		            		break;
		            	}
		            }
		        }
		       // System.out.println("Found method: "+methodCall);
		        
		        // search for type
		        if (methodCall!=null) {
					Class<?> retType = methodCall.getReturnType();
					try {
						//System.out.println("creating real def value of:  "+defValue+" for: "+retType.getName());
						Serializable defObject = null;
						if (Date.class.equals(retType)) {
							defObject = new Date();
						} else if (Calendar.class.equals(retType)) {
							defObject = Calendar.getInstance();
						} else {
							defObject = (Serializable) retType.getConstructor(String.class).newInstance(defValue);
						}
						//System.out.println("real object: "+defObject); 
						field.setDefaultValue(defObject);
					} catch (Exception e) {
						e.printStackTrace();
					}
		        }
			}
		}
		
	}
	
	private void addFields(VascEntryLocal entry,Class<?> modelClass) {
        for (Method method:modelClass.getMethods()) {
            if (method.getName().startsWith("get")==false) {  //a bit durty  
                continue;
            }
            if (method.getName().equals("getClass")) {  
                continue;
            }
            if (method.getReturnType().isAssignableFrom(Collection.class)) {
            	continue;
            }
            if (method.getReturnType().isAssignableFrom(List.class)) {
            	continue;
            }
            if (method.getReturnType().isAssignableFrom(Set.class)) {
            	continue;
            }
            if (method.getReturnType().isAssignableFrom(Map.class)) {
            	continue;
            }
            
            String fieldId = method.getName().substring(3,4).toLowerCase()+method.getName().substring(4);
    		VascEntryFieldLocal field = new DefaultVascEntryField();
    		field.setId(fieldId);
            
    		if (entry.getVascEntryFieldById(fieldId)==null) {
    			entry.addVascEntryField(field);
    		} else {
    			continue; // already set in xml
    		}
        }
	}
}