/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.impl.x4o;

import net.forwardfire.vasc.core.VascController;
import net.forwardfire.vasc.core.VascEntryController;
import net.forwardfire.vasc.core.VascEntryControllerLocal;
import net.forwardfire.vasc.core.VascEntryGroupLocal;
import net.forwardfire.vasc.core.VascEntryLocal;

import org.x4o.xml.element.AbstractElementConfigurator;
import org.x4o.xml.element.Element;
import org.x4o.xml.element.ElementConfiguratorException;

/**
 * Converts the type to the type object
 * 
 * @author Willem Cazander
 * @version 1.0 Nov 16, 2008
 */
public class VascEntryElementConfigurator extends AbstractElementConfigurator {
	
	/**
	 * @see org.x4o.xml.element.AbstractElementConfigurator#doConfigEndTag(org.x4o.xml.element.Element)
	 */
	@Override
	public void doConfigElement(Element element) throws ElementConfiguratorException {
		Object elementObject = element.getElementObject();
		VascController vascController = VascDriver.getVascController(element.getLanguageSession());
		VascEntryController entryController = vascController.getVascEntryController();
		
		if (entryController instanceof VascEntryControllerLocal) {
			if (elementObject instanceof VascEntryLocal) {
				VascEntryLocal entry = (VascEntryLocal)elementObject;
				((VascEntryControllerLocal)entryController).addVascEntry(entry);
			} else if (elementObject instanceof VascEntryGroupLocal) {
				VascEntryGroupLocal entryGroup = (VascEntryGroupLocal)elementObject;
				((VascEntryControllerLocal)entryController).addVascEntryGroup(entryGroup);
			} else {
				throw new ElementConfiguratorException(this,"ElementObject is unknown instance: "+elementObject);
			}
		} else {
			throw new ElementConfiguratorException(this,"Can not run configurator because we have no access to VascEntryControllerLocal interface.");
		}
	}
}