/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.impl.x4o;

import net.forwardfire.vasc.core.VascEntryLocal;

import org.x4o.xml.element.AbstractElement;
import org.x4o.xml.element.ElementException;

/**
 * Handed the eventListener xml tag.
 * 
 * @author Willem Cazander
 * @version 1.0 Oct 18, 2010
 */
public class VascEventListenerElement extends AbstractElement {

	/**
	 * @see org.x4o.xml.element.AbstractElement#doElementRun()
	 */
	@Override
	public void doElementRun() throws ElementException {
		String clazz = getAttributes().get("class");
		String type = getAttributes().get("type");
		String frontendType = getAttributes().get("frontendType");
		if (clazz==null) {
			throw new ElementException("No class attribute given.");
		}
		if (clazz.isEmpty()) {
			throw new ElementException("class attribute is empty.");
		}
		if (getParent().getElementObject() instanceof VascEntryLocal==false) {
			throw new ElementException("Parent object is not vasc entry.");
		}
		VascEntryLocal v = (VascEntryLocal)getParent().getElementObject();
		if ("backend".equalsIgnoreCase(type)) {
			v.addVascEntryBackendEventListener(clazz);
		} else if ("frontend".equalsIgnoreCase(type)) {
			v.addVascEntryFrontendEventListener(clazz, frontendType);
		} else {
			throw new ElementException("Unsupported type attribute: "+type);	
		}
	}
}