/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.validators;

import java.lang.annotation.Annotation;

/**
 * Validates
 * 
 * @author Willem Cazander
 * @version 1.0 Sep 5, 2008
 */
public class VascIntSizeValidator implements VascValidator {
	
	private static final long serialVersionUID = 5208748030708077917L;
	private int max = Integer.MAX_VALUE;
	private int min = Integer.MIN_VALUE;
	
	/**
	 * @see net.forwardfire.vasc.validators.VascValidator#isObjectValid(java.lang.Object)
	 */
	@Override
	public boolean isObjectValid(Object object) throws VascValidatorException {
		if ((object instanceof Integer)==false) {
			return false; // only Integer can be matched.
		}
		Integer value = (Integer)object;
		if (value > max) {
			return false;
		}
		if (value < min) {
			return false;
		}
		return true;
	}
	
	/**
	 * @see net.forwardfire.vasc.validators.VascValidator#getAnnotationType()
	 */
	@Override
	public Class<?> getAnnotationType() {
		return VascIntSizeValidator.class;
	}
	
	/**
	 * @see net.forwardfire.vasc.validators.VascValidator#initAnnotation(java.lang.annotation.Annotation)
	 */
	@Override
	public void initAnnotation(Annotation annotation) {
		VascIntSize a = VascIntSize.class.cast(annotation);
		setMin(a.min());
		setMax(a.max());
	}
	
	/**
	 * @return the max
	 */
	public int getMax() {
		return max;
	}

	/**
	 * @param max the max to set
	 */
	public void setMax(int max) {
		this.max = max;
	}

	/**
	 * @return the min
	 */
	public int getMin() {
		return min;
	}

	/**
	 * @param min the min to set
	 */
	public void setMin(int min) {
		this.min = min;
	}
	
	@Override
	public VascValidator clone() throws CloneNotSupportedException {
		VascIntSizeValidator result = new VascIntSizeValidator();
		result.min=min;
		result.max=max;
		return result;
	}
}