/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.validators;

import java.lang.annotation.Annotation;

/**
 * Validates
 * 
 * @author Willem Cazander
 * @version 1.0 Sep 5, 2008
 */
public class VascStringLengthValidator implements VascValidator {

	private static final long serialVersionUID = 7676298359239607685L;
	private Integer minLenght = 0;
	private Integer maxLenght = Integer.MAX_VALUE;
	private boolean nullable = false;
	
	/**
	 * @see net.forwardfire.vasc.validators.VascValidator#isObjectValid(java.lang.Object)
	 */
	@Override
	public boolean isObjectValid(Object object) throws VascValidatorException {
		if (nullable && object==null) {
			return true;
		}
		if ((object instanceof String)==false) {
			return false; // only string can be matched.
		}
		String result = (String)object;
		if (getMinLenght()!=null && result.length()<getMinLenght()) {
			return false;
		}
		if (getMaxLenght()!=null && result.length()>getMaxLenght()) {
			return false;
		}
		return true;
	}

	/**
	 * @return the minLenght
	 */
	public Integer getMinLenght() {
		return minLenght;
	}

	/**
	 * @param minLenght the minLenght to set
	 */
	public void setMinLenght(Integer minLenght) {
		this.minLenght = minLenght;
	}

	/**
	 * @return the maxLenght
	 */
	public Integer getMaxLenght() {
		return maxLenght;
	}

	/**
	 * @param maxLenght the maxLenght to set
	 */
	public void setMaxLenght(Integer maxLenght) {
		this.maxLenght = maxLenght;
	}
	
	/**
	 * @return the nullable
	 */
	public boolean isNullable() {
		return nullable;
	}

	/**
	 * @param nullable the nullable to set
	 */
	public void setNullable(boolean nullable) {
		this.nullable = nullable;
	}
	
	@Override
	public Class<?> getAnnotationType() {
		return VascStringLength.class;
	}
	
	/**
	 * @see net.forwardfire.vasc.validators.VascValidator#initAnnotation(java.lang.annotation.Annotation)
	 */
	@Override
	public void initAnnotation(Annotation annotation) {
		VascStringLength a = VascStringLength.class.cast(annotation);
		if (a.min()!=0) {
			setMinLenght(a.min());
		}
		if (a.max()!=0) {
			setMaxLenght(a.max());
		}
		setNullable(a.nullable());
	}
	
	@Override
	public VascValidator clone() throws CloneNotSupportedException {
		VascStringLengthValidator result = new VascStringLengthValidator();
		result.minLenght=minLenght;
		result.maxLenght=maxLenght;
		result.nullable=nullable;
		return result;
	}
}