/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.validators;

import java.lang.annotation.Annotation;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Validates with custum regex.
 * 
 * @author Willem Cazander
 * @version 1.0 Sep 5, 2008
 */
public class VascStringRegexValidator implements VascValidator {
	
	private static final long serialVersionUID = -7077554409967479437L;
	private Pattern pattern = null;
	
	/**
	 * Sets the regulare expression on which this object filters an String.<br>
	 * <br>
	 * When set to <code>Null</code> matching is disabled.<br>
	 * 
	 * @param regex	The regulare expression to match to.
	 */
	public void setRegEx(String regex) {
		pattern = Pattern.compile(regex);
	}
	/**
	 * Returns the current regulare expression. 
	 * @return	The regulare expression.
	 */
	public String getRegEx() {
		if(pattern==null) {
			return null;
		}
		return pattern.pattern();
	}
	
	/**
	 * @see net.forwardfire.vasc.validators.VascValidator#isObjectValid(java.lang.Object)
	 */
	@Override
	public boolean isObjectValid(Object object) throws VascValidatorException {
		if ((object instanceof String)==false) {
			return false; // only string can be matched.
		}
		//	test if regex matches
		Matcher m = pattern.matcher((String)object);
		return m.find();
	}
	
	/**
	 * @see net.forwardfire.vasc.validators.VascValidator#getAnnotationType()
	 */
	@Override
	public Class<?> getAnnotationType() {
		return VascStringRegex.class;
	}

	/**
	 * @see net.forwardfire.vasc.validators.VascValidator#initAnnotation(java.lang.annotation.Annotation)
	 */
	@Override
	public void initAnnotation(Annotation annotation) {
		VascStringRegex a = VascStringRegex.class.cast(annotation);
		setRegEx(a.regex());
	}
	
	@Override
	public VascValidator clone() throws CloneNotSupportedException {
		VascStringRegexValidator result = new VascStringRegexValidator();
		result.pattern=pattern;
		return result;
	}
}