/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.validators;

import java.lang.annotation.Annotation;

/**
 * Validates
 * 
 * TODO:
 * #Japanese postal codes
 * zipJP=^\d{3}-\d{4}$
 * #US postal codes
 * zipUS=^\d{5}\p{Punct}?\s?(?:\d{4})?$
 * #Dutch postal code
 * zipNL=^[0-9]{4}\s*[a-zA-Z]{2}$
 * #Argentinean postal code
 * zipAR=^\d{3}-\d{4}$
 * #Swedish postal code
 * zipSE=^(s-|S-){0,1}[0-9]{3}\s?[0-9]{2}$
 * #Canadian postal code
 * zipCA=^([A-Z]\d[A-Z]\s\d[A-Z]\d)$
 * #UK postal code
 * zipUK=^[a-zA-Z]{1,2}[0-9][0-9A-Za-z]{0,1} {0,1}[0-9][A-Za-z]{2}$
 * 
 * @author Willem Cazander
 * @version 1.0 Sep 5, 2008
 */
public class VascStringZipCodeValidator extends VascStringRegexValidator {

	private static final long serialVersionUID = -8625655465308689744L;
	static private final String ZIP_REGEX = "^[0-9]{4}\\s*[a-zA-Z]{2}$";
	
	/**
	 * The constructor which sets the regex for email validation
	 */
	public VascStringZipCodeValidator() {
		setRegEx(ZIP_REGEX);
	}
	
	/**
	 * @see net.forwardfire.vasc.validators.VascValidator#getAnnotationType()
	 */
	@Override
	public Class<?> getAnnotationType() {
		return VascStringZipCode.class;
	}

	/**
	 * @see net.forwardfire.vasc.validators.VascValidator#initAnnotation(java.lang.annotation.Annotation)
	 */
	@Override
	public void initAnnotation(Annotation annotation) {
		VascStringRegex a = VascStringRegex.class.cast(annotation);
		if (a.regex().equals("null")) {
			return;
		}
		setRegEx(a.regex());
	}
}