/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.validators;

import java.lang.annotation.Annotation;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

/**
 * The VascValidator interface
 * 
 * @author Willem Cazander
 * @version 1.0 Sep 14, 2008
 */
public class VascValidatorClassParser {

	private List<VascValidator> validators = null;
	
	public VascValidatorClassParser() {
		validators = new ArrayList<VascValidator>(20);
		
		validators.add(new VascDateFutureValidator());
		validators.add(new VascDatePastValidator());
		validators.add(new VascIntSizeValidator());
		validators.add(new VascLongSizeValidator());
		validators.add(new VascObjectNotNullValidator());
		validators.add(new VascObjectNullValidator());
		validators.add(new VascObjectUrlValidator());
		validators.add(new VascStringEmailValidator());
		validators.add(new VascStringLengthValidator());
		validators.add(new VascStringRegexValidator());
		validators.add(new VascStringZipCodeValidator());
	}
	
	public List<VascValidator> getValidatorsByPropertyName(Class<?> parseClass,String propertyName) {
		List<VascValidator> result = new ArrayList<VascValidator>(4);
		
        for(Method method:parseClass.getMethods()) {
            if (method.getName().equalsIgnoreCase("get"+propertyName)==false) {  //a bit dirty  
                continue;
            }
            for (VascValidator v:validators) {
            	for (Annotation a:method.getAnnotations()) {
            		if (a.annotationType().equals(v.getAnnotationType())) {
						try {
							VascValidator val = v.clone();
	            			val.initAnnotation(a);
	            			result.add(val);
						} catch (CloneNotSupportedException e) {
						}
            		}
            	}
            }
            break;
        }
		return result;
	}
}