/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.demo.client.swing;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.Serializable;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.JTree;
import javax.swing.ScrollPaneConstants;
import javax.swing.SwingUtilities;
import javax.swing.event.TreeModelListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeNode;

import net.forwardfire.vasc.backend.VascBackendControllerLocal;
import net.forwardfire.vasc.backend.crud.VascBackendCrud;
import net.forwardfire.vasc.core.VascController;
import net.forwardfire.vasc.core.VascEntryConfigControllerLocal;
import net.forwardfire.vasc.core.VascEntryControllerLocal;
import net.forwardfire.vasc.core.VascEntryLocal;
import net.forwardfire.vasc.core.VascException;
import net.forwardfire.vasc.demo.tech.domain.menu.VascMenuController;
import net.forwardfire.vasc.demo.tech.domain.menu.model.VascMenu;
import net.forwardfire.vasc.demo.tech.domain.menu.model.VascMenuGroup;
import net.forwardfire.vasc.ejb3.VascServiceManager;
import net.forwardfire.vasc.ejb3.VascServiceRemoteBackend;
import net.forwardfire.vasc.frontend.swing.SwingPanelIntegration;
import net.forwardfire.vasc.frontend.swing.SwingPanelTabbed;
import net.forwardfire.vasc.impl.DefaultVascFactory;
import net.forwardfire.vasc.lib.i18n.bundle.RootApplicationBundle;

/**
 * JMainPanel is the main panel/window of this demo.
 * 
 * @author Willem Cazander
 * @version 1.0 May 12, 2012
 */
public class JMainPanel extends JPanel {
	
	private static final long serialVersionUID = 5834715323973411147L;
	private VascController vc = null;
	private VascMenuController vmc = null;
	private VascServiceManager vsm = null;
	private SwingPanelIntegration spi = null;
	private JTabbedPane tabPane = null;
	private JTree vascTree = null;
	private JSplitPane treeSplitPane = null;

	
	public JMainPanel(VascServiceManager vsm,VascMenuController vmc) {
		this.vsm = vsm;
		this.vmc = vmc;
		setLayout(new BorderLayout());
		add(createTreeSplit(), BorderLayout.CENTER);
		
		vc = createVascController(vsm);
		rebuildTree();
	}
	
	/**
	 * @return the vascController
	 */
	public VascController createVascController(VascServiceManager vsm) {
		try {
		//	VascUserInfo vui = getVascUserInfo();
			
			
			// get local jvm plugging controller
			VascController c = DefaultVascFactory.getDefaultVascController();
			
			((VascEntryConfigControllerLocal)c.getVascEntryConfigController()).setResourceBundle(RootApplicationBundle.class.getName());
			/*
			DefaultVascEntryController con = new DefaultVascEntryController() {
				public VascEntry getVascEntryById(String id) {
					VascEntry ve = super.getVascEntryById(id);
					// add global listener to make sure we have all LAZY properties of a bean before we goto edit mode.
					ve.addVascEntryEventListener(VascEventType.DATA_SELECT, new RefreshObjectForLazyPropertiesListener());
					return ve;
				}
			};
			((DefaultVascController)c).setVascEntryController(con);
			*/
			
			
			VascServiceManager vascManager = vsm;
			List<String> entryIds = vascManager.getVascEntryIds();
			
			VascEntryControllerLocal localEntryController = (VascEntryControllerLocal)c.getVascEntryController();
			VascBackendControllerLocal localBackendController = (VascBackendControllerLocal)c.getVascBackendController();
			for (String id:entryIds) {
				VascEntryLocal ve = (VascEntryLocal)vascManager.getVascEntry(id);
				String backendId = ve.getBackendId();
				VascBackendCrud vb = new VascServiceRemoteBackend(vascManager,backendId);
				
				/*
				for (String key:ve.getEntryParameterKeys()) {
					Object value = ve.getEntryParameter(key);
					if (value instanceof String==false) {
						continue;
					}
					String paraValue = (String)value;
					if (paraValue.startsWith("userPara")==false) {
						continue;
					}
					String[] ps = paraValue.split(":");
					String type = ps[1];
					String paraTypeValue = ps[2];
					
					SetParameterBackendListener listener = new SetParameterBackendListener(vui.userId,vui.username);
					listener.setName(key);
					listener.setType(type);
					listener.setValue(paraTypeValue);
					
					VascBackendProxyEventExecutor localUserProxy = new VascBackendProxyEventExecutor();
					localUserProxy.addVascEntryBackendEventListener(listener);
					vb = localUserProxy;
				}
				*/
				localBackendController.addVascBackend(vb);
				localEntryController.addVascEntry(ve);
				
				c.getVascEntryConfigController().configVascEntry(c, ve);
			}
			return c;
		} catch (Exception e) {
			throw new RuntimeException("Could not create remote based vasc controller; "+e.getMessage(),e);
		}
	}
	
	private JSplitPane createTreeSplit() {
		JScrollPane sp0 = createTreePane();
		JScrollPane sp1 = createContentPane();
		treeSplitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,sp0,sp1);
		treeSplitPane.setOneTouchExpandable(true);
		treeSplitPane.setResizeWeight(0.2);
		treeSplitPane.setDividerLocation(170);
		sp0.setMinimumSize(new Dimension(200, 400));
		sp1.setMinimumSize(new Dimension(400, 400));
		return treeSplitPane;
	}
	
	class VascTreeModel extends DefaultTreeModel {
		public VascTreeModel(TreeNode root) {
			super(root);
		}

		private static final long serialVersionUID = -7436681803506994277L;

		@Override
		public void addTreeModelListener(TreeModelListener l) {
			super.addTreeModelListener(l);
		}
	}
	
	public void openVascEntry(String vascId) throws VascException {
		VascEntryLocal ee = (VascEntryLocal)vsm.getVascEntry(vascId);
		vc.getVascEntryConfigController().configVascFrontendController(vc, ee);
		spi.createNewVascView(ee);
	}
	
	private JScrollPane createTreePane() {
		
		DefaultMutableTreeNode root = new DefaultMutableTreeNode(new VascTreeNode(VascTreeNodeType.ROOT,"Menu"));
		
		vascTree = new JTree(new VascTreeModel(root));
		vascTree.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseClicked(MouseEvent e) {
				super.mouseClicked(e);
				if (e.getClickCount() == 2 && vascTree.getSelectionModel().isSelectionEmpty()==false) {
					try {
					DefaultMutableTreeNode node = (DefaultMutableTreeNode)vascTree.getSelectionModel().getSelectionPath().getLastPathComponent();
					if (node.getUserObject() instanceof String) {
						return;
					}
					VascTreeNode vascNode = (VascTreeNode)node.getUserObject();
					if (vascNode != null && vsm!=null) {
						if (vascNode.type == VascTreeNodeType.ENTRY) {
							openVascEntry(vascNode.id);
						}
					}
					} catch (Exception ee) {
						ee.printStackTrace();
					}
				}
			}
		});

		JPanel treePanel = new JPanel();
		treePanel.setLayout(new GridLayout(1,0));
		JScrollPane p = createJScrollPane(treePanel);
		p.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		p.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
		treePanel.add(vascTree);
		
		rebuildTree();
		return p;
	}
	
	private JScrollPane createContentPane() {
		JPanel contentPane = new JPanel();
		contentPane.setLayout(new GridLayout(1,0));
		JScrollPane p = createJScrollPane(contentPane);
		
		tabPane = new JTabbedPane();
		spi = new SwingPanelTabbed(tabPane);
		contentPane.add(tabPane);
		
		return p;
	}
	
	private JScrollPane createJScrollPane(JPanel innerPanel) {
		JScrollPane scrollPane = new JScrollPane(innerPanel);
		scrollPane.setBorder(BorderFactory.createEmptyBorder());
		scrollPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);
		scrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
		scrollPane.getVerticalScrollBar().setUnitIncrement(10);
		scrollPane.getHorizontalScrollBar().setUnitIncrement(10);
		//innerPanel.setParentScrollPane(scrollPane);
		return scrollPane;
	}
	
	class VascTreeNode implements Serializable {
		private static final long serialVersionUID = -1177727401194030822L;
		public VascTreeNode() {}
		public VascTreeNode(VascTreeNodeType type,String id) { this.type=type;this.id=id; }
		public VascTreeNode(VascTreeNodeType type,String id,String entryId) { this.type=type;this.id=id;this.entryId=entryId; }
		VascTreeNodeType type;
		String id;
		String entryId;
		@Override
		public String toString() {
			return id;
		}
		
	}
	enum VascTreeNodeType {
		ROOT,
		GROUP,
		ENTRY
	}
	
	public void rebuildTree() {
		

		if (vsm==null) {
			return;
		}
		
		DefaultMutableTreeNode root = (DefaultMutableTreeNode)vascTree.getModel().getRoot();
		root.removeAllChildren();
		
		List<VascMenuGroup> groups = vmc.getFilteredMenuGroup();
		for (VascMenuGroup group:groups) {
			
			DefaultMutableTreeNode groupNode = new DefaultMutableTreeNode(new VascTreeNode(VascTreeNodeType.GROUP,VascDemoSwingClient.getInstance().i18n(group.getTitleKey())));
			for (VascMenu vm:group.getMenus()) {
				DefaultMutableTreeNode entryNode = new DefaultMutableTreeNode(new VascTreeNode(VascTreeNodeType.ENTRY,VascDemoSwingClient.getInstance().i18n(vm.getTitleKey())));
				groupNode.add(entryNode);
			}
			root.add(groupNode);
		}
		
		SwingUtilities.updateComponentTreeUI(vascTree);
		
		/*
		// todo move
		Map<String,String> keys = new HashMap<String,String>(300);
		VascBundleCheckEntryKeys checker = new VascBundleCheckEntryKeys(ResourceBundle.getBundle("net.forwardfire.vasc.lib.i18n.bundle.RootApplicationBundle"));
		for (String veId:vascManager.getVascController().getVascEntryController().getVascEntryIds()) {
			VascEntry ve = vascManager.getVascController().getVascEntryController().getVascEntryById(veId);
			keys.putAll(checker.generateMissingKeys(ve));
		}
		if (keys.isEmpty()==false) {
			Properties p = new Properties();
			File dataDir = new File("data");
			
			if (dataDir.exists()==false) {
				dataDir.mkdirs();
			}
			File resourceFile = new File("data/vasc-bundle.properties");
			if (resourceFile.exists()) {
				readPropertiesFile(p,resourceFile);
			}
			for (String key:keys.keySet()) {
				if (key==null) {
					continue;
				}
				if (keys.get(key)==null) {
					continue;
				}
				p.put(key, keys.get(key));
			}
			writePropertiesFile(p,resourceFile);
			
			ResourceBundle.clearCache();
		}
		*/
	}
	
	public JTabbedPane getTabPane() {
		return tabPane;
	}

	public void changeEvent() {
		rebuildTree();
	}
	
}
