/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.demo.client.swing;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JTabbedPane;

import net.forwardfire.vasc.core.VascException;
import net.forwardfire.vasc.demo.tech.domain.menu.VascMenuController;

/**
 * JMainPanelMenuBar Adds all menu bar items.
 * 
 * @author Willem Cazander
 * @version 1.0 May 12, 2012
 */
public class JMainPanelMenuBar extends JMenuBar implements ActionListener {

	private static final long serialVersionUID = -2828428804621352725L;
	private JMainPanel mainPanel = null;
	//private List<JMenu> vascMenus = null;
	private JMenu clientFileMenu = null;
	private JMenu clientTabMenu = null;
	
	public JMainPanelMenuBar(VascMenuController vmc,JMainPanel mainPanel) {
		this.mainPanel=mainPanel;
		
		clientFileMenu = new JMenu("File");
		add(clientFileMenu);
		
		JMenuItem logoutItem = new JMenuItem("Logout");
		logoutItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				try {
					VascDemoSwingClient.getInstance().logout();
				} catch (Exception e1) {
					e1.printStackTrace();
				}
			}
		});
		clientFileMenu.add(logoutItem);
		clientFileMenu.addSeparator();
		JMenuItem exitItem = new JMenuItem("Exit");
		exitItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				VascDemoSwingClient.getInstance().exit();
			}
		});
		clientFileMenu.add(exitItem);
		
		/*
		List<VascMenuGroup> groups = vmc.getFilteredMenuGroup();
		for (VascMenuGroup group:groups) {
			JMenu menu = new JMenu(group.getTitleKey());
			for (VascMenu vm:group.getMenus()) {
				JMenuItem item = new JMenuItem(vm.getTitleKey());
				item.putClientProperty("vascEntryId", vm.getVascEntryId());
				item.addActionListener(this);
				menu.add(item);
			}
			add(menu);
		}
		*/
		
		clientTabMenu = new JMenu("Tabs");
		add(clientTabMenu);
		JMenuItem item = new JMenuItem("Close tab");
		item.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				JTabbedPane tabPane = getTabPane();
				if (tabPane.getSelectedIndex()>=0) {
					tabPane.removeTabAt(tabPane.getSelectedIndex()); // todo release vasc frontend
				}
			}
		});
		clientTabMenu.add(item);
		JMenuItem itemAll = new JMenuItem("Close All tabs");
		itemAll.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				JTabbedPane tabPane = getTabPane();
				for (int i=tabPane.getTabCount();i>0;i--) {
					tabPane.removeTabAt(i-1); // todo release vasc frontend
				}
			}
		});
		clientTabMenu.add(itemAll);
	}
	
	public JTabbedPane getTabPane() {
		return mainPanel.getTabPane();
	}
	
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() instanceof JMenuItem) {
			JMenuItem item = (JMenuItem)e.getSource();
			String vascId = (String)item.getClientProperty("vascEntryId");
			try {
				mainPanel.openVascEntry(vascId);
			} catch (VascException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
		}
		
	}
}
