/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.demo.client.swing;

import java.awt.Color;
import java.awt.Font;
import java.io.IOException;
import java.io.InputStream;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.MissingResourceException;
import java.util.Properties;
import java.util.ResourceBundle;
import java.util.logging.Logger;

import javax.naming.CommunicationException;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.security.auth.login.LoginContext;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.UIManager;

import net.forwardfire.vasc.demo.tech.domain.menu.VascMenuController;
import net.forwardfire.vasc.demo.tech.domain.user.ClientUserController;
import net.forwardfire.vasc.ejb3.VascServiceManager;
import net.forwardfire.vasc.lib.i18n.bundle.RootApplicationBundle;

import org.jdesktop.application.Application;
import org.jdesktop.application.FrameView;
import org.jdesktop.application.SingleFrameApplication;

/**
 * VascDemoSwingClient is the main swing remote client.
 * 
 * @author Willem Cazander
 * @version 1.0 Nov 10, 2012
 */
public class VascDemoSwingClient extends SingleFrameApplication {

	static protected Context context = null;
	static protected LoginContext loginContext = null;
	private Logger logger = null;
	
	static public void main(String[] args) {
		Application.launch(VascDemoSwingClient.class, new String[] {});
	}
	
	static public VascDemoSwingClient getInstance() {
		return (VascDemoSwingClient)SingleFrameApplication.getInstance();
	}
	
	@Override
	protected void startup() {
		
		logger = Logger.getLogger(VascDemoSwingClient.class.getName());
		installColorsLaF();
		doLogin();
		
		VascServiceManager vc = null;
		ClientUserController cuc = null;
		VascMenuController vmc = null;
		try {
			vc = (VascServiceManager)context.lookup("vascServiceManagerRemote");
			System.out.println("ob: "+vc);
			
			cuc = (ClientUserController)context.lookup("clientUserControllerRemote");
			System.out.println("fll: "+cuc.doClientLogin());
			
			// i18n fixme !!
			RootApplicationBundle bundle = (RootApplicationBundle)ResourceBundle.getBundle(RootApplicationBundle.class.getName());
			bundle.addBundleData(cuc.getResourceBundle(bundle.getLocale()));
			
			vmc = (VascMenuController)context.lookup("vascMenuControllerRemote");
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		JMainPanel mainPanel = new JMainPanel(vc,vmc);
		SingleFrameApplication a = (VascDemoSwingClient)getInstance(); // BIG NOTE because of 'launch' for auto conf swing.
		FrameView mainView = a.getMainView();
		mainView.getFrame().setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		mainView.setComponent(mainPanel);
		mainView.setMenuBar(new JMainPanelMenuBar(vmc,mainPanel));
		mainView.getFrame().setSize(1000, 600);
		mainView.getFrame().setVisible(true);
	}
	
	public void logout() {
		SingleFrameApplication a = (VascDemoSwingClient)getInstance(); // BIG NOTE because of 'launch' for auto conf swing.
		FrameView mainView = a.getMainView();
		mainView.getFrame().setVisible(false);
		
		startup(); // ugly ..
	}
	
	/**
	 * Do an login on the server
	 */
	private void doLogin() {
		VascDemoUserLoginDialog loginHandler = new VascDemoUserLoginDialog();
		loginHandler.setConnectUrl("http://localhost:8899/demo");
		/*
		try {
			loginContext = new LoginContext("vasc-auth-client", loginHandler);
			loginContext.login();
		} catch(Exception e) {
			JOptionPane.showMessageDialog(null, i18n(this,"dialogNoLoginContext"),i18n(this,"dialogTitle"), JOptionPane.WARNING_MESSAGE);
			System.exit(1);
		}
		*/
		
		try {
			loginHandler.handle();
			if(doLoginContext(loginHandler)) {
				return;
			}
			loginHandler.handle();
			if(doLoginContext(loginHandler)) {
				return;
			}
			loginHandler.handle();
			if(doLoginContext(loginHandler)) {
				return;
			}
			
			JOptionPane.showMessageDialog(null, i18n(this,"dialogUserLoginFailure"),i18n(this,"dialogTitle"), JOptionPane.WARNING_MESSAGE);
			System.exit(1);
		} catch (CommunicationException e) {
			List<String> messages = new ArrayList<String>(3);
			messages.add("Could not connect to Vasc Tech Client Swing server, please try again.");
			messages.add("If the problem persists please contact your systems administrator.");
			String connectionError = "Could not obtain connection to any of these urls: ";
			Integer connectionErrorLen = connectionError.length();
			String emsg = e.getMessage();
			if (emsg!=null && emsg.startsWith(connectionError)) {
				Integer urlEndIdx = emsg.substring(connectionErrorLen).indexOf(':');
				if (urlEndIdx>=0) {
					messages.add("Server unreachable at: "+emsg.substring(connectionErrorLen, connectionErrorLen+urlEndIdx));
				}
			}
			JOptionPane.showMessageDialog(null, messages.toArray(), "Vasc Tech Client Swing", JOptionPane.WARNING_MESSAGE);
			System.exit(1);
		} catch (Exception e) {
			//logger.log(Level.WARNING,e.getMessage(),e);
			JOptionPane.showMessageDialog(null, "Failure on login test.", "Vasc Tech Client Swing", JOptionPane.WARNING_MESSAGE);
			System.exit(1);
		}
	}
	
	/**
	 * Create naming context and check if we have 'login' role on server.
	 * @return
	 * @throws Exception
	 */
	private boolean doLoginContext(VascDemoUserLoginDialog loginHandler) throws Exception {
		Properties props = new Properties();
		String connectUrl = loginHandler.getConnectUrl()+"/ejb";
		logger.info("Connecting to: "+connectUrl);
		props.put(Context.PROVIDER_URL, connectUrl ); 
		props.put("openejb.authentication.realmName", "vasc-auth-server");
		props.put(Context.SECURITY_PRINCIPAL,loginHandler.getUsername());
		props.put(Context.SECURITY_CREDENTIALS,loginHandler.getPassword());
		props.put(Context.INITIAL_CONTEXT_FACTORY, "org.apache.openejb.client.RemoteInitialContextFactory");
		try {
			context = new InitialContext(props);
			ClientUserController loginManager = (ClientUserController)context.lookup("clientUserControllerRemote");
			loginManager.doClientLogin();
			return true;
		} catch (javax.naming.NameNotFoundException e) {
			JOptionPane.showMessageDialog(null, "Server is not deployed.\nPlease wait a couple of minuts and try again.");
			return false;
		} catch (Exception e) {
			JOptionPane.showMessageDialog(null, "Server error: "+e.getMessage());
			//e.printStackTrace();
			return false;
		}
	}
	
	private String installColorsLaF() {
		UIManager.put("TabbedPane.font",                Font.decode("SansSerif-BOLD-12"));
		UIManager.put("TitledBorder.font",              Font.decode("SansSerif-BOLD-16"));
		UIManager.put("FireDial.font",                  Font.decode("SansSerif-9"));

		String colorName = "laf-colors";
		ClassLoader cl = Thread.currentThread().getContextClassLoader();
		if (cl==null) {
			cl = this.getClass().getClassLoader();
		}
		InputStream in = cl.getResourceAsStream("net/forwardfire/vasc/demo/client/swing/resources/"+colorName+".properties");
		if (in==null) {
			logger.warning("Color schema not found: "+colorName);
			return "unknown";
		}
		try {
			Properties p = new Properties();
			p.load(in);
			for (Object key:p.keySet()) {
				String value = p.getProperty(key.toString());
				Color colorValue = Color.decode(value);
				UIManager.put(key,colorValue);
			}
		} catch (IOException e) {
			logger.warning("Could not load color schema: "+colorName+" error: "+e.getMessage());
		} finally {
			try {
				in.close();
			} catch (IOException e) {
			}
		}
		
		// Invert focus painters
		List<Object> keys = new ArrayList<Object>(UIManager.getLookAndFeelDefaults().keySet());
		for (Object keyObj:keys) {
			if ((keyObj instanceof String)==false) {
				continue;
			}
			String key = (String)keyObj;
			
			if (key.endsWith("[Focused].backgroundPainter")==false & key.endsWith("[Focused].iconPainter")==false) {
				continue;
			}
			String preKey = key.substring(0,key.indexOf("["));
			String postKey = "backgroundPainter";
			if (key.contains("iconPainter")) {
				postKey = "iconPainter";
			}
			
			logger.finer("Flipping painters of key: "+preKey);
			
			Object focusPainter = UIManager.getLookAndFeelDefaults().get(preKey+"[Focused]."+postKey);
			Object mouseOverPainter = UIManager.getLookAndFeelDefaults().get(preKey+"[MouseOver]."+postKey);
			UIManager.getLookAndFeelDefaults().put(preKey+"[Focused]."+postKey,mouseOverPainter);
			UIManager.getLookAndFeelDefaults().put(preKey+"[MouseOver]."+postKey,focusPainter);
			
			if (key.contains("iconPainter")) {
				Object focusPainterSelected = UIManager.getLookAndFeelDefaults().get(preKey+"[Focused+Selected]."+postKey);
				Object mouseOverPainterSelected = UIManager.getLookAndFeelDefaults().get(preKey+"[MouseOver+Selected]."+postKey);
				UIManager.getLookAndFeelDefaults().put(preKey+"[Focused+Selected]."+postKey,mouseOverPainterSelected);
				UIManager.getLookAndFeelDefaults().put(preKey+"[MouseOver+Selected]."+postKey,focusPainterSelected);
			}
		}
		
		return colorName;
	}
	
	public String i18n(Object obj,String key,Object...params) {
		if(obj==null) { throw new NullPointerException("Can't get key of null obj"); }
		if (key==null) { throw new NullPointerException("key may not be null"); }
		if(obj instanceof Class) {
			key=((Class<?>)obj).getName()+"."+key;
		} else {
			key=obj.getClass().getName()+"."+key;
		}
		return i18n(key,params);
	}
	
	public String i18n(String key,Object...params) {
		try {
			String text = ResourceBundle.getBundle(RootApplicationBundle.class.getName()).getString(key);
			if (params != null) {
				MessageFormat mf = new MessageFormat(text);
				text = mf.format(params, new StringBuffer(), null).toString();
			}
			return text;
		} catch(MissingResourceException e){
			return key;
		}
	}
}
