/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.demo.client.swing;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridLayout;
import java.awt.Image;
import java.awt.RenderingHints;
import java.awt.Toolkit;
import java.awt.event.WindowEvent;
import java.awt.event.WindowFocusListener;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Timer;
import java.util.TimerTask;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JTextField;

/**
 * VascDemoUserLoginDialog used be be true jaas auth module but now only dialog.
 * 
 * @author Willem Cazander
 * @version 1.0 Nov 10, 2012
 */
public class VascDemoUserLoginDialog {

	private static final int JPasswordFieldLen = 8;
	private int loginTry = 0;
	private String connectUrl = null;
	private String username = null;
	private String password = null;
	
	/**
	 * An interface for recording actions to carry out if the user clicks OK for
	 * the dialog.
	 */
	private static interface Action {
		void perform();
	}
	
	/**
	 * Handles the login dialog
	 */
	public void handle() {
		this.loginTry++;
		final JPanel loginDesign = new JPanel();
		loginDesign.setLayout(new BoxLayout(loginDesign, BoxLayout.PAGE_AXIS));
		final Image logoImage = Toolkit.getDefaultToolkit().createImage(getClass().getResource("/net/forwardfire/vasc/demo/client/swing/resources/logo.png"));
		final JPanel logoPanel = new JPanel() {
			private static final long serialVersionUID = 10l;
			protected void paintComponent(Graphics g) {
				Graphics2D g2d = (Graphics2D)g;
				g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
				g2d.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
				g2d.setRenderingHint(RenderingHints.KEY_RENDERING,RenderingHints.VALUE_RENDER_QUALITY);
				
				// Paint the default look of the panel.
				super.paintComponent(g2d);
				
				g2d.setColor(Color.BLACK);
				g2d.fillRect(0,0,getWidth(),getHeight());
				g2d.setColor(new Color(238,212,1));
				
				int w = (getWidth()/2)-(logoImage.getWidth(null)/2);
				int h = (getHeight()/2)-(logoImage.getHeight(null)/2);
				g2d.drawImage(logoImage, w, h, this);
			}
		};
		logoPanel.setPreferredSize(new Dimension(200, 170));
		logoPanel.setBorder(BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.LOWERED));
		loginDesign.add(logoPanel);
		loginDesign.add(Box.createVerticalStrut(5));
		JPanel namePanel = new JPanel();
		JLabel nameLabel = new JLabel("Vasc Demo Client");
		nameLabel.setFont(new Font("Lucida Sans", Font.BOLD, 24));
		namePanel.add(nameLabel);
		loginDesign.add(namePanel);
		loginDesign.add(Box.createVerticalStrut(5));
		
		final List<Action> okActions = new ArrayList<Action>(2);
		final GridLayout messagesLayout = new GridLayout(3,2);
		final JPanel messages = new JPanel();
		messages.setLayout(messagesLayout);
		
		// Add server field
		JLabel serverName = new JLabel("Server:");
		final JTextField serverField = new JTextField(getConnectUrl());
		messages.add(serverName);
		messages.add(serverField);
		okActions.add(new Action() {
			public void perform() {
				connectUrl = serverField.getText();
			}
		});
		
		// hackje for focus
		final JTextField name = new JTextField();
		final JPasswordField pass = new JPasswordField(JPasswordFieldLen);
		
		JLabel prompt = new JLabel("Username:");
		String defaultName = null;
		
		// if (check some property)
		defaultName = "";
		if (defaultName != null) {
			name.setText(defaultName);
		}
		// default to last user name
		if (username !=null ) {
			name.setText(username);
		}
		messages.add(prompt);
		messages.add(name);
		
		/* Store the name back into the callback if OK */
		okActions.add(new Action() {
			public void perform() {
				username = name.getText();
			}
		});

		
		prompt = new JLabel("Password:");
		
		//if (!pc.isEchoOn()) {
			pass.setEchoChar('*');
		//}
		messages.add(prompt);
		messages.add(pass);
		
		okActions.add(new Action() {
			public void perform() {
				password = new String(pass.getPassword());
			}
		});

		if (this.loginTry>1) {
			JPanel messagesPanel = new JPanel();
			messagesPanel.setBorder(BorderFactory.createEtchedBorder());
			JLabel iup = new JLabel("Invalid Username and Password:");
			iup.setForeground(Color.RED);
			messagesPanel.add(iup);
			loginDesign.add(messagesPanel);
			loginDesign.add(Box.createVerticalStrut(5));
		}

		JPanel messagesPanel = new JPanel();
		messagesPanel.setBorder(BorderFactory.createEtchedBorder());
		messagesPanel.add(messages);
		loginDesign.add(messagesPanel);
		/* Display the dialog */
		
		Object[] options = null;
		/// JOptionPane(Object message, int messageType, int optionType, Icon icon, Object[] options, Object initialValue)
		JOptionPane jop = new JOptionPane(loginDesign, JOptionPane.PLAIN_MESSAGE,JOptionPane.OK_CANCEL_OPTION,null,options,name);

		JComponent parentComponent = null;
		JDialog dialog = jop.createDialog(parentComponent,"Vasc Demo Login");

		// We alway request focus after the windows gets focus
		dialog.addWindowFocusListener(new WindowFocusListener() {
			public void windowGainedFocus(WindowEvent arg0) {
				name.requestFocus();
			}
			public void windowLostFocus(WindowEvent arg0) {
			}
		});
		
		// HACK, EVEN the listener above doesn't work 100% so just request focus request
		// until we have focus, then cancel task, see println output
		final Timer lTimer = new Timer();
		lTimer.schedule(new TimerTask() {
			public void run() {
				if(name.hasFocus()==false) {
					//System.out.println("request focus timer task");
					name.requestFocus();
				} else {
					cancel();
				}
			}
		}, 10, 5);
		// wait 10ms
		// execute every 5ms
		
		dialog.setAlwaysOnTop(true);
		dialog.setVisible(true);
		Integer result = (Integer)jop.getValue();
		dialog.dispose();
		
		if (result==null) {
			// is null when escaping by window closing
			result = JOptionPane.CANCEL_OPTION;
		}
				
		/* Perform the OK actions */
		if (result == JOptionPane.OK_OPTION || result == JOptionPane.YES_OPTION) {
			Iterator<Action> iterator = okActions.iterator();
			while (iterator.hasNext()) {
				iterator.next().perform();
			}
		} else if (result == JOptionPane.CANCEL_OPTION || result == JOptionPane.NO_OPTION) {
			System.exit(1);
		}
	}
	

	/**
	 * @return the username
	 */
	public String getUsername() {
		return username;
	}

	/**
	 * @return the password
	 */
	public String getPassword() {
		return password;
	}

	/**
	 * @return the connectUrl
	 */
	public String getConnectUrl() {
		return connectUrl;
	}

	/**
	 * @param connectUrl the connectUrl to set
	 */
	public void setConnectUrl(String connectUrl) {
		this.connectUrl = connectUrl;
	}
}