/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.demo.server.core;

import net.forwardfire.vasc.core.VascController;
import net.forwardfire.vasc.core.VascControllerProvider;
import net.forwardfire.vasc.core.VascEntryConfigControllerLocal;
import net.forwardfire.vasc.core.VascException;
import net.forwardfire.vasc.export.generic.VascEntryExportWriterCsv;
import net.forwardfire.vasc.export.generic.VascEntryExportWriterXml;
import net.forwardfire.vasc.export.jr4o.VascEntryExportWriterJR4O;
import net.forwardfire.vasc.export.json.VascEntryExportWriterJson;
import net.forwardfire.vasc.impl.DefaultVascFactory;
import net.forwardfire.vasc.impl.entry.DefaultVascEntryExport;

/**
 * DemoVascControllerProvider gets the static local jvm vasc controller for this tech demo.
 * 
 * @author Willem Cazander
 * @version 1.0 May 12, 2012
 */
public class VascTechDemoControllerConfig implements VascControllerProvider {

	/**
	 * @see net.forwardfire.vasc.core.VascControllerProvider#getVascController()
	 */
	public VascController getVascController() {
		
		try {
			VascController vascController = DefaultVascFactory.getDefaultVascController();
			
			VascEntryConfigControllerLocal vecc = (VascEntryConfigControllerLocal)vascController.getVascEntryConfigController();
			
			// Config all report export engines for demo.
			// Add all exporters in reverse ORDER
			
			DefaultVascEntryExport export;
			
			export = new DefaultVascEntryExport();
			export.setExportWriterClass(VascEntryExportWriterXml.class.getName());
			export.setId("xml");
			vecc.addVascEntryExporter(export);
			
			export = new DefaultVascEntryExport();
			export.setExportWriterClass(VascEntryExportWriterXml.class.getName());
			export.addWriterParameter("xmltree", "true");
			export.setId("xmltree");
			vecc.addVascEntryExporter(export);
			
			export = new DefaultVascEntryExport();
			export.setExportWriterClass(VascEntryExportWriterCsv.class.getName());
			export.setId("csv");
			vecc.addVascEntryExporter(export);
			
			export = new DefaultVascEntryExport();
			export.setExportWriterClass(VascEntryExportWriterJR4O.class.getName());
			export.addWriterParameter("reportType", "PDF");
			export.addWriterParameter("reportName", "generic-landscape");
			export.addWriterParameter("reportResource", "net/forwardfire/vasc/lib/jr4o/reports/dynamic-reports.xml");
			export.setId("jrPdfLandscape");
			vecc.addVascEntryExporter(export);
			
			export = new DefaultVascEntryExport();
			export.setExportWriterClass(VascEntryExportWriterJR4O.class.getName());
			export.addWriterParameter("reportType", "PDF");
			export.addWriterParameter("reportName", "generic-portrait");
			export.addWriterParameter("reportResource", "net/forwardfire/vasc/lib/jr4o/reports/dynamic-reports.xml");
			export.setId("jrPdfPortrait");
			vecc.addVascEntryExporter(export);
			
			export = new DefaultVascEntryExport();
			export.setExportWriterClass(VascEntryExportWriterJR4O.class.getName());
			export.addWriterParameter("reportType", "RTF");
			export.addWriterParameter("reportName", "generic-landscape");
			export.addWriterParameter("reportResource", "net/forwardfire/vasc/lib/jr4o/reports/dynamic-reports.xml");
			export.setId("jrRtf");
			vecc.addVascEntryExporter(export);
			
			export = new DefaultVascEntryExport();
			export.setExportWriterClass(VascEntryExportWriterJR4O.class.getName());
			export.addWriterParameter("reportType", "XLS");
			export.addWriterParameter("reportName", "generic-landscape");
			export.addWriterParameter("reportResource", "net/forwardfire/vasc/lib/jr4o/reports/dynamic-reports.xml");
			export.setId("jrXls");
			vecc.addVascEntryExporter(export);
			
			export = new DefaultVascEntryExport();
			export.setExportWriterClass(VascEntryExportWriterJR4O.class.getName());
			export.addWriterParameter("reportType", "XML");
			export.addWriterParameter("reportName", "generic-landscape");
			export.addWriterParameter("reportResource", "net/forwardfire/vasc/lib/jr4o/reports/dynamic-reports.xml");
			export.setId("jrXml");
			vecc.addVascEntryExporter(export);
			
			export = new DefaultVascEntryExport();
			export.setExportWriterClass(VascEntryExportWriterJR4O.class.getName());
			export.addWriterParameter("reportType", "CSV");
			export.addWriterParameter("reportName", "generic-landscape");
			export.addWriterParameter("reportResource", "net/forwardfire/vasc/lib/jr4o/reports/dynamic-reports.xml");
			export.setId("jrCsv");
			vecc.addVascEntryExporter(export);
			
			export = new DefaultVascEntryExport();
			export.setExportWriterClass(VascEntryExportWriterJson.class.getName());
			export.setId("json");
			vecc.addVascEntryExporter(export);
			
			// Config root bundle to load all resources.
			vecc.setResourceBundle("net.forwardfire.vasc.lib.i18n.bundle.RootApplicationBundle");
			
			// Increase some config defaults
			vecc.getMasterVascBackendState().setPageSize(200);
			vecc.getMasterVascBackendState().setPageSizeMax(2000);

			return vascController;
		} catch (VascException e) {
			throw new RuntimeException(e);
		}
		
	}
}
