/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.demo.server.core;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.util.Date;
import java.util.logging.Handler;
import java.util.logging.Logger;

import org.apache.commons.io.FileUtils;
import org.slf4j.bridge.SLF4JBridgeHandler;

import net.forwardfire.vasc.demo.server.core.service.DatabaseService;
import net.forwardfire.vasc.demo.server.core.service.ServerConfigService;
import net.forwardfire.vasc.demo.server.core.service.ServerGuiService;
import net.forwardfire.vasc.demo.server.core.service.VascControllerService;
import net.forwardfire.vasc.demo.server.core.service.ServerConfigService.ServerConfigKey;
import net.forwardfire.vasc.demo.server.tomcat.TomcatService;

/**
 * VascTechDemo init and starts the VascTechDemo 
 * 
 * @author Willem Cazander
 * @version 1.0 May 3, 2012
 */ 
public enum VascTechDemoStartup {

	INSTANCE;
	private Logger logger = null;
	private ServerConfigService serverConfigService = null;
	private DatabaseService databaseService = null;
	private TomcatService tomcatService = null;
	private VascControllerService vascControllerService = null;
	private ServerGuiService swingGuiService = null;
	private VascTechDemoBuildInfo buildInfo = null;
	private volatile boolean shutdownOnce = false;
	
	/**
	 * Starts this VascTechDemo instance
	 * @param args
	 */
	static public void main(String[] args) {
		INSTANCE.initialize(args);
		INSTANCE.startup();
	}
	
	/**
	 * Copy copfig/etc directory stucture to root of project in maven/eclipse run.
	 * @throws IOException
	 */
	private void setupAutoDir() throws IOException {
		
		//File deployDir = new File("deploy");
		File workDir = new File("workdir");
		File workDirTmp = new File("workdir/tmp");
		//if (deployDir.exists()==false) {
		//	deployDir.mkdir();
		//}
		if (workDir.exists()==false) {
			workDir.mkdir();
		}
		if (workDirTmp.exists()==false) {
			workDirTmp.mkdir();
		}
		System.setProperty("java.io.tmpdir", workDirTmp.getAbsolutePath());
		File tmpFile = File.createTempFile("test", "new-tmp");
		if (tmpFile.getAbsolutePath().contains(workDirTmp.getName())==false) {
			// Cant change tmp location.
		}
		tmpFile.delete();
		
		File confDir = new File("conf");
		if (confDir.exists()) {
			return;
		}
		if (isMavenRun()==false) {
			throw new RuntimeException("Can't start without conf dir.");
		}
		FileUtils.copyDirectory(new File("../vasc-demo-server-build/src/main/directory/"), new File("."));
	}
	
	/**
	 * Config logging and setup logger object.
	 */
	private void setupLogging() {
		
		// Set Config file property
		if (System.getProperty("logback.configurationFile")==null) {
			File logConfig = null;
			if (isMavenRun()) {
				logConfig = new File("conf/logback-server-console.xml");
			} else {
				logConfig = new File("conf/logback-server.xml");
			}
			try {
				System.setProperty("logback.configurationFile", logConfig.toURI().toURL().toExternalForm());
			} catch (MalformedURLException e) {
				e.printStackTrace();
			}
		}		
		Logger rootLogger = Logger.getAnonymousLogger();
		while (rootLogger.getParent()!=null) {
			rootLogger = rootLogger.getParent();
		}
		for (Handler h:rootLogger.getHandlers()) {
			rootLogger.removeHandler(h);
		}
		rootLogger.addHandler(new SLF4JBridgeHandler()); // This does also the init for us.		
		
		// Logback offical init method.
		//LoggerContext lc = (LoggerContext) LoggerFactory.getILoggerFactory();
		//StatusPrinter.print(lc);
		
		// Create an JUL logger for our application.
		logger = Logger.getLogger(VascTechDemoStartup.class.getName());
		logger.info("Logging is ready for application log;");
	}
	
	
	private void setupBuildInfo() {
		if (buildInfo!=null) {
			return;
		}
		try {
			Class<?> infoClass = Class.forName(VascTechDemoBuildInfo.class.getPackage().getName()+"."+VascTechDemoBuildInfo.class.getSimpleName()+"Impl");
			buildInfo = (VascTechDemoBuildInfo)infoClass.newInstance();
			return;
		} catch (Exception e) {
			logger.warning("Could not load build info impl fallback to local one.");
		}
		buildInfo = new VascTechDemoBuildInfo() {
			@Override
			public String getVersion() {
				return "0.0.0-dev";
			}
			@Override
			public String getBuildDate() {
				return new Date().toString();
			}
		};
	}	
	
	/**
	 * Init all demo service beans.
	 * @param argu
	 */
	protected void initialize(String[] argu) {
		long startTime = System.currentTimeMillis();
		try {
			Thread.currentThread().setName("startup");
			System.setProperty("java.security.auth.login.config", "file:conf/login.config");
			setupAutoDir();
			setupLogging();
			setupBuildInfo();
			logger.info("VascTechDemo initializing ...");
			databaseService = new DatabaseService();
			tomcatService = new TomcatService();
			serverConfigService = new ServerConfigService();
			vascControllerService = new VascControllerService();
			swingGuiService = new ServerGuiService();
			long stopTime = System.currentTimeMillis();
			logger.info("VascTechDemo initialized in "+(stopTime-startTime)+" ms.");
		} catch (Exception e1) {
			e1.printStackTrace();
			System.exit(1);
		}
	}
	
	/**
	 * Startup and deploy all service beans.
	 */
	protected void startup() {
		try {
			long startTime = System.currentTimeMillis();
			//if (serverConfigService.isServerConfigValueTrue(ServerConfigKey.START_GUI)) {
				swingGuiService.start();
			//}
			databaseService.start();
			tomcatService.start();
			serverConfigService.start();
			vascControllerService.start();
			if (serverConfigService.isServerConfigValueTrue(ServerConfigKey.START_EDITOR)) {
				vascControllerService.startEditor();
			}
			tomcatService.deploy(serverConfigService.getServerConfigValue(ServerConfigKey.DEPLOY_PATH));
			if (serverConfigService.isServerConfigValueTrue(ServerConfigKey.DEPLOY_DEBUG)) {
				tomcatService.deployDebug();
			}
			//if (serverConfigService.isServerConfigValueTrue(ServerConfigKey.START_GUI)) {
				swingGuiService.startDone();
			//}
			long stopTime = System.currentTimeMillis();
			logger.info("VascTechDemo startup in "+(stopTime-startTime)+" ms.");
		} catch (Exception e) {
			e.printStackTrace();
			System.exit(1);
		}
	}
	
	public void shutdown() {
		if (shutdownOnce) {
			return;
		}
		shutdownOnce = true;
		Thread t = new Thread(new Runnable() {
			public void run() {
				try {
					long startTime = System.currentTimeMillis();
					vascControllerService.stop();
					serverConfigService.stop();
					tomcatService.stop();
					databaseService.stop();
					
					long stopTime = System.currentTimeMillis();
					logger.info("VascTechDemo shutdown in "+(stopTime-startTime)+" ms.");
					
					swingGuiService.stop();// fixme kills logging
				} catch (Exception e) {
					e.printStackTrace();
					System.exit(1);
				}
			}
		});
		t.setName("shutdown");
		t.start();
	}
	
	static public VascTechDemoStartup getInstance() {
		return INSTANCE;
	}

	public boolean isMavenRun() {
		return System.getProperty("java.class.path").contains("classes");
	}
	
	public VascControllerService getVascControllerService() {
		return vascControllerService;
	}
	
	public TomcatService getTomcatService() {
		return tomcatService;
	}
	
	public ServerGuiService getSwingGuiService() {
		return swingGuiService;
	}
	
	public VascTechDemoBuildInfo getBuildInfo() {
		return buildInfo;
	}
}
