/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.demo.server.core.service;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;

import org.h2.tools.Server;

/**
 * DatabaseService starts and stops the embedded H2 demo database.
 * 
 * @author Willem Cazander
 * @version 1.0 May 13, 2012
 */
public class DatabaseService {

	private Logger logger = null;
	private Server server = null;
	
	public DatabaseService() {
		logger = Logger.getLogger(DatabaseService.class.getName());
	}
	
	public void start() {
		List<String> argu = new ArrayList<String>(10);
		argu.add("-tcp");
		argu.add("-tcpPort"); argu.add("9092");
		argu.add("-tcpPassword"); argu.add("stopSecret");
		argu.add("-baseDir"); argu.add("data/db");		
		String[] args = new String[argu.size()];
		args = argu.toArray(args);
		StringBuffer buf = new StringBuffer();
		for (String a:args) {
			buf.append(a);
			buf.append(" ");
		}
		logger.info("Start H2 Server with: "+buf);
		try {
			server = Server.createTcpServer(args).start();
		} catch (SQLException e) {
			e.printStackTrace();
		}
		if (server.isRunning(true)) {
			initDB();
		}
	}
	
	public void stop() {
		if (server==null) {
			return;
		}
		logger.info("Stopping H2 Server");
		server.stop();
	}
	
	private void initDB() {
		Connection conn = null;
		try {
			ClassLoader cl = Thread.currentThread().getContextClassLoader();
			if (cl==null) {
				cl = this.getClass().getClassLoader();
			}
			cl.loadClass("org.h2.Driver");
			conn = DriverManager.getConnection("jdbc:h2:tcp://localhost:9092/vasc-demo");
			
			ResultSet rs = conn.prepareStatement("show tables").executeQuery();
			if (rs.next()) {
				logger.info("Tables found so skipping auto init.");
				rs.close();
				return;
			}
			rs.close();
			
			String allSql = readResourceAsString("net/forwardfire/vasc/demo/server/core/service/resources/init-db.sql");
			String[] allSqlData = allSql.split(";");
			for(String sql:allSqlData) {
				sql = sql.trim();
				if (sql.isEmpty()) {
					continue;
				}
				//logger.info("exe init sql:"+sql);
				conn.prepareStatement(sql+";").executeUpdate();
			}
			logger.info("Done auto init total statements done: "+allSqlData.length);
			
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			if (conn!=null) {
				try {
					conn.close();
				} catch (SQLException e) {
					e.printStackTrace();
				}
			}
		}
	}
	
	private String readResourceAsString(String resource) throws IOException {
		ClassLoader cl = Thread.currentThread().getContextClassLoader();
		if (cl==null) {
			cl = this.getClass().getClassLoader();
		}
		StringBuffer fileData = new StringBuffer(1000);
		BufferedReader reader = new BufferedReader(new InputStreamReader(cl.getResourceAsStream(resource)));
		try {
			char[] buf = new char[1024];
			int numRead=0;
			while((numRead=reader.read(buf)) != -1){
				String readData = String.valueOf(buf, 0, numRead);
				fileData.append(readData);
				buf = new char[1024];
			}
		} finally {
			if (reader!=null) {
				reader.close();		
			}
		}
		return fileData.toString();
	}
}
