/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.demo.server.core.service;

import javax.naming.Context;
import javax.naming.NamingException;

import net.forwardfire.vasc.demo.server.core.VascTechDemoStartup;

/**
 * ServerConfigService reads demo server config parameters from jndi.
 * 
 * @author Willem Cazander
 * @version 1.0 May 19, 2012
 */
public class ServerConfigService {
	
	public ServerConfigService() {
	}
	
	public void start() {
	}
	
	public void stop() {
	}

	private Object readConfigKey(ServerConfigKey key) {
		try {			
			Context context = VascTechDemoStartup.getInstance().getTomcatService().getServer().getGlobalNamingContext();
			return context.lookup("config/"+key.name());
		} catch (NamingException e) {
			throw new IllegalStateException("Naming error:"+e.getMessage()+" from key: "+key.name(),e);
		}
	}
	
	public String getServerConfigValue(ServerConfigKey key) {
		if (key==null) {
			throw new NullPointerException("Can't get value for null key.");
		}
		Object v = readConfigKey(key);
		if (v==null) {
			return key.getDefaultValue();
		}
		return v.toString();
	}
	
	public boolean isServerConfigValueTrue(ServerConfigKey key) {
		String value = getServerConfigValue(key);
		return "true".equalsIgnoreCase(value);
	}
	
	public enum ServerConfigKey {
		
		START_GUI("true"),
		START_EDITOR("true"),
		DEPLOY_DEBUG("true"),
		DEPLOY_PATH("/demo");
		
		private String defaultValue = null;
		private ServerConfigKey(String defaultValue) {
			this.defaultValue=defaultValue;
		}
		public String getDefaultValue() {
			return defaultValue;
		}
	}
}
