/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.demo.server.core.service;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.lang.reflect.Field;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import java.util.Properties;
import java.util.ResourceBundle;
import java.util.Timer;
import java.util.TimerTask;
import java.util.logging.Logger;

import org.x4o.xml.io.X4OReader;

import com.sun.faces.application.ApplicationAssociate;
import com.sun.faces.application.ApplicationResourceBundle;

import net.forwardfire.vasc.backend.VascBackendControllerLocal;
import net.forwardfire.vasc.core.VascController;
import net.forwardfire.vasc.core.VascEntry;
import net.forwardfire.vasc.core.VascEntryControllerLocal;
import net.forwardfire.vasc.core.VascEntryGroup;
import net.forwardfire.vasc.core.VascEventChannelControllerLocal;
import net.forwardfire.vasc.core.VascEventControllerListener;
import net.forwardfire.vasc.core.VascEventControllerType;
import net.forwardfire.vasc.demo.server.core.VascTechDemoStartup;
import net.forwardfire.vasc.impl.DefaultVascFactory;
import net.forwardfire.vasc.impl.x4o.VascDriver;
import net.forwardfire.vasc.lib.i18n.bundle.RootApplicationBundle;
import net.forwardfire.vasc.test.i18n.VascBundleCheckEntryKeys;

/**
 * VascControllerService manages the demo vasc controller which gets init by jndi factory.
 * 
 * @author Willem Cazander
 * @version 1.0 May 18, 2012
 */
public class VascControllerService {

	private Logger logger = null;
	private VascController vascController = null;
	private Timer delayedI18nTimer;
	private I18NVascEventControllerListener i18nGenerator;
	private static final long TASK_DELAY = 1000*13;
	int bootDone = 20;
	
	public VascControllerService() {
		logger = Logger.getLogger(VascControllerService.class.getName());
		i18nGenerator = new I18NVascEventControllerListener();
		delayedI18nTimer = new Timer("i18n-generator");
		delayedI18nTimer.schedule(new DelayedTask(), TASK_DELAY + TASK_DELAY);
	}
	
	class DelayedTask extends TimerTask {
		@Override
		public void run() {
			if (i18nGenerator.generateRequest!=0) {
				i18nGenerator.generateBundle();
				i18nGenerator.generateRequest=0;
			}
			bootDone--;
			if (bootDone > 0) {
				delayedI18nTimer.schedule(new DelayedTask(), TASK_DELAY);
			}
		}
	};
	
	public void start() {
		logger.finer("Starting vascmanager");
		if (vascController!=null) {
			throw new RuntimeException("VascManager is already started.");
		}
		try {
			// Fetch from jndi
			vascController = (VascController)VascTechDemoStartup.getInstance().getTomcatService().getServer().getGlobalNamingContext().lookup("vasc/server-tech"); 
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
		// Inject our i18n listener for hot deploy demo
		VascEventChannelControllerLocal ev = (VascEventChannelControllerLocal)vascController.getVascEventChannelController();
		ev.addVascEventControllerListener(i18nGenerator);
	}
	
	public void stop() {
		if (vascController==null) {
			return;
		}
		delayedI18nTimer.cancel();
		
		VascBackendControllerLocal backends = (VascBackendControllerLocal)vascController.getVascBackendController();
		backends.clearAndStopBackends();
		
		vascController = null;
		logger.info("Stop manager, cleared all.");
	}
	
	public void startEditor() {
		try {
			vascController.getVascEventChannelController().fireEvent(VascEventControllerType.LOAD_ENTRIES_BEFORE, vascController);
			
			VascDriver driver = VascDriver.getInstance();
			X4OReader<VascController> reader = driver.createReader();
			driver.addVascController(reader, vascController);
			reader.readResource("net/forwardfire/vasc/opt/editor/vasc-editor.xml");
			DefaultVascFactory.fillVascControllerLocalEntries((VascEntryControllerLocal) vascController.getVascEntryController(), vascController);
			vascController.getVascEventChannelController().fireEvent(VascEventControllerType.LOAD_ENTRIES_AFTER, vascController);
			
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			//fireChangeEvent();
		}
	}
	
	
	public VascController getVascController() {
		return vascController;
	}
	
	class I18NVascEventControllerListener implements VascEventControllerListener {
		
		public long generateRequest = 0;
		
		@Override
		public VascEventControllerType[] getVascEventControllerTypes() {
			return new VascEventControllerType[] {VascEventControllerType.LOAD_ENTRIES_AFTER};
		}

		@Override
		public void controllerEvent(VascEventControllerType type,Object eventObject) {
			if (bootDone > 0) {
				generateRequest = System.currentTimeMillis();
			} else {
				generateBundle();
			}
		}
		
		public void generateBundle() {
			logger.info("Regenerating resource bundle keys...");
			
			Map<String,String> keys = new HashMap<String,String>(300);
			VascBundleCheckEntryKeys checker = new VascBundleCheckEntryKeys(ResourceBundle.getBundle("net.forwardfire.vasc.lib.i18n.bundle.RootApplicationBundle"));
			for (String veId:vascController.getVascEntryController().getVascEntryIds()) {
				VascEntry ve = vascController.getVascEntryController().getVascEntryById(veId);
				keys.putAll(checker.generateMissingKeys(ve));
			}
			for (String groupId:vascController.getVascEntryController().getVascEntryGroupIds()) {
				VascEntryGroup veg = vascController.getVascEntryController().getVascEntryGroupById(groupId);
				keys.putAll(checker.generateMissingKeys(veg));
			}
			if (keys.isEmpty()==false) {
				Properties p = new Properties();
				File dataDir = new File("data");
				
				if (dataDir.exists()==false) {
					dataDir.mkdirs();
				}
				File resourceFile = new File("data/vasc-bundle.properties");
				if (resourceFile.exists()) {
					readPropertiesFile(p,resourceFile);
				}
				for (String key:keys.keySet()) {
					if (key==null) {
						continue;
					}
					if (keys.get(key)==null) {
						continue;
					}
					p.put(key, keys.get(key));
				}
				writePropertiesFile(p,resourceFile);
				
				ResourceBundle.clearCache();
				
				//ApplicationResourceBundle appBundle = ApplicationAssociate.getCurrentInstance().getResourceBundles().get(RootApplicationBundle.class.getName());
				//Map<Locale, ResourceBundle> resources = getFieldValue(appBundle, "resources");
				//resources.clear();
			}
		}
		
		@SuppressWarnings("unchecked")
		private <T> T getFieldValue(Object object, String fieldName) {
			try {
				Field field = object.getClass().getDeclaredField(fieldName);
				field.setAccessible(true);
				return (T) field.get(object);
			} catch (Exception e) {
				return null;
			}
		}
		
		protected void writePropertiesFile(Properties p,File file) {
			try {
				writePropertiesStream(p,new FileOutputStream(file));
			} catch (Exception e) {
				throw new RuntimeException("Could not load resource file error: "+e.getMessage(),e);
			}
		}
		
		
		protected void readPropertiesFile(Properties p,File file) {
			try {
				readPropertiesStream(p,new FileInputStream(file));
			} catch (Exception e) {
				throw new RuntimeException("Could not load resource file error: "+e.getMessage(),e);
			}
		}
		
		protected void writePropertiesStream(Properties p,OutputStream out) {
			try {
				p.store(out, "Saved by vasc auto i18n.");
			} catch (Exception e) {
				throw new RuntimeException("Could not load properties error: "+e.getMessage(),e);
			} finally {
				if (out!=null) {
					try {
						out.close();
					} catch (IOException e) {
					}
				}
			}
		}
		
		protected void readPropertiesStream(Properties p,InputStream in) {
			try {
				p.load(in);
			} catch (Exception e) {
				throw new RuntimeException("Could not load properties error: "+e.getMessage(),e);
			} finally {
				if (in!=null) {
					try {
						in.close();
					} catch (IOException e) {
					}
				}
			}
		}
	}
}
