/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.demo.server.tomcat;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.HttpSession;

import org.apache.catalina.connector.Request;
import org.apache.catalina.connector.Response;
import org.apache.catalina.valves.ValveBase;

/**
 * AuthSessionTimeoutValve sets sessions timeout for Sessions which has user pricaiap.
 * 
 * @author Willem Cazander
 * @version 1.0 May 27, 2012
 */
public class AuthSessionTimeoutValve extends ValveBase {

	private boolean logInfo = false;
	private int minTimeout = 60;	
	private int maxTimeout = 60*60;
	private int authTimeout = 60*20;
	
    /**
     * The descriptive information about this implementation.
     */
    protected static final String info = AuthSessionTimeoutValve.class.getName()+"/1.0";
	
	@Override
	public void invoke(Request request, Response response) throws IOException,ServletException {
		getNext().invoke(request, response);
		HttpSession session = request.getSession(false);
		if (session==null) {
			return;
		}
		int curSessionTimeout = session.getMaxInactiveInterval();
		int newSessionTimeout = curSessionTimeout;
		if (curSessionTimeout < minTimeout) {
			newSessionTimeout = minTimeout;
		}
		if (curSessionTimeout > maxTimeout) {
			newSessionTimeout = maxTimeout;
		}
		if (request.getUserPrincipal()!=null) {
			newSessionTimeout = authTimeout;
		}
		if (curSessionTimeout != newSessionTimeout) {
			session.setMaxInactiveInterval(newSessionTimeout);
			logChange(session.getId(),curSessionTimeout,newSessionTimeout);
		}
	}
	
	private void logChange(String sessionId,int curSessionTimeout,int newSessionTimeout) {
		String log = "Changed session: "+sessionId+" from timeout: "+curSessionTimeout+" to: "+newSessionTimeout;
		if (logInfo) {
			getContainer().getLogger().info(log);
		} else {
			getContainer().getLogger().debug(log);
		}
	}

	/**
	 * @return the minTimeout
	 */
	public int getMinTimeout() {
		return minTimeout;
	}

	/**
	 * @param minTimeout the minTimeout to set
	 */
	public void setMinTimeout(int minTimeout) {
		this.minTimeout = minTimeout;
	}

	/**
	 * @return the maxTimeout
	 */
	public int getMaxTimeout() {
		return maxTimeout;
	}

	/**
	 * @param maxTimeout the maxTimeout to set
	 */
	public void setMaxTimeout(int maxTimeout) {
		this.maxTimeout = maxTimeout;
	}

	/**
	 * @return the authTimeout
	 */
	public int getAuthTimeout() {
		return authTimeout;
	}

	/**
	 * @param authTimeout the authTimeout to set
	 */
	public void setAuthTimeout(int authTimeout) {
		this.authTimeout = authTimeout;
	}

	/**
	 * @return the logInfo
	 */
	public boolean isLogInfo() {
		return logInfo;
	}

	/**
	 * @param logInfo the logInfo to set
	 */
	public void setLogInfo(boolean logInfo) {
		this.logInfo = logInfo;
	}
}
