package net.forwardfire.vasc.demo.server.tomcat;

import java.io.File;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.naming.NamingException;

import net.forwardfire.vasc.core.VascController;

import org.apache.catalina.Lifecycle;
import org.apache.catalina.LifecycleEvent;
import org.apache.catalina.LifecycleListener;
import org.apache.catalina.core.StandardServer;

public class JndiVascDeployerListener  implements LifecycleListener {

	private Logger logger = Logger.getLogger(JndiVascDeployerListener.class.getName());
	private JndiVascDeployer deployer = null;
	private String vascControllerName = null;
	private String scanPath = null;
	private int scanTime = 3;
	
	public void lifecycleEvent(LifecycleEvent event) {
		if ((event.getSource() instanceof StandardServer)==false) {
			return;
		}
		try {
			if ( Lifecycle.AFTER_START_EVENT.equals(event.getType())) {
				startDeployer((StandardServer)event.getSource());
			}
			if ( Lifecycle.BEFORE_STOP_EVENT.equals(event.getType())) {
				stopDeployer();
			}
		} catch (Exception e) {
			logger.log(Level.SEVERE, this.getClass().getSimpleName()+" can't control deployer: "+e.getMessage(), e);
		}
	}

	private void startDeployer(StandardServer server) throws NamingException {
		if (deployer!=null) {
			throw new IllegalStateException("Can't start deployer when it is already started.");
		}
		if (getScanPath()==null) {
			throw new NullPointerException("Can't start deployer with null scanPath.");
		}
		File deployDir = new File(getScanPath());
		
		
		VascController vascController = (VascController)server.getGlobalNamingContext().lookup(getVascControllerName());
		if (vascController == null) {
			throw new NullPointerException("Cannot lookup vascController: "+getVascControllerName());
		}
		
		deployer = new JndiVascDeployer();
		deployer.setDeployDir(deployDir);
		deployer.setScanPeriod(getScanTime());
		deployer.setServer(server);
		deployer.setVascController(vascController);
		deployer.start();
	}

	private void stopDeployer() {
		if (deployer==null) {
			return;
		}
		deployer.stop();
	}
	
	/**
	 * @return the vascControllerName
	 */
	public String getVascControllerName() {
		return vascControllerName;
	}

	/**
	 * @param vascControllerName the vascControllerName to set
	 */
	public void setVascControllerName(String vascControllerName) {
		this.vascControllerName = vascControllerName;
	}

	/**
	 * @return the scanPath
	 */
	public String getScanPath() {
		return scanPath;
	}

	/**
	 * @param scanPath the scanPath to set
	 */
	public void setScanPath(String scanPath) {
		this.scanPath = scanPath;
	}

	/**
	 * @return the scanTime
	 */
	public int getScanTime() {
		return scanTime;
	}

	/**
	 * @param scanTime the scanTime to set
	 */
	public void setScanTime(int scanTime) {
		this.scanTime = scanTime;
	}
}
