/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.demo.server.tomcat;

import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.Enumeration;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;
import java.util.logging.Logger;

import net.forwardfire.vasc.demo.server.core.VascTechDemoStartup;

import org.apache.catalina.Context;
import org.apache.catalina.Host;
import org.apache.catalina.Server;
import org.apache.catalina.Service;
import org.apache.catalina.Wrapper;
import org.apache.catalina.core.StandardContext;
import org.apache.catalina.startup.Bootstrap;
import org.apache.catalina.startup.ContextConfig;
import org.apache.naming.resources.VirtualDirContext;


/**
 * TomcatService config and starts Tomcat in semi embedded mode. 
 * 
 * @author Willem Cazander
 * @version 1.0 May 10, 2012
 */
public class TomcatService {

	private Logger logger = null;
	private Server server = null;
	private ClassLoader commonLoader = null;
	private Context applicationContext = null;
	
	public TomcatService() {
		logger = Logger.getLogger(TomcatService.class.getName());
	}
	
	public void start() throws Exception {

		/*
		// List all ejb-jar.xml resources.
		Enumeration<URL> ejbJars = this.getClass().getClassLoader().getResources("META-INF/ejb-jar.xml");
		while (ejbJars.hasMoreElements()) {
			URL url = ejbJars.nextElement();
			System.out.println("app = " + url);
		}
		*/
		
		//we use org.apache.tomee.catalina.ServerListener
		System.setProperty("openejb.servicemanager.enabled", "true");
		
		Bootstrap boot = new Bootstrap();
		boot.setCatalinaHome(System.getProperty("user.dir"));
		boot.init();
		boot.start();
		
		/// After startup get the server object from private method.
		for (Method m:boot.getClass().getDeclaredMethods()) {
			if (m.getName().equals("getServer")) {
				m.setAccessible(true);
				server = (Server)m.invoke(boot);
			}
		}
		if (server==null) {
			throw new RuntimeException("Could not get server by reflection from BootStrap.");
		}
		for (Field f:boot.getClass().getDeclaredFields()) {
			if (f.getName().equals("commonLoader")) {
				f.setAccessible(true);
				commonLoader = (ClassLoader)f.get(boot);
			}
		}
	}

	public void deploy(String deployContext) throws Exception {	
		Service service = server.findService("Catalina");
		Host host = (Host)service.getContainer().findChild("localhost");

		String deployPath = null;
		if (VascTechDemoStartup.getInstance().isMavenRun()) {
			String webappPathLocation = "../vasc-demo-tech-web/src/main/webapp/";
			deployPath = new File(webappPathLocation).getAbsolutePath();
			logger.info("Deploy demo app from workspace path: "+deployPath);
		} else {
			
			File techWarFile = null;
			for (File file:new File("libs").listFiles()) {
				if (file.getName().contains("vasc-demo-tech-web")) {
					techWarFile = file;
					break;
				}
			}
			if (techWarFile==null) {
				throw new NullPointerException("Could not locate war file in lib directory.");
			}
			File destDir = new File("workdir","tomcat.wars"+File.separator+deployContext); //techWarFile.getName()
			
			if (destDir.exists()==false) {
				destDir.mkdirs();			
				JarFile jar = new JarFile(techWarFile);
				Enumeration<JarEntry> jars = jar.entries();
				while (jars.hasMoreElements()) {
					JarEntry file = jars.nextElement();
					File f = new File(destDir+File.separator+file.getName());
					if (file.isDirectory()) {
						f.mkdir();
						continue;
					}
					InputStream is = jar.getInputStream(file);
					FileOutputStream fos = new FileOutputStream(f);
					while (is.available() > 0) {
						fos.write(is.read()); // slow copy
					}
					fos.close();
					is.close();
				}
			}
			deployPath = destDir.getAbsolutePath();
			logger.info("Deploy war path: "+deployPath);
		}
		
		Context ctx = new StandardContext();
		ctx.setName(deployContext);
		ctx.setPath(deployContext);
		ctx.setDocBase(deployPath);
		//ctx.setParentClassLoader(commonLoader);
		ctx.setConfigured(true);
		
		ContextConfig ctxCfg = new ContextConfig();
		ctx.addLifecycleListener(ctxCfg);
		
		//VirtualDirContext vDir = new VirtualDirContext();
		//vDir.setExtraResourcePaths("../vasc-demo-tech-web/target/classes;");
		//ctx.setResources(vDir);

		host.addChild(ctx);
		
		applicationContext = ctx;
	}
	
	public void deployDebug() throws Exception {
		if (applicationContext==null) {
			throw new NullPointerException("Can only deploy debug after deploy.");
			//return;
		}
		long startTime = System.currentTimeMillis();
		
		addServlet(applicationContext,"jdbcConsole","org.h2.server.web.WebServlet");
		applicationContext.addServletMapping("/debug/jdbc/console/*", "jdbcConsole");
		
		addServlet(applicationContext,"jndiView","net.forwardfire.vasc.demo.server.tomcat.JndiDebugServlet");
		applicationContext.addServletMapping("/debug/jndi/view", 		"jndiView");
		applicationContext.addServletMapping("/debug/jndi/view/*",		"jndiView");
		
		addServlet(applicationContext,"logbackClassicStatus","ch.qos.logback.classic.ViewStatusMessagesServlet");
		applicationContext.addServletMapping("/debug/logback/status/classic", "logbackClassicStatus");
		
		addServlet(applicationContext,"logbackAccessStatus","ch.qos.logback.access.ViewStatusMessagesServlet");
		applicationContext.addServletMapping("/debug/logback/status/access", "logbackAccessStatus");
		
		long stopTime = System.currentTimeMillis();
		logger.info("Deployed all debug resources in: "+(stopTime-startTime)+" ms.");
	}
	
	public Wrapper addServlet(Context ctx,String servletName,String servletClass) {
		Wrapper sw = ctx.createWrapper();
		sw.setServletClass(servletClass);
		sw.setName(servletName);
		ctx.addChild(sw);
		return sw;
	}
	
	public void stop() throws Exception {
		if (server==null) {
			return;
		}
		server.stop();
	}
	
	public ClassLoader getClassLoaderCommon() {
		return commonLoader;
	}
	
	public Server getServer() {
		return server;
	}

	/**
	 * @return the applicationContext
	 */
	public Context getApplicationContext() {
		return applicationContext;
	}
}
