/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.demo.server.ui;

import java.util.logging.Handler;
import java.util.logging.LogRecord;
import java.util.logging.Logger;
import java.util.logging.SimpleFormatter;

import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.ScrollPaneConstants;
import javax.swing.SpringLayout;
import javax.swing.SwingUtilities;

/**
 * JConsolePanel binds to JUL and display log messages in texteara.
 * 
 * @author Willem Cazander
 * @version 1.0 May 12, 2012
 */
public class JConsolePanel extends JPanel {
	
	private static final long serialVersionUID = 485766723433479054L;
	private UILogHandler logHandler = null;
	private JTextArea logTextArea = null;
	private int logLinesMax = 255;
	
	public JConsolePanel() {
		setLayout(new SpringLayout());
		setBorder(new JFireBorder("Console Log",this));
		add(createEditor());
		SpringLayoutGrid.makeCompactGrid(this, 1, 1, 6, 6, 6, 6);
		
		Logger rootLogger = Logger.getAnonymousLogger();
		while (rootLogger.getParent()!=null) {
			rootLogger = rootLogger.getParent();
		}
		
		logHandler = new UILogHandler();
		logHandler.setFormatter(new SimpleFormatter());
		rootLogger.addHandler(logHandler);

	}
	
	/**
	 * This needs release if playing the the this tab add/removal very multiple times.
	 */
	public void release() {
		Logger rootLogger = Logger.getAnonymousLogger();
		while (rootLogger.getParent()!=null) {
			rootLogger = rootLogger.getParent();
		}
		rootLogger.removeHandler(logHandler);
	}
	
	private JComponent createEditor() {	
		logTextArea = new JTextArea(6, 80);
		logTextArea.setAutoscrolls(true);
		logTextArea.setEditable(false);
		JScrollPane scrollPane = new JScrollPane(logTextArea);
		scrollPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		scrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
		return scrollPane;
	}
	
	class UILogHandler extends Handler {
		@Override
		public void close() throws SecurityException {	
		}
		@Override
		public void flush() {
		}
		@Override
		public void publish(LogRecord record) {
			final String recordStr = getFormatter().format(record);
			SwingUtilities.invokeLater(new Runnable() {
				public void run() {
					logTextArea.append(recordStr);
					if (logTextArea.getLineCount() > logLinesMax) {
						String t = logTextArea.getText();
						int l = 0;
						int rm = logLinesMax/2;
						for (int i=0;i<rm;i++) {
							int ll = t.indexOf('\n',l+1);
							if (ll==-1) {
								break;
							}
							l = ll;
						}
						String tt = t.substring(l,t.length());
						logTextArea.setText(tt);
					}
					logTextArea.setCaretPosition(logTextArea.getText().length());
				}
			});
		}
	}

}
