package net.forwardfire.vasc.demo.server.ui;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.GradientPaint;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Insets;
import java.awt.RenderingHints;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.JComponent;
import javax.swing.UIManager;
import javax.swing.border.Border;

/**
 * JFireBorder
 * 
 * @author Willem Cazander
 */
public class JFireBorder implements Border,MouseListener {

        private int radius;
        boolean entered = false;
        private JComponent comp = null;
        private String title;
        static private JFireBorder lastFireBorder = null;
        private GradientPaint gradientNormal = null;
        private GradientPaint gradientEntered = null;
        private int gradientWidth = 0;

        public JFireBorder(String title,JComponent comp) {
                this.radius = 10;
                this.comp=comp;
                this.title = title;
                comp.addMouseListener(this);
        }

        public Insets getBorderInsets(Component c) {
                 return new Insets(getTitleHeight(c)+1, 1, radius-2, 1);
        }

        public boolean isBorderOpaque() {
                return true;
        }


        public void paintBorder(Component c, Graphics g, int x, int y, int width, int height) {
                Graphics2D g2 = (Graphics2D)g;
                g2.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
                int titleHeight = getTitleHeight(c);

                if (gradientWidth!=width) {
                        gradientNormal = null;
                        gradientEntered = null;
                }
                gradientWidth = width;

                Color endColor = UIManager.getColor("control");
                if (endColor==null) {
                        endColor = c.getBackground();
                }
                Color startColor = UIManager.getColor("nimbusBorder");
                if (startColor==null) {
                        startColor = c.getForeground();
                }

                //BufferedImage titleImage = GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice().getDefaultConfiguration().createCompatibleImage(width, height, Transparency.TRANSLUCENT);
                if (gradientNormal==null) {
                        gradientNormal = new GradientPaint(
                                        0, 0, startColor,
                                        titleHeight*5, width/3,endColor,
                                        false);
                }
                if (gradientEntered==null) {
                        startColor = UIManager.getColor("nimbusFocus");
                        if (startColor==null) {
                                startColor = c.getForeground();
                        }
                        gradientEntered = new GradientPaint(
                                        0, 0, startColor,
                                        titleHeight*5, width/3,endColor,
                                        false);
                }
                if (entered) {
                        g2.setPaint(gradientEntered);
                } else {
                        g2.setPaint(gradientNormal);
                }
                g2.fillRoundRect(x, y, width, titleHeight, radius, radius);
                g2.fillRect(x,titleHeight/2,radius+1, titleHeight/2);
                g2.drawRoundRect(x,y,width-1,height-1,radius,radius);

                if (title==null) {
                        return;
                }
                Font font = UIManager.getFont("TitledBorder.font");
                g2.setColor(c.getForeground());
                FontMetrics metrics = c.getFontMetrics(font);
                g2.setFont(font);
                g2.drawString(title,x+8,y+(titleHeight-metrics.getHeight())/2 +metrics.getAscent()); 
        }

        protected int getTitleHeight(Component c) {
                Font font = UIManager.getFont("TitledBorder.font");
                FontMetrics metrics = c.getFontMetrics(font);
                return (int)(metrics.getHeight() * 1.40);
        }

        public GradientPaint getGradient() {
                if (entered) {
                        return gradientEntered;
                } else {
                        return gradientNormal;
                }
        }

        @Override
        public void mouseClicked(MouseEvent e) {
        }
        @Override
        public void mouseEntered(MouseEvent e) {
                if (entered==true && lastFireBorder==this) {
                        return;
                }
                entered = true;
                if (comp!=null) {
                        comp.repaint();
                        if (lastFireBorder!=null) {
                                lastFireBorder.entered = false;
                                lastFireBorder.comp.repaint();
                        }
                        lastFireBorder = this;
                }
        }
        @Override
        public void mouseExited(MouseEvent e) {
        }
        @Override
        public void mousePressed(MouseEvent e) {
        }
        @Override
        public void mouseReleased(MouseEvent e) {
        }
}
