/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.demo.server.ui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.lang.management.ManagementFactory;
import java.lang.management.ThreadInfo;
import java.lang.management.ThreadMXBean;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.SpringLayout;

import org.apache.catalina.Context;
import org.apache.catalina.Server;
import org.apache.catalina.Service;
import org.apache.catalina.connector.Connector;

import net.forwardfire.vasc.core.VascController;
import net.forwardfire.vasc.demo.server.core.VascTechDemoStartup;
import net.forwardfire.vasc.demo.server.ui.load.JLoadDialog;
import net.forwardfire.vasc.demo.server.ui.load.JLoadDialog.LoadType;

/**
 * JStatusPanel is the main panel/window of this demo.
 * 
 * @author Willem Cazander
 * @version 1.0 May 12, 2012
 */
public class JStatusPanel extends JPanel implements ActionListener {
	
	private static final long serialVersionUID = 5834715323973411147L;
	private JButton stopButton = null;
	private JButton playButton = null;
	private JButton shutdownButton = null;
	private JButton restartButton = null;
	private JButton importJdbcButton = null;
	private JButton importMongoButton = null;
	private JButton importDataFileButton = null;
	private JButton importDataPathButton = null;
	private JLabel infoStatus = null;
	private JLabel infoThreads = null;
	private JLabel infoWorkers = null;
	private JLabel infoSessions = null;
	private JLabel infoHttpPort = null;
	private JLabel infoVascGroups = null;
	private JLabel infoVascEntries = null;
	private JLabel infoVascBackends= null;
	
	public JStatusPanel() { 
		setLayout(new SpringLayout());
		
		JPanel main = new JPanel();
		main.setLayout(new SpringLayout());
		main.add(createPanelInfoServer());
		main.add(createPanelInfoVasc());
		main.add(createPanelAction());
		main.add(createPanelImport());
		SpringLayoutGrid.makeCompactGrid(main, 2,2,0,0,6,6);
		
		JConsolePanel consolePanel = new JConsolePanel();
		add(main);
		add(consolePanel);
		SpringLayoutGrid.makeCompactGrid(this, 2,1,6,6,0,0);
	}

	private JPanel createPanelInfoVasc() {
		JPanel statusPanel = new JPanel();
		statusPanel.setBorder(new JFireBorder("Info Vasc",statusPanel));
		statusPanel.setLayout(new SpringLayout());
		
		infoVascGroups = new JLabel("0");
		infoVascEntries = new JLabel("0");
		infoVascBackends = new JLabel("0");
		
		statusPanel.add(new JLabel("Artifact:"));
		statusPanel.add(new JLabel("vasc-demo-server-core"));
		
		statusPanel.add(new JLabel("Version:"));
		statusPanel.add(new JLabel(VascTechDemoStartup.getInstance().getBuildInfo().getVersion()));
		
		statusPanel.add(new JLabel("Groups:"));
		statusPanel.add(infoVascGroups);
		
		statusPanel.add(new JLabel("Entries:"));
		statusPanel.add(infoVascEntries);
		
		statusPanel.add(new JLabel("Backends:"));
		statusPanel.add(infoVascBackends);
		
		SpringLayoutGrid.makeCompactGrid(statusPanel, 5, 2);
		return statusPanel;
	}
	
	private JPanel createPanelInfoServer() {
		JPanel statusPanel = new JPanel();
		statusPanel.setBorder(new JFireBorder("Info Server",statusPanel));
		statusPanel.setLayout(new SpringLayout());
		
		infoStatus = new JLabel("booting");
		infoThreads = new JLabel("0");
		infoWorkers = new JLabel("0");
		infoSessions = new JLabel("0");
		infoHttpPort = new JLabel("0");
		
		statusPanel.add(new JLabel("Running:"));
		statusPanel.add(infoStatus);
		
		statusPanel.add(new JLabel("HttpPort:"));
		statusPanel.add(infoHttpPort);
		
		statusPanel.add(new JLabel("Threads:"));
		statusPanel.add(infoThreads);
		
		statusPanel.add(new JLabel("Workers:"));
		statusPanel.add(infoWorkers);
		
		statusPanel.add(new JLabel("Sessions:"));
		statusPanel.add(infoSessions);
		
		SpringLayoutGrid.makeCompactGrid(statusPanel, 5, 2);
		return statusPanel;
	}

	private JPanel createPanelImport() {
		JPanel statusPanel = new JPanel();
		statusPanel.setBorder(new JFireBorder("Import",statusPanel));
		statusPanel.setLayout(new SpringLayout());
		
		importJdbcButton = new JButton("Jdbc");
		importMongoButton = new JButton("Mongo");
		importDataFileButton = new JButton("DataFile");
		importDataPathButton = new JButton("DataPath");
		
		importJdbcButton.addActionListener(this);
		importMongoButton.addActionListener(this);
		importDataFileButton.addActionListener(this);
		importDataPathButton.addActionListener(this);
		
		importJdbcButton.setEnabled(false);
		importMongoButton.setEnabled(false);
		importDataFileButton.setEnabled(false);
		importDataPathButton.setEnabled(false);
		
		statusPanel.add(new JLabel("Import Jdbc:"));
		statusPanel.add(importJdbcButton);
		
		statusPanel.add(new JLabel("Import Mongo:"));
		statusPanel.add(importMongoButton);
		
		statusPanel.add(new JLabel("Import File:"));
		statusPanel.add(importDataFileButton);
		
		statusPanel.add(new JLabel("Data Directory:"));
		statusPanel.add(importDataPathButton);
		
		SpringLayoutGrid.makeCompactGrid(statusPanel,2,4,6,6,0,0);
		return statusPanel;
	}
	
	private JPanel createPanelAction() {
		JPanel actionPanel = new JPanel();
		actionPanel.setBorder(new JFireBorder("Actions",actionPanel));
		actionPanel.setLayout(new SpringLayout());
		
		actionPanel.add(new JLabel("Catalina:"));
		
		playButton = new JButton("Play");
		playButton.setEnabled(false);
		playButton.addActionListener(this);
		actionPanel.add(playButton);
		
		stopButton = new JButton("Stop");
		stopButton.setEnabled(false);
		stopButton.addActionListener(this);
		actionPanel.add(stopButton);
		
		actionPanel.add(new JLabel("Application:"));
		
		restartButton = new JButton("Restart");
		restartButton.setEnabled(false);
		restartButton.addActionListener(this);
		actionPanel.add(restartButton);
		
		shutdownButton = new JButton("Shutdown");
		shutdownButton.setEnabled(false);
		shutdownButton.addActionListener(this);
		actionPanel.add(shutdownButton);
		
		SpringLayoutGrid.makeCompactGrid(actionPanel,2,3,6,6,0,0);
		return actionPanel;
	}
	
	public void startupDone() {
		infoStatus.setText("running");
		
		//restartButton.setEnabled(true);
		shutdownButton.setEnabled(true);
		
		importJdbcButton.setEnabled(true);
		importMongoButton.setEnabled(true);
		//importDataFileButton.setEnabled(true);
		//importDataPathButton.setEnabled(true);
		
		updateInfo();
	}
	
	@Override
	public void actionPerformed(ActionEvent e) {
		if (e.getSource().equals(shutdownButton)) {
			int confirmResult = JOptionPane.showConfirmDialog(ServerGuiApplication.getInstance().getMainFrame(), "Are you sure to exit ?");
			if (confirmResult==JOptionPane.YES_OPTION) {
				VascTechDemoStartup.getInstance().shutdown();
			}
			return;
		} else if (e.getSource().equals(importJdbcButton)) {
			JLoadDialog dialog = new JLoadDialog(ServerGuiApplication.getInstance().getMainFrame(),LoadType.JDBC);
			dialog.setVisible(true);
		} else if (e.getSource().equals(importMongoButton)) {
			JLoadDialog dialog = new JLoadDialog(ServerGuiApplication.getInstance().getMainFrame(),LoadType.MONGODB);
			dialog.setVisible(true);
		} else if (e.getSource().equals(importDataFileButton)) {
			JLoadDialog dialog = new JLoadDialog(ServerGuiApplication.getInstance().getMainFrame(),LoadType.JDBC);
			dialog.setVisible(true);
		} else if (e.getSource().equals(importDataPathButton)) {
			JLoadDialog dialog = new JLoadDialog(ServerGuiApplication.getInstance().getMainFrame(),LoadType.JDBC);
			dialog.setVisible(true);
		}
	}
	
	public void updateInfo() {
		
		VascController vc = VascTechDemoStartup.getInstance().getVascControllerService().getVascController();
		if (vc==null) {
			return;
		}
		Server server = VascTechDemoStartup.getInstance().getTomcatService().getServer();
		if (server==null) {
			return; // still booting
		}
		Service service = server.findService("Catalina");
		if (service==null) {
			return;
		}
		Context demoContext  = VascTechDemoStartup.getInstance().getTomcatService().getApplicationContext();
		
		infoVascGroups.setText(""+vc.getVascEntryController().getVascEntryGroupIds().size());
		infoVascEntries.setText(""+vc.getVascEntryController().getVascEntryIds().size());
		infoVascBackends.setText(""+vc.getVascBackendController().getVascBackendIds().size());
		
		Connector[] connectors = service.findConnectors(); 
		if (connectors.length>0) {
			int httpPort = connectors[0].getPort();
			infoHttpPort.setText(""+httpPort);
		}
		infoThreads.setText(""+Thread.activeCount());
		
		int sessions = 0;
		int workers = 0;
		
		if (demoContext!=null) {
			sessions = demoContext.getManager().getActiveSessions();
		}
		
		ThreadMXBean man = ManagementFactory.getThreadMXBean();
		ThreadInfo[] infos = man.getThreadInfo(man.getAllThreadIds());
		for (ThreadInfo info:infos) {
			if (info.getThreadName()!=null && info.getThreadName().startsWith("http")) {
				workers++;
			}
		}
		
		infoWorkers.setText(""+workers);
		infoSessions.setText(""+sessions);
	}
	
	
	/*
	 * 
	 * 
	
	    	JMenuItem openXmlItem = new JMenuItem("Import Xml");
    	openXmlItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				try {
					final JFileChooser fc = new JFileChooser();
					//fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
					int returnVal = fc.showOpenDialog((JMenuItem)e.getSource());
					if (returnVal == JFileChooser.APPROVE_OPTION) {
						File file = fc.getSelectedFile();
						//VascTechDemoStartup.getInstance().getVascControllerService().openFile(file);
					}
				} catch (Exception e1) {
					e1.printStackTrace();
				}
			}
    		
    	});
	
	*/
}
