/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.demo.server.ui.load;

import java.awt.BorderLayout;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;

import org.eobjects.metamodel.jdbc.JdbcDataContext;
import org.eobjects.metamodel.mongodb.MongoDbDataContext;

import net.forwardfire.vasc.demo.server.ui.JFireBorder;
import net.forwardfire.vasc.demo.server.ui.ServerGuiApplication;

/**
 * JLoadDialog mini wizzard for loading.
 * 
 * @author Willem Cazander
 * @version 1.0 May 9, 2012
 */
public class JLoadDialog extends JDialog implements ActionListener {

	private static final long serialVersionUID = -8638394652416472734L;
	private JButton prev = null;
	private JButton next = null;
	private JButton cancel = null;
	private List<LoadStep> steps = null;
	private int currentStep = 0;
	private LoadStepData model = null;
	private JPanel centerPanel = null;
	private JLabel topLabel = null;
	
	public enum LoadType {
		CSV,
		JDBC,
		MONGODB
	}
	
	public JLoadDialog(Frame aFrame,LoadType type) {
		setTitle("Load Edit Data");
		setIconImage(ServerGuiApplication.getInstance().getServerIcon().getImage());
		setMinimumSize(new Dimension(640,400));
		setPreferredSize(new Dimension(640,400));
		setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent we) {
				clearAndHide();
			}
		});
		JPanel mainPanel = new JPanel();
	//	mainPanel.setBorder();
	//	mainPanel.setLayout(new SpringLayout());
		mainPanel.setLayout(new BorderLayout());
		mainPanel.add(createPanelTop(),BorderLayout.PAGE_START);
		
		JPanel wrap = new JPanel();
		wrap.setLayout(new GridLayout(1,1));
		wrap.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
		
		centerPanel = new JPanel();
		//centerPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
		centerPanel.setLayout(new GridLayout(1,1));
		centerPanel.setBorder(new JFireBorder(null,centerPanel));
		
		wrap.add(centerPanel);
		mainPanel.add(wrap,BorderLayout.CENTER);
		
		model = new LoadStepData();
		steps = new ArrayList<LoadStep>(10);
		
		switch(type) {
		case CSV:
			break;
		case JDBC:
			steps.add(new JLoadStepMetaJdbc());
			break;
		case MONGODB:
			steps.add(new JLoadStepMetaMongodb());
			break;
		}
		steps.add(new JLoadStepSelectTables());
		steps.add(new JLoadStepMiscInfo());
		steps.add(new JLoadStepWriteFile());
		
		centerPanel.add(steps.get(0).getPanel());
		
		mainPanel.add(createPanelBottom(),BorderLayout.PAGE_END);
		//SpringLayoutGrid.makeCompactGrid(mainPanel, 3, 1,6,6,6,6);
		getContentPane().add(mainPanel);
		pack();
		setLocationRelativeTo(aFrame);
	}
	
	public void clearAndHide() {
		setVisible(false);
		
		// clean up connection
		if (model.dc instanceof JdbcDataContext) {
			JdbcDataContext dc = (JdbcDataContext)model.dc;
			try {
				dc.getConnection().close();
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
		if (model.dc instanceof MongoDbDataContext) {
			MongoDbDataContext dc = (MongoDbDataContext)model.dc;
			dc.getMongoDb().getMongo().close();
		}
	}
	
	public JPanel createPanelTop() {
		JPanel top = new JPanel();
		top.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
		top.setLayout(new GridLayout(1,1));
		JPanel wrap = new JPanel();
		wrap.setLayout(new FlowLayout(FlowLayout.LEFT));
		wrap.setBorder(new JFireBorder("Load",top));
		topLabel = new JLabel("Load data to edit...");
		wrap.add(topLabel);
		top.add(wrap);
		return top;
	}
	
	public JPanel createPanelBottom() {
		JPanel bottom = new JPanel();
		bottom.setLayout(new FlowLayout(FlowLayout.RIGHT));
		
		prev = new JButton("Prev");
		next = new JButton("Next");
		cancel = new JButton("Cancel");
		
		prev.addActionListener(this);
		next.addActionListener(this);
		cancel.addActionListener(this);
		
		prev.setEnabled(false);
		
		bottom.add(prev);
		bottom.add(next);
		bottom.add(cancel);
		
		return bottom;
	}
	
	
	public void actionPerformed(ActionEvent e) {
		if (e.getSource().equals(prev)) {
			gotoStepPrev();
		} else if (e.getSource().equals(next)) {
			gotoStepNext();
		} if (e.getSource().equals(cancel)) {
			clearAndHide();
		}
	}
	
	public void gotoStepPrev() {
		if (currentStep <= 0) {
			return;
		}
		setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		try {
			currentStep--;
			centerPanel.removeAll();
			LoadStep step = steps.get(currentStep);
			centerPanel.add(step.getPanel());
			topLabel.setText(step.getStepTitle());
			step.setupStep(model);
			SwingUtilities.updateComponentTreeUI(centerPanel);
			if (currentStep==0) {
				prev.setEnabled(false);
			}
			next.setText("Next");
		} finally {
			setCursor(Cursor.getDefaultCursor());
		}
	}
	
	public void gotoStepNext() {
		if (currentStep == steps.size()-1) {
			// perform last step
			LoadStep step = steps.get(currentStep);
			boolean result = step.performStep(model);
			if (result==false) {
				return;
			}
			clearAndHide();
			return;
		}
		if (currentStep > steps.size()-1) {
			return;
		}
		setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		try {
			LoadStep step = steps.get(currentStep);
			boolean result = step.performStep(model);
			if (result==false) {
				return;
			}
			currentStep++;
			centerPanel.removeAll();
			step = steps.get(currentStep);
			centerPanel.add(step.getPanel());
			topLabel.setText(step.getStepTitle());
			step.setupStep(model);
			SwingUtilities.updateComponentTreeUI(centerPanel);
			if (currentStep==steps.size()-1) {
				next.setText("Finish");
			}
			prev.setEnabled(true);
		} finally {
			setCursor(Cursor.getDefaultCursor());
		}
	}
}
