/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.demo.server.ui.load;

import java.awt.FlowLayout;
import java.sql.Connection;

import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SpringLayout;

import org.eobjects.metamodel.jdbc.JdbcDataContext;

import net.forwardfire.vasc.backend.metamodel.MetaModelDataContextJdbc;
import net.forwardfire.vasc.demo.server.ui.SpringLayoutGrid;

/**
 * JDialogMetaJdbc Add and runs MetaModel Schema Auto Entry code.
 * 
 * @author Willem Cazander
 * @version 1.0 May 9, 2012
 */
public class JLoadStepMetaJdbc extends JPanel implements LoadStep {

	private static final long serialVersionUID = -8638394652416472734L;
	private JComboBox driverClassBox = null;
	private JTextField connectUrlField = null;
	private JTextField usernameField = null;
	private JTextField passwordField = null;
	

	public JLoadStepMetaJdbc() {
		setLayout(new FlowLayout(FlowLayout.LEFT));
		add(createPanelCenter());
	}
	
	public String getStepTitle() {
		return "Connect to database.";
	}
	
	public JPanel getPanel() {
		return this;
	}

	public JPanel createPanelCenter() {
		JPanel result = new JPanel();
		result.setLayout(new SpringLayout());
		
		result.add(new JLabel("Driver"));
		driverClassBox = new JComboBox(new String[] {"org.postgresql.Driver","com.mysql.jdbc.Driver","org.apache.derby.jdbc.EmbeddedDriver","org.hsqldb.jdbcDriver","org.sqlite.JDBC"});
		driverClassBox.setSelectedIndex(0);
		result.add(driverClassBox);
		
		result.add(new JLabel("Url"));
		connectUrlField = new JTextField("jdbc:postgresql://localhost/dellstore2");
		result.add(connectUrlField);
		
		result.add(new JLabel("Username"));
		usernameField = new JTextField("postgres");
		result.add(usernameField);
		
		result.add(new JLabel("Password"));
		passwordField = new JTextField("postgresql");
		result.add(passwordField);
		
		SpringLayoutGrid.makeCompactGrid(result, 4, 2);
		return result;
	}
	
	public void setupStep(LoadStepData model) {
		
	}
	
	@Override
	public boolean performStep(LoadStepData model) {
		String url = connectUrlField.getText();
		MetaModelDataContextJdbc ds = new MetaModelDataContextJdbc();
		ds.setDriverClass((String)driverClassBox.getSelectedItem());
		ds.setConnectUrl(url);
		ds.setUsername(usernameField.getText());
		ds.setPassword(passwordField.getText());
		String dbName = url.substring(url.lastIndexOf('/')+1,url.length());
		
		try {
			Connection c = ds.getConnection();
			c.close();
			
			if (model.dc instanceof JdbcDataContext) {
				JdbcDataContext dc = (JdbcDataContext)model.dc;
				dc.getConnection().close();
			}
			
			model.dcProvider = ds;
			model.dc=ds.getDataContext();
			model.name=dbName;
		} catch (Exception e) {
			e.printStackTrace();
			JOptionPane.showMessageDialog(null, "Fatal connect error:\n"+e.getMessage(), "Jdbc Connect Error", JOptionPane.ERROR_MESSAGE);
			return false;
		}
		return true;
	}
}
