/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.demo.server.ui.load;

import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SpringLayout;

import org.eobjects.metamodel.mongodb.MongoDbDataContext;

import com.mongodb.DB;

import net.forwardfire.vasc.backend.metamodel.MetaModelDataContextMongodb;
import net.forwardfire.vasc.demo.server.ui.SpringLayoutGrid;

/**
 * JLoadStepMetaMongodb Add and runs MetaModel Schema Auto Entry code.
 * 
 * @author Willem Cazander
 * @version 1.0 May 9, 2012
 */
public class JLoadStepMetaMongodb extends JPanel implements LoadStep,ActionListener {

	private static final long serialVersionUID = -8638394652416472734L;
	private JTextField hostNameField = null;
	private JTextField hostPortField = null;
	private JTextField databaseField = null;

	public JLoadStepMetaMongodb() {
		setLayout(new FlowLayout(FlowLayout.LEFT));
		add(createPanelCenter());
	}
	
	public String getStepTitle() {
		return "Connect to mongodb.";
	}
	
	public JPanel getPanel() {
		return this;
	}
	
	public JPanel createPanelCenter() {
		JPanel result = new JPanel();
		result.setLayout(new SpringLayout());
		
		result.add(new JLabel("Hostname"));
		hostNameField = new JTextField("localhost",25);
		result.add(hostNameField);
		
		result.add(new JLabel("Port"));
		hostPortField = new JTextField("27017");
		result.add(hostPortField);
		
		result.add(new JLabel("Database"));
		databaseField = new JTextField("lefiona");
		result.add(databaseField);

		SpringLayoutGrid.makeCompactGrid(result, 3, 2);
		return result;
	}
	
	public void actionPerformed(ActionEvent arg0) {
		// TODO Auto-generated method stub
		
	}
	
	public void setupStep(LoadStepData model) {
		
	}
	
	@Override
	public boolean performStep(LoadStepData model) {
		
		MetaModelDataContextMongodb ds = new MetaModelDataContextMongodb();
		ds.setHostname(hostNameField.getText());
		ds.setPort(new Integer(hostPortField.getText()));
		ds.setDatabase(databaseField.getText());
		
		try {
			DB mongoDB  = ds.getMongodbConnection();
			mongoDB.getMongo().close();
			
			if (model.dc instanceof MongoDbDataContext) {
				MongoDbDataContext dc = (MongoDbDataContext)model.dc;
				dc.getMongoDb().getMongo().close();
			}
			
			model.dcProvider = ds;
			model.dc=ds.getDataContext();
			model.name=ds.getDatabase();
		} catch (Exception e) {
			e.printStackTrace();
			JOptionPane.showMessageDialog(null, "Fatal connect error:\n"+e.getMessage(), "Jdbc Connect Error", JOptionPane.ERROR_MESSAGE);
			return false;
		}
		return true;
	}		
}
