/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.demo.server.ui.load;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ScrollPaneConstants;
import javax.swing.SpringLayout;
import javax.swing.table.AbstractTableModel;

import org.eobjects.metamodel.DataContext;

import net.forwardfire.vasc.demo.server.ui.SpringLayoutGrid;


/**
 * JLoadStepSelectTables select tables to load.
 * 
 * @author Willem Cazander
 * @version 1.0 May 9, 2012
 */
public class JLoadStepSelectTables extends JPanel implements LoadStep {

	private static final long serialVersionUID = -8638394652416472734L;
	private JTable table = null;
	private MetaModelTableModel tableModel = null;
	private DataContext dcOld = null;

	public JLoadStepSelectTables() {
		setLayout(new SpringLayout());
		add(createPanelCenter());
		SpringLayoutGrid.makeCompactGrid(this, 1, 1, 6, 6, 6, 6);
	}
	
	public String getStepTitle() {
		return "Select Tables";
	}
	
	public JPanel getPanel() {
		return this;
	}
	
	public JComponent createPanelCenter() {
		tableModel = new MetaModelTableModel();
		table = new JTable(tableModel);
		table.setFillsViewportHeight(true);
		table.setShowHorizontalLines(true);
		table.setAutoscrolls(true);
		//((JComponent) table.getDefaultRenderer(Boolean.class)).setOpaque(true);// let backgroup color work correctly
		JScrollPane scrollPane = new JScrollPane(table);
		scrollPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		scrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
		scrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
		return scrollPane;
	}

	public void setupStep(LoadStepData model) {
		if (model.tables.isEmpty() | (dcOld!=null && dcOld!=model.dc)) {
			tableModel.setData(Arrays.asList(model.dc.getDefaultSchema().getTableNames()));
			dcOld = model.dc;
		}
	}
	
	
	@Override
	public boolean performStep(LoadStepData model) {
		model.tables.clear();
		for (TableRow row:tableModel.getData()) {
			if (row.load) {
				model.tables.add(row.table);
			}
		}
		return true;
	}
	
	class TableRow {
		String table;
		boolean load = true;
	}
	
	class MetaModelTableModel extends AbstractTableModel {

		private static final long serialVersionUID = 5487313677918103654L;
		private List<TableRow> data = new ArrayList<TableRow>();
		
		public void setData(List<String> tables) {
			data.clear();
			for (String table:tables) {
				TableRow row = new TableRow();
				row.table=table;
				data.add(row);
			}
			fireTableDataChanged();
		}
		
		public List<TableRow> getData() {
			return data;
		}
		
		@Override
		public Class<?> getColumnClass(int columnIndex) {
			if (columnIndex==0) {
				return String.class;
			} else {
				return Boolean.class;
			}
		}
		
		
		@Override
		public int getColumnCount() {
			return 2;
		}

		@Override
		public String getColumnName(int columnIndex) {
			if (columnIndex==0) {
				return "TableName";
			} else {
				return "Import";
			}
		}

		@Override
		public int getRowCount() {
			return data.size();
		}

		@Override
		public Object getValueAt(int rowIndex, int columnIndex) {
			TableRow row = data.get(rowIndex);
			if (columnIndex==0) {
				return row.table;
			} else {
				return new Boolean(row.load);
			}
		}

		@Override
		public boolean isCellEditable(int rowIndex, int columnIndex) {
			return columnIndex>0;
		}

		@Override
		public void setValueAt(Object value, int rowIndex, int columnIndex) {
			if (columnIndex==1) {
				TableRow row = data.get(rowIndex);
				if (value instanceof Boolean) {
					row.load = (Boolean)value;
				}
			}
		}
	}
}
