/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.demo.server.ui.load;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;
import java.io.StringWriter;
import java.io.Writer;

import javax.swing.JComponent;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.ScrollPaneConstants;
import javax.swing.SpringLayout;

import org.x4o.xml.io.sax.ext.ContentWriterXml;

import net.forwardfire.vasc.core.VascController;
import net.forwardfire.vasc.demo.server.core.VascTechDemoStartup;
import net.forwardfire.vasc.demo.server.ui.SpringLayoutGrid;

/**
 * JLoadStepWriteFile writes the file.
 * 
 * @author Willem Cazander
 * @version 1.0 Nov 21, 2012
 */
public class JLoadStepWriteFile extends JPanel implements LoadStep,ActionListener {

	private static final long serialVersionUID = -8638394652416472734L;
	private JTextArea text = null;

	public JLoadStepWriteFile() {
		setLayout(new SpringLayout());
		add(createPanelCenter());
		SpringLayoutGrid.makeCompactGrid(this, 1, 1, 6, 6, 6, 6);
	}
	
	public String getStepTitle() {
		return "Write xml to file.";
	}
	
	public JPanel getPanel() {
		return this;
	}
	
	public JComponent createPanelCenter() {
		text = new JTextArea(13,45);
		text.setAutoscrolls(true);
		JScrollPane scrollPane = new JScrollPane(text);
		scrollPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		scrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
		return scrollPane;
	}
	
	public void actionPerformed(ActionEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	public void setupStep(LoadStepData model) {
		try {
			VascController vc = VascTechDemoStartup.getInstance().getVascControllerService().getVascController();
			StringWriter out = new StringWriter();
			ContentWriterXml outXml = new ContentWriterXml(out);
			LoadVascXmlWriter writer = new LoadVascXmlWriter(outXml,vc);
			writer.writeXml(model);
			
			text.setText(out.getBuffer().toString());
		} catch (Exception e) {
			e.printStackTrace();
			JOptionPane.showMessageDialog(null, "Fatal error:\n"+e.getMessage(), "Generate xml error", JOptionPane.ERROR_MESSAGE);
		}
	}
	
	@Override
	public boolean performStep(LoadStepData model) {
		try {
			File writeFile = new File("conf/vasc.d/"+model.filename);
			System.out.println("write to : "+writeFile);
			Writer out = new OutputStreamWriter(new FileOutputStream(writeFile));
			out.append(text.getText());
			out.flush();
			out.close();
		} catch (Exception e) {
			e.printStackTrace();
			JOptionPane.showMessageDialog(null, "Fatal error:\n"+e.getMessage(), "Write file error", JOptionPane.ERROR_MESSAGE);
			return false;
		}
		return true;
	}		
}
