/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.demo.server.ui.load;

import net.forwardfire.vasc.backend.metamodel.MetaModelDataContextJdbc;
import net.forwardfire.vasc.backend.metamodel.MetaModelDataContextMongodb;
import net.forwardfire.vasc.backend.metamodel.MetaModelVascBackendCrud;
import net.forwardfire.vasc.backend.object.VascSelectItemModelEntry;
import net.forwardfire.vasc.core.VascController;
import net.forwardfire.vasc.core.VascEntry;
import net.forwardfire.vasc.core.VascEntryField;
import net.forwardfire.vasc.core.VascEntryFieldType;
import net.forwardfire.vasc.core.VascEntryLink;
import net.forwardfire.vasc.core.VascEntryLinkType;
import net.forwardfire.vasc.opt.metamodel.MetaModelSchemaAutoEntry;

import org.x4o.xml.io.sax.ext.ContentWriterXml;
import org.xml.sax.SAXException;
import org.xml.sax.ext.DefaultHandler2;
import org.xml.sax.helpers.AttributesImpl;

/**
 * LoadVascXmlWriter writes the xml tags.
 * note: need to move this to x4o
 * 
 * @author Willem Cazander
 * @version 1.0 Nov 22, 2012
 */
public class LoadVascXmlWriter {

	private VascController vc = null;
	private ContentWriterXml xmlWriter = null;
	
	static private final String URI_VASC_ROOT = "http://vasc.forwardfire.net/xml/ns/vasc-root";
	static private final String URI_VASC_LANG = "http://vasc.forwardfire.net/xml/ns/vasc-lang";
	static private final String URI_META_MODEL = "http://vasc.forwardfire.net/xml/ns/vasc-backend-metamodel";
	static private final String URI_VASC_DEMO = "http://vasc.forwardfire.net/xml/ns/vasc-tech-demo";
	
	public LoadVascXmlWriter(ContentWriterXml xmlWriter,VascController vc) {
		this.xmlWriter=xmlWriter;
		this.vc=vc;
	}
	
	public void writeXml(LoadStepData model) throws SAXException {
		
		MetaModelSchemaAutoEntry autoEntry = new MetaModelSchemaAutoEntry();
		autoEntry.setVascGroupId(model.groupId);
		autoEntry.setEntryPrefix(model.name);
		autoEntry.setDataContextProvider(model.dcProvider);
		autoEntry.getTables().addAll(model.tables);
		autoEntry.autoFillResult(vc);
		
		xmlWriter.startDocument();
		
		char[] msg;
		msg = "\n".toCharArray();
		xmlWriter.ignorableWhitespace(msg,0,msg.length);
		
		xmlWriter.startPrefixMapping("vasc", URI_VASC_ROOT);
		xmlWriter.startPrefixMapping("v", URI_VASC_LANG);
		xmlWriter.startPrefixMapping("mm", URI_META_MODEL);
		xmlWriter.startPrefixMapping("td", URI_VASC_DEMO);
		
		AttributesImpl atts = new AttributesImpl();
		xmlWriter.startElement (URI_VASC_ROOT, "root", "", atts);
		
		msg = "Auto generated vasc xml example.".toCharArray();
		xmlWriter.comment(msg,0,msg.length);
		
		writeGroup(model.groupId,model.rolesGroup);
		writeMetaProvider(model);
		
		for (MetaModelVascBackendCrud vb:autoEntry.getResultBackends()) {
			writeEntryBackend(vb,model);
		}
		for (VascEntry ve:autoEntry.getResultEntries()) {
			writeEntry(ve,model);
		}
		
		
		xmlWriter.endElement(URI_VASC_ROOT, "root", "");
		xmlWriter.endDocument();
	}
	
	private void writeMetaProvider(LoadStepData model) throws SAXException {
		AttributesImpl atts = null;
		if (model.dcProvider instanceof MetaModelDataContextMongodb) {
			MetaModelDataContextMongodb mongo = (MetaModelDataContextMongodb)model.dcProvider;
			
			atts = new AttributesImpl();
			atts.addAttribute ("", "el.id", "", "", model.name+"DC");
			if (mongo.getHostname()!=null) {
				atts.addAttribute ("", "hostname", "", "", mongo.getHostname());
			}
			if (mongo.getDatabase()!=null) {
				atts.addAttribute ("", "database", "", "", mongo.getDatabase());
			}
			if (mongo.getUsername()!=null) {
				atts.addAttribute ("", "username", "", "", mongo.getUsername());
			}
			if (mongo.getPassword()!=null) {
				atts.addAttribute ("", "password", "", "", mongo.getPassword());
			}
			atts.addAttribute ("", "port", "", "", ""+mongo.getPort());
			
			xmlWriter.startElement (URI_META_MODEL, "mongodbDataContext", "", atts);
			xmlWriter.endElement(URI_META_MODEL, "mongodbDataContext", "");
			
		} else if (model.dcProvider instanceof MetaModelDataContextJdbc) {
			MetaModelDataContextJdbc jdbc = (MetaModelDataContextJdbc)model.dcProvider;
			
			atts = new AttributesImpl();
			atts.addAttribute ("", "name", "", "", "jdbc/load/"+model.name+"DS");
			atts.addAttribute ("", "auth", "", "", "Container");
			atts.addAttribute ("", "type", "", "", "javax.sql.DataSource");
			atts.addAttribute ("", "factory", "", "", "org.apache.tomcat.jdbc.pool.DataSourceFactory");
			atts.addAttribute ("", "initialSize", "", "", "1");
			atts.addAttribute ("", "minIdle", "", "", "1");
			atts.addAttribute ("", "username", "", "", jdbc.getUsername());
			atts.addAttribute ("", "password", "", "", jdbc.getPassword());
			atts.addAttribute ("", "driverClassName", "", "", jdbc.getDriverClass());
			atts.addAttribute ("", "url", "", "", jdbc.getConnectUrl());
			xmlWriter.startElement (URI_VASC_DEMO, "tomcatResource", "", atts);
			xmlWriter.endElement(URI_VASC_DEMO, "tomcatResource", "");
			
			atts = new AttributesImpl();
			atts.addAttribute ("", "el.id", "", "", model.name+"DC");
			atts.addAttribute ("", "jndiName", "", "", "java:jdbc/load/"+model.name+"DS");
			xmlWriter.startElement (URI_META_MODEL, "jndiDataSourceDataContext", "", atts);
			xmlWriter.endElement(URI_META_MODEL, "jndiDataSourceDataContext", "");
		}
	}
	
	private void writeGroup(String groupId,String roles) throws SAXException {
		AttributesImpl atts = new AttributesImpl();
		atts.addAttribute ("", "id", "", "", groupId);
		atts.addAttribute ("", "rolesView", "", "", roles);
		xmlWriter.startElement (URI_VASC_LANG, "entryGroup", "", atts);
		xmlWriter.endElement(URI_VASC_LANG, "entryGroup", "");
	}
	
	private void writeEntryBackend(MetaModelVascBackendCrud vb,LoadStepData model) throws SAXException {

		AttributesImpl atts = new AttributesImpl();
		atts.addAttribute ("", "id", "", "", vb.getId());
		atts.addAttribute ("", "dataContextProvider", "", "", "${"+model.name+ "DC}");
		atts.addAttribute ("", "table", "", "", vb.getTable());
		atts.addAttribute ("", "tableId", "", "", vb.getTableId());
		
		xmlWriter.startElement (URI_META_MODEL, "metaModelBackend", "", atts);
		xmlWriter.endElement(URI_META_MODEL, "metaModelBackend", "");
	}
	
	private void writeEntry(VascEntry ve,LoadStepData model) throws SAXException {
				
		AttributesImpl atts = new AttributesImpl();
		atts.addAttribute ("", "id", "", "", ve.getId());
		atts.addAttribute ("", "backendId", "", "", ve.getBackendId());
		atts.addAttribute ("", "primaryKeyFieldId", "", "", ve.getPrimaryKeyFieldId());
		if (ve.getDisplayNameFieldId()!=null) {
			atts.addAttribute ("", "displayNameFieldId", "", "", ve.getDisplayNameFieldId());
		}
		if (ve.getVascGroupId()!=null) {
			atts.addAttribute ("", "vascGroupId", "", "", ve.getVascGroupId());
		}
		if (ve.getAccessType()!=null) {
			atts.addAttribute ("", "accessType", "", "", ve.getAccessType().name());
		}
		xmlWriter.startElement (URI_VASC_LANG, "entry", "", atts);
		createFields(ve);
		
		for (VascEntryLink vel:ve.getVascEntryLinks()) {
			atts = new AttributesImpl();
			atts.addAttribute ("", "id", "", "", vel.getId());
			if (vel.getVascEntryLinkType()!=null && vel.getVascEntryLinkType()!=VascEntryLinkType.DEFAULT_TYPE) {
				atts.addAttribute ("", "vascEntryLinkType", "", "", ""+vel.getVascEntryLinkType());
			}
			atts.addAttribute ("", "vascEntryId", "", "", vel.getVascEntryId());
			xmlWriter.startElement (URI_VASC_LANG, "link", "", atts);
			
			for (String key:vel.getEntryParameterFieldIdKeys()) {
				String value = vel.getEntryParameterFieldId(key);
				atts = new AttributesImpl();
				atts.addAttribute ("", "name", "", "", key);
				atts.addAttribute ("", "valueFieldId", "", "", value);
				xmlWriter.startElement (URI_VASC_LANG, "linkParameter", "", atts);
				xmlWriter.endElement(URI_VASC_LANG, "linkParameter", "");
			}
			xmlWriter.endElement(URI_VASC_LANG, "link", "");
		}
		xmlWriter.endElement(URI_VASC_LANG, "entry", "");
	}
	
	private void createFields(VascEntry ve) throws SAXException {
		for (VascEntryField f:ve.getVascEntryFields()) {
			
			AttributesImpl atts = new AttributesImpl();
			atts.addAttribute ("", "id", "", "", f.getId());
			VascEntryFieldType type = f.getVascEntryFieldType();
			if (type!=null) {
				atts.addAttribute ("", "vascEntryFieldType", "", "", type.getId());
			}
			if (f.getEditReadOnly()!=null) {
				atts.addAttribute ("", "editReadOnly", "", "", ""+f.getEditReadOnly());
			}
			if (f.getDefaultValue()!=null) {
				atts.addAttribute ("", "defaultValue", "", "", ""+f.getDefaultValue());
			}
			if (f.getList()!=null) {
				atts.addAttribute ("", "list", "", "", ""+f.getList());
			}
			if (f.getEdit()!=null) {
				atts.addAttribute ("", "edit", "", "", ""+f.getEdit());
			}
			if (f.getCreate()!=null) {
				atts.addAttribute ("", "create", "", "", ""+f.getCreate());
			}
			
			//atts.addAttribute ("", "vascGroupId", "", "", model.groupId);
			xmlWriter.startElement (URI_VASC_LANG, "field", "", atts);
			
			if (type!=null && "ListField".equals(type.getId())) {
				VascSelectItemModelEntry link = (VascSelectItemModelEntry)type.getDataObject();
				atts = new AttributesImpl();
				atts.addAttribute ("", "entryId", "", "", ""+link.getEntryId());
				atts.addAttribute ("", "displayFieldId", "", "", ""+link.getDisplayFieldId());
				xmlWriter.startElement (URI_VASC_LANG, "vascSelectItemModel", "", atts);
				xmlWriter.endElement(URI_VASC_LANG, "vascSelectItemModel", "");
			}
			
			
			xmlWriter.endElement(URI_VASC_LANG, "field", "");
		}
	}
}
