/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.demo.server.x4o;

import java.util.Map;
import java.util.logging.Logger;

import net.forwardfire.vasc.demo.server.core.VascTechDemoStartup;
import net.forwardfire.vasc.demo.server.tomcat.TomcatService;

import org.apache.catalina.Server;
import org.apache.catalina.deploy.ContextResource;
import org.apache.catalina.deploy.NamingResources;
import org.x4o.xml.element.AbstractElement;
import org.x4o.xml.element.ElementException;

/**
 * TomcatResourceElement Add a global DataSource to tomcat jndi context.
 * 
 * @author Willem Cazander
 * @version 1.0 May 13, 2012
 */
public class TomcatResourceElement extends AbstractElement {

	private Logger logger = Logger.getLogger(TomcatResourceElement.class.getName());
	
	/**
	 * @see org.x4o.xml.element.AbstractElement#doElementRun()
	 */
	@Override
	public void doElementRun() throws ElementException {
		
		// Check needed attributes
		Map<String,String> attr = getAttributes();
		if (attr.containsKey("name")==false) {
			throw new ElementException("No attribute name found.");
		}
		if (attr.containsKey("auth")==false) {
			throw new ElementException("No attribute auth found.");
		}
		if (attr.containsKey("type")==false) {
			throw new ElementException("No attribute type found.");
		}
		
		// Set resource object fields.
		ContextResource resource = new ContextResource();
		String name = attr.remove("name");
		resource.setName(name);
		String auth = attr.remove("auth");
		resource.setAuth(auth);
		String type = attr.remove("type");
		resource.setType(type);
		String scope = attr.remove("scope");
		if (scope!=null) {
			resource.setScope(scope);
		}
		String closeMethod = attr.remove("closeMethod");
		if (closeMethod!=null) {
			resource.setCloseMethod(closeMethod);
		}
		String singleton = attr.remove("singleton");
		if (singleton!=null) {
			resource.setSingleton("true".equalsIgnoreCase(singleton));
		}
		String description = attr.remove("description");
		if (description!=null) {
			resource.setDescription(description);
		}
		
		// copy other stuff
		for (String key:attr.keySet()) {
			String value = attr.get(key);
			resource.setProperty(key, value);
		}
		
		// add to tomcat
		TomcatService tm = VascTechDemoStartup.getInstance().getTomcatService();
		Server server = tm.getServer();
		NamingResources jndiContext = server.getGlobalNamingResources();
		jndiContext.addResource(resource);
		logger.info("Added jndi global resource: "+resource.getName());
	}
}
