-- 
-- Create vasc demo database. 
--

CREATE TABLE vasc_user (
  id               IDENTITY not null primary key,
  username         varchar  not null,
  password         varchar  not null,
  description      varchar  not null,
);
CREATE UNIQUE INDEX vasc_user_username_idx ON vasc_user(username);

CREATE TABLE vasc_user_role (
  id             IDENTITY not null primary key,
  username       varchar  not null,
  role           varchar  not null
);
CREATE INDEX vasc_user_role_username_idx ON vasc_user_role(username);

CREATE TABLE vasc_user_change_field (
  id               IDENTITY not null primary key,
  field            varchar  not null,
  name             varchar  not null,
  active           boolean  not null
);
CREATE INDEX vasc_user_change_field_active_idx ON vasc_user_change_field(active);

CREATE TABLE vasc_user_change_log (
  id               IDENTITY not null primary key,
  user_id          integer  not null,
  change_field_id  integer  not null,
  value_old        varchar  not null,
  value_new        varchar  not null
);
CREATE INDEX vasc_user_change_log_user_id_idx	ON vasc_user_change_log(user_id);
CREATE INDEX vasc_user_change_log_id_idx 		ON vasc_user_change_log(user_id);

CREATE TABLE vasc_page (
  id             IDENTITY not null primary key,
  slug           varchar  not null,
  title          varchar  not null,
  active         boolean  not null,
  sitemap        boolean  not null,
  roles          varchar  not null
);
CREATE INDEX vasc_page_slug_idx		ON vasc_page(slug);
CREATE INDEX vasc_page_sitemap_idx	ON vasc_page(sitemap);

CREATE TABLE vasc_page_part (
  id             IDENTITY not null primary key,
  page_id        integer  not null,
  title          varchar  not null,
  text           varchar  not null,
  active         boolean  not null,
  sitemap        boolean  not null,
  part_order     integer  not null,
  part_type      varchar  not null,
  roles          varchar  not null
);
CREATE INDEX vasc_page_part_page_id_idx	ON vasc_page_part(page_id);
CREATE INDEX vasc_page_part_active_idx	ON vasc_page_part(active);
CREATE INDEX vasc_page_part_sitemap_idx	ON vasc_page_part(sitemap);

CREATE TABLE vasc_menu_web (
  id             IDENTITY not null primary key,
  href           varchar  not null,
  title          varchar  not null,
  target         varchar  not null,
  active         BOOLEAN  NOT NULL,
  roles          varchar  not null,
  menu_order     integer  not null,
  menu_type      varchar  not null
);
CREATE INDEX vasc_menu_web_active_idx	ON vasc_menu_web(active);
CREATE INDEX vasc_menu_web_type_idx		ON vasc_menu_web(menu_type);

--
-- Insert demo data.
--

INSERT INTO vasc_user VALUES(1, 'admin','admin123','Demo Admin user');
INSERT INTO vasc_user VALUES(2, 'demo', 'demo123', 'Demo user');

INSERT INTO vasc_user_role VALUES(1, 'admin',	'login');
INSERT INTO vasc_user_role VALUES(2, 'admin',	'admin');
INSERT INTO vasc_user_role VALUES(3, 'demo',	'login');

INSERT INTO vasc_user_change_field VALUES(1,'Username', 	'username',		TRUE);
INSERT INTO vasc_user_change_field VALUES(2,'Password', 	'password',		TRUE);
INSERT INTO vasc_user_change_field VALUES(3,'Description',	'description',	TRUE);
INSERT INTO vasc_user_change_field VALUES(4,'Birthdate', 	'date_age',		TRUE);

--							ID	SLUG		TITLE		ATIVE	SITEMAP
INSERT INTO vasc_page VALUES(1, 'contact',	'Contact',	TRUE,	TRUE,	'');
INSERT INTO vasc_page VALUES(2, 'techno',	'Techno',	TRUE,	TRUE,	'');
INSERT INTO vasc_page VALUES(3, 'help',		'Help',		TRUE,	TRUE,	'');
INSERT INTO vasc_page VALUES(4, 'config',	'Config',	TRUE,	FALSE,	'');

INSERT INTO vasc_page_part VALUES(1, 1, 'Project',	'The vasc project is build for amazing ease of editing data.',	TRUE,TRUE,1,'HTML',	'');
INSERT INTO vasc_page_part VALUES(2, 1, 'Website',	'VASC: <a href="http://vasc.forwardfire.net">Website</a>',	TRUE,TRUE,2,'HTML',	'');

INSERT INTO vasc_page_part VALUES(3, 2, 'Used technologies.',	'Example: XML,JSF,JDBC,etc',	TRUE,TRUE,1,'HTML',	'');
INSERT INTO vasc_page_part VALUES(4, 2, 'Used liberies',		'Todo make long list ?',		TRUE,TRUE,2,'HTML',	'');

INSERT INTO vasc_page_part VALUES(5, 3, 'Documenation.',	'See docs directory.',		TRUE,TRUE,1,'HTML',	'');
INSERT INTO vasc_page_part VALUES(6, 3, 'Mailing list.',	'Goto some page to see.',	TRUE,TRUE,2,'HTML',	'');

INSERT INTO vasc_page_part VALUES(10, 4, 'Vasc',		'Config entries in conf/vasc.d directory.',	TRUE,TRUE,1,'HTML',	'');
INSERT INTO vasc_page_part VALUES(11, 4, 'Jdbc',		'Config in vasc xml or context.xml',		TRUE,TRUE,2,'HTML',	'');
INSERT INTO vasc_page_part VALUES(12, 4, 'Jndi',		'Example of data source.',					TRUE,TRUE,3,'HTML',	'');
INSERT INTO vasc_page_part VALUES(13, 4, 'Logback',		'Change logging options.',					TRUE,TRUE,3,'HTML',	'');



-- INSERT INTO vasc_page VALUES(1, 'home','home','Welcome to the vasc demo, please login as admin to view all stuff.');

INSERT INTO vasc_menu_web VALUES(1, '/html/index.jsf','Home',		'',true,''		,1,'BAR_LEFT');
INSERT INTO vasc_menu_web VALUES(2, '/html/admin/debug.jsf','Debug','',true,'admin'	,2,'BAR_LEFT');
INSERT INTO vasc_menu_web VALUES(3, '/html/admin/index.jsf','Admin','',true,'admin'	,3,'BAR_LEFT');

INSERT INTO vasc_menu_web VALUES(4, '/html/index.jsf',	'Home',		'',true,''		,1,'BAR_BOTTOM');
INSERT INTO vasc_menu_web VALUES(5, '/page/contact',	'Contact',	'',true,''		,2,'BAR_BOTTOM');
INSERT INTO vasc_menu_web VALUES(6, '/page/help',		'Help',		'',true,''		,3,'BAR_BOTTOM');
INSERT INTO vasc_menu_web VALUES(7, '/page/techno',		'Techno',	'',true,''		,4,'BAR_BOTTOM');

INSERT INTO vasc_menu_web VALUES(8, '/page/contact',	'Contact',	'',true,''		,1,'BAR_RIGHT');
INSERT INTO vasc_menu_web VALUES(9, '/page/help',		'Help',		'',true,''		,2,'BAR_RIGHT');

INSERT INTO vasc_menu_web VALUES(10, '/html/user/profile.jsf',				'Profile',				'',true,'login',4,'MAIN_MENU_0');
INSERT INTO vasc_menu_web VALUES(11, '/vasc/VascEntry/list.jsf',			'VascEntries',			'',true,'admin',5,'MAIN_MENU_0');

INSERT INTO vasc_menu_web VALUES(20, '/vasc/AdminVascUser/list.jsf',				'Users',				'',true,'admin',4,'PAGE_ADMIN');
INSERT INTO vasc_menu_web VALUES(21, '/vasc/AdminVascUserRole/list.jsf',			'UserRoles',			'',true,'admin',5,'PAGE_ADMIN');
INSERT INTO vasc_menu_web VALUES(22, '/vasc/AdminVascUserChangeField/list.jsf',		'ChangeField',			'',true,'admin',5,'PAGE_ADMIN');
INSERT INTO vasc_menu_web VALUES(23, '/vasc/AdminVascUserChangeLog/list.jsf',		'ChangeFieldLog',		'',true,'admin',5,'PAGE_ADMIN');
INSERT INTO vasc_menu_web VALUES(24, '/vasc/AdminVascPage/list.jsf',				'Pages',				'',true,'admin',5,'PAGE_ADMIN');
INSERT INTO vasc_menu_web VALUES(25, '/vasc/AdminVascPagePart/list.jsf',			'PageParts',			'',true,'admin',5,'PAGE_ADMIN');
INSERT INTO vasc_menu_web VALUES(26, '/vasc/AdminVascMenu/list.jsf',				'Menu',					'',true,'admin',4,'PAGE_ADMIN');


