/*
 * Copyright 2009-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.demo.tech.ejb3;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.annotation.Resource;
import javax.ejb.SessionContext;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;

import net.forwardfire.vasc.core.VascController;
import net.forwardfire.vasc.core.VascEntry;
import net.forwardfire.vasc.core.VascEntryAccessType;
import net.forwardfire.vasc.core.VascEntryGroup;
import net.forwardfire.vasc.demo.tech.domain.menu.VascMenuControllerLocal;
import net.forwardfire.vasc.demo.tech.domain.menu.VascMenuControllerRemote;
import net.forwardfire.vasc.demo.tech.domain.menu.model.VascMenu;
import net.forwardfire.vasc.demo.tech.domain.menu.model.VascMenuGroup;
import net.forwardfire.vasc.demo.tech.domain.menu.model.VascMenuWeb;
import net.forwardfire.vasc.demo.tech.domain.menu.model.VascMenuWebComparator;
import net.forwardfire.vasc.demo.tech.domain.menu.model.VascMenuWebType;
import net.forwardfire.vasc.demo.tech.domain.petstore.model.AbstractPetStoreVascBackend.OrderBackend;

/**
 * MenuController Shows the menu for the user. 
 * 
 * @author Willem Cazander
 * @version 1.0 May 19, 2012
 */
public class VascMenuControllerImpl implements VascMenuControllerLocal,VascMenuControllerRemote {

	private VascMenuWebComparator vascMenuWebComparator = null;
	
	@Resource
	private SessionContext session;
	
	public VascMenuControllerImpl() {
		vascMenuWebComparator = new VascMenuWebComparator();
		
		
		OrderBackend ob = null;
	}
	
	public List<VascMenuWeb> fetchVascMenuWeb() {
		List<VascMenuWeb> result = new ArrayList<VascMenuWeb>(50);
		Connection connection = null;
		try {
			DataSource ds = getDataSource("openejb:Resource/jdbc/DemoManagerDataDS");
			connection = ds.getConnection();
			Statement s = connection.createStatement();
			s.execute("SELECT * FROM VASC_MENU_WEB");
			ResultSet rs = s.getResultSet();
			//int cols = rs.getMetaData().getColumnCount();
			while (rs.next()) {
				VascMenuWeb menu = new VascMenuWeb();
				menu.setId(rs.getInt(1));
				menu.setHref(rs.getString(2));
				menu.setTitle(rs.getString(3));
				menu.setTarget(rs.getString(4));
				menu.setActive(rs.getBoolean(5));
				menu.setRoles(rs.getString(6));
				menu.setMenuOrder(rs.getInt(7));
				menu.setMenuType(VascMenuWebType.valueOf(rs.getString(8)));
				if (filterVascMenuRoles(menu.getActive(),menu.getRoles())==false) {
					continue;
				}
				result.add(menu);
			}
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			if (connection!=null) {
				try {
					connection.close();
				} catch (Exception e) {
				}
			}
		}
		return result;
	}

	
	private DataSource getDataSource(String name) throws SQLException {
		try {
			Context initialContext = new InitialContext();
			DataSource datasource = (DataSource)initialContext.lookup(name);
			if ( datasource == null ) {
				throw new SQLException("Cannot lookup datasource: "+name);
			}
			return datasource;
		} catch ( NamingException e ) {
			throw new SQLException("Cannot get connection " + e,e);
		}
	}
	
	private boolean filterVascMenuRoles(Boolean active,String roles) {
		if (active!=null && active==false) {
			return false;
		}
		if (roles!=null && roles.isEmpty()==false) {
			String[] rolesArray = roles.split(",");
			for (String role:rolesArray) {
				if (role.isEmpty()) {
					continue;
				}
				
				if (session.isCallerInRole(role)==false) {
					return false;
				}
			}
		}
		return true;
	}
	
	public List<VascMenuGroup> getFilteredMenuGroup() {
		
		VascController vc = null;
		try {
			Context initialContext = new InitialContext();
			vc = (VascController)initialContext.lookup("openejb:Resource/vasc/server-tech");
		} catch (Exception e) {
			e.printStackTrace();
		}

		List<VascMenuGroup> result = new ArrayList<VascMenuGroup>(20);
		for (String groupdId:vc.getVascEntryController().getVascEntryGroupIds()) {
			VascEntryGroup veg = vc.getVascEntryController().getVascEntryGroupById(groupdId);
			if (filterVascMenuRoles(veg.getView(),veg.getRolesView())==false) {
				continue;
			}
			VascMenuGroup group = new VascMenuGroup();
			group.setId(veg.getId());
			group.setTitleKey(veg.getName());
			group.setMenus(getFilteredMenu(group.getId()));
			result.add(group);
		}
	//	Collections.sort(result,vascMenuGroupComparator);
		return result;	
	}
	
	public List<VascMenu> getFilteredMenu(String groupId) {
		if (groupId==null) {
			throw new NullPointerException("Can't filter on null groupId.");
		}
		VascController vc = null;
		try {
			Context initialContext = new InitialContext();
			vc = (VascController)initialContext.lookup("openejb:Resource/vasc/server-tech");
		} catch (Exception e) {
			e.printStackTrace();
		}
		List<VascMenu> result = new ArrayList<VascMenu>(15);
		for (String vascEntryId:vc.getVascEntryController().getVascEntryByGroupId(groupId)) {
			VascEntry ve = vc.getVascEntryController().getVascEntryById(vascEntryId);
			if (ve.getAccessType()!=null && (
					ve.getAccessType()==VascEntryAccessType.ENTRY_LINK |
					ve.getAccessType()==VascEntryAccessType.ENTRY_LIST
					)
				) {
				continue;
			}
			if (filterVascMenuRoles(ve.getView(),ve.getRolesView())==false) {
				continue;
			}
			VascMenu menu = new VascMenu();
			menu.setVascEntryId(ve.getId());
			menu.setVascGroupId(ve.getVascGroupId());
			menu.setTitleKey(ve.getName());
			result.add(menu);
		}
	//	Collections.sort(result,vascMenuComparator);
		return result;		
	}
	
	public List<VascMenuWeb> getFilteredMenuWeb(VascMenuWebType type) {
		if (type==null) {
			throw new NullPointerException("Can't filter on null type.");
		}
		List<VascMenuWeb> result = new ArrayList<VascMenuWeb>(15);
		for (VascMenuWeb menu:fetchVascMenuWeb()) {
			if (type.equals(menu.getMenuType())) {
				result.add(menu);
			}
		}
		Collections.sort(result,vascMenuWebComparator);
		return result;		
	}
}