/*
 * Copyright 2009-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.demo.tech.web.beans;

import java.io.Serializable;
import java.util.List;

import javax.naming.InitialContext;
import javax.naming.NamingException;

import net.forwardfire.vasc.demo.tech.domain.menu.VascMenuController;
import net.forwardfire.vasc.demo.tech.domain.menu.model.VascMenuGroup;
import net.forwardfire.vasc.demo.tech.domain.menu.model.VascMenuWeb;
import net.forwardfire.vasc.demo.tech.domain.menu.model.VascMenuWebType;

/**
 * MenuController Shows the menu for the user. 
 * 
 * @author Willem Cazander
 * @version 1.0 May 19, 2012
 */
public class MenuController implements Serializable {

	private static final long serialVersionUID = -6820749860984575869L;
	private UserController userController = null;
	private VascMenuController vascMenuController = null;
	
	private void init() {
		// RM ME
		if (vascMenuController==null) {
			try {
				InitialContext ctx = new InitialContext();
				vascMenuController = (VascMenuController)ctx.lookup("java:app/demo/vascMenuController");
			} catch (NamingException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}
	
	private List<VascMenuWeb> getMenuFiltered(VascMenuWebType type) {
		init();
		return vascMenuController.getFilteredMenuWeb(type);
	}
	
	public List<VascMenuGroup> getVascMenuGroup() {
		init();
		List<VascMenuGroup> result = vascMenuController.getFilteredMenuGroup();
		return result;
	}
	
	public List<VascMenuWeb> getMenuBarLeft() {
		return getMenuFiltered(VascMenuWebType.BAR_LEFT);
	}
	
	public List<VascMenuWeb> getMenuBarRight() {
		return getMenuFiltered(VascMenuWebType.BAR_RIGHT);
	}
	
	public List<VascMenuWeb> getMenuBarBottom() {
		return getMenuFiltered(VascMenuWebType.BAR_BOTTOM);
	}
	
	public List<VascMenuWeb> getMenuPageIndex() {
		return getMenuFiltered(VascMenuWebType.PAGE_INDEX);
	}
	
	public List<VascMenuWeb> getMenuPageUserLeft() {
		return getMenuFiltered(VascMenuWebType.PAGE_USER_LEFT);
	}
	
	public List<VascMenuWeb> getMenuPageUserRight() {
		return getMenuFiltered(VascMenuWebType.PAGE_USER_RIGHT);
	}
	
	public List<VascMenuWeb> getMenuPageAdmin() {
		return getMenuFiltered(VascMenuWebType.PAGE_ADMIN);
	}
	
	public List<VascMenuWeb> getMainMenu0() {
		return getMenuFiltered(VascMenuWebType.MAIN_MENU_0);
	}

	public List<VascMenuWeb> getMainMenu1() {
		return getMenuFiltered(VascMenuWebType.MAIN_MENU_1);
	}
	
	public List<VascMenuWeb> getMainMenu2() {
		return getMenuFiltered(VascMenuWebType.MAIN_MENU_2);
	}
	
	/**
	 * @return the userController
	 */
	public UserController getUserController() {
		return userController;
	}

	/**
	 * @param userController the userController to set
	 */
	public void setUserController(UserController userController) {
		this.userController = userController;
	}
}