/*
 * Copyright 2009-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.demo.tech.web.beans;

import java.io.IOException;
import java.io.Serializable;
import java.util.Properties;

import javax.faces.context.FacesContext;
import javax.faces.event.ActionEvent;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.servlet.http.HttpSession;

import net.forwardfire.vasc.demo.tech.domain.user.ClientUserController;
import net.forwardfire.vasc.demo.tech.web.models.WebUser;

/**
 * 
 * @author Willem Cazander
 * @version 1.0 Nov 1, 2009
 */
public class UserController implements Serializable {

	private static final long serialVersionUID = 1L;
	static public final String WEB_USER_SESSION_KEY = "webUser";
	
	public String getWebUserName() {
		WebUser user = getWebUser();
		if (user!=null) {
			return user.getFullName();
		}
		return "No-name";
	}
	
	public WebUser getWebUser() {
		WebUser user = (WebUser)FacesContext.getCurrentInstance().getExternalContext().getSessionMap().get(WEB_USER_SESSION_KEY);
		if (user!=null) {
			return user;
		}
		if (isUserLoggedin()==false) {
			try {
				FacesContext.getCurrentInstance().getExternalContext().redirect("/demo/html/auth/error.jsf");
			} catch (IOException ioe) {
				throw new RuntimeException("Could not redirect user to login-form.: "+ioe.getMessage(),ioe);
			}
		}
		
		HttpSession session = (HttpSession)FacesContext.getCurrentInstance().getExternalContext().getSession(true);
		session.setMaxInactiveInterval(20*60); // 20 minutes.
		
		// fetch user info and settings/etc/etc from db.
		user = new WebUser();
		user.setId(123l);
		user.setFullName(FacesContext.getCurrentInstance().getExternalContext().getRemoteUser());
		user.setLoginName(FacesContext.getCurrentInstance().getExternalContext().getRemoteUser());
		FacesContext.getCurrentInstance().getExternalContext().getSessionMap().put(WEB_USER_SESSION_KEY, user);

		/*
		try {
			Properties properties = new Properties();
			properties.setProperty (Context.INITIAL_CONTEXT_FACTORY,"org.apache.openejb.client.LocalInitialContextFactory");
			InitialContext ic = new InitialContext(properties);
			Object object = ic.lookup("java:app/demo/vascServiceManager");
			System.out.println("ob: "+object);
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		try {
			InitialContext ic = new InitialContext();
			LoginUserController object = (LoginUserController)ic.lookup("java:app/demo/loginUserController");
			System.out.println("ob: "+object);
			System.out.println("login: "+object.doClientLogin());
		} catch (Exception e) {
			e.printStackTrace();
		}
		 */
		
		return user;
	}
	
	public void logoutAction(ActionEvent event) {
		HttpSession session = (HttpSession)FacesContext.getCurrentInstance().getExternalContext().getSession(true);
		session.invalidate();
		try {
			FacesContext.getCurrentInstance().getExternalContext().redirect("/demo/html/auth/logout.jsf");
		} catch (IOException ioe) {
			throw new RuntimeException("Could not redirect user to login-form.: "+ioe.getMessage(),ioe);
		}
	}
	
	public boolean hasUserPrincipal() {
		return FacesContext.getCurrentInstance().getExternalContext().getUserPrincipal()!=null;
	}
	
	public boolean hasUserRole(String role) {
		return FacesContext.getCurrentInstance().getExternalContext().isUserInRole(role);
	}
	
	public boolean isUserLoggedin() {
		return hasUserRole("login");
	}
	
	public boolean isRoleUserLogin() {
		return hasUserRole("login");
	}
	
	public boolean isRoleUserAdmin() {
		return hasUserRole("admin");
	}
}