package net.forwardfire.vasc.demo.tech.web.faces;

import java.io.InputStream;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Properties;
import java.util.concurrent.atomic.AtomicReference;
 
 
import javax.faces.component.UIViewRoot;
import javax.faces.context.FacesContext;
import javax.faces.event.AbortProcessingException;
import javax.faces.event.PreRenderViewEvent;
import javax.faces.event.SystemEvent;
import javax.faces.event.SystemEventListener;
 
 
import org.richfaces.resource.ResourceKey;
 
 
//import com.sun.faces.renderkit.html_basic.ScriptRenderer;
//import com.sun.faces.renderkit.html_basic.StylesheetRenderer;
 
 
/**
 * <p>
 * Satisfies that links for selected RichFaces won't be put to renderer view.
 * </p>
 *
 * <p>
 * Resource which should not be loaded are selected by condition in {@link #shouldNotBeLoaded(ResourceKey, String)}.
 * </p>
 *
 * @author Lukas Fryc
 */
public class NoneLoadStyleResourceEventListener implements SystemEventListener {
 
 
    private static final String STATIC_MAPPING_FILE = "META-INF/richfaces/staticResourceMapping/Static.properties";
 
 
    private AtomicReference<Properties> propertiesRef = new AtomicReference<Properties>();
 
 
    public void processEvent(SystemEvent event) throws AbortProcessingException {
        if (event instanceof PreRenderViewEvent) {
            putSelectedResourceKeysToContextMap();
        }
    }
 
    public boolean isListenerForSource(Object source) {
        return source instanceof UIViewRoot;
    }
 
 
    /**
     * <p>
     * Adds all keys of resources which meets {@link #shouldNotBeLoaded(ResourceKey, String)} condition to context map.
     * </p>
     *
     * <p>
     * Adding resource key to context map ensures that {@link ScriptRenderer} {@link StylesheetRenderer} won't render the link
     * to page.
     * </p>
     */
    private void putSelectedResourceKeysToContextMap() {
        FacesContext facesContext = FacesContext.getCurrentInstance();
        loadStaticResourceMappingProperties();
        for (Entry<Object, Object> entry : propertiesRef.get().entrySet()) {
            ResourceKey resourceKey = ResourceKey.create((String) entry.getKey());
            if (shouldNotBeLoaded(resourceKey)) {
                putResourceKeyToContextMap(facesContext, resourceKey);
            }
        }
    }
 
 
    /**
     * Loads configuration of static resource mapping which lists all RichFaces resources.
     */
    private void loadStaticResourceMappingProperties() {
        if (propertiesRef.get() == null) {
            Properties loadedProperties = new Properties();
            InputStream inputStream = this.getClass().getClassLoader().getResourceAsStream(STATIC_MAPPING_FILE);
            try {
                loadedProperties.load(inputStream);
            } catch (Exception e) {
                throw new RuntimeException("Unable to load resource-mapping configuration file", e);
            }
            propertiesRef.compareAndSet(null, loadedProperties);
        }
    }
 
    /**
     * <p>
     * Adds resource key to context map.
     * </p>
     *
     * <p>
     * Adding resource key to context map ensures that {@link ScriptRenderer} {@link StylesheetRenderer} won't render the link
     * to page.
     * </p>
     *
     * @param facesContext the current {@link FacesContext} instance
     * @param resourceKey the key of the resource
     */
    private void putResourceKeyToContextMap(FacesContext facesContext, ResourceKey resourceKey) {
        Map<Object, Object> contextMap = facesContext.getAttributes();
 
        String resourceName = resourceKey.getResourceName();
        String libraryName = resourceKey.getLibraryName();
        String key = resourceName + libraryName;
        if (!contextMap.containsKey(key)) { // stylesheets (with this name + library) will not be rendered multiple
                                            // times per request
            contextMap.put(key, Boolean.TRUE);
        }
        if (libraryName == null || libraryName.isEmpty()) { // also store this in the context map with library as
                                                            // "null"
            libraryName = "null";
            key = resourceName + libraryName;
            if (!contextMap.containsKey(key)) {
                contextMap.put(key, Boolean.TRUE);
            }
        }
    }
 
    /**
     * Returns true if given resourceKey should not be loaded for current view.
     *
     * @param resourceKey the key of the resource
     * @return true if given resourceKey should not be loaded for current view; false otherwise
     */
    private boolean shouldNotBeLoaded(ResourceKey resourceKey) {
        String resourceName = resourceKey.getResourceName();
        return resourceName.endsWith(".js") || resourceName.endsWith(".css") || resourceName.endsWith(".ecss");
    }
}