/*
 * Copyright 2009-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.demo.tech.web.pages;

import java.io.Serializable;
import java.lang.reflect.Method;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;

import javax.faces.context.FacesContext;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.servlet.ServletRequest;
import javax.sql.DataSource;

import de.tudarmstadt.ukp.wikipedia.parser.ParsedPage;
import de.tudarmstadt.ukp.wikipedia.parser.Section;
import de.tudarmstadt.ukp.wikipedia.parser.html.HtmlWriter;
import de.tudarmstadt.ukp.wikipedia.parser.mediawiki.MediaWikiParser;
import de.tudarmstadt.ukp.wikipedia.parser.mediawiki.MediaWikiParserFactory;

import net.forwardfire.vasc.demo.tech.web.beans.UserController;
import net.forwardfire.vasc.demo.tech.web.pages.model.VascPage;
import net.forwardfire.vasc.demo.tech.web.pages.model.VascPagePart;
import net.forwardfire.vasc.demo.tech.web.pages.model.VascPagePartType;

/**
 * VascPageController very hacky page context controller. 
 * 
 * @author Willem Cazander
 * @version 1.0 May 19, 2012
 */
public class VascPageController implements Serializable {

	private static final long serialVersionUID = -6820749860984575869L;
	private UserController userController = null;
	private VascPage page = null;

	public VascPage fetchVascPage(String slug) {
		VascPage result = null;
		Connection connection = null;
		try {
			DataSource ds = getDataSource("java:comp/env/jdbc/DemoManagerDataDS");
			connection = ds.getConnection();
			Statement s = connection.createStatement();
			s.execute("SELECT * FROM VASC_PAGE WHERE SLUG='"+slug+"'"); // bad so redo .. 
			ResultSet rs = s.getResultSet();
			if (rs.next()) {
				result = new VascPage();
				result.setId(rs.getInt(1));
				result.setSlug(rs.getString(2));
				result.setTitle(rs.getString(3));
				result.setActive(rs.getBoolean(4));
				result.setSitemap(rs.getBoolean(5));
				result.setRoles(rs.getString(6));
				/*
				menu.setMenuOrder(rs.getInt(7));
				menu.setMenuType(VascMenuType.valueOf(rs.getString(8)));
				if (filterVascMenuRoles(menu)==false) {
					continue;
				}*/
			}
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			if (connection!=null) {
				try {
					connection.close();
				} catch (Exception e) {
				}
			}
		}
		return result;
	}
	
	public List<VascPagePart> fetchVascPageParts(int pageId) {
		List<VascPagePart> result = new ArrayList<VascPagePart>(50);
		Connection connection = null;
		try {
			DataSource ds = getDataSource("java:comp/env/jdbc/DemoManagerDataDS");
			connection = ds.getConnection();
			Statement s = connection.createStatement();
			s.execute("SELECT * FROM VASC_PAGE_PART WHERE PAGE_ID="+pageId); // bad
			ResultSet rs = s.getResultSet();
			while (rs.next()) {
				VascPagePart menu = new VascPagePart();
				menu.setId(rs.getInt(1));
				menu.setPageId(rs.getInt(2));
				menu.setTitle(rs.getString(3));
				menu.setText(rs.getString(4));
				menu.setActive(rs.getBoolean(5));
				menu.setSitemap(rs.getBoolean(6));
				menu.setPartOrder(rs.getInt(7));
				menu.setPartType(VascPagePartType.valueOf(rs.getString(8)));
				menu.setRoles(rs.getString(9));
				/*
				menu.setMenuOrder(rs.getInt(7));
				menu.setMenuType(VascMenuType.valueOf(rs.getString(8)));
				if (filterVascMenuRoles(menu)==false) {
					continue;
				}*/
				result.add(menu);
			}
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			if (connection!=null) {
				try {
					connection.close();
				} catch (Exception e) {
				}
			}
		}
		return result;
	}
	
	private DataSource getDataSource(String name) throws SQLException {
		try {
			Context initialContext = new InitialContext();
			DataSource datasource = (DataSource)initialContext.lookup(name);
			if ( datasource == null ) {
				throw new SQLException("Cannot lookup datasource: "+name);
			}
			return datasource;
		} catch ( NamingException e ) {
			throw new SQLException("Cannot get connection " + e,e);
		}
	}
	
	/*
	private boolean filterVascMenuRoles(VascMenu menu) {
		if (menu.getActive()!=null && menu.getActive()==false) {
			return false;
		}
		if (menu.getRoles()!=null && menu.getRoles().isEmpty()==false) {
			String[] roles = menu.getRoles().split(",");
			for (String role:roles) {
				if (role.isEmpty()) {
					continue;
				}
				if (userController.hasUserRole(role)==false) {
					return false;
				}
			}
		}
		return true;
	}
	*/
	
	public VascPage getPage() {
		
		FacesContext context = FacesContext.getCurrentInstance();
		String pageId = (String)((ServletRequest)context.getExternalContext().getRequest()).getAttribute("pageId");
		
		if (pageId==null) {
			return null; // todo send somewhere
		}
		if (page!=null && page.getSlug().equals(pageId)) {
			return page;
		}
		
		page = fetchVascPage(pageId);
		page.getPageParts().addAll(fetchVascPageParts(page.getId()));
		
		return page;
	}
	
	public String renderContent(VascPagePart part) {
		if (part==null) {
			return "";
		}
		if (VascPagePartType.HTML.equals(part.getPartType())) {
			return part.getText();
		} else if (VascPagePartType.PRE.equals(part.getPartType())) {
			return "<pre>"+part.getText()+"</pre>\n";
		} else {
			MediaWikiParserFactory pf = new MediaWikiParserFactory();
			MediaWikiParser parser = pf.createParser();
			ParsedPage page = parser.parse(part.getText());
			String result = FixedHtmlWriter.parsedPageToHtml(page);
			return result;
		}
	}
	
	static class FixedHtmlWriter extends HtmlWriter {
		public static String parsedPageToHtml( ParsedPage pp ){
			StringBuilder result = new StringBuilder();

			if( pp != null ) {
	    		//Title
	    		result.append(
	    			"<table class=\"ParsedPage\">\n"+
	    			"<tr><th class=\"ParsedPage\">ParsedPage: \n" +
	    			pp.getName()+
	    			"</th></tr>\n");

	    		//Sections
	    		result.append(
	    			"<tr><td class=\"ParsedPage\">\n"		);
	    		for( Section s: pp.getSections() ) {
					result.append( invokePrivate("sectionToHtml",s));
				}
	    		result.append(
	    			"</td></tr>\n");

	    		//Categories
	    		if( pp.getCategoryElement()!= null ){
	    			result.append("<tr><td class=\"ParsedPage\">\n");
	    			result.append("Categories:\n"  + invokePrivate("contentElementToHtml",pp.getCategoryElement() ));
	    			result.append("</td></tr>\n");
	    		}

	    		//Languages
	    		if( pp.getLanguagesElement()!= null ){
	    			result.append("<tr><td class=\"ParsedPage\">\n");
	    			result.append("Languages:\n"  + invokePrivate("contentElementToHtml",pp.getLanguagesElement() ));
	    			result.append("</td></tr>\n");
	    		}

	    		//Finalize
	    		result.append("</table>\n");
	        }

			return result.toString();
		}
		
		static private String invokePrivate(String method,Object arg) {
			for (Method m:FixedHtmlWriter.class.getDeclaredMethods()) {
				if (m.getName().equals(method)) {
					m.setAccessible(true);
					try {
						return (String)m.invoke(null, arg);
					} catch (Exception e) {
						e.printStackTrace();
					} 
				}
			}
			return "";
		}
	}
	
	/**
	 * @return the userController
	 */
	public UserController getUserController() {
		return userController;
	}

	/**
	 * @param userController the userController to set
	 */
	public void setUserController(UserController userController) {
		this.userController = userController;
	}
}