/*
 * Copyright 2009-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.demo.tech.web.pages;

import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.faces.FactoryFinder;
import javax.faces.application.ViewExpiredException;
import javax.faces.component.UIViewRoot;
import javax.faces.context.FacesContext;
import javax.faces.context.FacesContextFactory;
import javax.faces.lifecycle.Lifecycle;
import javax.faces.lifecycle.LifecycleFactory;
import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * VascPageServket  
 * 
 * @author Willem Cazander
 * @version 1.0 Nov 29, 2012
 */
public class VascPageFilter implements Filter {


	private Logger logger = null; 
	private String templateFile = null;
	private ServletContext servletContext = null;
	
	/**
	 * @see javax.servlet.Filter#destroy()
	 */
	public void destroy() {
		servletContext = null;
		templateFile = null;
		logger = null;
	}

	/**
	 * @see javax.servlet.Filter#init(javax.servlet.FilterConfig)
	 */
	public void init(FilterConfig config) throws ServletException {
		logger			= Logger.getLogger(VascPageFilter.class.getName());
		servletContext	= config.getServletContext();
		templateFile	= config.getInitParameter("templateFile");
		if (templateFile==null) {
			throw new ServletException("No templateFile init-param found.");
		}		
	}
	
	/**
	 * @see javax.servlet.Filter#doFilter(javax.servlet.ServletRequest, javax.servlet.ServletResponse, javax.servlet.FilterChain)
	 */
	public void doFilter(ServletRequest servletRequest,ServletResponse servletResponse,FilterChain chain) throws IOException, ServletException {
	
		HttpServletRequest  request  = (HttpServletRequest)servletRequest;
		HttpServletResponse response = (HttpServletResponse)servletResponse;
		
		// request info
		String path = request.getRequestURI();
		String v = "/page/";
		if (path.contains(v)) {
			path  = path.substring(path.indexOf(v)+v.length());
		}
		
		// stuff to fill
		String pageId = null;
		String actionName = null;
		String actionRecordId = null;
		
		// parse request info
		//path = path.substring(1);		// rm /
		int index = path.indexOf("/");	// check next /
		String actionString = null;
		if (index>0) {
			actionString = path.substring(index+1);
			pageId = path.substring(0,index);
		} else {
			pageId = path;
		}
		
		if (actionString!=null) {
			index = actionString.indexOf("/");
			String recordString = null;
			if (index>0) {
				recordString = actionString.substring(index+1);
				actionName = actionString.substring(0,index);  
			} else {
				actionName = actionString;	
			}
			if (recordString!=null) {
				actionRecordId = recordString;
			}
		}
		
		//log
		if (logger.isLoggable(Level.FINE)) {
			logger.log(Level.FINE,"pageId="+pageId+" actionName="+actionName+" actionRecordId="+actionRecordId);
		}
				
		// Acquire the FacesContext instance for this request
		FacesContext facesContext = FacesContext.getCurrentInstance();
		if (facesContext == null) {
			FacesContextFactory facesContextFactory = (FacesContextFactory)FactoryFinder.getFactory(FactoryFinder.FACES_CONTEXT_FACTORY);
			LifecycleFactory    lifecycleFactory    = (LifecycleFactory)FactoryFinder.getFactory(FactoryFinder.LIFECYCLE_FACTORY);
			Lifecycle           lifecycle           = lifecycleFactory.getLifecycle(LifecycleFactory.DEFAULT_LIFECYCLE);
			
			facesContext = facesContextFactory.getFacesContext(servletContext, request, response, lifecycle);
			ProtectedFacesContext.setFacesContextAsCurrentInstance(facesContext);
			
			UIViewRoot viewRoot = facesContext.getApplication().getViewHandler().createView(facesContext,pageId+actionName);
			facesContext.setViewRoot(viewRoot);
		}
		
		// add to request attributes
		request.setAttribute("pageId", pageId);
		
		// And dispatch to the vasc template file.
		try {
			request.getRequestDispatcher(templateFile).forward(request, response);
		} catch (ViewExpiredException e) {
			response.sendRedirect(request.getRequestURL().toString()); // lets try again
		}
	}
	
	private abstract static class ProtectedFacesContext extends FacesContext {
		protected static void setFacesContextAsCurrentInstance(FacesContext facesContext) {
			FacesContext.setCurrentInstance(facesContext);
		}
	}
}