/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.export.generic;

import java.io.IOException;
import java.io.OutputStream;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import net.forwardfire.vasc.backend.VascBackendException;
import net.forwardfire.vasc.core.VascEntry;
import net.forwardfire.vasc.core.VascEntryField;
import net.forwardfire.vasc.core.entry.VascEntryExport;
import net.forwardfire.vasc.core.entry.VascEntryExportWriter;
import net.forwardfire.vasc.core.VascException;

/**
 * AbstractVascEntryExportWriter make read of paged data into methods.
 * 
 * @author Willem Cazander
 * @version 1.0 Nov 23, 2012
 */
@SuppressWarnings("serial")
abstract public class AbstractVascEntryExportWriter implements VascEntryExportWriter {
	
	private VascEntryExport vascEntryExport = null;
	private VascEntry vascEntry = null;
	
	public void doInit(VascEntryExport vascEntryExport,VascEntry vascEntry) throws VascException {
		this.vascEntryExport=vascEntryExport;
		this.vascEntry=vascEntry;
	}
	
	public void doExport(OutputStream out) throws IOException,VascException {
		
		doExportInit(out);
		doExportStart();
		
		VascEntry entry = getVascEntry();
		int oldIndex = entry.getVascFrontendController().getVascEntryState().getVascBackendState().getPageIndex();
		Long total = entry.getVascFrontendController().getVascEntryState().getTotalBackendRecords();
		int pages = total.intValue()/entry.getVascFrontendController().getVascEntryState().getVascBackendState().getPageSize();
		for (int page=0;page<=pages;page++) {
			entry.getVascFrontendController().getVascEntryState().getVascBackendState().setPageIndex(page);
			entry.getVascFrontendController().getVascFrontendActions().refreshData();
			
			int dataSize = entry.getVascFrontendController().getVascEntryState().getEntryDataList().size();
			for (int dataIndex=0;dataIndex<dataSize;dataIndex++) {
				Serializable o = entry.getVascFrontendController().getVascEntryState().getEntryDataList().get(dataIndex);
				doExportRowStart(o);
				try {
					doExportRowFields(o);
				} catch (VascBackendException e) {
					throw new VascException(e);
				}
				doExportRowEnd(o,(dataIndex==dataSize-1));
			}
		}
		doExportEnd();
		
		// restore old page size
		entry.getVascFrontendController().getVascEntryState().getVascBackendState().setPageIndex(oldIndex);
	}
	
	protected VascEntry getVascEntry() {
		return vascEntry;
	}
	

	protected VascEntryExport getVascEntryExport() {
		return vascEntryExport;
	}
	
	abstract protected void doExportInit(OutputStream out);
	
	abstract protected void doExportStart() throws IOException;
	abstract protected void doExportEnd() throws IOException;
	
	abstract protected void doExportRowStart(Serializable row) throws IOException;
	abstract protected void doExportRowEnd(Serializable row,boolean isLast) throws IOException;
	
	protected void doExportRowFields(Serializable row) throws IOException,VascBackendException {
		// todo: rm me and use data selector
		List<VascEntryField> fieldList = new ArrayList<VascEntryField>(getVascEntry().getVascEntryFields());
		int fields = fieldList.size();
		for (int i=0;i<fields;i++) {
			VascEntryField c = fieldList.get(i);
			doExportRowField(row,c,(i==fields-1));
		}
	}
	
	abstract protected void doExportRowField(Serializable row,VascEntryField field,boolean isLast) throws IOException,VascBackendException;
}