/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.export.generic;

import java.io.OutputStream;
import java.io.PrintWriter;
import java.io.Serializable;

import net.forwardfire.vasc.backend.VascBackendException;
import net.forwardfire.vasc.core.VascEntryField;

/**
 * VascEntryExportWriterXml writes entry data to xml format.
 * 
 * @author Willem Cazander
 * @version 1.0 Mar 30, 2007
 */
public class VascEntryExportWriterXml extends AbstractVascEntryExportWriter {
	
	private static final long serialVersionUID = 3719424578585760828L;
	static public final String EXPORT_TYPE = "xml";
	private boolean xmlTree = false;
	private PrintWriter p = null;

	@Override
	protected void doExportInit(OutputStream out) {
		p = new PrintWriter(out);
		String xmlTreeString = getVascEntryExport().getWriterParameter("xmltree");
		if (xmlTreeString!=null && "true".equalsIgnoreCase(xmlTreeString)) {
			xmlTree = true;	
		}
	}

	@Override
	protected void doExportStart() {
		p.write("<?xml version=\"1.0\"?>\n");
		p.write("<data>\n");
	}

	@Override
	protected void doExportEnd() {
		p.write("</data>\n");
		p.flush();
	}

	@Override
	protected void doExportRowStart(Serializable row) {
		p.write("\t<row>\n");
	}

	@Override
	protected void doExportRowEnd(Serializable row,boolean isLast) {
		p.write("\t</row>\n");
		p.flush();
	}

	@Override
	protected void doExportRowField(Serializable o, VascEntryField c,boolean isLast) throws VascBackendException {
		if (xmlTree) {
			p.write("\t\t<"+c.getId()+"><![CDATA[");
			p.write(""+c.getVascEntryFieldValue().getValue(c.getBackendName(), o));
			p.write("]]></"+c.getId()+">\n");
		} else {
			p.write("\t\t<column name=\""+c.getId()+"\"><![CDATA[");
			p.write(""+c.getVascEntryFieldValue().getValue(c.getBackendName(), o));
			p.write("]]></column>\n");
		}
	}
	
	/**
	 * @see net.forwardfire.vasc.core.entry.VascEntryExport#getMineType()
	 */
	public String getMineType() {
		return "text/xml";
	}

	/**
	 * @see net.forwardfire.vasc.core.entry.VascEntryExport#getFileType()
	 */
	public String getFileType() {
		return EXPORT_TYPE;
	}
}
