/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.export.jr4o;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import net.forwardfire.vasc.backend.VascBackendException;
import net.forwardfire.vasc.core.VascEntry;
import net.forwardfire.vasc.core.VascEntryField;
import net.forwardfire.vasc.frontend.VascFrontendDataSelector.EntryFieldSelectType;
import net.forwardfire.vasc.lib.jr4o.data.AbstractJRDynamicDataSource;
import net.sf.jasperreports.engine.JRException;
import net.sf.jasperreports.engine.JRField;
import net.sf.jasperreports.engine.design.JRDesignField;
import net.sf.jasperreports.engine.design.JasperDesign;

/**
 * JRDynamicDataSourceVascEntry converts the Vasc backend data to JasperReports data.
 * 
 * @author Willem Cazander
 * @version 1.0 May 11, 2012
 */
public class JRDynamicDataSourceVascEntry extends AbstractJRDynamicDataSource {
	

	private VascEntry vascEntry = null;
	private List<VascEntryField> fields = null;
	private long total = -1;
	private int index = -1;
	private static final String PREFIX_COLUMN = "COLUMN_";
	private static final String PREFIX_HEADER = "HEADER_";
	private static final String INFIX_CLASS = "CLASS_";

	private List<List<String>> data = null;
	
	public JRDynamicDataSourceVascEntry(VascEntry vascEntry) {
		if (vascEntry==null) {
			throw new NullPointerException("vascEntry can't be null.");
		}
		this.vascEntry = vascEntry;
		total = vascEntry.getVascFrontendController().getVascEntryState().getTotalBackendRecords();
		
		fields = vascEntry.getVascFrontendController().getVascFrontendDataSelector().getFields(EntryFieldSelectType.EXPORT);
		
		// hackje until calc index to page and row.
		try {
			data = new ArrayList<List<String>>(1000);
			int pages = (int)total/vascEntry.getVascFrontendController().getVascEntryState().getVascBackendState().getPageSize();
			for (int page=0;page<=pages;page++) {
				vascEntry.getVascFrontendController().getVascEntryState().getVascBackendState().setPageIndex(page);
				vascEntry.getVascFrontendController().getVascFrontendActions().refreshData();
				for (Serializable o:vascEntry.getVascFrontendController().getVascEntryState().getEntryDataList()) {
					List<String> row = new ArrayList<String>(30);
					for (VascEntryField c:fields) {
						Object rowData = c.getVascEntryFieldValue().getValue(c.getBackendName(), o);
						if (rowData == null) {
							row.add(null);
						} else {
							row.add(rowData.toString());
						}
					}
					data.add(row);
				}
			}
		} catch (VascBackendException ve) {
			throw new RuntimeException(ve); // TODO: replace me
		}
	}

	/**
	 * @see net.sf.jasperreports.engine.JRDataSource#getFieldValue(net.sf.jasperreports.engine.JRField)
	 */
	public Object getFieldValue(JRField jrField) throws JRException {
		String fieldName = jrField.getName();
		if (fieldName.startsWith(PREFIX_COLUMN)) {
			return data.get(index).get(Integer.parseInt(fieldName.substring(7)));
		}
		if (fieldName.startsWith(PREFIX_HEADER)) {
			VascEntryField field = fields.get(Integer.parseInt(fieldName.substring(7)));
			return vascEntry.getVascFrontendController().getVascEntryResourceResolver().getTextValue(field.getName());
		}
		throw new JRException("Unknown column name : " + fieldName);
	}

	/**
	 * @see net.forwardfire.vasc.lib.jr4o.data.JRDynamicDataSource#addDynamicColumnClassFields(net.sf.jasperreports.engine.design.JasperDesign)
	 */
	public void addDynamicColumnClassFields(JasperDesign jd) throws JRException {
		JRDesignField field;
		for (int i=0;i<getDynamicColumnCount();i++) {
			field = new JRDesignField();
			field.setName(PREFIX_COLUMN+i);
			field.setValueClass(String.class); // todo 
			jd.addField(field);
			
			field = new JRDesignField();
			field.setName(PREFIX_HEADER+i);
			field.setValueClass(String.class);
			jd.addField(field);
		}
	}

	/**
	 * @see net.forwardfire.vasc.lib.jr4o.data.JRDynamicDataSource#addDynamicELBean()
	 */
	public Map<String, Object> addDynamicELBean() {
		Map<String,Object> result = new HashMap<String,Object>(10);
		for (int i=0;i<getDynamicColumnCount();i++) {
			result.put(PREFIX_COLUMN+INFIX_CLASS+i,String.class); // todo 
			result.put(PREFIX_HEADER+INFIX_CLASS+i,String.class);
		}
		return result;
	}

	/**
	 * @see net.forwardfire.vasc.lib.jr4o.data.JRDynamicDataSource#getDynamicColumnX(int)
	 */
	public int getDynamicColumnX(int col) {
		int result = 0;
		for (int i=0;i<col;i++) {
			result += 80;
		}
		return result;
	}

	/**
	 * @see net.forwardfire.vasc.lib.jr4o.data.JRDynamicDataSource#getDynamicColumnWidth(int)
	 */
	public int getDynamicColumnWidth(int col) {
		return 80; // todo
	}

	/**
	 * @see net.forwardfire.vasc.lib.jr4o.data.JRDynamicDataSource#getDynamicColumnCount()
	 */
	public int getDynamicColumnCount() {
		return vascEntry.getVascEntryFields().size();
	}

	/**
	 * @see net.sf.jasperreports.engine.JRDataSource#next()
	 */
	public boolean next() throws JRException {
		index++;
		return (index < total);
	}
	
	/**
	 * @see net.sf.jasperreports.engine.JRRewindableDataSource#moveFirst()
	 */
	public void moveFirst() throws JRException {
		index = -1;
	}
}