/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.export.jr4o;

import java.io.File;
import java.io.OutputStream;
import java.util.HashMap;
import java.util.Map;

import net.forwardfire.vasc.core.VascEntry;
import net.forwardfire.vasc.core.entry.VascEntryExport;
import net.forwardfire.vasc.core.entry.VascEntryExportWriter;
import net.forwardfire.vasc.core.VascException;
import net.forwardfire.vasc.lib.jr4o.JR4ODesignManager;
import net.forwardfire.vasc.lib.jr4o.JR4ODesignManager.JRExportType;

/**
 * VascEntryExportWriterJR4O creates an pdf view of the data.
 * 
 * @author Willem Cazander
 * @version 1.0 May 11, 2012
 */
public class VascEntryExportWriterJR4O implements VascEntryExportWriter {

	private static final long serialVersionUID = -3951608685719832654L;
	private VascEntry entry = null;
	private JRExportType reportType = JRExportType.PDF;
	private String reportResource = null;
	private String reportName = null;
	private File reportFile = null;
	
	/**
	 * @see net.forwardfire.vasc.core.entry.VascEntryExportWriter#doInit(net.forwardfire.vasc.core.entry.VascEntryExport, net.forwardfire.vasc.core.VascEntry)
	 */
	@Override
	public void doInit(VascEntryExport export, VascEntry vascEntry) throws VascException {
		this.entry = vascEntry;
		
		String reportTypeString = export.getWriterParameter("reportType");
		if (reportTypeString!=null && reportTypeString.isEmpty()==false) {
			reportType = JRExportType.valueOf(reportTypeString);
		}
		reportName = export.getWriterParameter("reportName");
		if (reportName==null) {
			throw new NullPointerException("Can work with null reportName parameter.");
		}
		
		String reportFileString = export.getWriterParameter("reportFile");
		if (reportFileString!=null && reportFileString.isEmpty()==false) {
			reportFile = new File(reportFileString);
		}
		reportResource = export.getWriterParameter("reportResource");
	}

	/**
	 * @see net.forwardfire.vasc.core.entry.VascEntryExport#doExport(java.io.OutputStream, net.forwardfire.vasc.core.VascEntry)
	 */
	public void doExport(OutputStream out) throws VascException {

		Map<String,Object> parameters = new HashMap<String,Object>(5);
		parameters.put("title", entry.getVascFrontendController().getVascEntryResourceResolver().getTextValue(entry.getName()));
		parameters.put("description", entry.getVascFrontendController().getVascEntryResourceResolver().getTextValue(entry.getListDescription()));
		parameters.put("titleSubject", entry.getVascFrontendController().getVascEntryResourceResolver().getTextValue(entry.getId()));
		
		int oldIndex = entry.getVascFrontendController().getVascEntryState().getVascBackendState().getPageIndex();
		String bundle = entry.getVascFrontendController().getVascController().getVascEntryConfigController().getResourceBundle();
		JRDynamicDataSourceVascEntry dataSource = new JRDynamicDataSourceVascEntry(entry);
		JR4ODesignManager jr4o = new JR4ODesignManager(dataSource,bundle,parameters);
		try {
			if (reportResource!=null) {
				jr4o.parseResource(reportResource);
			} else if (reportFile!=null) {
				jr4o.parseFile(reportFile);
			} else {
				throw new NullPointerException("No report input data defined,reportResource and reportFile are null.");
			}
			jr4o.saveReportStream(reportName, reportType, out);
		} catch (Exception e) {
			throw new VascException(e);
		} finally {
			entry.getVascFrontendController().getVascEntryState().getVascBackendState().setPageIndex(oldIndex); // restore page index
		}
	}

	/**
	 * @see net.forwardfire.vasc.core.entry.VascEntryExport#getMineType()
	 */
	public String getMineType() {
		return reportType.getMineType();
	}

	/**
	 * @see net.forwardfire.vasc.core.entry.VascEntryExport#getFileType()
	 */
	public String getFileType() {
		return reportType.getFileType();
	}
}