/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.export.json;

import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;

import org.json.simple.JSONObject;
import org.json.simple.JSONValue;

import net.forwardfire.vasc.backend.VascBackendException;
import net.forwardfire.vasc.core.VascEntryField;
import net.forwardfire.vasc.export.generic.AbstractVascEntryExportWriter;

/**
 * VascEntryExportWriterJson exports the entry data to json format.
 * 
 * @author Willem Cazander
 * @version 1.0 Nov 23, 2012
 */
public class VascEntryExportWriterJson extends AbstractVascEntryExportWriter {

	private static final long serialVersionUID = 2965474124451161061L;
	private PrintWriter p = null;
	private Map<String,Object> rowData = null;
	
	public VascEntryExportWriterJson() {
		rowData = new HashMap<String,Object>(60);
	}
	
	@Override
	public String getMineType() {
		return "text/json";
	}

	@Override
	public String getFileType() {
		return "json";
	}

	@Override
	protected void doExportInit(OutputStream out) {
		p = new PrintWriter(out);
	}

	private void writeField(String key,String value) {
		p.write("\"");
		p.write(key);
		p.write("\": \"");
		p.write(JSONValue.escape(value));
		p.write("\"");
	}
	
	@Override
	protected void doExportStart() {
		p.write("{\"entry\": {\n");
		writeField("id",getVascEntry().getId());
		p.write(",\n");
		writeField("vascGroupId",getVascEntry().getVascGroupId());
		p.write(",\n");
		p.write("\"data\": [\n");
	}

	@Override
	protected void doExportEnd() {
		p.write("]\n");
		p.write("}}\n");
		p.flush();
	}

	@Override
	protected void doExportRowStart(Serializable row) {
		rowData.clear();
	}

	@Override
	protected void doExportRowEnd(Serializable row,boolean isLast) throws IOException {
		JSONObject.writeJSONString(rowData, p);
		if (isLast==false) {
			p.write(",");	
		}
		p.write("\n");
		p.flush();
	}

	@Override
	protected void doExportRowField(Serializable row, VascEntryField field,boolean isLast) throws VascBackendException {
		String key = field.getId();
		Object data = field.getVascEntryFieldValue().getValue(field.getBackendName(), row);
		rowData.put(key, data);
	}
}
