/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.frontend.cxf.server.web;

import java.util.logging.Logger;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;

import net.forwardfire.vasc.core.VascController;
import net.forwardfire.vasc.core.VascControllerProvider;
import net.forwardfire.vasc.core.VascEntry;
import net.forwardfire.vasc.frontend.cxf.server.VascControllerServiceImpl;
import net.forwardfire.vasc.frontend.cxf.server.VascEntryServiceImpl;

import javax.xml.ws.Endpoint;

import org.apache.cxf.Bus;
import org.apache.cxf.BusFactory;
import org.apache.cxf.transport.servlet.CXFNonSpringServlet;

/**
 * VascCXFServlet exports the entries via webservices 
 * 
 * @author Willem Cazander
 * @version 1.0 May 12, 2012
 */
public class VascCXFServlet extends CXFNonSpringServlet  {

	private Logger logger = Logger.getLogger(VascCXFServlet.class.getName());
	private static final long serialVersionUID = 8125039180724973858L;
	private VascController vascController = null;
	private boolean exportTree = false;

	/**
	 * @see org.apache.cxf.transport.servlet.CXFNonSpringServlet#init(javax.servlet.ServletConfig)
	 */
	@Override
	public void init(ServletConfig sc) throws ServletException {
		
		String className = sc.getInitParameter("vascControllerProvider");
		if (className==null) {
			throw new ServletException("Can't start with null vascControllerProvider init parameter.");
		}
		String exportTreeStr = sc.getInitParameter("exportTree");
		if (exportTreeStr!=null && "true".equalsIgnoreCase(exportTreeStr)) {
			exportTree = true;
		}
		String classArgument = null;
		int atIndex = className.indexOf('@');
		if (atIndex > 0) {
			classArgument = className.substring(0,atIndex);
			className = className.substring(atIndex+1);
		}
		try {
			ClassLoader cl = Thread.currentThread().getContextClassLoader();
			if (cl == null) {
				cl = className.getClass().getClassLoader();
			}
			Class<?> clazz = cl.loadClass(className);
			VascControllerProvider vascControllerProvider = null;
			if (classArgument==null) {
				vascControllerProvider = (VascControllerProvider)clazz.newInstance();
			} else {
				vascControllerProvider = (VascControllerProvider)clazz.getConstructor(String.class).newInstance(classArgument);
			}
			vascController = vascControllerProvider.getVascController();
		} catch (Exception e) {
			throw new ServletException("Could not create VascControllerProvider: "+e.getMessage(),e);
		}
		super.init(sc); // runs loadBus
		logger.info("CXF Servlet started. (exportTree: "+exportTree+")");
	}

	@Override
	public void loadBus(ServletConfig servletConfig) {
		super.loadBus(servletConfig);
		Bus bus = getBus();
		BusFactory.setDefaultBus(bus);
		Endpoint.publish("/vascControllerService", new VascControllerServiceImpl(vascController));
		for (String id:vascController.getVascEntryController().getVascEntryIds()) {
			VascEntry ve = vascController.getVascEntryController().getVascEntryById(id);
			try {
				Endpoint.publish("/"+id, new VascEntryServiceImpl(vascController,ve));
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}
}
