/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.frontend.swing;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.Serializable;
import java.util.List;

import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.ListCellRenderer;

import net.forwardfire.vasc.core.VascEntry;
import net.forwardfire.vasc.core.VascException;
import net.forwardfire.vasc.core.actions.GlobalVascAction;
import net.forwardfire.vasc.core.entry.VascEntryFrontendEventListener;
import net.forwardfire.vasc.frontend.VascFrontendPageInfoNumber;
import net.forwardfire.vasc.frontend.VascFrontendException;

/**
 * SwingActionPanel renders some page and download actions.
 * 
 * @author Willem Cazander
 * @version 1.0 Jan 20, 2012
 */
public class SwingActionPanel extends JPanel implements VascEntryFrontendEventListener,ActionListener {
	
	private static final long serialVersionUID = -6935210323705891740L;
	private VascEntry vascEntry = null;
	private JTextField rowNumberField = null;
	private JComboBox gotoDirectPage = null;
	private JComboBox gotoDownload = null;
	private JLabel resultLabel = null;
	private boolean init = false;
	
	public SwingActionPanel(VascEntry vascEntry) {
		
		this.vascEntry=vascEntry;
		
		JPanel result = this; //new JPanel();
		//result.setBackground(Color.pink);
		//result.setBorder(BorderFactory.createLineBorder(Color.BLACK, 1));
		
		result.add(new JLabel("Row Numbers: "));
		rowNumberField = new JTextField(6);
		rowNumberField.addActionListener(this);
		rowNumberField.setText(""+vascEntry.getVascFrontendController().getVascEntryState().getVascBackendState().getPageSize());
		result.add(rowNumberField);
		result.add(new JLabel("Go to: "));
		gotoDirectPage = new JComboBox();
		gotoDirectPage.addActionListener(this);
		result.add(gotoDirectPage);
		result.add(new JLabel("Download: "));
		gotoDownload = new JComboBox();
		gotoDownload.addActionListener(this);
		gotoDownload.setRenderer(new DownloadListCellRenderer());
		result.add(gotoDownload);
		
		resultLabel = new JLabel("           ");
		result.add(resultLabel);
		
		//add(result);
		
		vascEntry.getVascFrontendController().addVascEntryFrontendEventListener(this);
	}
	
	class DownloadListCellRenderer extends JLabel implements ListCellRenderer {
		private static final long serialVersionUID = -2143588238414900498L;
		@Override
		public Component getListCellRendererComponent(JList list, Object value,int index, boolean isSelected, boolean hasFocus) {
			if (value==null) {
				setText("null");
			} else {
				setText(((GlobalVascAction)value).getName());
			}
			return this;
		}
	}
	
	@Override
	public void actionPerformed(ActionEvent event) {
		if (init) {
			return;
		}
		if (rowNumberField.equals(event.getSource())) {
			int pageSize = new Integer(rowNumberField.getText());
			vascEntry.getVascFrontendController().getVascEntryState().getVascBackendState().setPageSize(pageSize);
			vascEntry.getVascFrontendController().getVascFrontendActions().refreshData();
		}
		if (gotoDirectPage.equals(event.getSource())) {
			if (gotoDirectPage.getSelectedIndex()==0) {
				return;
			}
			vascEntry.getVascFrontendController().getVascFrontendActions().pageAction(gotoDirectPage.getSelectedIndex() - 1);
			gotoDirectPage.setSelectedIndex(0);
		}
		if (gotoDownload.equals(event.getSource())) {
			GlobalVascAction action = (GlobalVascAction)gotoDownload.getSelectedItem();
			if (action==null) {
				return;
			}
			String id = (String)action.getId();
			if (id==null) {
				return;
			}
			if ("null".equals(id)) {
				return;
			}
			try {
				action.doGlobalAction(vascEntry);
			} catch (VascException e) {
				throw new VascFrontendException(e);
			}
			
			// restore normal view for next request.
			/*
			comp.setRenderFacetState("listView");
			VascEntryExporter ex = getSelectedExporter();
			
			if (ex==null) {
				logger.fine("No exporter selected for download.");
				return;
			}
			*/
			gotoDownload.setSelectedIndex(0);
		}
	}
	
	
	/**
	 * @see net.forwardfire.vasc.core.entry.VascEntryFrontendEventListener#getEventTypes()
	 */
	@Override
	public VascFrontendEventType[] getEventTypes() {
		VascFrontendEventType[] result = {VascEntryFrontendEventListener.VascFrontendEventType.POST_READ};
		return result;
	}
	
	@Override
	public void vascEvent(VascEntry entry,Serializable dataNotUsed) {
		
		init = true;
	
		long pageSize = entry.getVascFrontendController().getVascFrontendPageInfo().getPageSize();
		long pageStart = entry.getVascFrontendController().getVascFrontendPageInfo().getPageStartCount();
		long pageStop = entry.getVascFrontendController().getVascFrontendPageInfo().getPageStopCount();
		long pageTotalCount = entry.getVascFrontendController().getVascFrontendPageInfo().getPageTotalRecordCount();
		String format = "Results %1$d-%2$d from %3$d rows";
		resultLabel.setText(String.format(format, pageStart, pageStop, pageTotalCount));
		rowNumberField.setText(""+pageSize);
		
		gotoDirectPage.removeAllItems();
		gotoDirectPage.addItem("Goto...");
		
		List<VascFrontendPageInfoNumber> pages = vascEntry.getVascFrontendController().getVascFrontendPageInfo().getTablePagesFromBackend();
		int i=0;
		for (VascFrontendPageInfoNumber page:pages) {
			//pages.setRowIndex(i);
			//VascBackendPageNumber page = (VascBackendPageNumber)getTablePagesDataModel().getRowData();
			// = new SelectItem();
			//String name = i18nMap.get("generic.vasc.jsf.table.page.name");
			//String description = i18nMap.get("generic.vasc.jsf.table.page.description");
			/*
			s.setLabel(name+page.getPageNumber()+"  "+(i*pageSize)+"-"+((i*pageSize)+pageSize));
			s.setDescription(description+page.getPageNumber());
			s.setValue(page.getPageNumber());
			result.add(s);
			*/
			//if (page.getSelected()) {
		//		setSelectedItem(arg0)
		//	}
			gotoDirectPage.addItem("page: "+page.getPageNumber()+"  "+(i*pageSize)+"-"+((i*pageSize)+pageSize));
			i++;
		}
		
		
		/*
		SelectItem s = new SelectItem();
		s.setLabel(i18nMap.get("generic.vasc.jsf.table.export.select"));
		s.setDescription(i18nMap.get("generic.vasc.jsf.table.export.select.alt"));
		s.setValue("null");
		result.add(s);*/
		gotoDownload.removeAllItems();
		
		GlobalVascAction empty = new GlobalVascAction() {
			
			public void setName(String name) {
				// TODO Auto-generated method stub
				
			}
			
			public void setImage(String image) {
				// TODO Auto-generated method stub
				
			}
			
			public void setId(String id) {
				// TODO Auto-generated method stub
				
			}
			
			public void setHelpId(String helpId) {
				// TODO Auto-generated method stub
				
			}
			
			public void setDescription(String description) {
				// TODO Auto-generated method stub
				
			}
			
			@Override
			public String getName() {
				return "...";
			}
			
			@Override
			public String getImage() {
				// TODO Auto-generated method stub
				return null;
			}
			
			@Override
			public String getId() {
				// TODO Auto-generated method stub
				return null;
			}
			
			@Override
			public String getHelpId() {
				// TODO Auto-generated method stub
				return null;
			}
			
			@Override
			public String getDescription() {
				// TODO Auto-generated method stub
				return null;
			}
			
			@Override
			public void doGlobalAction(VascEntry vascEntry) throws VascException {
				// TODO Auto-generated method stub
			}
			
			@Override
			public GlobalVascAction clone() {
				return this;
			}

			@Override
			public Integer getOrder() {
				// TODO Auto-generated method stub
				return null;
			}

			@Override
			public Boolean getView() {
				// TODO Auto-generated method stub
				return null;
			}

			@Override
			public String getRolesView() {
				// TODO Auto-generated method stub
				return null;
			}

			public Object cloneCreate() throws CloneNotSupportedException {
				// TODO Auto-generated method stub
				return null;
			}

			public void cloneFields(Object cloneObject)
					throws CloneNotSupportedException {
				// TODO Auto-generated method stub
				
			}
		};
		gotoDownload.addItem(empty);
		
		for (GlobalVascAction a:entry.getExportActions()) {
			gotoDownload.addItem(a);
		}
		
		
		init = false;
	}



}
