/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.frontend.swing;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.Serializable;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;

import net.forwardfire.vasc.core.VascEntry;
import net.forwardfire.vasc.core.entry.VascEntryFrontendEventListener;
import net.forwardfire.vasc.frontend.VascFrontendPageInfoNumber;

/**
 * SwingPagerPanel renders dynamic pager for swing.
 * 
 * @author Willem Cazander
 * @version 1.0 Jan 21, 2012
 */
public class SwingPagerPanel extends JPanel implements VascEntryFrontendEventListener,ActionListener {
	
	private static final long serialVersionUID = 2651163640840746196L;
	private VascEntry vascEntry = null;
	private JPanel result = null;

	public SwingPagerPanel(VascEntry vascEntry) {
		this.vascEntry=vascEntry;
		result = new JPanel();
		add(result);
		vascEntry.getVascFrontendController().addVascEntryFrontendEventListener(this);
	}
	
	@Override
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() instanceof JButton) {
			JButton but = (JButton)e.getSource();
			Integer page = new Integer(but.getText());
			vascEntry.getVascFrontendController().getVascFrontendActions().pageAction(page);
		}
	}

	class NextAction implements ActionListener {
		boolean next = true;
		public NextAction(boolean next) {
			this.next=next;
		}
		@Override
		public void actionPerformed(ActionEvent e) {
			int pageIndex = vascEntry.getVascFrontendController().getVascEntryState().getVascBackendState().getPageIndex();
			if (next) {
				if (vascEntry.getVascFrontendController().getVascFrontendPageInfo().getHasPageNextAction()) {
					vascEntry.getVascFrontendController().getVascFrontendActions().pageAction(pageIndex+1);
				}
			} else {
				if (vascEntry.getVascFrontendController().getVascFrontendPageInfo().getHasPagePreviousAction()) {
					vascEntry.getVascFrontendController().getVascFrontendActions().pageAction(pageIndex-1);
				}
			}
		}
	}
	
	@Override
	public void vascEvent(VascEntry entry, Serializable data) {
		
		for(Component c:result.getComponents()) {
			if (c instanceof JButton) {
				((JButton)c).removeActionListener(this);
			}
		}
		result.removeAll();
		
		
		if (entry.getVascFrontendController().getVascFrontendPageInfo().getHasExtendedPageMode()) {
			JButton prev = new JButton("Previous");
			prev.addActionListener(new NextAction(false));
			prev.setEnabled(entry.getVascFrontendController().getVascFrontendPageInfo().getHasPagePreviousAction());
			result.add(prev);
			List<VascFrontendPageInfoNumber> pages = entry.getVascFrontendController().getVascFrontendPageInfo().getTablePagesExtendedBegin();
			for(VascFrontendPageInfoNumber page:pages) {
				JButton but = new JButton(""+page.getPageNumber());
				but.addActionListener(this);
				if (page.getSelected()!=null && page.getSelected()) {
					but.setEnabled(false);
				}
				result.add(but);
			}
			result.add(new JLabel("..."));
			if (entry.getVascFrontendController().getVascFrontendPageInfo().getHasExtendedPageModeCenter()) {
				pages = entry.getVascFrontendController().getVascFrontendPageInfo().getTablePagesExtendedCenter();
				for(VascFrontendPageInfoNumber page:pages) {
					JButton but = new JButton(""+page.getPageNumber());
					but.addActionListener(this);
					if (page.getSelected()!=null && page.getSelected()) {
						but.setEnabled(false);
					}
					result.add(but);
				}
				result.add(new JLabel("..."));
			}
			pages = entry.getVascFrontendController().getVascFrontendPageInfo().getTablePagesExtendedEnd();
			for(VascFrontendPageInfoNumber page:pages) {
				JButton but = new JButton(""+page.getPageNumber());
				but.addActionListener(this);
				if (page.getSelected()!=null && page.getSelected()) {
					but.setEnabled(false);
				}
				result.add(but);
			}
			JButton next = new JButton("Next");
			next.setEnabled(entry.getVascFrontendController().getVascFrontendPageInfo().getHasPageNextAction());
			next.addActionListener(new NextAction(true));
			result.add(next);
		} else {
			JButton prev = new JButton("Previous");
			prev.addActionListener(new NextAction(false));
			prev.setEnabled(entry.getVascFrontendController().getVascFrontendPageInfo().getHasPagePreviousAction());
			result.add(prev);
			List<VascFrontendPageInfoNumber> pages = entry.getVascFrontendController().getVascFrontendPageInfo().getTablePagesNormal();
			for(VascFrontendPageInfoNumber page:pages) {
				JButton but = new JButton(""+page.getPageNumber());
				but.addActionListener(this);
				if (page.getSelected()!=null && page.getSelected()) {
					but.setEnabled(false);
				}
				result.add(but);
			}
			JButton next = new JButton("Next");
			next.setEnabled(entry.getVascFrontendController().getVascFrontendPageInfo().getHasPageNextAction());
			next.addActionListener(new NextAction(true));
			result.add(next);
		}
	}
	
	@Override
	public VascFrontendEventType[] getEventTypes() {
		VascFrontendEventType[] result = {VascEntryFrontendEventListener.VascFrontendEventType.POST_READ};
		return result;
	}
}
