/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.frontend.swing;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;

import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;

import net.forwardfire.vasc.core.VascEntry;
import net.forwardfire.vasc.core.VascException;
import net.forwardfire.vasc.core.entry.VascEntryFrontendEventListener;

/**
 * SwingPanelFrame to render into frames.
 * 
 *  
 *  
 * @author Willem Cazander
 * @version 1.0 Feb 6, 2012
 */
public class SwingPanelFrame implements SwingPanelIntegration {
	
	Map<JPanel,JFrame> panels = new HashMap<JPanel,JFrame>(5);
	boolean addVascMenu = true;
	
	public SwingPanelFrame() {
		this(true);
	}
	public SwingPanelFrame(boolean addVascMenu) {
		this.addVascMenu=addVascMenu;
	}
	
	@Override
    public void createNewVascView(VascEntry entry) throws VascException {
    	// define redering
    	JPanel panel = initVascView();
    	
    	SwingVascFrontend render = new SwingVascFrontend(panel,this);

    	// render
    	render.initEntry(entry);
    	render.renderView();
    	
    	entry.getVascFrontendController().addVascEntryFrontendEventListener(new VascEntryFrontendEventListener() {
			private static final long serialVersionUID = -6801954395965101748L;
			@Override
			public void vascEvent(VascEntry entry, Serializable data) {
				if (data instanceof Exception) {
					Exception e = (Exception)data;
					e.printStackTrace();
				}
			}
			@Override
			public VascFrontendEventType[] getEventTypes() {
				return new VascFrontendEventType[] {VascFrontendEventType.EXCEPTION};
			}
		});
    	
    	// get data
    	entry.getVascFrontendController().getVascFrontendActions().refreshData();
    	openVascView(panel, entry);
    }
	
	@Override
	public JPanel initVascView() {
		return new JPanel();
	}

	@Override
	public void openVascView(JPanel pane,final VascEntry entry) {
		JFrame frame = new JFrame();
    	frame.setTitle("Vasc Edit - Swing - "+entry.getName());
    	frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
    	frame.setLocationRelativeTo(null);
		
    	JMenuBar menubar = new JMenuBar();
        JMenu vascMenu = new JMenu("Vasc Entries");
        
        if (addVascMenu) {
	        for (final String id:entry.getVascFrontendController().getVascController().getVascEntryController().getVascEntryIds()) {
	        	JMenuItem item = new JMenuItem(id);
	        	item.addActionListener(new ActionListener() {
	        		@Override
					public void actionPerformed(ActionEvent e) {
						try {
							VascEntry ee = entry.getVascFrontendController().getVascController().getVascEntryController().getVascEntryById(id);
							createNewVascView(ee);
						} catch (Exception e1) {
							e1.printStackTrace();
						}
					}
	        		
	        	});
	        	vascMenu.add(item);
	        }
	        menubar.add(vascMenu);
	        frame.setJMenuBar(menubar);
        }
    	
    	frame.add(pane);
    	
    	// view
    	frame.pack();
    	frame.setVisible(true);
    	
    	panels.put(pane,frame);
	}

	@Override
	public void closeVascView(JPanel pane,VascEntry entry) {
		JFrame frame = panels.get(pane);
		if (frame!=null) {
			frame.setVisible(false);
		}
	}
}
