/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.frontend.swing;

import java.io.Serializable;

import javax.swing.JPanel;
import javax.swing.JTabbedPane;

import net.forwardfire.vasc.core.VascEntry;
import net.forwardfire.vasc.core.VascException;
import net.forwardfire.vasc.core.entry.VascEntryFrontendEventListener;

/**
 * SwingPanelTabbed to render into tabs.
 * 
 *  
 *  
 * @author Willem Cazander
 * @version 1.0 Feb 6, 2012
 */
public class SwingPanelTabbed implements SwingPanelIntegration {
	
	JTabbedPane tabbedPane = null;
	
	public SwingPanelTabbed(JTabbedPane tabbedPane) {
		this.tabbedPane=tabbedPane;
	}
	
	@Override
    public void createNewVascView(VascEntry entry) throws VascException {
    	// define redering
    	JPanel panel = initVascView();
    	SwingVascFrontend render = new SwingVascFrontend(panel,this);

    	// render
    	render.initEntry(entry);
    	render.renderView();
    	
    	entry.getVascFrontendController().addVascEntryFrontendEventListener(new VascEntryFrontendEventListener() {
			private static final long serialVersionUID = -6801954395965101748L;
			@Override
			public void vascEvent(VascEntry entry, Serializable data) {
				if (data instanceof Exception) {
					Exception e = (Exception)data;
					e.printStackTrace();
				}
			}
			@Override
			public VascFrontendEventType[] getEventTypes() {
				return new VascFrontendEventType[] {VascFrontendEventType.EXCEPTION};
			}
		});
    	
    	// get data
    	entry.getVascFrontendController().getVascFrontendActions().refreshData();
    	openVascView(panel, entry);
    }
	
	@Override
	public JPanel initVascView() {
		return new JPanel();
	}
	
	@Override
	public void openVascView(JPanel pane,final VascEntry entry) {
		tabbedPane.addTab(entry.getName(), pane);
		tabbedPane.setSelectedIndex(tabbedPane.getTabCount()-1);
	}
	
	@Override
	public void closeVascView(JPanel pane,VascEntry entry) {
		for (int i=0;i<tabbedPane.getTabCount();i++) {
			Object tab = tabbedPane.getComponentAt(i);
			if (pane==tab) {
				tabbedPane.removeTabAt(i);
				break;
			}
		}
	}
}
