/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.frontend.swing;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.Serializable;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SpringLayout;

import net.forwardfire.vasc.backend.VascBackendException;
import net.forwardfire.vasc.core.VascController;
import net.forwardfire.vasc.core.VascEntry;
import net.forwardfire.vasc.core.VascEntryField;
import net.forwardfire.vasc.core.VascEntryLink;
import net.forwardfire.vasc.core.VascEntryLocal;
import net.forwardfire.vasc.core.VascException;
import net.forwardfire.vasc.core.ui.VascColumnValueModelListener;
import net.forwardfire.vasc.core.ui.VascUIComponent;
import net.forwardfire.vasc.core.ui.VascValueModel;

/**
 * SwingVascEditDialog renders vasc entry edit dialog
 * 
 * @author Willem Cazander
 * @version 1.0 Mar 21, 2007
 */
public class SwingVascEditDialog extends JPanel {
	
	private static final long serialVersionUID = 10L;
	private String headerText = null;
	private String headerTextValue = null;
	private Serializable result = null;
	private Serializable bean = null;
	private VascEntry entry = null;
	private SwingVascFrontend swingFrontend = null;
	private SwingPanelIntegration panels = null;
	private JPanel editView = null;
	
	public SwingVascEditDialog(SwingVascFrontend swingFrontend,JPanel editView,VascEntry entry,Serializable bean,String beanValue,SwingPanelIntegration panels) throws VascException {
		super();
		this.headerText = "vasc.dialog.edit.message";
		this.headerTextValue = beanValue;
		this.bean = bean;
		this.entry = entry;
		this.swingFrontend=swingFrontend;
		this.panels=panels;
		this.editView=editView;
		
		setLayout(new 		BorderLayout());
		add(createHeader(),	BorderLayout.NORTH);
		add(createBody(),	BorderLayout.CENTER);
		add(createFooter(),	BorderLayout.SOUTH);
		
		editView.setLayout(new FlowLayout(FlowLayout.LEFT));
		editView.add(this);
	}
	
	protected JPanel createHeader() {
		JPanel header = new JPanel();
		header.setLayout(new BoxLayout(header,BoxLayout.PAGE_AXIS));
		JLabel l = new JLabel();
		l.setText(swingFrontend.i18n(headerText,headerTextValue));
		l.setFont(new Font(null,Font.BOLD, 14));
		//l.setToolTipText(i18n(headerText));
		header.add(l);
		
		JPanel b = new JPanel();
		b.setLayout(new SpringLayout());
		for (final VascEntryLink vle:entry.getVascEntryLinks()) {
			JButton but = new JButton(vle.getName());
			but.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent e) {
					try {
						VascController vc = entry.getVascFrontendController().getVascController();
						VascEntryLocal ee = (VascEntryLocal)vc.getVascEntryController().getVascEntryById(vle.getVascEntryId());
						vc.getVascEntryConfigController().configVascFrontendController(vc, ee);
						
						// Set parameters
						try {
							Serializable selected = bean;
							for (String parameterName:vle.getEntryParameterFieldIdKeys()) {
								String fieldId = vle.getEntryParameterFieldId(parameterName);
								VascEntryField v = entry.getVascEntryFieldById(fieldId);
								if (v==null) {
									//System.out.println("Could not get VascEntryField for fieldID: "+fieldId);
									continue;
								}
								if (v.getVascEntryFieldValue()==null) {
									//System.out.println("Could not get VascEntryFieldValue for fieldID: "+fieldId);
									continue;
								}
								Object selectedValue = v.getVascEntryFieldValue().getValue(v.getBackendName(), selected);
							
								// set data parameter on new vasc entry
								ee.getVascFrontendController().getVascEntryState().getVascBackendState().setDataParameter(parameterName, selectedValue);
								//System.out.println("Setting link parameter: "+parameterName+" with: "+selectedValue);
							}
							
							for (String fieldId:vle.getEntryCreateFieldValueKeys()) {
								String selectedfieldId = vle.getEntryParameterFieldId(fieldId);
								Object selectedValue = selected;
								if (selectedfieldId!=null) { 
									VascEntryField v = ee.getVascEntryFieldById(selectedfieldId);
									selectedValue = v.getVascEntryFieldValue().getValue(v.getBackendName(), selected);
								}
								
								// create listener for new objects
								//entry.getVascFrontendData().addVascEntryFrontendEventListener(new CreateEntryFieldValuesListener2(fieldId,selectedValue));
							}
						} catch (Exception eee) {
							throw new RuntimeException("error: "+eee.getMessage(),eee);
						}
						
						
						
						panels.createNewVascView(ee);
					} catch (Exception eee) {
						eee.printStackTrace();
					}
				}
			});
			b.add(but);
       	}
		SpringUtilities.makeCompactGrid(b, b.getComponentCount(),1,  6,6, 6,6);
		header.add(b);
		
		return header;
	}
	
	protected JPanel createBody() throws VascException {
		JPanel body = new JPanel();
		body.setLayout(new SpringLayout());
		int column = 0;		
        for (VascEntryField c:entry.getVascEntryFields()) {
        	if (c.getEdit()==false) {
        		continue;
        	}
        	//if (c.isEditReadOnly()==true) {
        	
        	for (int i=0;i<c.getVascEntryFieldType().getUIComponentCount(c);i++) {
        		
        		VascUIComponent label = c.getVascEntryFieldType().provideLabelUIComponent(i,c);
        		VascValueModel model = new VascValueModel();
        		model.setValue(swingFrontend.i18n(c.getName()));
        		label.createComponent(entry,c,model,body);
        		
        		VascUIComponent editor = c.getVascEntryFieldType().provideEditorUIComponent(i,c);
        		model = new VascValueModel(c.getVascEntryFieldType().provideEditorVascValueModel(i,c));
        		try {
					model.setValue(c.getVascEntryFieldValue().getValue(c.getBackendName(), bean));
				} catch (VascBackendException e) {
					throw new VascException(e);
				}
        		model.addListener(new VascColumnValueModelListener(c,bean));
        		Object g = editor.createComponent(entry,c,model,body);
        		
        		column++;
        		if (i==0) {
        			entry.getVascFrontendController().addFieldVascUIComponents(c, editor,g);
        		}
        	}
        }
        //JComponent,  rows, cols, initX, initY ,xPad, yPad 
		SpringUtilities.makeCompactGrid(body, column,2,  6,6, 6,6);
        return body;
	}
	public JPanel createFooter() {
		JPanel footer = new JPanel();
		JButton saveButton = new JButton();
        saveButton.setIcon(swingFrontend.getImageIcon("vasc.dialog.save.image"));
        saveButton.setText(swingFrontend.i18n("vasc.dialog.save.name"));
        saveButton.setToolTipText(swingFrontend.i18n("vasc.dialog.save.tooltip"));
		saveButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				//if(entry.getVascFrontendData().getVascFrontendHelper().validateObject(entry, bean)) {
				//	return;
				//}
				result = bean;
		        entry.getVascFrontendController().getVascEntryState().setEntryDataObject(result);
		        
		        if (entry.getVascFrontendController().getVascEntryState().isEditCreate()) {
		        	entry.getVascFrontendController().getVascFrontendActions().persistObject();
		        } else {
		        	entry.getVascFrontendController().getVascFrontendActions().mergeObject();
		        }
				
				panels.closeVascView(editView, SwingVascEditDialog.this.entry);
				//setVisible(false);
			}
		});
		footer.add(saveButton);
		
		JButton cancelButton = new JButton();
        cancelButton.setIcon(swingFrontend.getImageIcon("vasc.dialog.cancel.image"));
        cancelButton.setText(swingFrontend.i18n("vasc.dialog.cancel.name"));
        cancelButton.setToolTipText(swingFrontend.i18n("vasc.dialog.cancel.tooltip"));
		cancelButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				result = null;
				panels.closeVascView(editView, SwingVascEditDialog.this.entry);
				//setVisible(false);
			}
		});
		footer.add(cancelButton);
		return footer;
	}
}
