/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.frontend.swing;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Logger;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.Spring;
import javax.swing.SpringLayout;
import javax.swing.UIManager;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;

import net.forwardfire.vasc.backend.VascBackendException;
import net.forwardfire.vasc.core.VascEntryField;
import net.forwardfire.vasc.core.VascEntryListOption;
import net.forwardfire.vasc.core.actions.GlobalVascAction;
import net.forwardfire.vasc.core.actions.RowVascAction;
import net.forwardfire.vasc.core.entry.VascEntryExport;
import net.forwardfire.vasc.core.entry.VascEntryExportWriter;
import net.forwardfire.vasc.core.ui.VascOptionValueModelListener;
import net.forwardfire.vasc.core.ui.VascUIComponent;
import net.forwardfire.vasc.core.ui.VascValueModel;
import net.forwardfire.vasc.core.VascException;
import net.forwardfire.vasc.frontend.AbstractVascFrontend;
import net.forwardfire.vasc.frontend.VascFrontendController;
import net.forwardfire.vasc.frontend.VascFrontendException;
import net.forwardfire.vasc.frontend.swing.ui.SwingBoolean;
import net.forwardfire.vasc.frontend.swing.ui.SwingButton;
import net.forwardfire.vasc.frontend.swing.ui.SwingColorChooser;
import net.forwardfire.vasc.frontend.swing.ui.SwingDate;
import net.forwardfire.vasc.frontend.swing.ui.SwingLabel;
import net.forwardfire.vasc.frontend.swing.ui.SwingList;
import net.forwardfire.vasc.frontend.swing.ui.SwingText;
import net.forwardfire.vasc.frontend.swing.ui.SwingTextArea;

/**
 * SwingVascFrontend renders vasc entries in a swing panel.
 * 
 * @author Willem Cazander
 * @version 1.0 Mar 21, 2007
 */
public class SwingVascFrontend extends AbstractVascFrontend {
	
	private Logger logger = null;
	private JComponent parent = null;
	private boolean initOnce = true;
	private Map<RowVascAction,JButton> rowActions = null;
	private SwingPanelIntegration panels = null;

	public SwingVascFrontend(JComponent parent,SwingPanelIntegration panels) {
		logger = Logger.getLogger(SwingVascFrontend.class.getName());
		this.parent=parent;
		this.panels=panels;
		rowActions = new HashMap<RowVascAction,JButton>(5);
	}

	/**
	 * @see net.forwardfire.vasc.frontend.VascFrontend#getFrontendType()
	 */
	@Override
	public String getFrontendType() {
		return "swing";
	}
	
	/**
	 * Add swing implmented ui components
	 */
	@Override
	protected void addUiComponents() {
		VascFrontendController vfd = getVascEntry().getVascFrontendController();

		// required UI components
		vfd.putVascUIComponent(VascUIComponent.VASC_LABEL,SwingLabel.class.getName());
		vfd.putVascUIComponent(VascUIComponent.VASC_TEXT, SwingText.class.getName());
		vfd.putVascUIComponent(VascUIComponent.VASC_LIST, SwingList.class.getName());
		vfd.putVascUIComponent(VascUIComponent.VASC_BUTTON, SwingButton.class.getName());
		
		// optional UI components
		vfd.putVascUIComponent(VascUIComponent.VASC_BOOLEAN	, SwingBoolean.class.getName());
		vfd.putVascUIComponent(VascUIComponent.VASC_DATE	, SwingDate.class.getName());
		vfd.putVascUIComponent(VascUIComponent.VASC_TEXTAREA, SwingTextArea.class.getName());
		vfd.putVascUIComponent(VascUIComponent.VASC_COLOR,    SwingColorChooser.class.getName());
	}

	public ImageIcon getImageIcon(String imageResource) {
		/// TODO hack beter
		String key = entry.getVascFrontendController().getVascEntryResourceResolver().getTextValue(imageResource);
		//logger.fine("KEY======================="+key);
		
		if (key.startsWith("vasc.entry")) {
			return null;
		}
		
		if (key.indexOf("META-INF")>0 | key.indexOf("resource")>0) {
			return null;
			//return SwingImageHelper.getImageIcon(key);
		} else {
			return null;
		}
	}
	
	

	/**
	 * @see net.forwardfire.vasc.core.VascViewRenderer#renderEdit(net.forwardfire.vasc.core.VascEntry, java.lang.Object)
	 */
	@Override
	public void renderEdit() throws VascFrontendException {
		logger.fine("Rending Edit View");
		try {
			Serializable rowBean = entry.getVascFrontendController().getVascEntryState().getEntryDataObject();
			String beanValue = rowBean.toString();
			if (entry.getDisplayNameFieldId()!=null) {
				VascEntryField v = entry.getVascEntryFieldById(entry.getDisplayNameFieldId());
				
				Object vv = v.getVascEntryFieldValue().getValue(v.getBackendName(), rowBean);
				if (vv==null) {
					beanValue="";
				} else {
					beanValue=""+vv;
				}
				if (beanValue.length()>30) {
					beanValue=beanValue.substring(0, 30);
				}
			}
			
			JPanel editPanel = panels.initVascView();
			SwingVascEditDialog dialog = new SwingVascEditDialog(this,editPanel,entry,rowBean,beanValue,panels);
			
			entry.getVascFrontendController().getVascFrontendHelper().editReadOnlyUIComponents(entry);
			panels.openVascView(editPanel,entry);
		
		} catch (VascBackendException ve) {
			throw new VascFrontendException(ve);
		} catch (VascException ve) {
			throw new VascFrontendException(ve);
		}
	}
	
	@Override
	public void renderDelete() throws VascFrontendException {
		Serializable rowBean = entry.getVascFrontendController().getVascEntryState().getEntryDataObject();
		String beanValue = rowBean.toString();
		
		try {
			VascEntryField v = entry.getVascEntryFieldById(entry.getDisplayNameFieldId());
			beanValue = ""+v.getVascEntryFieldValue().getValue(v.getBackendName(), rowBean);
			if (beanValue.length()>30) {
				beanValue=beanValue.substring(0, 30);
			}
		} catch (VascBackendException ve) {
			throw new VascFrontendException(ve);
		}

		int response = JOptionPane.showOptionDialog(
				parent                       // Center in window.
				, i18n("vasc.dialog.delete.message",beanValue)    // Message
				, i18n("vasc.dialog.delete.title")                // Title in titlebar
				, JOptionPane.YES_NO_OPTION  // Option type
				, JOptionPane.PLAIN_MESSAGE  // messageType
				, null                       // Icon (none)
				, null                    	// Button text as above.
				, null 					   // Default button's label
			);
		if (response==JOptionPane.YES_OPTION) {
			entry.getVascFrontendController().getVascFrontendActions().deleteObject();
		}
	}
	
	/**
	 * @see net.forwardfire.vasc.core.VascViewRenderer#renderExport(net.forwardfire.vasc.core.VascEntry, net.forwardfire.vasc.core.VascDataExporter)
	 */
	@Override
	public void renderExport(VascEntryExport exporter) throws VascFrontendException {
		
		String fileName = null;
		JFileChooser c = new JFileChooser();
		int rVal = c.showSaveDialog(null);
		if (rVal == JFileChooser.APPROVE_OPTION) {
			fileName = c.getSelectedFile().getAbsolutePath();
		}
		if (rVal == JFileChooser.CANCEL_OPTION) {
			return;
		}
		
		logger.fine("Export fileName: "+fileName);
		if (fileName == null) {
			return;
		}
		OutputStream out = null;
		try {
			VascEntryExportWriter veew = exporter.createExportWriter();
			veew.doInit(exporter, entry);
			out = new FileOutputStream(fileName);
			veew.doExport(out);
		} catch (VascException e) {
			throw new VascFrontendException(e);
		} catch (IOException e) {
			throw new VascFrontendException(e);
		} finally {
			if (out!=null) {
				try {
					out.close();
				} catch (IOException e) {
				}
			}
		}
	}
	
	/**
	 * @see net.forwardfire.vasc.core.VascViewRenderer#renderView(net.forwardfire.vasc.core.VascEntry)
	 */
	@Override
	public void renderView() throws VascFrontendException {

		if (initOnce==false) {
			return;
		}
		
		JPanel topPanel = new JPanel();
		topPanel.setLayout(new BorderLayout());
		//topPanel.setBackground(Color.PINK);
		
		JPanel n = new JPanel();
		topPanel.add(n,BorderLayout.NORTH);
		renderHeader(n);
		
		JPanel c = new JPanel();
		c.setLayout(new BorderLayout());
		topPanel.add(c,BorderLayout.CENTER);
		renderBody(c);
		
		JPanel f = new JPanel();
		//f.setBackground(Color.BLUE);
		topPanel.add(f,BorderLayout.SOUTH);
		renderFooter(f);
		
		//parent.setBackground(Color.CYAN);
		parent.setLayout(new BorderLayout());
		parent.add(topPanel);
		
		initOnce = false;
	}

	private void renderHeader(JComponent parent2) {
				
		JPanel header = new JPanel();
		header.setLayout(new BorderLayout());
		//header.setBackground(Color.WHITE);
		header.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
		
		if(entry.getListImage()!=null) {
			JLabel l = new JLabel();
			// TODO: hack images working
			//l.setIcon(new ImageIcon(Toolkit.getDefaultToolkit().createImage(getClass().getResource(entry.getHeaderImage())).getScaledInstance(32, 32, Image.SCALE_SMOOTH)));
			if (entry.getListDescription()!=null) {
				l.setToolTipText(i18n(entry.getListDescription()));
			}
			header.add(l,BorderLayout.WEST);
		}
		
		if(entry.getName()!=null) {
			JLabel l = new JLabel(i18n(entry.getName()));
			//l.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
			l.setFont(new Font(null,Font.BOLD, 18));
			header.add(l,BorderLayout.NORTH);
		}
		if (entry.getListDescription()!=null) {
			JLabel l = new JLabel(i18n(entry.getListDescription()));
			//l.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
			header.add(l,BorderLayout.CENTER);
		}

		//	create options
		JPanel optionPanel = new JPanel();
		//optionPanel.setBackground(Color.GREEN);
		
		JPanel body = optionPanel; //new JPanel();
		body.setLayout(new SpringLayout());
		int column = 0;
		try {
			for(VascEntryListOption c:entry.getVascEntryListOptions()) {
				for (int i=0;i<c.getVascEntryFieldType().getUIComponentCount(c);i++) {
					
					VascUIComponent label = c.getVascEntryFieldType().provideLabelUIComponent(i,c);
					VascValueModel model = new VascValueModel();
					model.setValue(i18n(c.getName()));
					label.createComponent(entry,c,model,body);
					
					VascUIComponent editor = c.getVascEntryFieldType().provideEditorUIComponent(i,c);
					model = new VascValueModel(c.getVascEntryFieldType().provideEditorVascValueModel(i,c));
					model.addListener(new VascOptionValueModelListener(c));
					model.setValue(c.getDefaultValue());
					Object g = editor.createComponent(entry,c,model,body);
					
					column++;
					if (i==0) {
						entry.getVascFrontendController().addFieldVascUIComponents(c, editor,g);
					}
					body.add(new JLabel()); // fill :(
				}
			}
			entry.getVascFrontendController().getVascFrontendHelper().headerOptionsCreatedFillData(entry);
			
			// add search
			column++;
			JLabel searchLabel = new JLabel("Search: ");
			body.add(searchLabel);
			final JTextField searchField = new JTextField();
			body.add(searchField);
			searchField.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent arg0) {
					String searchText = searchField.getText();
					entry.getVascFrontendController().getVascFrontendActions().searchAction(searchText);
				}
			});
			JButton searchButton = new JButton("Search");
			searchButton.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent arg0) {
					String searchText = searchField.getText();
					entry.getVascFrontendController().getVascFrontendActions().searchAction(searchText);
				}
			});
			body.add(searchButton);
		
		} catch (VascException ve) {
			throw new VascFrontendException(ve);
		}	
		SpringUtilities.makeCompactGrid(body, column,3,  6,6, 6,6);
		
		// create pager bar
		JPanel bottomPanel = new JPanel();
		bottomPanel.setLayout(new BoxLayout(bottomPanel, BoxLayout.PAGE_AXIS));
		bottomPanel.add(new SwingPagerPanel(entry));
		bottomPanel.add(new SwingActionPanel(entry));
		
		//top.add(header,BorderLayout.NORTH);
		parent2.setLayout(new BorderLayout());
		parent2.add(header,BorderLayout.NORTH);
		parent2.add(optionPanel,BorderLayout.CENTER);
		parent2.add(bottomPanel,BorderLayout.SOUTH);
	}
	
	private void renderBody(JComponent parent2) {
		
		SwingVascTableModel model = new SwingVascTableModel(entry);
		//TODO: entry.getVascEntryController().addEventListener(model);
		
		JTable table = new JTable(); 
		// this regs the listeners for the sorting
		//tableSorter.setTableHeader(table.getTableHeader());
		//tableSorter.setTableModel(model);
		
		table.setModel(model);
		table.getTableHeader().setReorderingAllowed(false);
		table.getTableHeader().addMouseListener(new ColumnListener(table));
		
		// remove auto columns :(
		int cols = table.getColumnModel().getColumnCount();
		for (int i=0;i<cols;i++) {
			TableColumn c = table.getColumnModel().getColumn(0); // idd, just remove index 0 every time
			table.removeColumn(c);
		}
		table.revalidate();
		
		TableCellRenderer renderer = new JComponentTableHeaderCellRenderer();
		int counter=0;
		for(VascEntryField c:entry.getVascEntryFields()) {
			if (c.getList()==false) {
				continue;
			}
			TableColumn t = new TableColumn();
			if (c.getSizeList()!=null) {
				t.setPreferredWidth(c.getSizeList());
			} else {
				t.setPreferredWidth(200);
			}
			t.setHeaderValue(c);
			t.setHeaderRenderer(renderer);
			t.setModelIndex(counter);
			table.addColumn(t);
			counter++;
		}
		table.getSelectionModel().addListSelectionListener(new EntrySectionListener(table));
		table.getSelectionModel().setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		table.addMouseListener(new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent e) {
				if (e.getClickCount() == 2) {				
					Object o = entry.getVascFrontendController().getVascEntryState().getEntryDataObject();
					if (o==null) {
						return;
					}
					entry.getVascFrontendController().getVascFrontend().renderEdit();
				}
			}
		});
		JScrollPane scrollPane = new JScrollPane(table);
		parent2.add(scrollPane);
	}
	class ColumnListener extends MouseAdapter {
		protected JTable table;
		private String sortField = null;
		private boolean sortOrder = true;
		
		public ColumnListener(JTable t) {
			table = t;
		}
		
		@Override
		public void mouseClicked(MouseEvent e) {
			TableColumnModel colModel = table.getColumnModel();
			int columnModelIndex = colModel.getColumnIndexAtX(e.getX());
			TableColumn tc = colModel.getColumn(columnModelIndex);
			VascEntryField field = (VascEntryField)tc.getHeaderValue();
			entry.getVascFrontendController().getVascFrontendActions().sortAction(field);
			sortOrder = entry.getVascFrontendController().getVascEntryState().getVascBackendState().isSortAscending();
			sortField = field.getId();
			entry.getVascFrontendController().getVascFrontend().renderView();
			table.repaint();
		}
	}
	class EntrySectionListener implements ListSelectionListener {
		JTable table;
		EntrySectionListener(JTable table) {
			this.table = table;
		}
		
		@Override
		public void valueChanged(ListSelectionEvent e) {
			if (e.getValueIsAdjusting()) {
				return;
			}
			
			if (table.getSelectedRowCount()==0) {
				for (RowVascAction action:rowActions.keySet()) {
					JButton but = rowActions.get(action);
					if (action.getId().contains("add")==false) {
						but.setEnabled(false);
					}
				}
			} else if (table.getSelectedRowCount()==1) {
				for (JButton but:rowActions.values()) {
					but.setEnabled(true);
				}
			} else {
				for (RowVascAction action:rowActions.keySet()) {
					JButton but = rowActions.get(action);
					if (action.isMultiRowAction()) {
						but.setEnabled(true);	
					} else {
						if (action.getId().contains("add")==false) {
							but.setEnabled(false);
						}
					}
				}
			}
			
			int rowIndex = table.getSelectedRow();
			
			if (rowIndex!=-1) {
				// temp; gets index by sorter
				//rowIndex = tableSorter.modelIndex(rowIndex);
				Serializable data = entry.getVascFrontendController().getVascEntryState().getEntryDataList().get(rowIndex);
				entry.getVascFrontendController().getVascEntryState().setEntryDataObject(data);
			} else {
				entry.getVascFrontendController().getVascEntryState().setEntryDataObject(null);
			}
		}
	}
	class JComponentTableHeaderCellRenderer extends DefaultTableCellRenderer {
		private static final long serialVersionUID = 10L;
		@Override
		public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected,boolean hasFocus, int row, int column) {
			VascEntryField c = (VascEntryField)value;
			setText(i18n(c.getName()));
			setToolTipText(i18n(c.getDescription()));
			
			if(c.getImage()!=null) {
				setIcon(getImageIcon(c.getImage()));
			} else {
				setIcon(null);
			}
			
			if (entry != null) {
				JTableHeader header = table.getTableHeader();
				if (header != null) {
					setForeground(header.getForeground());
					setBackground(header.getBackground());
					setFont(header.getFont());
				}
			}
			setBorder(UIManager.getBorder("TableHeader.cellBorder"));
			return this;
		}
	}
	
	
	private void renderFooter(JComponent parent2) {
		
		// create pager bar
		JPanel bottomPanel = new JPanel();
		bottomPanel.setLayout(new BorderLayout());
		bottomPanel.add(new SwingActionPanel(entry),BorderLayout.NORTH);
		bottomPanel.add(new SwingPagerPanel(entry),BorderLayout.CENTER);
		
		logger.finest("Creating footer");
		JPanel panel = new JPanel();
		for(RowVascAction action:entry.getRowActions()) {
			JButton but = new JButton();
			but.setText(i18n(action.getName()));
			but.setToolTipText(i18n(action.getDescription()));
			but.setIcon(getImageIcon(action.getImage()));
			but.addActionListener(new RowActionListener(action));
			if (action.getId().contains("add")==false) {
				but.setEnabled(false);	
			}
			rowActions.put(action, but);
			panel.add(but);
		}
		bottomPanel.add(panel,BorderLayout.SOUTH);
		
		//parent2.setLayout(new BorderLayout());
		//parent2.add(bottomPanel,BorderLayout.WEST);
		parent2.add(bottomPanel);
	}
	
	
	class RowActionListener implements ActionListener {
		
		private RowVascAction action = null;
		
		public RowActionListener(RowVascAction action) {
			this.action=action;
		}
		
		@Override
		public void actionPerformed(ActionEvent event) {
			logger.fine("Row Action");
			try {
				action.doRowAction(entry, entry.getVascFrontendController().getVascEntryState().getEntryDataObject());
			} catch (VascException e) {
				throw new VascFrontendException(e);
			}
		}
	}

	class GlobalActionListener implements ActionListener {
		
		private GlobalVascAction action = null;
		
		public GlobalActionListener(GlobalVascAction action) {
			this.action=action;
		}
		
		@Override
		public void actionPerformed(ActionEvent event) {
			logger.fine("Global Action");
			try {
				action.doGlobalAction(entry);
			} catch (VascException e) {
				throw new VascFrontendException(e);
			}
		}
	}
	
}

/**
 * A 1.4 file that provides utility methods for creating form- or grid-style
 * layouts with SpringLayout. These utilities are used by several programs, such
 * as SpringBox and SpringCompactGrid.
 */

class SpringUtilities {

  /**
   * Aligns the first <code>rows</code>*<code>cols</code> components of
   * <code>parent</code> in a grid. Each component is as big as the maximum
   * preferred width and height of the components. The parent is made just big
   * enough to fit them all.
   * 
   * @param rows
   *            number of rows
   * @param cols
   *            number of columns
   * @param initialX
   *            x location to start the grid at
   * @param initialY
   *            y location to start the grid at
   * @param xPad
   *            x padding between cells
   * @param yPad
   *            y padding between cells
   */
  public static void makeGrid(Container parent, int rows, int cols,int initialX, int initialY, int xPad, int yPad) {
    SpringLayout layout;
    try {
      layout = (SpringLayout) parent.getLayout();
    } catch (ClassCastException exc) {
      throw new IllegalArgumentException("parent container has not StringLayout layoutmanager.");
    }

    Spring xPadSpring = Spring.constant(xPad);
    Spring yPadSpring = Spring.constant(yPad);
    Spring initialXSpring = Spring.constant(initialX);
    Spring initialYSpring = Spring.constant(initialY);
    int max = rows * cols;

    //Calculate Springs that are the max of the width/height so that all
    //cells have the same size.
    Spring maxWidthSpring = layout.getConstraints(parent.getComponent(0))
        .getWidth();
    Spring maxHeightSpring = layout.getConstraints(parent.getComponent(0))
        .getWidth();
    for (int i = 1; i < max; i++) {
      SpringLayout.Constraints cons = layout.getConstraints(parent
          .getComponent(i));

      maxWidthSpring = Spring.max(maxWidthSpring, cons.getWidth());
      maxHeightSpring = Spring.max(maxHeightSpring, cons.getHeight());
    }

    //Apply the new width/height Spring. This forces all the
    //components to have the same size.
    for (int i = 0; i < max; i++) {
      SpringLayout.Constraints cons = layout.getConstraints(parent
          .getComponent(i));

      cons.setWidth(maxWidthSpring);
      cons.setHeight(maxHeightSpring);
    }

    //Then adjust the x/y constraints of all the cells so that they
    //are aligned in a grid.
    SpringLayout.Constraints lastCons = null;
    SpringLayout.Constraints lastRowCons = null;
    for (int i = 0; i < max; i++) {
      SpringLayout.Constraints cons = layout.getConstraints(parent
          .getComponent(i));
      if (i % cols == 0) { //start of new row
        lastRowCons = lastCons;
        cons.setX(initialXSpring);
      } else { //x position depends on previous component
        cons.setX(Spring.sum(lastCons.getConstraint(SpringLayout.EAST),
            xPadSpring));
      }

      if (i / cols == 0) { //first row
        cons.setY(initialYSpring);
      } else { //y position depends on previous row
        cons.setY(Spring.sum(lastRowCons
            .getConstraint(SpringLayout.SOUTH), yPadSpring));
      }
      lastCons = cons;
    }

    //Set the parent's size.
    SpringLayout.Constraints pCons = layout.getConstraints(parent);
    pCons.setConstraint(SpringLayout.SOUTH, Spring.sum(Spring
        .constant(yPad), lastCons.getConstraint(SpringLayout.SOUTH)));
    pCons.setConstraint(SpringLayout.EAST, Spring.sum(
        Spring.constant(xPad), lastCons
            .getConstraint(SpringLayout.EAST)));
  }

  /* Used by makeCompactGrid. */
  private static SpringLayout.Constraints getConstraintsForCell(int row,
      int col, Container parent, int cols) {
    SpringLayout layout = (SpringLayout) parent.getLayout();
    Component c = parent.getComponent(row * cols + col);
    return layout.getConstraints(c);
  }

  /**
   * Aligns the first <code>rows</code>*<code>cols</code> components of
   * <code>parent</code> in a grid. Each component in a column is as wide as
   * the maximum preferred width of the components in that column; height is
   * similarly determined for each row. The parent is made just big enough to
   * fit them all.
   * 
   * @param rows
   *            number of rows
   * @param cols
   *            number of columns
   * @param initialX
   *            x location to start the grid at
   * @param initialY
   *            y location to start the grid at
   * @param xPad
   *            x padding between cells
   * @param yPad
   *            y padding between cells
   */
  public static void makeCompactGrid(Container parent, int rows, int cols,int initialX, int initialY, int xPad, int yPad) {
    SpringLayout layout;
    try {
      layout = (SpringLayout) parent.getLayout();
    } catch (ClassCastException exc) {
    	 throw new IllegalArgumentException("parent container has not StringLayout layoutmanager.");
    }

    //Align all cells in each column and make them the same width.
    Spring x = Spring.constant(initialX);
    for (int c = 0; c < cols; c++) {
      Spring width = Spring.constant(0);
      for (int r = 0; r < rows; r++) {
        width = Spring.max(width, getConstraintsForCell(r, c, parent,
            cols).getWidth());
      }
      for (int r = 0; r < rows; r++) {
        SpringLayout.Constraints constraints = getConstraintsForCell(r,
            c, parent, cols);
        constraints.setX(x);
        constraints.setWidth(width);
      }
      x = Spring.sum(x, Spring.sum(width, Spring.constant(xPad)));
    }

    //Align all cells in each row and make them the same height.
    Spring y = Spring.constant(initialY);
    for (int r = 0; r < rows; r++) {
      Spring height = Spring.constant(0);
      for (int c = 0; c < cols; c++) {
        height = Spring.max(height, getConstraintsForCell(r, c, parent,
            cols).getHeight());
      }
      for (int c = 0; c < cols; c++) {
        SpringLayout.Constraints constraints = getConstraintsForCell(r,
            c, parent, cols);
        constraints.setY(y);
        constraints.setHeight(height);
      }
      y = Spring.sum(y, Spring.sum(height, Spring.constant(yPad)));
    }

    //Set the parent's size.
    SpringLayout.Constraints pCons = layout.getConstraints(parent);
    pCons.setConstraint(SpringLayout.SOUTH, y);
    pCons.setConstraint(SpringLayout.EAST, x);
  }
}
           
