/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.frontend.swing;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import javax.swing.table.AbstractTableModel;

import net.forwardfire.vasc.core.VascEntry;
import net.forwardfire.vasc.core.VascEntryField;
import net.forwardfire.vasc.core.entry.VascEntryFrontendEventListener;

/**
 * SwingVascTableModel table model for gui component.
 * 
 * @author Willem Cazander
 * @version 1.0 Mar 21, 2007
 */
public class SwingVascTableModel extends AbstractTableModel implements VascEntryFrontendEventListener {
	
	private static final long serialVersionUID = -7046372314261802293L;
	private VascEntry entry = null;
	
	public SwingVascTableModel(VascEntry entry) {
		this.entry=entry;
		entry.getVascFrontendController().addVascEntryFrontendEventListener(this);
	}
	
	@Override
	public void vascEvent(VascEntry entry,Serializable o) {
		fireTableDataChanged();
	}
	
	/**
	 * @see javax.swing.entry.entryModel#getColumnCount()
	 */
	@Override
	public int getColumnCount() {
		int result = 0;		
		for(VascEntryField c:entry.getVascEntryFields()) {
			if (c.getList()!=null && c.getList()==false) {
				continue;
			}
			result++;
		}
		return result;
	}

	/**
	 * @see javax.swing.entry.entryModel#getRowCount()
	 */
	@Override
	public int getRowCount() {
		if (entry.getVascFrontendController().getVascEntryState().getEntryDataList()==null) {
			return 0;
		}
		return entry.getVascFrontendController().getVascEntryState().getEntryDataList().size();
	}

	/**
	 * @see javax.swing.entry.entryModel#getValueAt(int, int)
	 */
	@Override
	public Object getValueAt(int rowIndex, int columnIndex) {
		Serializable bean = entry.getVascFrontendController().getVascEntryState().getEntryDataList().get(rowIndex);
		
		// TODO: this is slowing....
		List<VascEntryField> list = new ArrayList<VascEntryField>();
		for(VascEntryField c:entry.getVascEntryFields()) {
			if (c.getList()!=null && c.getList()==false) {
				continue;
				
			}
			list.add(c);
		}	
		
		VascEntryField vtc = list.get(columnIndex);
		try {
			//if (vtc.getVascColumnRenderer()!=null) {
//				return vtc.getVascColumnRenderer().rendererColumn(vtc,bean);
			//} else {
				return ""+vtc.getVascEntryFieldValue().getValue(vtc.getBackendName(),bean);
		//	}
		} catch (Exception e) {
			return "Error";
		}
	}
	
	@Override
	public VascFrontendEventType[] getEventTypes() {
		VascFrontendEventType[] result = {VascFrontendEventType.POST_UPDATE,VascFrontendEventType.POST_READ,VascFrontendEventType.POST_DELETE};
		return result;
	}
}
