/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.frontend.swing.ui;

import java.awt.Color;

import javax.swing.JComponent;
import javax.swing.JLabel;

import net.forwardfire.vasc.core.VascEntry;
import net.forwardfire.vasc.core.VascEntryField;
import net.forwardfire.vasc.core.ui.VascUIComponent;
import net.forwardfire.vasc.core.ui.VascValueModel;
import net.forwardfire.vasc.core.VascException;




/**
 * 
 * @author Willem Cazander
 * @version 1.0 Sep 21, 2008
 */
public class SwingLabel implements VascUIComponent {
	
	private JLabel label = null;
	private Color orgBackgroundColor = null;
	
	@Override
	public Object createComponent(VascEntry table,VascEntryField entryField,VascValueModel model,Object gui) throws VascException {
		label = new JLabel();
		label.setHorizontalAlignment(JLabel.TRAILING);
		orgBackgroundColor = label.getBackground();
		label.setText(""+model.getValue());
		((JComponent)gui).add(label);
		return label;
	}

	/**
	 * @see net.forwardfire.vasc.core.ui.VascUIComponent#getErrorText()
	 */
	@Override
	public String getErrorText() {
		if (label==null) {
			return null;
		}
		return label.getToolTipText();
	}

	/**
	 * @see net.forwardfire.vasc.core.ui.VascUIComponent#setErrorText(java.lang.String)
	 */
	@Override
	public void setErrorText(String text) {
		if (label==null) {
			return;
		}
		label.setToolTipText(text);
		if (text==null) {
			label.setBackground(orgBackgroundColor);
		} else {
			label.setBackground(Color.RED);
		}
	}
	
	/**
	 * @see net.forwardfire.vasc.core.ui.VascUIComponent#isDisabled()
	 */
	@Override
	public boolean isDisabled() {
		return !label.isEnabled();
	}

	/**
	 * @see net.forwardfire.vasc.core.ui.VascUIComponent#setDisabled(boolean)
	 */
	@Override
	public void setDisabled(boolean disabled) {
		label.setEnabled(!disabled);
	}
	
	/**
	 * @see net.forwardfire.vasc.core.ui.VascUIComponent#isRendered()
	 */
	@Override
	public boolean isRendered() {
		return label.isVisible();
	}

	/**
	 * @see net.forwardfire.vasc.core.ui.VascUIComponent#setRendered(boolean)
	 */
	@Override
	public void setRendered(boolean rendered) {
		label.setVisible(rendered);
	}
}