/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.frontend.swing.ui;

import java.awt.Color;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.ListCellRenderer;

import net.forwardfire.vasc.backend.select.VascBackendSelectItem;
import net.forwardfire.vasc.core.VascEntry;
import net.forwardfire.vasc.core.VascEntryField;
import net.forwardfire.vasc.core.ui.VascSelectItemModel;
import net.forwardfire.vasc.core.ui.VascUIComponent;
import net.forwardfire.vasc.core.ui.VascValueModel;
import net.forwardfire.vasc.core.VascException;
import net.forwardfire.vasc.frontend.VascFrontendException;




/**
 * 
 * @author Willem Cazander
 * @version 1.0 Aug 12, 2007
 */
public class SwingList implements VascUIComponent {
	
	private JComboBox comboBox = null;
	private Color orgBackgroundColor = null;
	
	@Override
	public Object createComponent(final VascEntry entry,VascEntryField entryField,final VascValueModel model,Object gui) throws VascException {				
		VascSelectItemModel items = (VascSelectItemModel)entryField.getVascEntryFieldType().getDataObject();
		if (items==null) {
			comboBox = new JComboBox(); 
		} else {
			comboBox = new JComboBox(items.getVascSelectItems(entry).toArray());
		}
		orgBackgroundColor = comboBox.getBackground();
		((JComponent)gui).add(comboBox);	
		comboBox.setRenderer(new MyCellRenderer());
		comboBox.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				VascBackendSelectItem i = (VascBackendSelectItem)((JComboBox)e.getSource()).getSelectedItem();
				try {
					model.setValue(i.getValue());
				} catch (VascException ee) {
					throw new VascFrontendException(ee);
				}
			}
		});
		
		
		// set default !!
		if (model.getValue()!=null) {
			for (int i=0;i<comboBox.getModel().getSize();i++) {
				Object o = comboBox.getModel().getElementAt(i);
				VascBackendSelectItem i2 = (VascBackendSelectItem)o;
				if ( model.getValue().equals(i2.getValue()) ) {
					comboBox.setSelectedItem(i2);
					return comboBox;
				}
			}
		}
		if (comboBox.getModel().getSize()>0) {
			// else select the top one.
			comboBox.setSelectedIndex(0);
		}
		return comboBox;
	}
	
	/**
	 * @see net.forwardfire.vasc.core.ui.VascUIComponent#getErrorText()
	 */
	@Override
	public String getErrorText() {
		if (comboBox==null) {
			return null;
		}
		return comboBox.getToolTipText();
	}

	/**
	 * @see net.forwardfire.vasc.core.ui.VascUIComponent#setErrorText(java.lang.String)
	 */
	@Override
	public void setErrorText(String text) {
		if (comboBox==null) {
			return;
		}
		comboBox.setToolTipText(text);
		if (text==null) {
			comboBox.setBackground(orgBackgroundColor);
		} else {
			comboBox.setBackground(Color.RED);
		}
	}
	
	/**
	 * @see net.forwardfire.vasc.core.ui.VascUIComponent#isDisabled()
	 */
	@Override
	public boolean isDisabled() {
		return !comboBox.isEnabled();
	}

	/**
	 * @see net.forwardfire.vasc.core.ui.VascUIComponent#setDisabled(boolean)
	 */
	@Override
	public void setDisabled(boolean disabled) {
		comboBox.setEnabled(!disabled);
	}
	
	/**
	 * @see net.forwardfire.vasc.core.ui.VascUIComponent#isRendered()
	 */
	@Override
	public boolean isRendered() {
		return comboBox.isVisible();
	}

	/**
	 * @see net.forwardfire.vasc.core.ui.VascUIComponent#setRendered(boolean)
	 */
	@Override
	public void setRendered(boolean rendered) {
		comboBox.setVisible(rendered);
	}
}

class MyCellRenderer extends JLabel implements ListCellRenderer {

	private static final long serialVersionUID = 10L;

	public MyCellRenderer() {
		setOpaque(true);
	}
	
	@Override
	public Component getListCellRendererComponent(JList list,Object value,int index,boolean isSelected,boolean cellHasFocus) {
		VascBackendSelectItem i = (VascBackendSelectItem)value;
		if (i!=null) {
			setText(i.getLabel());
		} else {
			setText("Error");
		}
		return this;
	}
}