/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.frontend.swing.ui;

import java.awt.Color;

import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.ScrollPaneConstants;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import net.forwardfire.vasc.core.VascEntry;
import net.forwardfire.vasc.core.VascEntryField;
import net.forwardfire.vasc.core.ui.VascUIComponent;
import net.forwardfire.vasc.core.ui.VascValueModel;
import net.forwardfire.vasc.core.VascException;

/**
 * 
 * @author Willem Cazander
 * @version 1.0 Sep 21, 2008
 */
public class SwingTextArea implements VascUIComponent {
	
	private JTextArea textArea = null;
	private Color orgBackgroundColor = null;
	
	@Override
	public Object createComponent(VascEntry entry,VascEntryField entryField,VascValueModel model,Object gui) throws VascException {
		int colsNum = 40;
		int rowsNum = 8;
		String cols = entryField.getVascEntryFieldType().getProperty("editor.columns");
		if (cols!=null && cols.isEmpty()==false) {
			colsNum = new Integer(cols);
		}
		String rows = entryField.getVascEntryFieldType().getProperty("editor.rows");
		if (rows!=null && rows.isEmpty()==false) {
			rowsNum = new Integer(rows);
		}
		textArea = new JTextArea(rowsNum,colsNum);
		textArea.setBorder(BorderFactory.createEtchedBorder());
		
		orgBackgroundColor = textArea.getBackground();
		textArea.setText(""+model.getValue());
		textArea.getDocument().addDocumentListener(new TextAreaListener(model,entry,this));
		
		JScrollPane scrollPane = new JScrollPane(textArea);
		scrollPane.setBorder(BorderFactory.createEmptyBorder());
		scrollPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		scrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
		scrollPane.getVerticalScrollBar().setUnitIncrement(10);
		scrollPane.getHorizontalScrollBar().setUnitIncrement(10);
		scrollPane.setSize(colsNum*10, rowsNum*20);
		
		((JComponent)gui).add(scrollPane);
		return scrollPane; /// TODO: mmm check return this object type 
	}

	/**
	 * @see net.forwardfire.vasc.core.ui.VascUIComponent#getErrorText()
	 */
	@Override
	public String getErrorText() {
		if (textArea==null) {
			return null;
		}
		return textArea.getToolTipText();
	}

	/**
	 * @see net.forwardfire.vasc.core.ui.VascUIComponent#setErrorText(java.lang.String)
	 */
	@Override
	public void setErrorText(String text) {
		if (textArea==null) {
			return;
		}
		textArea.setToolTipText(text);
		if (text==null) {
			textArea.setBackground(orgBackgroundColor);
		} else {
			textArea.setBackground(Color.RED);
		}
	}
	
	/**
	 * @see net.forwardfire.vasc.core.ui.VascUIComponent#isDisabled()
	 */
	@Override
	public boolean isDisabled() {
		return !textArea.isEnabled();
	}

	/**
	 * @see net.forwardfire.vasc.core.ui.VascUIComponent#setDisabled(boolean)
	 */
	@Override
	public void setDisabled(boolean disabled) {
		textArea.setEnabled(!disabled);
	}
	
	/**
	 * @see net.forwardfire.vasc.core.ui.VascUIComponent#isRendered()
	 */
	@Override
	public boolean isRendered() {
		return textArea.isVisible();
	}
	
	/**
	 * @see net.forwardfire.vasc.core.ui.VascUIComponent#setRendered(boolean)
	 */
	@Override
	public void setRendered(boolean rendered) {
		textArea.setVisible(rendered);
	}
}

class TextAreaListener implements DocumentListener {
	
	private VascValueModel model = null;
	private SwingTextArea textArea = null;
	
	public TextAreaListener(VascValueModel model,VascEntry entry,SwingTextArea textArea) {
		this.model=model;
		this.textArea=textArea;
	}
		
	/**
	 * @see javax.swing.event.DocumentListener#changedUpdate(javax.swing.event.DocumentEvent)
	 */
	@Override
	public void changedUpdate(DocumentEvent e) {
		update(e);
	}

	/**
	 * @see javax.swing.event.DocumentListener#insertUpdate(javax.swing.event.DocumentEvent)
	 */
	@Override
	public void insertUpdate(DocumentEvent e) {
		update(e);
	}
	
	/**
	 * @see javax.swing.event.DocumentListener#removeUpdate(javax.swing.event.DocumentEvent)
	 */
	@Override
	public void removeUpdate(DocumentEvent e) {
		update(e);
	}
	
	public void update(DocumentEvent event) {
		try {
			String value = event.getDocument().getText(0, event.getDocument().getLength());
			model.setValue(value);
			textArea.setErrorText(null);
		} catch (Exception ee) {
			textArea.setErrorText(ee.getLocalizedMessage());
			//table.getVascTableController().handleException(ee, table);
		}	
	}
}