/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JTabbedPane;

import net.forwardfire.vasc.core.VascController;
import net.forwardfire.vasc.core.VascEntry;
import net.forwardfire.vasc.core.VascEntryControllerLocal;
import net.forwardfire.vasc.core.VascEntryLocal;
import net.forwardfire.vasc.frontend.swing.SwingPanelIntegration;
import net.forwardfire.vasc.frontend.swing.SwingPanelTabbed;
import net.forwardfire.vasc.impl.DefaultVascFactory;
import net.forwardfire.vasc.test.frontend.data.TestModelData;
import junit.framework.TestCase;

/**
 * SwingTestTabbed shows interfaces in tabs.
 * 
 * @author Willem Cazander
 * @version 1.0 Feb 06, 2012
 */
public class SwingTestTabbed extends TestCase {
    
    public void testNull() throws Exception {
    	assertEquals(true, true);
    	//main(new String[] {});
    }

    public static void main(String[] argu){    	
    	SwingTestTabbed s = new SwingTestTabbed();
    	try {
			s.open();
		} catch (Exception e) {
			e.printStackTrace();
		}
    }
    
    public void open() throws Exception {
    	TestModelData testData = new TestModelData();
    	final VascController vc = testData.getTestVascController();
    	testData.createVascEntries(vc,(VascEntryControllerLocal)vc.getVascEntryController());
    	    	
		JFrame frame = new JFrame();
    	frame.setTitle("Vasc Tabbed Test");
    	frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
    	frame.setLocationRelativeTo(null);
		
    	final JTabbedPane pane = new JTabbedPane();
    	final SwingPanelIntegration spi = new SwingPanelTabbed(pane);
    	frame.add(pane);
    	
    	JMenuBar menubar = new JMenuBar();
        JMenu vascMenu = new JMenu("Entries");
        for (final String id:vc.getVascEntryController().getVascEntryIds()) {
        	JMenuItem item = new JMenuItem(id);
        	item.addActionListener(new ActionListener() {
        		@Override
				public void actionPerformed(ActionEvent e) {
					try {
						VascEntryLocal ve = (VascEntryLocal)vc.getVascEntryController().getVascEntryById(id);
						vc.getVascEntryConfigController().configVascFrontendController(vc, ve);
						spi.createNewVascView(ve);
					} catch (Exception e1) {
						e1.printStackTrace();
					}
				}
        		
        	});
        	vascMenu.add(item);
        }
        JMenuItem item = new JMenuItem("Close tab.");
    	item.addActionListener(new ActionListener() {
    		@Override
			public void actionPerformed(ActionEvent e) {
				if (pane.getSelectedIndex()>=0) {
					pane.removeTabAt(pane.getSelectedIndex()); // todo release vasc frontend
				}
			}
    		
    	});
        vascMenu.add(item);
        menubar.add(vascMenu);
        frame.setJMenuBar(menubar);
    	
    	// view
    	frame.setMinimumSize(new Dimension(800,600));
    	frame.pack();
    	frame.setVisible(true);
    	
    	VascEntryLocal ve = testData.getTestEntry(vc);
    	vc.getVascEntryConfigController().configVascFrontendController(vc, ve);
    	spi.createNewVascView(ve);
    }
}