/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.frontend.swt;

import java.io.Serializable;
import java.util.List;

import net.forwardfire.vasc.backend.select.VascBackendSelectItem;
import net.forwardfire.vasc.core.VascEntry;
import net.forwardfire.vasc.core.actions.GlobalVascAction;
import net.forwardfire.vasc.core.entry.VascEntryFrontendEventListener;
import net.forwardfire.vasc.frontend.VascFrontendPageInfoNumber;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

/**
 * SwtActionPanel renders action bar
 * 
 * @author Willem Cazander
 * @version 1.0 May 13, 2009
 */
public class SwtActionPanel implements VascEntryFrontendEventListener {

	private static final long serialVersionUID = 3747573488962487970L;
	private VascEntry entry = null;
	private Text rowNumberField = null;
	private Combo pageBox = null;
	private Combo downloadBox = null;
	private Label resultLabel = null;
	private boolean vascEvent = false;
	
	public SwtActionPanel(VascEntry entry) {
		this.entry=entry;
		entry.getVascFrontendController().addVascEntryFrontendEventListener(this);
	}
	
	public void createComposite(Composite parent) {
        Composite actionRow = new Composite(parent, SWT.NONE);
        GridLayout bodyLayout = new GridLayout();
		bodyLayout.numColumns = 8;
		actionRow.setLayout(bodyLayout);
		actionRow.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		//actionRow.setBackground(c);
        
		Label l = new Label(actionRow, SWT.CENTER);
        l.setText("Row Numbers");
        rowNumberField = new Text(actionRow, SWT.NONE | SWT.BORDER);
        rowNumberField.setText(""+entry.getVascFrontendController().getVascEntryState().getVascBackendState().getPageSize()+"  ");
        rowNumberField.addModifyListener(new ModifyListener() {
        	@Override
			public void modifyText(ModifyEvent arg0) {
				if (vascEvent) {
					return;
				}
				int pageSize = new Integer(rowNumberField.getText());
				entry.getVascFrontendController().getVascEntryState().getVascBackendState().setPageSize(pageSize);
				entry.getVascFrontendController().getVascFrontendActions().refreshData();
			
			}
		});
        
        l = new Label(actionRow, SWT.CENTER);
        l.setText("Goto: ");
        pageBox = new Combo(actionRow,SWT.SINGLE | SWT.BORDER);
        pageBox.addSelectionListener(new SelectionAdapter() {
        	@Override
			public void widgetSelected(SelectionEvent e) {
				entry.getVascFrontendController().getVascFrontendActions().pageAction(pageBox.getSelectionIndex()-1);
				pageBox.select(0);
			}
		});
        
        l = new Label(actionRow, SWT.CENTER);
        l.setText("Download: ");
        downloadBox = new Combo(actionRow,SWT.SINGLE | SWT.BORDER);
        downloadBox.add("...");
        for (GlobalVascAction a:entry.getExportActions()) {
			downloadBox.add(a.getName());
		}
        downloadBox.addSelectionListener(new SelectionAdapter() {
        	@Override
			public void widgetSelected(SelectionEvent e) {
			
				downloadBox.select(0);
			}
		});
        
        resultLabel = new Label(actionRow, SWT.CENTER);
        resultLabel.setText("Result......................................");
	}
	
	
	/**
	 * @see net.forwardfire.vasc.core.entry.VascEntryFrontendEventListener#getEventTypes()
	 */
	@Override
	public VascFrontendEventType[] getEventTypes() {
		VascFrontendEventType[] result = {VascEntryFrontendEventListener.VascFrontendEventType.POST_READ};
		return result;
	}
	
	@Override
	public void vascEvent(VascEntry entry,Serializable dataNotUsed) {
		vascEvent = true;

		long pageSize = entry.getVascFrontendController().getVascFrontendPageInfo().getPageSize();
		long pageStart = entry.getVascFrontendController().getVascFrontendPageInfo().getPageStartCount();
		long pageStop = entry.getVascFrontendController().getVascFrontendPageInfo().getPageStopCount();
		long pageTotalCount = entry.getVascFrontendController().getVascFrontendPageInfo().getPageTotalRecordCount();
		String format = "Results %1$d-%2$d from %3$d rows";
		resultLabel.setText(String.format(format, pageStart, pageStop, pageTotalCount));
		resultLabel.redraw();
		rowNumberField.setText(""+pageSize);
		rowNumberField.redraw();
		
		pageBox.removeAll();
		pageBox.add("Goto...");
		List<VascFrontendPageInfoNumber> pages = entry.getVascFrontendController().getVascFrontendPageInfo().getTablePagesFromBackend();
		int i=0;
		for (VascFrontendPageInfoNumber page:pages) {
			pageBox.add("page: "+page.getPageNumber()+"  "+(i*pageSize)+"-"+((i*pageSize)+pageSize));
			i++;
		}
		
		vascEvent = false;
	}
}
