/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.frontend.swt;

import java.io.Serializable;

import net.forwardfire.vasc.backend.VascBackendException;
import net.forwardfire.vasc.core.VascEntry;
import net.forwardfire.vasc.core.VascEntryField;
import net.forwardfire.vasc.core.ui.VascColumnValueModelListener;
import net.forwardfire.vasc.core.ui.VascUIComponent;
import net.forwardfire.vasc.core.ui.VascValueModel;
import net.forwardfire.vasc.core.VascException;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;


/**
 * SwtVascEditDialog renders swt edit dialog
 * 
 * @author Willem Cazander
 * @version 1.0 May 13, 2009
 */
public class SwtVascEditDialog extends Dialog {

	private VascEntry entry = null;
	private Shell shell = null;
	
	public SwtVascEditDialog (VascEntry entry) {
		super (Display.getCurrent().getActiveShell(), 0);
		this.entry=entry;
	}
	
	protected String i18n(String key,Object...params) {
		return entry.getVascFrontendController().getVascEntryResourceResolver().getTextValue(key,params);
	}
	
	protected Object i18nImage(String key) {
		if (entry.getVascFrontendController().getVascEntryResourceImageResolver()==null) {
			return null;
		}
		return entry.getVascFrontendController().getVascEntryResourceImageResolver().getImageValue(entry,key);
	}
	
	public void open() throws VascException {
		shell = new Shell(getParent(), SWT.TITLE | SWT.BORDER | SWT.APPLICATION_MODAL);
		shell.setText(i18n(entry.getName()));
		
		GridLayout layout = new GridLayout();
		layout.marginHeight = 0;
		layout.marginWidth = 0;
		shell.setLayout(layout);
    		
		Composite header = new Composite(shell, SWT.NONE);
		GridLayout headerLayout = new GridLayout();
		headerLayout.numColumns = 6;
		header.setLayout(headerLayout);
		header.setLayoutData(new GridData(GridData.FILL_BOTH));
    		
		Composite body = new Composite(shell, SWT.NONE);
		GridLayout bodyLayout = new GridLayout();
		bodyLayout.numColumns = 1;
		body.setLayout(bodyLayout);  
		body.setLayoutData(new GridData(GridData.FILL_BOTH));
    		
		Composite footer = new Composite(shell, SWT.NONE);
		GridLayout footerLayout = new GridLayout();
		footerLayout.numColumns = 6;
		footer.setLayout(footerLayout);
		footer.setLayoutData(new GridData(SWT.NONE));
    	
		createHeader(header);
		createBody(body);
		createFooter(footer);
	
		shell.pack();
		shell.open();

		Display display = shell.getDisplay();
		while (!shell.isDisposed()) {
			if (!display.readAndDispatch()) {
				display.sleep();
			}
		}
	}
	
	public void createHeader(Composite header) {
		
		String displayFieldId = entry.getDisplayNameFieldId();
		VascEntryField dis = entry.getVascEntryFieldById(displayFieldId);
		if (dis==null) {
			throw new RuntimeException("Could not find: "+displayFieldId+" from: "+entry.getId());
		}
		String name = null;
		try {
			Serializable bean = entry.getVascFrontendController().getVascEntryState().getEntryDataObject();
			name = ""+dis.getVascEntryFieldValue().getValue(dis.getBackendName(), bean);
		} catch (VascBackendException e) {
			throw new RuntimeException("Could not display value from "+entry.getId(),e);
		}
		
		Color c = new Color(header.getDisplay(),255,255,255);
		header.setBackground(c);
		
		Label img = new Label(header, SWT.NONE);
		if (entry.getVascFrontendController().getVascEntryState().isEditCreate()) {
			img.setImage((Image)i18nImage(entry.getCreateImage()));
		} else {
			img.setImage((Image)i18nImage(entry.getEditImage()));
		}
		img.setBackground(c);
		
		Font headerFont = new Font(header.getDisplay(), "verdana", 14, SWT.NONE);
		Label l = new Label(header, SWT.CENTER);
		if (entry.getVascFrontendController().getVascEntryState().isEditCreate()) {
			l.setText(i18n(entry.getCreateDescription(),name));
		} else {
			l.setText(i18n(entry.getEditDescription(),name));
		}
		l.setFont(headerFont);
		l.setBackground(c);
}
	
	public void createBody(Composite body) throws VascException {
		body.setLayout(new GridLayout(2, true));
		body.setLayoutData(new GridData(GridData.FILL_BOTH));
		Serializable bean = entry.getVascFrontendController().getVascEntryState().getEntryDataObject();
		
		entry.getVascFrontendController().clearFieldRenderObjects(); // only needed for swt use 
		
        for (VascEntryField c:entry.getVascEntryFields()) {
        	if (entry.getVascFrontendController().getVascFrontendDataSelector().isFieldEdit(c)==false) {
				continue;
			}
        	
        	for (int i=0;i<c.getVascEntryFieldType().getUIComponentCount(c);i++) {
        		
        		VascUIComponent label = c.getVascEntryFieldType().provideLabelUIComponent(i,c);
        		VascValueModel model = new VascValueModel();
        		model.setValue(i18n(c.getName()));
        		Control realLabel = (Control)label.createComponent(entry,c,model,body);
        		realLabel.setToolTipText(i18n(c.getDescription()));
        		
        		VascUIComponent editor = c.getVascEntryFieldType().provideEditorUIComponent(i,c);
        		model = new VascValueModel(c.getVascEntryFieldType().provideEditorVascValueModel(i,c));
        		try {
					model.setValue(c.getVascEntryFieldValue().getValue(c.getBackendName(), bean));
				} catch (VascBackendException e) {
					throw new VascException(e);
				}
        		model.addListener(new VascColumnValueModelListener(c,bean));
        		Control edit = (Control )editor.createComponent(entry,c,model,body);
        		        		
        		if (edit.getLayoutData()==null) { // is for textarea which has custum layout data
        			GridData gridData = new GridData();
        			gridData.grabExcessHorizontalSpace = true;
        			gridData.grabExcessVerticalSpace = true;
        			gridData.horizontalAlignment = GridData.FILL;
        			gridData.verticalAlignment = GridData.FILL;
        			edit.setLayoutData(gridData);
        		}
        		
        		if (i==0) {
        			entry.getVascFrontendController().addFieldVascUIComponents(c, editor,edit);
        		}
        	}
        }
        entry.getVascFrontendController().getVascFrontendHelper().editReadOnlyUIComponents(entry);
	}
	
	public void createFooter(Composite body) {
		Button saveButton = new Button(body, SWT.NONE);
        saveButton.setImage((Image)i18nImage("vasc.action.saveAction.image"));
        saveButton.setText(i18n("vasc.action.saveAction.name"));
        saveButton.setToolTipText(i18n("vasc.action.saveAction.description"));
		saveButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				try {
					boolean error = entry.getVascFrontendController().getVascFrontendHelper().validateAndSetErrorText(entry);
					if (error) {
						return;
					}
					entry.getVascFrontendController().getVascFrontendActions().mergeObject();
					shell.dispose();
				} catch (Exception ee) {
					ee.printStackTrace();
				}
			}
		});
		Button cancelButton = new Button(body, SWT.NONE);
        cancelButton.setImage((Image)i18nImage("vasc.action.cancelAction.image"));
        cancelButton.setText(i18n("vasc.action.cancelAction.name"));
        cancelButton.setToolTipText(i18n("vasc.action.cancelAction.description"));
		cancelButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				entry.getVascFrontendController().getVascEntryState().setEntryDataObject(null);
				shell.dispose();
			}
		});
	}
}
