/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.frontend.swt.ui;

import net.forwardfire.vasc.core.VascEntry;
import net.forwardfire.vasc.core.VascEntryField;
import net.forwardfire.vasc.core.ui.VascUIComponent;
import net.forwardfire.vasc.core.ui.VascValueModel;
import net.forwardfire.vasc.core.VascException;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;



/**
 * 
 *
 * @author Willem Cazander
 * @version 1.0 Jan 13, 2009
 */
public class SwtButton implements VascUIComponent {
	
	private Button button = null;
	private Color orgBackgroundColor = null;
	private Color errorColor = null;
	
	@Override
	public Object createComponent(VascEntry table,VascEntryField entryField,VascValueModel model,Object gui) throws VascException {
		button = new Button((Composite)gui, SWT.NONE);
		orgBackgroundColor = button.getBackground();
		errorColor = new Color(((Composite)gui).getDisplay(),255,0,0);
		//button.setImage(getImageDescriptor("vasc.dialog.save.image").createImage());
		button.setText(model.getValue()+"");
		//button.setToolTipText(i18n("vasc.dialog.save.tooltip"));
		button.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
			}
		});
		return button;
	}
	
	/**
	 * @see net.forwardfire.vasc.core.ui.VascUIComponent#getErrorText()
	 */
	@Override
	public String getErrorText() {
		if (button==null) {
			return null;
		}
		return button.getToolTipText();
	}

	/**
	 * @see net.forwardfire.vasc.core.ui.VascUIComponent#setErrorText(java.lang.String)
	 */
	@Override
	public void setErrorText(String text) {
		if (button==null) {
			return;
		}
		button.setToolTipText(text);
		if (text==null) {
			button.setBackground(orgBackgroundColor);
		} else {
			button.setBackground(errorColor);
		}
	}
	
	/**
	 * @see net.forwardfire.vasc.core.ui.VascUIComponent#isDisabled()
	 */
	@Override
	public boolean isDisabled() {
		return !button.isEnabled();
	}

	/**
	 * @see net.forwardfire.vasc.core.ui.VascUIComponent#setDisabled(boolean)
	 */
	@Override
	public void setDisabled(boolean disabled) {
		button.setEnabled(!disabled);
	}
	
	/**
	 * @see net.forwardfire.vasc.core.ui.VascUIComponent#isRendered()
	 */
	@Override
	public boolean isRendered() {
		return button.isVisible();
	}

	/**
	 * @see net.forwardfire.vasc.core.ui.VascUIComponent#setRendered(boolean)
	 */
	@Override
	public void setRendered(boolean rendered) {
		button.setVisible(rendered);
	}
}