/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.frontend.swt.ui;


import java.util.List;

import net.forwardfire.vasc.backend.select.VascBackendSelectItem;
import net.forwardfire.vasc.core.VascEntry;
import net.forwardfire.vasc.core.VascEntryField;
import net.forwardfire.vasc.core.ui.VascSelectItemModel;
import net.forwardfire.vasc.core.ui.VascUIComponent;
import net.forwardfire.vasc.core.ui.VascValueModel;
import net.forwardfire.vasc.core.VascException;
import net.forwardfire.vasc.frontend.VascFrontendException;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;



/**
 * 
 *
 * @author Willem Cazander
 * @version 1.0 May 10, 2009
 */
public class SwtList implements VascUIComponent {
	
	private Combo combo = null;
	private Color orgBackgroundColor = null;
	private Color errorColor = null;
	private List<VascBackendSelectItem> data = null;
	
	@Override
	public Object createComponent(final VascEntry entry,VascEntryField entryField,final VascValueModel model,Object gui) throws VascException {
		combo = new Combo((Composite)gui,SWT.SINGLE | SWT.BORDER);
		combo.setVisibleItemCount(8);
		VascSelectItemModel items = (VascSelectItemModel)entryField.getVascEntryFieldType().getDataObject();
		if (items!=null) {
			data = items.getVascSelectItems(entry);
		}
		fillCombo();
		orgBackgroundColor = combo.getBackground();
		errorColor = new Color(((Composite)gui).getDisplay(),255,0,0);
		combo.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				VascBackendSelectItem item = findItem(combo.getText());
				if (item!=null) {
					try {
						model.setValue(item.getValue());
					} catch (VascException ve) {
						throw new VascFrontendException(ve);
					}
				}
			}
		});
		
		// sets default selected
		for (VascBackendSelectItem i:data) {
			Object iValue = i.getValue();
			Object mValue = model.getValue();
			if (iValue==null && mValue==null) {
				combo.setText(i.getLabel());
				break;
			}
			if (iValue==null) {
				continue; 
			}
			if (iValue.equals(mValue)) {
				combo.setText(i.getLabel());
				break;
			}
		}
		if (combo.getText()==null) {
			combo.setText(data.get(0).getLabel()); // select top one.
		}
		return combo;
	}
	
	private void fillCombo() {
		if (data==null | combo==null) {
			return;
		}
		for (VascBackendSelectItem i:data) {
			if (i.isDisabled()) {
				continue;
			}
			combo.add(i.getLabel());
		}
	}
	
	private VascBackendSelectItem findItem(String text) {
		for (VascBackendSelectItem i:data) {
			if (i.getLabel().equals(text)) {
				return i;
			}
		}
		return null;
	}
	
	/**
	 * @see net.forwardfire.vasc.core.ui.VascUIComponent#getErrorText()
	 */
	@Override
	public String getErrorText() {
		if (combo==null) {
			return null;
		}		
		return combo.getToolTipText();
	}

	/**
	 * @see net.forwardfire.vasc.core.ui.VascUIComponent#setErrorText(java.lang.String)
	 */
	@Override
	public void setErrorText(String text) {
		if (combo==null) {
			return;
		}
		combo.setToolTipText(text);
		if (text==null) {
			combo.setBackground(orgBackgroundColor);
		} else {
			combo.setBackground(errorColor);
		}
	}
	
	/**
	 * @see net.forwardfire.vasc.core.ui.VascUIComponent#isDisabled()
	 */
	@Override
	public boolean isDisabled() {
		return !combo.isEnabled();
	}

	/**
	 * @see net.forwardfire.vasc.core.ui.VascUIComponent#setDisabled(boolean)
	 */
	@Override
	public void setDisabled(boolean disabled) {
		combo.setEnabled(!disabled);
	}
	
	/**
	 * @see net.forwardfire.vasc.core.ui.VascUIComponent#isRendered()
	 */
	@Override
	public boolean isRendered() {
		return combo.isVisible();
	}

	/**
	 * @see net.forwardfire.vasc.core.ui.VascUIComponent#setRendered(boolean)
	 */
	@Override
	public void setRendered(boolean rendered) {
		combo.setVisible(rendered);
	}
}