/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.frontend.swt.ui;

import java.io.Serializable;

import net.forwardfire.vasc.core.VascEntry;
import net.forwardfire.vasc.core.VascEntryField;
import net.forwardfire.vasc.core.ui.VascUIComponent;
import net.forwardfire.vasc.core.ui.VascValueModel;
import net.forwardfire.vasc.core.VascException;
import net.forwardfire.vasc.frontend.VascFrontendException;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.FontMetrics;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Text;


/**
 * 
 *
 * @author Willem Cazander
 * @version 1.0 Jan 13, 2009
 */
public class SwtTextArea implements VascUIComponent {
	
	private Text text = null;
	private Color orgBackgroundColor = null;
	private Color errorColor = null;
	
	@Override
	public Object createComponent(VascEntry entry,VascEntryField entryField,VascValueModel model,Object gui) throws VascException {
		text = new Text((Composite)gui, SWT.MULTI | SWT.BORDER);
		orgBackgroundColor = text.getBackground();
		errorColor = new Color(((Composite)gui).getDisplay(),255,0,0);
		
		int cols = 25;
		int rows = 3;
		
		String colsString = entryField.getVascEntryFieldType().getProperty("editor.columns");
		if (colsString!=null && "".equals(colsString)==false) {
			cols = new Integer(colsString);
		}
		String rowsString = entryField.getVascEntryFieldType().getProperty("editor.rows");
		if (rowsString!=null && "".equals(rowsString)==false) {
			rows = new Integer(rowsString);
		}
		
		GC gc = new GC(text);
		FontMetrics fm = gc.getFontMetrics ();
		gc.dispose ();
		int width = cols * fm.getAverageCharWidth();
		int height = rows * fm.getHeight();
		GridData data = new GridData();
		data.widthHint = width;
		data.heightHint = height;
		text.setLayoutData(data);
		Object value = model.getValue();
		if (value!=null) {
			text.setText(value.toString());
		}
		text.addModifyListener(new TextListener(model,entry));
		return text;
	}
	
	class TextListener implements ModifyListener {
		private VascValueModel model = null;
		private VascEntry entry = null;
		
		public TextListener(VascValueModel model,VascEntry entry) {
			this.model=model;
			this.entry=entry;
		}
		
		@Override
		public void modifyText(ModifyEvent e) {
			Serializable value = text.getText();
			try {
				model.setValue(value);
			} catch (VascException ve) {
				throw new VascFrontendException(ve);
			}
		}
	}
	
	/**
	 * @see net.forwardfire.vasc.core.ui.VascUIComponent#getErrorText()
	 */
	@Override
	public String getErrorText() {
		if (text==null) {
			return null;
		}		
		return text.getToolTipText();
	}

	/**
	 * @see net.forwardfire.vasc.core.ui.VascUIComponent#setErrorText(java.lang.String)
	 */
	@Override
	public void setErrorText(String textString) {
		if (text==null) {
			return;
		}
		text.setToolTipText(textString);
		if (textString==null) {
			text.setBackground(orgBackgroundColor);
		} else {
			text.setBackground(errorColor);
		}
	}
	
	/**
	 * @see net.forwardfire.vasc.core.ui.VascUIComponent#isDisabled()
	 */
	@Override
	public boolean isDisabled() {
		return !text.isEnabled();
	}

	/**
	 * @see net.forwardfire.vasc.core.ui.VascUIComponent#setDisabled(boolean)
	 */
	@Override
	public void setDisabled(boolean disabled) {
		text.setEnabled(!disabled);
	}
	
	/**
	 * @see net.forwardfire.vasc.core.ui.VascUIComponent#isRendered()
	 */
	@Override
	public boolean isRendered() {
		return text.isVisible();
	}

	/**
	 * @see net.forwardfire.vasc.core.ui.VascUIComponent#setRendered(boolean)
	 */
	@Override
	public void setRendered(boolean rendered) {
		text.setVisible(rendered);
	}
}