/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.frontend.web.jsf;

import java.io.Serializable;
import java.util.List;
import java.util.Map;

import net.forwardfire.vasc.backend.list.VascBackendListRequest;
import net.forwardfire.vasc.core.VascController;
import net.forwardfire.vasc.core.VascEntry;
import net.forwardfire.vasc.core.VascEntryField;
import net.forwardfire.vasc.core.VascEntryLocal;
import net.forwardfire.vasc.frontend.VascFrontendController;
import net.forwardfire.vasc.impl.DefaultVascFactory;

/**
 * AbstractJSFVascFacesControllerBase for making jsf frontend.
 * 
 * @author Willem Cazander
 * @version 1.0 Dec 21, 2011
 */
public abstract class AbstractJSFVascFacesControllerBase {

	abstract public VascFrontendController getNewVascFrontendData();
	abstract public VascController getVascController();
	
	public void setVascController(VascController vascController) {		
	}
	
	/**
	 * 
	 */
    public List<Serializable> executeVascList(String entryId,Map<String,Object> para) {
    	try {
    		// plug all object 
    		VascEntryLocal entry = (VascEntryLocal)getVascController().getVascEntryController().getVascEntryById(entryId);
    		getVascController().getVascEntryConfigController().configVascFrontendController(getVascController(), entry);

			// set def para
			VascBackendListRequest state = entry.getVascFrontendController().getVascEntryState().getVascBackendState();
			for (String key2:entry.getEntryParameterKeys()) {
				Object value = entry.getEntryParameter(key2);
				state.setDataParameter(key2, value);
			}
			// copy para
    		if (para!=null) {
    			for (String key:para.keySet()) {
    				state.setDataParameter(key, para.get(key));
    			}
    		}
			
			// set list para
			for (VascEntryField vef:entry.getVascEntryListOptions()) {
				Object def = vef.getDefaultValue();
				if (def==null) {
					continue;
				}
				String backendName = vef.getBackendName();
				state.setDataParameter(backendName, def);
			}
    		
    		// done, execute and return data.
    		entry.getVascFrontendController().getVascFrontendActions().refreshData();
			return entry.getVascFrontendController().getVascEntryState().getEntryDataList();
		} catch (Exception e) {
			throw new RuntimeException("Could not execute list: "+e.getMessage(),e);
		}
    }
	
}
