/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.frontend.web.jsf;

import java.util.Collections;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.ResourceBundle;

import javax.naming.Context;
import javax.naming.InitialContext;

import net.forwardfire.vasc.backend.VascBackendControllerLocal;
import net.forwardfire.vasc.backend.crud.VascBackendCrud;
import net.forwardfire.vasc.backend.proxy.VascBackendProxyEventExecutor;
import net.forwardfire.vasc.core.VascController;
import net.forwardfire.vasc.core.VascEntryControllerLocal;
import net.forwardfire.vasc.core.VascEntryLocal;
import net.forwardfire.vasc.ejb3.VascServiceManager;
import net.forwardfire.vasc.ejb3.VascServiceRemoteBackend;
import net.forwardfire.vasc.impl.DefaultVascFactory;
import net.forwardfire.vasc.impl.entry.SetParameterBackendListener;



/**
 * Base faces session object for managing vasc entries.
 * 
 * @author Willem Cazander
 * @version 1.0 Sep 21, 2010
 */
abstract public class AbstractJSFVascFacesControllerEJB extends AbstractJSFVascFacesControllerBase {

	private VascController vascController = null;
	
	abstract public String getVascSericeManagerJNDI();
	abstract public String getResourceBundleWEB();
	abstract public boolean getDisableI18N();
	abstract public VascUserInfo getVascUserInfo();
	
	public class VascUserInfo {
		Long userId;
		String username;
		String[] roles;
		public VascUserInfo(Long userId,String username,String[] roles) {
			this.userId=userId;
			this.username=username;
			this.roles=roles;
		}
	}
	
	/*
	class RefreshObjectForLazyPropertiesListener implements VascEntryFrontendEventListener {
		public VascFrontendEventType[] getEventTypes() {
			VascFrontendEventType[] result = {VascEntryFrontendEventListener.VascFrontendEventType.POST_UPDATE};
			return result;
		}
		public void vascEvent(VascEntry entry,Object data) {
			if (data.getClass().getName().startsWith("some.class.models.jpa")==false) {
				return; // only refresh model classes
			}
			
			// TODO: mmm rewrite events to change to different object to use ejb3++ solution ?
			// Use hiberbate session to attach a detected object again. (note: this is not possible with EJB3, uses merge() which returns a new object)
			//((Session)entityManager.getDelegate()).refresh(data);
		}
	}
	*/
	
	class MergedResourceBundle extends ResourceBundle {
		Map<String,String> data = new HashMap<String,String>(500);
		
		public void addDateMap(Map<String,String> dataMap) {
			data.putAll(dataMap);
		}
		public void addDataBundle(ResourceBundle bundle) {
			for (String key:bundle.keySet()) {
				String value = bundle.getString(key);
				data.put(key, value);
			}
		}
		
		@Override
		public Object handleGetObject(String key) {
			return data.get(key);
		}
		
		@Override
		public Enumeration<String> getKeys() {
			return Collections.enumeration(data.keySet());
		}
	}
	
	/**
	 * Config vasc frontend data object for jsf component to use.
	 * 
	
	public VascFrontendController getNewVascFrontendData() {		
		Locale locale = FacesContext.getCurrentInstance().getViewRoot().getLocale();
		MergedResourceBundle bundle = new MergedResourceBundle();
		bundle.addDataBundle(ResourceBundle.getBundle(getResourceBundleWEB(), locale));
		
		try {
			Context context = new InitialContext();
			VascServiceManager vascManager = (VascServiceManager)context.lookup(getVascSericeManagerJNDI());
			Map<String,String> remoteI18n = vascManager.getResourceBundle(locale.toString());
			bundle.addDateMap(remoteI18n);
		} catch (Exception e) {
			throw new RuntimeException("Could not get the ejb resource bundle: "+e.getMessage(),e);
		}
		
		VascFrontendController vascFrontendData = DefaultVascFactory.getDefaultVascFrontendData(bundle);
		vascFrontendData.addVascEntryFrontendEventListener(new VascEntryFrontendEventListener() {
			private static final long serialVersionUID = 1L;
			public VascFrontendEventType[] getEventTypes() {
				VascFrontendEventType[] result = {VascEntryFrontendEventListener.VascFrontendEventType.EXCEPTION};
				return result;
			}
			public void vascEvent(VascEntry entry, Object data) {
				if (data instanceof Exception) {
					((Exception)data).printStackTrace();
				}
			}
		});
	
		// config frontend data for ISF usage
		if (getDisableI18N()) {
			vascFrontendData.setVascEntryResourceResolver(new DefaultVascEntryResourceResolver());
		} else {
			vascFrontendData.setVascEntryResourceResolver(new DefaultVascEntryResourceResolver(bundle));
		}
        return vascFrontendData;
	}
		 * @return
	 */
		
	
	/**
	 * @return the vascController
	 */
	@Override
	public VascController getVascController() {
		if (vascController!=null) {
			return vascController;
		}
		try {
			VascUserInfo vui = getVascUserInfo();
			
			
			// get local jvm plugging controller
			VascController c = DefaultVascFactory.getDefaultVascController();
				
			/*
			DefaultVascEntryController con = new DefaultVascEntryController() {
				public VascEntry getVascEntryById(String id) {
					VascEntry ve = super.getVascEntryById(id);
					// add global listener to make sure we have all LAZY properties of a bean before we goto edit mode.
					ve.addVascEntryEventListener(VascEventType.DATA_SELECT, new RefreshObjectForLazyPropertiesListener());
					return ve;
				}
			};
			((DefaultVascController)c).setVascEntryController(con);
			*/
			
			Context context = new InitialContext();
			VascServiceManager vascManager = (VascServiceManager)context.lookup(getVascSericeManagerJNDI());
			List<String> entryIds = vascManager.getVascEntryIds();
			
			VascEntryControllerLocal localEntryController = (VascEntryControllerLocal)c.getVascEntryController();
			VascBackendControllerLocal localBackendController = (VascBackendControllerLocal)c.getVascBackendController();
			for (String id:entryIds) {
				VascEntryLocal ve = (VascEntryLocal)vascManager.getVascEntry(id);
				String backendId = ve.getBackendId();
				VascBackendCrud vb = new VascServiceRemoteBackend(vascManager,backendId);
				
				for (String key:ve.getEntryParameterKeys()) {
					Object value = ve.getEntryParameter(key);
					if (value instanceof String==false) {
						continue;
					}
					String paraValue = (String)value;
					if (paraValue.startsWith("userPara")==false) {
						continue;
					}
					String[] ps = paraValue.split(":");
					String type = ps[1];
					String paraTypeValue = ps[2];
					
					SetParameterBackendListener listener = new SetParameterBackendListener(vui.userId,vui.username);
					listener.setName(key);
					listener.setType(type);
					listener.setValue(paraTypeValue);
					
					VascBackendProxyEventExecutor localUserProxy = new VascBackendProxyEventExecutor();
					localUserProxy.addVascEntryBackendEventListener(listener);
					vb = localUserProxy;
				}
				localBackendController.addVascBackend(vb);
				localEntryController.addVascEntry(ve);
				
				c.getVascEntryConfigController().configVascEntry(c, ve);
			}
			vascController = c;
			return vascController;
		} catch (Exception e) {
			throw new RuntimeException("Could not create remote based vasc controller; "+e.getMessage(),e);
		}
	}
}
