/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.frontend.web.jsf;


import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Collections;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;
import java.util.ResourceBundle;

import javax.xml.parsers.ParserConfigurationException;

import org.x4o.xml.io.X4OConnectionException;
import org.x4o.xml.io.X4OReader;
import org.xml.sax.SAXException;

import net.forwardfire.vasc.core.VascController;
import net.forwardfire.vasc.core.VascEntryControllerLocal;
import net.forwardfire.vasc.core.VascException;
import net.forwardfire.vasc.frontend.VascFrontendException;
import net.forwardfire.vasc.impl.DefaultVascFactory;
import net.forwardfire.vasc.impl.x4o.VascDriver;



/**
 * Base faces session object for managing vasc entries.
 * 
 * @author Willem Cazander
 * @version 1.0 Sep 21, 2010
 */
abstract public class AbstractJSFVascFacesControllerLocal extends AbstractJSFVascFacesControllerBase  {

	protected VascController vascController = null;
	protected Map<String,Object> vascGlobalELBeans = new HashMap<String,Object>(3);
	
	abstract public String getResourceBundleWEB();
	abstract public boolean getDisableI18N();
	abstract public VascUserInfo getVascUserInfo();
	abstract public String[] getVascParseResources();
	
	public class VascUserInfo {
		Long userId;
		String username;
		String[] roles;
		public VascUserInfo(Long userId,String username,String[] roles) {
			this.userId=userId;
			this.username=username;
			this.roles=roles;
		}
	}
	
	class MergedResourceBundle extends ResourceBundle {
		Map<String,String> data = new HashMap<String,String>(500);
		
		public void addDateMap(Map<String,String> dataMap) {
			data.putAll(dataMap);
		}
		public void addDataBundle(ResourceBundle bundle) {
			for (String key:bundle.keySet()) {
				String value = bundle.getString(key);
				data.put(key, value);
			}
		}
		
		@Override
		public Object handleGetObject(String key) {
			return data.get(key);
		}
		
		@Override
		public Enumeration<String> getKeys() {
			return Collections.enumeration(data.keySet());
		}
	}
	
	/**
	 * Config vasc frontend data object for jsf component to use.

	public VascFrontendController getNewVascFrontendData() {		
		Locale locale = FacesContext.getCurrentInstance().getViewRoot().getLocale();
		MergedResourceBundle bundle = new MergedResourceBundle();
		bundle.addDataBundle(ResourceBundle.getBundle(getResourceBundleWEB(), locale));

		VascFrontendController vascFrontendData = DefaultVascFactory.getDefaultVascFrontendData(bundle);
		vascFrontendData.addVascEntryFrontendEventListener(new VascEntryFrontendEventListener() {
			private static final long serialVersionUID = 1L;
			public VascFrontendEventType[] getEventTypes() {
				VascFrontendEventType[] result = {VascEntryFrontendEventListener.VascFrontendEventType.EXCEPTION};
				return result;
			}
			public void vascEvent(VascEntry entry, Object data) {
				if (data instanceof Exception) {
					((Exception)data).printStackTrace();
				}
			}
		});
		
		// config frontend data for ISF usage
		if (getDisableI18N()) {
			vascFrontendData.setVascEntryResourceResolver(new DefaultVascEntryResourceResolver());
		} else {
			vascFrontendData.setVascEntryResourceResolver(new DefaultVascEntryResourceResolver(bundle));
		}
        return vascFrontendData;
	}
		 * 
	 * @return
	 */
	
	/**
	 * @return the vascController
	 */
	@Override
	public VascController getVascController() {
		if (vascController!=null) {
			return vascController;
		}
		try {
			// get local jvm plugging controller
			VascUserInfo vui = getVascUserInfo();
			VascController c = DefaultVascFactory.getDefaultVascController();
			
			// Load vasc resources
			for (String res:getVascParseResources()) {
				VascDriver driver = VascDriver.getInstance();
				X4OReader<VascController> reader = driver.createReader();
				driver.addVascController(reader, c);
				for (String name:vascGlobalELBeans.keySet()) {
					Object value = vascGlobalELBeans.get(name);
					reader.addELBeanInstance(name, value);
				}
				reader.readResource(res);
			}
			DefaultVascFactory.fillVascControllerLocalEntries((VascEntryControllerLocal) c.getVascEntryController(), c);
			
			vascController = c;
			return vascController;
		} catch (VascException e) {
			throw new VascFrontendException(e);
		} catch (FileNotFoundException e) {
			throw new VascFrontendException(e);
		} catch (SecurityException e) {
			throw new VascFrontendException(e);
		} catch (NullPointerException e) {
			throw new VascFrontendException(e);
		} catch (SAXException e) {
			throw new VascFrontendException(e);
		} catch (IOException e) {
			throw new VascFrontendException(e);
		} catch (X4OConnectionException e) {
			throw new VascFrontendException(e);
		}
	}
	
}
