/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.frontend.web.jsf;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import javax.el.ValueExpression;
import javax.faces.context.FacesContext;

import net.forwardfire.vasc.core.VascEntry;
import net.forwardfire.vasc.core.entry.VascEntryFrontendEventListener;


/**
 * JSFVascEntryEventListener fills data in support bean.
 * 
 * @author Willem Cazander
 * @version 1.0 Sep 11, 2009
 */
public class JSFVascEntryEventListener implements VascEntryFrontendEventListener {

	private static final long serialVersionUID = 1765054259934158076L;
	private String entrySupportVar = null;
	
	public JSFVascEntryEventListener(String entrySupportVar) {
		this.entrySupportVar=entrySupportVar;
	}
	
	/**
	 * @see net.forwardfire.vasc.core.entry.VascEntryFrontendEventListener#getEventTypes()
	 */
	@Override
	public VascFrontendEventType[] getEventTypes() {
		VascFrontendEventType[] result = {
				VascEntryFrontendEventListener.VascFrontendEventType.POST_READ,
				VascEntryFrontendEventListener.VascFrontendEventType.POST_DELETE,
				VascEntryFrontendEventListener.VascFrontendEventType.POST_UPDATE
			};
		return result;
	}
	
	@Override
	public void vascEvent(VascEntry entry,Serializable dataNotUsed) {
		/* moved to fillVascEntryFrontend
		try {
			for (VascEntryField field:entry.getVascEntryFields()) {
				if (field.getVascEntryFieldValue()==null) {
					// TODO: fix this for better remote support
					VascEntryField fieldClone = field.clone();
					fieldClone.getVascValidators().clear();
					
					VascEntryFieldValue v = entry.getVascFrontendData().getVascEntryState().getVascBackend().provideVascEntryFieldValue(fieldClone);
					field.setVascEntryFieldValue(v);
				}
			}
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
		*/
		List<Serializable> data = entry.getVascFrontendController().getVascEntryState().getEntryDataList();
		List<Object> result = new ArrayList<Object>(data.size());
		int index = 0;
		for (Serializable o:data) {
			VascDataBackendBean b = new VascDataBackendBean(entry,o,index);
			result.add(b);
			index++;
		}

		ValueExpression ve2 = FacesContext.getCurrentInstance().getApplication().getExpressionFactory().createValueExpression(FacesContext.getCurrentInstance().getELContext(), "#{"+entrySupportVar+".tableDataModel.wrappedData}", Object.class);
		ve2.setValue(FacesContext.getCurrentInstance().getELContext(), result);
		ValueExpression ve3 = FacesContext.getCurrentInstance().getApplication().getExpressionFactory().createValueExpression(FacesContext.getCurrentInstance().getELContext(), "#{"+entrySupportVar+".tablePagesDataModel.wrappedData}", Object.class);
		ve3.setValue(FacesContext.getCurrentInstance().getELContext(), entry.getVascFrontendController().getVascFrontendPageInfo().getTablePagesFromBackend());
	}
}
