/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.frontend.web.jsf;


import java.io.IOException;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.logging.Logger;

import javax.faces.component.UIComponent;
import javax.faces.component.html.HtmlCommandButton;
import javax.faces.component.html.HtmlCommandLink;
import javax.faces.context.FacesContext;
import javax.faces.event.ActionEvent;
import javax.faces.event.PhaseEvent;
import javax.faces.event.PhaseId;
import javax.faces.event.PhaseListener;
import javax.faces.event.ValueChangeEvent;
import javax.faces.model.DataModel;
import javax.faces.model.ListDataModel;
import javax.faces.model.SelectItem;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;

import net.forwardfire.vasc.backend.crud.VascBackendCrud;
import net.forwardfire.vasc.core.VascEntry;
import net.forwardfire.vasc.core.VascEntryField;
import net.forwardfire.vasc.core.VascEntryState;
import net.forwardfire.vasc.core.VascEntryLink;
import net.forwardfire.vasc.core.VascEntryLinkType;
import net.forwardfire.vasc.core.VascException;
import net.forwardfire.vasc.core.actions.GlobalVascAction;
import net.forwardfire.vasc.core.actions.RowVascAction;
import net.forwardfire.vasc.core.entry.VascEntryExport;
import net.forwardfire.vasc.core.entry.VascEntryExportWriter;
import net.forwardfire.vasc.frontend.VascFrontendPageInfoNumber;
import net.forwardfire.vasc.frontend.VascFrontendException;
import net.forwardfire.vasc.impl.actions.AddRowAction;
import net.forwardfire.vasc.impl.actions.DeleteRowAction;
import net.forwardfire.vasc.impl.actions.EditRowAction;



/**
 * JSFVascEntrySupportBean add EL support for renderer.
 * 
 * @author Willem Cazander
 * @version 1.0 Sep 10, 2009
 */
public class JSFVascEntrySupportBean implements Serializable {

	private static final long serialVersionUID = 1L;
	private Logger logger = null;
	private VascEntry entry = null;
	private JSFVascSupportI18nMapController i18nMap = null;
	private VascDataBackendBean selected = null;
	private String searchString = null;
	private DataModel tableDataModel = null;
	private DataModel tablePagesDataModel = null;
	private Boolean backendSortable = null;
	private Boolean backendMoveable = null;
	private Boolean backendSearchable = null;
	private Boolean backendPageable = null;
	private VascEntryExport selectedExporter = null;
	private String selectedExporterAction = "null";
	private String selectedDirectPage = "null";
	private String selectedMultiRowAction = "null";
	private Map<String,Object> editSelectItemModels = null;
	
	public JSFVascEntrySupportBean(VascEntry entry) {
		if (entry==null) {
			throw new NullPointerException("Can't support null entry.");
		}
		this.logger = Logger.getLogger(JSFVascEntrySupportBean.class.getName());	
		this.entry=entry;
		
		// init
		tableDataModel = new ListDataModel();
		tablePagesDataModel = new ListDataModel();
		editSelectItemModels = new HashMap<String,Object>(6);
		
		// cache some values
		VascBackendCrud backend = entry.getVascFrontendController().getVascEntryState().getVascBackend();
		backendPageable = backend.isPageable();
		backendMoveable = backend.isRecordMoveable();
		backendSearchable = backend.isSearchable();
		backendSortable = backend.isSortable();
		
		
		i18nMap = new JSFVascSupportI18nMapController(entry);
		
		logger.finer("Support Bean created for vascEntry: "+entry.getId());
	}
	
	public VascEntry getVascEntry() {
		return entry;
	}
	
	public Map<String,String> getI18nMap() {
		return i18nMap;
	}
	
	public VascDataBackendBean getSelectedTableRecord() {
		Object selected = tableDataModel.getRowData();
		if (selected==null) {
			return null;
		}
		VascDataBackendBean r = (VascDataBackendBean) selected; 
		return r;
	}
	
	public String getParentSelectedDisplayName() {
		return entry.getVascFrontendController().getVascFrontendHelper().getParentSelectedDisplayName(entry);
	}
	
	public String getSelectedDisplayName() {
		return entry.getVascFrontendController().getVascFrontendHelper().getSelectedDisplayName(entry);
	}
	
	public int getTotalColumnCount() {
		int t = 0;
		t += getTotalFieldColumnCount();
		t += getTotalActionColumnCount();
		t += getTotalLinkColumnCount();
		return t;
	}
	
	public int getTotalFieldColumnCount() {
		int t = 0;
		if (getHasMultiRowActions()) {
			t++;	// auto add of select boxes
		}
		for (VascEntryField c:entry.getVascEntryFields()) {
			if (entry.getVascFrontendController().getVascFrontendDataSelector().isFieldList(c)==false) {
				continue;
			}
			t++;
		}
		return t;
	}
	
	public int getTotalActionColumnCount() {
		int t = 0;
		if (entry.getVascFrontendController().getVascEntryState().getVascBackend().isRecordMoveable()) {
			t++;
			t++;
		}
		t+=entry.getRowActions().size();
		return t;
	}
	
	public int getTotalLinkColumnCount() {
		return getVascLinkEntriesList().size();
	}
	
	public List<VascEntryLink> getVascLinkEntriesList() {
		return entry.getVascFrontendController().getVascFrontendHelper().getVascLinkEntryByType(entry,VascEntryLinkType.LIST);
	}
	public List<VascEntryLink> getVascLinkEntriesEditTab() {
		return entry.getVascFrontendController().getVascFrontendHelper().getVascLinkEntryByType(entry,VascEntryLinkType.EDIT_TAB);
	}
	public List<VascEntryLink> getVascLinkEntriesEditInline() {
		return entry.getVascFrontendController().getVascFrontendHelper().getVascLinkEntryByType(entry,VascEntryLinkType.EDIT_INLINE);
	}
	public List<VascEntryLink> getVascLinkEntriesEditTabParentState() {
		if (entry.getVascFrontendController().getVascEntryState().getParent()==null) {
			List<VascEntryLink> result = new ArrayList<VascEntryLink>(0);
			return result;
		}
		return entry.getVascFrontendController().getVascFrontendHelper().getVascLinkEntryByType(entry.getVascFrontendController().getVascEntryState().getParent().getVascEntry(),VascEntryLinkType.EDIT_TAB);
	}
	
	
	public long getPageTotalRecordCount() {
		return entry.getVascFrontendController().getVascFrontendPageInfo().getPageTotalRecordCount();
	}
	
	public long getPageStartCount() {
		return entry.getVascFrontendController().getVascFrontendPageInfo().getPageStartCount();
	}
	public long getPageStopCount() {
		return entry.getVascFrontendController().getVascFrontendPageInfo().getPageStopCount();
	}
	
	public List<SelectItem> getGlobalExportItems() {
		List<SelectItem> result = new ArrayList<SelectItem>(5);
		SelectItem s = new SelectItem();
		s.setLabel(i18nMap.get("generic.vasc.jsf.table.export.select"));
		s.setDescription(i18nMap.get("generic.vasc.jsf.table.export.select.alt"));
		s.setValue("null");
		result.add(s);
		
		for (GlobalVascAction a:entry.getExportActions()) {
			s = new SelectItem();
			s.setLabel(i18nMap.get(a.getName()));
			s.setDescription(i18nMap.get(a.getDescription()));
			s.setValue(a.getId());
			result.add(s);
		}
		return result;
	}
	
	public List<SelectItem> getDirectPageItems() {
		List<SelectItem> result = new ArrayList<SelectItem>(5);
		SelectItem s = new SelectItem();
		s.setLabel(i18nMap.get("generic.vasc.jsf.table.page.select"));
		s.setDescription(i18nMap.get("generic.vasc.jsf.table.page.select.alt"));
		s.setValue("null");
		result.add(s);
		
		int pageSize = getVascEntry().getVascFrontendController().getVascEntryState().getVascBackendState().getPageSize();
		for (int i=0;i<getTablePagesDataModel().getRowCount();i++) {
			getTablePagesDataModel().setRowIndex(i);
			VascFrontendPageInfoNumber page = (VascFrontendPageInfoNumber)getTablePagesDataModel().getRowData();
			s = new SelectItem();
			String name = i18nMap.get("generic.vasc.jsf.table.page.name");
			String description = i18nMap.get("generic.vasc.jsf.table.page.description");
			s.setLabel(name+page.getPageNumber()+"  "+(i*pageSize)+"-"+((i*pageSize)+pageSize));
			s.setDescription(description+page.getPageNumber());
			s.setValue(page.getPageNumber());
			result.add(s);
		}
		
		return result;
	}
	
	public boolean getHasOnlySinglePage() {
		return entry.getVascFrontendController().getVascFrontendPageInfo().getHasOnlySinglePage();
	}
	
	public boolean getHasExtendedPageMode() {
		return entry.getVascFrontendController().getVascFrontendPageInfo().getHasExtendedPageMode();
	}
	
	public boolean getHasExtendedPageModeCenter() {
		return entry.getVascFrontendController().getVascFrontendPageInfo().getHasExtendedPageModeCenter();
	}
	
	public List<VascFrontendPageInfoNumber> getTablePagesExtendedBegin() {
		return entry.getVascFrontendController().getVascFrontendPageInfo().getTablePagesExtendedBegin();
	}
	
	public List<VascFrontendPageInfoNumber> getTablePagesExtendedEnd() {
		return entry.getVascFrontendController().getVascFrontendPageInfo().getTablePagesExtendedEnd();
	}
	
	public List<VascFrontendPageInfoNumber> getTablePagesExtendedCenter() {
		return entry.getVascFrontendController().getVascFrontendPageInfo().getTablePagesExtendedCenter();
	}
	
	private String getComponentType(UIComponent comp) {
		if (comp instanceof HtmlCommandLink) {
			return ((HtmlCommandLink)comp).getType();
		}
		if (comp instanceof HtmlCommandButton) {
			return ((HtmlCommandButton)comp).getType();
		}
		throw new IllegalArgumentException("Component is of link of button type: "+comp);
	}
	
	public List<RowVascAction> getParentCustomRowActions() {
		List<RowVascAction> result = new ArrayList<RowVascAction>(5);
		VascEntry entry = getVascEntry();
		if (entry.getVascFrontendController().getVascEntryState().getParent()==null) {
			return result;
		}
		VascEntry parent = entry.getVascFrontendController().getVascEntryState().getParent().getVascEntry();
		for (RowVascAction action:parent.getRowActions()) {
			String actionId = action.getId();
			if (AddRowAction.ACTION_ID.equals(actionId)) {
				continue;
			}
			if (DeleteRowAction.ACTION_ID.equals(actionId)) {
				continue;
			}
			if (EditRowAction.ACTION_ID.equals(actionId)) {
				continue;
			}
			result.add(action);
		}
		return result;
	}
	
	public void parentCustomRowaction(ActionEvent event) {
		String actionIdString = getComponentType(event.getComponent());
		logger.fine("parentCustomRowaction: "+actionIdString);
		JSFVascUIComponent comp = JSFVascUIComponent.findVascParent(event.getComponent());
		VascEntry entry = comp.getVascEntry();
		if (entry.getVascFrontendController().getVascEntryState().getParent()==null) {
			return;
		}
		VascEntry parent = entry.getVascFrontendController().getVascEntryState().getParent().getVascEntry();
	
		RowVascAction action = parent.getRowActionById(actionIdString);
		Serializable parentSelected = entry.getVascFrontendController().getVascEntryState().getParent().getEntryDataObject();
		logger.fine("parentCustomRowaction do on: "+action+" parentSelection: "+parentSelected);
		try {
			action.doRowAction(parent,parentSelected);
		} catch (VascException e) {
			throw new VascFrontendException(e);
		}
		logger.fine("parentCustomRowaction DONE");
	}
	
	
	public boolean getHasMultiRowActions() {
		int size = getVascEntry().getVascFrontendController().getVascFrontendHelper().getMultiRowActions(getVascEntry()).size();
		if (size==0) {
			return false;
		}
		return true;
	}
	
	public List<SelectItem> getMultiRowActionItems() {
		List<SelectItem> result = new ArrayList<SelectItem>(5);
		SelectItem s = new SelectItem();
		s.setLabel(i18nMap.get("generic.vasc.jsf.multiAction.name"));
		s.setDescription(i18nMap.get("generic.vasc.jsf.multiAction.description"));
		s.setValue("null");
		result.add(s);
		
		for (RowVascAction a:getVascEntry().getVascFrontendController().getVascFrontendHelper().getMultiRowActions(getVascEntry())) {
			s = new SelectItem();
			s.setLabel(i18nMap.get(a.getName()));
			s.setDescription(i18nMap.get(a.getDescription()));
			s.setValue(a.getId());
			result.add(s);
		}
		return result;
	}
	
	public void processMultiRowActionChange(ValueChangeEvent event) {
		// first do normal global selection of action aka: globalAction(event); copyed:
		JSFVascUIComponent comp = JSFVascUIComponent.findVascParent(event.getComponent());
		VascEntry entry = comp.getVascEntry();
		String actionIdString = (String)event.getNewValue();
		logger.finer("multri row id "+actionIdString);
		
		// TODO: FIX this change listener is called before save action in row EDIT...
		if (actionIdString==null) {
			return;
		}
		
		// Default selection for fancy gui
		if ("null".equals(actionIdString)) {
			return;
		}
		
		logger.info("MultiRowAction: "+actionIdString);
		FacesContext.getCurrentInstance().getViewRoot().addPhaseListener(new MultiRowAction(entry,actionIdString));
	}
	
	class MultiRowAction implements PhaseListener,Serializable {
		private static final long serialVersionUID = 1L;
		VascEntry vascEntry;
		String actionId;
		public MultiRowAction(VascEntry vascEntry,String actionId) {
			this.vascEntry=vascEntry;
			this.actionId=actionId;
		}
		
		@Override
		public void beforePhase(PhaseEvent PhaseEvent) {
		}
		
		@Override
		public void afterPhase(PhaseEvent PhaseEvent) {
			if (vascEntry==null) {
				return;
			}
			RowVascAction action = vascEntry.getRowActionById(actionId);
			VascEntryState state = vascEntry.getVascFrontendController().getVascEntryState(); 
			
			try {
				setSelectedMultiRowAction("null"); // reset to selected ... value
				
				for (Integer rowId:state.getMultiActionSelection().keySet()) {
					Boolean value = state.getMultiActionSelection().get(rowId);
					logger.fine("multiRow selected: "+rowId+" value: "+value);
					if (value!=null && value==true) {
						Serializable row = state.getEntryDataList().get(rowId);
						logger.finer("row: "+row);
						action.doRowAction(vascEntry, row);
						if (action.getId().equals(DeleteRowAction.ACTION_ID)) {
							return;
						}
					}
				}
				state.getMultiActionSelection().clear(); // after down deselect all options
			} catch (Exception e) {
				throw new RuntimeException("Could not do multirowaction",e);
			} finally {
				vascEntry=null;
			}
		}
		
		@Override
		public PhaseId getPhaseId(){
			return PhaseId.UPDATE_MODEL_VALUES;
		}
	}
	
	
	// All Actions
		
	public void searchAction(ActionEvent event) {
		logger.fine("searchAction");
		JSFVascUIComponent comp = JSFVascUIComponent.findVascParent(event.getComponent());
		VascEntry entry = comp.getVascEntry();
		entry.getVascFrontendController().getVascFrontendActions().searchAction(searchString);
	}
	
	public void sortAction(ActionEvent event) {
		logger.fine("sortAction");
		String fieldIdString = ((HtmlCommandLink)event.getComponent()).getType();
		JSFVascUIComponent comp = JSFVascUIComponent.findVascParent(event.getComponent());
		VascEntry entry = comp.getVascEntry();
		VascEntryField field = entry.getVascEntryFieldById(fieldIdString);
		entry.getVascFrontendController().getVascFrontendActions().sortAction(field);
	}
	
	public boolean getRenderBackAction() {
		return getVascEntry().getVascFrontendController().getVascEntryState().getParent()!=null;
	}
	
	public boolean getRenderBackEditAction() {
		if (getVascEntry().getVascFrontendController().getVascEntryState().getParent()!=null) {
			if (getVascEntry().getVascFrontendController().getVascEntryState().getParent().getVascEntry().getRowActionById("editRowAction")==null) {
				return false; // parent is not editable
			}
			return true;
		}
		return false;
	}
	
	public void backAction(ActionEvent event) {
		logger.fine("backAction");
		JSFVascUIComponent comp = JSFVascUIComponent.findVascParent(event.getComponent());
		comp.initGoto(entry.getVascFrontendController().getVascEntryState().getParent());
		entry.getVascFrontendController().getVascFrontend().renderView();
	}
	
	public void backEditAction(ActionEvent event) {
		logger.fine("backEditAction");
		JSFVascUIComponent comp = JSFVascUIComponent.findVascParent(event.getComponent());
		VascEntry entry = getVascEntry();
		
		// select record to edit
		Serializable rowObject = entry.getVascFrontendController().getVascEntryState().getParent().getEntryDataObject();
		comp.initGoto(entry.getVascFrontendController().getVascEntryState().getParent(),rowObject);
	}
	
	
	public void pageAction(ActionEvent event) {
		logger.fine("pageAction");
		Integer pageIndex = new Integer(getComponentType(event.getComponent()));
		JSFVascUIComponent comp = JSFVascUIComponent.findVascParent(event.getComponent());
		VascEntry entry = comp.getVascEntry();
		selectedDirectPage=pageIndex+"";
		entry.getVascFrontendController().getVascFrontendActions().pageAction(pageIndex);
	}
	
	public void pageNextAction(ActionEvent event) {
		logger.fine("pageNextAction");
		JSFVascUIComponent comp = JSFVascUIComponent.findVascParent(event.getComponent());
		VascEntry entry = comp.getVascEntry();
		
		int pageIndex = entry.getVascFrontendController().getVascEntryState().getVascBackendState().getPageIndex();
		pageIndex++;
		selectedDirectPage=pageIndex+"";
		entry.getVascFrontendController().getVascFrontendActions().pageAction(pageIndex);
		entry.getVascFrontendController().getVascFrontend().renderView();
	}
	
	public void pagePreviousAction(ActionEvent event) {
		logger.fine("pagePreviousAction");
		JSFVascUIComponent comp = JSFVascUIComponent.findVascParent(event.getComponent());
		VascEntry entry = comp.getVascEntry();
		
		int pageIndex = entry.getVascFrontendController().getVascEntryState().getVascBackendState().getPageIndex();
		pageIndex--;
		selectedDirectPage=pageIndex+"";
		entry.getVascFrontendController().getVascFrontendActions().pageAction(pageIndex);
		entry.getVascFrontendController().getVascFrontend().renderView();
	}
	
	public boolean getHasPageNextAction() {
		return entry.getVascFrontendController().getVascFrontendPageInfo().getHasPageNextAction();
	}
	
	public boolean getHasPagePreviousAction() {
		return entry.getVascFrontendController().getVascFrontendPageInfo().getHasPagePreviousAction();
	}
	
	
	public void rowAction(ActionEvent event) {
		String actionIdString = getComponentType(event.getComponent());
		logger.fine("RowAction: "+actionIdString);
		JSFVascUIComponent comp = JSFVascUIComponent.findVascParent(event.getComponent());
		VascEntry entry = comp.getVascEntry();
		RowVascAction action = entry.getRowActionById(actionIdString);
		//Object selected = null;
		if (actionIdString.contains("add")==false) {
			selected = comp.getSupportBean().getSelectedTableRecord();
		} else {
			selected = null;
		}
		logger.fine("RowAction do on: "+action);
		try {
			if (selected==null) {
				action.doRowAction(entry, null);
			} else {
				action.doRowAction(entry, selected.getRecord());
			}
		} catch (VascException e) {
			throw new VascFrontendException(e);
		}
		logger.fine("RowAction DONE");
	}

	public void globalAction(ActionEvent event) {
		JSFVascUIComponent comp = JSFVascUIComponent.findVascParent(event.getComponent());
		VascEntry entry = comp.getVascEntry();
		String id = getComponentType(event.getComponent());
		logger.fine("globalAction id: "+id);
		GlobalVascAction action = entry.getGlobalActionById(id);
		try {
			action.doGlobalAction(entry);
		} catch (VascException e) {
			throw new VascFrontendException(e);
		}
	}
	
	
	public void addAction(ActionEvent event) {
		logger.fine("addAction");
		JSFVascUIComponent comp = JSFVascUIComponent.findVascParent(event.getComponent());
		VascEntry entry = comp.getVascEntry();
		RowVascAction action = entry.getRowActionById("addRowAction");
		try {
			action.doRowAction(entry, null);
		} catch (VascException e) {
			throw new VascFrontendException(e);
		}
	}
	
	public void linkAction(ActionEvent event) {
		logger.fine("linkAction");
		String linkId = getComponentType(event.getComponent());
		JSFVascUIComponent comp = JSFVascUIComponent.findVascParent(event.getComponent());
		VascDataBackendBean selected = comp.getSupportBean().getSelectedTableRecord();
		logger.finer("Set selected: "+selected);
		VascEntry entry = comp.getVascEntry();
		entry.getVascFrontendController().getVascEntryState().setEntryDataObject(selected.getRecord());
		VascEntryLink l = entry.getVascEntryLinkById(linkId);
		comp.initGoto(l);
		entry.getVascFrontendController().getVascFrontend().renderView();
		this.selected=selected; // renderView clears selected !
	}
	
	public void moveAction(ActionEvent event) {
		logger.fine("moveAction");
		String moveAction = getComponentType(event.getComponent());
		JSFVascUIComponent comp = JSFVascUIComponent.findVascParent(event.getComponent());
		Serializable selected = comp.getSupportBean().getSelectedTableRecord().getRecord();
		if ("up".equals(moveAction)) {
			entry.getVascFrontendController().getVascFrontendActions().moveUpAction(selected);	
		}
		if ("down".equals(moveAction)) {
			entry.getVascFrontendController().getVascFrontendActions().moveDownAction(selected);
		}
	}
	
	public void linkEditAction(ActionEvent event) {
		logger.fine("linkEditAction");
		String linkId = getComponentType(event.getComponent());
		JSFVascUIComponent comp = JSFVascUIComponent.findVascParent(event.getComponent());
		VascDataBackendBean selected = this.selected;
		logger.fine("Set selected: "+selected);
		VascEntry entry = comp.getVascEntry();
		VascEntryLink l = entry.getVascEntryLinkById(linkId);
		comp.initGoto(l);
		entry.getVascFrontendController().getVascFrontend().renderView();
		this.selected=selected; // renderView clears selected !
	}
	
	public void linkListAction(ActionEvent event) {
		logger.fine("linkListAction");
		String linkId = getComponentType(event.getComponent());
		JSFVascUIComponent comp = JSFVascUIComponent.findVascParent(event.getComponent());
		VascEntryLink link = comp.getVascEntry().getVascFrontendController().getVascEntryState().getParent().getVascEntry().getVascEntryLinkById(linkId);
		comp.initGoto(link,comp.getVascEntry().getVascFrontendController().getVascEntryState().getParent());
		
		Serializable o = comp.getVascEntry().getVascFrontendController().getVascEntryState().getParent().getEntryDataObject();
		int index = comp.getVascEntry().getVascFrontendController().getVascEntryState().getParent().getEntryDataList().indexOf(o);
		VascDataBackendBean selected = new VascDataBackendBean(entry,o,index);
		this.selected=selected;
		entry.getVascFrontendController().getVascFrontend().renderView();
		this.selected=selected;
	}
	
	public void cancelAction(ActionEvent event) {
		logger.fine("cancelAction");
		JSFVascUIComponent comp = JSFVascUIComponent.findVascParent(event.getComponent());
		VascEntry entry = comp.getVascEntry();
		if (getSelected()!=null) {
			getSelected().setRealValue(false);
		}
		entry.getVascFrontendController().getVascEntryState().setEntryDataObject(null);
		entry.getVascFrontendController().getVascFrontend().renderView();
	}
	
	public void saveAction(ActionEvent event) {
		logger.fine("saveAction");
		JSFVascUIComponent comp = JSFVascUIComponent.findVascParent(event.getComponent());
		VascEntry entry = comp.getVascEntry();
		logger.fine("Savng: "+selected.getRecord());
		
		/*
		try {
			for (Method m:selected.getRecord().getClass().getMethods()) {
				if (m.getName().startsWith("get")) {
					Object value = m.invoke(selected.getRecord(), new Object[0]);
					logger.info("property: "+m.getName()+" value: "+value);
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		*/
		
		entry.getVascFrontendController().getVascEntryState().setEntryDataObject(selected.getRecord());
		
		if (entry.getVascFrontendController().getVascEntryState().isEditCreate()) {
			entry.getVascFrontendController().getVascFrontendActions().persistObject();
		} else {
			entry.getVascFrontendController().getVascFrontendActions().mergeObject();
		}
		
		entry.getVascFrontendController().getVascFrontend().renderView();
	}
	
	public void deleteAction(ActionEvent event) {
		logger.fine("deleteAction");
		JSFVascUIComponent comp = JSFVascUIComponent.findVascParent(event.getComponent());
		VascEntry entry = comp.getVascEntry();
		VascEntryState state = entry.getVascFrontendController().getVascEntryState();

		List<Serializable> sels = new ArrayList<Serializable>(5);
		for (Integer rowId:state.getMultiActionSelection().keySet()) {
			Boolean value = state.getMultiActionSelection().get(rowId);
			logger.fine("multiRow delete: "+rowId+" value: "+value);
			if (value!=null && value==true) {
				Serializable row = state.getEntryDataList().get(rowId);
				sels.add(row);
			}
		}
		if (sels.isEmpty()==false) {
			for (Serializable row:sels) {
				entry.getVascFrontendController().getVascEntryState().setEntryDataObject(row);
				entry.getVascFrontendController().getVascFrontendActions().deleteObject();				
			}
			state.getMultiActionSelection().clear(); // after down deselect all options
		} else {
			entry.getVascFrontendController().getVascFrontendActions().deleteObject();	
		}
		
		entry.getVascFrontendController().getVascFrontend().renderView();
	}
	
	public void processDirectDownloadChange(ValueChangeEvent event){
		
		// first do normal global selection of action aka: globalAction(event); copyed:
		JSFVascUIComponent comp = JSFVascUIComponent.findVascParent(event.getComponent());
		VascEntry entry = comp.getVascEntry();
		
		String id = (String)event.getNewValue();
		
		// TODO: FIX this change listener is called before save action in row EDIT...
		if (id==null) {
			logger.finer("FIXME: unexcepted call to direct download");
			return;
		}
		
		// Default selection for fancy gui
		if ("null".equals(id)) {
			return;
		}
		
		logger.fine("exportDownloadAction id: "+id);
		
		GlobalVascAction action = entry.getExportActionById(id);
		try {
			action.doGlobalAction(entry);
		} catch (VascException e1) {
			throw new VascFrontendException(e1);
		}
		
		// restore normal view for next request.
		comp.setRenderFacetState("listView");
		VascEntryExport ex = getSelectedExporter();
		
		if (ex==null) {
			logger.fine("No exporter selected for download.");
			return;
		}
		selectedExporterAction = "null"; // reset selection to top one.
		
		FacesContext fc = FacesContext.getCurrentInstance();
		try {
			VascEntryExportWriter veew = ex.createExportWriter();
			veew.doInit(ex, entry);
			
			HttpServletResponse response = (HttpServletResponse)fc.getExternalContext().getResponse();
			String filename = "export-list."+veew.getFileType();
			response.setHeader("Content-disposition", "attachment; filename=" + filename);
			String contentType = veew.getMineType();
			response.setContentType(contentType);
			ServletOutputStream out = response.getOutputStream();
						
			veew.doExport(out);
			out.close();
		} catch (VascException e) {
			throw new VascFrontendException(e);
		} catch (IOException e) {
			throw new VascFrontendException(e);
		} finally {
			fc.responseComplete();
		}
	}
	
	
	public void processDirectPageChange(ValueChangeEvent event){
				
		// first do normal global selection of action aka: globalAction(event); copyed:
		JSFVascUIComponent comp = JSFVascUIComponent.findVascParent(event.getComponent());
		VascEntry entry = comp.getVascEntry();
		
		String id = (String)event.getNewValue();
		
		// TODO: FIX this change listener is called before save action in row EDIT...
		if (id==null) {
			logger.finer("FIXME: unexcepted call to direct page change");
			return;
		}
		
		// Default selection for fancy gui
		if ("null".equals(id)) {
			return;
		}
		
		logger.fine("directPageChangeAction id: "+id);
		//selectedDirectPage = "null";
		
		entry.getVascFrontendController().getVascFrontendActions().pageAction(new Integer(id));
		entry.getVascFrontendController().getVascFrontend().renderView();
	}
	
	// Pure get/setters 
	
	/**
	 * @return the searchString
	 */
	public String getSearchString() {
		return searchString;
	}

	/**
	 * @param searchString the searchString to set
	 */
	public void setSearchString(String searchString) {
		this.searchString = searchString;
	}
	
	/**
	 * @return the selected
	 */
	public VascDataBackendBean getSelected() {
		return selected;
	}

	/**
	 * @param selected the selected to set
	 */
	public void setSelected(VascDataBackendBean selected) {
		logger.fine("Set selected records: "+selected+" on: "+this);
		this.selected = selected;
	}

	/**
	 * @return the backendSortable
	 */
	public Boolean getBackendSortable() {
		return backendSortable;
	}

	/**
	 * @return the backendPageable
	 */
	public Boolean getBackendPageable() {
		return backendPageable;
	}

	/**
	 * @return the backendMoveable
	 */
	public Boolean getBackendMoveable() {
		return backendMoveable;
	}

	/**
	 * @return the backendSearchable
	 */
	public Boolean getBackendSearchable() {
		return backendSearchable;
	}

	/**
	 * @return the tableDataModel
	 */
	public DataModel getTableDataModel() {
		return tableDataModel;
	}

	/**
	 * @return the tablePagesDataModel
	 */
	public DataModel getTablePagesDataModel() {
		return tablePagesDataModel;
	}

	/**
	 * @return the selectedExporter
	 */
	public VascEntryExport getSelectedExporter() {
		return selectedExporter;
	}

	/**
	 * @param selectedExporter the selectedExporter to set
	 */
	public void setSelectedExporter(VascEntryExport selectedExporter) {
		this.selectedExporter = selectedExporter;
	}

	/**
	 * @return the selectedExporterAction
	 */
	public String getSelectedExporterAction() {
		return selectedExporterAction;
	}

	/**
	 * @param selectedExporterAction the selectedExporterAction to set
	 */
	public void setSelectedExporterAction(String selectedExporterAction) {
		this.selectedExporterAction = selectedExporterAction;
	}

	/**
	 * @return the selectedDirectPage
	 */
	public String getSelectedDirectPage() {
		return selectedDirectPage;
	}

	/**
	 * @param selectedDirectPage the selectedDirectPage to set
	 */
	public void setSelectedDirectPage(String selectedDirectPage) {
		this.selectedDirectPage = selectedDirectPage;
	}

	/**
	 * @return the editSelectItemModels
	 */
	public Map<String, Object> getEditSelectItemModels() {
		return editSelectItemModels;
	}

	/**
	 * @return the selectedMultiRowAction
	 */
	public String getSelectedMultiRowAction() {
		return selectedMultiRowAction;
	}

	/**
	 * @param selectedMultiRowAction the selectedMultiRowAction to set
	 */
	public void setSelectedMultiRowAction(String selectedMultiRowAction) {
		this.selectedMultiRowAction = selectedMultiRowAction;
	}

	/**
	 * @return the selectAllValue
	 */
	public Boolean getSelectAllValue() {
		return false;
	}

	/**
	 * @param selectAllValue the selectAllValue to set
	 */
	public void setSelectAllValue(Boolean selectAllValue) {
	}
}
class JSFVascSupportI18nMapController implements Map<String,String> {
	private VascEntry entry = null;
	public JSFVascSupportI18nMapController(VascEntry entry) {
		this.entry=entry;
	}
	
	@Override
	public void clear() {
	}
	@Override
	public boolean containsKey(Object key) {
		return true;
	}
	@Override
	public boolean containsValue(Object value) {
		return true;
	}
	@Override
	public Set<Map.Entry<String, String>> entrySet() {
		return null;
	}
	@Override
	public String get(Object key) {
		String result = entry.getVascFrontendController().getVascEntryResourceResolver().getTextValue((String)key);
		return result;
	}
	@Override
	public boolean isEmpty() {
		return false;
	}
	@Override
	public Set<String> keySet() {
		return null;
	}
	@Override
	public String put(String key, String value) {
		return null;
	}
	@SuppressWarnings("rawtypes")
	@Override
	public void putAll(Map m) {
	}
	@Override
	public String remove(Object key) {
		return null;
	}
	@Override
	public int size() {
		return 0;
	}
	@Override
	public Collection<String> values() {
		return null;
	}
}