/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.frontend.web.jsf;

import java.io.Serializable;
import java.util.logging.Logger;

import javax.faces.component.UIViewRoot;
import javax.faces.context.FacesContext;

import net.forwardfire.vasc.core.entry.VascEntryExport;
import net.forwardfire.vasc.frontend.AbstractVascFrontend;
import net.forwardfire.vasc.frontend.VascFrontendException;
import net.forwardfire.vasc.frontend.VascFrontendController;
import net.forwardfire.vasc.frontend.web.jsf.ui.JSFBoolean;
import net.forwardfire.vasc.frontend.web.jsf.ui.JSFLabel;
import net.forwardfire.vasc.frontend.web.jsf.ui.JSFList;
import net.forwardfire.vasc.frontend.web.jsf.ui.JSFText;
import net.forwardfire.vasc.frontend.web.jsf.ui.JSFTextArea;

/**
 * Given the jsf vasc renderer its own class
 * 
 *
 * @author Willem Cazander
 * @version 1.0 Nov 12, 2009
 */
public class JSFVascFrontendRenderer extends AbstractVascFrontend implements Serializable { 
	
	private static final long serialVersionUID = 1L;
	private Logger logger = null;
	
	public JSFVascFrontendRenderer() {
		logger = Logger.getLogger(JSFVascFrontendRenderer.class.getName());
	}
	
	/**
	 * @see net.forwardfire.vasc.frontend.VascFrontend#getFrontendType()
	 */
	@Override
	public String getFrontendType() {
		return "jsf";
	}
	
	@Override
	protected void addUiComponents() {
		VascFrontendController vfd = getVascEntry().getVascFrontendController();
		
		// required UI components
		vfd.putVascUIComponent(net.forwardfire.vasc.core.ui.VascUIComponent.VASC_LABEL,JSFLabel.class.getName());
		vfd.putVascUIComponent(net.forwardfire.vasc.core.ui.VascUIComponent.VASC_TEXT, JSFText.class.getName());
		vfd.putVascUIComponent(net.forwardfire.vasc.core.ui.VascUIComponent.VASC_LIST, JSFList.class.getName());
		//vfd.putVascUIComponent(net.forwardfire.vasc.core.ui.VascUIComponent.VASC_BUTTON, JSFButton.class.getName());
		
		// optional UI components
		vfd.putVascUIComponent(net.forwardfire.vasc.core.ui.VascUIComponent.VASC_BOOLEAN	, JSFBoolean.class.getName());
		//vfd.putVascUIComponent(net.forwardfire.vasc.core.ui.VascUIComponent.VASC_DATE		, JSFDate.class.getName());
		vfd.putVascUIComponent(net.forwardfire.vasc.core.ui.VascUIComponent.VASC_TEXTAREA	, JSFTextArea.class.getName());
		//vfd.putVascUIComponent(net.forwardfire.vasc.core.ui.VascUIComponent.VASC_COLOR	, JSFColorChooser.class.getName());

	}
	
	/**
	 * @see net.forwardfire.vasc.frontend.VascFrontend#renderDelete(java.lang.Object)
	 */
	@Override
	public void renderDelete() throws VascFrontendException {
		logger.finer("renderDelete");
		UIViewRoot viewRoot = FacesContext.getCurrentInstance().getViewRoot();
		JSFVascUIComponent comp = JSFVascUIComponent.findVascChild(viewRoot,getVascEntry().getId());
		comp.setRenderFacetState("deleteView");
	}
	
	/**
	 * @see net.forwardfire.vasc.frontend.VascFrontend#renderEdit(java.lang.Object)
	 */
	@Override
	public void renderEdit() throws VascFrontendException {
		logger.finer("renderEdit");
		UIViewRoot viewRoot = FacesContext.getCurrentInstance().getViewRoot();
		JSFVascUIComponent comp = JSFVascUIComponent.findVascChild(viewRoot,getVascEntry().getId());
		comp.setRenderFacetState("editView");		
		entry.getVascFrontendController().getVascFrontendHelper().editReadOnlyUIComponents(entry);
		
		VascDataBackendBean selBean = null;
		if (entry.getVascFrontendController().getVascEntryState().isEditCreate()) {
			int index = entry.getVascFrontendController().getVascEntryState().getEntryDataList().size()+1;
			selBean = new VascDataBackendBean(entry,entry.getVascFrontendController().getVascEntryState().getEntryDataObject(),index);	
		} else {
			selBean = comp.getSupportBean().getSelectedTableRecord();
		}
		selBean.setRealValue(true);
		
		comp.getSupportBean().setSelected(selBean);
	}
	
	/**
	 * @see net.forwardfire.vasc.frontend.VascFrontend#renderExport(net.forwardfire.vasc.core.entry.VascEntryExport)
	 */
	@Override
	public void renderExport(VascEntryExport exporter) throws VascFrontendException {
		logger.finer("renderExport");
		UIViewRoot viewRoot = FacesContext.getCurrentInstance().getViewRoot();
		JSFVascUIComponent comp = JSFVascUIComponent.findVascChild(viewRoot,getVascEntry().getId());
		comp.setRenderFacetState("exportView");
		comp.getSupportBean().setSelectedExporter(exporter);
	}
	
	/**
	 * @see net.forwardfire.vasc.frontend.VascFrontend#renderView()
	 */
	@Override
	public void renderView() throws VascFrontendException {
		logger.finer("renderView");
		UIViewRoot viewRoot = FacesContext.getCurrentInstance().getViewRoot();
		JSFVascUIComponent comp = JSFVascUIComponent.findVascChild(viewRoot,getVascEntry().getId());
		comp.setRenderFacetState("listView");
	}
}
