/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.frontend.web.jsf;

import java.io.IOException;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Logger;

import javax.el.ValueExpression;
import javax.faces.component.UIComponent;
import javax.faces.component.UIComponentBase;
import javax.faces.context.FacesContext;

import net.forwardfire.vasc.backend.VascBackendException;
import net.forwardfire.vasc.core.VascController;
import net.forwardfire.vasc.core.VascEntry;
import net.forwardfire.vasc.core.VascEntryField;
import net.forwardfire.vasc.core.VascEntryLocal;
import net.forwardfire.vasc.core.VascEntryState;
import net.forwardfire.vasc.core.VascEntryLink;
import net.forwardfire.vasc.core.entry.VascEntryFrontendEventListener;
import net.forwardfire.vasc.core.entry.VascEntryFrontendEventListener.VascFrontendEventType;
import net.forwardfire.vasc.core.VascException;
import net.forwardfire.vasc.frontend.VascFrontendController;
import net.forwardfire.vasc.frontend.VascFrontendException;
import net.forwardfire.vasc.frontend.web.jsf.ui.JSFListModel;

/**
 * Renders an JSF vasc entry views.
 * 
 * This is a already less hacky this is second incarnation of this bean
 *
 * @author Willem Cazander
 * @version 1.0 Nov 16, 2008
 */
public class JSFVascUIComponent extends UIComponentBase {
	
	public static final String FAMILY = "vasc.jsf.component.family";
	public static final String ENTRY_SUPPORT_VAR_KEY = "entrySupportVar";
	public static final String TABLE_RECORD_VAR_KEY = "tableRecordVar";
	public static final String INJECT_EDIT_FIELDS_ID = "injectEditFieldsId";
	public static final String INJECT_TABLE_OPTIONS_ID = "injectTableOptionsId";
	public static final String INJECT_TABLE_COLUMNS_ID = "injectTableColumnsId";
	public static final String DISABLE_LINK_COLUMNS = "disableLinkColumns";
	public static final String INJECT_TABLE_OPTIONS_ON_CHANGE = "injectTableOptionOnChange";
	
	private JSFVascFrontendRenderer renderer = null;
	private JSFVascEntrySupportBean supportBean = null;
	private String renderFacetState = null;
	private VascEntryLink link = null;
	private VascEntryState linkState = null;
	private VascEntryState state = null;
	private Serializable stateEditId = null;
	private Logger logger = null;
	private Boolean initClear = null;
	private String injectTableOptionOnChange = null;
	
	public JSFVascUIComponent() {
		logger = Logger.getLogger(JSFVascUIComponent.class.getName());
	}
	
	@Override
	public String getFamily() {
		return FAMILY;
	}
	
	@Override
	public Object saveState(FacesContext facesContext) {
		logger.fine("Save State");
		Object values[] = new Object[5];
		values[0] = super.saveState(facesContext);
		values[1] = renderer;
		values[2] = supportBean;
		values[3] = renderFacetState;
		values[4] = this.getAttributes().get(INJECT_TABLE_OPTIONS_ON_CHANGE);
		return values;
	}

	@Override
	public void restoreState(FacesContext facesContext, Object state) {
		logger.fine("Resotre State");
		Object values[] = (Object[])state;
		super.restoreState(facesContext,					values[0]);
		renderer = (JSFVascFrontendRenderer)				values[1];
		supportBean = (JSFVascEntrySupportBean)				values[2];
		renderFacetState = (String)							values[3];
		injectTableOptionOnChange = (String)				values[4];

		// TODO: check if we can move this some day...
		String entrySupportVar = (String)getAttributes().get(JSFVascUIComponent.ENTRY_SUPPORT_VAR_KEY);		
		ValueExpression ve2 = FacesContext.getCurrentInstance().getApplication().getExpressionFactory().createValueExpression(FacesContext.getCurrentInstance().getELContext(), "#{"+entrySupportVar+"}", Object.class);
		ve2.setValue(FacesContext.getCurrentInstance().getELContext(), getSupportBean());
	}
	
	static public JSFVascUIComponent findVascParent(UIComponent comp) {
		if (comp==null) {
			return null;
		}
		if (comp instanceof JSFVascUIComponent) {
			return (JSFVascUIComponent)comp;
		}
		return findVascParent(comp.getParent());
	}
	
	static public JSFVascUIComponent findVascChild(UIComponent comp,String entryId) {
		if (comp==null) {
			return null;
		}
		if (comp instanceof JSFVascUIComponent) {
			JSFVascUIComponent ui = (JSFVascUIComponent)comp;
			if (entryId==null) {
				return ui;
			}
			if (entryId.equals(ui.getVascEntry().getId())) {
				return ui;
			}
			return null; // this is a other entry on this view
		}
		for (UIComponent c:comp.getChildren()) {
			Object res = findVascChild(c,entryId);
			if (res!=null) {
				return (JSFVascUIComponent)res; // found
			}
		}
		return null;
	}
	
	/**
	  * Finds component with the given id
	  */
	static public UIComponent findComponentById(UIComponent c, String id) {
		if (id.equals(c.getId())) {
			return c;
		}
		Iterator<UIComponent> kids = c.getFacetsAndChildren();
		while (kids.hasNext()) {
			UIComponent found = findComponentById(kids.next(), id);
			if (found != null) {
				return found;
			}
		}
		return null;
	}
	
	public VascEntry getVascEntry() {
		return renderer.getVascEntry();
	}
	
	protected void initGoto(VascEntryLink link) {
		logger.fine("init goto "+link);
		this.link=link;
	}
	protected void initGoto(VascEntryLink link,VascEntryState state) {
		logger.fine("init goto link: "+link);
		this.link=link;
		this.linkState=state;
	}
	
	protected void initGoto(VascEntryState state) {
		logger.fine("init goto "+state);
		this.state=state;
	}
	
	protected void initGoto(VascEntryState state,Serializable stateEditId) {
		logger.fine("init goto "+state);
		this.state=state;
		this.stateEditId=stateEditId;
	}
	
	public Boolean getInitClear() {
		return initClear;
	}
	
	@Override
	public void encodeBegin(FacesContext context) throws IOException {
		logger.fine("Comp encodeBegin link: "+link);
		// no need to add multiple
		if (renderer==null) {
			logger.finer("Adding phase listener: JSFVascValidatePhaseListener");
			context.getViewRoot().addPhaseListener(new JSFVascValidatePhaseListener());
		}	
		boolean init = false;
		if (renderer==null | link!=null | state!=null) {
			renderFacetState = "listView";
			VascEntry entry = createClonedVascEntry();
			renderer = new JSFVascFrontendRenderer();
			try {
				renderer.initEntry(entry);
			} catch (Exception e) {
				throw new RuntimeException("Could not initEntry: "+e.getMessage(),e);
			}
			supportBean = new JSFVascEntrySupportBean(entry);
			// copy search string for preadded search string from application layer
			supportBean.setSearchString(entry.getVascFrontendController().getVascEntryState().getVascBackendState().getSearchString());
			
			init = true;
		}
		
		if (stateEditId!=null) {
			Serializable rowObject = stateEditId;
			
			// setup for renderEdit() of frontend
			List<Object> list = new ArrayList<Object>(1);
			list.add(new VascDataBackendBean(supportBean.getVascEntry(),rowObject,0));
			supportBean.getTableDataModel().setWrappedData(list);
			supportBean.getTableDataModel().setRowIndex(0);
			
			// edit action copyed
			try {
				VascEntry entry = supportBean.getVascEntry();
				entry.getVascFrontendController().getVascEntryState().setEditCreate(false);
				entry.getVascFrontendController().fireVascFrontendEvent(entry, VascFrontendEventType.SELECT, rowObject);
				entry.getVascFrontendController().getVascEntryState().setEntryDataObject(rowObject);
				entry.getVascFrontendController().getVascFrontend().renderEdit();
			} catch (Exception e) {
				throw new RuntimeException("Could not renderEdit: "+e.getMessage(),e);
			}	
		}
		
		// refresh all drop down lists..
		for (String key:supportBean.getEditSelectItemModels().keySet()) {
			Object o = supportBean.getEditSelectItemModels().get(key);
			if (o instanceof JSFListModel) {
				JSFListModel t = (JSFListModel)o;
				t.requestRefresh();
			}
		}
		
		// set the support bean
		String entrySupportVar = (String)getAttributes().get(JSFVascUIComponent.ENTRY_SUPPORT_VAR_KEY);		
		ValueExpression ve2 = FacesContext.getCurrentInstance().getApplication().getExpressionFactory().createValueExpression(FacesContext.getCurrentInstance().getELContext(), "#{"+entrySupportVar+"}", Object.class);
		ve2.setValue(FacesContext.getCurrentInstance().getELContext(), getSupportBean());
		
		// set init for component renderer
		if (init) {
			initClear = false;
			if (link!=null | state!=null) {
				initClear = true;	
			}
		}
		super.encodeBegin(context);
	}
	
	public UIComponent getCurrentView() {
		UIComponent result = getFacet(renderFacetState);
		if (result==null) {
			throw new IllegalArgumentException("Could not get facet: "+renderFacetState);
		}
		return result;
	}
	
	public JSFVascEntrySupportBean getSupportBean() {
		return supportBean;
	}
	
	public void setRenderFacetState(String renderFacetState) {
		this.renderFacetState=renderFacetState;
	}
	
	public String getRenderFacetState() {
		return renderFacetState;
	}
	
	/**
	 * @return the injectTableOptionOnChange
	 */
	public String getInjectTableOptionOnChange() {
		return injectTableOptionOnChange;
	}
	
	/**
	 * @param injectTableOptionOnChange the injectTableOptionOnChange to set
	 */
	public void setInjectTableOptionOnChange(String injectTableOptionOnChange) {
		this.injectTableOptionOnChange = injectTableOptionOnChange;
	}
	
	public VascEntry createClonedVascEntry() {
		VascController vascController = (VascController)FacesContext.getCurrentInstance().getExternalContext().getRequestMap().get(VascRequestFacesFilter.REQ_VASC_CONTOLLER);
		String entryId = (String)FacesContext.getCurrentInstance().getExternalContext().getRequestMap().get(VascRequestFacesFilter.REQ_VASC_ENTRY_ID);
		
		if (link!=null) {
			entryId = link.getVascEntryId();
		}
		if (state!=null) {
			entryId = state.getVascEntry().getId();
		}
		
		VascEntryLocal entry = (VascEntryLocal)vascController.getVascEntryController().getVascEntryById(entryId);
		if (entry==null) {
			throw new NullPointerException("Could not get '"+entryId+"' from : "+vascController);
		}
		try {
			vascController.getVascEntryConfigController().configVascFrontendController(vascController, entry);
		} catch (Exception e1) {
			throw new RuntimeException(e1);
		} 
		VascFrontendController frontendData = entry.getVascFrontendController();
		try {
			frontendData.initFrontendListeners(entry,"jsf");
		} catch (Exception e1) {
			e1.printStackTrace();
		}
		
		if (state!=null) {
			// copy prevois parent 
			frontendData.getVascEntryState().setParent(state.getParent());
		}
		
		if (link!=null) {
			// save state
			if (linkState==null) {
				frontendData.getVascEntryState().setParent(getVascEntry().getVascFrontendController().getVascEntryState());
			} else {
				frontendData.getVascEntryState().setParent(linkState);
			}
			
			// Set parameters
			try {
				Serializable selected = getSupportBean().getSelected().getRecord();
				for (String parameterName:link.getEntryParameterFieldIdKeys()) {
					String fieldId = link.getEntryParameterFieldId(parameterName);
					VascEntryField v = getVascEntry().getVascEntryFieldById(fieldId);
					if (v==null) {
						logger.warning("Could nog get VascEntryField for fieldID: "+fieldId);
						continue;
					}
					if (v.getVascEntryFieldValue()==null) {
						logger.warning("Could not get VascEntryFieldValue for fieldID: "+fieldId);
						continue;
					}
					Object selectedValue = v.getVascEntryFieldValue().getValue(v.getBackendName(), selected);
				
					// set data parameter on new vasc entry
					//was; entry.getVascFrontendController().getVascEntryState().getVascBackendState().setDataParameter(parameterName, selectedValue);
					entry.setEntryParameter(parameterName, selectedValue);
					logger.fine("Setting link parameter: "+parameterName+" with: "+selectedValue);
				}
				
				for (String fieldId:link.getEntryCreateFieldValueKeys()) {
					String selectedfieldId = link.getEntryParameterFieldId(fieldId);
					Serializable selectedValue = selected;
					if (selectedfieldId!=null) { 
						VascEntryField v = getVascEntry().getVascEntryFieldById(selectedfieldId);
						selectedValue = v.getVascEntryFieldValue().getValue(v.getBackendName(), selected);
					}
					
					// create listener for new objects
					entry.getVascFrontendController().addVascEntryFrontendEventListener(new CreateEntryFieldValuesListener2(fieldId,selectedValue));
				}
			} catch (Exception e) {
				throw new RuntimeException("error: "+e.getMessage(),e);
			}
		}
		
		
		String entrySupportVar = (String)getAttributes().get(JSFVascUIComponent.ENTRY_SUPPORT_VAR_KEY);	
		entry.getVascFrontendController().addVascEntryFrontendEventListener(new JSFVascEntryEventListener(entrySupportVar));
		
		return entry;
	}
	
	class CreateEntryFieldValuesListener2 implements VascEntryFrontendEventListener {
		private static final long serialVersionUID = 1L;
		private String fieldId = null;
		private Serializable value = null;
		public CreateEntryFieldValuesListener2(String fieldId,Serializable value) {
			if (fieldId==null) {
				throw new NullPointerException("fieldId may not be null");
			}
			this.fieldId=fieldId;
			this.value=value;
		}
		
		@Override
		public VascFrontendEventType[] getEventTypes() {
			VascFrontendEventType[] result = {VascEntryFrontendEventListener.VascFrontendEventType.PRE_CREATE};
			return result;
		}
		
		@Override
		public void vascEvent(VascEntry entry,Serializable data) {
			VascEntryField field = entry.getVascEntryFieldById(fieldId);
			try {
				field.getVascEntryFieldValue().setValue(field.getBackendName(), data, value);
			} catch (VascBackendException e) {
				throw new VascFrontendException(e);
			}
		}
	}	
}
