/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.frontend.web.jsf;

import java.io.IOException;
import java.io.Serializable;
import java.lang.reflect.Constructor;
import java.util.List;
import java.util.logging.Logger;

import javax.el.ELContext;
import javax.el.MethodExpression;
import javax.el.ValueExpression;
import javax.faces.FacesException;
import javax.faces.application.Application;
import javax.faces.application.FacesMessage;
import javax.faces.component.UIColumn;
import javax.faces.component.UIComponent;
import javax.faces.component.UIInput;
import javax.faces.component.UIOutput;
import javax.faces.component.UIViewRoot;
import javax.faces.component.html.HtmlCommandLink;
import javax.faces.component.html.HtmlInputText;
import javax.faces.component.html.HtmlInputTextarea;
import javax.faces.component.html.HtmlMessage;
import javax.faces.component.html.HtmlOutputText;
import javax.faces.component.html.HtmlSelectBooleanCheckbox;
import javax.faces.component.html.HtmlSelectManyListbox;
import javax.faces.component.html.HtmlSelectManyMenu;
import javax.faces.component.html.HtmlSelectOneListbox;
import javax.faces.component.html.HtmlSelectOneMenu;
import javax.faces.component.html.HtmlSelectOneRadio;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.event.AbortProcessingException;
import javax.faces.event.ActionEvent;
import javax.faces.event.MethodExpressionActionListener;
import javax.faces.event.ValueChangeEvent;
import javax.faces.event.ValueChangeListener;
import javax.faces.render.Renderer;
import javax.faces.validator.Validator;
import javax.faces.validator.ValidatorException;

import net.forwardfire.vasc.core.VascEntry;
import net.forwardfire.vasc.core.VascEntryField;
import net.forwardfire.vasc.core.VascEntryListOption;
import net.forwardfire.vasc.core.VascEntryLink;
import net.forwardfire.vasc.core.VascEntryLinkType;
import net.forwardfire.vasc.core.actions.RowVascAction;
import net.forwardfire.vasc.core.ui.VascOptionValueModelListener;
import net.forwardfire.vasc.core.ui.VascUIComponent;
import net.forwardfire.vasc.core.ui.VascValueModel;
import net.forwardfire.vasc.core.VascException;
import net.forwardfire.vasc.impl.actions.AddRowAction;



/**
 * Renders an JSF vasc entry views.
 * 
 * This is a bit hacky because I'm not a JSF guro.
 *
 * @author Willem Cazander
 * @version 1.0 Nov 16, 2008
 */
public class JSFVascUIComponentRenderer extends Renderer {
	
	private Logger logger = null;
	
	public JSFVascUIComponentRenderer() {
		logger = Logger.getLogger(JSFVascUIComponentRenderer.class.getName());
	}
	
	@Override
    public void encodeBegin(FacesContext facesContext,UIComponent component) throws IOException {
		super.encodeBegin(facesContext, component);
		ResponseWriter writer = facesContext.getResponseWriter();
		JSFVascUIComponent comp = (JSFVascUIComponent)component;
		logger.fine("renderen encodeBegin for: "+comp.getRenderFacetState());
		writer.startElement("div", component);
		//String styleClass = (String)attributes.get(Shuffler.STYLECLASS_ATTRIBUTE_KEY);
		//writer.writeAttribute("class", styleClass, null);

		// check if injection is needed
		if (comp.getInitClear()!=null) {
			injectAll(facesContext,comp,comp.getInitClear());
			
			// hack for edit back action to set field disabled
			VascEntry entry = comp.getVascEntry();
			entry.getVascFrontendController().getVascFrontendHelper().editReadOnlyUIComponents(entry);
		}
		
		// render the current facet of the vasc component
		UIComponent view = comp.getCurrentView();
		view.encodeAll(facesContext);
	}

	@Override
	public void encodeEnd(FacesContext facesContext,UIComponent component) throws IOException {
		ResponseWriter writer = facesContext.getResponseWriter();
		writer.endElement("div");
	}
	
	// ========== All private
	
	private void injectAll(FacesContext context,JSFVascUIComponent comp,boolean clean) {
		
		String injectEditFieldsId = (String)comp.getAttributes().get(JSFVascUIComponent.INJECT_EDIT_FIELDS_ID);		
		String injectTableOptionsId = (String)comp.getAttributes().get(JSFVascUIComponent.INJECT_TABLE_OPTIONS_ID);		
		String injectTableColumnsId = (String)comp.getAttributes().get(JSFVascUIComponent.INJECT_TABLE_COLUMNS_ID);		
		
		UIComponent injectEditFieldsComponent = JSFVascUIComponent.findComponentById(comp.getFacet("editView"),injectEditFieldsId);
		UIComponent injectTableOptionsComponent = JSFVascUIComponent.findComponentById(comp.getFacet("listView"),injectTableOptionsId);
		UIComponent injectTableColumnsComponent = JSFVascUIComponent.findComponentById(comp.getFacet("listView"),injectTableColumnsId);
		if (injectEditFieldsComponent==null) {
			throw new NullPointerException("Could not find injectEditFieldsId: "+injectEditFieldsId);
		}
		if (injectTableOptionsComponent==null) {
			throw new NullPointerException("Could not find injectTableOptionsId: "+injectTableOptionsId);
		}
		if (injectTableColumnsComponent==null) {
			throw new NullPointerException("Could not find injectTableColumnsId: "+injectTableColumnsId);
		}
		
		if (clean) {
			logger.finer("Cleaning of all dynamic JSF components.");
			injectEditFieldsComponent.getChildren().clear();
			injectTableOptionsComponent.getChildren().clear();
			injectTableColumnsComponent.getChildren().clear();
		}
		logger.finer("Injection of all dynamic JSF components.");
		try {
			addEditFields(context,  injectEditFieldsComponent);
			addTableOptions(context, injectTableOptionsComponent,comp);
			addColumns(context,comp,injectTableColumnsComponent );
		} catch (Exception e) {
			throw new RuntimeException("Error while injecting; "+e.getMessage(),e);
		}
	}
	
	private String i18n(VascEntry entry,String key,Object...params) {
		return entry.getVascFrontendController().getVascEntryResourceResolver().getTextValue(key,params);
	}
	
	
	private void addEditFields(FacesContext fc,UIComponent grid) throws FacesException, VascException {
		
		Application application = fc.getApplication();
		UIViewRoot viewRoot = fc.getViewRoot();
		
		JSFVascUIComponent comp = JSFVascUIComponent.findVascParent(grid);
		VascEntry entry = comp.getVascEntry();
		if (entry==null) {
			throw new NullPointerException("No parent vasc component found.");
		}
		
		String entrySupportVar = (String)comp.getAttributes().get(JSFVascUIComponent.ENTRY_SUPPORT_VAR_KEY);		
		for (VascEntryField c:entry.getVascEntryFields()) {
			if (entry.getVascFrontendController().getVascFrontendDataSelector().isFieldEdit(c)==false) {
				continue;
			}
			for (int i=0;i<c.getVascEntryFieldType().getUIComponentCount(c);i++) {
				
				VascUIComponent label = c.getVascEntryFieldType().provideLabelUIComponent(i,c);
				VascValueModel model = new VascValueModel();
				model.setValue(i18n(entry,c.getName()));
				label.createComponent(entry,c,model,grid);
				
				VascUIComponent editor = c.getVascEntryFieldType().provideEditorUIComponent(i,c);
				model = new VascValueModel(c.getVascEntryFieldType().provideEditorVascValueModel(i,c));
				//model.setValue(c.getVascEntryFieldValue().getValue(c, bean));
				//model.addListener(new VascColumnValueModelListener(c,bean));
				UIInput jsfEdit = (UIInput)editor.createComponent(entry,c,model,grid);
				jsfEdit.addValidator(new VascJSFInputValidator2(c.getId()));
				
				int index = VascDataBackendBean.getIndexId(c);
				
				ValueExpression ve7 = FacesContext.getCurrentInstance().getApplication().getExpressionFactory().createValueExpression(FacesContext.getCurrentInstance().getELContext(), "#{"+entrySupportVar+".selected.field"+index+"}", Object.class);	
				jsfEdit.setValueExpression("value", ve7);
				
				ValueExpression ve8 = FacesContext.getCurrentInstance().getApplication().getExpressionFactory().createValueExpression(FacesContext.getCurrentInstance().getELContext(), "#{"+entrySupportVar+".selected != null}", Boolean.class);	
				jsfEdit.setValueExpression("rendered", ve8);
				
				HtmlMessage message = (HtmlMessage)application.createComponent(HtmlMessage.COMPONENT_TYPE);
				message.setId(viewRoot.createUniqueId());
				message.setFor(jsfEdit.getId());
				
				message.setErrorClass("style_textBlue");
				message.setFatalClass("style_textRed");
				message.setInfoClass("style_textBlack");
				message.setWarnClass("style_textGreen");
				
				grid.getChildren().add(message);
				
				//column++;
				// i==0 is for multi field editor support... which is still very in progress
				if (i==0) {
					entry.getVascFrontendController().addFieldVascUIComponents(c, editor,jsfEdit);
				}
			}
		}
	}
	
	class VascJSFInputValidator2 implements Validator,Serializable {
		private static final long serialVersionUID = -5715250529474737642L;
		private Logger logger = null;
		private String fieldId = null;
		
		public VascJSFInputValidator2(String fieldId) {
			this.fieldId=fieldId;
			logger = Logger.getLogger(VascJSFInputValidator2.class.getName());
		}
		
		@Override
		public void validate(FacesContext context, UIComponent component,Object object) throws ValidatorException {
			// always oke, we are runned by phase listener
		}
		
		public void validatePhase(FacesContext context, UIComponent component,Object object) throws ValidatorException {
			JSFVascUIComponent comp = JSFVascUIComponent.findVascParent(component);
			VascEntry entry = comp.getVascEntry();
			StringBuffer buf;
			try {
			VascEntryField field = entry.getVascEntryFieldById(fieldId);
			
			int index = VascDataBackendBean.getIndexId(field);
			String entrySupportVar = (String)comp.getAttributes().get(JSFVascUIComponent.ENTRY_SUPPORT_VAR_KEY);
			logger.fine("sup-sel: "+comp.getSupportBean().getSelected()+" index: "+index);
			
			// note we have to set the value to the vasc backend before we can run validation
			ValueExpression ve7 = FacesContext.getCurrentInstance().getApplication().getExpressionFactory().createValueExpression(FacesContext.getCurrentInstance().getELContext(), "#{"+entrySupportVar+".selected.field"+index+"}", Object.class);	
			ve7.setValue(FacesContext.getCurrentInstance().getELContext(), object);
			
			List<String> errors = entry.getVascFrontendController().getVascFrontendHelper().validateObjectField(field);
			logger.fine("Validate: "+component+" errors: "+errors.size()+" value: "+object);
			if (errors.isEmpty()) {
				return; // no errors
			}
			
			buf = new StringBuffer(200);
			for (String err:errors) {
				buf.append(err);
				buf.append('\n');
			}
			} catch (Exception e) {
				e.printStackTrace();
				return;
			}
			
			FacesMessage message = new FacesMessage(buf.toString());
			message.setSeverity(FacesMessage.SEVERITY_ERROR);
			throw new ValidatorException(message);
		}
	}
	
	private void addTableOptions(FacesContext fc,UIComponent grid,JSFVascUIComponent comp) throws FacesException, VascException {
		VascEntry entry = comp.getVascEntry();
		for (VascEntryListOption option:entry.getVascEntryListOptions()) {
			for (int i=0;i<option.getVascEntryFieldType().getUIComponentCount(option);i++) {
				
				VascUIComponent label = option.getVascEntryFieldType().provideLabelUIComponent(i,option);
				VascValueModel model = new VascValueModel();
				model.setValue(i18n(entry,option.getName()));
				label.createComponent(entry,option,model,grid);
				
				VascUIComponent editor = option.getVascEntryFieldType().provideEditorUIComponent(i,option);
				model = new VascValueModel(option.getVascEntryFieldType().provideEditorVascValueModel(i,option));
				model.addListener(new VascOptionValueModelListener(option));
				model.setValue(option.getDefaultValue());
				UIInput jsfEdit = (UIInput)editor.createComponent(entry,option,model,grid);
				jsfEdit.addValueChangeListener(new ModelChangeListener(model));
				
				if (comp.getInjectTableOptionOnChange()!=null && comp.getInjectTableOptionOnChange().isEmpty()==false) {
					if (jsfEdit instanceof HtmlSelectBooleanCheckbox) {
						((HtmlSelectBooleanCheckbox)jsfEdit).setOnchange(comp.getInjectTableOptionOnChange());
					}
					if (jsfEdit instanceof HtmlSelectManyMenu) {
						((HtmlSelectManyMenu)jsfEdit).setOnchange(comp.getInjectTableOptionOnChange());
					}
					if (jsfEdit instanceof HtmlInputText) {
						((HtmlInputText)jsfEdit).setOnchange(comp.getInjectTableOptionOnChange());
					}
					if (jsfEdit instanceof HtmlInputTextarea) {
						((HtmlInputTextarea)jsfEdit).setOnchange(comp.getInjectTableOptionOnChange());
					}
					if (jsfEdit instanceof HtmlSelectManyListbox) {
						((HtmlSelectManyListbox)jsfEdit).setOnchange(comp.getInjectTableOptionOnChange());
					}
					if (jsfEdit instanceof HtmlSelectOneMenu) {
						((HtmlSelectOneMenu)jsfEdit).setOnchange(comp.getInjectTableOptionOnChange());
					}
					if (jsfEdit instanceof HtmlSelectOneRadio) {
						((HtmlSelectOneRadio)jsfEdit).setOnchange(comp.getInjectTableOptionOnChange());
					}
					if (jsfEdit instanceof HtmlSelectOneListbox) {
						((HtmlSelectOneListbox)jsfEdit).setOnchange(comp.getInjectTableOptionOnChange());
					}
				}
				
				Class<?> clazz = option.getVascEntryFieldType().getAutoDetectClass();
				if (clazz!=null && model.getValue()!=null) {
					if (clazz.equals(model.getValue().getClass())==false) {
						// lets try setting correct default.
						try {
							Constructor<?> c = clazz.getConstructor(String.class);
							Object value = c.newInstance(model.getValue());
							jsfEdit.setValue(value);
						} catch (Exception e) {
							throw new VascException(e);
						}
					} else {
						jsfEdit.setValue(model.getValue()); // set default value
					}
				}
				// i==0 is for multi field editor support... which is stell very in progress aka not working
				if (i==0) {
					entry.getVascFrontendController().addFieldVascUIComponents(option, editor,jsfEdit);
				}
			}
		}
		entry.getVascFrontendController().getVascFrontendHelper().headerOptionsCreatedFillData(entry);
	}
	
	public class ModelChangeListener implements ValueChangeListener,Serializable {
		private static final long serialVersionUID = 1L;
		private final VascValueModel model;
		
		public ModelChangeListener(VascValueModel  model) {
			this.model=model;
		}
		
		@Override
		public void processValueChange(ValueChangeEvent event) throws AbortProcessingException {
			try {
				model.setValue((Serializable)event.getNewValue());
			} catch (VascException e) {
				throw new AbortProcessingException(e);
			}
		}
	}
	
	private void addColumns(FacesContext fc,JSFVascUIComponent comp,UIComponent table) {
		
		Application application = fc.getApplication();
		UIViewRoot viewRoot = fc.getViewRoot();
		
		VascEntry entry = comp.getVascEntry();
		String entrySupportVar = (String)comp.getAttributes().get(JSFVascUIComponent.ENTRY_SUPPORT_VAR_KEY);
		String tableRecordVar = (String)comp.getAttributes().get(JSFVascUIComponent.TABLE_RECORD_VAR_KEY);
		
		if (!entry.getVascFrontendController().getVascFrontendHelper().getMultiRowActions(entry).isEmpty()) {
			UIColumn colUp = (UIColumn)application.createComponent(UIColumn.COMPONENT_TYPE);
			colUp.setId(viewRoot.createUniqueId());
		
			/*
			// Select box head in table 
			HtmlCommandLink selectAll = (HtmlCommandLink)application.createComponent(HtmlCommandLink.COMPONENT_TYPE);
			selectAll.setId(viewRoot.createUniqueId());
			selectAll.setType("selectAll");
			MethodExpression actionExpression = getMethodExpression("#{"+entrySupportVar+".selectAllAction}");
			MethodExpressionActionListener meActionListener = new MethodExpressionActionListener(actionExpression);
			selectAll.addActionListener(meActionListener);
		
			HtmlOutputText orderUp = (HtmlOutputText)application.createComponent(HtmlOutputText.COMPONENT_TYPE);
			orderUp.setId(viewRoot.createUniqueId());
			orderUp.setEscape(false);
			orderUp.setValue("selectAll");
			
			selectAll.getChildren().add(orderUp);
			colUp.setHeader(selectAll);
			*/
			
			HtmlSelectBooleanCheckbox select = (HtmlSelectBooleanCheckbox)application.createComponent(HtmlSelectBooleanCheckbox.COMPONENT_TYPE);
			select.setId(viewRoot.createUniqueId());
			select.setStyleClass("js_table_select_all");
			ValueExpression ve1 = application.getExpressionFactory().createValueExpression(FacesContext.getCurrentInstance().getELContext(),"#{"+entrySupportVar+".vascEntry.vascFrontendController.vascEntryState.multiActionSelection["+tableRecordVar+".recordId]}",Object.class);
			select.setValueExpression("value", ve1);
			
			colUp.getChildren().add(select);
			table.getChildren().add(colUp);		
		}
		
		
		for (VascEntryField c:entry.getVascEntryFields()) {
			if (entry.getVascFrontendController().getVascFrontendDataSelector().isFieldList(c)==false) {
				continue;
			}
			UIColumn col = (UIColumn)application.createComponent(UIColumn.COMPONENT_TYPE);
			col.setId(viewRoot.createUniqueId());
   
			HtmlCommandLink link = (HtmlCommandLink)application.createComponent(HtmlCommandLink.COMPONENT_TYPE);
			link.setId(viewRoot.createUniqueId());
			link.setType(c.getId());
			
			MethodExpression actionExpression = getMethodExpression("#{"+entrySupportVar+".sortAction}");
			MethodExpressionActionListener meActionListener = new MethodExpressionActionListener(actionExpression);
			link.addActionListener(meActionListener);
			
			HtmlOutputText out2 = (HtmlOutputText)application.createComponent(HtmlOutputText.COMPONENT_TYPE);
			out2.setId(viewRoot.createUniqueId());
			out2.setEscape(false);
			out2.setValue(i18n(entry,c.getName())+"&nbsp;");
			
			HtmlOutputText orderUp = (HtmlOutputText)application.createComponent(HtmlOutputText.COMPONENT_TYPE);
			orderUp.setId(viewRoot.createUniqueId());
			orderUp.setEscape(false);
			orderUp.setValue("&#x2191;"); // ↑
			ValueExpression ren2 = application.getExpressionFactory().createValueExpression(FacesContext.getCurrentInstance().getELContext(), "#{"+entrySupportVar+".vascEntry.vascFrontendController.vascEntryState.vascBackendState.sortAscending==true and "+entrySupportVar+".vascEntry.vascFrontendController.vascEntryState.vascBackendState.sortField=='"+c.getId()+"'}", Boolean.class);
			orderUp.setValueExpression("rendered", ren2);
			
			HtmlOutputText orderDown = (HtmlOutputText)application.createComponent(HtmlOutputText.COMPONENT_TYPE);
			orderDown.setId(viewRoot.createUniqueId());
			orderDown.setEscape(false);
			orderDown.setValue("&#x2193;"); // ↓
			ValueExpression ren3 = application.getExpressionFactory().createValueExpression(FacesContext.getCurrentInstance().getELContext(), "#{"+entrySupportVar+".vascEntry.vascFrontendController.vascEntryState.vascBackendState.sortAscending==false and "+entrySupportVar+".vascEntry.vascFrontendController.vascEntryState.vascBackendState.sortField=='"+c.getId()+"'}", Boolean.class);
			orderDown.setValueExpression("rendered", ren3);
			
			link.getChildren().add(out2);
			link.getChildren().add(orderUp);
			link.getChildren().add(orderDown);
			col.setHeader(link);
			
			HtmlOutputText out = (HtmlOutputText)application.createComponent(HtmlOutputText.COMPONENT_TYPE);
			out.setId(viewRoot.createUniqueId());
			int index = VascDataBackendBean.getIndexId(c);
			ValueExpression ve1 = application.getExpressionFactory().createValueExpression(FacesContext.getCurrentInstance().getELContext(),"#{"+tableRecordVar+".field"+index+"}",Object.class);
			out.setValueExpression("value", ve1);
			
			col.getChildren().add(out);
			table.getChildren().add(col);
		}

		if (entry.getVascFrontendController().getVascEntryState().getVascBackend().isRecordMoveable()) {
			UIColumn colUp = (UIColumn)application.createComponent(UIColumn.COMPONENT_TYPE);
			colUp.setId(viewRoot.createUniqueId());
		
			HtmlCommandLink linkUp = (HtmlCommandLink)application.createComponent(HtmlCommandLink.COMPONENT_TYPE);
			linkUp.setId(viewRoot.createUniqueId());
			linkUp.setType("up");
			MethodExpression actionExpression = getMethodExpression("#{"+entrySupportVar+".moveAction}");
			MethodExpressionActionListener meActionListener = new MethodExpressionActionListener(actionExpression);
			linkUp.addActionListener(meActionListener);
		
			HtmlOutputText orderUp = (HtmlOutputText)application.createComponent(HtmlOutputText.COMPONENT_TYPE);
			orderUp.setId(viewRoot.createUniqueId());
			orderUp.setEscape(false);
			orderUp.setValue("&#x2191;&nbsp;up"); // ↑
			
			linkUp.getChildren().add(orderUp);
			colUp.getChildren().add(linkUp);
			table.getChildren().add(colUp);		
			
			HtmlCommandLink linkDown = (HtmlCommandLink)application.createComponent(HtmlCommandLink.COMPONENT_TYPE);
			linkDown.setId(viewRoot.createUniqueId());
			linkDown.setType("down");
			MethodExpression actionExpression2 = getMethodExpression("#{"+entrySupportVar+".moveAction}");
			MethodExpressionActionListener meActionListener2 = new MethodExpressionActionListener(actionExpression2);
			linkDown.addActionListener(meActionListener2);
			
			UIColumn colDown = (UIColumn)application.createComponent(UIColumn.COMPONENT_TYPE);
			colDown.setId(viewRoot.createUniqueId());
			HtmlOutputText orderDown = (HtmlOutputText)application.createComponent(HtmlOutputText.COMPONENT_TYPE);
			orderDown.setId(viewRoot.createUniqueId());
			orderDown.setEscape(false);
			orderDown.setValue("&#x2193;&nbsp;down"); // ↓
			
			linkDown.getChildren().add(orderDown);
			colDown.getChildren().add(linkDown);
			table.getChildren().add(colDown);
		}
		
		for (VascEntryLink vascLink:entry.getVascFrontendController().getVascFrontendHelper().getVascLinkEntryByType(entry, VascEntryLinkType.LIST)) {
			UIColumn col = (UIColumn)application.createComponent(UIColumn.COMPONENT_TYPE);
			col.setId(viewRoot.createUniqueId());
   
			UIOutput colHeader = (UIOutput)application.createComponent(UIOutput.COMPONENT_TYPE);
			colHeader.setId(viewRoot.createUniqueId());
			colHeader.setValue("link");
			col.setHeader(colHeader);
			
			HtmlCommandLink link = (HtmlCommandLink)application.createComponent(HtmlCommandLink.COMPONENT_TYPE);
			link.setId(viewRoot.createUniqueId());
			link.setType(vascLink.getId());
			MethodExpression actionExpression = getMethodExpression("#{"+entrySupportVar+".linkAction}");
			MethodExpressionActionListener meActionListener = new MethodExpressionActionListener(actionExpression);
			link.addActionListener(meActionListener);
			
			HtmlOutputText out = (HtmlOutputText)application.createComponent(HtmlOutputText.COMPONENT_TYPE);
			out.setId(viewRoot.createUniqueId());
			out.setValue(i18n(entry,vascLink.getName()));
			
			link.getChildren().add(out);
			col.getChildren().add(link);
			
			table.getChildren().add(col);
		}
		
		for (RowVascAction action:entry.getRowActions()) {
			if (entry.getVascFrontendController().getVascFrontendDataSelector().isActionAllowed(action)==false) {
				continue;
			}
			if (AddRowAction.ACTION_ID.equals(action.getId())) {
				continue;
			}
			
			UIColumn col = (UIColumn)application.createComponent(UIColumn.COMPONENT_TYPE);
			col.setId(viewRoot.createUniqueId());
   
			UIOutput colHeader = (UIOutput)application.createComponent(UIOutput.COMPONENT_TYPE);
			colHeader.setId(viewRoot.createUniqueId());
			colHeader.setValue(i18n(entry,action.getName()));
			col.setHeader(colHeader);
			
			HtmlCommandLink link = (HtmlCommandLink)application.createComponent(HtmlCommandLink.COMPONENT_TYPE);
			link.setId(viewRoot.createUniqueId());
			link.setType(action.getId());
			MethodExpression actionExpression = getMethodExpression("#{"+entrySupportVar+".rowAction}");
			MethodExpressionActionListener meActionListener = new MethodExpressionActionListener(actionExpression);
			link.addActionListener(meActionListener);
			
			HtmlOutputText out = (HtmlOutputText)application.createComponent(HtmlOutputText.COMPONENT_TYPE);
			out.setId(viewRoot.createUniqueId());
			out.setValue(i18n(entry,action.getName()));
			
			link.getChildren().add(out);
			col.getChildren().add(link);
			
			table.getChildren().add(col);
		}		
	}
	
	private MethodExpression getMethodExpression(String name) {
		Class<?>[] argtypes = new Class[1];
		argtypes[0] = ActionEvent.class;
		FacesContext facesCtx = FacesContext.getCurrentInstance();
		ELContext elContext = facesCtx.getELContext();
		return facesCtx.getApplication().getExpressionFactory().createMethodExpression(elContext, name, null, argtypes);
	}
	
};