/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.frontend.web.jsf;


import javax.el.ValueExpression;
import javax.faces.component.UIComponent;
import javax.faces.webapp.UIComponentELTag;

/**
 * 
 *
 * @author Willem Cazander
 * @version 1.0 Nov 16, 2008
 */
public class JSFVascUIComponentTag extends UIComponentELTag {
	
	public static final String COMPONENT_TYPE = "vasc.jsf.component";
    public static final String RENDERER_TYPE  = "vasc.jsf.component.renderer";
	
	private ValueExpression entrySupportVar = null;
	private ValueExpression tableRecordVar = null;
	private ValueExpression injectEditFieldsId = null;
	private ValueExpression injectTableOptionsId = null;
	private ValueExpression injectTableColumnsId = null;
	private ValueExpression disableLinkColumns = null;
	
	@Override
	public String getComponentType() {
		return COMPONENT_TYPE;
	}
	
	@Override
	public String getRendererType() {
		return RENDERER_TYPE;
	}
	
	/**
	 * @see javax.faces.webapp.UIComponentELTag#setProperties(javax.faces.component.UIComponent)
	 */
	@Override
	protected void setProperties(UIComponent component) {
        super.setProperties(component);
        //processProperty(component, entryName, 			JSFVascUIComponent.ENTRY_NAME_KEY);
        processProperty(component, entrySupportVar, 	JSFVascUIComponent.ENTRY_SUPPORT_VAR_KEY);
        processProperty(component, tableRecordVar,	 	JSFVascUIComponent.TABLE_RECORD_VAR_KEY);
        processProperty(component, injectEditFieldsId, 	JSFVascUIComponent.INJECT_EDIT_FIELDS_ID);
        processProperty(component, injectTableOptionsId,JSFVascUIComponent.INJECT_TABLE_OPTIONS_ID);
        processProperty(component, injectTableColumnsId,JSFVascUIComponent.INJECT_TABLE_COLUMNS_ID);
        processProperty(component, disableLinkColumns  ,JSFVascUIComponent.DISABLE_LINK_COLUMNS);
    }

	@Override
    public void release() {
        super.release();
        entrySupportVar = null;
        tableRecordVar = null;
        injectEditFieldsId = null;
        injectTableOptionsId = null;
        injectTableColumnsId = null;
        disableLinkColumns = null;
    }
	
    protected final void processProperty(final UIComponent component, final ValueExpression property,final String propertyName) {
    	if (property != null) {
    		if(property.isLiteralText()) {
    			component.getAttributes().put(propertyName, property.getExpressionString());
    		} else {
    			component.setValueExpression(propertyName, property);
    		}
    	}
    }
	
	// ============= BEAN FIELDS
	
	/**
	 * @return the entrySupportVar
	 */
	public ValueExpression getEntrySupportVar() {
		return entrySupportVar;
	}

	/**
	 * @param entrySupportVar the entrySupportVar to set
	 */
	public void setEntrySupportVar(ValueExpression entrySupportVar) {
		this.entrySupportVar = entrySupportVar;
	}

	/**
	 * @return the injectEditFieldsId
	 */
	public ValueExpression getInjectEditFieldsId() {
		return injectEditFieldsId;
	}

	/**
	 * @param injectEditFieldsId the injectEditFieldsId to set
	 */
	public void setInjectEditFieldsId(ValueExpression injectEditFieldsId) {
		this.injectEditFieldsId = injectEditFieldsId;
	}

	/**
	 * @return the injectTableOptionsId
	 */
	public ValueExpression getInjectTableOptionsId() {
		return injectTableOptionsId;
	}

	/**
	 * @param injectTableOptionsId the injectTableOptionsId to set
	 */
	public void setInjectTableOptionsId(ValueExpression injectTableOptionsId) {
		this.injectTableOptionsId = injectTableOptionsId;
	}

	/**
	 * @return the injectTableColumnsId
	 */
	public ValueExpression getInjectTableColumnsId() {
		return injectTableColumnsId;
	}

	/**
	 * @param injectTableColumnsId the injectTableColumnsId to set
	 */
	public void setInjectTableColumnsId(ValueExpression injectTableColumnsId) {
		this.injectTableColumnsId = injectTableColumnsId;
	}

	/**
	 * @return the tableRecordVar
	 */
	public ValueExpression getTableRecordVar() {
		return tableRecordVar;
	}

	/**
	 * @param tableRecordVar the tableRecordVar to set
	 */
	public void setTableRecordVar(ValueExpression tableRecordVar) {
		this.tableRecordVar = tableRecordVar;
	}

	/**
	 * @return the disableLinkColumns
	 */
	public ValueExpression getDisableLinkColumns() {
		return disableLinkColumns;
	}

	/**
	 * @param disableLinkColumns the disableLinkColumns to set
	 */
	public void setDisableLinkColumns(ValueExpression disableLinkColumns) {
		this.disableLinkColumns = disableLinkColumns;
	}
}