/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.frontend.web.jsf;

import java.util.Iterator;
import java.util.Map;
import java.util.logging.Logger;

import javax.faces.application.FacesMessage;
import javax.faces.component.UIComponent;
import javax.faces.component.UIInput;
import javax.faces.context.FacesContext;
import javax.faces.event.PhaseEvent;
import javax.faces.event.PhaseId;
import javax.faces.event.PhaseListener;
import javax.faces.validator.Validator;
import javax.faces.validator.ValidatorException;

import net.forwardfire.vasc.core.VascEntry;
import net.forwardfire.vasc.frontend.web.jsf.JSFVascUIComponentRenderer.VascJSFInputValidator2;


/**
 * JSFVascValidatePhaseListener runs the vasc field edit fields.
 * 
 * @author Willem Cazander
 * @version 1.0 Nov 15, 2009
 */
public class JSFVascValidatePhaseListener implements PhaseListener {

	private static final long serialVersionUID = 1L;
	private Logger logger = null;
	
	public JSFVascValidatePhaseListener() {
		logger = Logger.getLogger(JSFVascValidatePhaseListener.class.getName());
	}
	
	private static final String SAVE_ACTION_ID_HACK = "VASC_SAVE";
	
	@Override
	public void afterPhase(PhaseEvent event) {
		FacesContext context = event.getFacesContext();
		
		// we really only want validation this early when saveing.
		// this is currently a bit hacky impl;		
		Map<String,String> requestMap = context.getExternalContext().getRequestParameterMap();
		for (String key:requestMap.keySet()) {
			if (key.contains(SAVE_ACTION_ID_HACK)) {
				validateUIInput(context.getViewRoot(),context);
				return;
			}
		}		
	}
	
	@Override
	public void beforePhase(PhaseEvent event) {
	}
	
	@Override
	public PhaseId getPhaseId() {
		return PhaseId.PROCESS_VALIDATIONS;
	}
	private void validateUIInput(UIComponent component,FacesContext context) {
		if (component instanceof UIInput) {
			JSFVascUIComponent comp = JSFVascUIComponent.findVascParent(component);
			logger.finer("Validate UIInput: "+component+" comp: "+comp);
			if (comp==null) {
				return; // non-vasc ui-input
			}
			if (comp.getSupportBean().getSelected()==null) {
				return; // no editing
			}
			VascEntry entry = comp.getVascEntry();
			if (entry.getVascFrontendController().getVascEntryState().getEntryDataObject()==null) {
				return; // we are not in edit mode.
			}
			UIInput in = (UIInput)component;
			for (Validator v:in.getValidators()) {
				if (v instanceof VascJSFInputValidator2) {
					VascJSFInputValidator2 validator = (VascJSFInputValidator2)v;
					try {
						in.setValid(true);
						validator.validatePhase(context, in, in.getValue());
					} catch (ValidatorException ve) {
						in.setValid(false);
						
						// note: ve has the message already but this is the UIInput way
						FacesMessage message;
						String validatorMessageString = in.getValidatorMessage();
						if (null != validatorMessageString) {
							message = new FacesMessage(FacesMessage.SEVERITY_ERROR,validatorMessageString,validatorMessageString);
							message.setSeverity(FacesMessage.SEVERITY_ERROR);
						} else {
							message = ve.getFacesMessage();
						}
						if (message != null) {
							context.addMessage(in.getClientId(context), message);
						}
					}
				}
			}
		}
		Iterator<UIComponent> kids = component.getFacetsAndChildren();
		while (kids.hasNext()) {
			UIComponent child = kids.next();
			validateUIInput(child,context);
		}
	}	
}
