/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.frontend.web.jsf;


import java.io.Serializable;

import net.forwardfire.vasc.backend.VascBackendException;
import net.forwardfire.vasc.backend.VascBackendAccessDataRecord;
import net.forwardfire.vasc.core.VascEntry;
import net.forwardfire.vasc.core.VascEntryField;
import net.forwardfire.vasc.frontend.VascFrontendDataSelector.EntryFieldSelectType;

/**
 * 
 * 
 * @author Willem Cazander
 * @version 1.0 May 03, 2009
 */
public class VascDataBackendBean implements Serializable {

	private static final long serialVersionUID = 3881688974089760074L;
	private VascEntry entry = null;
	private Serializable record = null;
	private boolean realValue = false;
	private int recordId = 0;
	
	public VascDataBackendBean(VascEntry entry,Serializable record,int recordId) {
		this.entry=entry;
		this.record=record;
		this.recordId=recordId;
	}
	
	public boolean isRealValue() {
		return realValue;
	}
	public void setRealValue(boolean realValue) {
		this.realValue=realValue;
	}
	
	static public VascEntryField getFieldIdByIndex(VascEntry entry,int index) {
		if (index>entry.getVascEntryFields().size()) {
			throw new IllegalArgumentException("Index is bigger then total field size: "+index);
		}
		return entry.getVascFrontendController().getVascFrontendDataSelector().getFields(EntryFieldSelectType.ALL).get(index);
	}
	
	static public int getIndexId(VascEntryField field) {
		int i =0;
		for (VascEntryField f:field.getVascEntry().getVascEntryFields()) {
			if (f.getId().equals(field.getId())) {
				return i;
			}
			i++;
		}
		throw new IllegalArgumentException("Could not find field.");
	}
	
	private Object getValue(int index) {
		try {
			VascEntryField field = getFieldIdByIndex(entry,index);
			VascBackendAccessDataRecord value = field.getVascEntryFieldValue();
			if (isRealValue()) {
				return value.getValue(field.getBackendName(), record);
			}
			return value.getDisplayValue(field.getBackendName(), record);
		} catch (Exception e) {
			return "Error: "+e.getMessage();
		}
	}
	
	private void setValue(int index,Object valueObject) {
		try {
			VascEntryField field = getFieldIdByIndex(entry,index);
			VascBackendAccessDataRecord<Serializable> value = field.getVascEntryFieldValue();
			value.setValue(field.getBackendName(), record,(Serializable)valueObject); // TODO: rm cast
		} catch (VascBackendException e) {
			throw new RuntimeException("Could not set value on record.",e);
		}			
	}
	
	public Serializable getRecord() {
		return record;
	}
	
	public int getRecordId() {
		return recordId;
	}
	
	// GET/SET bean methods for using in EL
	
	
	public Object getField0() {
		return getValue(0);
	}
	
	public void setField0(Object o) {
		setValue(0,o);
	}
	
	public Object getField1() {
		return getValue(1);
	}
	
	public void setField1(Object o) {
		setValue(1,o);
	}
	
	public Object getField2() {
		return getValue(2);
	}
	
	public void setField2(Object o) {
		setValue(2,o);
	}
	
	public Object getField3() {
		return getValue(3);
	}
	
	public void setField3(Object o) {
		setValue(3,o);
	}
	
	public Object getField4() {
		return getValue(4);
	}
	
	public void setField4(Object o) {
		setValue(4,o);
	}
	
	public Object getField5() {
		return getValue(5);
	}
	
	public void setField5(Object o) {
		setValue(5,o);
	}
	
	public Object getField6() {
		return getValue(6);
	}
	
	public void setField6(Object o) {
		setValue(6,o);
	}
	
	public Object getField7() {
		return getValue(7);
	}
	
	public void setField7(Object o) {
		setValue(7,o);
	}
	
	public Object getField8() {
		return getValue(8);
	}
	
	public void setField8(Object o) {
		setValue(8,o);
	}
	
	public Object getField9() {
		return getValue(9);
	}
	
	public void setField9(Object o) {
		setValue(9,o);
	}
	
	
	public Object getField10() {
		return getValue(10);
	}
	
	public void setField10(Object o) {
		setValue(10,o);
	}
	
	public Object getField11() {
		return getValue(11);
	}
	
	public void setField11(Object o) {
		setValue(11,o);
	}
	
	public Object getField12() {
		return getValue(12);
	}
	
	public void setField12(Object o) {
		setValue(12,o);
	}
	
	public Object getField13() {
		return getValue(13);
	}
	
	public void setField13(Object o) {
		setValue(13,o);
	}
	
	public Object getField14() {
		return getValue(14);
	}
	
	public void setField14(Object o) {
		setValue(14,o);
	}
	
	public Object getField15() {
		return getValue(15);
	}
	
	public void setField15(Object o) {
		setValue(15,o);
	}
	
	public Object getField16() {
		return getValue(16);
	}
	
	public void setField16(Object o) {
		setValue(16,o);
	}
	
	public Object getField17() {
		return getValue(17);
	}
	
	public void setField17(Object o) {
		setValue(17,o);
	}
	
	public Object getField18() {
		return getValue(18);
	}
	
	public void setField18(Object o) {
		setValue(18,o);
	}
	
	public Object getField19() {
		return getValue(19);
	}
	
	public void setField19(Object o) {
		setValue(19,o);
	}
	
	
	public Object getField20() {
		return getValue(20);
	}
	
	public void setField20(Object o) {
		setValue(20,o);
	}
	
	public Object getField21() {
		return getValue(21);
	}
	
	public void setField21(Object o) {
		setValue(21,o);
	}
	
	public Object getField22() {
		return getValue(22);
	}
	
	public void setField22(Object o) {
		setValue(22,o);
	}
	
	public Object getField23() {
		return getValue(23);
	}
	
	public void setField23(Object o) {
		setValue(23,o);
	}
	
	public Object getField24() {
		return getValue(24);
	}
	
	public void setField24(Object o) {
		setValue(24,o);
	}
	
	public Object getField25() {
		return getValue(25);
	}
	
	public void setField25(Object o) {
		setValue(25,o);
	}
	
	public Object getField26() {
		return getValue(26);
	}
	
	public void setField26(Object o) {
		setValue(26,o);
	}
	
	public Object getField27() {
		return getValue(27);
	}
	
	public void setField27(Object o) {
		setValue(27,o);
	}
	
	public Object getField28() {
		return getValue(28);
	}
	
	public void setField28(Object o) {
		setValue(28,o);
	}
	
	public Object getField29() {
		return getValue(29);
	}
	
	public void setField29(Object o) {
		setValue(29,o);
	}
	
	
	public Object getField30() {
		return getValue(30);
	}
	
	public void setField30(Object o) {
		setValue(30,o);
	}
	
	public Object getField31() {
		return getValue(31);
	}
	
	public void setField31(Object o) {
		setValue(31,o);
	}
	
	public Object getField32() {
		return getValue(32);
	}
	
	public void setField32(Object o) {
		setValue(32,o);
	}
	
	public Object getField33() {
		return getValue(33);
	}
	
	public void setField33(Object o) {
		setValue(33,o);
	}
	
	public Object getField34() {
		return getValue(34);
	}
	
	public void setField34(Object o) {
		setValue(34,o);
	}
	
	public Object getField35() {
		return getValue(35);
	}
	
	public void setField35(Object o) {
		setValue(35,o);
	}
	
	public Object getField36() {
		return getValue(36);
	}
	
	public void setField36(Object o) {
		setValue(36,o);
	}
	
	public Object getField37() {
		return getValue(37);
	}
	
	public void setField37(Object o) {
		setValue(37,o);
	}
	
	public Object getField38() {
		return getValue(38);
	}
	
	public void setField38(Object o) {
		setValue(38,o);
	}
	
	public Object getField39() {
		return getValue(39);
	}
	
	public void setField39(Object o) {
		setValue(39,o);
	}
	
	
	public Object getField40() {
		return getValue(40);
	}
	
	public void setField40(Object o) {
		setValue(40,o);
	}
	
	public Object getField41() {
		return getValue(41);
	}
	
	public void setField41(Object o) {
		setValue(41,o);
	}
	
	public Object getField42() {
		return getValue(42);
	}
	
	public void setField42(Object o) {
		setValue(42,o);
	}
	
	public Object getField43() {
		return getValue(43);
	}
	
	public void setField43(Object o) {
		setValue(43,o);
	}
	
	public Object getField44() {
		return getValue(44);
	}
	
	public void setField44(Object o) {
		setValue(44,o);
	}
	
	public Object getField45() {
		return getValue(45);
	}
	
	public void setField45(Object o) {
		setValue(45,o);
	}
	
	public Object getField46() {
		return getValue(46);
	}
	
	public void setField46(Object o) {
		setValue(46,o);
	}
	
	public Object getField47() {
		return getValue(47);
	}
	
	public void setField47(Object o) {
		setValue(47,o);
	}
	
	public Object getField48() {
		return getValue(48);
	}
	
	public void setField48(Object o) {
		setValue(48,o);
	}
	
	public Object getField49() {
		return getValue(49);
	}
	
	public void setField49(Object o) {
		setValue(49,o);
	}
}
