/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.frontend.web.jsf;

import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.faces.FactoryFinder;
import javax.faces.application.ViewExpiredException;
import javax.faces.component.UIViewRoot;
import javax.faces.context.FacesContext;
import javax.faces.context.FacesContextFactory;
import javax.faces.lifecycle.Lifecycle;
import javax.faces.lifecycle.LifecycleFactory;
import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import net.forwardfire.vasc.core.VascController;
import net.forwardfire.vasc.core.VascControllerProvider;
import net.forwardfire.vasc.core.VascEntry;

/**
 * Parses the Request info path info 3 vasc attributes an dispatches the request to the templateFile.
 *
 * @author Willem Cazander
 * @version 1.0 Mar 21, 2009
 */
public class VascRequestFacesFilter implements Filter {

	private Logger logger = null; 
	private String templateFile = null;
	private ServletContext servletContext = null;
	private VascController vascController = null;
	private VascControllerProvider vascControllerProvider = null;
	static public final String REQ_VASC_CONTOLLER = "requestScopeVascController";
	static public final String REQ_VASC_ENTRY_ID = "requestScopeVascEntryId";
	static public final String REQ_VASC_URI_PATH = "requestScopeVascUriPath";
	
	/**
	 * @see javax.servlet.Filter#destroy()
	 */
	@Override
	public void destroy() {
		servletContext = null;
		templateFile = null;
		logger = null;
	}

	/**
	 * @see javax.servlet.Filter#init(javax.servlet.FilterConfig)
	 */
	@Override
	public void init(FilterConfig config) throws ServletException {
		logger			= Logger.getLogger(VascRequestFacesFilter.class.getName());
		servletContext	= config.getServletContext();
		templateFile	= config.getInitParameter("templateFile");
		if (templateFile==null) {
			throw new ServletException("No templateFile init-param found.");
		}
		String className = config.getInitParameter("vascControllerProvider");
		if (className==null) {
			throw new ServletException("Can't start with null vascControllerProvider init parameter.");
		}
		String classArgument = null;
		int atIndex = className.indexOf('@');
		if (atIndex > 0) {
			classArgument = className.substring(0,atIndex);
			className = className.substring(atIndex+1);
		}
		try {
			ClassLoader cl = Thread.currentThread().getContextClassLoader();
			if (cl == null) {
				cl = className.getClass().getClassLoader();
			}
			Class<?> clazz = cl.loadClass(className);
			if (classArgument==null) {
				vascControllerProvider = (VascControllerProvider)clazz.newInstance();
			} else {
				vascControllerProvider = (VascControllerProvider)clazz.getConstructor(String.class).newInstance(classArgument);
			}
		} catch (Exception e) {
			throw new ServletException("Could not create VascControllerProvider: "+e.getMessage(),e);
		}
	}
	
	/**
	 * @see javax.servlet.Filter#doFilter(javax.servlet.ServletRequest, javax.servlet.ServletResponse, javax.servlet.FilterChain)
	 */
	@Override
	public void doFilter(ServletRequest servletRequest,ServletResponse servletResponse,FilterChain chain) throws IOException, ServletException {
	
		HttpServletRequest  request  = (HttpServletRequest)servletRequest;
		HttpServletResponse response = (HttpServletResponse)servletResponse;
		
		// request info
		String path = request.getRequestURI();
		String v = "/vasc/";
		if (path.contains(v)) {
			path  = path.substring(path.indexOf(v)+v.length());
		}
		
		// stuff to fill
		String entryId = null;
		String actionName = null;
		String actionRecordId = null;
		
		// parse request info
		//path = path.substring(1);		// rm /
		int index = path.indexOf("/");	// check next /
		String actionString = null;
		if (index>0) {
			actionString = path.substring(index+1);
			entryId = path.substring(0,index);
		} else {
			entryId = path;
		}
		
		if (actionString!=null) {
			index = actionString.indexOf("/");
			String recordString = null;
			if (index>0) {
				recordString = actionString.substring(index+1);
				actionName = actionString.substring(0,index);  
			} else {
				actionName = actionString;	
			}
			if (recordString!=null) {
				actionRecordId = recordString;
			}
		}
		
		//log
		if (logger.isLoggable(Level.FINE)) {
			logger.log(Level.FINE,"entryId="+entryId+" actionName="+actionName+" actionRecordId="+actionRecordId);
		}
				
		// Acquire the FacesContext instance for this request
		FacesContext facesContext = FacesContext.getCurrentInstance();
		if (facesContext == null) {
			FacesContextFactory facesContextFactory = (FacesContextFactory)FactoryFinder.getFactory(FactoryFinder.FACES_CONTEXT_FACTORY);
			LifecycleFactory    lifecycleFactory    = (LifecycleFactory)FactoryFinder.getFactory(FactoryFinder.LIFECYCLE_FACTORY);
			Lifecycle           lifecycle           = lifecycleFactory.getLifecycle(LifecycleFactory.DEFAULT_LIFECYCLE);
			
			facesContext = facesContextFactory.getFacesContext(servletContext, request, response, lifecycle);
			ProtectedFacesContext.setFacesContextAsCurrentInstance(facesContext);
			
			UIViewRoot viewRoot = facesContext.getApplication().getViewHandler().createView(facesContext,entryId+actionName);
			facesContext.setViewRoot(viewRoot);
		}
		
		if (vascController==null) {
			synchronized (this) {
				vascController = vascControllerProvider.getVascController();
			}
		}
		
		// Fetch entry and do checks
		VascEntry vascEntry = vascController.getVascEntryController().getVascEntryById(entryId);
		if (vascEntry == null) {
			response.sendRedirect("/");
			return;
		}
		
		// add to request attributes
		request.setAttribute(REQ_VASC_CONTOLLER, vascController);
		request.setAttribute(REQ_VASC_ENTRY_ID,  vascEntry.getId());
		request.setAttribute(REQ_VASC_URI_PATH,  request.getRequestURI());
		
		// And dispatch to the vasc template file.
		try {
			request.getRequestDispatcher(templateFile).forward(request, response);
		} catch (ViewExpiredException e) {
			response.sendRedirect(request.getRequestURL().toString()); // lets try again
		}
	}
	
	private abstract static class ProtectedFacesContext extends FacesContext {
		protected static void setFacesContextAsCurrentInstance(FacesContext facesContext) {
			FacesContext.setCurrentInstance(facesContext);
		}
	}
}