/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.frontend.web.jsf;

import java.io.IOException;
import java.util.Locale;

import javax.faces.FacesException;
import javax.faces.application.ViewHandler;
import javax.faces.component.UIViewRoot;
import javax.faces.context.FacesContext;
import javax.servlet.http.HttpServletRequest;

/**
 * Fixes the action url
 * 
 * @author Willem Cazander
 * @version 1.0 May 05, 2009
 */
public class VascViewHandler extends ViewHandler  {

	private ViewHandler parentViewHandler = null;
	
	
	public VascViewHandler(ViewHandler viewHandler) {
		super();
		this.parentViewHandler = viewHandler;
	}

	/**
	 * @see javax.faces.application.ViewHandler#getActionURL(javax.faces.context.FacesContext, java.lang.String)
	 */
	@Override
	public String getActionURL(FacesContext context, String name) {
		HttpServletRequest request = (HttpServletRequest)context.getExternalContext().getRequest();
		if (request.getAttribute(VascRequestFacesFilter.REQ_VASC_URI_PATH)==null) {
			return parentViewHandler.getActionURL(context, name);
		}
		String path = (String)request.getAttribute(VascRequestFacesFilter.REQ_VASC_URI_PATH);
		return path;
	}

	/**
	 * @see javax.faces.application.ViewHandler#calculateLocale(javax.faces.context.FacesContext)
	 */
	@Override
	public Locale calculateLocale(FacesContext context) {
		return parentViewHandler.calculateLocale(context);
	}

	/**
	 * @see javax.faces.application.ViewHandler#calculateRenderKitId(javax.faces.context.FacesContext)
	 */
	@Override
	public String calculateRenderKitId(FacesContext context) {
		return parentViewHandler.calculateRenderKitId(context);
	}
	
	/**
	 * @see javax.faces.application.ViewHandler#createView(javax.faces.context.FacesContext, java.lang.String)
	 */
	@Override
	public UIViewRoot createView(FacesContext context, String name) {
		return parentViewHandler.createView(context, name);
	}

	/**
	 * @see javax.faces.application.ViewHandler#getResourceURL(javax.faces.context.FacesContext, java.lang.String)
	 */
	@Override
	public String getResourceURL(FacesContext context, String resource) {
		return parentViewHandler.getResourceURL(context, resource);
	}

	/**
	 * @see javax.faces.application.ViewHandler#renderView(javax.faces.context.FacesContext, javax.faces.component.UIViewRoot)
	 */
	@Override
	public void renderView(FacesContext context, UIViewRoot viewRoot) throws IOException, FacesException {
		parentViewHandler.renderView(context, viewRoot);
	}

	/**
	 * @see javax.faces.application.ViewHandler#restoreView(javax.faces.context.FacesContext, java.lang.String)
	 */
	@Override
	public UIViewRoot restoreView(FacesContext context, String name) {
		return parentViewHandler.restoreView(context, name);
	}

	/**
	 * @see javax.faces.application.ViewHandler#writeState(javax.faces.context.FacesContext)
	 */
	@Override
	public void writeState(FacesContext context) throws IOException {
		parentViewHandler.writeState(context);
	}
}
