/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.frontend.web.jsf.ui;

import java.io.Serializable;
import java.util.List;

import javax.el.ValueExpression;
import javax.faces.application.Application;
import javax.faces.component.UIComponent;
import javax.faces.component.UISelectItems;
import javax.faces.component.UIViewRoot;
import javax.faces.component.html.HtmlSelectOneMenu;
import javax.faces.context.FacesContext;
import javax.faces.convert.Converter;
import javax.faces.convert.ConverterException;

import net.forwardfire.vasc.backend.select.VascBackendSelectItem;
import net.forwardfire.vasc.core.VascEntry;
import net.forwardfire.vasc.core.VascEntryField;
import net.forwardfire.vasc.core.ui.VascValueModel;
import net.forwardfire.vasc.core.VascException;
import net.forwardfire.vasc.frontend.web.jsf.JSFVascUIComponent;




/**
 * 
 * 
 * @author Willem Cazander
 * @version 1.0 Mar 28, 2009
 */
public class JSFList extends AbstractJSFBaseComponent {
	
	@Override
	public Object createComponent(VascEntry entry,VascEntryField entryField,VascValueModel model,Object gui) throws VascException {
		Application application = FacesContext.getCurrentInstance().getApplication();
		UIViewRoot viewRoot = FacesContext.getCurrentInstance().getViewRoot();
		HtmlSelectOneMenu component = (HtmlSelectOneMenu)application.createComponent(HtmlSelectOneMenu.COMPONENT_TYPE);
		component.setId(viewRoot.createUniqueId());
		componentId = component.getId();
		
		JSFVascUIComponent comp = JSFVascUIComponent.findVascParent((UIComponent)gui);
		String entrySupportVar = (String)comp.getAttributes().get(JSFVascUIComponent.ENTRY_SUPPORT_VAR_KEY);
		ValueExpression itemsTestVE = FacesContext.getCurrentInstance().getApplication().getExpressionFactory().createValueExpression(FacesContext.getCurrentInstance().getELContext(), "#{"+entrySupportVar+".editSelectItemModels['jsfListItems_"+componentId+"']}", JSFListModel.class);
				
		JSFListModel t = new JSFListModel(entryField);
		itemsTestVE.setValue(FacesContext.getCurrentInstance().getELContext(), t);
		
		component.setConverter(new VascConverter(t));
		((UIComponent)gui).getChildren().add(component);
		
		setValueTestModel(component);
		
		return component;
	}
	
	/**
	 * @see net.forwardfire.vasc.core.ui.VascUIComponent#isDisabled()
	 */
	@Override
	public boolean isDisabled() {
		HtmlSelectOneMenu component = (HtmlSelectOneMenu)JSFVascUIComponent.findComponentById(FacesContext.getCurrentInstance().getViewRoot(),componentId);
		return component.isDisabled();
	}

	/**
	 * @see net.forwardfire.vasc.core.ui.VascUIComponent#setDisabled(boolean)
	 */
	@Override
	public void setDisabled(boolean disabled) {
		HtmlSelectOneMenu component = (HtmlSelectOneMenu)JSFVascUIComponent.findComponentById(FacesContext.getCurrentInstance().getViewRoot(),componentId);
		component.setDisabled(disabled);
	}
	
	private void setValueTestModel(HtmlSelectOneMenu component) {
		Application application = FacesContext.getCurrentInstance().getApplication();
		UIViewRoot viewRoot = FacesContext.getCurrentInstance().getViewRoot();
		
		UISelectItems item = (UISelectItems)application.createComponent(UISelectItems.COMPONENT_TYPE);
		item.setId(viewRoot.createUniqueId());
		
		JSFVascUIComponent comp = JSFVascUIComponent.findVascChild(viewRoot,null);
		String entrySupportVar = (String)comp.getAttributes().get(JSFVascUIComponent.ENTRY_SUPPORT_VAR_KEY);
		String id = component.getId();
		ValueExpression itemsVE = FacesContext.getCurrentInstance().getApplication().getExpressionFactory().createValueExpression(FacesContext.getCurrentInstance().getELContext(), "#{"+entrySupportVar+".editSelectItemModels['jsfListItems_"+id+"']}", List.class);
		item.setValueExpression("value", itemsVE);
		
		component.getChildren().clear();
		component.getChildren().add(item);
	}
}
class VascConverter implements Converter,Serializable {

	private static final long serialVersionUID = 6198813637409531713L;
	private JSFListModel testModel = null;
	
	public VascConverter(JSFListModel testModel) {
		this.testModel=testModel;
	}
	
	/**
	 * Convert an ID to an Object
	 */
	@Override
	public Object getAsObject(FacesContext context,UIComponent component,String value) throws ConverterException {
		if (context == null) { throw new NullPointerException("context may not be null"); }
		if (component == null) { throw new NullPointerException("component may not be null"); }
		if (value == null) { return null; }
		if ((value=value.trim()).length() == 0) { return null; }

		for (VascBackendSelectItem v:testModel.getVascItems()) {
			if (value.equals(v.getKeyValue())) {
				return v.getValue();
			}
		}
		throw new ConverterException("Could not convert value as object: "+value);
	}
	
	/**
	 * Converts an Object to an ID.
	 */
	@Override
	public String getAsString(FacesContext context,UIComponent component,Object value) throws ConverterException {
		if (context == null) { throw new NullPointerException("context may not be null"); }
		if (component == null) { throw new NullPointerException("component may not be null"); }
		if (value == null) { return ""; }
		
		for (VascBackendSelectItem v:testModel.getVascItems()) {
			if (value.equals(v.getValue())) {
				return v.getKeyValue();
			}
		}
		throw new ConverterException("Could not convert value as string: "+value);
	}
}