/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.frontend.web.jsf.ui;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;

import javax.faces.model.SelectItem;

import net.forwardfire.vasc.backend.select.VascBackendSelectItem;
import net.forwardfire.vasc.core.VascEntryField;
import net.forwardfire.vasc.core.ui.VascSelectItemModel;
import net.forwardfire.vasc.core.VascException;




/**
 * 
 * 
 * @author Willem Cazander
 * @version 1.0 Mar 28, 2009
 */
public class JSFListModel implements Serializable,List<SelectItem> {
	
	private static final long serialVersionUID = -5266070864271715383L;
	private VascEntryField entryField;
	private List<VascBackendSelectItem> items = null;
	private List<SelectItem> selectItems = null;
	private boolean refresh = true;
	
	public JSFListModel() {
		items = new ArrayList<VascBackendSelectItem>(40);
		selectItems = new ArrayList<SelectItem>(40);
	}
	
	public JSFListModel(VascEntryField entryField) {
		this();
		this.entryField=entryField;
	}
	
	public List<VascBackendSelectItem> getVascItems() {
		refresh();
		return items;
	}

	public List<SelectItem> getSelectItems() {
		refresh();
		return selectItems;
	}
	
	public void requestRefresh() {
		refresh = true;
	}
	
	private void refresh() {
		if (refresh==false) {
			return;
		}
		refresh=false;
		items.clear();
		selectItems.clear();
		
		VascSelectItemModel itemModel = (VascSelectItemModel)entryField.getVascEntryFieldType().getDataObject();
		try {
			items = itemModel.getVascSelectItems(entryField.getVascEntry());
		} catch (VascException e) {
			throw new RuntimeException(e);
		}
		for (VascBackendSelectItem v:items) {
			selectItems.add(convert(v));
		}
	}
	
	private SelectItem convert(VascBackendSelectItem v) {
		SelectItem si = new SelectItem();
		si.setLabel(v.getLabel());			
		si.setValue(v.getValue());
		si.setDisabled(v.isDisabled());
		return si;
	}
	
	// === LIST interface
	
	@Override
	public void add(int arg0, SelectItem arg1) {
		getSelectItems().add(arg0,arg1);
	}
	@Override
	public boolean add(SelectItem arg0) {
		return getSelectItems().add(arg0);
	}
	@Override
	public boolean addAll(Collection<? extends SelectItem> arg0) {
		return getSelectItems().addAll(arg0);
	}
	@Override
	public boolean addAll(int arg0, Collection<? extends SelectItem> arg1) {
		return getSelectItems().addAll(arg0,arg1);
	}
	@Override
	public void clear() {
		getSelectItems().clear();
	}
	@Override
	public boolean contains(Object arg0) {
		return getSelectItems().contains(arg0);
	}
	@Override
	public boolean containsAll(Collection<?> arg0) {
		return getSelectItems().containsAll(arg0);
	}
	@Override
	public SelectItem get(int index) {
		return getSelectItems().get(index);
	}
	@Override
	public int indexOf(Object arg0) {
		return getSelectItems().indexOf(arg0);
	}
	@Override
	public boolean isEmpty() {
		return getSelectItems().isEmpty();
	}
	@Override
	public Iterator<SelectItem> iterator() {
		return getSelectItems().iterator();
	}
	@Override
	public int lastIndexOf(Object arg0) {
		return getSelectItems().lastIndexOf(arg0);
	}
	@Override
	public ListIterator<SelectItem> listIterator() {
		return getSelectItems().listIterator();
	}
	@Override
	public ListIterator<SelectItem> listIterator(int arg0) {
		return getSelectItems().listIterator(arg0);
	}
	@Override
	public SelectItem remove(int arg0) {
		return getSelectItems().remove(arg0);
	}
	@Override
	public boolean remove(Object arg0) {
		return getSelectItems().remove(arg0);
	}
	@Override
	public boolean removeAll(Collection<?> arg0) {
		return getSelectItems().removeAll(arg0);
	}
	@Override
	public boolean retainAll(Collection<?> arg0) {
		return getSelectItems().retainAll(arg0);
	}
	@Override
	public SelectItem set(int arg0, SelectItem arg1) {
		return getSelectItems().set(arg0, arg1);
	}
	@Override
	public int size() {
		return getSelectItems().size();
	}
	@Override
	public List<SelectItem> subList(int arg0, int arg1) {
		return getSelectItems().subList(arg0, arg1);
	}
	@Override
	public Object[] toArray() {
		return getSelectItems().toArray();
	}
	@Override
	public <T> T[] toArray(T[] a) {
		return getSelectItems().toArray(a);
	}
}
