/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.frontend.web.jsf.ui;

import java.io.Serializable;

import javax.faces.application.Application;
import javax.faces.component.UIComponent;
import javax.faces.component.UIViewRoot;
import javax.faces.component.html.HtmlInputText;
import javax.faces.context.FacesContext;
import javax.faces.convert.Converter;
import javax.faces.convert.ConverterException;

import net.forwardfire.vasc.core.VascEntry;
import net.forwardfire.vasc.core.VascEntryField;
import net.forwardfire.vasc.core.ui.VascValueModel;
import net.forwardfire.vasc.core.VascException;
import net.forwardfire.vasc.frontend.web.jsf.JSFVascUIComponent;




/**
 * 
 * 
 * @author Willem Cazander
 * @version 1.0 Mar 25, 2009
 */
public class JSFText extends AbstractJSFBaseComponent {
	
	@Override
	public Object createComponent(VascEntry table,VascEntryField entryField,VascValueModel model,Object gui) throws VascException {
		Application application = FacesContext.getCurrentInstance().getApplication();
		UIViewRoot viewRoot = FacesContext.getCurrentInstance().getViewRoot();
		HtmlInputText component = (HtmlInputText)application.createComponent(HtmlInputText.COMPONENT_TYPE);
		component.setId(viewRoot.createUniqueId());
		componentId = component.getId();
		
		component.setConverter(new VascNullConverter());
		
		Integer sizeEdit = entryField.getSizeEdit();
		if (sizeEdit!=null) {
			component.setSize(sizeEdit);
		}
		((UIComponent)gui).getChildren().add(component);
		return component;
	}
	
	/**
	 * @see net.forwardfire.vasc.core.ui.VascUIComponent#isDisabled()
	 */
	@Override
	public boolean isDisabled() {
		HtmlInputText component = (HtmlInputText)JSFVascUIComponent.findComponentById(FacesContext.getCurrentInstance().getViewRoot(),componentId);
		return component.isDisabled();
	}
	
	/**
	 * @see net.forwardfire.vasc.core.ui.VascUIComponent#setDisabled(boolean)
	 */
	@Override
	public void setDisabled(boolean disabled) {
		HtmlInputText component = (HtmlInputText)JSFVascUIComponent.findComponentById(FacesContext.getCurrentInstance().getViewRoot(),componentId);
		component.setDisabled(disabled);
	}
}
class VascNullConverter implements Converter,Serializable {

	private static final long serialVersionUID = 61988136374095313L;
		
	/**
	 * Convert an ID to an Object
	 */
	@Override
	public Object getAsObject(FacesContext context,UIComponent component,String value) throws ConverterException {
		//System.out.println("getAsObject: "+component+" value: "+value);
		if (context == null) { throw new NullPointerException("context may not be null"); }
		if (component == null) { throw new NullPointerException("component may not be null"); }
		if (value == null) { return ""; }
		if ((value=value.trim()).length() == 0) { return ""; }
		return value;
	}
	
	/**
	 * Converts an Object to an ID.
	 */
	@Override
	public String getAsString(FacesContext context,UIComponent component,Object value) throws ConverterException {
		//System.out.println("getAsString: "+component+" value: "+value);
		if (context == null) { throw new NullPointerException("context may not be null"); }
		if (component == null) { throw new NullPointerException("component may not be null"); }
		if (value == null) { return ""; }
		return ""+value;
	}
	
	
}