/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.lib.i18n;

import java.util.List;
import java.util.Locale;

import net.forwardfire.vasc.lib.i18n.config.BundleConfig;
import net.forwardfire.vasc.lib.i18n.config.BundleConfigParser;

/**
 * AbstractRootApplicationBundle adds the logic to auto init the MergeableResourceBundle.
 * 
 * @author Willem Cazander
 * @version 1.0 May 8, 2012
 */
abstract public class AbstractRootApplicationBundle extends MergeableResourceBundle {

	private List<Locale> applicationSupportedLocales = null;
	private List<BundleConfig> bundleConfigs = null;
	
	/**
	 * The method needs to be implemented by the language class bundle of this.
	 * @return
	 */
	abstract public SupportedBundleLocale getSupportedBundleLocale();

	/**
	 * @see java.util.ResourceBundle#getLocale()
	 */
	@Override
	public Locale getLocale() {
		return getSupportedBundleLocale().toLocale();
	}

	/**
	 * @see com.mbuyu.m4n.i18n.MergeableResourceBundle#init()
	 */
	@Override
	protected void init() {
		clear();
		BundleConfigParser config = new BundleConfigParser();
		config.parseConfig();
		applicationSupportedLocales = config.getApplicationSupportedLocales();
		bundleConfigs = config.getBundleConfigs();
		for (BundleConfig bundleConfig:bundleConfigs) {
			addBundleData(config.loadBundleConfigData(bundleConfig,getLocale()));
		}
	}

	/**
	 * Reloads the config and the bundles.
	 */
	public void reload() {
		init();
	}
	
	/**
	 * Returns an collection of BundleConfig objects which are loaded.
	 * @return
	 */
	public List<BundleConfig> getBundleConfigs() {
		return bundleConfigs;
	}

	/**
	 * @return the applicationSupportedLocales
	 */
	public List<Locale> getApplicationSupportedLocales() {
		return applicationSupportedLocales;
	}
}
