/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.lib.i18n;

import java.util.Collections;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;
import java.util.ResourceBundle;

/**
 * MergeableResourceBundle is an ResourceBundle which can merge different bundles into one.
 * 
 * @author Willem Cazander
 * @version 1.0 Mar 8, 2012
 */
public class MergeableResourceBundle extends ResourceBundle {
	
	private Map<String,String> bundleData = null;
	
	/**
	 * Creates an new MergeableResourceBundle instance.
	 */
	public MergeableResourceBundle() {
		bundleData = new HashMap<String,String>(500);
		init();
	}
	
	/**
	 * Protected init method to hook on the lazy init loading of bundles.
	 */
	protected void init() {
	}
	
	/**
	 * Protected method to clear the bundle data.
	 */
	protected void clear() {
		bundleData.clear();
	}
	
	/**
	 * Returns copy of data of this loaded merged bundle.
	 */
	public Map<String,String> getBundleData() {
		Map<String,String> result = new HashMap<String,String>(bundleData.size());
		result.putAll(bundleData);
		return result;
	}
	
	/**
	 * @see java.util.ResourceBundle#handleGetObject(java.lang.String)
	 */
	@Override
	protected Object handleGetObject(String key) {
		return bundleData.get(key);
	}

	/**
	 * @see java.util.ResourceBundle#getKeys()
	 */
	@Override
	public Enumeration<String> getKeys() {
		return Collections.enumeration(bundleData.keySet());
	}
	
	/**
	 * Add all data from map to bundleData.
	 * @param dataMap
	 */
	public void addBundleData(Map<String,String> dataMap) {
		bundleData.putAll(dataMap);
	}
	
	/**
	 * Add all data from ResourceBundle to bundleData.
	 * @param bundle
	 */
	public void addBundleData(ResourceBundle bundle) {
		for (String key:bundle.keySet()) {
			String value = bundle.getString(key);
			bundleData.put(key, value);
		}
	}	
}
