/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.lib.i18n;

import java.util.Locale;

/**
 * SupportedBundleLocale defines all the supported locale's of the RootApplicationBundle.
 * 
 * @author Willem Cazander
 * @version 1.0 May 8, 2012
 */
public enum SupportedBundleLocale {
	
	en,
	de,
	fr,
	pt,
	it,
	ru,
	nl,
	es,
	pl,
	by,
	da,
	ro,
	sv;
	
	/**
	 * The default locale if an unsupported locale is requested.
	 */
	public static final SupportedBundleLocale DEFAULT_LOCALE = en;

	/**
	 * The locale object of this language.
	 */	
	private Locale locale = null;
	
	/**
	 * Private constructor for enum.
	 */
	private SupportedBundleLocale() {
		locale = new Locale(name()); 
	}
	
	/**
	 * The locale object of this enum language.
	 * @return
	 */
	public Locale toLocale() {
		return locale;
	}
}
