package net.forwardfire.vasc.lib.i18n.config;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URL;
import java.net.URLConnection;
import java.nio.charset.Charset;
import java.util.Locale;
import java.util.PropertyResourceBundle;
import java.util.ResourceBundle;
import java.util.ResourceBundle.Control;

/**
 * BundleCharSetControl load ResourceBundles with correct CharSet.
 * 
 * @author Willem Cazander
 * @version 1.0 May 8, 2012
 */
public class BundleCharSetControl extends Control {
	
	/** The default extension for bundles. */
	protected static final String BUNDLE_EXTENSION = "properties";
	
	/** The charSet to load with **/
	protected Charset charSet = null;
	
	/**
	 * Creates BundleCharSetControl to read ResourceBundle with charSet.
	 */
	public BundleCharSetControl(Charset charSet) {
		this.charSet=charSet;
	}
	
	/**
	 * Does the loading of the ResourceBundle
	 */
	@Override
    public ResourceBundle newBundle(String baseName, Locale locale, String format, ClassLoader loader, boolean reload) throws IllegalAccessException, InstantiationException, IOException {
        String bundleName = toBundleName(baseName, locale);
        String resourceName = toResourceName(bundleName, BUNDLE_EXTENSION);
        ResourceBundle bundle = null;
        InputStream stream = null;
        if (reload) {
            URL url = loader.getResource(resourceName);
            if (url != null) {
                URLConnection connection = url.openConnection();
                if (connection != null) {
                    connection.setUseCaches(false);
                    stream = connection.getInputStream();
                }
            }
        } else {
            stream = loader.getResourceAsStream(resourceName);
        }
        if (stream != null) {
            try {
                bundle = new PropertyResourceBundle(new InputStreamReader(stream, charSet));
            } finally {
                stream.close();
            }
        }
        return bundle;
    }
}
