package net.forwardfire.vasc.lib.i18n;

import java.util.List;
import java.util.Locale;
import java.util.ResourceBundle;

import net.forwardfire.vasc.lib.i18n.bundle.RootApplicationBundle;
import net.forwardfire.vasc.lib.i18n.config.BundleConfig;
import net.forwardfire.vasc.lib.i18n.config.BundleConfigParser;

import junit.framework.TestCase;

/**
 * Test some keys and loading options.
 * 
 * @author Willem Cazander
 * @version 1.0 May 8, 2012
 */
public class RootBundleTest extends TestCase {
	
	static private final String TEST_BUNDLE = RootApplicationBundle.URI;
	static private final Locale TEST_LOCALE = new Locale("en");
	static private final Locale TEST_LOCALE_NL = new Locale("nl");
	
	public void testParser() {
		BundleConfigParser p = new BundleConfigParser();
		p.parseConfig();
		assertEquals(2,p.getBundleConfigs().size());
		assertEquals(2,p.getApplicationSupportedLocales().size());
	}

	public void testBundleOrder() {
		BundleConfigParser p = new BundleConfigParser();
		p.parseConfig();
		List<BundleConfig> conf = p.getBundleConfigs();
		assertTrue(conf.isEmpty()==false);
		BundleConfig bundle = conf.get(0);
		assertEquals("bundle2",bundle.getName());
	}
	
	public void testBundleOrderValue() {
		ResourceBundle bundle = ResourceBundle.getBundle(TEST_BUNDLE,TEST_LOCALE);
		assertEquals("bundle1",bundle.getString("bundle.key"));
	}
	
	public void testValues_en() {
		ResourceBundle bundle = ResourceBundle.getBundle(TEST_BUNDLE,TEST_LOCALE);
		assertEquals(7,bundle.keySet().size());
		assertEquals("key1_en",bundle.getString("bundle1.key1"));
		assertEquals("key2_en",bundle.getString("bundle1.key2"));
		assertEquals("key3_en",bundle.getString("bundle1.key3"));
		assertEquals("key1_en",bundle.getString("bundle2.key1"));
		assertEquals("key2_en",bundle.getString("bundle2.key2"));
		assertEquals("key3_en",bundle.getString("bundle2.key3"));
	}

	public void testValues_nl() {
		ResourceBundle bundle = ResourceBundle.getBundle(TEST_BUNDLE,TEST_LOCALE_NL);
		assertEquals(7,bundle.keySet().size());
		assertEquals("key1_nl",bundle.getString("bundle1.key1"));
		assertEquals("key2_nl",bundle.getString("bundle1.key2"));
		assertEquals("key3_nl",bundle.getString("bundle1.key3"));
		assertEquals("key1_nl",bundle.getString("bundle2.key1"));
		assertEquals("key2_nl",bundle.getString("bundle2.key2"));
		assertEquals("key3_nl",bundle.getString("bundle2.key3"));
	}
}
