/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.lib.jr4o;


import javax.el.ValueExpression;

import net.forwardfire.vasc.lib.jr4o.data.JRDynamicDataSource;

import org.x4o.xml.X4ODriver;
import org.x4o.xml.element.ElementException;
import org.x4o.xml.lang.X4OLanguageSession;

/**
 * JR4OParser parses xml layer for JasperReports
 *
 * jr4o is currently not the nicest x4o language because it is ported from old version.
 *
 * @author Willem Cazander
 * @version 1.0 Aug 31, 2007
 */
public class JR4ODriver extends X4ODriver<JR4ODesignRoot> {
	
	/** The jr40 language name */
	static public final String LANGUAGE_NAME = "jr4o";
	static public final String[] LANGUAGE_VERSIONS = new String[]{X4ODriver.DEFAULT_LANGUAGE_VERSION};
	static public final String EL_DATASOURCE_NAME = "dynamicDataSource";
	
	/**
	 * creates an JR4OParser
	 */
	public JR4ODriver() {
	}
	
	@Override
	public String getLanguageName() {
		return LANGUAGE_NAME;
	}
	
	@Override
	public String[] getLanguageVersions() {
		return LANGUAGE_VERSIONS;
	}
	
	/**
	 * Returns the JRDynamicDataSource for the JRDynamic*Element classes.
	 * @param elementLanguage	The ElementLanguage to get the datasource from.
	 * @return The JRDynamicDataSource of this parser instance.
	 */
	static public JRDynamicDataSource getELDataSource(X4OLanguageSession elementLanguage) throws ElementException {
		ValueExpression ee = elementLanguage.getExpressionLanguageFactory().createValueExpression(elementLanguage.getExpressionLanguageContext(),"${"+EL_DATASOURCE_NAME+"}",JRDynamicDataSource.class);
		JRDynamicDataSource dataSource = (JRDynamicDataSource)ee.getValue(elementLanguage.getExpressionLanguageContext());
		if (dataSource==null) {
			throw new ElementException("Could not find dataSouce in context as: "+EL_DATASOURCE_NAME);
		}
		return dataSource;
	}
}