/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.lib.jr4o.data;

import java.util.HashMap;
import java.util.Map;

import javax.swing.table.TableModel;

import net.sf.jasperreports.engine.JRException;
import net.sf.jasperreports.engine.JRField;
import net.sf.jasperreports.engine.design.JRDesignField;
import net.sf.jasperreports.engine.design.JasperDesign;

/**
 * JRDynamicDataSourceTableModel is the dynamic datasource which supportes dynamic columns in Jasper reports.
 *
 * @author Willem Cazander
 * @version 1.0 Aug 31, 2007
 */
public class JRDynamicDataSourceTableModel extends AbstractJRDynamicDataSource {
	

	private TableModel tableModel = null;
	private int index = -1;
	private int[] columnWidths = null;
	private static final String PREFIX_COLUMN = "COLUMN_";
	private static final String PREFIX_HEADER = "HEADER_";
	private static final String INFIX_CLASS = "CLASS_";

	public JRDynamicDataSourceTableModel(TableModel tableModel,int[] columnWidths) {
		if (tableModel==null) {
			throw new NullPointerException("TableModel can't be null.");
		}
		this.tableModel = tableModel;
		this.columnWidths = columnWidths;
	}

	/**
	 * @see net.sf.jasperreports.engine.JRDataSource#getFieldValue(net.sf.jasperreports.engine.JRField)
	 */
	public Object getFieldValue(JRField jrField) throws JRException {
		String fieldName = jrField.getName();
		if (fieldName.startsWith(PREFIX_COLUMN)) {
			return tableModel.getValueAt(index, Integer.parseInt(fieldName.substring(7)));
		}
		if (fieldName.startsWith(PREFIX_HEADER)) {
			return tableModel.getColumnName(Integer.parseInt(fieldName.substring(7)));
		}
		throw new JRException("Unknown column name : " + fieldName);
	}

	/**
	 * @see net.forwardfire.vasc.lib.jr4o.data.JRDynamicDataSource#addDynamicColumnClassFields(net.sf.jasperreports.engine.design.JasperDesign)
	 */
	public void addDynamicColumnClassFields(JasperDesign jd) throws JRException {
		JRDesignField field;
		for (int i=0;i<tableModel.getColumnCount();i++) {
			field = new JRDesignField();
			field.setName(PREFIX_COLUMN+i);
			field.setValueClass(tableModel.getColumnClass(i));
			jd.addField(field);
			
			field = new JRDesignField();
			field.setName(PREFIX_HEADER+i);
			field.setValueClass(String.class);
			jd.addField(field);
		}
	}

	/**
	 * @see net.forwardfire.vasc.lib.jr4o.data.JRDynamicDataSource#addDynamicELBean()
	 */
	public Map<String, Object> addDynamicELBean() {
		Map<String,Object> result = new HashMap<String,Object>(10);
		for (int i=0;i<tableModel.getColumnCount();i++) {
			result.put(PREFIX_COLUMN+INFIX_CLASS+i,tableModel.getColumnClass(i));
			result.put(PREFIX_HEADER+INFIX_CLASS+i,String.class);
		}
		return result;
	}

	/**
	 * @see net.forwardfire.vasc.lib.jr4o.data.JRDynamicDataSource#getDynamicColumnX(int)
	 */
	public int getDynamicColumnX(int col) {
		int result = 0;
		for (int i=0;i<col;i++) {
			if (i > columnWidths.length) {
				break;
			}
			result += columnWidths[i];
		}
		return result;
	}

	/**
	 * @see net.forwardfire.vasc.lib.jr4o.data.JRDynamicDataSource#getDynamicColumnWidth(int)
	 */
	public int getDynamicColumnWidth(int col) {
		if (col > columnWidths.length) {
			return 50;
		}
		return columnWidths[col];
	}

	/**
	 * @see net.forwardfire.vasc.lib.jr4o.data.JRDynamicDataSource#getDynamicColumnCount()
	 */
	public int getDynamicColumnCount() {
		return tableModel.getColumnCount();
	}

	/**
	 * @see net.sf.jasperreports.engine.JRDataSource#next()
	 */
	public boolean next() throws JRException {
		index++;
		return (index < tableModel.getRowCount());
	}
	
	/**
	 * @see net.sf.jasperreports.engine.JRRewindableDataSource#moveFirst()
	 */
	public void moveFirst() throws JRException {
		index = -1;
	}
}