/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.lib.jr4o.x4o;

import org.x4o.xml.element.AbstractElementBindingHandler;
import org.x4o.xml.element.Element;
import org.x4o.xml.element.ElementBindingHandlerException;

import net.sf.jasperreports.engine.JRChild;
import net.sf.jasperreports.engine.design.JRDesignElement;
import net.sf.jasperreports.engine.design.JRDesignElementGroup;

/**
 * JR4OElementGroupBindingHandler
 *
 * @author Willem Cazander
 * @version 1.0 Aug 31, 2007
 */
public class JR4OElementGroupBindingHandler extends AbstractElementBindingHandler<JRDesignElementGroup> {
	
	private final static Class<?>[] CLASSES_CHILD = new Class[] {
		JRDesignElement.class,
		JRDesignElementGroup.class
	};
	
	/**
	 * @see org.x4o.xml.element.ElementBindingHandler#getBindParentClass()
	 */
	public Class<?> getBindParentClass() {
		return JRDesignElementGroup.class;
	}

	/**
	 * @see org.x4o.xml.element.ElementBindingHandler#getBindChildClasses()
	 */
	public Class<?>[] getBindChildClasses() {
		return CLASSES_CHILD;
	}
	
	@Override
	public void bindChild(Element childElement, JRDesignElementGroup parentObject,Object childObject) throws ElementBindingHandlerException {
		if (childObject instanceof JRDesignElementGroup) {
			parentObject.addElementGroup((JRDesignElementGroup)childObject);
		}
		if (childObject instanceof JRDesignElement) {
			parentObject.addElement((JRDesignElement)childObject);
		}
	}
	
	@Override
	public void createChilderen(Element parentElement, JRDesignElementGroup parentObject)	throws ElementBindingHandlerException {
		for (JRChild child:parentObject.getChildren()) {
			if (child instanceof JRDesignElement) {
				createChild(parentElement, child);
			}
			if (child instanceof JRDesignElementGroup) {
				createChild(parentElement, child);
			}
		}
	}
}
