/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.lib.jr4o.x4o;

import org.x4o.xml.element.AbstractElementBindingHandler;
import org.x4o.xml.element.Element;
import org.x4o.xml.element.ElementBindingHandlerException;

import net.sf.jasperreports.engine.JRField;
import net.sf.jasperreports.engine.JRParameter;
import net.sf.jasperreports.engine.JRStyle;
import net.sf.jasperreports.engine.design.JasperDesign;

/**
 * JR4OJasperDesignBindingHandler
 *
 * @author Willem Cazander
 * @version 1.0 Aug 31, 2007
 */
public class JR4OJasperDesignBindingHandler extends AbstractElementBindingHandler<JasperDesign> {
	
	private final static Class<?>[] CLASSES_CHILD = new Class[] {
		JRField.class,
		JRParameter.class,
		JRStyle.class
	};
	
	/**
	 * @see org.x4o.xml.element.ElementBindingHandler#getBindParentClass()
	 */
	public Class<?> getBindParentClass() {
		return JasperDesign.class;
	}

	/**
	 * @see org.x4o.xml.element.ElementBindingHandler#getBindChildClasses()
	 */
	public Class<?>[] getBindChildClasses() {
		return CLASSES_CHILD;
	}
	
	@Override
	public void bindChild(Element childElement, JasperDesign parentObject,Object childObject) throws ElementBindingHandlerException {
		try {
			if (childObject instanceof JRField) {
				parentObject.addField((JRField)childObject);	
			}
			if (childObject instanceof JRParameter) {
				parentObject.addParameter((JRParameter)childObject);
			}
			if (childObject instanceof JRStyle) {	
				parentObject.addStyle((JRStyle)childObject);
			}
		} catch (Exception e) {
			throw new ElementBindingHandlerException(e.getMessage(),e);
		}
	}
	
	@Override
	public void createChilderen(Element parentElement, JasperDesign parentObject)	throws ElementBindingHandlerException {
		for (JRField field:parentObject.getFieldsList()) {
			createChild(parentElement, field);
		}
		for (JRParameter parameter:parentObject.getParametersList()) {
			createChild(parentElement, parameter);
		}
		for (JRStyle style:parentObject.getStylesList()) {
			createChild(parentElement, style);
		}
	}
}
