/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.lib.jr4o.x4o;

import javax.el.ValueExpression;

import org.x4o.xml.element.AbstractElement;
import org.x4o.xml.element.ElementException;

import net.forwardfire.vasc.lib.jr4o.JR4ODriver;
import net.forwardfire.vasc.lib.jr4o.data.JRDynamicDataSource;
import net.sf.jasperreports.engine.JRStyle;
import net.sf.jasperreports.engine.design.JRDesignElementGroup;
import net.sf.jasperreports.engine.design.JRDesignExpression;
import net.sf.jasperreports.engine.design.JRDesignTextField;
import net.sf.jasperreports.engine.type.ModeEnum;

/**
 * JRDynamicColumnElement
 *
 * @author Willem Cazander
 * @version 1.0 Sep 24, 2007
 */
public class JRDynamicColumnElement extends AbstractElement {

	@Override
	public void doElementRun() throws ElementException {
		JRDynamicDataSource dataSource = JR4ODriver.getELDataSource(getLanguageSession());

		ValueExpression ee = getLanguageSession().getExpressionLanguageFactory().createValueExpression(getLanguageSession().getExpressionLanguageContext(),"${Arial_Bold}",JRStyle.class);
		JRStyle style = (JRStyle)ee.getValue(getLanguageSession().getExpressionLanguageContext());
		if (style==null) {
			throw new ElementException("Could not get the font.");
		}
		style.setMode(ModeEnum.OPAQUE);
		
		Object parent = getParent().getElementObject();
		if (parent instanceof JRDesignElementGroup == false) {
			throw new ElementException("parent is not an JRDesignElementGroup.");
		}
		JRDesignElementGroup group = (JRDesignElementGroup)parent;
		
		int max = dataSource.getDynamicColumnCount();
		for (int i=0;i<max;i++) {
			JRDesignTextField column = new JRDesignTextField();
			column.setX(dataSource.getDynamicColumnX(i));
			column.setY(dataSource.getDynamicColumnY(i));
			column.setWidth(dataSource.getDynamicColumnWidth(i));
			column.setHeight(dataSource.getDynamicColumnHeight(i));
			column.setStretchWithOverflow(dataSource.isDynamicColumnStretchWithOverflow(i));
			column.setBlankWhenNull(dataSource.isDynamicColumnBlankWhenNull(i));
			column.setHorizontalAlignment(dataSource.getDynamicColumnHorizontalAlignment(i));
			column.setStyle(style);
			
			JRDesignExpression ex = new JRDesignExpression();
			ex.setText("$F{COLUMN_"+i+"}");
			ex.setValueClass(String.class);
			column.setExpression(ex);
			
			group.addElement(column);
		}
	}
}
