/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.xpql;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import net.forwardfire.vasc.xpql.query.Query;


/**
 * Holds all the queries.
 * 
 * @author Willem Cazander
 * @version 1.0 Aug 30, 2010
 */
public class QueryStore {

	private Map<String,Query> queryMap = new HashMap<String,Query>(100);
	
	public void addQuery(Query query) {
		if (query==null) {
			throw new NullPointerException("Can't add null query.");
		}
		if (query.getName()==null) {
			throw new NullPointerException("Can't add query with null name.");
		}
		if (query.getName().isEmpty()) {
			throw new IllegalArgumentException("Can't add query with empty name.");
		}
		if (query.getQueryStore()!=null) {
			throw new IllegalArgumentException("Query has already an store.");
		}
		query.setQueryStore(this);
		queryMap.put(query.getName(),query);
	}
	
	public Collection<Query> getQueries() {
		return queryMap.values();
	}
	
	public Query getQuery(String name) {
		return queryMap.get(name);
	}
	
	public Set<String> getQueryNames() {
		return queryMap.keySet();
	}
	
	public void addAll(QueryStore store) {
		for (String key:store.getQueryNames()) {
			Query q = store.getQuery(key);
			addQuery(q);
		}
	}
}