/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */

package net.forwardfire.vasc.xpql.execute.jpa;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.logging.Logger;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import javax.ejb.SessionContext;
import javax.ejb.Stateless;
import javax.ejb.TransactionManagement;
import javax.ejb.TransactionManagementType;
import javax.persistence.EntityManager;
import javax.transaction.UserTransaction;

import org.x4o.xml.io.X4OReader;

import net.forwardfire.vasc.xpql.QueryStore;
import net.forwardfire.vasc.xpql.XPQLDriver;
import net.forwardfire.vasc.xpql.query.Query;
import net.forwardfire.vasc.xpql.query.QueryParameterValue;



/**
 * XpqlQueryManagerImpl
 * 
 * @author Willem Cazander
 * @version 1.0 16 Jul 2007
 */
@Stateless(name="ejb/xpqlQueryManager")
@TransactionManagement(TransactionManagementType.BEAN)
public class XpqlQueryManagerImpl implements XpqlQueryManager.ILocal,XpqlQueryManager.IRemote {

	private static final Logger logger = Logger.getLogger(XpqlQueryManagerImpl.class.getName());
	//private EntityManager entityManager;
	
	@Resource
	private UserTransaction utx;
	
	@Resource
	private SessionContext context;
	
	private Map<String,Query> queries = new HashMap<String,Query>(100);
	
	@PostConstruct
	public void loadAll() {
		logger.fine("XtesManager created now loading resources...");
		try {
			long s = System.currentTimeMillis();
			
			Map<String,String> keys = loadKeys(); 
			for (String key:keys.keySet()) {
				String value = keys.get(key);
				if (key.startsWith("load")) {
					loadResource(value);
				}
				if ("persistenceUnit".equals(key)) {
					logger.fine("getting EntityManager: RMED."+value);
					//entityManager =  EntityManager.class.cast(context.lookup(value));
					//entityManager = (EntityManager)new InitialContext().lookup(value);
				}
			}
			long t = System.currentTimeMillis()-s;
			logger.info("Total loaded xpql queries: "+queries.size()+" in "+t+" ms.");
		} catch (Exception e) {
			throw new RuntimeException("Error while load xpql resources error: "+e.getMessage(),e);
		}
	}
	
	/**
	 * Loads xtes-xpql-ejb3.xml from meta-inf and gives the keys 
	 */
	protected Map<String,String> loadKeys() throws IOException {
		
		Properties properties = new Properties();
		logger.fine("Getting urls");
		Enumeration<URL> e = Thread.currentThread().getContextClassLoader().getResources("META-INF/xtes-xpql-ejb3.xml");
		while(e.hasMoreElements()) {
			URL u = e.nextElement();
			logger.finer("Loading reletive namespaces of: "+u+" for: META-INF/xtes-xpql-ejb3.xml");
			InputStream in = u.openStream();
			try {
				properties.loadFromXML(in);
			} finally {
				if (in!=null) {
					in.close();
				}
			}
		}
		
		e = Thread.currentThread().getContextClassLoader().getResources("/META-INF/xtes-xpql-ejb3.xml");
		while(e.hasMoreElements()) {
			URL u = e.nextElement();
			logger.finer("Loading root namespaces of: "+u+" for: /META-INF/xtes-xpql-ejb3.xml");
			InputStream in = u.openStream();
			try {
				properties.loadFromXML(in);
			} finally {
				if (in!=null) {
					in.close();
				}
			}
		}
		logger.fine("done loading");
		
		Map<String,String> result = new HashMap<String,String>(20);
		for (Object key:properties.keySet()) {
			if (key instanceof String) {
				String key2 = (String) key;
				String value = properties.getProperty(key2);
				result.put(key2, value);
			}
		}
		return result;
	}
	
	private void loadResource(String resource) {
		XPQLDriver driver = XPQLDriver.getInstance();
		X4OReader<QueryStore> reader = driver.createReader();
		QueryStore store = null;
		logger.fine("Try loading: "+resource);
		try {
			store = reader.readResource(resource);
		} catch (Exception e) {
			throw new RuntimeException("Could not load xpql resource: "+resource+" error: "+e.getMessage(),e);
		}
		
		for (String key:store.getQueryNames()) {
			Query q = store.getQuery(key);
			
			queries.put(q.getName(),q);
			
			String out = "\n\n=========== SQL: "+q.getName()+"\n";
			//out +=q.toSQL(q);
			out +="=========== Edit: "+q.getName()+"\n";
			out +=q.toEdit(q);
			out +="=========== Prepard: "+q.getName()+"\n";
			out +=q.toPreparedSQL(q)+"\n";
			logger.finer("FOUND Query: "+out);
		}
	}
	
	@Override
	public Query getQuery(String name) {
		Query q = queries.get(name);
		return q;
	}
	
	@SuppressWarnings("unchecked")
	@Override
	public List<Object> execute(EntityManager entityManager,Query query) {
		String sql = query.toPreparedSQL(query);
		logger.finer("Executing: "+query.getName()+" sql: "+sql);
		javax.persistence.Query q2 = null;
		if (Query.QueryType.hql==query.getType()) {
			q2 = entityManager.createQuery(sql);
		}
		if (Query.QueryType.sql==query.getType()) {
			q2 = entityManager.createNativeQuery(sql);	
		}
		
		// TODO: move this hack to xtest xpql
		if (query.getProperty("limit")!=null) {
			int max = new Integer(query.getPropertyString("limit"));
			q2.setMaxResults(max);		
		}
		if (query.getProperty("offset")!=null) {
			int offset = new Integer(query.getPropertyString("offset"));
			q2.setFirstResult(offset);		
		}
		
		List<QueryParameterValue> options = query.getOrderQueryParameterValues();
		int i=1;
		for ( QueryParameterValue value : options ) {
			Object para = value.getValue();
			if(para==null) {
				para = value.getDefaultValue();
			}
			logger.finest( "nr: " + i + " value: " + para+" name: "+value.getName());
			q2.setParameter(i,para);
			i++;				
		}
		return q2.getResultList();
	}
	
	@Override
	public Object executeObject(EntityManager entityManager,Query query) {
		String sql = query.toPreparedSQL(query);
		logger.finer("Executing: "+query.getName()+" sql: "+sql);
		javax.persistence.Query q2 = null;
		if (Query.QueryType.hql==query.getType()) {
			q2 = entityManager.createQuery(sql);
		}
		if (Query.QueryType.sql==query.getType()) {
			q2 = entityManager.createNativeQuery(sql);	
		}
		
		// TODO: move this hack to xtest xpql
		if (query.getProperty("limit")!=null) {
			int max = new Integer(query.getPropertyString("limit"));
			q2.setMaxResults(max);		
		}
		if (query.getProperty("offset")!=null) {
			int offset = new Integer(query.getPropertyString("offset"));
			q2.setFirstResult(offset);		
		}
		
		List<QueryParameterValue> options = query.getOrderQueryParameterValues();
		int i=1;
		for ( QueryParameterValue value : options ) {
			Object para = value.getValue();
			if(para==null) {
				para = value.getDefaultValue();
			}
			logger.finest( "nr: " + i + " value: " + para+" name: "+value.getName());
			q2.setParameter(i,para);
			i++;				
		}
		try {
			utx.begin();
			Object result = null;
			try {
				result = q2.getSingleResult();
				utx.commit();
			} finally {
				if (result==null) {
					utx.rollback();
				}
			}
			return result;
		} catch (Exception se) {
			throw new RuntimeException("Transaction error: "+se.getMessage(),se);
		}
	}
	
	@Override
	public List<Object> execute(EntityManager entityManager,String name,Map<String,Object> parameters) {
		Query query = getQuery(name);
		if (parameters!=null) {
			for (String key:parameters.keySet()) {
				Object value = parameters.get(key);
				query.setQueryParameter(key, value);
			}
		}
		return execute(entityManager,query);
	}
	
	@Override
	public Object executeObject(EntityManager entityManager,String name,Map<String,Object> parameters) {
		Query query = getQuery(name);
		if (parameters!=null) {
			for (String key:parameters.keySet()) {
				Object value = parameters.get(key);
				query.setQueryParameter(key, value);
			}
		}
		return executeObject(entityManager,query);
	}
	
	@Override
	public Integer executeUpdate(EntityManager entityManager,Query query) {
		String sql = query.toPreparedSQL(query);
		logger.fine("Executing: "+query.getName()+" sql: "+sql);
		try {
			javax.persistence.Query q2 = null;
			if (Query.QueryType.hql==query.getType()) {
				q2 = entityManager.createQuery(sql);
				List<QueryParameterValue> options = query.getOrderQueryParameterValues();
				int i=1;
				for ( QueryParameterValue value : options ) {
					Object para = value.getValue();
					if(para==null) {
						para = value.getDefaultValue();
					}
					logger.finest( "nr: " + i + " value: " + para+" name: "+value.getName());
					q2.setParameter(i,para);
					i++;
				}
				utx.begin();
				Integer result = q2.executeUpdate();
				utx.commit();
				return result;
			}
			
			if (Query.QueryType.sql==query.getType()) {
				q2 = entityManager.createNativeQuery(sql);
				List<QueryParameterValue> options = query.getOrderQueryParameterValues();
				int i=1;
				for ( QueryParameterValue value : options ) {
					Object para = value.getValue();
					if(para==null) {
						para = value.getDefaultValue();
					}
					logger.finest( "nr: " + i + " value: " + para+" name: "+value.getName());
					q2.setParameter(i,para);
					i++;
				}
				utx.begin();
				Integer result = q2.executeUpdate();
				utx.commit();
				return result;
			}
		} catch (Exception se) {
			throw new RuntimeException("Transaction error: "+se.getMessage(),se);
		}
		throw new IllegalStateException("unknow query type");
	}
	
	@Override
	public Integer executeUpdate(EntityManager entityManager,String name,Map<String, Object> parameters) {
		Query query = getQuery(name);
		for (String key:parameters.keySet()) {
			Object value = parameters.get(key);
			query.setQueryParameter(key, value);
		}
		return executeUpdate(entityManager,query);
	}
}
