/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.xpql.impl;

import net.forwardfire.vasc.xpql.query.AbstractQuery;
import net.forwardfire.vasc.xpql.query.Query;
import net.forwardfire.vasc.xpql.query.QueryParameterValue;
import net.forwardfire.vasc.xpql.query.QueryPart;

/**
 * This class defines an executable SQL query.
 * 
 * @author Willem Cazander
 * @version 1.0 Mar 4, 2007
 * @since 1.0
 */
public class DefaultQuery extends AbstractQuery {
	
	/**
	 * @see net.forwardfire.vasc.xpql.query.AbstractQueryPart#toEdit()
	 */
	@Override
	public String toEdit(Query query) {
		StringBuffer queryBuffer = new StringBuffer();
		for ( QueryPart queryPart:getQueryParts() ) {
			queryBuffer.append( queryPart.toEdit(query) );
		}
		return queryBuffer.toString();
	}
	
	/**
	 * @see net.forwardfire.vasc.xpql.query.AbstractQueryPart#toPreparedSQL()
	 */
	@Override
	public String toPreparedSQL(Query query) {
		this.getOrderQueryParameterValues().clear();
		StringBuffer queryBuffer = new StringBuffer();
		for ( QueryPart queryPart:getQueryParts() ) {
			queryBuffer.append( queryPart.toPreparedSQL(query) );
		}
		return queryBuffer.toString();
	}
	
	/**
	 * @see net.forwardfire.vasc.xpql.query.AbstractQueryPart#toSQL()
	 */
	@Override
	public String toSQL(Query query) {
		StringBuffer queryBuffer = new StringBuffer();
		for ( QueryPart queryPart:getQueryParts() ) {
			queryBuffer.append( queryPart.toSQL(query) );
		}
		return queryBuffer.toString();
	}
	
	@Override
	public Query clone() throws CloneNotSupportedException {
		Query query = new DefaultQuery();
		//query.setTemplateStore(getTemplateStore());
		query.setName(getName());
		query.setType(getType());
		
		for (String key:getPropertyKeys()) {
			query.setProperty(key,getProperty(key));
		}
		for (QueryPart part:getQueryParts()) {
			query.addQueryPart(part.clone());
		}
		for (String com:getQueryComments()) {
			query.addQueryComment(com);
		}
		
		for (QueryParameterValue q:getLocalQueryParameterValues()) {
			query.addQueryParameterValue(q.clone());
		}
		return query;
	}
}
