/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.xpql.impl;

import org.x4o.xml.io.X4OReader;

import net.forwardfire.vasc.xpql.QueryStore;
import net.forwardfire.vasc.xpql.XPQLDriver;
import net.forwardfire.vasc.xpql.query.Query;
import net.forwardfire.vasc.xpql.query.QueryPart;

/**
 * Handled included query parts
 * 
 * @author Willem Cazander
 * @version 1.0 Mar 4, 2007
 */
public class QueryInclude implements QueryPart {
	
	private String name = null;
	private String resource = null;
	
	/**
	 * @return the name
	 */
	public String getName() {
		return name;
	}
	
	/**
	 * @param name the name to set
	 */
	public void setName(String name) {
		this.name = name;
	}
	
	/**
	 * @return the resource
	 */
	public String getResource() {
		return resource;
	}
	
	/**
	 * @param resource the resource to set
	 */
	public void setResource(String resource) {
		this.resource = resource;
	}
	
	@Override
	public String toSQL(Query query) {
		if (query.getQueryStore().getQuery(getName())==null) {
			if (resource!=null) {
				try {
					XPQLDriver driver = XPQLDriver.getInstance();
					X4OReader<QueryStore> reader = driver.createReader();
					QueryStore store = reader.readResource(resource);
					query.getQueryStore().addAll(store);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		}
		return query.getQueryStore().getQuery(getName()).toSQL(query);
	}
	
	@Override
	public String toPreparedSQL(Query query) {
		if (query.getQueryStore().getQuery(getName())==null) {
			if (resource!=null) {
				try {
					XPQLDriver driver = XPQLDriver.getInstance();
					X4OReader<QueryStore> reader = driver.createReader();
					QueryStore store = reader.readResource(resource);
					query.getQueryStore().addAll(store);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		}
		return query.getQueryStore().getQuery(getName()).toPreparedSQL(query);
	}
	
	@Override
	public String toEdit(Query query) {
		if (resource==null) {
			return "@inc:"+name;
		}
		return "@inc:"+name+"@"+resource;
	}
	
	@Override
	public QueryPart clone() throws CloneNotSupportedException {
		QueryInclude include = new QueryInclude();
		include.setName(getName());
		include.setResource(getResource());
		return include;
	}
}
