/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.xpql.impl;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import net.forwardfire.vasc.xpql.query.Query;
import net.forwardfire.vasc.xpql.query.Query.QueryType;
import net.forwardfire.vasc.xpql.query.QueryParameterValue;
import net.forwardfire.vasc.xpql.query.QueryPart;

/**
 * QueryParameter.
 * 
 * @author Willem Cazander
 * @version 1.0 Mar 4, 2007
 */
public class QueryParameter implements QueryPart {
	
	private String name = null;
	
	/**
	 * @return the name
	 */
	public String getName() {
		return name;
	}
	
	/**
	 * @param name the name to set
	 */
	public void setName(String name) {
		this.name = name;
	}
	
	@SuppressWarnings("unchecked")
	@Override
	public String toSQL(Query query) {
		QueryParameterValue value = query.getQueryParameterValue(getName());
		if ( value == null ) {
			return "";
		}
		if ( value.getValue() == null) {
			if (value.getDefaultValue() == null ) {
				return "";
			}
			return value.getDefaultValue().toString();
		}
		
		if (value.getType()==null) {
			value.setType(QueryParameterValue.QueryParameterType.parameter);
		}
		if (value.getType()==QueryParameterValue.QueryParameterType.parameterList) {
			List valueList = null;
			if (value.getValue() instanceof List) {
				valueList = (List)value.getValue();
			}
			if (value.getValue() instanceof String) {
				valueList = new ArrayList();
				String[] array = ((String)value.getValue()).split(",");
				for (String a:array) {
					valueList.add(a);	
				}
			}
			if (valueList==null) {
				valueList = new ArrayList();
				valueList.add(value.getValue());
			}
			StringBuilder builder = new StringBuilder(valueList.size()+3);
			Iterator i = valueList.iterator();
			while (i.hasNext()) {
				Object v = i.next();
				builder.append(v.toString());
				if (i.hasNext()) {
					builder.append(',');
				}
			}
			return builder.toString();	
		}
		
		return value.getValue().toString();
	}
	
	@SuppressWarnings("unchecked")
	@Override
	public String toPreparedSQL(Query query) {
		
		QueryParameterValue value = query.getQueryParameterValue(getName());
		if ( value == null ) {
			return "";
		}
		if (value.getType()==null) {
			value.setType(QueryParameterValue.QueryParameterType.parameter);
		}
		
		String result = null;
		switch (value.getType()) {
			default:
			case parameter:
				//if (query.getOrderQueryParameterValues().contains(value)==false) {
				query.addOrderQueryParameterValue(value);
				if (QueryType.hql.equals(query.getType())) {
					result =  "?"+query.getOrderQueryParameterValues().size();
				} else {
					result =  "?";
				}
				break;
			case parameterList:
				
				// add extra method so , toObjectList, which does al the prepare work.
				
				Object valueO = value.getValue();
				if (valueO==null) {
					return "";
				}
				List valueList = null;
				if (valueO instanceof List) {
					valueList = (List)valueO;
				}
				if (valueO instanceof String) {
					valueList = new ArrayList();
					String[] array = ((String)valueO).split(",");
					for (String a:array) {
						valueList.add(a);	
					}
				}
				if (valueList==null) {
					valueList = new ArrayList();
					valueList.add(value);
				}
				
			
				//boolean doAdd = false;
				//if (query.getOrderQueryParameterValues().contains(value)==false) {
				//}
				
				StringBuilder builder = new StringBuilder(valueList.size()+3);
				Iterator i = valueList.iterator();
				while (i.hasNext()) {
					Object v = i.next();
					//if (doAdd) {
						QueryParameterValue vv = new DefaultQueryParameterValue();
						vv.setName(value.getName());
						vv.setValue(v);
						vv.setType(QueryParameterValue.QueryParameterType.parameter);
						query.addOrderQueryParameterValue(vv);
					//}
					if (QueryType.hql.equals(query.getType())) {
						builder.append("?"+query.getOrderQueryParameterValues().size());
					} else {
						builder.append('?');
					}
					if (i.hasNext()) {
						builder.append(',');
					}
				}
				result = builder.toString();
				break;
			case raw:
				// The structural type of a "value" indicates that we're dealing with a SQL construct 
				// instead of a 'value' in the sense of values for prepared statements. In the XPQL dialect,
				// options can be structure as well as values.
				if (value.getValue()!=null) {
					result =  value.getValue().toString();
				} else if (value.getDefaultValue()!=null) {
					result =  value.getDefaultValue().toString();
				} else {
					result = "";
				}
				break;
			case xpql:
				result = null;
				break;
		}
		return result;
	}
	
	@Override
	public String toEdit(Query query) {
		return "@"+getName();
	}
	
	@Override
	public QueryPart clone() throws CloneNotSupportedException {
		QueryParameter p = new QueryParameter();
		p.setName(getName());
		return p;
	}
}
