/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.xpql.impl.x4o;

import net.forwardfire.vasc.xpql.impl.QueryInclude;
import net.forwardfire.vasc.xpql.impl.QueryParameter;
import net.forwardfire.vasc.xpql.query.Query;
import net.forwardfire.vasc.xpql.query.QueryParameterValue;
import net.forwardfire.vasc.xpql.query.QueryPart;

import org.x4o.xml.element.AbstractElementBindingHandler;
import org.x4o.xml.element.Element;
import org.x4o.xml.element.ElementBindingHandlerException;


/**
 * QueryBindingHandler binds query elements.
 * 
 * @author Willem Cazander
 * @version 1.0 Jan 21, 2007
 */
public class QueryBindingHandler extends AbstractElementBindingHandler<Query> {
	
	private final static Class<?>[] CLASSES_CHILD = new Class[] {
		QueryPart.class,
		QueryParameterValue.class
	};
	
	/**
	 * @see org.x4o.xml.element.ElementBindingHandler#getBindParentClass()
	 */
	@Override
	public Class<?> getBindParentClass() {
		return Query.class;
	}

	/**
	 * @see org.x4o.xml.element.ElementBindingHandler#getBindChildClasses()
	 */
	@Override
	public Class<?>[] getBindChildClasses() {
		return CLASSES_CHILD;
	}
	
	@Override
	public void bindChild(Element childElement, Query parentObject,Object childObject) throws ElementBindingHandlerException {
		// skip those with different life cycle because of sql characters adding elements in sax phase.
		if (childObject instanceof QueryParameter) {
			return;
		}
		if (childObject instanceof QueryInclude) {
			return;
		}
		if (childObject == parentObject) {
			return; // skip sql element
		}
		if (childObject instanceof QueryPart) {
			parentObject.addQueryPart((QueryPart)childObject);
		}
		if (childObject instanceof QueryParameterValue) {
			parentObject.addLocalQueryParameterValue((QueryParameterValue)childObject);
			parentObject.addQueryParameterValue((QueryParameterValue)childObject);
		}
	}
	
	@Override
	public void createChilderen(Element parentElement, Query parentObject)	throws ElementBindingHandlerException {
		for (QueryPart part:parentObject.getQueryParts()) {
			createChild(parentElement, part);
		}
		for (QueryParameterValue value:parentObject.getQueryParameterValues()) {
			createChild(parentElement, value);
		}
	}
}
