/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.xpql.query;

import java.util.Collection;
import java.util.List;
import java.util.Map;

import net.forwardfire.vasc.xpql.QueryStore;


/**
 * Query which is build out of QueryParts.
 * 
 * @author Willem Cazander
 * @version 1.0 Mar 4, 2007
 */
public interface Query extends QueryPart {
	
	static public enum QueryType {
		sql,
		hql
	}
	
	void setName(String name);
	String getName();
	
	void setType(QueryType type);
	QueryType getType();
	
	void addQueryPart(QueryPart queryPart);
	List<QueryPart> getQueryParts();
	
	void addQueryComment(String comment);
	List<String> getQueryComments();
	
	void addQueryParameterValue(QueryParameterValue value);
	void addLocalQueryParameterValue(QueryParameterValue value);
	void addOrderQueryParameterValue(QueryParameterValue value);
	Collection<QueryParameterValue> getQueryParameterValues();
	Collection<QueryParameterValue> getLocalQueryParameterValues();
	List<QueryParameterValue> getOrderQueryParameterValues();
	QueryParameterValue getQueryParameterValue(String name);
	
	void setQueryParameter(String name,Object value);
	void setQueryParameters(Map<String,Object> parameters);
	List<String> getQueryParameters();
	
	void setProperty(String name,Object value);
	Object getProperty(String name);
	String getPropertyString(String name);
	Collection<String> getPropertyKeys();
	
	void setQueryStore(QueryStore store);
	QueryStore getQueryStore();
	
	/**
	 * Force impl to have public clone methode
	 * @return
	 * @throws CloneNotSupportedException
	 */
	@Override
	Query clone() throws CloneNotSupportedException;
}
