/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.xpql.query;

/**
 * A QueryParameterValue
 * 
 * 
 * @author Willem Cazander
 * @version 1.0 Mar 4, 2007
 */
public interface QueryParameterValue {
	
	/**
	 * The value type
	 */
	static public enum QueryParameterType {
		/** one parameter value */
		parameter,
		/** Multiple value */
		parameterList,
		/** parser the value as xpql syntax */
		xpql,
		/** Put the value in raw in the output */
		raw
	}
	
	/**
	 * Sets the name
	 * @param name	The name to set.
	 */
	void setName(String name);
	
	/**
	 * Gets the name
	 * @return	Returns the name
	 */
	String getName();
	
	/**
	 * Sets the value
	 * @param value	The value to set.
	 */
	void setValue(Object value);
	
	/**
	 * Gets the value
	 * @return	Returns the value
	 */
	Object getValue();
	
	/**
	 * Sets the defaultValue
	 * @param defaultValue	The defaultValue to set.
	 */
	void setDefaultValue(Object defaultValue);
	
	/**
	 * Gets the defaultValue
	 * @return	Returns the defaultValue
	 */
	Object getDefaultValue();
	
	/**
	 * Sets type value Type
	 * @param type	The type to set.
	 */
	void setType(QueryParameterType type);
	
	/**
	 * Gets the type.
	 * @return	Returns the type
	 */
	QueryParameterType getType();
	
	/**
	 * @return the valueType
	 */
	Class<?> getValueType();
	
	/**
	 * @param valueType the valueType to set
	 */
	void setValueType(Class<?> valueType);
	
	/**
	 * Force impl to have public clone methode
	 * @return
	 * @throws CloneNotSupportedException
	 */
	QueryParameterValue clone() throws CloneNotSupportedException;
}
