/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.xpql.query;

/**
 * QueryPart is a sub part of a full query.
 * 
 * @author Willem Cazander
 * @version 1.0 Mar 4, 2007
 */
public interface QueryPart extends Cloneable {
	
	/**
	 * This method is for building the SQL version of the query part.
	 * This version is intended for execution.
	 * @return
	 */
	String toSQL(Query query);
	
	/**
	 * This method is for building the prepared statement SQL version of the query part.
	 * This version is intended for execution via a prepared statement.
	 * @return
	 */
	String toPreparedSQL(Query query);
	
	/**
	 * This method is for building the edit mode version of the query part.
	 * This version is intended for humans working with the format in an editor.
	 * @return
	 */
	String toEdit(Query query);
	
	/**
	 * Force impl to have public clone method.
	 * @return
	 * @throws CloneNotSupportedException
	 */
	QueryPart clone() throws CloneNotSupportedException;
}
