package net.forwardfire.vasc.opt.editor;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import net.forwardfire.vasc.backend.VascBackendException;
import net.forwardfire.vasc.backend.VascBackendAccessDataRecord;
import net.forwardfire.vasc.backend.crud.VascBackendCrud;
import net.forwardfire.vasc.backend.data.BeanVascBackendAccessDataRecord;
import net.forwardfire.vasc.backend.list.DefaultVascBackendListResult;
import net.forwardfire.vasc.backend.list.VascBackendListResult;
import net.forwardfire.vasc.backend.list.VascBackendListRequest;
import net.forwardfire.vasc.core.VascController;
import net.forwardfire.vasc.core.VascEntry;
import net.forwardfire.vasc.core.VascEntryControllerLocal;
import net.forwardfire.vasc.core.VascEntryField;
import net.forwardfire.vasc.core.VascEntryFieldLocal;
import net.forwardfire.vasc.core.VascEntryFieldSet;
import net.forwardfire.vasc.core.VascEntryFieldSetLocal;
import net.forwardfire.vasc.core.VascEntryLinkLocal;
import net.forwardfire.vasc.core.VascEntryListOption;
import net.forwardfire.vasc.core.VascEntryListOptionLocal;
import net.forwardfire.vasc.core.VascEntryLocal;
import net.forwardfire.vasc.core.VascEntryLink;
import net.forwardfire.vasc.impl.DefaultVascEntry;
import net.forwardfire.vasc.impl.DefaultVascEntryField;
import net.forwardfire.vasc.impl.DefaultVascEntryFieldSet;
import net.forwardfire.vasc.impl.DefaultVascEntryLink;

public class VirtualVascBackend implements VascBackendCrud<Serializable,Serializable> {

	private VascController vascController = null;
	private String vascType = "entry";
	private String entryId = null;
	
	@Override
	public VascBackendListResult<Serializable> execute(VascBackendListRequest state) throws VascBackendException {
		List<Serializable> result = new ArrayList<Serializable>(100);
		entryId = (String)state.getDataParameter("entry_id");
		if ("entry".equals(vascType)) {
			for (String id:vascController.getVascEntryController().getVascEntryIds()) {
				result.add(vascController.getVascEntryController().getVascEntryById(id));
			}
		} else if ("field".equals(vascType)) {
			if (entryId!=null) {
				VascEntry ve = vascController.getVascEntryController().getVascEntryById(entryId);
				result.addAll(ve.getVascEntryFields());
			}
		} else if ("fieldset".equals(vascType)) {
			if (entryId!=null) {
				VascEntry ve = vascController.getVascEntryController().getVascEntryById(entryId);
				result.addAll(ve.getVascEntryFieldSets());
			}
		} else if ("linkentries".equals(vascType)) {
			if (entryId!=null) {
				VascEntry ve = vascController.getVascEntryController().getVascEntryById(entryId);
				result.addAll(ve.getVascEntryLinks());
			}
		} else if ("listoptions".equals(vascType)) {
			if (entryId!=null) {
				VascEntry ve = vascController.getVascEntryController().getVascEntryById(entryId);
				result.addAll(ve.getVascEntryListOptions());
			}
		}
		
		return new DefaultVascBackendListResult<Serializable>(result);
	}
	
	@Override
	public Serializable persist(Serializable object) throws VascBackendException {
		return edit(object,false,true);
	}
	
	@Override
	public Serializable merge(Serializable object) throws VascBackendException {
		return edit(object,true,true);
	}
	
	@Override
	public void delete(Serializable object) throws VascBackendException {
		edit(object,true,false);
	}
	
	public Serializable edit(Serializable object,boolean removeReal,boolean addEdit) throws VascBackendException {
		
		if (object instanceof VascEntry) {
			entryId = ((VascEntry)object).getId();
		}
		VascEntryLocal veReal = (VascEntryLocal)((VascEntryControllerLocal)vascController.getVascEntryController()).getMasterVascEntryById(entryId);
		
		if ("entry".equals(vascType)) {
			
		} else if ("field".equals(vascType)) {
			VascEntryFieldLocal vef = (VascEntryFieldLocal)object;
			if (removeReal) {
				VascEntryField vefReal = veReal.getVascEntryFieldById(vef.getId());
				veReal.removeVascEntryField((VascEntryFieldLocal)vefReal);
			}
			if (addEdit) {
				veReal.addVascEntryField(vef);
			}
		} else if ("fieldset".equals(vascType)) {
			VascEntryFieldSetLocal vefs = (VascEntryFieldSetLocal)object;
			if (removeReal) {
				VascEntryFieldSet vefsReal = veReal.getVascEntryFieldSetById(vefs.getId());
				veReal.removeVascEntryFieldSet((VascEntryFieldSetLocal)vefsReal);
			}
			if (addEdit) {
				veReal.addVascEntryFieldSet(vefs);
			}
		} else if ("linkentries".equals(vascType)) {
			VascEntryLinkLocal vefs = (VascEntryLinkLocal)object;
			if (removeReal) {
				VascEntryLink vefsReal = veReal.getVascEntryLinkById(vefs.getId());
				veReal.removeVascEntryLink((VascEntryLinkLocal)vefsReal);
			}
			if (addEdit) {
				veReal.addVascEntryLink(vefs);
			}
		} else if ("listoptions".equals(vascType)) {
			VascEntryListOptionLocal vef = (VascEntryListOptionLocal)object;
			if (removeReal) {
				VascEntryListOption vefReal = veReal.getVascEntryListOptionById(vef.getId());
				veReal.removeVascEntryListOption((VascEntryListOptionLocal)vefReal);
			}
			if (addEdit) {
				veReal.addVascEntryListOption(vef);
			}
		}
		return object;
	}
	
	@Override
	public VascBackendAccessDataRecord provideVascEntryFieldValue() {
		return new BeanVascBackendAccessDataRecord();
	}
	
	/**
	 * @see net.forwardfire.vasc.backend.crud.VascBackendCrud#fetch(java.io.Serializable)
	 */
	@Override
	public Serializable fetch(Serializable recordPK) throws VascBackendException {
		// TODO Auto-generated method stub
		return null;
	}
	
	/**
	 * @see net.forwardfire.vasc.backend.crud.VascBackendCrud#newRecord()
	 */
	@Override
	public Serializable newRecord() throws VascBackendException {
		if ("entry".equals(vascType)) {
			return new DefaultVascEntry(); // FIXME: where is loader?
		} else if ("field".equals(vascType)) {
			return new DefaultVascEntryField();
		} else if ("fieldset".equals(vascType)) {
			return new DefaultVascEntryFieldSet();
		} else if ("linkentries".equals(vascType)) {
			return new DefaultVascEntryLink();
		} else if ("listoptions".equals(vascType)) {
			return new DefaultVascEntryField();
		}
		return null;
	}
	
	/**
	 * @return the vascController
	 */
	public VascController getVascController() {
		return vascController;
	}
	
	/**
	 * @param vascController the vascController to set
	 */
	public void setVascController(VascController vascController) {
		this.vascController = vascController;
	}
	
	/**
	 * @return the vascType
	 */
	public String getVascType() {
		return vascType;
	}
	
	/**
	 * @param vascType the vascType to set
	 */
	public void setVascType(String vascType) {
		this.vascType = vascType;
	}
}
