/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.test.frontend.data;

import net.forwardfire.vasc.backend.VascBackendControllerLocal;
import net.forwardfire.vasc.core.VascController;
import net.forwardfire.vasc.core.VascEntry;
import net.forwardfire.vasc.core.VascEntryControllerLocal;
import net.forwardfire.vasc.core.VascEntryField;
import net.forwardfire.vasc.core.VascEntryLocal;
import net.forwardfire.vasc.impl.DefaultVascEntry;
import net.forwardfire.vasc.impl.DefaultVascEntryField;
import net.forwardfire.vasc.impl.DefaultVascEntryListOption;
import net.forwardfire.vasc.impl.DefaultVascFactory;

/**
 * TestModelData provided some vasc entries with object list datamodel to test frontends.
 * 
 * @author Willem Cazander
 * @version 1.0 Aug 2, 2007
 */
public class TestModelData {
	
	public VascController getTestVascController() throws Exception {
		VascController c = DefaultVascFactory.getDefaultVascController();

		// for test
		TestModelVascDataSource backends = new TestModelVascDataSource();
		backends.generatorData(10876);
		
		//backends.orderModel.setId("orderModelBackend");
		((VascBackendControllerLocal)c.getVascBackendController()).addVascBackend(backends.orderModel);

		//backends.orderLineModel.setId("orderLineModelBackend");
		((VascBackendControllerLocal)c.getVascBackendController()).addVascBackend(backends.orderLineModel);
		
		return c;
	}

	public VascEntryLocal getTestEntry(VascController vc) {
		return (VascEntryLocal)vc.getVascEntryController().getVascEntryById("orderModel");
	}
	
	public void createVascEntries(VascController vc,VascEntryControllerLocal vecl) throws Exception {		
		DefaultVascEntry ve = new DefaultVascEntry();
		ve.setId("orderModel");
		ve.setBackendId("orderModelBackend");

		DefaultVascEntryField field = new DefaultVascEntryField();
		field.setId("name");
		ve.addVascEntryField(field);
		
		field = new DefaultVascEntryField();
		field.setId("description");
		field.setVascEntryFieldType(vc.getVascEntryFieldTypeController().getVascEntryFieldTypeById("TextAreaField"));
		ve.addVascEntryField(field);

		field = new DefaultVascEntryField();
		field.setId("active");
		field.setVascEntryFieldType(vc.getVascEntryFieldTypeController().getVascEntryFieldTypeById("BooleanField"));
		ve.addVascEntryField(field);
		
		field = new DefaultVascEntryField();
		field.setId("date");
		field.setVascEntryFieldType(vc.getVascEntryFieldTypeController().getVascEntryFieldTypeById("DateField"));
		ve.addVascEntryField(field);
		
		field = new DefaultVascEntryField();
		field.setId("hexColor");
		ve.addVascEntryField(field);
		
		field = new DefaultVascEntryField();
		field.setId("testModel");
		// LIST and vascEntryFieldType.dataObject="#{vascController.vascBackendControllerResolver.vascBackendController.vascBackendById('testBackend1')}"
		ve.addVascEntryField(field);

		// table options
		DefaultVascEntryListOption option = new DefaultVascEntryListOption();
		option.setId("active");
		option.setVascEntryFieldType(vc.getVascEntryFieldTypeController().getVascEntryFieldTypeById("BooleanField"));
		option.setBackendName("active");
		option.setDefaultValue("true"); // obj does also work ! we are first need to fix string defaults.
		
		ve.addVascEntryListOption(option);
		
		vecl.addVascEntry(ve);
		

		ve = new DefaultVascEntry();
		ve.setId("orderLineModel");
		ve.setBackendId("orderLineModelBackend");

		field = new DefaultVascEntryField();
		field.setId("name");
		ve.addVascEntryField(field);
		
		field = new DefaultVascEntryField();
		field.setId("price");
		field.setVascEntryFieldType(vc.getVascEntryFieldTypeController().getVascEntryFieldTypeById("FloatField"));
		ve.addVascEntryField(field);

		field = new DefaultVascEntryField();
		field.setId("date");
		field.setVascEntryFieldType(vc.getVascEntryFieldTypeController().getVascEntryFieldTypeById("DateField"));
		ve.addVascEntryField(field);
		
		vecl.addVascEntry(ve);
		
		DefaultVascFactory.fillVascControllerLocalEntries(vecl, vc);
	}
}