/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.test.frontend.data;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.EnumSet;
import java.util.List;

import org.x4o.xml.element.DefaultElementObjectPropertyValue;
import org.x4o.xml.element.ElementObjectPropertyValueException;

import net.forwardfire.vasc.backend.VascBackendException;
import net.forwardfire.vasc.backend.VascBackendAccessDataKey;
import net.forwardfire.vasc.backend.VascBackendAccessDataRecord;
import net.forwardfire.vasc.backend.crud.VascBackendCrud;
import net.forwardfire.vasc.backend.data.BeanVascBackendAccessDataRecord;
import net.forwardfire.vasc.backend.list.DefaultVascBackendListResult;
import net.forwardfire.vasc.backend.list.VascBackendList;
import net.forwardfire.vasc.backend.list.VascBackendListFeature;
import net.forwardfire.vasc.backend.list.VascBackendListResult;
import net.forwardfire.vasc.backend.list.VascBackendListRequest;

/**
 * 
 * @author Willem Cazander
 * @version 1.0 Mar 21, 2007
 */
public class TestModelVascDataSource {

	//private List<TestModelOrder> testModels = null;
	public ListObjectVascDataSource<Integer,TestModelOrder> orderModel = null;
	public ListObjectVascDataSource<Integer,TestModelOrderLine> orderLineModel = null;
	
	public TestModelVascDataSource() {
		this(new ArrayList<TestModelOrder>(200),new ArrayList<TestModelOrderLine>(200));
	}
	
	@SuppressWarnings({ "unchecked", "rawtypes" })
	public TestModelVascDataSource(List<TestModelOrder> testModels,List<TestModelOrderLine> testModelLine) {
		orderModel = new ListObjectVascDataSource((List)testModels);
		orderLineModel = new ListObjectVascDataSource((List)testModelLine);
	}
	
	public void generatorData(int size) {
		for (int i=0;i<size/3;i++) {
			TestModelOrder t = new TestModelOrder();
			t.setDate(new Date());
			t.setDescription("Some long text in this....");
			t.setName("BeanName_"+i);
			t.setPrice(19.2f+i*4);
			if (i%4==0) {
				t.setActive(false);
			} else {
				t.setActive(true);
			}
			if (orderModel.getObjectList().isEmpty()==false) {
				t.setTestModel(orderModel.getObjectList().get(0));
			}
			orderModel.getObjectList().add(t);
			
			for (int ii=0;ii<size/1000;ii++) {
				TestModelOrderLine line = new TestModelOrderLine();
				line.setDate(new Date());
				line.setName("OrderLine"+ii);
				line.setPrice(new Float(123+ii));
				line.setOrder(t);
				orderLineModel.getObjectList().add(line);
			}
		}
	}
	
	class ListObjectVascDataSource<DATA_PK extends Serializable,DATA_RECORD extends Serializable> implements VascBackendList<DATA_PK,DATA_RECORD> {
		private List<DATA_RECORD> modelList = null;
	
		public ListObjectVascDataSource(List<DATA_RECORD> modelList) {
			this.modelList=modelList;
		}
		
		public List<DATA_RECORD> getObjectList() {
			return modelList;
		}
		
		@Override
		public VascBackendListResult<DATA_RECORD> execute(VascBackendListRequest state) throws VascBackendException {
			if (state.getDataParameterKeys().isEmpty()==false) {
				List<DATA_RECORD> result = new ArrayList<DATA_RECORD>(modelList.size()/2);
				DefaultElementObjectPropertyValue helper = new DefaultElementObjectPropertyValue(); 
				for (DATA_RECORD row:modelList) {
					boolean addRow = true;
					for (String key:state.getDataParameterKeys()) {
						Object keyValue = state.getDataParameter(key);
						Object propValue = null;
						try {
							propValue = helper.getProperty(row, key);
						} catch (ElementObjectPropertyValueException e) {
							e.printStackTrace();
						}
						if (keyValue==null && propValue==null) {
							continue;
						}
						if (keyValue.equals(propValue)) {
							continue;
						}
						addRow = false;
					}
					if (addRow) {
						result.add(row);
					}
				}
				return new DefaultVascBackendListResult<DATA_RECORD>(result);
			} else {
				List<DATA_RECORD> result = new ArrayList<DATA_RECORD>(modelList.size());
				for (DATA_RECORD row:modelList) {
					result.add(row);
				}
				return new DefaultVascBackendListResult<DATA_RECORD>(result);
			}
		}

		@Override
		public EnumSet<VascBackendListFeature> getSupportedFeatures() {
			// TODO Auto-generated method stub
			return null;
		}

		@Override
		public List<String> getSupportedRowActions() {
			// TODO Auto-generated method stub
			return null;
		}

		@Override
		public VascBackendAccessDataRecord<DATA_PK, DATA_RECORD> createVascBackendAccessDataRecord() {
			// TODO Auto-generated method stub
			return null;
		}

		@Override
		public String getId() {
			// TODO Auto-generated method stub
			return null;
		}

		@Override
		public VascBackendAccessDataKey<DATA_PK> createVascBackendAccessDataKey() {
			// TODO Auto-generated method stub
			return null;
		}

		@Override
		public void executeRowAction(VascBackendListRequest request, List<DATA_PK> recordPKs, String actionName)
				throws VascBackendException {
			// TODO Auto-generated method stub
			
		}
	
	}
}