/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.test.i18n;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.ResourceBundle;

import net.forwardfire.vasc.core.VascEntry;
import net.forwardfire.vasc.core.VascEntryField;
import net.forwardfire.vasc.core.VascEntryFieldSet;
import net.forwardfire.vasc.core.VascEntryGroup;
import net.forwardfire.vasc.core.VascEntryLink;
import net.forwardfire.vasc.core.actions.VascAction;


/**
 * Parses the Vasc annotations and generates all i18n keys for you.
 * 
 * @author Willem Cazander
 * @version 1.0 Mar 23, 2009
 */
public class VascBundleCheckEntryKeys {

	private ResourceBundle resourceBundle = null;
	private String imagePrefix = null;
	private String imagePostfix = null;
	
	public VascBundleCheckEntryKeys(ResourceBundle resourceBundle) {
		imagePrefix  = "resources/images/models/";
		imagePostfix = ".png";
		this.resourceBundle=resourceBundle;
	}
	
	public Map<String,String> collectVascKeys(VascEntry ve) {
		String value = ve.getId();
		if (ve.getVascGroupId()!=null) {
			if (value.toLowerCase().startsWith(ve.getVascGroupId().toLowerCase())) {
				value = value.substring(ve.getVascGroupId().length());
				if (Character.isLetterOrDigit(value.charAt(0))==false) {
					value = value.substring(1);
				}
			}
		}
		Map<String,String> keys = new HashMap<String,String>(300);
		keys.put(ve.getName(),				value);
		keys.put(ve.getImage(),				ve.getId());
		keys.put(ve.getListDescription(),	"View list of "+value);
		keys.put(ve.getListImage(),			imagePrefix+ve.getId()+imagePostfix);
		keys.put(ve.getEditDescription(),	"Edit your "+value);
		keys.put(ve.getEditImage(),			imagePrefix+ve.getId()+imagePostfix);
		keys.put(ve.getDeleteDescription(),	"Delete this "+value);
		keys.put(ve.getDeleteImage(),		imagePrefix+ve.getId()+imagePostfix);
		keys.put(ve.getCreateDescription(),	"Create new "+value);
		keys.put(ve.getCreateImage(),		imagePrefix+ve.getId()+imagePostfix);
		for (VascEntryField vef:ve.getVascEntryFields()) {
			keys.put(vef.getName(),			vef.getId());
			keys.put(vef.getImage(),		vef.getId());
			keys.put(vef.getDescription(),	vef.getId());
		}
		for (VascEntryFieldSet s:ve.getVascEntryFieldSets()) {
			keys.put(s.getName(),			s.getId());
			keys.put(s.getDescription(),	s.getId());
			keys.put(s.getImage(),			s.getId());
		}
		for (VascEntryField vef:ve.getVascEntryListOptions()) {
			keys.put(vef.getName(),			vef.getId());
			keys.put(vef.getImage(),		vef.getId());
			keys.put(vef.getDescription(),	vef.getId());
		}
		for (VascEntryLink vef:ve.getVascEntryLinks()) {
			keys.put(vef.getName(),			vef.getId());
		}
		for (VascAction a:ve.getGlobalActions()) {
			keys.put(a.getName(),			a.getId());
			keys.put(a.getDescription(),	a.getId());
			keys.put(a.getImage(),			a.getId());
		}
		for (VascAction a:ve.getRowActions()) {
			keys.put(a.getName(),			a.getId());
			keys.put(a.getDescription(),	a.getId());
			keys.put(a.getImage(),			a.getId());
		}
		for (VascAction a:ve.getColumnActions()) {
			keys.put(a.getName(),			a.getId());
			keys.put(a.getDescription(),	a.getId());
			keys.put(a.getImage(),			a.getId());
		}
		return keys;
	}
	
	public Map<String,String> collectVascKeys(VascEntryGroup veg) {
		Map<String,String> keys = new HashMap<String,String>(30);
		keys.put(veg.getName(),			veg.getId());
		keys.put(veg.getImage(),		veg.getId());
		keys.put(veg.getDescription(),	veg.getId());
		return keys;
	}
	
	public Map<String,String> generateMissingKeys(VascEntryGroup veg) {
		Map<String,String> keys = collectVascKeys(veg);
		List<String> keysResult = new ArrayList<String>(keys.keySet());
		keysResult.removeAll(resourceBundle.keySet());
		Collections.sort(keysResult);
		Map<String,String> data = new HashMap<String,String>(300);
		for (String key:keysResult) {
			data.put(key, keys.get(key));
		}
		return data;
	}
	
	public Map<String,String> generateMissingKeys(VascEntry ve) {
		Map<String,String> keys = collectVascKeys(ve);
		List<String> keysResult = new ArrayList<String>(keys.keySet());
		keysResult.removeAll(resourceBundle.keySet());
		Collections.sort(keysResult);
		Map<String,String> data = new HashMap<String,String>(300);
		for (String key:keysResult) {
			data.put(key, keys.get(key));
		}
		return data;
	}
	
	public Map<String,String> generateRemovedKeys(VascEntry ve) {
		Map<String,String> keys = collectVascKeys(ve);
		List<String> keysResult = new ArrayList<String>(resourceBundle.keySet());
		keysResult.removeAll(keys.keySet());
		Collections.sort(keysResult);
		Map<String,String> data = new HashMap<String,String>(300);
		for (String key:keysResult) {
			data.put(key, keys.get(key));
		}
		return data;
	}

	/**
	 * @return the resourceBundle
	 */
	public ResourceBundle getResourceBundle() {
		return resourceBundle;
	}

	/**
	 * @param resourceBundle the resourceBundle to set
	 */
	public void setResourceBundle(ResourceBundle resourceBundle) {
		this.resourceBundle = resourceBundle;
	}

	/**
	 * @return the imagePrefix
	 */
	public String getImagePrefix() {
		return imagePrefix;
	}

	/**
	 * @param imagePrefix the imagePrefix to set
	 */
	public void setImagePrefix(String imagePrefix) {
		this.imagePrefix = imagePrefix;
	}

	/**
	 * @return the imagePostfix
	 */
	public String getImagePostfix() {
		return imagePostfix;
	}

	/**
	 * @param imagePostfix the imagePostfix to set
	 */
	public void setImagePostfix(String imagePostfix) {
		this.imagePostfix = imagePostfix;
	}
}
